/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include <sys/stat.h>
#include <unistd.h> // for getcwd

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  char line[500];
  int l, len, flag;
  char oneword[50];
  struct stat fst;
  char moddir1[50];
  char moddir2[50];
  char moddir3[50];
  char moddir4[50];
  char tempname[100];
  int issubmodel=0;

  char systemcom[500];

  if(argc < 2){
    fprintf(stderr, "ERROR in archivemodel.c: you must enter the name of the system you wish to archive.\n");
    return -1;
  }

  // check if the model to be archived exists and is not a submodel
  fprintf(stderr, "Checking if the model exists and is not a submodel.\n");

  snprintf(moddir1, 50, "models/modules/%s", argv[1]);
  snprintf(moddir2, 50, "models/modeldescs/%s", argv[1]);
  snprintf(moddir3, 50, "models/inputs/%s", argv[1]);
  snprintf(moddir4, 50, "models/pvals/%s", argv[1]);

  if((stat(moddir1, &fst) != -1) && (stat(moddir2, &fst) != -1) && (stat(moddir3, &fst) != -1) && (stat(moddir4, &fst) != -1)){}
  else{
    fprintf(stderr, "The model does not appear to be a well-formed model suitable for archiving. The following are missing:\n");
    if (stat(moddir1, &fst) == -1)
      fprintf(stderr, "\tThe directory models/modules/%s\n", argv[1]);
    if (stat(moddir2, &fst) == -1)
      fprintf(stderr, "\tThe directory models/modeldescs/%s\n", argv[1]);
    if (stat(moddir3, &fst) == -1)
      fprintf(stderr, "\tThe directory models/inputs/%s\n", argv[1]);
    if (stat(moddir4, &fst) == -1)
      fprintf(stderr, "\tThe directory models/pvals/%s\n", argv[1]);

    fprintf(stderr, "Aborting the archiving process.\n");
    return -1;
  }
  fprintf(stderr, "...The model exists and is well formed.\n");

  snprintf(tempname, 100, "models/modeldescs/%s/alldata.dat", argv[1]);
  fd = fopen(tempname, "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"%s\". Aborting the archiving process.\n", tempname);
    return -1;
  }

  issubmodel=0;
  flag = 0;
  while((len = getline(fd, line, 500))> 0){
    if (!iscomsep(line) && strstr(line, "endMODFILES"))
      flag = 0;

    if(flag != 0 && !iscomsep(line)){
      issubmodel = 1;
      break;
    }
    if(!iscomsep(line) && strstr(line, "MODFILES") && !strstr(line, "endMODFILES"))
      flag = 1;
  }

  fclose(fd);

  if(issubmodel == 1){
    fprintf(stderr, "%s is a submodel which contains shared files. Before you archive it, you must convert it to a model.\n", argv[1]);
    return -1;
  }

  fprintf(stderr, "...The model is not a submodel. Hence it is suitable for archiving.\n");
  fprintf(stderr, "...Removing existing archive files.\n");

  // remove any existing archive files

  snprintf(systemcom, 500, "rm -fr models/archives/%s.tar", argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR with removing existing directory models/archives/%s_archive. Now exiting.\n", argv[1]);
    return -1;
  }

  snprintf(systemcom, 500, "rm -fr tempfiles/%s_bits", argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR with removing file models/archives/%s_bits. Now exiting.\n", argv[1]);
    return -1;
  }



  // the harder parts
  
  fd1 = fopen("userfiles/model_defaults", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_defaults\".\n");
    return -1;
  }

  fprintf(stderr, "Creating the file tempfiles/%s_bits which will store various bits of information about the model.\n", argv[1]);

  snprintf(systemcom, 500, "tempfiles/%s_bits", argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  fd = fopen(systemcom, "w");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file %s.\n", systemcom);
    fclose(fd1);
    return -1;
  }

  fprintf(stderr, "Getting data from \"userfiles/model_defaults\"\n");

  fprintf(fd, "USERFILES/MODEL_DEFAULTS\n");

  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);
      if(strcmp(argv[1], oneword)==0)
	fprintf(fd, "%s", line);
    }
  }
  fprintf(fd, "******\n");

  fclose(fd1);

  // from file model_types_make

  fprintf(stderr, "Getting data from \"model_types_make\"\n");
  fd1 = fopen("userfiles/model_types_make", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types_make\".\n");
    return -1;
  }

  fprintf(fd, "MODEL_TYPES_MAKE\n");

  flag = 0;
  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);

      if(strcmp(argv[1], oneword)==0){
	flag = 1;
      }

      if(flag == 1)
	fprintf(fd, "%s", line);

      if(strcmp("endif", oneword)==0)
	flag = 0;

    }
  }

  fprintf(fd, "******\n");

  fclose(fd1);


  // from file userfiles/model_types.dat

  fprintf(stderr, "Getting data from \"userfiles/model_types.dat\"\n");
  fd1 = fopen("userfiles/model_types.dat", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types.dat\".\n");
    return -1;
  }

  fprintf(fd, "USERFILES/MODEL_TYPES.DAT\n");

  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);
      if(strcmp(argv[1], oneword)==0)
	fprintf(fd, "%s", line);
    }
  }

  fprintf(fd, "******\n");

  fclose(fd1);
  fclose(fd);

  // tar the relevant directories and files
  snprintf(moddir3, 50, "models/autoreginputs/%s", argv[1]);
  if (fileexists(moddir3))
    snprintf(systemcom, 500, "tar cvf models/archives/%s.tar models/modules/%s models/modeldescs/%s models/inputs/%s models/autoreginputs/%s models/pvals/%s tempfiles/%s_bits", argv[1], argv[1], argv[1], argv[1], argv[1], argv[1], argv[1]);
  else
    snprintf(systemcom, 500, "tar cvf models/archives/%s.tar models/modules/%s models/modeldescs/%s models/inputs/%s models/pvals/%s tempfiles/%s_bits", argv[1], argv[1], argv[1], argv[1], argv[1], argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR with creating archive archive/%s.tar. Now exiting.\n", argv[1]);
    return -1;
  }

  fprintf(stderr, "Model \"%s\" successfully archived.\n", argv[1]);

  return 0;
}
