#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "syssize.h"

// every number is in base bas. What is the kth digit, counting from the end?

int nthdig(int val, int bas, int k){
  if(k<1){
    fprintf(stderr, "ERROR in nthdig: digit must be greater than 0\n");
    return 0;
  }
  return  (val%(int)(pow(bas, k))-val%(int)(pow(bas, k-1)))/(pow(bas, k-1));
}


int main(int argc, char *argv[]){
  int i, j, m, r, pm;
  FILE *fd;
  int l;
  char systemcom[1000];
  double diff;
  double vval;
  double vtot, vtmp;
  char modname[100];
  char paramfile[100];
  char datfile[100];
  char fittime[20];
  double ftime;
  char maxsteps[20];
  int maxst;
  char *paramchanges;
  int numchanges;
  char *mphs[30];
  char *outp[10];
  int numoutp;
  int nummphs;
  char *varstofit;
  double **darray;
  int s1=0, s2=0;
  double strt, finl;
  char **settable=NULL;
  int numsettable=0;
  char *pval;
  double pv;
  double pv1;
  double vmin, vmax;
  double tmpd;
  double *bestvals;

  if(argc<2){
    fprintf(stderr, "ERROR: you need the name of the input file.\n");
    return -1;
  }


  if(getoption(argv[1], "model:", modname, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"model\". Exiting.\n");
    return -1;
  }

  snprintf(systemcom, 1000, "make %s", modname);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't run the command \"make %s\"\n", modname);
    return -1;
  }
  fprintf(stderr, "make %s\n", modname);

  l=system("make");
  if(l!=0){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't run the command \"make\"\n");
    return -1;
  }


  paramchanges = getrstfromfile(argv[1], "params:");
  if(isonlyspace(paramchanges)){
      fprintf(stderr, "ERROR in autofitting. Parameters for fitting not provided.\n");
      free_datarray(mphs, numchanges);
      free(paramchanges);
      return -1;
  }
  else{
    numchanges = chemgets(paramchanges, mphs, 30);
    if(numchanges%3 != 0){
      fprintf(stderr, "ERROR in autofitting on the params: line. Expecting a multiple of three arguments.\n");
      free_datarray(mphs, numchanges);
      free(paramchanges);
      return -1;
    }
  }
  free(paramchanges);

  nummphs = numchanges/3;

  settable = genarraydat("sysfiles/settable.dat", &numsettable);

  for(j=0;j<nummphs;j++){
    if(isinarray(settable, numsettable, mphs[3*j]) < 0){
      fprintf(stderr, "ERROR in params line: %s is not a settable parameter.\n", mphs[3*j]);
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }

    if(!isnumber(mphs[3*j+1])){
      fprintf(stderr, "ERROR in params line: %s is not a number.\n", mphs[3*j+1]);
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }
    if(!isnumber(mphs[3*j+2])){
      fprintf(stderr, "ERROR in params line: %s is not a number.\n", mphs[3*j+2]);
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }
  }

  if(getoption(argv[1], "paramfile:", paramfile, 100)!=0){ // use default
    fprintf(stderr, "using default parameter file.\n");
    if(!getfileblk("sysfiles/sysstate/which_param_file", paramfile, 100)){
      fprintf(stderr, "ERROR in reading default parameter file. Exiting.\n");
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }
  }
  else{
    fd = fopen("sysfiles/sysstate/which_param_file", "w");
    if(!fd){
      fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }
    fprintf(fd, "%s", paramfile);
    fclose(fd);
  }

  // set output file

  fd = fopen("sysfiles/sysstate/which_output_file", "w");
  if(!fd){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    free_datarray(mphs, numchanges);
    freearraydat(settable, numsettable);
    return -1;
  }
  fprintf(fd, "outdat/tempout.dat");
  fclose(fd);

  // set input file

  fd = fopen("sysfiles/sysstate/which_input_file", "w");
  if(!fd){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    free_datarray(mphs, numchanges);
    freearraydat(settable, numsettable);
    return -1;
  }
  fprintf(fd, "models/inputs/_tmp_autofit.dat");
  fclose(fd);



  if(getoption(argv[1], "fittime:", fittime, 20)!=0 || !isnumber(fittime)){
    sprintf(fittime, "1000.0"); // default
    fprintf(stderr, "Default fitting time of 1000 seconds being used.\n");
  }
  ftime=atof(fittime);

  if(getoption(argv[1], "maxsteps:", maxsteps, 20)!=0 || !ispureint(maxsteps)){
    sprintf(maxsteps, "10"); // default
    fprintf(stderr, "Default maximum steps in each direction = 10 being used.\n");
  }

  maxst=atoi(maxsteps);

  if(getoption(argv[1], "datafile:", datfile, 100)!=0 || isonlyspace(datfile)){
    fprintf(stderr, "ERROR with getting the option \"datafile\". Exiting.\n");
    free_datarray(mphs, numchanges);
    return -1;
  }


  varstofit = genfloatmdat(datfile, &darray, &s1, &s2);

  for(i=0;i<s1;i++){
    for(j=0;j<s2;j++){
      fprintf(stderr, "%.4e\t", darray[i][j]);
    }
    fprintf(stderr, "\n");
  }

  if(isonlyspace(varstofit) || !darray || s1==0 || s2==0){
    fprintf(stderr, "ERROR reading data file %s. Exiting.\n", datfile);
    free_datarray(mphs, numchanges);
    if(darray){free_dmatrix(darray, 0, s1-1, 0, s2-1);}
    free(varstofit);
    return -1;
  }

  numoutp = chemgets(varstofit, outp, 10);
  if(numoutp != s2){ // info line and 
    fprintf(stderr, "ERROR reading data file %s. Exiting.\n", datfile);
    free_datarray(mphs, numchanges);
    free_datarray(outp, numoutp);
    free_dmatrix(darray, 0, s1-1, 0, s2-1);
    free(varstofit);
    return -1;
  }
  free(varstofit);

  pval = getcvalfromfile(paramfile, outp[0]);
  pv1=atof(pval);free(pval);
  fprintf(stderr, "nummphs = %d\n", nummphs);

  bestvals=dvector(0, nummphs-1);
  pm=pow(maxst+1, nummphs);
  for(j=0;j<pm;j++){ // params to change
    fprintf(stderr, "***  %d percent  ***\n", (int)((float)j/((float)pm)*100.0));
 
    vtot=0.0;

    for(r=0;r<s1;r++){ // data points
      //create input file

      fd = fopen("models/inputs/_tmp_autofit.dat", "w");
      if(!fd){
	fprintf(stderr, "ERROR opening \"models/inputs/_tmp_autofit.dat\". Exiting..\n");
	fprintf(stderr, "ERROR reading data file %s. Exiting.\n", datfile);
	free_datarray(mphs, numchanges);
	free_datarray(outp, numoutp);
	free_dmatrix(darray, 0, s1-1, 0, s2-1);
	return -1;
      }
      
      fprintf(fd, "reset: %s, %.6e, %.6e, ", outp[0], pv1, darray[r][0]);
      for(i=0;i<nummphs;i++){
	//	fprintf(stderr, "\n");
	strt = atof(mphs[3*i+1]);finl = atof(mphs[3*i+2]);
	diff=(finl-strt)/maxst;
	pval = getcvalfromfile(paramfile, mphs[3*i]);
	pv=atof(pval);free(pval);
	fprintf(fd, "%s, %.6e, %.6e, ", mphs[3*i], pv, strt + nthdig(j, maxst+1, i+1)*diff);
	//	fprintf(stderr, "%s, %.6e, %.6e, ", mphs[3*i], pv, strt + nthdig(j, maxst+1, i+1)*diff);
      }
      fprintf(fd, "\n******\n");
      fprintf(fd, "%.6e\n", 0.9*ftime);
      fprintf(fd, "%.6e\n", 0.1*ftime);
      fclose(fd);

      // force
      snprintf(systemcom, 1000, "src/lib/P_genoutput");
      l=system(systemcom);

      // run the simulation
      snprintf(systemcom, 1000, "src/lib/P_run models/inputs/_tmp_autofit.dat %s outdat/tempout.dat > outdat/tempout.dat", paramfile);
      l=system(systemcom);

      if(l!=0){
      	fprintf(stderr, "ERROR in autofitting.c. Couldn't run the command \"%s\"\n", systemcom);
      	free_datarray(mphs, numchanges);
      	free_datarray(outp, numoutp);
      	free_dmatrix(darray, 0, s1-1, 0, s2-1);
      	return -1;
      }
      vtmp=0;
      for(m=1;m<numoutp;m++){
	tmpd = timeaverage("outdat/tempout.dat", 0.89*ftime, ftime, outp[m]);
      	vval = pow(tmpd - darray[r][m], 2);
	fprintf(stderr, "d(%s) = %.4e\t[%.4e -> %.4e]\n", outp[m], sqrt(vval), darray[r][m], tmpd);
      	vtmp+=vval;
      }
      if(numoutp>1)
      	vtmp/=(numoutp-1);

      vtot+=vtmp;
      
    }
    if(j==0){vmin=vtot+1.0;vmax=vtot-1.0;}

    if(vtot<vmin){
      fprintf(stderr, "\t*\n\t*\nvtot = %.4e\n\t*\n\t*\n", vtot);
      vmin=vtot;
      for(i=0;i<nummphs;i++){
	strt = atof(mphs[3*i+1]);finl = atof(mphs[3*i+2]);
	diff=(finl-strt)/maxst;
	bestvals[i] = strt + nthdig(j, maxst+1, i+1)*diff;
      }
    }
    if(vtot>vmax){vmax=vtot;}


  }

  fprintf(stderr, "\nbest values found: \n");
  fprintf(stderr, "\tdist\t%.4e\t[%.4e to %.4e]\n", vmin, vmin, vmax);
  for(i=0;i<nummphs;i++){
    fprintf(stderr, "\t%s\t%.4e\t[%.4e to %.4e]\n", mphs[3*i], bestvals[i], atof(mphs[3*i+1]), atof(mphs[3*i+2]));
  }
    
  free_dvector(bestvals, 0, nummphs-1);
  free_datarray(mphs, numchanges);
  free_datarray(outp, numoutp);
  free_dmatrix(darray, 0, s1-1, 0, s2-1);

  return 0;
}
