#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "syssize.h"


int outflag=0; // keep this global

// every number is in base bas. What is the kth digit, counting from the end?

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


double multidist(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime);

double amotsa(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime, double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*funk)(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime), int ihi, double *yhi, double fac);


double ran1(long *idum);

void amebsa(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime, double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*funk)(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime), int *iter, double temptr);


#define GET_PSUM \
  for (n=1;n<=ndim;n++) {\
  for (sum=0.0,m=1;m<=mpts;m++) sum += p[m][n];\
  psum[n]=sum;}
long idum;
double tt;

int nthdig(int val, int bas, int k){
  if(k<1){
    fprintf(stderr, "ERROR in nthdig: digit must be greater than 0\n");
    return 0;
  }
  return  (val%(int)(pow(bas, k))-val%(int)(pow(bas, k-1)))/(pow(bas, k-1));
}


int main(int argc, char *argv[]){
  int i, j;
  FILE *fd;
  int l;
  char systemcom[1000];
  char modname[100];
  char paramfile[100];
  char datfile[100];
  char fittime[20];
  double ftime;
  char tempr[20];
  double temptr;
  char numt[10];
  int numtemps;
  char pert[10];
  int numpert;
  double **darray;
  char *paramchanges;
  int numchanges;
  char *mphs[30];
  char *outp[10];
  int numoutp;
  int nummphs;
  int iter;
  char *varstofit;
  int s1=0, s2=0;
  char **settable=NULL;
  int numsettable=0;
  char **tmparray=NULL;
  int numtmparray=0;
  int *iv;


  double **pinit;
  double *finit;
  double *tempv;
  double *pb;
  double yb;
  char *pval;
  double pv;

  if(argc<2){
    fprintf(stderr, "ERROR: you need the name of the input file.\n");
    return -1;
  }

  if(!fileexists(argv[1])){
    fprintf(stderr, "ERROR: file %s does not exist.\n", argv[1]);
    return -1;
  }

  if(getoptq(argv[1], "model:", modname, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"model\" in file %s. Exiting.\n", argv[1]);
    return -1;
  }

  snprintf(systemcom, 1000, "make %s", modname);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in autofitting1.c. Couldn't run the command \"make %s\"\n", modname);
    return -1;
  }
  fprintf(stderr, "make %s\n", modname);

  l=system("make");
  if(l!=0){
    fprintf(stderr, "ERROR in autofitting1.c. Couldn't run the command \"make\"\n");
    return -1;
  }

  snprintf(systemcom, 1000, "src/lib/P_genoutput"); // generates sysfiles/output.dat
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in autofitting1.c. Couldn't run the command \"src/lib/P_genoutput\"\n");
    return -1;
  }


  paramchanges = getrstfromfile(argv[1], "params:");
  if(isonlyspace(paramchanges)){
    fprintf(stderr, "ERROR in autofitting1.c. Parameters for fitting not provided.\n");
    free(paramchanges);
    return -1;
  }
  else
    numchanges = chemgets(paramchanges, mphs, 30);
  free(paramchanges);
  if(numchanges<1){
    fprintf(stderr, "ERROR in autofitting1.c. Parameters for fitting not provided.\n");
    free_datarray(mphs, numchanges);
    return -1;
  }

  // check if all the parameters to be optimised are settable parameters for the model
  nummphs = numchanges;
  settable = genarraydat("sysfiles/settable.dat", &numsettable);

  for(j=0;j<nummphs;j++){
    if(isinarray(settable, numsettable, mphs[j]) < 0){
      fprintf(stderr, "ERROR in params line: %s is not a settable parameter.\n", mphs[j]);
      free_datarray(mphs, numchanges);
      freearraydat(settable, numsettable);
      return -1;
    }
  }
  freearraydat(settable, numsettable);

  // Get the parameter file to use
  if(getoptq(argv[1], "paramfile:", paramfile, 100)!=0){ // use default
    if(!getfileblk("sysfiles/sysstate/which_param_file", paramfile, 100)){
      fprintf(stderr, "ERROR in reading default parameter file. Exiting.\n");
      free_datarray(mphs, numchanges);
      return -1;
    }
    fprintf(stderr, "Using default parameter file: %s\n", paramfile);
  }
  else{
    fd = fopen("sysfiles/sysstate/which_param_file", "w");
    if(!fd){
      fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
      free_datarray(mphs, numchanges);
      return -1;
    }
    fprintf(fd, "%s", paramfile);
    fclose(fd);
  }

  // set output file

  fd = fopen("sysfiles/sysstate/which_output_file", "w");
  if(!fd){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    free_datarray(mphs, numchanges);
    return -1;
  }
  fprintf(fd, "outdat/tempout.dat");
  fclose(fd);

  // set input file

  fd = fopen("sysfiles/sysstate/which_input_file", "w");
  if(!fd){
    fprintf(stderr, "ERROR in autofitting.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    free_datarray(mphs, numchanges);
    return -1;
  }
  fprintf(fd, "models/inputs/_tmp_autofit.dat");
  fclose(fd);

  // How long for each simulation to converge to the static value?

  if(getoptq(argv[1], "fittime:", fittime, 20)!=0 || !isnumber(fittime)){
    sprintf(fittime, "1000.0"); // default
    fprintf(stderr, "Default fitting time of 1000 seconds being used.\n");
  }
  ftime=atof(fittime);

  // What is the initial "temperature", i.e. noise level
  // 2 is I think quite high

  if(getoptq(argv[1], "temperature:", tempr, 20)!=0 || !isnumber(tempr)){
    sprintf(tempr, "2.0"); // default
    fprintf(stderr, "Default temperature of 2.0 being used.\n");
  }
  temptr=atof(tempr);

  // Temperature is halved at each step - if you have a high initial temperature, then you want
  // a large number of steps

  if(getoptq(argv[1], "numtemps:", numt, 20)!=0 || !ispureint(numt)){
    sprintf(numt, "10"); // default
    fprintf(stderr, "Default number of temperature drops of 10 being used.\n");
  }
  numtemps=atoi(numt);

  // How many interations at each temperature step

  if(getoptq(argv[1], "pertemp:", pert, 20)!=0 || !ispureint(pert)){
    sprintf(pert, "100"); // default
    fprintf(stderr, "Default number of iterations at each temperature: 100 being used.\n");
  }
  numpert=atoi(pert);

  // Get the (measured) data which you are trying to get your simulation to match

  if(getoptq(argv[1], "datafile:", datfile, 100)!=0 || isonlyspace(datfile)){
    fprintf(stderr, "ERROR with getting the option \"datafile\". Exiting.\n");
    free_datarray(mphs, numchanges);
    return -1;
  }
  varstofit = genfloatmdat(datfile, &darray, &s1, &s2);

  // uncomment this if you want to see the data
/*   for(i=0;i<s1;i++){ */
/*     for(j=0;j<s2;j++) */
/*       fprintf(stderr, "%.4e\t", darray[i][j]); */
/*     fprintf(stderr, "\n"); */
/*   } */

  if(isonlyspace(varstofit) || !darray || s1==0 || s2==0){
    fprintf(stderr, "ERROR reading data file %s. Exiting.\n", datfile);
    free_datarray(mphs, numchanges);
    if(darray){free_dmatrix(darray, 0, s1-1, 0, s2-1);}
    free(varstofit);
    return -1;
  }

  numoutp = chemgets(varstofit, outp, 10);
  free(varstofit);
  if(numoutp != s2){ // info line and 
    fprintf(stderr, "ERROR reading data file %s. Exiting.\n", datfile);
    free_datarray(mphs, numchanges);
    free_datarray(outp, numoutp);
    free_dmatrix(darray, 0, s1-1, 0, s2-1);
    return -1;
  }
  if(numoutp <2){
    fprintf(stderr, "ERROR: must have something to fit. Exiting.\n");
    free_datarray(mphs, numchanges);
    free_datarray(outp, numoutp);
    free_dmatrix(darray, 0, s1-1, 0, s2-1);
    return -1;
  }
  

  // check if variables to fit are output by the model

  iv=ivector(0, numoutp-1);

  iv[0]=1; // first is a parameter
  for(i=1;i<numoutp;i++){iv[i]=0;}
  tmparray=genarraydat("sysfiles/output.dat", &numtmparray);
  for(i=1;i<numoutp;i++){
    if(isinarray(tmparray, numtmparray, outp[i]) >=0)
      iv[i]=1;
  }
  freearraydat(tmparray, numtmparray);
  tmparray=genarraydat("sysfiles/alltempstrs2.dat", &numtmparray);
  for(i=1;i<numoutp;i++){
    if(iv[i]==0){
      if(isinarray(tmparray, numtmparray, outp[i]) >=0){
	iv[i]=2;
        outflag=1; // some temporary variables -- need to augment the data files
      }
    }
  }
  freearraydat(tmparray, numtmparray);
  for(i=0;i<numoutp;i++){
    if(iv[i]==0){ // some variables to fit not found
      fprintf(stderr, "ERROR: output variable %s doesn't seem to exist for this model. Exiting.\n", outp[i]);
      free_datarray(mphs, numchanges);
      free_datarray(outp, numoutp);
      free_dmatrix(darray, 0, s1-1, 0, s2-1);
      free_ivector(iv, 0, numoutp-1);
      return -1;
    }
    if(iv[i]==2)
      outflag=1; // some temporary variables -- need to augment the data files
  }
  free_ivector(iv, 0, numoutp-1);

  if(outflag==1){ // you'll need to augment data files
    fprintf(stderr, "\nData files will need to be augmented.\n");
    l=system("make src/lib/P_augmentdatfile");
  }

  // Output some information

  fprintf(stderr, "Parameters being optimised:\n\t");
  for(i=0;i<nummphs-1;i++)
    fprintf(stderr, "%s\t", mphs[i]);
  fprintf(stderr, "%s\n", mphs[i]);

  // specific to this particular fitting

  pinit=dmatrix(1, nummphs+1, 1, nummphs);
  finit=dvector(1, nummphs+1);
  tempv=dvector(1, nummphs);
  pb=dvector(1, nummphs);




  for(i=1;i<=nummphs+1;i++){
    for(j=1;j<=nummphs;j++){
      if(i==j+1) // scale 
	pinit[i][j]=1.05;
      else
	pinit[i][j]=1.0;
    }
  }


  for(i=1;i<=nummphs+1;i++){
    for(j=1;j<=nummphs;j++)
      tempv[j] = pinit[i][j];
    finit[i] = multidist(paramfile, mphs, outp, numoutp, darray, s1, tempv, nummphs, ftime);
  }

  yb=10000; // so that the best value is stored. 
  for(i=0;i<numtemps;i++){
    fprintf(stderr, "i=%d\n", i);
    iter=numpert; //=200 normally
    //    iter=12-2*i;
    temptr *=0.5;
    amebsa(paramfile, mphs, outp, numoutp, darray, s1, tempv, nummphs, ftime, pinit, finit, nummphs, pb, &yb, 0.001, multidist, &iter, temptr);

    fprintf(stderr, "*\n*\n*\niteration %d\tBest so far: ", i+1);

    fprintf(stderr, "\tdist\t%.4e\n", yb);
    for(j=0;j<nummphs;j++){
      pval = getcvalfromfile(paramfile, mphs[j]);
      pv=atof(pval);free(pval);
      fprintf(stderr, "\t%s\t%.4e\n", mphs[j], pb[j+1]*pv);
    }
  }

  fprintf(stderr, "\nbest values found: \n");
  fprintf(stderr, "\tdist\t%.4e\n", yb);
  for(i=0;i<nummphs;i++){
    pval = getcvalfromfile(paramfile, mphs[i]);
    pv=atof(pval);free(pval);
    fprintf(stderr, "\t%s\t%.4e\n", mphs[i], pb[i+1]*pv);
  }

  free_dmatrix(pinit, 1, nummphs+1, 1, nummphs);
  free_dvector(finit, 1, nummphs+1);
  free_dvector(tempv, 1, nummphs);
  free_dvector(pb, 1, nummphs);
  free_datarray(mphs, numchanges);
  free_datarray(outp, numoutp);
  free_dmatrix(darray, 0, s1-1, 0, s2-1);

  return 0;
}

double multidist(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime)
{
  FILE *fd;
  int i, l;
  double vtmp;
  char systemcom[200];
  int r;
  char *pval;
  double pv;
  double pv1;
  double tmpd;
  double vval;
  double vtot;
  int m;

  pval = getcvalfromfile(paramfile, outp[0]);
  pv1=atof(pval);free(pval);
  vtot=0;
  for(r=0;r<s1;r++){ // data points
    //create input file

    fd = fopen("models/inputs/_tmp_autofit.dat", "w");
    if(!fd){
      fprintf(stderr, "ERROR opening \"models/inputs/_tmp_autofit.dat\". Exiting..\n");
      return -1;
    }
      
    fprintf(fd, "reset: %s, %.6e, %.6e, ", outp[0], pv1, darray[r][0]);
    for(i=0;i<nummphs;i++){
      pval = getcvalfromfile(paramfile, mphs[i]);
      pv=atof(pval);free(pval);
      fprintf(fd, "%s, %.6e, %.6e, ", mphs[i], pv, vvals[i+1]*pv);
    }
    fprintf(fd, "\n******\n");
    fprintf(fd, "%.6e\n", 0.9*ftime);
    fprintf(fd, "%.6e\n", 0.1*ftime);
    fclose(fd);

    // run the simulation
    snprintf(systemcom, 1000, "src/lib/P_run models/inputs/_tmp_autofit.dat %s outdat/tempout.dat > outdat/tempout.dat", paramfile);
    l=system(systemcom);

    if(l!=0){
      fprintf(stderr, "ERROR in autofitting.c. Couldn't run the command \"%s\"\n", systemcom);
      return -1;
    }
    vtmp=0;

    // simulation failed? (either the word "EXIT" or "nan" is found in the output file)
    if(!isgooddatfile("outdat/tempout.dat"))
      fprintf(stderr, "WARNING: A simulation failed and is being ignored.\n");
    else{
      if(outflag==1){ // augment the output file
	cpfile("outdat/tempout1.dat", "outdat/tempout.dat");
	l=system("src/lib/P_augmentdatfile outdat/tempout1.dat outdat/tempout.dat");
	if(l!=0){
	  fprintf(stderr, "ERROR in autofitting.c. Couldn't augment the data file\n");
	  return -1;
	}
      }
      for(m=1;m<numoutp;m++){ // variables to fit
	tmpd = timeaverage("outdat/tempout.dat", 0.89*ftime, ftime, outp[m]);
	vval = pow(tmpd - darray[r][m], 2);
	fprintf(stderr, "d(%s) = %.4e\t[%.4e -> %.4e]\n", outp[m], sqrt(vval), darray[r][m], tmpd);
	vtmp+=vval;
      }
      if(numoutp>1)
	vtmp/=(numoutp-1);
    }
    vtot+=vtmp;
      
  }
  fprintf(stderr, "\ndist=%6f\n", vtot);
  return vtot;

  /*   pen=0.0; */
  /*   for(i=0;i<4;i++) */
  /*     pen+=(vvals[i+1]-1.0)*(vvals[i+1]-1.0); */
  /*   pen=10.0*sqrt(pen); */

  /*   fprintf(stderr, "diff1 + pen = %.4e + %.4e\n", diff1, pen); */

  /*   return diff1+pen; */

}


// these routines are all adapted from numerical recipes in C

double ran1(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;

  if (*idum <= 0 || !iy) {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum < 0) *idum += IM;
      if (j < NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}

// This core routine performs one fitting step at a given temperature


void amebsa(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime, double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*funk)(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime), int *iter, double temptr)
{
  int i,ihi,ilo,j,m,n,mpts=ndim+1;
  double rtol,sum,swap,yhi,ylo,ynhi,ysave,yt,ytry,*psum;

  psum=dvector(1,ndim);
  tt = -temptr;
  GET_PSUM
    for (;;) {
      ilo=1;
      ihi=2;
      ynhi=ylo=y[1]+tt*log(ran1(&idum));
      yhi=y[2]+tt*log(ran1(&idum));
      if (ylo > yhi) {
	ihi=1;
	ilo=2;
	ynhi=yhi;
	yhi=ylo;
	ylo=ynhi;
      }
      for (i=3;i<=mpts;i++) {
	yt=y[i]+tt*log(ran1(&idum));
	if (yt <= ylo) {
	  ilo=i;
	  ylo=yt;
	}
	if (yt > yhi) {
	  ynhi=yhi;
	  ihi=i;
	  yhi=yt;
	} else if (yt > ynhi) {
	  ynhi=yt;
	}
      }
      rtol=2.0*fabs(yhi-ylo)/(fabs(yhi)+fabs(ylo));
      if (rtol < ftol || *iter < 0) {
	swap=y[1];
	y[1]=y[ilo];
	y[ilo]=swap;
	for (n=1;n<=ndim;n++) {
	  swap=p[1][n];
	  p[1][n]=p[ilo][n];
	  p[ilo][n]=swap;
	}
	break;
      }
      *iter -= 2;
      ytry=amotsa(paramfile, mphs, outp, numoutp, darray, s1, vvals, nummphs, ftime, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,-1.0);
      if (ytry <= ylo) {
	ytry=amotsa(paramfile, mphs, outp, numoutp, darray, s1, vvals, nummphs, ftime, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,2.0);
      } else if (ytry >= ynhi) {
	ysave=yhi;
	ytry=amotsa(paramfile, mphs, outp, numoutp, darray, s1, vvals, nummphs, ftime, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,0.5);
	if (ytry >= ysave) {
	  for (i=1;i<=mpts;i++) {
	    if (i != ilo) {
	      for (j=1;j<=ndim;j++) {
		psum[j]=0.5*(p[i][j]+p[ilo][j]);
		p[i][j]=psum[j];
	      }
	      y[i]=(*funk)(paramfile, mphs, outp, numoutp, darray, s1, psum, nummphs, ftime);
	    }
	  }
	  *iter -= ndim;
	  GET_PSUM
	    }
      } else ++(*iter);
    }
  free_dvector(psum,1,ndim);
}


double amotsa(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime, double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*funk)(char *paramfile, char *mphs[], char *outp[], int numoutp, double **darray, int s1, double vvals[], int nummphs, double ftime), int ihi, double *yhi, double fac)
{
  int j;
  double fac1,fac2,yflu,ytry,*ptry;

  ptry=dvector(1,ndim);
  fac1=(1.0-fac)/ndim;
  fac2=fac1-fac;
  for (j=1;j<=ndim;j++)
    ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
  ytry=(*funk)(paramfile, mphs, outp, numoutp, darray, s1, ptry, nummphs, ftime);
  if (ytry <= *yb) {
    for (j=1;j<=ndim;j++) pb[j]=ptry[j];
    *yb=ytry;
  }
  yflu=ytry-tt*log(ran1(&idum));
  if (yflu < *yhi) {
    y[ihi]=ytry;
    *yhi=yflu;
    for (j=1;j<=ndim;j++) {
      psum[j] += ptry[j]-p[ihi][j];
      p[ihi][j]=ptry[j];
    }
  }
  free_dvector(ptry,1,ndim);
  return yflu;
}

