/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// This program computes the autoregulation curves for the system. 
// At each stage the system is integrated, allowed to settle, 
// and then the autoregulation parameter is changed. 

#include <unistd.h>
#include "sys.h"
#include "radau5/radau/radau.h"
#include "common/stringmanip.h"
#include "common/nrsimp.h"

const double Pi = 3.14159265; 

int numvars;
char **vararray;
int numparams;
char **paramarray;
double *rpar;
int numautoregvars;
char **autoregvars;
int nummatvars;
char **matvars;
int *ord;
FILE *brf=NULL;
char **qvars;
int numqvars=0;

//#define CONTIN // are we taking as initial conditions the values stored in runparamvals.c?
#ifdef CONTIN
#include "../sysfiles/runparamvals.c" 
#endif

// the right-hand side function 
void rhs(int *n, double *x, double *v, double *fv, double *rpar, int *ipar);

// the mass matrix 
void mas(int *n, double *am, int *lmas, double *rpar, int *ipar);

// the parameter setting function

void pfunc(double *pvals, double *outvals); // in ../sysfiles/paramfunc.c

int inputautoregdat(char *infile, int *numchangeparams, char *chosen_param[], int maxpars, double *time_step, double *step_vals, double *init_vals, double *min_val, double *max_val, int *hyst, int *nummorphs, char *morphs[], double morphvals[], int maxmorphs, char *paramfilename);

int numchangeparams;
char *chosen_param[10];
double time_step;
double step_vals[10];
double init_vals[10];
double min_val;
double max_val;
int hyst;
int nummorphs; 
char *morphs[10]; 
double morphvals[20]; 
int maxmorphs;

int ktot;

// dummy variables for solmat
int fixedstep=0;
double Pvals[0][0];

// to let the output function know that this isn't an autoregulation curve plotter.

int autoreg = 1;

// This needs to be external because the solmat routine needs to know the iteration number

int kps; // iteration number


// declared externally so they can be accessed by the solout routines 

FILE *fd, *fd1, *fd2;
char *file, *file2;


void solmat0();
void solmat(int *nr,double *xold,double *x,
	    double *y,
	    double *cont,int *lrc,
	    int *n,
	    double *rpar,int *ipar,
	    int *irtrn);


//
//
// routines
//
//


void solautoreg0(FILE *fp);

// held in sysfiles/autoregout.cc

//void solautoreg(FILE *fp, double *y, paramlist *tp, double *rpar);
void solautoreg(FILE *fp, double *y, double *rpar);

void solfinal(FILE *fd, double *x, double *y, double *rpar, char *vararray[], int numvars)
     // my routine - for final values 
{
  int j;
  for(j=0;j<numvars;j++){
    fprintf(fd, "(%.4e): %s %.4e\n", *x, vararray[j], y[j]);
  }
}


void runvals(FILE *fd, double *x, double *y, int numvars, char *vararray[], double *rpar)
     // my routine - to output the last-run values 
{
  int j;
  fprintf(fd, "/* This file is generated by \"autoreg.cc\"  */\n/* It contains the last-run values of the variables */\n\n");
  fprintf(fd, "#define RUNFILE\n\n");
  fprintf(fd, "int numrunsetparams = %d;\n\n", numvars);
  fprintf(fd, "struct paramstr runparamset[%d] = {\n", numvars);
  for(j=0;j<numvars-1;j++){
    fprintf(fd, "{\"%s_run\", %.4e},\n", vararray[j], y[j]);
  }
  fprintf(fd, "{\"%s_run\", %.4e}};\n\n", vararray[j], y[j]);
}


/////////////////////////////////////////
//                                     //
//                                     //
//                                     //
//       NOW ENERING MAIN              //
//                                     //
//                                     //
//                                     //
/////////////////////////////////////////

int main(int argc, char *argv[])
{

  // for the definition of the (static) workspace 

  int ND;
  int NS = 20;
  int LWORK;
  int LIWORK;

  double *yvq;
  double *work;
  int *iwork;
  double xtm,xend;
  int i, j, idid;
  int kdown;
  int kup;
  double stdytime=0.0;

  int input_suc;
  int m;

  double *fy; // (for checking purposes) 
  double *fz;
  char *paramfilename; // name of the file holding current values of the parameters	
  // there are parameters in the differential equation 

  char **settable;
  int numsettable = 0;
  double *setpvals;


  //dimension of the system 
  int nmv;

  // do not compute the jacobian analytically 
  int ijac=0;

  // structure of the jacobian 
  int mljac; // jacobian computed numerically so shouldn't matter 
  int mujac=0;
  // differential equation is not in explicit form 
  int imas=1;
  int mlmas; // these seem to be mainly about storage 
  int mumas=0;

  int lmas;

  int ipar=0; // no integer parameters 
  // output routine is used during integration 
  int iout=1;
	
  // required tolerance 
  int itol=1;
  double *rtol;
  double *atol;

  double *rtol1;
  double *atol1;

  // initial step size 
  double h=1.0e-6;
	
  int lwork;
  int liwork;
  int numDEs=0;
  char *nDEs;

  FILE *ftmp;

  char staticfile[100];
  char staticpfile[100];
  char statictfile[100];
  char dynamicfile[100];
  char dynamicpfile[100];
  char dynamictfile[100];

  /////////////////////////////////
  //                             //
  //      DECLARATIONS OVER      //
  //                             //
  /////////////////////////////////

  if(argc<4){
    fprintf(stderr, "ERROR in src/autoreg.cc. You need to supply four arguments.\n");
    return -1;
  }

  snprintf(staticfile, 100, "%s", argv[3]);
  snprintf(staticpfile, 100, "%s.pvals", argv[3]);
  snprintf(statictfile, 100, "%s.tvars", argv[3]);
  snprintf(dynamicfile, 100, "%s_dyn", argv[3]);
  snprintf(dynamicpfile, 100, "%s_dyn.pvals", argv[3]);
  snprintf(dynamictfile, 100, "%s_dyn.tvars", argv[3]);
  cpfile(dynamictfile, "sysfiles/alltempstrs2.dat");
  cpfile(statictfile, "sysfiles/alltempstrs2.dat");

  if (argc>=5){
    qvars = genarraydat(argv[4], &numqvars);
    brf=fopen("outdat/__tmp_contplot.dat", "w");
    if(!brf){fprintf(stderr, "WARNING: brief output file \"outdat/__tmp_contplot.dat\" could not be opened.\n");}
    cpfile("outdat/__tmp_contplot.dat.tvars", "sysfiles/alltempstrs2.dat");
  }

  nDEs = getcvalfromfile("sysfiles/othersysparams.dat", "numDEs");
  numDEs = atoi(nDEs);
  free(nDEs);

  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  settable = genarraydat("sysfiles/settable.dat", &numsettable);
  autoregvars = genarraydat("sysfiles/autoregout.dat", &numautoregvars);
  matvars = genarraydat("sysfiles/output.dat", &nummatvars);
  ord = ivector(0, nummatvars-1);

  ND = numvars;
  LWORK = (int) ((NS+1)*ND*ND+(3*NS+3)*ND+20);
  LIWORK = (int) ((2+(NS-1)/2)*ND+20);
  nmv = numvars;
  mljac = numvars;
  mlmas = numvars;
  lmas = numvars;
  lwork = LWORK;
  liwork = LIWORK;
  work = dvector(0, LWORK-1);
  iwork = ivector(0, LIWORK-1);
  yvq = dvector(0, ND);
  rtol = dvector(0, numvars-1);
  atol = dvector(0, numvars-1);
  rtol1 = dvector(0, numvars-1);
  atol1 = dvector(0, numvars-1);
  fz = dvector(0, numvars*numvars-1);
  rpar = dvector(0, numparams-1);
  setpvals = dvector(0, numsettable-1);

  //
  // files
  //


  file = "outdat/output";
  file2 = "sysfiles/runparamvals.c";
  fd = fopen(file, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"autoreg.cc\": file %s could not be opened for writing.\n", file);
    return -1;
  }
  fd1 = fopen(staticfile, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"autoreg.cc\": file %s could not be opened for writing.\n", staticfile);
    return -1;
  }
  fd2 = fopen(file2, "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"autoreg.cc\": file %s could not be opened for writing.\n", file2);
    return -1;
  }

  // let the system know that the program is running

  ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
  if(ftmp == NULL){
    fprintf(stderr, "WARNING in \"autoreg.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
  }
  else{
    fprintf(ftmp, "%d\n", getpid());
    fclose(ftmp);
  }


  fprintf(fd, "\n***Parameter values***\n\n");

  // initialise the tolerances 

  for (i=0;i<numvars;i++){
    rtol[i] = 2.0e-3;
    atol[i] = 1.0e-2*rtol[i];
  }

  for (i=0;i<numDEs;i++){ // the differential variables 
    rtol[i] = 1.0e-6;
    atol[i] = 1.0e-4*rtol[i];
  }

  for (i=0;i<numvars;i++){ // after the initial step... 
    rtol1[i] = 1.0e-6;
    atol1[i] = 1.0e-4*rtol[i];
  }

  // 
  // 
  // parameter and variable handling
  // 
  // 

  paramfilename = strdup(argv[2]);

  if(setpvalsfromfile(paramfilename, settable, setpvals, numsettable)!=0){
    free(paramfilename);
    fprintf(stderr, "EXITING because of parameter setting problem.\n");
    // ensuring that the system knows that the simulation has finished
    ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
    if(ftmp == NULL){
      fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
    }
    else{
      fprintf(ftmp, "END PROBLEM\n");
      fclose(ftmp);
    }
    return -1;
  }
  pfunc(setpvals, rpar);


  free(paramfilename);

  // initialise the parameters and print them out 


  setval1("", 2.0, paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);

  for (i=0;i<numparams;i++){
    fprintf(fd, "%s =  %.4g\n", paramarray[i], rpar[i]);
  }

  //
  //
  // initial conditions
  //
  //


  fprintf(fd, "\n\n***initial conditions***\n\n");
	
  // set initial values and print them out 
  xtm=0.0;
  setinits1(yvq, rpar);

  for (i=0;i<numvars;i++){
    fprintf(fd, "%s =  %.4g\n", vararray[i], yvq[i]);
  }

  // rhs check 

  fprintf(fd, "\n\n**rhs check - if we are near an equilibrium, these should all be small**\n\n");

  fy = dvector(0, numvars);
  rhs(&numvars, &xtm, yvq, fy, rpar, &ipar);
  fprintf(fd, "\n\n");
  for(i=0;i<numvars;i++)
    fprintf(fd, "D%s = %.4e\n", vararray[i], fy[i]);
  fprintf(fd, "\n\n");
	
  // mass check 
  fprintf(fd, "**mass check**\n");

  mas(&numvars, fz, &lmas, rpar, &ipar);
  fprintf(fd, "\n\n");
  for(i=0;i<numvars;i++)
    for(j=0;j<numvars;j++)		
      if(fz[numvars*i+j] != 0.0)
	fprintf(fd, "fz[%d][%d] = %.4e\n", i, j, fz[numvars*i+j]);
	

  // endpoint of integration
  //xend=60.0;

  // set default values 
  for(i=0; i<20; i++){
    iwork[i]=0;
    work[i] = 0.0;	
  }

  // remember here that work[0] here means work[1] in the fortran literature, etc.
  iwork[3] = 2;
  iwork[4] = numDEs; // number of index 1 variables
  iwork[5] = numvars-numDEs; // number of index 2 variables
  work[0] = 1.0e-15; // rounding unit
  work[6] = 100.0; // maximum step size

  // check parameters being passed 

  fprintf(fd, "n = %d\n", nmv);
  fprintf(fd, "rtol = %.4e\n", *rtol);
  fprintf(fd, "atol = %.4e\n", *atol);
  fprintf(fd, "ijac = %d\n", ijac);
  fprintf(fd, "mljac = %d\n", mljac);
  fprintf(fd, "mujac = %d\n", mujac);
  fprintf(fd, "imas = %d\n", imas);
  fprintf(fd, "mlmas = %d\n", mlmas);
  fprintf(fd, "mumas = %d\n", mumas);
  fprintf(fd, "iout = %d\n", iout);
  fprintf(fd, "ipar = %d\n", ipar);


  // initial values - are we using the last run set of values or not? 
#ifndef CONTIN // we set this
  setinits1(yvq, rpar);
#elif ! defined RUNFILE // this is defined if runparamvals.c exists
  setinits1(yvq, rpar);
#else
  for(i=0;i<numvars;i++)
    yvq[i] = runparamset[i].pvalue;
#endif

  h=1.0e-6;


  input_suc = inputautoregdat(argv[1], &numchangeparams, chosen_param, 10, &time_step, step_vals, init_vals, &min_val, &max_val, &hyst, &nummorphs, morphs, morphvals, 10, dynamicpfile);

  if(input_suc != 0){
    fprintf(stderr, "ERROR in \"autoreg.cc\" - data input unsuccessful. Exit status = %d. Now exiting.\n", input_suc);
    ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
    if(ftmp == NULL){
      fprintf(stderr, "WARNING in \"autoreg.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
    }
    else{
      fprintf(ftmp, "END PROBLEM\n");
      fclose(ftmp);
    }
    while (--numchangeparams >= 0)
      free (chosen_param[numchangeparams]); /* free elements of array */

    while (--nummorphs >= 0)
      free (morphs[nummorphs]); /* free elements of array */

    return -1;
  }

  fprintf(stderr, "nummorphs = %d\n", nummorphs);

  //
  // parameters to be reset
  //

  for(i=0;i<nummorphs;i++){
    fprintf(stderr, "Morphing %s:  %.4f --> %.4f\n", morphs[i], morphvals[2*i], morphvals[2*i+1]);
  }

  if(nummorphs>0){
    multimorph1(&nmv,rhs,yvq,&h,
	       rtol,atol,&itol,
 	       jac_dummy,&ijac,&mljac,&mujac,
 	       mas,&imas,&mlmas,&mumas,
 	       soldummy,&iout,
 	       work,&lwork,iwork,&liwork,rpar,&ipar,&idid,
	       numparams, paramarray, morphs, morphvals, nummorphs,
	       10, 100000, dynamicpfile, setpvals, settable, numsettable);

    fprintf(stderr, "Morphing complete. Now simulating.\n");
  }

  solmat0(); // print the heading
  solautoreg0(fd1); // header for the autoreg file

  xtm=0.0; //time 

  if (numchangeparams!=0 && step_vals[0]!=0){
    kdown = (int)(floor((init_vals[0] - min_val)/step_vals[0]));
    kup = (int)(floor((max_val - min_val)/step_vals[0]));
    if(hyst == 1)
      ktot = kdown + 2*kup;
    else
      ktot = kdown + kup + 1;
  }
  else if (numchangeparams==0){ // Poincare map
    stdytime=init_vals[0];
    kdown=(int)stdytime;
    kup=(int)(step_vals[0]);
    ktot=kdown+ kup+1;
  }
  else{
    fprintf(stderr, "WARNING in \"autoreg.cc\" - step_vals: must not be empty. No simulation will be run\n");
    kdown=0;
    kup=0;
    ktot=0;
  }

  for(kps=0;kps<ktot;kps++){ 
    fprintf(stderr, "*");

    // endpoint of integration
    xend=(kps+1)*time_step;

    // set default values
    for(i=0; i<20; i++){
      iwork[i]=0;
      work[i]=0.0;
    }

  // remember here that work[0] here means work[1] in the fortran literature, etc.

    iwork[3] = 2;
    iwork[4] = numDEs; // number of index 1 variables
    iwork[5] = numvars-numDEs; // number of index 2 variables
    work[0] = 1.0e-15; 
    work[6] = 100.0; // maximum step size

    fprintf(fd, "\n**********\n* radau5 *\n**********\n\n");


    if (kps<=kdown){
      for (m=0;m<numchangeparams;m++){
	setval1(chosen_param[m], init_vals[m] - step_vals[m]*kps, paramarray, numparams, rpar, dynamicpfile, 1, settable, numsettable, setpvals);
      }
    }
    else{
      if(hyst == 1 && kps>kdown+kup){
        for (m=0;m<numchangeparams;m++){
	  setval1(chosen_param[m], init_vals[m]+step_vals[m]*(kup - kdown) - step_vals[m]*(kps-kdown-kup), paramarray, numparams, rpar, dynamicpfile, 1, settable, numsettable, setpvals);
        }
      }
      else{
        for (m=0;m<numchangeparams;m++){
	  setval1(chosen_param[m], init_vals[m]-step_vals[m]*kdown + step_vals[m]*(kps-kdown), paramarray, numparams, rpar, dynamicpfile, 1, settable, numsettable, setpvals);
        }
      }
    }
    cpfile(staticpfile, dynamicpfile);
    cpfile("outdat/__tmp_contplot.dat.pvals", dynamicpfile);

    RADAU5(&nmv,rhs,&xtm,yvq,&xend,&h,
	   rtol,atol,&itol,
	   //rtol1, atol1,
	   jac_dummy,&ijac,&mljac,&mujac,
	   mas,&imas,&mlmas,&mumas,
	   solmat,&iout,
	   work,&lwork,iwork,&liwork,rpar,&ipar,&idid);
    // print final solution 

    if (kps > kdown){
      solautoreg(fd1, yvq, rpar);
      solfinal(fd, &xtm, yvq, rpar, vararray, numvars);
    }

    // print statistics 
    fprintf(fd, "rtol = %1.2e\n", *rtol);
    fprintf(fd, "fcn= %i jac= %i step= %i accpt= %i rejct= %i dec= %i sol= %i\n", iwork[13],iwork[14],iwork[15],iwork[16],iwork[17],iwork[18],iwork[19]);

  }
  runvals(fd2, &xtm, yvq, numvars, vararray, rpar);

  fclose(fd);
  fclose(fd1);
  fclose(fd2);
  if(brf){fclose(brf);}
  fprintf(stderr, "\n");

  // Let the system know that the program has finished

  ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
  if(ftmp == NULL){
    fprintf(stderr, "WARNING in \"autoreg.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
  }
  else{
    fprintf(ftmp, "END\n");
    fclose(ftmp);
  }

  while (--numchangeparams >= 0)
    free (chosen_param[numchangeparams]); /* free elements of array */

  while (--nummorphs >= 0)
    free (morphs[nummorphs]); /* free elements of array */


  free_dvector(work, 0, LWORK-1);
  free_ivector(iwork, 0, LIWORK-1);
  free_dvector(yvq, 0, ND);
  free_dvector(rtol, 0, numvars-1);
  free_dvector(atol, 0, numvars-1);
  free_dvector(rtol1, 0, numvars-1);
  free_dvector(atol1, 0, numvars-1);
  free_dvector(fz, 0, numvars*numvars-1);
  free_dvector(rpar, 0, numparams-1);
  freearraydat(vararray, numvars);
  freearraydat(paramarray, numparams);
  freearraydat(matvars, nummatvars);
  freearraydat(autoregvars, numautoregvars);
  free_ivector(ord, 0, nummatvars-1);

  //  compdatfiles("outdat/plot", "../sysarchiveold/braincirc2.0/outdat/plot");

  return 0;
}


