/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// Checks the modules for redefinition of parameters or variables

#include <fcntl.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "common/stringmanip.h"
#include "syssize.h"
#define MAX_PATH 1024
void varloc_fcn(char *n, FILE *outfile);
void procloc_fcn(char *n, FILE *outfile);
void dirwalk(char *dir, void (*fcn)(char *, FILE *), char *outfilename);

int numvars=0;
char *vars[MAXVARS];

int numparams=0;
char *params[MAXPARAMS];

// The file sysfiles/varloc.dat will store the locations of the various variables and parameters...

int main()
{
  FILE *fd;
  char line[2000];
  char *varname;
  char *varname1;
  fd = fopen("sysfiles/sysstate/which_moduledir_file", "r");
  if(fd==NULL){
    fprintf(stderr, "ERROR in checkvp.c: file \"sysfiles/sysstate/which_moduledir_file\" could not be opened.\n");
    return -1;
  }
  getline(fd, line, 2000);
  varname1 = getnthblk(line, 1);
  varname = strchop2(varname1, 0, strlen(varname1)-1);
  fclose(fd);
  fprintf(stderr, "checking for repetition of parameters or variables...\n");
  dirwalk(varname, &varloc_fcn, "sysfiles/varloc.dat");
  dirwalk(varname, &procloc_fcn, "sysfiles/procloc.dat");

  free_datarray(vars, numvars);
  free_datarray(params, numparams);
  free(varname);
  free(varname1);
  return 0;
}

void varloc_fcn(char *n, FILE *outfile){
  FILE *fd;
  char line[2000] = "";
  char tempname[50] = "";
  char *varname;
  int numvars1;
  int numparams1;
  int len, len1;
  extern int numvars;
  extern char *vars[];
  extern int numparams;
  extern char *params[];

  //
  // variables
  //

  //  fprintf(stderr, "\t%s\n", n);
  if(strstr(n, ".dt") == NULL)
    return;
  if(strstr(n, "~") != NULL) // back-up file
    return;
  fd = fopen(n, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in routine varloc_fcn in checkvp.c: file: %s could not be opened...\n", n);
    return;
  }

  len=10;
  while(len > 0){

    // stop when you get to VARS or TEMPVS
    while ((len = getline(fd, line, 2000)) > 0 && strstr(line, "VARS") == NULL && strstr(line, "TEMPVS") == NULL){}

    if(strstr(line, "VARS") != NULL)
      strcpy(tempname, "endVARS"); // safe
    else if(strstr(line, "TEMPVS") != NULL)
      strcpy(tempname, "endTEMPVS"); // safe

    while ((len = getline(fd, line, 2000)) > 0 && strstr(line, tempname) == NULL){
      if(!iscomsep(line)){
	varname = getnthwd(line, 1);
	numvars1 = addvar1(numvars, varname, vars, MAXVARS);
	if(numvars1 == numvars)
	  fprintf(stderr, "WARNING: The variable \"%s\" is defined more than once.\n", varname);
	else{
	  // only the end part of the filename...
	  len1=strlen(n);
	  while(len1>=0 && (n[len1] != '/'))
	    len1--;
	  if(len1==0)
	    fprintf(outfile, "%s\t%s\n", varname, n);
	  else
	    fprintf(outfile, "%s\t%s\n", varname, n+len1+1);
	  numvars = numvars1;
	}
	free(varname);
      }
    }
  }


  fclose(fd);


  //
  // now the parameters
  //

  fd = fopen(n, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in routine checkvp.c: file: %s could not be opened...\n", n);
    return;
  }
  while ((len = getline(fd, line, 2000)) >0 && strstr(line, "PARAMS") == NULL){}
  while ((len = getline(fd, line, 2000)) >0 && strstr(line, "endPARAMS") == NULL){
    if(!iscomsep(line)){
      varname = getnthwd(line, 1);
      numparams1 = addvar1(numparams, varname, params, MAXPARAMS);
      if(numparams1 == numparams)
	fprintf(stderr, "WARNING: The parameter \"%s\" is defined more than once.\n", varname);
      else{
	// only the end part of the filename...
	len1=strlen(n);
	while(len1>=0 && (n[len1] != '/'))
	  len1--;
	if(len1==0)
	  fprintf(outfile, "%s\t%s\n", varname, n);
	else
	  fprintf(outfile, "%s\t%s\n", varname, n+len1+1);
	numparams = numparams1;
      }
      free(varname);
    }
  }
  fclose(fd);
  return;
}

void procloc_fcn(char *n, FILE *outfile){
  FILE *fd;
  char line[2000] = "";
  char tempname[50] = "";
  char varname[50];
  int len, len1;

  //
  // variables
  //

  //  fprintf(stderr, "\t%s\n", n);
  if(strstr(n, ".dt") == NULL)
    return;
  if(strstr(n, "~") != NULL) // back-up file
    return;
  fd = fopen(n, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in routine checkvp.c: file: %s could not be opened...\n", n);
    return;
  }

  len=10;
  while(len>0){
    // stop when you get to REACTIONS or ALGRELS of DIFFEQS
    while (((len = getline(fd, line, 2000)) > 0) && !strstr(line, "REACTIONS") && !strstr(line, "ALGRELS") & !strstr(line, "DIFFEQS") && !strstr(line, "TEMPLATES")){}

    if(strstr(line, "REACTIONS"))
      strcpy(tempname, "endREACTIONS"); // safe
    else if(strstr(line, "ALGRELS"))
      strcpy(tempname, "endALGRELS"); // safe
    else if(strstr(line, "DIFFEQS"))
      strcpy(tempname, "endDIFFEQS"); // safe
    else if(strstr(line, "TEMPLATES"))
      strcpy(tempname, "endTEMPLATES"); // safe

    while ((len = getline(fd, line, 2000)) > 0 && strstr(line, tempname) == NULL){
      if(!iscomsep(line)){
	getnthword(line, varname, 50, 1);
	if(strcmp(varname, "name") == 0){
	  getnthword(line, varname, 50, 2);
	  len1=strlen(n);
	  while(len1>=0 && (n[len1] != '/'))
	    len1--;
	  if(len1==0)
	    fprintf(outfile, "%s\t%s\n", varname, n);
	  else
	    fprintf(outfile, "%s\t%s\n", varname, n+len1+1);

	}
      }
    }
  }

  fclose(fd);
  return;
}

void dirwalk(char *dir, void (*fcn)(char *, FILE *), char *outfilename)
{
  // adapted from Kernighan and Ritchie
  char name[MAX_PATH];
  FILE *fd;
  struct dirent *dp;
  DIR *dfd;

  if ((dfd = opendir(dir)) == NULL){
    fprintf(stderr, "ERROR in dirwalk: can't open %s\n", dir);
    return;
  }

  fd = fopen(outfilename, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in dirwalk: can't open %s\n", outfilename);
    closedir(dfd);
    return;
  }

  fprintf(fd, "// Generated by checkvp.c. Do not edit.\n\n");

  while ((dp = readdir(dfd)) != NULL){
    if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0 || strstr(dp->d_name, "~") != NULL)
      continue;
    if(strlen(dir)+strlen(dp->d_name)+2 > sizeof(name))
      fprintf(stderr, "ERROR in dirwalk in checkvp.c: name %s/%s too long\n", dir, dp->d_name);
    else{
      snprintf(name, MAX_PATH, "%s/%s", dir, dp->d_name);
      (*fcn)(name, fd);
    }

  }
  fclose(fd);
  closedir(dfd);
}

