/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

*/

#include "stringmanip.h"
#include "nrsimp.h"
#include <sys/stat.h>

char *getvarname(char fname[], int varindex);
int writetwocommands(char *filename, char *datafilename, char *var1, char *ind1, char *var2, char *ind2, char *var3, char *ind3, char *ops, int cont); // for two plot variables


int mathcount(char *s){
  int i=0;
  int count=0;
  while(s[i]!='\0'){
    if(((s[i] == '+') && (i!=0) && (s[i-1] != 'e')) || ((s[i] == '-') && (i!=0) && (s[i-1] != 'e')) || (s[i] == '/') || ((s[i] == '*') && (s[i-1]!='*'))){
      count++;
    }
    i++;
  }
  return count;
}



char *getvarname(char fname[], int varindex)
{
  int len, i=0;
  FILE *fd;
  char temp[3000];
  char *outstr;

  
  fd = fopen(fname, "r");
  if (fd == NULL){
    fprintf(stderr, "ERROR in getvarname in \"src/common/outfuncs.c\": %s could not be opened for reading.\n", fname);
    outstr = strdup("");
    return outstr;
  }

  if((len = getline(fd, temp, 3000)) > 0){ // max length 3000
    while((temp[i] == '#') || (temp[i] == '%') || (temp[i] == '/') || (temp[i] == '*')){i++;} /* ignore initial characters */
    if(i == 0){ /* no marker character */
      fprintf(stderr, "First line must start with \"#\" or \"%%\" or \"/\" or \"*\"...\n");
      outstr = strdup("");
      fclose(fd);
      return outstr;
    }
    else{
      outstr = getnthtkn(temp+i, varindex);
      fclose(fd);
      return outstr;
    }
  }
  else{
    fprintf(stderr, "Error in getvarname: %s is not a valid data file...\n", fname);
    fclose(fd);
    outstr = strdup("");
    return outstr;
  }
  outstr=strdup("");
  return outstr;
}



int writetwocommands(char *filename, char *datafilename, char *var1, char *ind1, char *var2, char *ind2, char *var3, char *ind3, char *ops, int cont){

  FILE *fd1;

  fd1 = fopen(filename, "w");

  if (fd1 == NULL){
    fprintf(stderr, "File %s cannot be opened\n", filename);
    return -1; //exit: command file can't be opened
  }

  fprintf(fd1, "#gnuplot script file for plotting\n");
  fprintf(fd1, "#the data in %s\n", datafilename);

  fprintf(fd1, "set multiplot\n");
  fprintf(fd1, "set key\n");
  fprintf(fd1, "set title \"\"\n");
  fprintf(fd1, "set xlabel \"%s\"\n", var1);
  fprintf(fd1, "set ylabel \"%s\"\n", var2);
  fprintf(fd1, "set size 1.0,0.5\n");
  fprintf(fd1, "set origin 0.0,0.0\n");
  fprintf(fd1, "%s\n", ops);
  fprintf(fd1, "plot \"%s\" using (%s):(%s) t \"%s\"\n", datafilename, ind1, ind2, var2);
  fprintf(fd1, "set key\n");
  fprintf(fd1, "set title \"%s and %s vs %s in file %s\"\n", var3, var2, var1, datafilename);
  fprintf(fd1, "set xlabel \"\"\n");
  fprintf(fd1, "set ylabel \"%s\"\n", var3);
  fprintf(fd1, "set size 1.0,0.5\n");
  fprintf(fd1, "set origin 0.0,0.5\n");
  fprintf(fd1, "%s\n", ops);
  fprintf(fd1, "plot \"%s\" using (%s):(%s) t \"%s\"\n", datafilename, ind1, ind3, var3);
  fprintf(fd1, "set nomultiplot\n");
  if(cont == 1){
    fprintf(fd1, "pause 1\n");
    fprintf(fd1, "if(`cat sysfiles/sysstate/braincircrunning`==1) reread\n");

  }
  //  fprintf(fd1, "pause 100\n");

  fclose(fd1);
  return 0;
}


char *transexpr(char *s, char *temparray[], char *transarray[], int numvars, char *datafilename, char *paramfilestr, char *rems[], int numrems){
  char *tempstr;
  char *temp, *temp1;
  int numvars1, numvars2;
  char *vars[100];
  int max = 100;
  char *indcs[100];
  int j=0, ind, l;
  char pval[50];
  char systemcom[60];

  if(!fileexists(datafilename)){
    tempstr = strdup("");
    fprintf(stderr, "WARNING in transexpr: file %s does not seem to exist.\n", datafilename);
    return tempstr;
  }

  temp = transfunc2it(s, temparray, transarray, numvars);
  temp1 = transpowit(temp);
  numvars2=getnumallvars(temp1);
  numvars1=getnumvars(temp1, rems, numrems);
  if(checkbracs(temp1)!=0){
    tempstr = strdup("");
    free(temp); free(temp1);
    return tempstr;
  }
/*   if((getnumtokens(temp1)+numvars1-numvars2)!=(mathcount(temp1)+1)){ */
/*     fprintf(stderr, "%s is a dodgy mathematical expression.\nIt appears to have %d operators and %d values.\n", temp1, mathcount(temp1), getnumtokens(temp1)+numvars1-numvars2); */
/*     tempstr = strdup(""); */
/*     free(temp); free(temp1); */
/*     return tempstr; */
/*   }*/

  numvars1=getvars(temp1, vars, max, rems, numrems);

  // construct a translation table

  for (j=0;j<numvars1;j++){ // for each variable within the expression
    ind = getdatfileindex(datafilename, vars[j]);
    if(ind < 0){ // not a variable
      snprintf(systemcom, 60, "test -r %s", paramfilestr);
      l=system(systemcom);
      if(l!=0){ // problem with the parameter value file
	fprintf(stderr, "WARNING in transexpr: %s not a variable in %s, and parameter file %s could not be found.\n", vars[j], datafilename, paramfilestr);
	tempstr = strdup("");
	free_datarray(vars, numvars1); free_datarray(indcs, j); free(temp); free(temp1);
	return tempstr;
      }
      else{
	ind = getparamvaluestr(paramfilestr, vars[j], pval, 50);
	if(ind != 1){
	  fprintf(stderr, "WARNING in transexpr: variable %s is not a temporary variable, and could not be found in file %s or file %s.\n", vars[j], datafilename, paramfilestr);
	  tempstr = strdup("");
	  free_datarray(vars, numvars1); free_datarray(indcs, j); free(temp); free(temp1);
	  return tempstr;
	}
	else{ // a parameter
	  indcs[j] = strdup(pval);
	}
      }
    }
    else{ // an output variable
      indcs[j] = (char*) malloc(sizeof(char) * (10));
      sprintf(indcs[j], "$%d", ind);
    }

  } // next variable within an expression

  tempstr = transfunc2(temp1, vars, indcs, numvars1);
  free(temp); free(temp1);

  free_datarray(vars, numvars1);
  free_datarray(indcs, numvars1);
  return tempstr;

}

//
// print a list of all dependencies of a plot expression to a file
// This allows the construction of brief output files which only list
// the variables being chosen for continuous output.
//

int printexprdeps(char *fname, char *s, char *tempstrfile, char *outvarfilename, char *paramfilename, char *remsfile){
  char **rems;
  int numrems=0;
  char **remsall;
  char *temp;
  int maxv = 200;
  char *varray[maxv];
  int numvarray;
  int i;
  FILE *fd;
  char **parray;
  char **deps;
  int nump;

  nump = genbracarraysfromdat(tempstrfile, &parray, &deps);
  temp = transfunc2it(s, parray, deps, nump);
  freearraydat(parray, nump);   freearraydat(deps, nump);

  fd=fopen(fname, "w");
  if(!fd){
    fprintf(stderr, "ERROR in printexprdeps: file %s could not be opened.\n", fname);
    free(temp);
    return -1;
  }

  rems=genarraydat(remsfile, &numrems);
  remsall = (char **) malloc(sizeof(char *) * (numrems + 3));
  for(i=0;i<numrems;i++)
    remsall[i]=strdup(rems[i]);
  remsall[i++]=strdup("xvar");
  remsall[i++]=strdup("multi");
  remsall[i++]=strdup("sim");
  freearraydat(rems, numrems);

  numvarray=getvars(temp, varray, maxv, remsall, numrems+3);
  for(i=0;i<numvarray;i++)
    fprintf(fd, "%s\n", varray[i]);
  fclose(fd);

  free(temp);
  free_datarray(varray, numvarray);
  free_datarray(remsall, numrems+3);
  free((char *) (remsall));
  return 0;

}

int makemultiplotfile(char *xindex, char *buf, char *filename, char *datafilename, char *ops, int cont, char *remsfile)
{
  char **rems;
  int numrems = 0;
  char *v[50];
  int *suc;
  int i, j, k, max = 50;
  int numexprs;
  char **outstrs;
  char **fvarstr;
  int totexprs = 0;
  FILE *fd1;
  char *datfiles[10];
  char **paramfiles;
  char **tvarfiles;
  int numdatfiles=0;
  int totnum;
  int *nd;
  int ftot=0;

  char **temparray = NULL;
  char **transarray = NULL;
  int numvars=0;
  char *tmp;

  rems=genarraydat(remsfile, &numrems);
  numdatfiles = chemgets3(datafilename, datfiles, 10, " :,");
  if(numdatfiles<=0){
    fprintf(stderr, "ERROR in makemultiplotfile: no data files specified\n");
    freearraydat(rems, numrems);
    free_datarray(datfiles, numdatfiles);
    return -1; // xvar not found
  }

  paramfiles=charar(0, numdatfiles-1);
  tvarfiles=charar(0, numdatfiles-1);
  for(k=0;k<numdatfiles;k++){ // remove terminal ':' if necessary, add 'outdat/' if necessary
    if (datfiles[k][strlen(datfiles[k])-1]==':' || datfiles[k][strlen(datfiles[k])-1]==','){datfiles[k][strlen(datfiles[k])-1]=0;}
    tmp=addifnecc(datfiles[k], "outdat/", "");
    free(datfiles[k]);datfiles[k]=strdup(tmp);free(tmp);
    paramfiles[k] = (char*) malloc(sizeof(char) * (strlen(datfiles[k]) + 10));
    tvarfiles[k] = (char*) malloc(sizeof(char) * (strlen(datfiles[k]) + 10));
    sprintf(paramfiles[k], "%s.pvals", datfiles[k]); // safe
    sprintf(tvarfiles[k], "%s.tvars", datfiles[k]); //safe
  }

  // x variable

  fvarstr=charar(0, numdatfiles-1);
  nd=ivector(0, numdatfiles-1);
  for(k=0;k<numdatfiles;k++){
    numvars = genbracarraysfromdat(tvarfiles[k], &temparray, &transarray);
    fvarstr[k] = transexpr(xindex, temparray, transarray, numvars, datfiles[k], paramfiles[k], rems, numrems);
    if(!isonlyspace(fvarstr[k])){nd[k]=1;ftot++;}
    else{nd[k]=0;}
    freearraydat(temparray, numvars);
    freearraydat(transarray, numvars);
  }

  if(ftot==0){
    freearraydat(rems, numrems);
    free_datarray(datfiles, numdatfiles);
    freearraydat(fvarstr, numdatfiles);
    free_ivector(nd, 0, numdatfiles-1);
    fprintf(stderr, "ERROR in makemultiplotfile: Either no files exist or the x variable %s could not be translated.\n", xindex);
    return -1; // xvar not found
  } 

  // expressions

  numexprs = chemgets2(buf, v, max, ':');

  if(numexprs==max)
    fprintf(stderr, "WARNING: too many expressions. A maximum of %d will be translated.\n", max);


  totnum=numdatfiles*numexprs; // total number of things to translate
  outstrs=charar(0, totnum-1);
  suc=ivector(0, totnum-1);

  for(k=0;k<numdatfiles;k++){
    if(nd[k]==0){ // problem with file or x variable
      for(i=0;i< numexprs;i++){
	outstrs[k*numexprs+i] = strdup("");
	suc[k*numexprs+i] = 0;
      } // next expression
    }
    else{
      numvars = genbracarraysfromdat(tvarfiles[k], &temparray, &transarray);
      for(i=0;i< numexprs;i++){ // for each expression
	outstrs[k*numexprs+i] = transexpr(v[i], temparray, transarray, numvars, datfiles[k], paramfiles[k], rems, numrems);
	if(!isonlyspace(outstrs[k*numexprs+i])){
	  totexprs++;
	  suc[k*numexprs+i] = 1;
	}      
	else
	  suc[k*numexprs+i] = 0;
      } // next expression
      freearraydat(temparray, numvars);
      freearraydat(transarray, numvars);
    }
  }

  if(totexprs <= 0){
    fprintf(stderr, "ERROR: none of the expressions could be translated.\n");
    freearraydat(fvarstr, numdatfiles);
    free_datarray(v, numexprs);
    freearraydat(outstrs, totnum);
    freearraydat(rems, numrems);
    free_datarray(datfiles, numdatfiles);free_ivector(suc, 0, totnum-1);free_ivector(nd, 0, numdatfiles-1);

    return -2;
  }

  fd1 = fopen(filename, "w");
  if (!fd1){
    fprintf(stderr, "File %s cannot be opened for writing\n", filename);
    freearraydat(fvarstr, numdatfiles);free_datarray(v, numexprs); freearraydat(outstrs, totnum); freearraydat(rems, numrems); 
    free_datarray(datfiles, numdatfiles);free_ivector(suc, 0, totnum-1);free_ivector(nd, 0, numdatfiles-1);
    return -1; //exit: command file can't be opened
  }

  fprintf(fd1, "#gnuplot script file for plotting\n#the data in:\n");
  for(k=0;k<numdatfiles;k++)
    fprintf(fd1, "#\t\t%s\n", datfiles[k]);
  fprintf(fd1, "set key\nset title \"");
  for(i=0;i<numexprs;i++){
    if(suc[i] == 1)
      fprintf(fd1, "%s ", v[i]);
  }

  fprintf(fd1, "vs %s in file(s) ", xindex);
  for(k=0;k<numdatfiles;k++)
    fprintf(fd1, "%s ", datfiles[k]);
  fprintf(fd1, "\"\nset xlabel \"%s\"%s\n\nplot ", xindex, ops);


  j=0;
  for(k=0;k<numdatfiles;k++){
    for(i=0;i<numexprs;i++){
      if(suc[k*numexprs+i] == 1 && j < totexprs-1){
	fprintf(fd1, "\"%s\" using (%s):(%s) t \"%s (%s)\", ", datfiles[k], fvarstr[k], outstrs[k*numexprs+i], v[i], datfiles[k]);
	j++;
      }
      else if(suc[k*numexprs+i] == 1 && j == totexprs-1){
	fprintf(fd1, "\"%s\" using (%s):(%s) t \"%s (%s)\"\n", datfiles[k], fvarstr[k], outstrs[k*numexprs+i], v[i], datfiles[k]);
	j++;
      }
    }
  }


  if(cont == 1){
    fprintf(fd1, "pause 1\n");
    fprintf(fd1, "if(`cat sysfiles/sysstate/braincircrunning`==1) reread\n");
  }

  fclose(fd1);
  
  freearraydat(fvarstr, numdatfiles); free_datarray(v, numexprs);   freearraydat(outstrs, totnum);  freearraydat(rems, numrems);
  free_datarray(datfiles, numdatfiles);free_ivector(suc, 0, totnum-1);free_ivector(nd, 0, numdatfiles-1);
  return numdatfiles*numexprs - totexprs;

}




int plotall(char *xvar, char *plotstr, char *outputfile, char *ops, int contplot, char *remsfile)
{
  int i;
  char *xvarname;
  char systemcom[200];
  char psheaderstr[100];
  char plotheaderstr[100];
  char plotcomsstr[100];
  char gnuplotcom[100];
  char makecom[100];
  int numtokens;
  int flag = 0;
  int onevar = 0;
  int suc;
  char plotstr1[500];
  char plotstr2[550];
  char **parray = NULL;
  char **deps = NULL;
  int nump;
  char *v[10];
  char *outstrs[10];
  int i1=-1, i2=-1;
  int max=10;
  char **rems;
  int numrems = 0;
  char *outputfstr;
  char *xvarstr;
  int startind;
  char *tmp;
  int numf;
  char *tvarfile;
  char *pvalfile;



  if(strlen(plotstr) > 400){
    fprintf(stderr, "ERROR in plotall: plot string %s is too long.\n", plotstr);
    return -3; // generic return
  }

  // option getting failures

  if(getoption("userfiles/gen_options", "GNUPLOT", gnuplotcom, 100)!=0)
    return -9;
  if(getoption("userfiles/gen_options", "PLOTCOMS", plotcomsstr, 100)!=0)
    return -9;
  if(getoption("userfiles/gen_options", "PLOTHEADER", plotheaderstr, 100)!=0)
    return -9;
  if(getoption("userfiles/gen_options", "PSHEADER", psheaderstr, 100)!=0)
    return -9;
  if(getoption("userfiles/gen_options", "MAKE", makecom, 100)!=0)
    return -9;

  rems=genarraydat(remsfile, &numrems);

  numf=getnumblocks(outputfile, " :,");

  if(numf<=0){
    fprintf(stderr, "ERROR in plotall: no files specified.\n");
    freearraydat(rems, numrems);
    return -1; // generic return
  }

  tmp = getnthblk(outputfile, 1); // in case of multiple files
  if (tmp[strlen(tmp)-1]==':' || tmp[strlen(tmp)-1]==','){tmp[strlen(tmp)-1]=0;}
  outputfstr=addifnecc(tmp, "outdat/", "");
  free(tmp); 

  // get x variable and remove from string

  if((strcmp(xvar, "") == 0) && !strstr(plotstr, "xvar:")){
    // no x variable specified - take first variable in first file
    xvarname = getvarname(outputfstr, 1);
    if(isonlyspace(xvarname)){
      free(xvarname); free(outputfstr);freearraydat(rems, numrems);
      return -4;
    }
    strcpy(plotstr1, plotstr); // safe
  }
  else if(strstr(plotstr, "xvar:")){
    // x variable specified in the input string
    xvarname = getname("xvar:", plotstr);
    if (xvarname[strlen(xvarname)-1]==':') // remove terminal colon if necessary
      xvarname[strlen(xvarname)-1]=0;
    startind = getstartind("xvar:", plotstr);
    strchop(plotstr1, plotstr, startind, 0);
  }
  else{
    xvarname=strdup(xvar);
    strcpy(plotstr1, plotstr);
  }
  
  // get plot string

  if(strcmp(plotstr1, "") == 0){ //empty string
    free(outputfstr); freearraydat(rems, numrems);free(xvarname);
    return -1; 
  }
  else if(strstr(plotstr1, "sim:") != NULL){

    if(numf>1){
      fprintf(stderr, "WARNING in plotall: with the \"sim\" option only one file is read.\n");
      flag= -13;
    }

    tvarfile=(char*) malloc(sizeof(char) * (strlen(outputfstr) + 10));
    sprintf(tvarfile, "%s.tvars", outputfstr); //safe
    pvalfile=(char*) malloc(sizeof(char) * (strlen(outputfstr) + 10));
    sprintf(pvalfile, "%s.pvals", outputfstr); //safe

    nump = genbracarraysfromdat(tvarfile, &parray, &deps);
    getrest("sim:", plotstr1, plotstr2, 550);
    numtokens =  chemgets2(plotstr2, v, max, ':');

    if(numtokens>2){numtokens=2; flag= -8;} // too many for sim

    onevar=0;
    xvarstr=transexpr(xvarname, parray, deps, nump, outputfstr, pvalfile, rems, numrems);
    for(i=0;i<numtokens;i++){
      outstrs[i] = transexpr(v[i], parray, deps, nump, outputfstr, pvalfile, rems, numrems);
      if(!isonlyspace(outstrs[i])){
	onevar++;
	if(i1<0){i1=i;}else if(i2<0){i2=i;}
      }
    }

    if(onevar ==0){ // no expressions could be translated
      freearraydat(parray, nump); freearraydat(deps, nump); free_datarray(v, numtokens); 
      free(outputfstr); freearraydat(rems, numrems); free(xvarname);free(xvarstr); free_datarray(outstrs, numtokens);
      return -12; 
    }
    else if(onevar ==2){
      if(contplot == 0){
	writetwocommands(plotcomsstr, outputfstr, xvarname, xvarstr, v[i1], outstrs[i1], v[i2], outstrs[i2], ops, 0);
	snprintf(systemcom, 200, "%s -persist %s %s 2> tempfiles/templog &", gnuplotcom, plotheaderstr, plotcomsstr);
      }
      else if(contplot == 1){
	writetwocommands(plotcomsstr, outputfstr, xvarname, xvarstr, v[i1], outstrs[i1], v[i2], outstrs[i2], ops, 1);
	snprintf(systemcom, 200, "%s %s %s 2> tempfiles/templog &", gnuplotcom, plotheaderstr, plotcomsstr);
      }
      else if (contplot == 2){
	writetwocommands(plotcomsstr, outputfstr, xvarname, xvarstr, v[i1], outstrs[i1], v[i2], outstrs[i2], ops, 0);
	snprintf(systemcom, 200, "%s %s %s %s > tempfiles/_temp_psfile.ps 2> tempfiles/templog", gnuplotcom, psheaderstr, plotheaderstr, plotcomsstr);

      }
      system(systemcom);
      freearraydat(parray, nump); freearraydat(deps, nump); free_datarray(v, numtokens); free_datarray(outstrs, numtokens);
      free(outputfstr); freearraydat(rems, numrems); free(xvarname);free(xvarstr);
      return flag;
    }
    else if (flag==-8 || numtokens==1) // too many or too few
      strcpy(plotstr1, plotstr2);
    else{
      strcpy(plotstr1, plotstr2);
      flag=-6; // some variables not found
    }
    free(tvarfile);free(pvalfile);
    freearraydat(parray, nump); freearraydat(deps, nump);

  }

  // default behaviour
  if(strstr(plotstr1, "multi:")){ // compatibility with old versions
    getrest("multi:", plotstr1, plotstr2, 550);
    strcpy(plotstr1, plotstr2); // safe
  }

  suc = makemultiplotfile(xvarname, plotstr1, plotcomsstr, outputfile, ops, contplot, remsfile);

  if(suc >= 0){
    if (contplot == 0)
      snprintf(systemcom, 200, "%s -persist %s %s 2> tempfiles/templog &", gnuplotcom, plotheaderstr, plotcomsstr);
    else if (contplot == 1)
      snprintf(systemcom, 200, "%s %s %s 2> tempfiles/templog &", gnuplotcom, plotheaderstr, plotcomsstr);
    else if (contplot == 2)
      snprintf(systemcom, 200, "%s %s %s %s > tempfiles/_temp_psfile.ps 2> tempfiles/templog", gnuplotcom, psheaderstr, plotheaderstr, plotcomsstr);
    system(systemcom);
  }
  else // failure in makemultiplotfile
    flag=-12;

  if(suc > 0)// some variables not found
    flag = -6;

  // partial or complete success
  free(outputfstr); freearraydat(rems, numrems); free(xvarname);

  return flag;

}
