/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h> // for atof
#include <math.h>
#include <ctype.h> // isalnum, etc
#include <dirent.h> // for directory parsing
#include "reac.h"
#include <time.h>
#include <sys/stat.h>

int overrideparamsfromfile(char *readfile, char *paramarray[], int numparams, double *rpar);
int setparamsfromfile(char *readfile, char *settable[], int numsettable, double *setpvals, double *rpar, void pfunc(double *, double *));
void unwrap(char *s);
void unwrap1(char *s);
char *timestr();
void wrap(char *s, int max);
int trim(char s[]);
void ent(char entered[], int max);
char *lrtrim(char s[]);
char *addifnecc(char *s, char *lstr, char *rstr);
int startswith(char *s, char *lstr);
int endswith(char *s, char *rstr);
char *getprefix(char s[]);
int getvindex(char s[]);
char *getpostindex(char s[]);
char *getgenname(char s[]);
int numflines(char *fname, int *maxline);
int numflinesdat(char *fname, int *maxline);
char *delunder(char *in);
char *wordtolower(char *in);
char *transpow(char *s);
char *transpowit(char s[]);
char *transsum(char *s);
char *transsumit(char *s);
char *strdup12(char *str, int i1, int i2);
char *enclose(char *s, char *lstr, char *rstr);
int bcheck(char *t);
char *bracifnecc(char *s);
int filehasstr(char *fname, char *str);
int iscom(char s[]);
int iscomsep(char s[]);
int iscomsepdat(char s[]);
int iscomblock(char s[]);
int isonlyspace(char *s);
int fileexists(char *fname);
char *getdname(char *fname);
char *getdmask(char *fname, char *rstr);
char *getdnamef(char *fname);
char *getfname(char *fname);
int modelexists(char *modname);
int archiveexists(char *modname);
void strchop(char *s, char *t, int n, int n1);
char *strchp(char *t, char *lstr, char *rstr);
void strchop1(char *s, int lim, char *t, int n, int n1);
char *strchop2(char *t, int n, int n1);
int stepvars(char *s);
int catarrays(char *out[], int max, char *in1[], int num1, char *in2[], int num2);
int augmentarray(int curnum, char *out[], int max1, char *in1[], int num1);
int combarrays(char ***out, char *in1[], int num1, char *in2[], int num2);
int isinstring(char *a, char *b);
int samestr(char *s, char *t);
int findfunc1(char *readfile, char *s, char *t, int max, int clean);
int findfuncsimp(char *readfile, char *s, char *t, int max);
int gettvs(char *fname, char *tempv[], int max);
int getdepsfromfunc(char *fn, char *deps[], int maxdeps, char *rems[], int numrems);
int getdepson(char *pname, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps);
char **getdfilevars(char *fname, int *num);
int isgooddatfile(char *fname);
int augmentdatfile(char *datfile, char *newdatfile, char *setfile, char *varfile, char *paramfile, void pfunc(double*, double *), void tvfunc(double *, double *, double *, double *));
int getalldepson(char *pname, char *paramarray[], int numparams, char *depsfile, char *depsarray[], int max);

int transfsubf(char *fn, char *vname, char *outstr, int max);
int simplinefile(char *infname, char *outfname);
int simpblockfile(char *infname, char *outfname, char end[]);
int isinlist(int i, int lst[], int tot);
int sublist(int slst[], int totslst, int lst[], int tot);
int parse(char *exprs, char *vars[], int totv, int indcs[], int max);
int getrecvars(char *str, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps);
int getrecvars1(char *str, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps, char *paramarray[], int numparams);
int getalldeps(char *pname, char *paramarray[], int numparams, char *depsfile, char *depsarray[], int max);
int checkprocsfile(char *infname, char end[]);
int checkfuncsfile(char *infname, char end[]);
int isinarray(char *v[], int numv, char *s);
int cisinarray(char *v[], int numv, char *s);
int isvarstart(char *s, int j);
int isinvar(char *s, int j);
int isvar(char *s, char *rem[], int numrems);
char *getvar(char *fname, char *type, char *var);
char **getvs(char *s, int *num, char *rem[], int numrems);
int getvars(char *s, char *v[], int max, char *rem[], int numrems);
int getnumvars(char *s, char *rem[], int numrems);
int getallvars(char *s, char *v[], int max);
int getallrecvars(char *DEfile, char *algfile, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps);
int ordertree(char *vars[], char *exprs[], int tot, int ord[], int max);
int getnumallvars(char *s);
char *getvalfromfile(char *fname, char *vname);
char *getcvalfromfile(char *fname, char *vname);
char *getvalfromfileq(char *fname, char *vname);
char *getcvalfromfileq(char *fname, char *vname);
char *getnthwd(char *s, int n);
int getnthword(char *s, char *v, int len, int n);
int getnthtoken(char *s, char *v, int len, int n);
char *getnthtkn(char *s, int n);
int getnthblock(char *s, char *v, int len, int n);
char *getnthblk(char *s, int n);
int getoption(char *fname, char *optname, char v[], int max);
int getoptq(char *fname, char *optname, char v[], int max);
int getfileblk(char *fname, char v[], int max);
char *getmoddef(char *fname, char *modname, char *obj);
char *getmoddef1(char *fname, char *modname, char *obj);
int changemoddef(char *fname, char *modname, char *obj, char *newval);
int isnumber(char *s);
int ispureint(char *s);
int getnumtokens(char *s);
int getfield(char *descfile, char varname[], char *field, char vardesc[], int lim);
int getfieldsimp(char *descfile, char varname[], char vardesc[], int lim);
int chemgets(char *s, char *v[], int max);
int chemgets1(char *s, char *v1[], char *v2[], int max);
int chemgets2(char *s, char *v[], int max, char sep);
int chemgets3(char *s, char *v[], int max, char *seps);
int getnumblocks(char *s, char *seps);
int setpvalsfromfile(char *readfile, char *paramarray[], double *pvals, int numparams);
char *getname(char s[], char t[]);
int getstartind(char s[], char t[]);
int getrest(char s[], char t[], char v[], int vsize);
int getrestfromfile(char *fname, char t[], char v[], int vsize);
char **getvarlistfromfile(char *fname, char *varname, int *num, char *rem[], int numrems);
char **getvardepsfromfile(char *fname, char *varname, int *num, char *rem[], int numrems);
char *getrst(char s[], char t[]);
char *getrstfromfile(char *fname, char t[]);
int getline(FILE *fp, char s[], int lim);
int cpfile(char *outfile, char *infile);
int catfile(char *outfile, char *infile);
int getline0(FILE *fp, char s[], int lim);
int getlinefromblock(int *pos, char *block, char s[], int lim);
int getcleanblock(FILE *fp, char s[], char end[], int lim);
int getblock(FILE *fp, char s[], char end[], int lim);
int getblockmin(FILE *fp, char s[], char end[], char errorvar[], int lim);
int cleanblock(char *s, char *t, int len);
int ins(int k, char t[], char s[], int lim);
int inspow(int k, char t[], char t1[], char s[], int lim);
void reverse(char s[]);
void itoa(char s[], int n);
int getstring(char *s, char *t, int len);
int isgood(char *s, int j);
int isgood1(char *s, int j);
void trans(char t[], char s[], char *varname, char *vararray[], int numvars);
int addifpos(char *tempoutst, int j, int maxlen, char *tempvar);
char **transarray(char *s[], int max, char addon[]);
char *transalt1(char s[], char *varname, char *vararray[], int numvars);
int addvar1(int k, char *s, char *t[], int tmax);
double getparamvalue(char *readfile, char *vname);
int getparamvaluestr(char *readfile, char *vname, char *vval, int lim);
void transalt2(char t[], int maxlen, char s[], char *readfile);
int getstrg(char *s, char *t, int len);
int delvar(char *array[], int lim, char *var);
int getnoncomlines(char *datfile);
int genarraysfromdat(char *datfile, int maxl, char *array1[], char *array2[], int lim);
int genarraysdat(char *datfile, char ***array1, char ***array2);
int genbracarraysfromdat(char *datfile, char ***array1, char ***array2);
char *genfloatmdat(char *datfile, double ***darray, int *s1, int *s2);
double *genfloatvec(char *line, int *num);
char *transfunc1(char s[], char ins_string[], char *vararray[], int numvars);
char *transfunc1a(char s[], char ins_string1[], char ins_string2[], char *vararray[], int numvars);
char *transfunc2(char s[], char *array1[], char *array2[], int numvars);
char *transfunc2it(char s[], char *array1[], char *array2[], int numvars);
int free_datarray(char *array[], int lim);
int checkbracs(char *s);
int resmple(char *fname1, char *fname2, int **v1, double **v2);
double getdiff1(char *fname1, char *fname2, char *vname, int *v1, double *v2, int num1, double **diffs);
int getalldiffs(char *fname1, char *fname2, char *fout);
double timeaverage(char *datfile, double time1, double time2, char *varname);
double distaverage(char *datfile, double time1, double time2, char *varname1, double off1, double scale1, char *varname2, double off2, double scale2, int type, int mom);
char **genarraydat(char *datfile, int *num);
char **genlinearraydat(char *datfile, int *num);
int freearraydat(char *array[], int lim);
char *getproc(char *fname, char *procname);
int getdesc(char *descfile, char varname[], char vardesc[], int lim);
int insintofile(char *fname, int linenum, char *str);
int addnewobj(char *fname, char *type, char *str);
int addreactofile(char *fname);
int addalgtofile(char *fname);
int addDEtofile(char *fname);
int addDEtermtofile(char *fname);
int getobj(FILE *fd, int *parsenum, char *type, char str[], int maxlen);
char *getnamedobj(char *fname, char *name, char *type);
char *gnameobj(char *fname, char *name);
char *transtemp(char *s, char *vars[], int tot);
int scanmodfile(char *fname, char *remsfile, char *procdir, char *keyvars[], int maxkey, char *nonkeyvars[], int maxnonkey, int *nkey, int *nnonkey);
int quickparse(char *fname, char *remsfile);
int quickscan(char *fname, char *remsfile, char outstring[], int lim);
int quickscandir(char *dname, char *descdname, char *remsfile, char outstr[], int lim);
int getdc(char *dname, char *filter, char *out[], int max);
int changeparname(char *modname, char *parname[], char *newparname[], int numpars, char *remsfile);
int checkparname(char *modname, char *parname[], int numpars, int *comcount);
int pchange(char *pfile, char *pname, char *valold, char *valnew);
int undoparamsession(char *pchangefile);
double alldist(char *datfile, char *datfile1, double time1, double time2);
double gettval(char *s);
int midline (char *line1, char *line2, char *line3, char lineout[], int max, int type);
char *getltxname(char *descfile, char varname[]);
char *cleansq(char *t);
char *getltxname1(char *descfile, char *descfile1, char varname[]);
char *getltx(char **namearray, char **ltarray, int numlt, char *varname);
void latexview(struct react myreact);
void latexview1(struct react myreact, char *ltxfile, int withhtml);
void quickview(struct react myreact);
char *ltx1(char *t);
char *bracket(char t[]);
char *ltx(char t[]);
char *transltx(char *varname);
char *strcut(char *t, int n);
struct react reactcopy(struct react react1);
void free_reac(struct react react1);
void free_reacarray(struct react reactarray[], int numreacs);
int readreac(struct react *tempreac, char *block);
int readreacfile(struct react reacarray[], int max, char *filename);
struct DEterm DEtermcopy(struct DEterm DEterm1);
void free_DEterm(struct DEterm DEterm1);
void free_DEtermarray(struct DEterm DEtermarray[], int numDEterms);
int readDEterm(struct DEterm *tempDEterm, char *block);
int readDEtermfile(struct DEterm DEtermarray[], int max, char *filename);
struct algrel algrelcopy(struct algrel algrel1);
void free_algrel(struct algrel algrel1);
void free_algarray(struct algrel algarray[], int numalgs);
int readalgrel(struct algrel *tempalg, char *block, char *vararray[], int numvars, char *allparamarray[], int numallparams, char *rems[], int numrems);
int readalgfile(struct algrel algarray[], int max, char *filename, char *varfile, char *allparamfile, char *remsfile);
struct DE DEcopy(struct DE DE1);
void free_DE(struct DE ODE);
void free_DEarray(struct DE DEarray[], int numDEs);
int readDE(struct DE *tempDE, char *block, char *vararray[], int numvars, char *allparamarray[], int numallparams, char *rems[], int numrems);
int readDEfile(struct DE DEarray[], int max, char *filename, char *varfile, char *allparamfile, char *remsfile);
void printchems(int numreacs, struct react allreacs[]);
int getdatfileindex(char fname[], char varname[]);
int getdatafilevals(char fname[], int varindex, double varval, double max_error, int varindex1[], int numvars, double varvals[]);
int getdatfiledata(char fname[], char varname[], double varval, char *outvarname[], int numoutvars, double outvarvals[], double max_error);
int compdatfiles(char fname[], char fname1[]);
int orderlines(char *pname, char *type, char *fname, char ***lines, char othertype[], int lim);





