/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */


#include "paramcontrol.h"
#include "varcontrol.h"

int wrongdir(char *wdir); // a function used in several routines. 
class smoothToggle;


//
//
// plotToggle
//
//

class plotToggle: public BasicComponent{

 protected:

  messagelabel **label; // a pointer to the label to be set
  Widget *inf;
  Widget *outf;
  Widget *mname;
  char infstate[100];
  char autoinfstate[100];
  char outfstate[100];
  smoothToggle **stog;


 public:

  plotToggle(Widget parent, Widget *modelname, Widget *infile, Widget *outfile, char *name, char *display, messagelabel **label1, smoothToggle **stog1);
  
  static void plottogCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  void activate(Widget widget, XtPointer client_data, XtPointer call_data);
  void help();
};

//
//
// plotControl
//
//

class plotControl: public BasicComponent{

 protected:

  messagelabel **label; // a pointer to the label to be set
  Widget *text; //pointer to the text field holding the variable to be plotted
  Widget *datfile; // pointer to widget holding datafile name

 public:

  Widget *toplevel;
  Widget *text_comout;
  Widget *pfile;

  plotControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *plotfile, Widget *text_pfile, char *name, char *display, messagelabel **label1, Widget *text1);

  static void plotCallback(Widget, XtPointer client_data, XtPointer);

  void plot();

  static void helpCallback(Widget widget, XtPointer client_data, XtPointer client_data);

  void help();

};


//
//
// smoothToggle
//
//

class smoothToggle: public BasicComponent{

 protected:

  messagelabel **label; // a pointer to the label to be set

 public:

  // constructor

  smoothToggle(Widget parent, char *name, char *display, messagelabel **label1);

  void smooth();

  static void smoothCallback(Widget, XtPointer client_data, XtPointer);

  void help();
  static void helpCallback(Widget, XtPointer client_data, XtPointer);


};


//
//
// contplotToggle
//
//

class contplotToggle: public BasicComponent{

 protected:

  messagelabel **label; // a pointer to the label to be set

 public:

  // constructor

  contplotToggle(Widget parent, char *name, char *display, messagelabel **label1);

  void contplot();
  void help();

  static void contplotCallback(Widget, XtPointer client_data, XtPointer);

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

};


//
//
// makeControl
//
//

class makeControl: public BasicComponent{

  // returns 0 if the current working directory doesn't match global variable workdir

  plotToggle *tog;

 protected:

  messagelabel **label; // a pointer to the label to be set

 public:

  Widget *toplevel;
  void make();

  makeControl(Widget parent, Widget *toplevel1, char *name, char display[], messagelabel **label1, plotToggle *tog1);

  static void makeCallback(Widget, XtPointer client_data, XtPointer);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

};


//
//
// runControl
//
//

class runControl: public BasicComponent{
  
 public:

  messagelabel **label; // a pointer to the label to be set

  void run();

  static void runCallback(Widget, XtPointer client_data, XtPointer);

  runControl(Widget parent, char *name, char display[], messagelabel **label1);

};


//
//
// makerunControl
//
//

class makerunControl: public BasicComponent{

  plotToggle **tog;
  smoothToggle *stog;
  contplotToggle **ctog;
  Boolean stopped;
  Widget dialog;
  XtAppContext *app1;
  int answer;
  int fuse;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;
  Widget *outputfile;
  char outputfstr[100];
  Widget *modtype;
  Widget *pfile;
  char systemcom[200];
  char ifile[100];
  char aifile[100];
  char ofile[100];
  char aofile[100];
  Widget *toplevel;
  Widget *text_comout;
  Widget *text;

  void makerun();

  static void makerunCallback(Widget, XtPointer client_data, XtPointer call_data);
  void TimeoutCursors(Boolean on, Boolean interruptable);
  void stop(Widget dialog, XtPointer client_data, XtPointer call_data);
  Boolean CheckForInterrupt();
  static void stopCallback(Widget w1, XtPointer client_data, XtPointer call_data);

  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);

  void changedlg(Widget widget, XtPointer client_data, XtPointer call_data);
  static void changedlg_callback(Widget widget, XtPointer client_data, XtPointer call_data);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);



  //constructor

  makerunControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *modeltype, Widget *infile, Widget *outfile, Widget *paramfile, char *name, char display[], messagelabel **label1, plotToggle **tog1, smoothToggle *tog2, contplotToggle **tog3, Widget *text1, XtAppContext *app1, int *firstuse);

};

//
//
// editfileControl
//
//

class editfileControl: public BasicComponent{

  plotToggle **tog;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;

  void editfile();

  static void editfileCallback(Widget, XtPointer client_data, XtPointer);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  editfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, plotToggle **tog1);

};

//
//
// cpinputfileControl
//
//

class cpinputfileControl: public BasicComponent{


 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *text_filename;
  char *inputfstr;
  char *inputfilereal;

  void cpinputfile();

  static void cpinputfileCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);
  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);

  void help();

  static void helpCallback(Widget widget, XtPointer client_data, XtPointer call_data);

  //constructor

  cpinputfileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1);

};


//
//
// editfilesimpControl
//
//

class editfilesimpControl: public BasicComponent{

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;

  void editfilesimp();

  static void editfilesimpCallback(Widget, XtPointer client_data, XtPointer);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  editfilesimpControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1);

};



//
//
// viewpfileControl
//
//

class viewpfileControl: public BasicComponent{

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;

  void viewpfile();

  static void viewpfileCallback(Widget, XtPointer client_data, XtPointer);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  viewpfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1);

};



//
//
// setpfileControl
//
//

class setpfileControl: public BasicComponent{

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;

  void setpfile();

  static void setpfileCallback(Widget, XtPointer client_data, XtPointer);

  void help();

  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  setpfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1);

};



//
//
// searchvarControl
//
//

class searchvarControl: public BasicComponent{

  messagelabel **label; // a pointer to the label to be set
  Widget *input;
  char systemcom[100];
  Boolean stopped;
  Widget *toplevel;

public:

  void searchvar();
  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);
  void TimeoutCursors(Boolean on);
  void help();

  static void searchvarCallback(Widget, XtPointer client_data, XtPointer);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void helpCallback(Widget widget, XtPointer client_data, XtPointer call_data);

  //constructor

  searchvarControl(Widget parent, Widget *toplevel1, char *name, char display[], messagelabel **label1);

};



//
//
// selectfileControl
//
//

class selectfileControl: public BasicComponent{

  plotToggle **tog;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;
  Widget *fsdialog;
  char *inputfstr;
  char *inputfilereal;
  char systemcom[100];

  void selectfile();

  static void selectfileCallback(Widget, XtPointer client_data, XtPointer);

  void help();
  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  selectfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1, plotToggle **tog1);

};



//
//
// selectpfileControl
//
//

class selectpfileControl: public BasicComponent{

  plotToggle **tog;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *inputfile;
  Widget *fsdialog;
  char *inputfstr;
  char *inputfilereal;
  char systemcom[100];

  void selectpfile();

  static void selectpfileCallback(Widget, XtPointer client_data, XtPointer);

  void help();
  static void helpCallback(Widget, XtPointer client_data, XtPointer);

  //constructor

  selectpfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1, plotToggle **tog1);

};


//
//
// selectoutvarControl
//
//

class selectoutvarControl: public BasicComponent{

  plotToggle **tog;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *text_outvar;
  char *inputfstr;
  char *inputfilereal;
  char *filename;
  Widget *toplevel;
  Widget *text_comout;

  void selectoutvar();

  static void selectoutvarCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);
  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);

  void help();

  static void helpCallback(Widget widget, XtPointer client_data, XtPointer call_data);

  //constructor

  selectoutvarControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *text_outvar1, char *name, char display[], char *filename1, messagelabel **label1, plotToggle **tog1);

};



//
//
// selectmodelControl
//
//

class selectmodelControl: public BasicComponent{

  plotToggle **tog;

 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *text_outvar;
  Widget *plot_text;
  Widget *text_pfile;
  Widget *text_mfile;
  Widget *text_datfile;
  char *inputfstr;
  char *inputfilereal;
  char *filename;
  Widget *toplevel;
  Widget *text_comout;
  Widget *text_infile;
  paramControl **pc;
  varControl **vc;

  void selectmodel();

  static void selectmodelCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void helpCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);
  void help();

  //constructor

  selectmodelControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *text_outvar1, Widget *text_infile, Widget *plot_text1, Widget *text_paramfile, Widget *text_modulefile, Widget *text_datafile, char *name, char display[], char *filename1, messagelabel **label1, plotToggle **tog1, paramControl **pc1, varControl **vc1);

};





//
//
// reducefileControl
//
//

class reducefileControl: public BasicComponent{


 public:

  messagelabel **label; // a pointer to the label to be set
  Widget *text_filename;
  char *inputfstr;
  char *inputfilereal;

  void reducefile();

  static void reducefileCallback(Widget widget, XtPointer client_data, XtPointer call_data);
  static void dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data);
  void dlg(Widget widget, XtPointer client_data, XtPointer call_data);

  void help();

  static void helpCallback(Widget widget, XtPointer client_data, XtPointer call_data);

  //constructor

  reducefileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1);

};

