/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  int l;
  char dirname[60] = "models/modeldescs/";
  char mkdircall[70] = "mkdir ";
  char tempname[70];
  char infofilename[100];
  char mkmoduledircall[100] = "mkdir models/modules/";
  char mkpvaldircall[100] = "mkdir models/pvals/";
  char systemcom[200];
  char *oneword;
  char *twoword;
  char *definput;
  char *defautoinput;
  char *defoutvar;
  char *defpfile;
  char *defmodfile;
  char *defdescfile;
  char *defauto;
  char modfile[100];
  int flag;
  char inputfile[100];
  char autoinputfile[100];
  char *defpname;
  char *tmp;
  
  autoinputfile[0]=0;

  if(argc < 3){
    fprintf(stderr, "ERROR in cpsys.c: you must enter the names of the old system and the new system you wish to create (new system first, and then existing system).\n");
    return -1;
  }
  if(strlen(argv[1]) > 30){
    fprintf(stderr, "ERROR in cpsys.c: new system name %s too long.\n", argv[1]);
    return -1;
  }
  if(strlen(argv[2]) > 30){
    fprintf(stderr, "ERROR in cpsys.c: old system name %s too long.\n", argv[2]);
    return -1;
  }
  
  if(strcmp(argv[1], argv[2]) == 0){
    fprintf(stderr, "ERROR in cpsys.c: the new system must have a different name from the old system.\n");
    return -1;
  }

  if(modelexists(argv[1])){
    fprintf(stderr, "ERROR in cpsys1: a system by the name %s already exists. If you want to remove it, use the remove model program.\n", argv[1]);
    return -2;
  }

  if(!modelexists(argv[2])){
    fprintf(stderr, "ERROR in cpsys: There does not appear to be a model called %s to copy. Exiting.\n", argv[2]);
    return -2;
  }

  // get existing defaults for the current model

  definput = getmoddef("userfiles/model_defaults", argv[2], "input");
  if(!isonlyspace(definput)){flag=1;}
  defautoinput = getmoddef("userfiles/model_defaults", argv[2], "autoinput");
  defoutvar = getmoddef("userfiles/model_defaults", argv[2], "var");
  defpfile = getmoddef("userfiles/model_defaults", argv[2], "pvals");
  defauto = getmoddef("userfiles/model_defaults", argv[2], "auto");
  defmodfile = getmoddef("userfiles/model_defaults", argv[2], "module");
  tmp = getmoddef("userfiles/model_defaults", argv[2], "modeldesc");
  defdescfile=getfname(tmp);
  free(tmp);

  // the modeldescs directory

  strcat(dirname, argv[1]); // safe. set the modeldescs directory name
  strcat(mkdircall, dirname); // safe. make the directory
  fprintf(stderr, "%s\n", mkdircall);
  system(mkdircall);

  // copying all files in modeldesc directory

  snprintf(systemcom, 200, "cp -fr models/modeldescs/%s/* models/modeldescs/%s", argv[2], argv[1]);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR while copying model descriptor files. Now exiting unsuccessfully.\n");
    return -1;
  }

  // the module directory

  strcat(mkmoduledircall, argv[1]); // safe
  fprintf(stderr, "%s\n", mkmoduledircall);
  system(mkmoduledircall);

  // copying all files in modules directory

  fprintf(stderr, "Copying module files.\n");
  snprintf(systemcom, 200, "cp -fr models/modules/%s/* models/modules/%s", argv[2], argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR while copying module files. Now exiting unsuccessfully.\n");
    return -1;
  }

  // writing the info file

  strcpy(infofilename, dirname); // safe
  strcat(infofilename, "/"); // safe
  strcat(infofilename, argv[1]); // safe
  strcat(infofilename, "info"); // safe
  fprintf(stderr, "Creating the file %s\n", infofilename);
  fd1 = fopen(infofilename, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in cpsys.c: %s could not be opened...\n", infofilename);
    return -1;
  }

  // writing the info file

  fprintf(fd1, "PARAMDATAFILE solvedata.dat\n");

  fclose(fd1);


  // the parameter directory

  strcat(mkpvaldircall, argv[1]); // safe
  fprintf(stderr, "%s\n", mkpvaldircall);
  system(mkpvaldircall);

  defpname=getfname(defpfile);

  // copying all files in parameter directory

  fprintf(stderr, "Copying parameter files.\n");
  snprintf(systemcom, 200, "cp -fr models/pvals/%s/* models/pvals/%s", argv[2], argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR while copying parameter files. Perhaps there are no files to be copied.\n");
  }


  // copying all input files or the default input file

  strcpy(mkmoduledircall, "mkdir models/inputs/"); // safe
  strcat(mkmoduledircall, argv[1]); // safe
  fprintf(stderr, "%s\n", mkmoduledircall);
  system(mkmoduledircall);

  fprintf(stderr, "Copying input files.\n"); 

  snprintf(tempname, 70, "models/inputs/%s", argv[2]); // is there a models/inputs directory for this model?
  fprintf(stderr, "definput = %s\n", definput);
  if (!fileexists(tempname)){ // no input directory
    // find the default input file
    if(flag == 0){ // no default input file found
      fprintf(stderr, "WARNING: default input file for model %s not found. An empty input file is being created.\n", argv[2]);
      snprintf(tempname, 70, "models/inputs/%s/%s_default.dat", argv[1], argv[1]);
      fd = fopen(tempname, "w");
      if(fd == NULL){
        fprintf(stderr, "ERROR in cpsys.c: %s could not be opened. Exiting unsuccessfully\n", tempname);
        return -1;
      }
      fprintf(fd, "//default model behaviour\nsteady: none\nchosen_param:\n******\n10\n");
      fclose(fd);
      snprintf(inputfile, 100, "%s/%s_default.dat", argv[1], argv[1]);
    }
    else{ // copying and renaming default input file
      snprintf(systemcom, 200, "cp %s models/inputs/%s/%s_default.dat", definput, argv[1], argv[1]);
      if((l= system(systemcom)) != 0){
        fprintf(stderr, "ERROR while copying input files. Now exiting unsuccessfully.\n");
	return -1;
      }
      snprintf(inputfile, 100, "%s/%s_default.dat", argv[1], argv[1]);
    }

  }
  else{ // input directory found: copy all files in this directory
    snprintf(systemcom, 200, "cp -fr models/inputs/%s/* models/inputs/%s", argv[2], argv[1]);
    if((l= system(systemcom)) != 0){
      fprintf(stderr, "ERROR while copying input files. Now exiting unsuccessfully.\n");
      return -1;
    }
    if(flag == 0){ // no default input file found
      snprintf(inputfile, 100, "%s/%s_default.dat", argv[1], argv[2]);
    }
    else{
      snprintf(systemcom, 200, "/%s/", argv[2]);
      if(!strstr(definput, systemcom)){
       	tmp=getfname(definput);
        snprintf(inputfile, 100, "%s/%s", argv[1], tmp);
	snprintf(systemcom, 200, "cp -fr %s models/inputs/%s/_%s", definput, argv[1], tmp);        
	if((l= system(systemcom)) != 0){
          fprintf(stderr, "ERROR while copying an input file. Carrying on.\n");
        }
	snprintf(inputfile, 100, "%s/_%s", argv[1], tmp);
	free(tmp);
      }
      else{
	tmp=getfname(definput);
        snprintf(inputfile, 100, "%s/%s", argv[1], tmp);
	free(tmp);
      }
      //      fprintf(stderr, "inputfile = %s\n", inputfile);
    }
  }

  // copying any autoreginput files (if there are any)


  snprintf(tempname, 70, "models/autoreginputs/%s", argv[2]); // is there a models/inputs directory for this model?
  if (fileexists(tempname)){
    fprintf(stderr, "Copying autoreginput files.\n"); 
    strcpy(mkmoduledircall, "mkdir models/autoreginputs/"); // safe
    strcat(mkmoduledircall, argv[1]); // safe

    fprintf(stderr, "%s\n", mkmoduledircall);
    system(mkmoduledircall);

    snprintf(systemcom, 200, "cp -fr models/autoreginputs/%s/* models/autoreginputs/%s", argv[2], argv[1]);
    if((l= system(systemcom)) != 0){
      fprintf(stderr, "ERROR while copying autoreginput files. Now exiting unsuccessfully.\n");
      return -1;
    }
    tmp=getfname(defautoinput);
    snprintf(autoinputfile, 100, "%s/%s", argv[1], tmp);
    free(tmp);
  }

  // add the relevant bit to the Makefile

  fd = fopen("userfiles/model_types_make", "a");
  if(fd == NULL){
    fprintf(stderr, "ERROR in cpsys.c: \"userfiles/model_types_make\" could not be opened...\n");
    return -1;
  }
  fprintf(stderr, "Adding data to the file \"userfiles/model_types_make\"\n");
  fprintf(fd, "%s: src/lib/P_makemodel userfiles/model_defaults\nifneq ($(MODEL), %s)\n\techo %s > sysfiles/sysstate/which_model_file\n\t./src/lib/P_makemodel\nendif\n", argv[1], argv[1], argv[1]);
  fclose(fd);

  // add relevant bit to the model_defaults file

  oneword = enclose(argv[2], "/", "/");
  twoword = enclose(argv[2], "", "/");
  if(strstr(defmodfile, oneword)){
    tmp=getfname(defmodfile);
    snprintf(modfile, 100, "%s/%s", argv[1], tmp);
    free(tmp);
  }
  else{
    if(strlen(defmodfile) < 99)
      strcpy(modfile, defmodfile); // safe
    else{
      fprintf(stderr, "ERROR in cpsys.c: default module file name %s is too long.\n", defmodfile);
      return -1;
    }
  }

  fd = fopen("userfiles/model_defaults", "a");
  if(fd == NULL){
    fprintf(stderr, "ERROR in cpsys.c: \"userfiles/model_defaults\" could not be opened...\n");
    return -1;
  }

  fprintf(stderr, "Adding data to the file \"userfiles/model_defaults\"\n");

  if(!startswith(defpfile, twoword) && !strstr(defpfile, oneword)){ // default parameter file in base directory
    fprintf(fd, "%s\t%s\t%s\t%s\t%s\t%s\t%s/%s\t%s\n\n", argv[1], inputfile, defoutvar, defpfile, defauto, modfile, argv[1], defdescfile, autoinputfile);
  }
  else{
    fprintf(fd, "%s\t%s\t%s\t%s/%s\t%s\t%s\t%s/%s\t%s\n\n", argv[1], inputfile, defoutvar, argv[1], defpname, defauto, modfile, argv[1], defdescfile, autoinputfile);
  }

  fclose(fd);

  fprintf(stderr, "System %s successfully copied to %s.\n", argv[2], argv[1]);
  free(oneword);
  free(twoword);
  free(definput);
  free(defoutvar);
  free(defpfile);
  free(defauto);
  free(defmodfile);
  free(defpname);
  free(defdescfile);

  return 0;
}
