/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include "common/nrsimp.h"

const double Pi = 3.14159265;
void pfunc(double *pvals, double *outvals);
void vfunc(double *rpar, double *y);

int main()
{
  int i, suc=0;
  FILE *fd;
  char *outfile = "sysfiles/allcurrent_pvals.dat";
  char *outfile2 = "sysfiles/allcurrentinits.dat";
  char *paramvalfile;
  char line[2000];
  double *rpar;
  char **settable;
  int numsettable = 0;
  double *setpvals;
  char **paramarray;
  int numparams = 0;
  char **vararray;
  int numvars = 0;
  double *y;

  fd = fopen("sysfiles/sysstate/which_param_file", "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in genallcurrentvals.cc: file \"sysfiles/sysstate/which_param_file\" could not be opened.\n");
    return -1;
  }
  getline(fd, line, 500);
  paramvalfile = getnthblk(line, 1);
  fclose(fd);

  settable = genarraydat("sysfiles/settable.dat", &numsettable);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  setpvals = dvector(0, numsettable - 1);
  rpar = dvector(0, numparams-1);
  y = dvector(0, numvars - 1);



  suc = setparamsfromfile(paramvalfile, settable, numsettable, setpvals, rpar, pfunc);
  free(paramvalfile);
  if(suc!=0){
    fprintf(stderr, "\tIt is possible that this error was because the model does not match the parameter value file.\n");
    return -2;
  }
  vfunc(rpar, y);

  fd=fopen(outfile, "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in genallcurrentvals.cc: file %s could not be opened.\n", outfile);
    return -1;
  }

  fprintf(fd, "/* Generated by the file \"genallcurrentvals.cc\" */\n");

  for(i=0;i<numparams;i++){
    fprintf(fd, "%s\t%.4e\n", paramarray[i], rpar[i]); 
  }
  fclose(fd);

  fd = fopen(outfile2, "w");

  if(fd == NULL){
    fprintf(stderr, "ERROR in genallcurrentvals.cc: file %s could not be opened.\n", outfile2);
    return -1;
  }

  fprintf(fd, "/* Generated by the file \"genallcurrentvals.cc\" */\n");

  for(i=0;i<numvars;i++){
    fprintf(fd, "%s\t%.4e\n", vararray[i], y[i]);
  }
  fclose(fd);

  freearraydat(paramarray, numparams);
  freearraydat(vararray, numvars);
  freearraydat(settable, numsettable);
  free_dvector(setpvals, 0, numsettable-1);
  free_dvector(rpar, 0, numparams-1);
  free_dvector(y, 0, numvars-1);

  return 0;
}

