/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

*/


#include "common/stringmanip.h"
#include "common/nrsimp.h"


int main()
{
  FILE *fd, *fd1;
  int i, j, numpars, len;
  char line[5000];
  char *tempexpr;
  char *varname;
  char **outvars;
  int numall;
  char **allarray;
  int getall=0;
  char *infile;
  int numchangeparams;
  char *changeparams[40];
  char *pdeplist[1000];
  char *allpdeps[1000];
  int numdeps=0;
  int totdeps=0;
  int ind;
  int flag;

  int unf=0;
  char *unfound[10];

  char **vararray;
  int numvars=0;
  char **paramarray;
  int numparams=0;
  int numout;
  int nv;

  // get the name of the input file

  fd = fopen("sysfiles/sysstate/which_autoreginput_file", "r");
  if (fd==NULL){
    fprintf(stderr, "Couldn't open the file: \"sysfiles/sysstate/which_autoreginput_file\". Exiting...\n");
    return -1;
  }
  getline(fd, line, 5000);
  infile = getnthblk(line, 1);
  fclose(fd);

  fd = fopen(infile, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in genautoregout.c: %s could not be opened...\n", infile);
    free(infile);
    return -1;
  }
  free(infile);

  while((len=getline(fd, line, 5000)) > 0){
    j=0;
    if(!iscomsep(line)){
      if(strstr(line, "chosen_param:") != NULL){
	tempexpr = getrst("chosen_param:", line);
	numchangeparams = chemgets(tempexpr, changeparams, 40);
	free(tempexpr);
	fprintf(stderr, "numchangeparams = %d\n", numchangeparams);
	break;
      }
    }
  }
  fclose(fd);
  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  numall = combarrays(&allarray, vararray, numvars, paramarray, numparams);


  // read in the paramlist and get all the parameters which 
  // depend on the parameters being changed

  for(i=0;i<numchangeparams;i++){

    if((ind = isinarray(paramarray, numparams, changeparams[i])) >= 0){
      numdeps = getalldepson(changeparams[i], paramarray, numparams, "sysfiles/alldeps.dat", pdeplist, 1000);
      totdeps = augmentarray(totdeps, allpdeps, 1000, pdeplist, numdeps);
      free_datarray(pdeplist, numdeps);
    }
    else{
      unf = addvar1(unf, changeparams[i], unfound, 10);
      fprintf(stderr, "\nWARNING in genautoregout.c: parameter %s could not be found.\n\n", changeparams[i]);
    }

  }


  // check if we're getting all parameters...

  fd = fopen("sysfiles/autoregoutvars.dat", "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in genautoregout.c: sysfiles/autoregoutvars.dat could not be opened...\n");
    free_datarray(allpdeps, totdeps);
    return -1;
  }

  flag=0;
  while(flag==0 && (len=getline(fd, line, 5000)) > 0){
    if(!iscomsep(line)){
      varname = getnthblk(line, 1);
      if(strcmp(varname, "ALL")==0){
	getall = 1;
	flag=1;
      }
      free(varname);
    }
  }
  fclose(fd);

  //
  // if not too many variables and parameters, then output them all
  //


  fd1 = fopen("sysfiles/autoregout.dat", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in genautoregout.c: sysfiles/autoregout.dat could not be opened for writing...\n");
    freearraydat(allarray, numall);
    freearraydat(vararray, numvars);
    freearraydat(paramarray, numparams);
    free_datarray(changeparams, numchangeparams);
    free_datarray(allpdeps, totdeps);
    free_datarray(unfound, unf);
    return -1;
  }

  fprintf(fd1, "// generated by genautoregout.c\n\n");

  if((numvars + totdeps + unf < 50) || (getall == 1))
    numpars = numvars + totdeps + unf;
  else{
    outvars = genarraydat("sysfiles/autoregoutvars.dat", &numout);
    nv=0;
    for(i=0;i<numout;i++){
      if(isinarray(allarray, numall, outvars[i]) >= 0)
	nv++;
      else
	fprintf(stderr, "WARNING: Output variable %s not found.\n", outvars[i]);
    }
    numpars = nv + totdeps + unf;
  }


  if(numpars != 0){
    // the changing parameters
    for(i=0;i<totdeps;i++)
      fprintf(fd1, "%s\n", allpdeps[i]);

    // the unfound parameters
    for(i=0;i<unf;i++)
      fprintf(fd1, "%s\n", unfound[i]);

    if((numvars + totdeps + unf < 50) || (getall == 1)){
      for(i=0;i<numvars;i++)
	fprintf(fd1, "%s\n", vararray[i]);
    }
    else{
      for(i=0;i<numout;i++){
	if(isinarray(allarray, numall, outvars[i]) >= 0)
	  fprintf(fd1, "%s\n", outvars[i]);
      }
      freearraydat(outvars, numout);
    }
  }

  fclose(fd1);
  free_datarray(changeparams, numchangeparams);
  free_datarray(unfound, unf);
  free_datarray(allpdeps, totdeps);

  freearraydat(allarray, numall);
  freearraydat(vararray, numvars);
  freearraydat(paramarray, numparams);


  return 0;
}
