/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// generates DEterms from the reactions
// This program needs to be radically simplified. All it needs to do 
// for the general reaction types is compute the general rate term and then 
// for each chemical multiply by the corresponding stoichiometry

/* This file generates, amongst other things, the file sysfiles/chemterms.c. It appears to generate the terms correctly, but has not been thoroughly checked*/

#include "syssize.h" 
#include "common/stringmanip.h"
#include "common/nrsimp.h"

void shaveplus(char *s);

void printeqs(FILE *fd, FILE *fd2, int numreacs, int totchems, struct react allreacs[], char *chemarray[]);

int MAterm(int z1, struct react allreacs[], int j, char lr[], int k, int l, char *leftterms[], char *rterm, int rtermlength, char *sign);
int MA2term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength);
int prodterm(int z1, struct react allreacs[], int j, int k, int l, char *rterm, int rtermlength);
int useterm(int z1, struct react allreacs[], int j, int k, int l, char *rterm, int rtermlength);
int MM1term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength);
int electerm(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength);
int MM1sub2term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength);
int MM1sub3term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength);
int MM1sub4term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength);
int MM1sub5term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength);
int FDterm(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength);
int ratetermsimp(int z1, struct react allreacs[], int j, char lr[], int k, char *rterm, int rtermlength);
int rateterm(int z1, struct react allreacs[], int j, char lr[], int k, char *rterm, int rtermlength);

int main()
{

  int i, j, k, totchems;
  char *chemarray[MAXVARS];


  FILE *fd, *fd2, *fd3; /* the output files */

  struct react allreacs[MAXREAC];
  int numreacs;


  fd = fopen("sysfiles/chemterms.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in genchemDEterms.c: file: \"sysfiles/chemterms.dat\" could not be opened...\n");
    return -1;
  }
  fd2 = fopen("sysfiles/reacrates.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in genchemDEterms.c: file:\"sysfiles/reacrates.dat\" could not be opened...\n");
    fclose(fd);
    return -1;
  }
  fd3 = fopen("sysfiles/chemnames.dat", "w");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in genchemDEterms.c: file: \"sysfiles/chemnames.dat\" could not be opened for writing...\n");
    fclose(fd); fclose(fd2);
    return -1;
  }

  numreacs = readreacfile(allreacs, MAXREAC, "sysfiles/allreacs.dat");
  if(numreacs<0){
    fprintf(stderr, "ERROR in genchemDEterms.c: Some reactions could not be read. Exiting...\n");
    fclose(fd); fclose(fd2); fclose(fd3);
    return -1;
  }
  //  fprintf(stderr, "genchemDEterms: numreacs = %d\n", numreacs);

  // get the chemicals

  k=0;
  for(i=0;i<numreacs;i++){
    for(j=0;j<allreacs[i].numleft;j++)
      k = addvar1(k, allreacs[i].varsleft[j], chemarray, MAXVARS); 
    for(j=0;j<allreacs[i].numright;j++)
      k = addvar1(k, allreacs[i].varsright[j], chemarray, MAXVARS);
  }

  totchems = k;

  fprintf(fd, "// This file is generated by \"../genchemDEterms.c\"\n\n");
  fprintf(fd2, "// This file is generated by \"../genchemDEterms.c\" */\n// It contains the rterms for use inclusion in the o2funcs file.\n\n"); 

  fprintf(fd3, "// This file is generated by \"genchemDEterms.c\" It contains the names of all the chemicals in the system - i.e. all the left and right variables in the chemical reactions \n\n");

  if(totchems!=0){
    for(i=0;i<totchems;i++)
      fprintf(fd3, "%s\n", chemarray[i]);
  }
  fclose(fd3);
	
  printeqs(fd, fd2, numreacs, totchems, allreacs, chemarray); 

  free_reacarray(allreacs, numreacs);
  free_datarray(chemarray, totchems);

  fclose(fd);
  fclose(fd2);


  return 0;
}

//
//
// End of Main
//
//

void printeqs(FILE *fd, FILE *fd2, int numreacs, int totchems, struct react allreacs[], char *chemarray[]){
  // this routine generates the DEterms. fd is data file, fd2 is the file which holds the abstract DE terms for use in the functions. 
  int i, j, z, maxtermlength, maxrtermlength;
  char *rname, *rtype;
  char tempname[100];
  char *leftterms[numreacs], *rightterms[numreacs];
  char *rterm; /* to store the term */

  maxtermlength = 1000; /* the maximum length of a term - should be fine unless the reaction has lots and lots of substrates or products */
  maxrtermlength = 2000; /* maximum length of an rterm*/
  for(i=0;i<numreacs;i++){ 
    leftterms[i] = cvector(0, maxtermlength);
    rightterms[i] = cvector(0, maxtermlength);
  }

  for(i=0;i<numreacs;i++){  /* for each reaction */

    //
    // generate the left terms 
    //

    z = 0;

    for(j=0;j<allreacs[i].numleft;j++){
      if (allreacs[i].stoics == 0){
	if (atoi(allreacs[i].valsleft[j]) != 1){
	  if(j==0){
	    z = inspow(z, allreacs[i].varsleft[j], allreacs[i].valsleft[j], leftterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", leftterms[i], maxtermlength);
	    z = inspow(z, allreacs[i].varsleft[j], allreacs[i].valsleft[j], leftterms[i], maxtermlength);
	  }
	}
	else{
	  if(j==0){
	    z = ins(z, allreacs[i].varsleft[j], leftterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", leftterms[i], maxtermlength);
	    z = ins(z, allreacs[i].varsleft[j], leftterms[i], maxtermlength);
	  }
	}
      }
      else{ // stoics != 0
	if (atoi(allreacs[i].stoicsleft[j]) != 1){
	  if(j==0){
	    z = inspow(z, allreacs[i].varsleft[j], allreacs[i].stoicsleft[j], leftterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", leftterms[i], maxtermlength);
	    z = inspow(z, allreacs[i].varsleft[j], allreacs[i].stoicsleft[j], leftterms[i], maxtermlength);
	  }
	}
	else{
	  if(j==0){
	    z = ins(z, allreacs[i].varsleft[j], leftterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", leftterms[i], maxtermlength);
	    z = ins(z, allreacs[i].varsleft[j], leftterms[i], maxtermlength);
	  }
	}
      }
    }

    leftterms[i][z] = '\0';

    //
    // generate the right terms
    //

    z = 0;
    for(j=0;j<allreacs[i].numright;j++){
      if (allreacs[i].stoics == 0){
	if (atoi(allreacs[i].valsright[j]) != 1){
	  if(j==0)
	    z = inspow(z, allreacs[i].varsright[j], allreacs[i].valsright[j], rightterms[i], maxtermlength);
	  else{
	    z = ins(z, "*", rightterms[i], maxtermlength);
	    z = inspow(z, allreacs[i].varsright[j], allreacs[i].valsright[j], rightterms[i], maxtermlength);
	  }
	}
	else{
	  if(j==0){
	    z = ins(z, allreacs[i].varsright[j], rightterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", rightterms[i], maxtermlength);
	    z = ins(z, allreacs[i].varsright[j], rightterms[i], maxtermlength);
	  }
	}
      }
      else{ // stoics != 0
	if (atoi(allreacs[i].stoicsright[j]) != 1){
	  if(j==0)
	    z = inspow(z, allreacs[i].varsright[j], allreacs[i].stoicsright[j], rightterms[i], maxtermlength);
	  else{
	    z = ins(z, "*", rightterms[i], maxtermlength);
	    z = inspow(z, allreacs[i].varsright[j], allreacs[i].stoicsright[j], rightterms[i], maxtermlength);
	  }
	}
	else{
	  if(j==0){
	    z = ins(z, allreacs[i].varsright[j], rightterms[i], maxtermlength);
	  }
	  else{
	    z = ins(z, "*", rightterms[i], maxtermlength);
	    z = ins(z, allreacs[i].varsright[j], rightterms[i], maxtermlength);
	  }
	}
      }
    }
    rightterms[i][z] = '\0';
  
  } // end of "for each reaction..."

  //
  //
  // Generate the DEterms
  //
  //

  rterm = cvector(0, maxrtermlength);
  rterm[0] = '\0';
  fprintf(stderr, "There are %d chemicals involved.\n\n", totchems);
  
  for(j=0;j<numreacs;j++){ // for each reaction
    rtype = strdup(allreacs[j].reacttype);
    rname = strdup(allreacs[j].reactname);

    //
    //
    // create the file containing the rterms
    //
    //

    z = 0; // pointer in rterm
    if(strcmp(rtype, "MA1") == 0)
      z = MAterm(z, allreacs, j, "right", -1, 0, leftterms, rterm, maxrtermlength, "+");
    else if(strcmp(rtype, "rateterm") == 0)
      z = rateterm(z, allreacs, j, "right", -1, rterm, maxrtermlength);
    else if(strcmp(rtype, "ratetermsimp") == 0 || strcmp(rtype, "simp") == 0)
      z = ratetermsimp(z, allreacs, j, "right", -1, rterm, maxrtermlength);
    else if(strcmp(rtype, "MA2") == 0)
      z = MA2term(z, allreacs, j, "right", -1, leftterms, rightterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1") == 0)
      z = MM1term(z, allreacs, j, "right", -1, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "elec") == 0)
      z = electerm(z, allreacs, j, "right", -1, leftterms, rightterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub2") == 0)
      z = MM1sub2term(z, allreacs, j, "right", -1, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub3") == 0)
      z = MM1sub3term(z, allreacs, j, "right", -1, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub4") == 0)
      z = MM1sub4term(z, allreacs, j, "right", -1, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub5") == 0)
      z = MM1sub5term(z, allreacs, j, "right", -1, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "FD") == 0)
      z = FDterm(z, allreacs, j, "right", -1, leftterms, rightterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "prod") == 0)
      z = prodterm(z, allreacs, j, -1, 0, rterm, maxrtermlength);
    else if(strcmp(rtype, "use") == 0)
      z = useterm(z, allreacs, j, -1, 0, rterm, maxrtermlength);
    else fprintf(stderr, "ERROR in printeqs: %s is not a known reaction type\n", rtype);

    rterm[z] = '\0'; /* close off rterm */
    shaveplus(rterm);
    fprintf(fd2, "T_%s\t%s\n", rname, rterm);

    for(i=0;i<allreacs[j].numleft;i++){
      fprintf(fd, "******\n");
      fprintf(fd, "name: %s%s\n", allreacs[j].varsleft[i], rname);
      fprintf(fd, "keyvar: %s\n", allreacs[j].varsleft[i]); 

      if(allreacs[j].comps > 1){
	if(strlen(allreacs[j].compsleft[i]) > 50){fprintf(stderr, "ERROR in genchemDEterms: compartment name %s too long.\n", allreacs[j].compsleft[i]);fclose(fd);return;}
	if(strcmp(allreacs[j].compsleft[i], "1")==0 || strcmp(allreacs[j].compsleft[i], "1.0")==0)
	  strcpy(tempname, ""); // safe
	else
	  snprintf(tempname, 100, "/%s", allreacs[j].compsleft[i]);
      }
      else
	strcpy(tempname, ""); // safe

      // alternative

      if(strcmp(allreacs[j].valsleft[i], "1.0")== 0 || strcmp(allreacs[j].valsleft[i], "1")== 0)
        fprintf(fd, "term: -(T_%s)%s\n", rname, tempname);
      else
	fprintf(fd, "term: -%s*(T_%s)%s\n", allreacs[j].valsleft[i], rname, tempname);	   
    }
    for(i=0;i<allreacs[j].numright;i++){
      fprintf(fd, "******\n");
      fprintf(fd, "name: %s%s\n", allreacs[j].varsright[i], rname);
      fprintf(fd, "keyvar: %s\n", allreacs[j].varsright[i]);

      if(allreacs[j].comps > 1){
	if(strlen(allreacs[j].compsright[i]) > 50){fprintf(stderr, "ERROR in genchemDEterms: compartment name %s too long.\n", allreacs[j].compsright[i]);fclose(fd);return;}
	if(strcmp(allreacs[j].compsright[i], "1")==0 || strcmp(allreacs[j].compsright[i], "1.0")==0)
	  strcpy(tempname, ""); // safe
	else
	  snprintf(tempname, 100, "/%s", allreacs[j].compsright[i]);
      }
      else
	strcpy(tempname, ""); // safe

      // alternative

      if(strcmp(allreacs[j].valsright[i], "1.0")== 0 || strcmp(allreacs[j].valsright[i], "1")== 0)
	fprintf(fd, "term: +(T_%s)%s\n", rname, tempname);
      else
	fprintf(fd, "term: +%s*(T_%s)%s\n", allreacs[j].valsright[i], rname, tempname);

    }
    fprintf(fd, "******\n");

    //
    //
    // Now the terms divided by the first parameter if possible
    //
    //

    z = 0; // pointer in rterm
    if(strcmp(rtype, "MM1") == 0)
      z = MM1term(z, allreacs, j, "right", -2, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "rateterm") == 0)
      z = rateterm(z, allreacs, j, "right", -2, rterm, maxrtermlength);
    else if(strcmp(rtype, "MA2") == 0){
      if(strcmp(allreacs[j].rates[0], allreacs[j].rates[1]) == 0){
	z = MA2term(z, allreacs, j, "right", -2, leftterms, rightterms, rterm, maxrtermlength);
      }
    }
    else if(strcmp(rtype, "MM1sub2") == 0)
      z = MM1sub2term(z, allreacs, j, "right", -2, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub3") == 0)
      z = MM1sub3term(z, allreacs, j, "right", -2, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub4") == 0)
      z = MM1sub4term(z, allreacs, j, "right", -2, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "MM1sub5") == 0)
      z = MM1sub5term(z, allreacs, j, "right", -2, leftterms, rterm, maxrtermlength);
    else if(strcmp(rtype, "FD") == 0)
      z = FDterm(z, allreacs, j, "right", -2, leftterms, rightterms, rterm, maxrtermlength);
    rterm[z] = '\0'; /* close off rterm */
    if(!isonlyspace(rterm))
      fprintf(fd2, "R_%s\t%s\n", rname, rterm);

    free(rtype);
    free(rname);

  } // "for each reaction..." ends



}

//
//
//  End of printeqs
//
//

//
//
// In all of the files below, a negative value of "k" means 
// that we generate the abstract term. In particular if "k" = -2
// this means that we divide by the first parameter (Vmax or k_ 
// if both k's are identical)
//
//

int MAterm(int z1, struct react allreacs[], int j, char lr[], int k, int l, char *lrterms[], char *rterm, int rtermlength, char *sign)
{
  int z;
  z = z1;

  if(k == -2)
    fprintf(stderr, "WARNING: No functionality for k = -2 in MAterm yet.\n");

  if((strcmp(sign, "+") != 0) && (strcmp(sign, "-") != 0))
    fprintf(stderr, "ERROR in routine MAterm: %s is not a valid sign\n", sign);
  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "ERROR in routine MAterm: %s is not left or right\n", lr);


  if(strcmp(lr, "left") == 0){
    z = ins(z, " ", rterm, rtermlength);
    z = ins(z, sign, rterm, rtermlength);
    z = ins(z, " ", rterm, rtermlength);

    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
    if(strcmp(lrterms[j], "") == 0){}
    else{
      z = ins(z, "*", rterm, rtermlength);
      z = ins(z, lrterms[j], rterm, rtermlength);
    }
  }
  else if(strcmp(lr, "right") == 0){
    z = ins(z, " ", rterm, rtermlength);
    z = ins(z, sign, rterm, rtermlength);
    z = ins(z, " ", rterm, rtermlength);

    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
    if(strcmp(lrterms[j], "") == 0){}
    else{
      z = ins(z, "*", rterm, rtermlength);
      z = ins(z, lrterms[j], rterm, rtermlength);
    }
  }

  if(k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;
}


int MA2term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength)
{
  int z, flag=0;
  z = z1;

  // both rates the same

  if(strcmp(allreacs[j].rates[0], allreacs[j].rates[1]) == 0){
    flag = 1;
  }

  if(k == -2){
    if (flag == 1){}
    else{
      fprintf(stderr, "ERROR: No functionality for k = -2 in MA2term unless the two rates are equal.\n");
    }
  }

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "ERROR in routine MAterm: %s is not left or right\n", lr);

  //
  // rates are different
  //

  if(flag == 0){
    if(strcmp(lr, "left") == 0){

      if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
        z = ins(z, " - ", rterm, rtermlength);
        z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
        z = ins(z, "*(", rterm, rtermlength);
      }
      else
        z = ins(z, " -(", rterm, rtermlength);
    }
    else if(strcmp(lr, "right") == 0){
      if(k >=0 && atoi(allreacs[j].valsright[k]) != 1){
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
        z = ins(z, "*(", rterm, rtermlength);
      }
      else{
        if(k>=0)
	  z = ins(z, " +(", rterm, rtermlength);
        else
	  z = ins(z, "(", rterm, rtermlength);
      }
    }
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    if(strcmp(leftterms[j], "") == 0){}
    else{
      z = ins(z, "*", rterm, rtermlength);
      z = ins(z, leftterms[j], rterm, rtermlength);
    }
    z = ins(z, " - ", rterm, rtermlength);
    z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
    if(strcmp(rightterms[j], "") == 0){}
    else{
      z = ins(z, "*", rterm, rtermlength);
      z = ins(z, rightterms[j], rterm, rtermlength);
    }
    z = ins(z, ")", rterm, rtermlength);
  } // matches "if (flag == 0)..."
  else{ // both rate constants the same
    if(strcmp(lr, "left") == 0){
      if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
        z = ins(z, " - ", rterm, rtermlength);
        z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
        z = ins(z, "*", rterm, rtermlength);
	z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
        z = ins(z, "*(", rterm, rtermlength);
      }
      else{
	if(k==-2){
	  z = ins(z, " -(", rterm, rtermlength);
	}
	else{
	  z = ins(z, " - ", rterm, rtermlength);
	  z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
	  z = ins(z, "*(", rterm, rtermlength);
	}
      }
    }
    else if(strcmp(lr, "right") == 0){
      if(k >=0 && atoi(allreacs[j].valsright[k]) != 1){
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
        z = ins(z, "*", rterm, rtermlength);
	z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
        z = ins(z, "*(", rterm, rtermlength);
      }
      else{
	if(k==-2){
	  z = ins(z, "(", rterm, rtermlength);
	}
	else{
	  z = ins(z, " + ", rterm, rtermlength);
	  z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
	  z = ins(z, "*(", rterm, rtermlength);
	}
      }
    } // matches "else if(strcmp(lr, "right") == 0)..."

    if(strcmp(leftterms[j], "") == 0){
      z = ins(z, "1.0", rterm, rtermlength);
    }
    else{
      z = ins(z, leftterms[j], rterm, rtermlength);
    }
    z = ins(z, " - ", rterm, rtermlength);
    if(strcmp(rightterms[j], "") == 0){
      z = ins(z, "1.0", rterm, rtermlength);
    }
    else{
      z = ins(z, rightterms[j], rterm, rtermlength);
    }
    z = ins(z, ")", rterm, rtermlength);
  } // end of both rate constants the same

  if(k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;
}


int ratetermsimp(int z1, struct react allreacs[], int j, char lr[], int k, char *rterm, int rtermlength)
{
  int z;
  z = z1;
  if(k == -2)
    fprintf(stderr, "WARNING: No functionality for k = -2 in reaction type simp.\n");

  if(strcmp(lr, "left") == 0){
    z = ins(z, " - ", rterm, rtermlength);

    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    z = ins(z, "(", rterm, rtermlength);
    z = ins(z, allreacs[j].rateterm, rterm, rtermlength);
    z = ins(z, ")", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else if(k>=0)
      z = ins(z, " + ", rterm, rtermlength);

    z = ins(z, "(", rterm, rtermlength);
    z = ins(z, allreacs[j].rateterm, rterm, rtermlength);
    z = ins(z, ")", rterm, rtermlength);
  }

  if(k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0)
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }
  return z;
}

int rateterm(int z1, struct react allreacs[], int j, char lr[], int k, char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if(strcmp(lr, "left") == 0){
    z = ins(z, " - ", rterm, rtermlength);
    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else if(k>=0)
      z = ins(z, " + ", rterm, rtermlength);
  }

  if (k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }
  z = ins(z, "(", rterm, rtermlength);
  z = ins(z, allreacs[j].rateterm, rterm, rtermlength);
  z = ins(z, ")", rterm, rtermlength);
  if(k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0)
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }
  return z;
}


int MM1term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      z = ins(z, " - ", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k >=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if (k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/(", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
  z = ins(z, " + ", rterm, rtermlength);
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, ")", rterm, rtermlength);

  if (k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}

int electerm(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if(k == -2)
    fprintf(stderr, "WARNING: No functionality for k = -2 in electerm yet.\n");

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1)
      fprintf(stderr, "ERROR in function electerm in file \"genchemDEterms.c\": stoichiometry of chemicals can only be one\n");
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k >=0 && atoi(allreacs[j].valsright[k]) != 1)
      fprintf(stderr, "ERROR in function electerm in file \"genchemDEterms.c\": stoichiometry of chemicals can only be one\n");
    else
      z = ins(z, " - ", rterm, rtermlength);
  }

  z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
  z = ins(z, "*", rterm, rtermlength);
  z = ins(z, "(PD_mem_tisn - 58.0*log10(", rterm, rtermlength);
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/", rterm, rtermlength);
  z = ins(z, rightterms[j], rterm, rtermlength);
  z = ins(z, "))", rterm, rtermlength);

  if (k >=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;
}


int MM1sub2term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength)
     // two substrates on the left hand side
     // rates must be listed in the order Vmax, Km1, Km2, etc.
{
  int z;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k >=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      z = ins(z, " - ", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if (k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/((", rterm, rtermlength);
  if(allreacs[j].stoics == 0){
    if (atoi(allreacs[j].valsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].valsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].valsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }

    z = ins(z, ")*(", rterm, rtermlength);
    if (atoi(allreacs[j].valsleft[1]) != 1){
      z = inspow(z, allreacs[j].rates[2], allreacs[j].valsleft[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[1], allreacs[j].valsleft[1], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[2], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[1], rterm, rtermlength);
    }
    z = ins(z, "))", rterm, rtermlength);
  }
  else{ // stoics != 0 
    if (atoi(allreacs[j].stoicsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].stoicsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].stoicsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }

    z = ins(z, ")*(", rterm, rtermlength);
    if (atoi(allreacs[j].stoicsleft[1]) != 1){
      z = inspow(z, allreacs[j].rates[2], allreacs[j].stoicsleft[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[1], allreacs[j].stoicsleft[1], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[2], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[1], rterm, rtermlength);
    }
    z = ins(z, "))", rterm, rtermlength);
  }

  if(k>=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0)
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}

int MM1sub3term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength)
     // three substrates on the left hand side
{
  int z;
  int m;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k>=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else{
      z = ins(z, " - ", rterm, rtermlength);
    }
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if(k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }

  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/((", rterm, rtermlength);
  if(allreacs[j].stoics == 0){
    if (atoi(allreacs[j].valsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].valsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].valsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }
 
    for(m=1;m<3;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if (atoi(allreacs[j].valsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].valsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].valsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }
  else{ // stoics != 0
    if (atoi(allreacs[j].stoicsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].stoicsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].stoicsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }
 
    for(m=1;m<3;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if (atoi(allreacs[j].stoicsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].stoicsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].stoicsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }

  if(k>=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0)
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}


int MM1sub4term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength)
     // four substrates on the left hand side
{
  int z;
  int m;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k>=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else{
      z = ins(z, " - ", rterm, rtermlength);
    }
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if(k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/((", rterm, rtermlength);
  
  if(allreacs[j].stoics == 0){
    if (atoi(allreacs[j].valsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].valsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].valsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }

    for(m=1;m<4;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if (atoi(allreacs[j].valsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].valsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].valsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }
  else{ // stoics != 0
   if (atoi(allreacs[j].stoicsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].stoicsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].stoicsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }

    for(m=1;m<4;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if (atoi(allreacs[j].stoicsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].stoicsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].stoicsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }

  if(k>=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}

int MM1sub5term(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rterm, int rtermlength)
     // five substrates on the left hand side
{
  int z;
  int m;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k>=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      z = ins(z, " - ", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if(k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
  }
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, "/((", rterm, rtermlength);
  
  if(allreacs[j].stoics == 0){
    if(atoi(allreacs[j].valsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].valsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].valsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }
    for(m=1;m<5;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if(atoi(allreacs[j].valsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].valsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].valsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }
  else{ // stoics != 0
    if(atoi(allreacs[j].stoicsleft[0]) != 1){
      z = inspow(z, allreacs[j].rates[1], allreacs[j].stoicsleft[0], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = inspow(z, allreacs[j].varsleft[0], allreacs[j].stoicsleft[0], rterm, rtermlength);
    }
    else{
      z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].varsleft[0], rterm, rtermlength);
    }
    for(m=1;m<5;m++){
      z = ins(z, ")*(", rterm, rtermlength);
      if(atoi(allreacs[j].valsleft[m]) != 1){
        z = inspow(z, allreacs[j].rates[m+1], allreacs[j].stoicsleft[m], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = inspow(z, allreacs[j].varsleft[m], allreacs[j].stoicsleft[m], rterm, rtermlength);
      }
      else{
        z = ins(z, allreacs[j].rates[m+1], rterm, rtermlength);
        z = ins(z, " + ", rterm, rtermlength);
        z = ins(z, allreacs[j].varsleft[m], rterm, rtermlength);
      }
    }
    z = ins(z, "))", rterm, rtermlength);
  }

  if(k>=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}


int FDterm(int z1, struct react allreacs[], int j, char lr[], int k, char *leftterms[], char *rightterms[], char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if((strcmp(lr, "left") != 0) && (strcmp(lr, "right") != 0))
    fprintf(stderr, "%s is not left or right\n", lr);

  if(strcmp(lr, "left") == 0){
    if(k>=0 && atoi(allreacs[j].valsleft[k]) != 1){
      z = ins(z, " - ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      z = ins(z, " - ", rterm, rtermlength);
  }
  else if(strcmp(lr, "right") == 0){
    if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
      z = ins(z, " + ", rterm, rtermlength);
      z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
      z = ins(z, "*", rterm, rtermlength);
    }
    else
      if(k>=0)
	z = ins(z, " + ", rterm, rtermlength);
  }

  if (k != -2){
    z = ins(z, allreacs[j].rates[0], rterm, rtermlength);
    z = ins(z, "*(", rterm, rtermlength);
  }
  else{
    z = ins(z, "(", rterm, rtermlength);
  }
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, " - ", rterm, rtermlength);
  z = ins(z, rightterms[j], rterm, rtermlength);
  z = ins(z, ")/((", rterm, rtermlength);
  z = ins(z, leftterms[j], rterm, rtermlength);
  z = ins(z, " + ", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
  z = ins(z, "*(1.0 + ", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[2], rterm, rtermlength);
  z = ins(z, "))*(", rterm, rtermlength);
  z = ins(z, rightterms[j], rterm, rtermlength);
  z = ins(z, " + ", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
  z = ins(z, "*(1.0 + ", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[2], rterm, rtermlength);
  z = ins(z, ")) - pow(", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[1], rterm, rtermlength);
  z = ins(z, ", 2)*pow(", rterm, rtermlength);
  z = ins(z, allreacs[j].rates[2], rterm, rtermlength);
  z = ins(z, ", 2))", rterm, rtermlength);

  if(k>=0){
    if(allreacs[j].comps > 1){
      z = ins(z, "/", rterm, rtermlength);
      if(strcmp(lr, "left") == 0){
        z = ins(z, allreacs[j].compsleft[k], rterm, rtermlength);
      }
      else
        z = ins(z, allreacs[j].compsright[k], rterm, rtermlength);
    }
  }

  return z;

}


int prodterm(int z1, struct react allreacs[], int j, int k, int l, char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if(k == -2)
    fprintf(stderr, "WARNING: No functionality for k = -2 in prodterm yet.\n");

  if(k>=0 && atoi(allreacs[j].valsright[k]) != 1){
    z = ins(z, " + ", rterm, rtermlength);
    z = ins(z, allreacs[j].valsright[k], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
  }
  else{
    if(k>=0)
      z = ins(z, " + ", rterm, rtermlength);
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
  }
  return z;
}

int useterm(int z1, struct react allreacs[], int j, int k, int l, char *rterm, int rtermlength)
{
  int z;
  z = z1;

  if(k == -2)
    fprintf(stderr, "WARNING: No functionality for k = -2 in useterm yet.\n");

  if(k>=0 && atoi(allreacs[j].valsleft[k]) != 1){
    z = ins(z, " - ", rterm, rtermlength);
    z = ins(z, allreacs[j].valsleft[k], rterm, rtermlength);
    z = ins(z, "*", rterm, rtermlength);
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
  }
  else{
    z = ins(z, " - ", rterm, rtermlength);
    z = ins(z, allreacs[j].rates[l], rterm, rtermlength); 
  }
  return z;
}

// get rid of a leading plus sign

void shaveplus(char *s){
  int i=0;
  char *temp = strdup(s);
  while(isspace(temp[i]) || temp[i] == '+')
    i++;
  strcpy(s, temp+i); // safe
  free(temp);
}




