/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// copies the existing DEs and collates the DEterms into further DEs and outputs a data file containing all the DEs

#include "syssize.h"
#include "common/stringmanip.h"
#include "common/nrsimp.h"

int main()
{
  int i=0, j=0, k, m, maxrtermlength;
  int numexistDEs=0;
  char *kv;
  int z=0; /* tracks the key variable */
  int z1; /* the pointer in rterm */
  FILE *fd1, *fd2;
  char *temp = NULL; // to hold the names of keyvars
  char *keyvars[MAXVARS];
  char *rterm; /* to store the term */
  char *oneline;
  char *twoline;
  char **DEnames;
  char defile[100];
  int DEcounter;
  char *pstr;
  char *chems[100];
  char *chems2[100];
  int numvals;

  int numfixedvars=0;
  char **fixedvars;

  struct DEterm allDEterms[MAXDETERM];
  int numDEterms;

  strcpy(defile, "sysfiles/sysstate/DEs.dat"); // safe

 
  fixedvars = genarraydat("sysfiles/fixedvars.dat", &numfixedvars);

  fd1 = fopen("sysfiles/allDEs.dat", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in gendatDEs.c: \"sysfiles/allDEs.dat\" could not be opened...\n");
    return -1;
  }

  fd2 = fopen("sysfiles/readable1", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gendatDEs.c: \"sysfiles/readable1\" could not be opened...\n");
    return -1;
  }

  fprintf(fd1, "// This file is generated by the program \"../gendatDEs.c\"\n"); 
  /* first copy the existing DEs */
  fprintf(fd1, "// first the DEs which have been defined as DEs\n******\n");

  fprintf(fd2, "// This file is generated by the program \"../gendatDEs.c\". It contains the names of the DEterms which go to make up the DEs for a particular variable.\n\n"); 
  fprintf(fd2, "// First the DEs which have been pre-defined as DEs - here the DE is simply printed\n******\n\n");

  // Get the existing DEs

  DEnames = genarraydat(defile, &numexistDEs);
  DEcounter=0;

  for(i=0;i<numexistDEs;i++){
    pstr=getproc("sysfiles/DEs/allDEs.dat", DEnames[i]);
    if(pstr[0]==0){
      fprintf(stderr, "ERROR in \"gendatDEs.c\": DE %s could not be found in file \"sysfiles/DEs/allDEs.dat\"\n", DEnames[i]);
    }
    else{
      DEcounter++;
      kv = getname("keyvar:", pstr);
      if(isonlyspace(kv)){ // key variable implicit
	oneline = getrst("lterm:", pstr);
	numvals = chemgets1(oneline, chems, chems2, 100);
	free(oneline);
	if(numvals>0){
	  free(kv);
	  kv = strdup(chems2[0]);
	}
	free_datarray(chems, numvals);
	free_datarray(chems2, numvals);
      }
      if(isinarray(fixedvars, numfixedvars, kv)>=0){
	fprintf(fd1, "name: %s\nkeyvar: %s\n", DEnames[i], kv);
	oneline = getrst("lterm:", pstr);
	fprintf(fd1, "lterm: %s\nrterm: 0.0\n******\n", oneline);
	fprintf(fd2, "%s: fixed\n******\n", kv);
	free(oneline);
      }
      else{ // not a fixed variable
	fprintf(fd1, "%s", pstr);
	twoline = getrst("rterm:", pstr);
	fprintf(fd2, "%s: %s\n******\n", kv, twoline);
	free(twoline);
      }
      free(kv);
    }
    free(pstr);
  }

  if (DEcounter < numexistDEs)
    fprintf(stderr, "WARNING in \"gendatDEs.c\": some pre-defined DEs could not be found...\n");

  freearraydat(DEnames, numexistDEs);


  // now the DE terms
  // only generates DEs with one variable on the left
  // get the list of key variables and store in keyvars

  // read in the DEterms

  numDEterms = readDEtermfile(allDEterms, MAXDETERM, "sysfiles/allDEterms.dat");
  maxrtermlength = 2000;
  rterm = cvector(0, maxrtermlength);

  for(i=0;i<numDEterms;i++)
    z = addvar1(z, allDEterms[i].keyvar, keyvars, MAXVARS);

  fprintf(fd1, "\n// Now the DEs generated from the chemical and other DEterms. There are %d DEs defined in this way. \n\n", z);
  fprintf(fd2, "\n// Now the DEs generated from the chemical and other DEterms. There are %d DEs defined in this way. This time the terms are printed.\n\n", z);

  j=DEcounter;

  for(i=0;i<z;i++){ /* for each key variable */
    temp = strdup(keyvars[i]);
    fprintf(fd1, "******\nname: %s_sys\nkeyvar: %s\nlterm: 1.0, %s\n", temp, temp, temp);
    fprintf(fd2, "******\nD%s: ", temp);
    if(isinarray(fixedvars, numfixedvars, temp)>=0)
      fprintf(fd2, "[fixed] ");

    z1 = 0; /* the pointer in rterm */

    for(k=0;k<numDEterms;k++){
      if(strcmp(temp, allDEterms[k].keyvar) == 0){ 
	fprintf(fd2, "%s  ", allDEterms[k].name);
	m=0;
        while(isspace((int) allDEterms[k].rterm[m])){m++;}
        if((allDEterms[k].rterm[m] == '-') || (allDEterms[k].rterm[m] == '+'))  
	  z1 = ins(z1, allDEterms[k].rterm, rterm, maxrtermlength);
        else{
          z1 = ins(z1, " + ", rterm, maxrtermlength);
	  z1 = ins(z1, allDEterms[k].rterm, rterm, maxrtermlength);
        }
      }
    }
    fprintf(fd2, "\n");
    rterm[z1] = '\0';
    if(isinarray(fixedvars, numfixedvars, temp)>=0)
      fprintf(fd1, "rterm: 0.0\n");
    else
      fprintf(fd1, "rterm: %s\n", rterm);
    j++;
    free(temp);
  }

  fprintf(fd1, "******\n");
  fprintf(fd2, "******\n");

  fclose(fd1);
  fclose(fd2);

  free_datarray(keyvars, z);

  fd1 = fopen("sysfiles/othersysparams.dat", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in gendatDEs.c: \"sysfiles/othersysparams.dat\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd1, "/* This file just contains the number of system DEs, for the purposes of setting error tolerances */\n\nnumDEs %d\n", j);
  fclose(fd1);

  fprintf(stderr, "Total number of DEs = %d\n", j);

  freearraydat(fixedvars, numfixedvars);
  free_DEtermarray(allDEterms, numDEterms);
  return 0;
}
