/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */
#include "common/stringmanip.h"

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  char dirname[60] = "models/modeldescs/";
  char mkdircall[70] = "mkdir ";
  char tempname[100];
  char infofilename[110];
  char modulefilename[40];
  char mkmodulecall[100] = "cp -i models/modules/template.dat models/modules/";
  char mkmoduledircall[100] = "mkdir models/modules/";
  char mkpvaldircall[100] = "mkdir models/pvals/";
  char pvalname[100];

  if(argc < 2){
    fprintf(stderr, "ERROR in gennewsys.c: you must enter the name of the new system you wish to create.\n");
    return -1;
  }
  if(strlen(argv[1]) > 30){ // for safety
    fprintf(stderr, "ERROR in gennewsys.c: the chosen system name \"%s\" is too long. Make it less than 30 characters long.\n", argv[1]);
    return -1;
  }
  // the modeldescs directory



  strcat(dirname, argv[1]); // safe
  if (modelexists(argv[1])){
    fprintf(stderr, "ERROR in gennewsys: a system by the name %s already exists. If you want to remove it, use the remove model program.\n", argv[1]);
    return -2;
  }
  strcat(mkdircall, dirname); // safe
  fprintf(stderr, "%s\n", mkdircall);
  system(mkdircall);

 

  // the info file

  sprintf(infofilename, "%s/%sinfo", dirname, argv[1]); // safe
  fprintf(stderr, "Creating the file %s\n", infofilename);
  fd1 = fopen(infofilename, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: %s could not be opened...\n", infofilename);
    return -1;
  }

  // the module directory


  strcat(mkmoduledircall, argv[1]); // safe
  fprintf(stderr, "%s\n", mkmoduledircall);
  system(mkmoduledircall);

  // the module file

  sprintf(modulefilename, "%s.dt", argv[1]); // safe
  strcat(mkmodulecall, argv[1]); // safe
  strcat(mkmodulecall, "/"); // safe
  strcat(mkmodulecall, modulefilename); // safe
  fprintf(stderr, "%s\n", mkmodulecall);
  system(mkmodulecall);


  // the file alldata.dat

  // reactions
  sprintf(tempname, "%s/alldata.dat", dirname); // safe
  fd = fopen(tempname, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: %s could not be opened...\n", tempname);
    return -1;
  }
  fprintf(stderr, "Writing the model description file \"%s\"\n", tempname);
  fprintf(fd, "******** REACS ********\n// the reactions\n\n******** endREACS ********\n\n\n");
  //Algrels
  fprintf(fd, "******** ALGRELS ********\n// the algebraic relations\n\n******** endALGRELS ********\n\n\n");
  //DEterms
  fprintf(fd, "******** DETERMS ********\n// the DEterms\n\n******** endDETERMS ********\n\n\n");
  // DEs
  fprintf(fd, "******** DEQS ********\n// the ODEs\n\n******** endDEQS ********\n\n\n");
  // transvars
  fprintf(fd, "******** TRANSVARS ********\n// Any DEterms or reactions defined via templates\n\n******** endTRANSVARS ********\n\n\n");
  fprintf(fd, "******** MODFILES ********\n// the names of any general module files used in the model...\n\n\n\n******** endMODFILES ********\n\n");
  // output params
  fprintf(fd, "******** OUTPUT_PARAMS ********\n// the output params\n\noutput_params: \noutput_vars: \n\n******** endOUTPUT_PARAMS ********\n\n\n");
  // outvars
  fprintf(fd, "******** OUTVARS ********\n\n// enter variables you want the system to output when you run simulations.\n// Default is all\n\nALL\n\n******** endOUTVARS ********\n\n\n");

  // autoregoutvars
  fprintf(fd, "******** AUTOREGOUTVS ********\n\n// enter variables you want the system to output when you run static simulation.\n// Default is all\n\nALL\n\n******** endAUTOREGOUTVS ********\n\n******** endALLDATFILE ********\n\n\n");

  fclose(fd);

  fprintf(fd1, "PARAMDATAFILE solvedata.dat\n");

  //
  // parameter file
  //

  strcat(mkpvaldircall, argv[1]); // safe
  fprintf(stderr, "%s\n", mkpvaldircall);
  system(mkpvaldircall);

  sprintf(pvalname, "models/pvals/%s/%s_pvals.dat", argv[1], argv[1]); // safe
  fd = fopen(pvalname, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: %s could not be opened...\n", pvalname);
    return -1;
  }
  fprintf(stderr, "Creating the default parameter value file \"%s\"\n", pvalname);
  fprintf(fd, "// model parameters\n");
  fclose(fd);


  //
  // the autoreginputs directory
  //

  sprintf(mkmoduledircall, "mkdir models/autoreginputs/%s", argv[1]); // safe
  fprintf(stderr, "%s\n", mkmoduledircall);
  system(mkmoduledircall);

  sprintf(tempname, "models/autoreginputs/%s/%s_defauto.dat", argv[1], argv[1]); // safe
  fprintf(stderr, "writing file %s\n", tempname);
  fd = fopen(tempname, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: %s could not be opened...\n", tempname);
    return -1;
  }
  fprintf(fd, "// edit\n\nchosen_param: \ntime_step: \nstep_vals: \ninit_vals: \nmin_val: \nmax_val: \n");
  fclose(fd);


  //
  // the default input file in the models/inputs directory
  //

  sprintf(mkmoduledircall, "mkdir models/inputs/%s", argv[1]); // safe
  fprintf(stderr, "%s\n", mkmoduledircall);
  system(mkmoduledircall);

  sprintf(tempname, "models/inputs/%s/%s_default.dat", argv[1], argv[1]); // safe
  fprintf(stderr, "writing file %s\n", tempname);
  fd = fopen(tempname, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: %s could not be opened...\n", tempname);
    return -1;
  }
  fprintf(fd, "//default model behaviour\nsteady: none\n//chosen_param:\n******\n10\n");
  fclose(fd);

  fclose(fd1);

  // add the relevant bit to the Makefile

  fd = fopen("userfiles/model_types_make", "a");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: \"userfiles/model_types_make\" could not be opened...\n");
    return -1;
  }
  fprintf(stderr, "Adding data to the file \"userfiles/model_types_make\"\n"); 
  fprintf(fd, "%s: src/lib/P_makemodel userfiles/model_defaults\nifneq ($(MODEL), %s)\n\techo %s > sysfiles/sysstate/which_model_file\n\t./src/lib/P_makemodel\nendif\n", argv[1], argv[1], argv[1]);

  fclose(fd);

  fd = fopen("userfiles/model_defaults", "a");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gennewsys.c: \"userfiles/model_defaults\" could not be opened...\n");
    return -1;
  }
  fprintf(stderr, "Adding data to the file \"userfiles/model_defaults\"\n"); 
  fprintf(fd, "%s\t%s/%s_default.dat\t?\t%s/%s_pvals.dat\tnoauto\t%s/%s.dt\t%s/alldata.dat\t%s/%s_defauto.dat\n", argv[1], argv[1], argv[1], argv[1], argv[1], argv[1], argv[1], argv[1], argv[1], argv[1]);

  fclose(fd);

  fprintf(stderr, "New system %s successfully created.\n", argv[1]);
  return 0;
}
