/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */
#include "common/stringmanip.h"

void geno2funcsfile(char *paramdatfile, char *vardatfile, char *ratesdatfile, char *otherdatfile, char *readfile, char *writefile);

int main()
{
  geno2funcsfile("sysfiles/params.dat", "sysfiles/varbles.dat", "sysfiles/reacrates.dat", "sysfiles/alltempstrs.dat", "sysfiles/o2funcsr.cc", "sysfiles/o2funcs.cc");
  geno2funcsfile("sysfiles/params.dat", "sysfiles/varbles.dat", "sysfiles/reacrates.dat", "sysfiles/alltempstrs.dat", "sysfiles/initfuncsr.cc", "sysfiles/initfuncs.cc");
  return 0;
}

void geno2funcsfile(char *paramdatfile, char *vardatfile, char *ratesdatfile, char *otherdatfile, char *readfile, char *writefile)
{
  // uses the routine genarraydat to generate an appropriate array of parameter names from the function file, and then uses the routine transfunc1 and transfunc2 to write the file. Omits all comments and empty lines

  // 1) Replace ratenames
  // 2) Replace other tokens
  // 3) Add n to variable names (from the first two steps)
  // 4) Add "->getval" to parameter names
  char line[2000];
  char *line1;
  char *line2;
  char *line3;
  char *line4;
  char **paramarray;
  char **vararray;
  char **ratenamearray;
  char **ratearray;
  char **othernamearray;
  char **othertermarray;
  int numparams=0;
  int numvars=0;
  int numrates;
  int numotherterms;
  FILE *fd, *fd1;
  int linelen;

  //  fprintf(stderr, "number of varbles = %d\n\n", numvars);
  fd = fopen(readfile, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in geno2funcs.c: %s could not be opened...\n", readfile);
    return;
  }
  fd1 = fopen(writefile, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in geno2funcs.c: %s could not be opened...\n", writefile);
    fclose(fd);
    return;
  }

  paramarray = genarraydat(paramdatfile, &numparams);
  vararray = genarraydat(vardatfile, &numvars);
  numrates = genarraysdat(ratesdatfile, &ratenamearray, &ratearray);
  numotherterms = genarraysdat(otherdatfile, &othernamearray, &othertermarray);


  fprintf(fd1, "// This file is automatically generated from o2funcsr.cc by \"geno2funcs.c\"\n// Do not edit it directly.\n\n");

  while((linelen = getline(fd, line, 2000)) > 0){

    if(iscomsep(line)){
      if(strstr(line, "//////")) // only print delimiters
        fprintf(fd1, "%s", line);
    }
    else{
      if(strstr(line, "printf")){ // to avoid circularity
	fprintf(fd1, "%s", line);
      }
      else{ 

	line1 = transfunc2it(line, ratenamearray, ratearray, numrates);
	line2 = transfunc2(line1, othernamearray, othertermarray, numotherterms);
	line3 = transfunc1(line2, "n", vararray, numvars);
	line4 = transfunc1(line3, "->getval()", paramarray, numparams);
	fprintf(fd1, "%s", line4);
	free(line1);
	free(line2);
	free(line3);
	free(line4);
      }
    }
  }
  fclose(fd);
  fclose(fd1);

  freearraydat(paramarray, numparams);
  freearraydat(vararray, numvars);
  freearraydat(ratenamearray, numrates);
  freearraydat(ratearray, numrates);
  freearraydat(othernamearray, numotherterms);
  freearraydat(othertermarray, numotherterms);

  return;
}

