/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// creates sysfiles/vararray.dat sysfiles/paramarray.dat

// note that it is essential that we get the key variables for the DEs first and then those for the algrels, from the point of view of setting error tolerances in sys.cc
// Gets all the variables from the datDEs and the datalgs files (i.e. all the keyvars.)
// Gets all the tokens which aren't key variables and aren't rems (e.g. function names, names of constants, etc.)
// Checks that the latter are available in allparams

#include "syssize.h"
#include "common/stringmanip.h"

int main()
{
  int len, i=0, j, k, numleftval, numrightval, numparamsval, numpval, numvarsval, maxrtermlength, numvarsrems;
  FILE *fd, *fd1, *fd2, *fd3;
  char *algfile = "sysfiles/allalgrels1.dat";
  char *DEfile = "sysfiles/allDEs1.dat";
  char *keyvars[MAXVARS];
  char *varsrems[MAXVARS];
  char *nonkeyvars[MAXPARAMS];
  char *trueparams[MAXPARAMS];
  //  char *tvars[MAXVARS];
  char **tvars;
  int numt=0;
  int maxgets1 = 500;
  char *chems[500];
  char **tmparray=NULL;
  int numtmp=0;
  char **tmparray1=NULL;
  int numtmp1=0;
  char block[10000];
  char block1[10000];
  char *oneline; // for comments too
  int numrems;
  char **rems;
  char *tempvar;
  char *pname;
  char *kname;
  int numallvars;
  char **allvararray;
  int numallparams;
  char **allparamarray;
  int numtempvs;
  char **tempvs;
  char *pf, *vl, *oneword, *twoword, *blk;
  int suc;
  char **ratenamearray;
  char **ratearray;
  char *block2;
  int numrates;
  char outpfile[100];
  char outpstr[1000];
  char *outp[1000];
  int numoutp;
  char *outp1[1000];
  int numoutp1;

  strcpy(outpfile, "sysfiles/sysstate/output_params.dat"); //safe

  getrestfromfile(outpfile, "output_vars:", outpstr, 1000);
  numoutp = getallvars(outpstr, outp, 1000);

  getrestfromfile(outpfile, "output_params:", outpstr, 1000);
  numoutp1 = getallvars(outpstr, outp1, 1000);

  allvararray = genarraydat("sysfiles/allvarbles.dat", &numallvars); 
  // includes reaction rates
  tempvs = genarraydat("sysfiles/alltempstrs.dat", &numtempvs); 
  allparamarray = genarraydat("sysfiles/allparams.dat", &numallparams);
  rems = genarraydat("userfiles/remsfile.dat", &numrems);
  tvars = genarraydat("sysfiles/tempvars.dat", &numt);

  maxrtermlength = 2000;

  //
  // First a check that no reserved words have been used as variables or parameters
  //

  for(i=0;i<numallvars;i++){
    if((isinarray(rems, numrems, allvararray[i]) >= 0)){ 
      fprintf(stderr, "ERROR: you have created a variable called %s:\n\t%s is a reserved word. Please rename the variable.\n", allvararray[i], allvararray[i]);
      return -1;
    }
  }
  for(i=0;i<numallparams;i++){
    if((isinarray(rems, numrems, allparamarray[i]) >= 0)){ 
      fprintf(stderr, "ERROR: you have created a parameter called %s:\n\t%s is a reserved word. Please rename the parameter.\n", allparamarray[i], allparamarray[i]);
      return -1;
    }
  }
  for(i=0;i<numt;i++){
    if((isinarray(rems, numrems, tvars[i]) >= 0)){ 
      fprintf(stderr, "ERROR: you have created a temporary variable called %s:\n\t%s is a reserved word. Please rename the variable.\n", tvars[i], tvars[i]);
      return -1;
    }
  }

  freearraydat(tvars, numt);

  //
  // Get the keyvars - from the DEs first
  //

  numvarsval = 0;
  fd2 = fopen(DEfile, "r");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"gensysvp.c\" - File: %s could not be opened...\n", DEfile);
    return -1;
  }
  while((len = getblock(fd2, block, "******", 10000)) > 0){
    pname = getname("name:", block);
    if(strcmp(pname, "") != 0){
      kname = getname("keyvar:", block);
      if(strcmp(kname, "") == 0){ // keyvar doesn't have to be provided
	oneline = getrst("lterm:", block);
	tempvar = getnthtkn(oneline, 2);
	free(oneline);
	if(strcmp(tempvar, "") == 0){
	  fprintf(stderr, "ERROR in \"gensysvp.c\": in sysfiles/allDEs1.dat you must provide a keyvar in the ODE for %s.\n", pname);
	  free(pname); free(kname); free(tempvar); fclose(fd2);
	  return -1;
	}
	else if (numvarsval==MAXVARS){
	  fprintf(stderr, "ERROR in \"gensysvp.c\": maximum number of variables reached. Exiting.\n");
	  free(pname); free(kname); free(tempvar); fclose(fd2);
	  return -1;
	}
	else
	  numvarsval = addvar1(numvarsval, tempvar, keyvars, MAXVARS);
	free(tempvar);
      }
      else if (numvarsval==MAXVARS){
	fprintf(stderr, "ERROR in \"gensysvp.c\": maximum number of variables reached. Exiting.\n");
	free(pname); free(kname); fclose(fd2);
	return -1;
      }
      else
	numvarsval = addvar1(numvarsval, kname, keyvars, MAXVARS);
      free(kname);
    }
    free(pname);
  }
  fclose(fd2);

  // .. then the algrels

  fd3 = fopen(algfile, "r");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in \"gensysvp.c\" - File: %s could not be opened...\n", algfile);
    return -1;
  }
  while((len = getblock(fd3, block, "******", 10000)) > 0){
    pname = getname("name:", block);
    if(strcmp(pname, "") != 0){
      kname = getname("keyvar:", block);
      if(strcmp(kname, "") == 0){
	fprintf(stderr, "ERROR in \"gensysvp.c\": in sysfiles/allalgrels1.dat you must provide a keyvar.\n");
	free(pname); free(kname); fclose(fd3);
	return 1;
      }
      else if (numvarsval==MAXVARS){
	fprintf(stderr, "ERROR in \"gensysvp.c\": maximum number of variables reached. Exiting.\n");
	free(pname); free(kname); fclose(fd3);
	return -1;
      }
      else
	numvarsval = addvar1(numvarsval, kname, keyvars, MAXVARS);
      free(kname);
    }
    free(pname);
  }
  fclose(fd3);

  //
  // keyvars found - write vararray.dat
  //

  fd2 = fopen("sysfiles/vararray.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gensysvp.c: File \"sysfiles/vararray.dat\" could not be opened...\n");
    fclose(fd1);
    return -1;
  }
  fprintf(fd2, "// This file is generated by the program \"../gensysvp.c\". The file uses the information in \"sysfiles/allDEs1.dat\" and \"sysfiles/allalgrels1.dat\" \n\n");

  for(i=0;i<numvarsval-1;i++){
    fprintf(fd2, "%s\n", keyvars[i]);
  }
  if(numvarsval>0)
    fprintf(fd2, "%s\n", keyvars[i]);

  fclose(fd2);

  //
  // varbles.dat file (differs from vararray.dat - it has the init functions)
  //


  fd2 = fopen("sysfiles/varbles.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/varbles.dat\" could not be opened for writing.\n");
    return -1;
  }

  fprintf(fd2, "//Generated by gensysvp.c\n\n");
  for(i=0;i<numvarsval;i++){
    vl = getcvalfromfile("sysfiles/allvarbles.dat", keyvars[i]);
    if(!isonlyspace(vl))
      fprintf(fd2, "%s\t%s\n", keyvars[i], vl);
    else{
      fprintf(stderr, "ERROR in \"gensysvp.c\": An initial value/rule for %s could not be found. Exiting.\n", keyvars[i]);
      fclose(fd2); free(vl);
      return -1;
    }
    free(vl);
  }

  fclose(fd2);



  //
  // cleaned initfuncs.cc file
  //

  fd = fopen("sysfiles/varbles.dat", "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"gensysvp.c\": File \"sysfiles/varbles.dat\" could not be opened for reading.\n");
    return -1;
  }
  fd1 = fopen("sysfiles/temp/initfuncsr.cc", "r");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"gensysvp.c\": File \"sysfiles/temp/initfuncsr.cc\" could not be opened for reading.\n");
    return -1;
  }
  fd2 = fopen("sysfiles/initfuncsr.cc", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"gensysvp.c\": File \"sysfiles/initfuncsr.cc\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd2, "//Generated by gensysvp.c\n\n");



  // preamble
  getblock(fd1, block, "COMMON", 10000);
  getblock(fd1, block, "SPECIFIC", 10000);
  fprintf(fd2, "%s", block);
  fclose(fd1);
  // main part

  while((len = getline(fd, block, 10000)) > 0){
    if(!iscomsep(block)){
      oneword = getnthwd(block, 1);
      blk = getrst(oneword, block);
      twoword = getnthtkn(block, 2);
      if (strstr(twoword, "fcn") != NULL){
	suc = findfunc1("sysfiles/temp/initfuncsr.cc", oneword, block1, 10000, 2);
	if(suc == -1){
	  fprintf(stderr, "ERROR in gensysvp.c: function for %s couldn't be found in file \"sysfiles/temp/initfuncsr.cc\"\n", oneword);
	}
	else
	  fprintf(fd2, "%s", block1);
      }
      else if(!(isnumber(blk)))
	fprintf(fd2, "double %s_fcn()\n{\n  return %s;\n}\n///////////\n", oneword, blk);
      else // a number
	fprintf(fd2, "double %s_fcn()\n{\n  return %s;\n}\n///////////\n", oneword, blk);
      free(blk); free(twoword); free(oneword);
    }
  }

  fclose(fd);
  fclose(fd2);

  //
  // Now translate the reacrates file
  //

  fd = fopen("sysfiles/reacratesn.dat", "w");
  if(!fd){
    fprintf(stderr, "ERROR in \"gensysvp.c\": File \"sysfiles/reacratesn.dat\" could not be opened for writing.\n");
    return -1;
  }

  fprintf(fd, "// Generated by gensysvp.c\n\n");
  numrates = genarraysdat("sysfiles/reacrates.dat", &ratenamearray, &ratearray);
  for(i=0;i<numrates;i++){
    block2 = transfunc1(ratearray[i], "n", keyvars, numvarsval);
    fprintf(fd, "%s\t%s\n", ratenamearray[i], block2);
    free(block2);
  }
  freearraydat(ratenamearray, numrates);
  freearraydat(ratearray, numrates);
  fclose(fd);

  //
  // a dummy file, needed nevertheless
  //

  fd = fopen("sysfiles/reacratesn1.dat", "w");
  if(!fd){
    fprintf(stderr, "ERROR in \"gensysvp.c\": File \"sysfiles/reacratesn1.dat\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd, "// Generated by gensysvp.c\n\n");
  fclose(fd);

  //
  // now the parameters
  //

  numvarsrems = catarrays(varsrems, MAXVARS, keyvars, numvarsval, rems, numrems);
  numparamsval = 0;

  // .. first from the initialisation functions

  for(i=0;i<numvarsval;i++){
    vl = getcvalfromfile("sysfiles/varbles.dat", keyvars[i]);
    if(!isnumber(vl)){
      suc = findfunc1("sysfiles/initfuncsr.cc", keyvars[i], block1, 10000, 1);
      if(suc == -1){
	fprintf(stderr, "ERROR in \"gensysvp.c\": initialisation function for %s couldn't be found in file \"sysfiles/initfuncsr.cc\".\n", keyvars[i]);
      }
      else{
	numpval = getdepsfromfunc(block1, chems, maxgets1, varsrems, numvarsrems);
	for(k=0;k<numpval;k++){
	  numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
	}
	free_datarray(chems, numpval);
      }

    }
    free(vl);
  }



  // .. now the DEs

  fd3 = fopen(DEfile, "r");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in gensysvp.c: File %s could not be opened...\n", DEfile);
    return -1;
  }
  while((len = getblock(fd3, block, "******", 10000)) > 0){
    pname = getname("name:", block);
    if(strcmp(pname, "") != 0){
      oneline = getrst("lterm:", block);
      numleftval = getvars(oneline, chems, maxgets1, varsrems, numvarsrems);
      free(oneline);
      for(k=0;k<numleftval;k++){
	numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
      }
      free_datarray(chems, numleftval);

      oneline = getrst("params:", block);
      numpval = getvars(oneline, chems, maxgets1, varsrems, numvarsrems);
      free(oneline);
      for(k=0;k<numpval;k++){
	numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
      }
      free_datarray(chems, numpval);

      oneline = getrst("rterm:", block);
      numrightval = getvars(oneline, chems, maxgets1, varsrems, numvarsrems);
      free(oneline);
      for(k=0;k<numrightval;k++){
	numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
      }
      free_datarray(chems, numrightval);
    }
    free(pname);
  } // matches "while((len = getblock(fd2, block, "******", "

  fclose(fd3);


  //.. then the algrels

  fd2 = fopen(algfile, "r");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gensysvp.c: File %s could not be opened...\n", algfile);
    return -1;
  }
  while((len = getblock(fd2, block, "******", 10000)) > 0){
    pname = getname("name:", block);
    if(strcmp(pname, "") != 0){
      oneline = getrst("eqn:", block);
      numrightval = getvars(oneline, chems, maxgets1, varsrems, numvarsrems);
      free(oneline);
      for(k=0;k<numrightval;k++)
	numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
      free_datarray(chems, numrightval);
    } 
    free(pname);
  } 
  fclose(fd2);

  // Parameters associated with temp variables definitely needed by the 
  // DEs and algebraic equations in the model

  fd2 = fopen("sysfiles/tempvars2.dat", "r");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gensysvp.c: File \"sysfiles/tempvars2.dat\" could not be opened...\n");
    return -1;
  }
  while((len = getline(fd2, block, 10000)) > 0){
    if(!iscomsep(block)){
      tempvar = getnthtkn(block, 1);
      oneline = getrst(tempvar, block);
      free(tempvar);
      numrightval = getvars(oneline, chems, maxgets1, varsrems, numvarsrems);
      free(oneline);
      for(k=0;k<numrightval;k++)
	numparamsval = addvar1(numparamsval, chems[k], nonkeyvars, MAXPARAMS);
      free_datarray(chems, numrightval);
    } 
  } 
  fclose(fd2);

  // parameters associated with those temporary variables required only for output...

  for(i=0;i<numoutp;i++){
    tmparray1 = getvarlistfromfile("sysfiles/alltempstrs.dat", outp[i], &numtmp1, varsrems, numvarsrems);
    for(j=0;j<numtmp1;j++){
      tmparray = getvardepsfromfile("sysfiles/alltempstrs.dat", tmparray1[j], &numtmp, varsrems, numvarsrems);
      for(k=0;k<numtmp;k++){
	numparamsval = addvar1(numparamsval, tmparray[k], nonkeyvars, MAXPARAMS);
      }
      freearraydat(tmparray, numtmp);
    }
    freearraydat(tmparray1, numtmp1);
  }

  // parameters not otherwise used, required only for output...

  for(i=0;i<numoutp1;i++){
    numparamsval = addvar1(numparamsval, outp1[i], nonkeyvars, MAXPARAMS);
  }


  // output any problems

  for(i=0;i<numvarsval;i++){
    if(isinarray(allvararray, numallvars, keyvars[i]) < 0){
      pf=getgenname(keyvars[i]); // check generic name
      if(isonlyspace(pf) || (isinarray(allvararray, numallvars, pf)<0)){
	fprintf(stderr, "ERROR in \"gensysvp.c\": %s is a key variable, but has not been defined as a variable.\n", keyvars[i]);
	free(pf);
	return -1;
      }
      free(pf);
    }
  }


  for(i=0;i<numparamsval;i++){
    if(cisinarray(allparamarray, numallparams, nonkeyvars[i]) >= 0 || cisinarray(tempvs, numtempvs, nonkeyvars[i]) >= 0){}
    else{
      fprintf(stderr, "ERROR in \"gensysvp.c\": %s is not a key variable and\ncould not be defined as a parameter or a temporary variable\nor a reaction rate. Perhaps the expression depended on some\nunrecognised quantities.\n\n", nonkeyvars[i]);
      return -1;
    }
  }

  fprintf(stderr, "gensysvp: All variables and parameters in the system are defined.\n");

  // Write the file paramarray.dat

  // first separate out the temporary variables and rates

  numpval=0;


  for(i=0;i<numparamsval;i++){
    if(isinarray(tempvs, numtempvs, nonkeyvars[i]) >= 0){}
    else if(cisinarray(allparamarray, numallparams, nonkeyvars[i]) >= 0){
      trueparams[numpval] = strdup(nonkeyvars[i]);
      numpval++;
    }
    else{
      fprintf(stderr, "ERROR in \"gensysvp.c\": %s is not a key variable or a temporary variable, and has not been defined as a parameter.\n", nonkeyvars[i]);
      return -1;
    }
  }

  fd2 = fopen("sysfiles/paramarray.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gensysvp.c: File \"sysfiles/paramarray.dat\" could not be opened...\n");
    return -1;
  }
  fprintf(fd2, "// This file is generated by the program \"../gensysvp.c\". The file uses the information in \"sysfiles/allDEs1.dat\" and \"sysfiles/allalgrels1.dat\"\n\n");


  for(i=0;i<numpval-1;i++){
    fprintf(fd2, "%s\n", trueparams[i]);
  }
  if(numpval>0)
    fprintf(fd2, "%s\n", trueparams[i]);

  fclose(fd2);

  free_datarray(varsrems, numvarsrems);
  free_datarray(nonkeyvars, numparamsval);
  free_datarray(trueparams, numpval);
  free_datarray(keyvars, numvarsval);
  free_datarray(outp, numoutp);
  free_datarray(outp1, numoutp1);

  freearraydat(tempvs, numtempvs);
  freearraydat(allvararray, numallvars);
  freearraydat(allparamarray, numallparams);
  freearraydat(rems, numrems);

  //  getalldiffs("outdat/dat", "../sysarchiveold/braincirc2.0/outdat/dat", "outdat/tmp");
  //    fprintf(stderr, "\navdiff = %.4e\n", resmple("outdat/dat", "../sysarchiveold/braincirc2.0/outdat/dat", "r_1"));


  return 0;
}

