/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// This program uses templates to construct DEterms for variables 
// listed in the transvars section of a model descriptor file. 
// Templates behave much as reactions do - computationally they can 
// be seen as a generalisation of reactions.

#include "common/stringmanip.h"
#include "common/nrsimp.h"


int main()
{
  int i, j, len;
  int lim = 2000;
  char oneline[2000]; // for comments too
  int numbvars;
  int maxgets = 100;
  char *bvars[100];
  FILE *fd, *fd1, *fd2;
  char *temp;
  char transfile[100];
  char *tplate;
  char *chems[100];
  int numvals;
  char *vline;
  char *typ;
  int temptyp=0;
  char *wd2;
  int mult;
  char *p;
  char *multstr;
  char **vs;
  char **prefs;
  int *vinds;

  strcpy(transfile, "sysfiles/sysstate/transvars.dat"); // safe

  fd1 = fopen(transfile, "r");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in gentransportterms.c: File %s could not be opened...\n", transfile);
    free(tplate);
    return -1;
  }
  fd = fopen("sysfiles/transportterms.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in gentransportterms.c: File \"sysfiles/transportterms.dat\" could not be opened...\n");
    free(tplate);fclose(fd1);
    return -1;
  }
  fd2 = fopen("sysfiles/templatereacs.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in gentransportterms.c: File \"sysfiles/templatereacs.dat\" could not be opened...\n");
    free(tplate);fclose(fd1);fclose(fd);
    return -1;
  }

  fprintf(fd, "// This file contains the DE terms arising from transport processes. It is generated by the file \"../gentransportterms.c\". It uses a list of variables stored in \"%s\"\n\n", transfile);

  fprintf(fd2, "// The reactions generated by \"gentransportterms.c\" from templates. It uses a list of variables stored in \"%s\"\n\n", transfile);

  while((len = getline(fd1, oneline, lim)) > 0){
    if(!(iscomsep(oneline))){
      if(strstr(oneline, ":")){
	typ=getnthwd(oneline, 1);
	tplate=gnameobj("sysfiles/alltemplates.dat", typ);
	if(isonlyspace(tplate)){
	  fprintf(stderr, "ERROR in gentransportterms.c: A template called \"%s\" could not be found...\n", typ);
	  free(tplate);free(typ);
	  fclose(fd);fclose(fd1);
	  return -1;
	}

	vline = getrst("vars:", tplate);
	numvals = getallvars(vline, chems, 100);
	free(vline);
	vline = getrst("category:", tplate);
	if(strstr(vline, "DEterm")){temptyp=1;}
	else if (strstr(vline, "reac")){temptyp=2;}
	free(vline);

	wd2 = getnthwd(oneline+strlen(typ), 1);
	if(strstr(wd2, "times") && isnumber(wd2+5)){
	  mult = atoi(wd2+5);
	  multstr = strdup(wd2+5);
	  p = strstr(oneline, "times")+strlen(wd2);
	  numbvars = getallvars(p, bvars, maxgets);
	  if(numbvars!=numvals){
	    fprintf(stderr, "ERROR in gentransportterms.c: the number of \"%s\" is %d. It must be %d. Exiting.\n", typ, numbvars, numvals);
	    free_datarray(chems, numvals);
	    free_datarray(bvars, numbvars);
	    free(tplate);free(typ);free(multstr);
	    fclose(fd);fclose(fd1);
	    return -1;
	  }

	  vs=charar(0, numbvars-1);
	  prefs=charar(0, numbvars-1);
	  vinds = ivector(0, numbvars-1);
	  for(j=0;j<numbvars;j++){
	    prefs[j] = getprefix(bvars[j]);
	    vs[j] = (char*) malloc(sizeof(char) * (strlen(prefs[j]) + strlen(multstr) + 1));
	    vinds[j] = getvindex(bvars[j]);
	  }
	  for(i=0;i<mult;i++){
	    for(j=0;j<numbvars;j++){
	      sprintf(vs[j], "%s%d", prefs[j], vinds[j]+i+1);
	    }
	    temp = transtemp(tplate, vs, numvals);
	    if(temptyp==1)
	      fprintf(fd, "%s", temp);
	    else if (temptyp==2){
	      fprintf(fd2, "%s", temp);
	      //    fprintf(stderr, "%s", temp);
	    }
	    free(temp);
	  }
	  freearraydat(vs, numbvars);
	  freearraydat(prefs, numbvars);
	  free_ivector(vinds, 0, numbvars-1);
	}
	else{
	  free(wd2);
	  numbvars = getallvars(oneline+strlen(typ), bvars, maxgets);

	  if(numbvars%numvals!=0){
	    fprintf(stderr, "ERROR in gentransportterms.c: the number of \"%s\" is %d. It must be a multiple of %d. Exiting.\n", typ, numbvars, numvals);
	    free_datarray(chems, numvals);
	    free_datarray(bvars, numbvars);
	    free(tplate);free(typ);
	    fclose(fd);fclose(fd1);
	    return -1;
	  }

	  for(i=0;i< numbvars/numvals;i++){
	    temp = transtemp(tplate, bvars+numvals*i, numvals);
	    if(temptyp==1)
	      fprintf(fd, "%s", temp);
	    else if (temptyp==2)
	      fprintf(fd2, "%s", temp);
	    free(temp);
	  }
	}
	free_datarray(chems, numvals);
	free_datarray(bvars, numbvars);
	free(tplate);free(typ);
      }
    }
  }

  fprintf(fd, "******\n");
  fprintf(fd2, "******\n");
  fclose(fd);
  fclose(fd1);
  fclose(fd2);
  return 0;
}

