/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

/* A program to help generate the web of relations */

#include "syssize.h"
#include "common/stringmanip.h"
#define MAXDEterms 500
#define MAXCHEMS 300

int main()
{
  int i, j;
  FILE *fd;
  char *file;
  struct DE allDEs[MAXDE];
  int numDEs;
  struct algrel allalgrels[MAXALG];
  int numalgrels;

  // Read in the DEs and the algrels from the data files

  numDEs = readDEfile(allDEs, MAXDE, "sysfiles/allDEs1.dat", "sysfiles/vararray.dat", "sysfiles/allparams.dat", "userfiles/remsfile.dat");
  numalgrels = readalgfile(allalgrels, MAXALG, "sysfiles/allalgrels1.dat", "sysfiles/vararray.dat", "sysfiles/allparams.dat", "userfiles/remsfile.dat");


  file = "varweb.gdl"; /* the default output files */

  fd = fopen(file, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in genvarweb.c: File %s could not be opened for writing...\n", file);
    return -1;
  }

  /* get all the information. */
  fprintf(stderr, "going through the DEs... \n\n");
  fprintf(fd, "// the uclsysweb - generated by genvarweb.c //\n\n");
  fprintf(fd, "graph: { title: \"mygraph\"\nscaling: maxspect\ncolor: 15\nedge.color: 8\nedge.thickness: 2\nlayout_algorithm: forcedir\n\n");

  for(i=0;i<numDEs;i++){
    fprintf(fd, "\nnode: { color: 7 textcolor: 0 title:\"%s\" label:\"%s\" }\n", allDEs[i].keyvar, allDEs[i].keyvar);
    for(j=0;j<allDEs[i].numleft;j++){
      if(strcmp(allDEs[i].varsleft[j], allDEs[i].keyvar) == 0){
      }
      else
	fprintf(fd, "edge: { sourcename: \"%s\" targetname: \"%s\" color: 1 }\n", allDEs[i].varsleft[j], allDEs[i].keyvar);
    }
    for(j=0;j<allDEs[i].numright;j++){
      fprintf(fd, "edge: { sourcename: \"%s\" targetname: \"%s\" color: 1 }\n", allDEs[i].varsright[j], allDEs[i].keyvar);
    }
  }
  fprintf(stderr, "going through the algrels... \n\n");
  for(i=0;i<numalgrels;i++){
    fprintf(fd, "\nnode: { color: 7 textcolor: 0 title:\"%s\" label:\"%s\" }\n", allalgrels[i].keyvar, allalgrels[i].keyvar);
    for(j=0;j<allalgrels[i].numvars;j++){
      if(strcmp(allalgrels[i].vars[j], allalgrels[i].keyvar) == 0){
      }
      else
	fprintf(fd, "edge: { sourcename: \"%s\" targetname: \"%s\" color: 1 }\n", allalgrels[i].vars[j], allalgrels[i].keyvar);
      
    }
  }

  fprintf(fd, "}\n\n");
  fclose(fd);

  free_algarray(allalgrels, numalgrels);
  free_DEarray(allDEs, numDEs);

  return 0;
}
