/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// stores the body for the button classes

#include "BasicComponent.h"
#include "messagelabel.h"
#include <unistd.h> // for getcwd
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <X11/cursorfont.h>
#include <Xm/FileSB.h>
#include <stdio.h> // for printf
#include <stdlib.h> // for system
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "controls.h"
#include "../common/stringmanip.h"
#include "../common/nrsimp.h"

// in outfuncs.c
int plotall(char *xvar, char *plotstr, char *outputfstr, char *ops, int contplot, char *remsfile);
int printexprdeps(char *fname, char *s, char *tempstrfile, char *outvarfilename, char *paramfilename, char *remsfile);

int ANS;
long checkfile(char *fname){
  struct stat fst;
  stat(fname, &fst);
  return fst.st_size; // in bytes
}

void rewrap(char *s, int rat, int min, int max)
{
  int x;
  int len=strlen(s);
  if(rat==0 || len==0){return;}
  x=(int) floor(rat*pow(len/rat, 0.5));
  if(x>max){x=max;}
  if(x<min){x=min;}
  unwrap1(s);
  wrap(s, x);
}


void genhelpCallback(Widget widget, XtPointer client_data, XtPointer call_data); // in uclface.cc

void telluser(messagelabel **label, char mess[]){
  char *tmp;
  tmp=strdup(mess);
  unwrap(tmp); // get rid of newlines
  (*label)->setmessage(tmp);
  free(tmp);
  //  fprintf(stderr, "%s\n", mess);
}

typedef struct {
    char *label;
    char **strings;
    int size;
} ListItem;

void DestWidget(Widget widget, XtPointer client_data, XtPointer call_data)
{
  XtDestroyWidget (widget);
}

void changedialog(Widget widget, XtPointer client_data, XtPointer call_data)
{
  XmAnyCallbackStruct *cbs = 
    (XmAnyCallbackStruct *) call_data;
  //  reducefileControl *obj = (reducefileControl *) client_data;

  if (cbs->reason == XmCR_OK){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    ANS=1;
  }
  else if (cbs->reason == XmCR_CANCEL){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    ANS=2;
  }
  return;
}


Widget PostDialog(Widget parent, int dialog_type, char *msg)
{
  Widget dialog;
  XmString text;

  dialog = XmCreateMessageDialog (parent, "dialog", NULL, 0);
  text = XmStringCreateLtoR (msg, "MY_TAG");
  XtVaSetValues (dialog,
    XmNdialogType,    dialog_type,
    XmNmessageString, text,
    NULL);
  XmStringFree (text);
  XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
  XtSetSensitive (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  //XtAddCallback (dialog, XmNokCallback, XtDestroyWidget, NULL);
  XtAddCallback (dialog, XmNokCallback, DestWidget, NULL);
  XtManageChild (dialog);
  return dialog;
}


int ex_cmnd(char *cmnd, Widget *text_comout, Widget *toplevel){
  int pos;
  char buf[1000];
  FILE *pp;

  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), cmnd);
  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\n");
  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
  XmUpdateDisplay(*toplevel);
  // run the command
  if (!(pp = popen(cmnd, "r")))
    perror (cmnd);
  for (pos=XmTextGetLastPosition(*text_comout);fgets(buf, sizeof buf, pp); pos += strlen (buf)){
    XmTextReplace (*text_comout, pos, pos, buf);
    XmUpdateDisplay(*toplevel);
    XmTextShowPosition (*text_comout, pos);
  }
  return pclose(pp);

}

int wrongdir(char *wdir){
  int dirflag;
  char wdname[200];
  getcwd(wdname, 200);
  if(strcmp(wdname, wdir) == 0)
    dirflag = 0;
  else
    dirflag = 1;
  return dirflag;
}


//
//
// PostErrorDialog() routine
//
//
//

Widget PostErrorDialog(Widget parent, messagelabel **label, char *msg)
{
  Widget dialog;
  XmString text, titlestring;
  dialog = XmCreateErrorDialog(parent, "error", NULL, 0);
  text = XmStringCreateLtoR(msg, "MY_TAG");
  titlestring = XmStringCreateLocalized("ERROR");
  XtVaSetValues(dialog, XmNmessageString, text, XmNdialogTitle, titlestring, NULL);
  XmStringFree(text);
  XmStringFree(titlestring);
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
  telluser(label, msg);
  return dialog;
}

Widget PostEDialog(Widget parent, char *msg) // without the label setting...
{
  Widget dialog;
  XmString text, titlestring;
  dialog = XmCreateErrorDialog(parent, "error", NULL, 0);
  text = XmStringCreateLtoR(msg, "MY_TAG");
  titlestring = XmStringCreateLocalized("ERROR");
  XtVaSetValues(dialog, XmNmessageString, text, XmNdialogTitle, titlestring, NULL);
  XmStringFree(text);
  XmStringFree(titlestring);
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
  return dialog;
}

Widget PostLongDialog(Widget parent, char *title, char *msg)
{
  Widget dialog;
  XmString text, titlestring;
  dialog = XmCreateErrorDialog(parent, "error", NULL, 0);
  text = XmStringCreateLtoR(msg, "MY_TAG");
  titlestring = XmStringCreateLtoR(title, "MY_TAG");
  XtVaSetValues(dialog, XmNmessageString, text, XmNdialogTitle, titlestring, NULL);
  XmStringFree(text);
  XmStringFree(titlestring);
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
  return dialog;
}


Widget PostHelpDialog(Widget parent, char *title, char *msg)
{
  Widget dialog;
  XmString text, titlestring;
  dialog = XmCreateInformationDialog(parent, "help", NULL, 0);
  text = XmStringCreateLtoR(msg, "MY_TAG");
  titlestring = XmStringCreateLtoR(title, "MY_TAG");
  XtVaSetValues(dialog, XmNmessageString, text, XmNdialogTitle, titlestring, NULL);
  XmStringFree(text);
  XmStringFree(titlestring);
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
  return dialog;
}


//
//
// A simple general timeoutcursors routine - seems to work. 
// Be aware of the static variable though!
//


void TOC(Boolean on, Widget *wid){
  static int locked; // initialised to zero by default - internal static variables are private to a function, but are not reinitialised each time the function is called. 
  static Cursor cursor;
  XSetWindowAttributes attrs;
  Display *dpy = XtDisplay (*wid);
  XEvent event;

  if (on)
    locked++;
  else
    locked--;
  if(locked > 1 || locked ==1 && on == 0)
    return;

  //  fprintf(stderr, "locked = %d\n", locked);

  if(!cursor){
    cursor = XCreateFontCursor(dpy, XC_watch);
    //    fprintf(stderr, "cursor created\n");
  }

  attrs.cursor = on ? cursor : None;

  XChangeWindowAttributes (dpy, XtWindow (*wid), CWCursor, &attrs);

  XFlush (dpy);

  if (on){}
  else{
    while (XCheckMaskEvent (dpy,
                            ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
			    | PointerMotionMask | KeyPressMask, &event)){}
  }
}

// identical to TOC. We need a copy in case we need the timeoutcursors to work for more than one widget... 

void TOC1(Boolean on, Widget *wid){
  static int locked; // initialised to zero by default - internal static variables are private to a function, but are not reinitialised each time the function is called. 
  static Cursor cursor;
  XSetWindowAttributes attrs;
  Display *dpy = XtDisplay (*wid);
  XEvent event;

  if (on)
    locked++;
  else
    locked--;
  if(locked > 1 || locked ==1 && on == 0)
    return;

  if(!cursor){
    cursor = XCreateFontCursor(dpy, XC_watch);
  }

  attrs.cursor = on ? cursor : None;

  XChangeWindowAttributes (dpy, XtWindow (*wid), CWCursor, &attrs);

  XFlush (dpy);

  if (on){}
  else{
    while (XCheckMaskEvent (dpy,
                            ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
			    | PointerMotionMask | KeyPressMask, &event)){}
  }

}


//
//
// plotControl
//
//


void plotControl::plotCallback(Widget, XtPointer client_data, XtPointer){
  plotControl *obj = (plotControl *) client_data;
  obj->plot();
};

void plotControl::plot()
{
  int l;
  char buf[500];
  char outputfstr[100];
  char plotheaderstr[100];
  char plotcomsstr[100];
  char gnuplotcom[100];
  char makecom[50];
  char systemcom[200];
  Widget dialog;
  char remsfile[50];

  if(fileexists("userfiles/mathrems.dat"))
    strcpy(remsfile, "userfiles/mathrems.dat"); // safe
  else
    strcpy(remsfile, "userfiles/remsfile.dat"); // safe


  // get the gnuplot command
  if(getoption("userfiles/gen_options", "GNUPLOT", gnuplotcom, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  // get the file which will store the plot commands
  if(getoption("userfiles/gen_options", "PLOTCOMS", plotcomsstr, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }
  // get the file which stores the plot header information
  if(getoption("userfiles/gen_options", "PLOTHEADER", plotheaderstr, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  // get the file which stores the plot header information
  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }


  strncpy(buf, XmTextGetString(*text), 500);
  strncpy(outputfstr, XmTextGetString(*datfile), 100);

  if(isonlyspace(buf)){
    dialog = PostErrorDialog(_w, label, "ERROR: No variables to plot entered.");
    return;
  }
  if(isonlyspace(outputfstr)){
    dialog = PostErrorDialog(_w, label, "ERROR: No output files specified");
    return;
  }


  //  numdatfiles = chemgets(outputfstr, datfiles, 10);

  TOC(True, toplevel);
  l=plotall("", buf, outputfstr, "", 0, remsfile);
  TOC(False, toplevel);

  if(l<0){
    if (l == -9){dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");}
    else if(l == -1){dialog = PostErrorDialog(_w, label, "No variable/file entered...");}
    else if(l == -2){
      snprintf(systemcom, 200, "Check that the model matches the parameter\nvalue file.");
      dialog = PostLongDialog(_w, "ERROR", systemcom);
      telluser(label, "ERROR: Parameter value file does not seem to match the model.");
    }
    else if (l == -3){
      dialog = PostLongDialog(_w, "Plotting error", "Problem with plotting: see terminal for details");
      telluser(label, "Plotting Error");
    }
    else if (l == -4){dialog = PostErrorDialog(_w, label, "Output file unavailable or invalid"); }
    else if (l == -5){dialog = PostErrorDialog(_w, label, "Output variable not found");}
    else if (l == -6){dialog = PostErrorDialog(_w, label, "WARNING: some variables/files not found\nSee terminal for details.");}
    else if (l == -7){dialog = PostErrorDialog(_w, label, "Neither output variable found");}
    else if (l == -8){dialog = PostErrorDialog(_w, label, "WARNING: maximum two variables can be plotted");}
    else if (l == -10){dialog = PostErrorDialog(_w, label, "WARNING: first output variable not found");}
    else if (l == -11){dialog = PostErrorDialog(_w, label, "WARNING: second output variable not found");}
    else if (l == -12){dialog = PostErrorDialog(_w, label, "None of the output variables/expressions were found");}
    else if (l == -13){dialog = PostErrorDialog(_w, label, "WARNING: with the sim option only one file is read");}
    return;
  }


}

void plotControl::helpCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  plotControl *obj = (plotControl *) client_data;
  obj->help();
}

void plotControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "gnuplot_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on plotting", vardesc);

}

plotControl::plotControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *plotfile, Widget *text_pfile, char *name, char *display, messagelabel **label1, Widget *text1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  text = text1; // set pointer to the correct label
  datfile = plotfile;
  toplevel = toplevel1;
  text_comout = text_comout1;
  pfile = text_pfile;

  // Set what to display on the button

  XmString xmstr;
  xmstr = XmStringCreateSimple(display);

  // Create the button

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&plotControl::plotCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&plotControl::helpCallback,
		(XtPointer) this);
}; // end of constructor


//
//
// plotToggle - slightly misnamed - it's about static vs dynamic
//
//

plotToggle::plotToggle(Widget parent, Widget *modelname, Widget *infile, Widget *outfile, char *name, char *display, messagelabel **label1, smoothToggle **stog1):BasicComponent (name){

  label = label1; // a pointer to the label to be set
  XmString xmstr;
  inf = infile;
  outf = outfile;
  mname = modelname;
  autoinfstate[0] = 0; // initialise
  infstate[0] = 0; // initialise
  outfstate[0] = 0; // initialise
  stog = stog1;

  xmstr = XmStringCreateSimple(display);

  _w = XtVaCreateManagedWidget 
    ("plottoggle", 
     xmToggleButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNvalueChangedCallback,
		&plotToggle::plottogCallback,
		(XtPointer) this);

  // the help callback

  XtAddCallback(_w, XmNhelpCallback,
		&plotToggle::helpCallback,
		(XtPointer) this);
}

void plotToggle::activate(Widget widget, XtPointer client_data, XtPointer call_data)
{ 
  char *infstatetmp;
  char *outfstatetmp;
  char *modname;
  infstatetmp = XmTextGetString(*inf);
  outfstatetmp = XmTextGetString(*outf);
  modname = XmTextGetString(*mname);
  XmToggleButtonCallbackStruct *state = (XmToggleButtonCallbackStruct *) call_data;
  char datfile[100];
  Widget dialog;
  char *tmp;



  if(getoption("userfiles/gui_options", "DATAFILE", datfile, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if (state->set == true){
    XtSetSensitive((*stog)->baseWidget(), FALSE);
    if(autoinfstate[0]==0){
      tmp=getmoddef("userfiles/model_defaults", modname, "autoinput");
      if(strcmp(tmp, "models/autoreginputs/")!=0){
	XmTextSetString(*inf, tmp);
	XmTextSetCursorPosition (*inf, strlen (tmp));
      }
      else{
	XmTextSetString(*inf, "models/autoreginputs/pres.dat");
	XmTextSetCursorPosition (*inf, strlen ("models/autoreginputs/pres.dat"));
      }
      free(tmp);
    }
    else{
      XmTextSetString(*inf, autoinfstate);
      XmTextSetCursorPosition (*inf, strlen (autoinfstate));
    }
   

    if(outfstate[0]==0)
      XmTextSetString(*outf, "outdat/plot");
    else{
      XmTextSetString(*outf, outfstate);
      XmTextSetCursorPosition (*outf, strlen (outfstate));
    }
  }
  else{
    XtSetSensitive((*stog)->baseWidget(), TRUE);
    XmTextSetString(*inf, infstate);
    XmTextSetCursorPosition (*inf, strlen (infstate));

    if(outfstate[0]==0){
      XmTextSetString(*outf, datfile);
      XmTextSetCursorPosition (*outf, strlen (datfile));
    }
    else{
      XmTextSetString(*outf, outfstate);
      XmTextSetCursorPosition (*outf, strlen (outfstate));
    }

  }

  if(strlen(infstatetmp)<99)
    strcpy(infstate, infstatetmp); // safe
  else
    fprintf(stderr, "WARNING in controls.cc: plotToggle::activate: input file name too long.\n");

   if(strlen(outfstatetmp)<99)
     strcpy(outfstate, outfstatetmp); // safe
   else
     fprintf(stderr, "WARNING in controls.cc: plotToggle::activate: output file name too long.\n");

  XtFree(infstatetmp);
  XtFree(outfstatetmp);
}

void plotToggle::plottogCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  plotToggle *obj = (plotToggle *) client_data;
  obj->activate(widget, client_data, call_data);
}

void plotToggle::helpCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  plotToggle *obj = (plotToggle *) client_data;
  obj->help();
}

void plotToggle::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "autoreg_togglebutton", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on static vs dynamic simulations", vardesc);

}



//
//
// smoothToggle
//
//

smoothToggle::smoothToggle(Widget parent, char *name, char *display, messagelabel **label1):BasicComponent (name){

  label = label1; // a pointer to the label to be set
  XmString xmstr;

  xmstr = XmStringCreateSimple(display);

  _w = XtVaCreateManagedWidget 
    ("smoothtoggle", 
     xmToggleButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XtAddCallback(_w, XmNhelpCallback,
		&smoothToggle::helpCallback,
		(XtPointer) this);

  XmStringFree (xmstr); 
};

void smoothToggle::smooth()
{ 
  // no action is actually performed by pressing the smooth button. However when the simulation is run, then the file is smoothed first.

}

void smoothToggle::helpCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  smoothToggle *obj = (smoothToggle *) client_data;
  obj->help();
}

void smoothToggle::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "smooth_togglebutton", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on input smoothing", vardesc);

}


void smoothToggle::smoothCallback(Widget, XtPointer client_data, XtPointer){
  smoothToggle *obj = (smoothToggle *) client_data;
  obj->smooth();
}



//
//
// contplotToggle
//
//

contplotToggle::contplotToggle(Widget parent, char *name, char *display, messagelabel **label1):BasicComponent (name){

  label = label1; // a pointer to the label to be set
  XmString xmstr;

  xmstr = XmStringCreateSimple(display);

  _w = XtVaCreateManagedWidget 
    ("contplottoggle", 
     xmToggleButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XtAddCallback(_w, XmNhelpCallback,
		&contplotToggle::helpCallback,
		(XtPointer) this);

  XmStringFree (xmstr); 
};

void contplotToggle::contplot()
{ 
  // currently, simply pressing the contplot togglebutton causes no action to be taken. Only when the simulation is actually run is the state of this button important. 

}

void contplotToggle::contplotCallback(Widget, XtPointer client_data, XtPointer){
  contplotToggle *obj = (contplotToggle *) client_data;
  obj->contplot();
}

void contplotToggle::helpCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  contplotToggle *obj = (contplotToggle *) client_data;
  obj->help();
}

void contplotToggle::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "contplot_togglebutton", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on run-time plotting", vardesc);

}

//
//
// makeControl
//
//


void makeControl::make()
{
  int k;
  bool autoreg;
  Widget dialog;
  char systemcom[200];
  char makecom[50];


  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }


  autoreg = XmToggleButtonGetState(tog->baseWidget());

  telluser(label, "Now trying to make the project...");

  if(autoreg==false){
    TOC(True, toplevel);
    k = system(makecom);
    TOC(False, toplevel);
    if (k==0)
      dialog = PostHelpDialog(_w, "SUCCESS", "Making successful");
    else
      dialog = PostErrorDialog(_w, label, "Model could not be made: see terminal for details");
  }
  else{
    snprintf(systemcom, 200, "%s src/lib/P_autoreg", makecom);
    TOC(True, toplevel);
    k = system(systemcom);
    TOC(False, toplevel);
    if (k==0)
      dialog = PostHelpDialog(_w, "SUCCESS", "Making successful");
    else
      dialog = PostErrorDialog(_w, label, "Model could not be made: see terminal for details");
  }
};

void makeControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  makeControl *obj = (makeControl *) client_data;
  obj->help();
};

void makeControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "make_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on making a model", vardesc);

}


makeControl::makeControl(Widget parent, Widget *toplevel1, char *name, char display[], messagelabel **label1, plotToggle *tog1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  tog = tog1;
  // Set what to display on the button
  toplevel=toplevel1;
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);

  // Create the button

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 8,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
	        &makeControl::makeCallback,
	        (XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&makeControl::helpCallback,
		(XtPointer) this);
} // end of constructor

void makeControl::makeCallback(Widget, XtPointer client_data, XtPointer){
  makeControl *obj = (makeControl *) client_data;
  obj->make();
}

//
//
// runControl
//
//

void runControl::run()
{ 
  int k;

  telluser(label, "executing... please wait");
  k = system("make dat");
  telluser(label, "execution finished...");
}

void runControl::runCallback(Widget, XtPointer client_data, XtPointer){
  runControl *obj = (runControl *) client_data;
  obj->run();
}

runControl::runControl(Widget parent, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  // Set what to display on the button

  XmString xmstr;
  xmstr = XmStringCreateSimple(display);

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&runControl::runCallback,
		(XtPointer) this);
} // end of constructor


//
//
// makerunControl
//
//

void makerunControl::stop()
{
  fprintf(stderr, "interrupting...\n");
  stopped = true;
}


void makerunControl::TimeoutCursors(Boolean on, Boolean interruptable)
{
  static int locked;
  static Cursor cursor;
  XSetWindowAttributes attrs;
  Display *dpy = XtDisplay (*toplevel);
  XEvent event;
  Arg args[5];
  int n;
  XmString str;


  if (on)
    locked++;
  else
    locked--;

  if(locked > 1 || locked ==1 && on == 0)
    return;

  stopped=false;

  if(!cursor)
    cursor = XCreateFontCursor(dpy, XC_watch);

  attrs.cursor = on ? cursor : None;

  XChangeWindowAttributes (dpy, XtWindow (*toplevel), CWCursor, &attrs);

  XFlush (dpy);

  if (on){
    n=0;
    str = XmStringCreateLocalized ("Busy -- Please Wait.");
    XtSetArg (args[n], XmNmessageString, str); n++;
    dialog = XmCreateWorkingDialog (*toplevel, "busy", args, n);
    XmStringFree (str);
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_OK_BUTTON));
    XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));
    if (interruptable){
      str=XmStringCreateLocalized ("Stop");
      XtVaSetValues (dialog, XmNcancelLabelString, str, NULL);
      XmStringFree (str);
      XtAddCallback (dialog, XmNcancelCallback, &makerunControl::stopCallback, (XtPointer) this);
    }
    else
      XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
    XtManageChild (dialog);
  }
  else{
    while (XCheckMaskEvent (dpy,
                            ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
			    | PointerMotionMask | KeyPressMask, &event)){

    }
    XtDestroyWidget (dialog);
  }

}


Boolean makerunControl::CheckForInterrupt()
{
  Display *dpy = XtDisplay (*toplevel);
  Window win = XtWindow (dialog);
  XEvent event;

  XFlush (dpy);

  XmUpdateDisplay (*toplevel);

  while (XCheckMaskEvent (dpy,
                          ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
			  | PointerMotionMask | KeyPressMask, &event)){
    if(event.xany.window == win)
      XtDispatchEvent(&event);
    else
      XBell (dpy, 50);
  }
  return stopped;
}

void makerunControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  makerunControl *obj = (makerunControl *) client_data;
  obj->dlg(widget, client_data, call_data);
}

void makerunControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  XmAnyCallbackStruct *cbs = 
    (XmAnyCallbackStruct *) call_data;

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "makerun_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
    }
    Helpdialog = PostHelpDialog(widget, "Help on simulation", vardesc);
    break;
  default:
    break;
  }

  if (cbs->reason == XmCR_OK){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    answer=1;
  }
  else if (cbs->reason == XmCR_CANCEL){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    answer=2;
  }
  return;
}

// These two are actually quite generic for modal dialogs

void makerunControl::changedlg_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  makerunControl *obj = (makerunControl *) client_data;
  obj->changedlg(widget, client_data, call_data);
}

void makerunControl::changedlg(Widget widget, XtPointer client_data, XtPointer call_data)
{

  XmAnyCallbackStruct *cbs = 
    (XmAnyCallbackStruct *) call_data;

  if (cbs->reason == XmCR_OK){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    answer=1;
  }
  else if (cbs->reason == XmCR_CANCEL){
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    answer=2;
  }
  return;
}


void makerunControl::makerun()
{
  int l;
  bool autoreg, smth, contplot;
  char *inputfstr;
  char *modnamestr;
  char *pfilestr;
  char *inputfilereal;
  char buf[1000];
  FILE *fd;
  char systemcom1[400];
  struct stat fst;
  off_t sizeold=0;
  int q, q1, q2;
  char varname[100];
  int needtomake;
  int runpid=-4800;
  int kld=0;
  int interruptnum=0;
  Widget errdialog;
  Widget confirmdialog;
  XmString str, yes, titlestring;
  int size_flag;
  size_flag=0;
  int maxfsize;
  int outfilechanged=0;
  char maxfsizestr[100];
  char plotheaderstr[100];
  char plotcomsstr[100];
  char gnuplotcom[100];
  char makecom[50];
  char allpfilestr[100];
  char remsfile[50];
  char autodyn[100];
  char *fnm;
  int numf=0;
  char *tmpout;
  char *fullout;

  if(fileexists("userfiles/mathrems.dat"))
    strcpy(remsfile, "userfiles/mathrems.dat"); // safe
  else
    strcpy(remsfile, "userfiles/remsfile.dat"); // safe


  if(!getfileblk("sysfiles/sysstate/which_input_file", ifile, 100)){
    errdialog = PostErrorDialog(_w, label, "An error occurred: see terminal for details");
    return;
  }

  if(!getfileblk("sysfiles/sysstate/which_autoreginput_file", aifile, 100)){
    // default behaviour
    snprintf(aifile, 99, "/models/autoreginputs/pres.dat");
  }

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if(getoption("userfiles/gen_options", "MAX_OUTFILE_SIZE", maxfsizestr, 100)==0)
    maxfsize = atoi(maxfsizestr);
  else{
    errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  // get the gnuplot command
  if(getoption("userfiles/gen_options", "GNUPLOT", gnuplotcom, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }
  // get the file which will store the plot commands
  if(getoption("userfiles/gen_options", "PLOTCOMS", plotcomsstr, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }
  // get the file which stores the plot header information
  if(getoption("userfiles/gen_options", "PLOTHEADER", plotheaderstr, 100)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }


  // give some information on first GUI use

  if (fuse == 1){
    fuse=0; // so you don't have the same info each time

    answer=0;
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
    snprintf(systemcom1, 400, "Note that you might want to change the following global\noption before running simulations. The maximum file size\nallowed for an output file is set by default to\n100 MB. Long simulations may result in output\nfiles which exceed this size. If you have a large\nhard disk, you can change this option by editing\nthe file \"userfiles/gen_options\"\n\nTo continue with the simulation, press \"Continue\".\n");
    titlestring = XmStringCreateLocalized("First use: Info on output files");
    str = XmStringCreateLtoR (systemcom1, "MY_TAG");

    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedlg_callback, (XtPointer) this);
    XtAddCallback(confirmdialog, XmNcancelCallback, dialog_callback, (XtPointer) this);

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (answer == 0)
      XtAppProcessEvent (*app1, XtIMAll);

    if(answer==2) // cancel button pressed
      return;

  }
  // end of first use information

  autoreg = XmToggleButtonGetState((*tog)->baseWidget());
  smth = XmToggleButtonGetState(stog->baseWidget());
  contplot = XmToggleButtonGetState((*ctog)->baseWidget());

  inputfstr = XmTextGetString(*inputfile);
  modnamestr = XmTextGetString(*modname);
  pfilestr = XmTextGetString(*pfile);

  // confirm all the options first

  answer=0;

  confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);

  snprintf(systemcom1, 400, "You have chosen the following options:\n  1) model: %s\n  2) %s\n  3) input file: %s\n  4) %s\n  5) plot variable during the simulation: %s\n\nContinue with the simulation?", modnamestr, (autoreg==False) ? "dynamic simulation" : "static simulation (autoregulation curve)", inputfstr, (smth==False) ? "no smoothing" : "with smoothing", (contplot==False) ? "none" : XmTextGetString(*text));

  titlestring = XmStringCreateLocalized("Confirm the options");

  str = XmStringCreateLtoR (systemcom1, "MY_TAG");
  yes = XmStringCreateLocalized ("Continue");

  XtVaSetValues (confirmdialog,
		 XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		 XmNokLabelString, yes,
		 XmNmessageString, str,
		 XmNdialogTitle,     titlestring,
		 NULL);

  XmStringFree(yes);
  XmStringFree (str);
  XmStringFree(titlestring);

  XtAddCallback(confirmdialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(confirmdialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(confirmdialog, XmNcancelCallback, dialog_callback, (XtPointer) this); // the widget must only be destroyed when answer is set to 1 - don't call DestWidget

  XtManageChild (confirmdialog);

  XtPopup (XtParent (confirmdialog), XtGrabNone);

  // don't return control until we have an answer. 

  while (answer == 0)
    XtAppProcessEvent (*app1, XtIMAll);

  //  XtPopdown(XtParent(confirmdialog));

  if(answer==2){ // cancel button pressed
    XtFree(inputfstr); XtFree(modnamestr); XtFree(pfilestr);
    return;
  }

  // end of confirming options

  // parameter file:

  if(!getfileblk("sysfiles/sysstate/which_param_file", varname, 100)){
    errdialog = PostErrorDialog(_w, label, "An error occurred: see terminal for details");
    XtFree(inputfstr); XtFree(modnamestr); XtFree(pfilestr);
    return;
  }

  if(strcmp(pfilestr, varname)!=0){
    snprintf(systemcom1, 400, "echo %s > sysfiles/sysstate/which_param_file", pfilestr);
    system(systemcom1);
    telluser(label, "Parameter file reset");
  }


  // processing starts

  tmpout=XmTextGetString(*outputfile);
  fullout=lrtrim(tmpout);
  if(strcmp(fullout, tmpout)!=0){
    XmTextSetString(*outputfile, fullout);
    XmTextSetCursorPosition (*outputfile, strlen (fullout));
  }
  free(tmpout);

  numf=getnumblocks(fullout, " :,");
  fnm=getnthblk(fullout, 1);
  strncpy(outputfstr, fnm, 99);
  if(!startswith(fullout, "outdat/")){
    strcpy(outputfstr, "outdat/"); // safe
    strncat(outputfstr, fnm, 99-strlen(outputfstr));
    tmpout=enclose(fullout, "outdat/", "");
    XmTextSetString(*outputfile, tmpout);
    XmTextSetCursorPosition (*outputfile, strlen (tmpout));
    free(tmpout);
    outfilechanged=1; // outdat added
  }
  if(numf>1){
    if(outfilechanged==0)
      outfilechanged=2; // only first file being taken
    else
      outfilechanged=3; // outdat added and only first file being taken
  }

  free(fnm);

  // warn the user that the output file name has changed

  answer=0;
  if(outfilechanged!=0){

    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);

    if(outfilechanged==1)
      snprintf(systemcom1, 400, "The output file name that you chose was\nnot an allowed name. It has been changed\nto \"%s\". If you would like\nto continue with the simulation using the\nnew file name, click on \"Continue\", otherwise\ncancel and choose an allowed output file name.", outputfstr);
    else if (outfilechanged==2)
      snprintf(systemcom1, 400, "Multiple output files were displayed, but only the\nfirst will be used for output. If this is okay click\non \"Continue\", otherwise cancel.");
    else if (outfilechanged==3)
      snprintf(systemcom1, 400, "Multiple output files were displayed, but only the\nfirst will be used for output.\n\nThe first output file name was not an allowed name.\nIt has been changed to \"%s\". If you would like\nto continue with the simulation using the new file name,\nclick on \"Continue\", otherwise cancel and choose an\nallowed output file name.", outputfstr);

    titlestring = XmStringCreateLocalized("Confirm new output file name");

    str = XmStringCreateLtoR (systemcom1, "MY_TAG");
    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedlg_callback, (XtPointer) this);
    //  XtAddCallback(confirmdialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
    XtAddCallback(confirmdialog, XmNcancelCallback, changedlg_callback, (XtPointer) this);    // the widget must only be destroyed when answer is set to 1 - don't call DestWidget

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (answer == 0)
      XtAppProcessEvent (*app1, XtIMAll);

    //  XtPopdown(XtParent(confirmdialog));

    if(answer==2){ // cancel button pressed
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      return;
    }

  }
  // end of warning user that output file has changed


  if(autoreg==false){
    if (ofile == NULL || strcmp(ofile, outputfstr) !=0){
      snprintf(systemcom, 200, "echo %s > sysfiles/sysstate/which_output_file", outputfstr);
      ex_cmnd(systemcom, text_comout, toplevel);
      if(strlen(outputfstr) < 99)
	strcpy(ofile, outputfstr); // safe
      else
	fprintf(stderr, "WARNING in controls.cc: Output file name too long to copy to variable \"ofile\"\n");
    }
    snprintf(autodyn, 100, "%s", outputfstr); // no need
  }
  else{
    if (aofile == NULL || strcmp(aofile, outputfstr) !=0){
      snprintf(systemcom, 200, "echo %s_dyn > sysfiles/sysstate/which_output_file", outputfstr);
      ex_cmnd(systemcom, text_comout, toplevel);
      snprintf(systemcom, 200, "echo %s > sysfiles/sysstate/which_autoregoutput_file", outputfstr);
      ex_cmnd(systemcom, text_comout, toplevel);
      if(strlen(outputfstr) < 99)
	strcpy(aofile, outputfstr); // safe
      else
	fprintf(stderr, "WARNING in controls.cc: Output file name too long to copy to variable \"ofile\"\n");
    }
    snprintf(autodyn, 100, "%s_dyn", outputfstr);
  }


  inputfilereal = strdup(inputfstr);
  if(autoreg==false)
    snprintf(allpfilestr, 100, "%s.pvals", outputfstr);
  else
    snprintf(allpfilestr, 100, "%s.pvals", autodyn);
    


  // input file

  if(autoreg==false){
    if (strcmp(ifile, inputfilereal) !=0){
      snprintf(systemcom, 200, "echo %s > sysfiles/sysstate/which_input_file", inputfilereal);
      ex_cmnd(systemcom, text_comout, toplevel);
      strncpy(ifile, inputfilereal, 99);
      if(strlen(inputfilereal) > 99){
        fprintf(stderr, "WARNING: Input file name %s longer than 99 characters.\n", inputfilereal);
      }
    }
  }
  else{

  }

  // make the set of quantities to be output

  snprintf(systemcom, 200, "%s sysfiles/output.dat", makecom);
  ex_cmnd(systemcom, text_comout, toplevel);


  if(autoreg==false){
    if(smth==true){
      telluser(label, "smoothing...");
      snprintf(systemcom, 200, "%s src/lib/P_interp", makecom);
      //      system("make src/lib/P_interp");
      ex_cmnd(systemcom, text_comout, toplevel);
      snprintf(systemcom, 200, "./src/lib/P_interp -200 -1 %s models/inputs/__tempsmoothfile.dat", inputfilereal); // negative time-step means the algorithm chooses it
      telluser(label, systemcom);

      fprintf(stderr, "%s\n", systemcom);
      l = system(systemcom);
      if(l !=0){
	XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nSmoothing failed\n\n");
	XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	XmUpdateDisplay(*toplevel);
	errdialog = PostErrorDialog(_w, label, "Smoothing failed: see terminal for details...");
      }
      else{
	free(inputfilereal);
	inputfilereal = strdup("models/inputs/__tempsmoothfile.dat");
      }
    }

    // has the input file changed
    if (strcmp(ifile, inputfilereal) !=0){
      snprintf(systemcom, 200, "echo %s > sysfiles/sysstate/which_input_file", inputfilereal);
      ex_cmnd(systemcom, text_comout, toplevel);
    }

    // make

    telluser(label, "making...");
    snprintf(systemcom, 200, "%s %s", makecom, modnamestr);
    fprintf(stderr, "%s\n", systemcom);
    l = ex_cmnd(systemcom, text_comout, toplevel);
    if(l !=0){
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nChosen model type does not seem to exist... now exiting\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      errdialog = PostErrorDialog(_w, label, "Chosen model type does not seem to exist...");
      free(inputfilereal);
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      return;
    }
//    l = system(systemcom);

    // in case this takes a while, put on a timeoutcursor

    TOC(True, toplevel);

    //make
    l = ex_cmnd(makecom, text_comout, toplevel);
    if(l !=0){
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProblem with making... now exiting (see terminal for details)\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      errdialog = PostLongDialog(_w, "ERROR", "Problem with making (see terminal for details)");
      TOC(False, toplevel);
      free(inputfilereal);
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      return;
    }

    //
    // check the input file
    //

    snprintf(systemcom, 200, "%s src/lib/P_checkinput", makecom);
    ex_cmnd(systemcom, text_comout, toplevel);
    snprintf(systemcom, 200, "./src/lib/P_checkinput %s", inputfilereal);
    l=system(systemcom);
    if(l!=0){

      if(l/256==1){ // some input parameters not found

        // warn the user that the input file has unrecognised parameters

	answer=0;
	confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
        snprintf(systemcom1, 400, "The input file has some unrecognised parameters\n(see terminal for details.) Continue with the\nsimulation?");
        titlestring = XmStringCreateLocalized("Some input parameters not found");

        str = XmStringCreateLtoR (systemcom1, "MY_TAG");
        yes = XmStringCreateLocalized ("Continue");

        XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

        XmStringFree(yes);
        XmStringFree (str);
        XmStringFree(titlestring);

        XtAddCallback(confirmdialog, XmNokCallback, changedlg_callback, (XtPointer) this);
        XtAddCallback(confirmdialog, XmNcancelCallback, changedlg_callback, (XtPointer) this);    // the widget must only be destroyed when answer is set to 1 - don't call DestWidget

        XtManageChild (confirmdialog);
        XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

        XtPopup (XtParent (confirmdialog), XtGrabNone);

        // don't return control until we have an answer. 

        while (answer == 0)
          XtAppProcessEvent (*app1, XtIMAll);

        //  XtPopdown(XtParent(confirmdialog));

        if(answer==2){ // cancel button pressed
	  TOC(False, toplevel);
          XtFree(inputfstr);
          XtFree(modnamestr);
          XtFree(pfilestr);
	  free(inputfilereal);
          return;
	}
        // end of warning user that output file has changed
      }
      else{ // some other error
        XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProblem with input file... now exiting (see terminal for details)\n\n");
        XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
        XmUpdateDisplay(*toplevel);
        errdialog = PostLongDialog(_w, "ERROR", "Problem with input file (see terminal for details)");
        TOC(False, toplevel);
        XtFree(inputfstr);
        XtFree(modnamestr);
        XtFree(pfilestr);
        free(inputfilereal);
        return;
      }
    }

    if(contplot==true){  // continuous plotting     
      strncpy(buf, XmTextGetString(*text), 999);
      l=printexprdeps("sysfiles/_tmp_contvars.dat", buf, "sysfiles/alltempstrs.dat", "sysfiles/vararray.dat", "sysfiles/params.dat", "userfiles/remsfile.dat"); 
      if(l==0)
        snprintf(systemcom1, 400, "src/lib/P_run %s %s %s sysfiles/_tmp_contvars.dat > %s &", inputfilereal, pfilestr, outputfstr, outputfstr);  // background process
      else{
	fprintf(stderr, "Continuous plotting failed.\n");
	snprintf(systemcom1, 400, "src/lib/P_run %s %s %s > %s &", inputfilereal, pfilestr, outputfstr, outputfstr);
      }
    }
    else
      snprintf(systemcom1, 400, "src/lib/P_run %s %s %s > %s &", inputfilereal, pfilestr, outputfstr, outputfstr);  // background process

    TOC(False, toplevel);
    telluser(label, systemcom);

    // make -q returns non-zero if the targets need to be made. This seems to work on 
    // all versions of make we've encountered

    snprintf(systemcom, 200, "%s -q dat1", makecom);
    needtomake = system(systemcom);
    fprintf(stderr, "needtomake = %d\n", needtomake);
    if(needtomake!=0){

      TimeoutCursors (True, True);

      system(systemcom1);
      system("echo 1 > sysfiles/sysstate/braincircrunning");

      sleep(1); // for safety

      fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
      if((fgets(buf, sizeof buf, fd))!=NULL){
	if(strstr(buf, "END") != NULL){
	  if(strstr(buf, "PROBLEM") !=NULL){
	    snprintf(systemcom, 200, "\nProgram terminated abnormally. See terminal for details.\n\n");
	    errdialog = PostLongDialog(_w, "ERROR", "Program terminated abnormally. See terminal for details.");
	  }
	  else{
	    snprintf(systemcom, 200, "\nProgram finished normally\n\n");
	  }
	  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
	  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	  XmUpdateDisplay(*toplevel);
	  fclose(fd);
	  system("touch dat1");
	  fprintf(stderr, "touch dat1\n");
	  system("echo 0 > sysfiles/sysstate/braincircrunning");
	  XtFree(inputfstr);
	  XtFree(modnamestr);
	  XtFree(pfilestr);
	  free(inputfilereal);
	  TimeoutCursors (False, False);
	  return;
	}
	else
	  runpid=(atoi(buf));
      }

      fprintf(stderr, "run process number %d\n", runpid);
      fclose(fd);

      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nExecuting - please be patient...\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      stat(outputfstr, &fst);
      sizeold=fst.st_size;q=0;q1=1;q2=0;
      for(;;){
        if(stat(outputfstr, &fst)==0){
	  // checking output file length not getting too long
	  if((size_flag == 0) && (fst.st_size>maxfsize/4)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (maxfsize/4));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 25 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=1;
	  }
	  else if((size_flag == 1) && (fst.st_size>maxfsize/2)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (maxfsize/2));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 50 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=2;
	  }
	  else if((size_flag == 2) && (fst.st_size>3*maxfsize/4)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (3*maxfsize/4));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 75 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=3;
	  }
	  else if((size_flag == 3) && (fst.st_size>maxfsize)){
	    fprintf(stderr, "\nERROR: Output file size is more than %d bytes - the maximum you have allowed in the file \"userfiles/gen_options\"\n", (maxfsize));
	    fprintf(stderr, "        ...Trying to interrupt\n");

	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nERROR: Output file size more than maximum given in \"userfiles/gen_options\"\n        ...Trying to interrupt...\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=4;

	    fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	    if((fgets(buf, sizeof buf, fd))!=NULL){
	      if(strstr(buf, "END") != NULL){
		XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProgram finished anyway.\n\nIf you want to allow a bigger file size, then alter the file \"userfiles/gen_options\"\n");
		XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		XmUpdateDisplay(*toplevel);
	      }
	      else{
		runpid=(atoi(buf));
		fprintf(stderr, "killing process number %d\n", runpid);
		snprintf(systemcom1, 400, "kill %d", runpid);
		l=system(systemcom1);
		if(l!=-1){
		  kld=1;
		  fclose(fd);
		  fd = fopen("sysfiles/sysstate/temp_pid_file", "w");
		  fprintf(fd, "END PROBLEM\n"); // program has ended (abnormally)
		  fclose(fd);
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProgram was killed because the output file became too long. If you want to allow a bigger file size, then alter the file \"userfiles/gen_options\"\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		  break; // exit the loop because process successfully killed
		}
		else{
		  fprintf(stderr, "\t...kill unsuccessful\n");
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...kill unsuccessful...\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		}
	      }
	    }
	    else{
	      fprintf(stderr, "\t...can't find process to kill\n");
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...can't find process to kill\n");
	      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	      XmUpdateDisplay(*toplevel);
	    }
	    fclose(fd);


	  }
	  // end of checking on output file length


	  //	fprintf(stderr, "%d***", fst.st_size);
	  if(fst.st_size>sizeold){
	    q=0; // measures number of iterations at a single file-size

	    if(contplot==true){  // continuous plotting
	      q2++; // how many times has the file increased in size?
              if(q2==1){ // only set this off once
		strncpy(buf, XmTextGetString(*text), 999);
		l = plotall("", buf, "outdat/__tmp_contplot.dat", "", 1, remsfile);
	      }
	    }

	    if(q1%50==0){
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "*\n");
	      XmUpdateDisplay(*toplevel);
	    }
	    else
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "*");
	    q1++;
	  }
	  else{
	    q++;
	    if(q1%50==0)
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), ".\n");
	    else
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), ".");
	    q1++;
	  }
	  sizeold = fst.st_size;

        }

        if(CheckForInterrupt()){
	  interruptnum++;
	  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	  if(interruptnum==1){ // only do this once
	    fprintf(stderr, "        ...Trying to interrupt\n");
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "        ...Trying to interrupt...\n");

	    fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	    if((fgets(buf, sizeof buf, fd))!=NULL){
	      if(strstr(buf, "END") != NULL){
		if(strstr(buf, "PROBLEM") !=NULL){
		  snprintf(systemcom, 200, "\nProgram terminated abnormally. See terminal for details.\n\n");
		  errdialog = PostErrorDialog(_w, label, "Program terminated abnormally. See terminal for details.");
		}
		else{
		  snprintf(systemcom, 200, "\nProgram finished normally\n\n");
		}
		XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
		XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		XmUpdateDisplay(*toplevel);
	      }
	      else{
		runpid=(atoi(buf));
		fprintf(stderr, "killing process number %d\n", runpid);
		snprintf(systemcom1, 400, "kill %d", runpid);
		l=system(systemcom1);
		if(l!=-1){
		  kld=1;
		  fclose(fd);
		  fd = fopen("sysfiles/sysstate/temp_pid_file", "w");
		  fprintf(fd, "END PROBLEM\n"); // program has ended (abnormally)
		  fclose(fd);
		  break; // exit the loop because process successfully killed
		}
		else{
		  fprintf(stderr, "\t...kill unsuccessful\n");
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...kill unsuccessful...\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		}
	      }
	    }
	    else{
	      fprintf(stderr, "\t...can't find process to kill\n");
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...can't find process to kill\n");
	      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	      XmUpdateDisplay(*toplevel);
	    }
	    fclose(fd);

	  }

        }
	//	fprintf(stderr, "got here.\n");
	fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	if((fgets(buf, sizeof buf, fd))!=NULL){
	  if(strstr(buf, "END") != NULL){
	    fclose(fd);
	    break; // exit the loop because process ended
	  }
	}
        fclose(fd);

        XmUpdateDisplay(*toplevel);
        sleep(1);

      }

      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));

    // make sure we really have finished the simulation

      system("echo 0 > sysfiles/sysstate/braincircrunning");
      if(kld==0){
        system("touch dat1");
        XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\n\nExecution complete...\n\n");
	XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	XmUpdateDisplay(*toplevel);
        telluser(label, "Execution complete...");
      }
      else{
        XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\n\nProcess killed...\n\n");
        telluser(label, "Process killed...");
      }

      TimeoutCursors (False, False);

    }
    else{
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "Everything is up to date\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
    }
    
  }

  //
  //
  //    AUTOREGULATION CURVE
  //
  //

  else{ // autoregulation curve
    if (aifile == NULL || strcmp(aifile, inputfilereal) !=0){
      snprintf(systemcom, 200, "echo %s > sysfiles/sysstate/which_autoreginput_file", inputfilereal);
      ex_cmnd(systemcom, text_comout, toplevel);
    }

    telluser(label, "making...");
    snprintf(systemcom, 200, "%s src/lib/P_autoreg", makecom);
    l = ex_cmnd(systemcom, text_comout, toplevel);
    if(l!=0){
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProblem with making... now exiting (see terminal for details)\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      errdialog = PostErrorDialog(_w, label, "Problem with making (see terminal for details)");
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      free(inputfilereal);
      return;
    }

    snprintf(systemcom, 200, "%s sysfiles/autoregout.dat", makecom);
    l = ex_cmnd(systemcom, text_comout, toplevel);
    if(l!=0){
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProblem with making... now exiting (see terminal for details)\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      errdialog = PostErrorDialog(_w, label, "Problem with making (see terminal for details)");
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      free(inputfilereal);
      return;
    }

    snprintf(systemcom, 200, "%s src/lib/P_checkinput", makecom);
    ex_cmnd(systemcom, text_comout, toplevel);
    snprintf(systemcom, 200, "./src/lib/P_checkinput %s", inputfilereal);
    l=system(systemcom);
    if(l!=0){
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProblem with input file... now exiting (see terminal for details)\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      errdialog = PostErrorDialog(_w, label, "Problem with input file (see terminal for details)");
      XtFree(inputfstr);
      XtFree(modnamestr);
      XtFree(pfilestr);
      free(inputfilereal);
      return;
    }

    if(contplot==true){  // continuous plotting     
      strncpy(buf, XmTextGetString(*text), 999);
      printexprdeps("sysfiles/_tmp_contvars.dat", buf, "sysfiles/alltempstrs.dat", "sysfiles/vararray.dat", "sysfiles/params.dat", "userfiles/remsfile.dat"); 
      snprintf(systemcom1, 400, "src/lib/P_autoreg %s %s %s sysfiles/_tmp_contvars.dat > %s &", inputfilereal, pfilestr, outputfstr, autodyn); // background process, but periodically check its status
    }
    else{
      snprintf(systemcom1, 400, "src/lib/P_autoreg %s %s %s > %s &", inputfilereal, pfilestr, outputfstr, autodyn); // background process, but periodically check its status
    }
    telluser(label, systemcom);

    snprintf(systemcom, 200, "%s -q autoregplot1", makecom);
    needtomake=system(systemcom);
    fprintf(stderr, "needtomake = %d\n", needtomake);
    if(needtomake!=0){

      TimeoutCursors (True, True);

      system(systemcom1);
      system("echo 1 > sysfiles/sysstate/braincircrunning");

      sleep(1); // for safety

//       system("ps -o pid= -C autoreg > sysfiles/sysstate/temp_pid_file");
//       sleep(1); // for safety

      fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
      if((fgets(buf, sizeof buf, fd))!=NULL){
	if(strstr(buf, "END") != NULL){
	  if(strstr(buf, "PROBLEM") !=NULL){
	    snprintf(systemcom, 200, "\nProgram terminated abnormally. See terminal for details.\n\n");
	    errdialog = PostErrorDialog(_w, label, "Program terminated abnormally. See terminal for details.");
	  }
	  else{
	    snprintf(systemcom, 200, "\nProgram finished normally.\n\n");
	  }
	  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
	  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	  XmUpdateDisplay(*toplevel);
	  fclose(fd);
	  TimeoutCursors (False, False);
	  system("touch autoregplot1");
	  system("echo 0 > sysfiles/sysstate/braincircrunning");
	  XtFree(inputfstr);
	  XtFree(modnamestr);
	  XtFree(pfilestr);
	  free(inputfilereal);
	  return;
	}
	else
	  runpid=(atoi(buf));
      }

      fprintf(stderr, "run process number %d\n", runpid);
      fclose(fd);

      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nExecuting - please be patient...\n\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
      stat(autodyn, &fst);
      sizeold=fst.st_size;q=0;q1=1;q2=0;
      for(;;){
        if(stat(autodyn, &fst)==0){

	  // checking output file length not getting too long
	  if((size_flag == 0) && (fst.st_size>maxfsize/4)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (maxfsize/4));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 25 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=1;
	  }
	  else if((size_flag == 1) && (fst.st_size>maxfsize/2)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (maxfsize/2));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 50 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=2;
	  }
	  else if((size_flag == 2) && (fst.st_size>3*maxfsize/4)){
	    fprintf(stderr, "\nWARNING: Output file size is more than %d bytes\n", (3*maxfsize/4));
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: Output file size more than 75 per cent of maximum given in \"userfiles/gen_options\"\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=3;
	  }
	  else if((size_flag == 3) && (fst.st_size>maxfsize)){
	    fprintf(stderr, "\nERROR: Output file size is more than %d bytes - the maximum you have allowed in the file \"userfiles/gen_options\"\n", (maxfsize));
	    fprintf(stderr, "        ...Trying to interrupt\n");

	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nERROR: Output file size more than maximum given in \"userfiles/gen_options\"\n        ...Trying to interrupt...\n");
	    XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	    XmUpdateDisplay(*toplevel);
	    size_flag=4;

	    fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	    if((fgets(buf, sizeof buf, fd))!=NULL){
	      if(strstr(buf, "END") != NULL){
		XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProgram finished anyway.\n\nIf you want to allow a bigger file size, then alter the file \"userfiles/gen_options\"\n");
		XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		XmUpdateDisplay(*toplevel);
	      }
	      else{
		runpid=(atoi(buf));
		fprintf(stderr, "killing process number %d\n", runpid);
		snprintf(systemcom1, 400, "kill %d", runpid);
		l=system(systemcom1);
		if(l!=-1){
		  kld=1;
		  fclose(fd);
		  fd = fopen("sysfiles/sysstate/temp_pid_file", "w");
		  fprintf(fd, "END PROBLEM\n"); // program has ended (abnormally)
		  fclose(fd);
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nProgram was killed because the output file became too long. If you want to allow a bigger file size, then alter MAX_OUTFILE_SIZE in the file \"userfiles/gen_options\"\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		  break; // exit the loop because process successfully killed
		}
		else{
		  fprintf(stderr, "\t...kill unsuccessful\n");
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...kill unsuccessful...\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		}
	      }
	    }
	    else{
	      fprintf(stderr, "\t...can't find process to kill\n");
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...can't find process to kill\n");
	      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	      XmUpdateDisplay(*toplevel);
	    }
	    fclose(fd);


	  }
	  // end of checking on output file length


	  if(fst.st_size>sizeold){
	    q=0;

	    if(contplot==true){ // continuous plotting
	      q2++; // how many times has the file increased in size?
              if(q2==1){ // only set this off once
		strncpy(buf, XmTextGetString(*text), 199);
		l = plotall("", buf, "outdat/__tmp_contplot.dat", "", 1, remsfile);
 	      }
	    }

	    if(q1%50==0){
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "*\n");
	      XmUpdateDisplay(*toplevel);
	    }
	    else
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "*");
	    q1++;
	  }
	  else{
	    q++;
	    if (q>30)
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\nWARNING: A long period of inactivity - perhaps the system has crashed.\n");
	    if(q1%50==0)
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), ".\n");
	    else
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), ".");
	    q1++;
	  }
	  sizeold = fst.st_size;
    
        }

        if(CheckForInterrupt()){
	  interruptnum++;
 	  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	  if(interruptnum==1){ // only do this once
	    fprintf(stderr, "\n...Trying to interrupt\n");
	    XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "        ...Trying to interrupt...\n");
	    fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	    if((fgets(buf, sizeof buf, fd))!=NULL){
	      if(strstr(buf, "END") != NULL){
		if(strstr(buf, "PROBLEM") !=NULL){
		  snprintf(systemcom, 200, "\nProgram terminated abnormally. See terminal for details.\n\n");
		  errdialog = PostErrorDialog(_w, label, "Program terminated abnormally. See terminal for details.");
		}
		else{
		  snprintf(systemcom, 200, "\nProgram finished normally.\n\n");
		}
		XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), systemcom);
		XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		XmUpdateDisplay(*toplevel);
	      }
	      else{
		runpid=(atoi(buf));
		fprintf(stderr, "\n...killing process number %d\n", runpid);
		snprintf(systemcom1, 400, "kill %d", runpid);
		l=system(systemcom1);
		if(l!=-1){
		  kld=1;
		  fclose(fd);
		  fd = fopen("sysfiles/sysstate/temp_pid_file", "w");
		  fprintf(fd, "END\n"); // program has ended (abnormally)
		  fclose(fd);
		  break; // exit the loop because process successfully killed
		}
		else{
		  fprintf(stderr, "\t...kill unsuccessful\n");
		  XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...kill unsuccessful...\n");
		  XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
		  XmUpdateDisplay(*toplevel);
		}
	      }
	    }
	    else{
	      fprintf(stderr, "\t...can't find process to kill\n");
	      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\t...can't find process to kill\n");
	      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
	      XmUpdateDisplay(*toplevel);
	    }
	    fclose(fd);

	  }

        }

	//        system("ps -o pid= -C autoreg > sysfiles/sysstate/temp_pid_file");

	//
	// Has the program terminated?
	//

	fd = fopen("sysfiles/sysstate/temp_pid_file", "r");
	if((fgets(buf, sizeof buf, fd))!=NULL){
	  if(strstr(buf, "END") != NULL){
	    fclose(fd);
	    break; // exit the loop because process ended
	  }
	}
        fclose(fd);

        XmUpdateDisplay(*toplevel);
        sleep(1);

      }

      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));


      // make sure we really have finished the simulation

      system("echo 0 > sysfiles/sysstate/braincircrunning");
      if(kld==0){
        system("touch autoregplot1");
        XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\n\nExecution complete...\n\n");
        telluser(label, "Execution complete...");
      }
      else{
        XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "\n\nProcess killed...\n\n");
        telluser(label, "Process killed...");
      }

      TimeoutCursors (False, False);
    }
    else{
      XmTextInsert (*text_comout, XmTextGetLastPosition(*text_comout), "Everything is up to date\n");
      XmTextShowPosition (*text_comout, XmTextGetLastPosition(*text_comout));
      XmUpdateDisplay(*toplevel);
    }

  }


  //  TimeoutCursors (False, False);

  free(inputfilereal);
  XtFree(inputfstr);
  XtFree(modnamestr);
  XtFree(pfilestr);
}

void makerunControl::makerunCallback(Widget, XtPointer client_data, XtPointer call_data){
  makerunControl *obj = (makerunControl *) client_data;
  obj->makerun();
}

void makerunControl::stopCallback(Widget w1, XtPointer client_data, XtPointer call_data){
  makerunControl *obj = (makerunControl *) client_data;
  obj->stop();
}


void makerunControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  makerunControl *obj = (makerunControl *) client_data;
  obj->help();
};

void makerunControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "makerun_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on running simulations", vardesc);

}



//constructor

makerunControl::makerunControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *modelname, Widget *infile, Widget *outfile, Widget *paramfile, char *name, char display[], messagelabel **label1, plotToggle **tog1, smoothToggle *tog2, contplotToggle **tog3, Widget *text1, XtAppContext *app, int *firstuse): BasicComponent (name){
  label = label1; // set pointer to the correct label
  tog = tog1;
  stog = tog2;
  ctog = tog3;
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  outputfile = outfile;
  modname = modelname;
  pfile = paramfile;
  // initialise the strings 
  ifile[0] = 0;
  aifile[0] = 0;
  ofile[0] = 0;
  aofile[0] = 0;
  toplevel = toplevel1;
  app1=app;
  text_comout = text_comout1;
  text = text1; // which variable to output
  // Create the widget
  fuse = *firstuse; // is it the first time the gui is being used?

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 8,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&makerunControl::makerunCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&makerunControl::helpCallback,
		(XtPointer) this);
} // end of constructor






//
//
// editfileControl
//
//

void editfileControl::editfile()
{
  bool autoreg;
  char *inputfstr;
  char systemcom[200];
  char editorstr[50];
  Widget Errdialog;
  char *inputfilereal;


  autoreg = XmToggleButtonGetState((*tog)->baseWidget());
  inputfstr = XmTextGetString(*inputfile);

  if(autoreg==false){
    if(!startswith(inputfstr, "models/inputs/"))
      inputfilereal = enclose(inputfstr, "models/inputs/", "");
    else
      inputfilereal = strdup(inputfstr);
  }
  else{
    if(!startswith(inputfstr, "models/autoreginputs/"))
      inputfilereal = enclose(inputfstr, "models/autoreginputs/", "");
    else
      inputfilereal = strdup(inputfstr);
  }

  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    XtFree(inputfstr);
    free(inputfilereal);
    return;
  }

  snprintf(systemcom, 200, "%s %s &", editorstr, inputfilereal);
  telluser(label, systemcom);

  system(systemcom);

  XtFree(inputfstr);
  free(inputfilereal);
}

void editfileControl::editfileCallback(Widget, XtPointer client_data, XtPointer){
  editfileControl *obj = (editfileControl *) client_data;
  obj->editfile();
};

void editfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  editfileControl *obj = (editfileControl *) client_data;
  obj->help();
};

void editfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "edit_inputfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on editing input files", vardesc);

}


//constructor

editfileControl::editfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, plotToggle **tog1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  tog = tog1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&editfileControl::editfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&editfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor

//
//
// cpinputfileControl
//
//

void cpinputfileControl::cpinputfile()
{
  Widget dialog;
  XmString t, tempstr;
  FILE *fd;
  char systemcom[200];
  Widget errdialog;
  Arg args[5];
  int n=0;
  char *fnamefull, *fname;

  fnamefull=XmTextGetString(*text_filename);

  // get the filename without the directory name
  fname = getfname(fnamefull);

  // check the input file exists
  fd = fopen(fnamefull, "r");
  if(fd == NULL){
    snprintf(systemcom, 200, "file %s can't be opened.", fnamefull);
    errdialog = PostErrorDialog(_w, label, systemcom);
    free(fnamefull);free(fname);
    return;
  }
  fclose(fd);


  snprintf(systemcom, 200, "Copy file %s to:", fname);
  free(fnamefull);free(fname);
  t = XmStringCreateLocalized (systemcom);

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Copy an existing input file");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void cpinputfileControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[200];
  char *value;
  char vardesc[1000];
  int l;
  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  char *tmp;
  char *oldfname, *newfname, *dnamefull;
  char *dname;

  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "cpinputfile_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(widget, "Help on copying input files", vardesc);
      return;
    default:
      break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }
  if(!endswith(value, ".dat")){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "Input files must end in \".dat\"\nTry again.");
    XtFree(value);
    return;
  }

  // current directory

  tmp=XmTextGetString(*text_filename);
  oldfname=lrtrim(tmp);free(tmp);
  dnamefull=getdnamef(oldfname); // with terminal '/'

  // construct the new filename

  if(!startswith(value, dnamefull))
    newfname=enclose(value, dnamefull, "");
  free(dnamefull);

  dname=getdname(newfname); // not redundant!
  if(!fileexists(dname)){
    snprintf(systemcom, 200, "Directory %s does not exist\nTry again.", dname);
    dialog = PostLongDialog(_w, "Problem with what you've typed", systemcom);
    free(oldfname);free(dname);free(newfname); XtFree(value);
    return;
  }
  free(dname);

  // check if file exists
  if(fileexists(newfname)){
    snprintf(systemcom, 200, "File %s already exists", value);
    dialog = PostErrorDialog(_w, label, systemcom);
    free(oldfname);free(newfname); XtFree(value);
    return;
  }

  // Try to copy

  if(cpfile(newfname, oldfname)<0){
    dialog = PostErrorDialog(_w, label, "Error in copying - see terminal for details");
    free(oldfname);free(newfname);
    XtFree(value);
    return;
  }

  snprintf(systemcom, 200, "File \"%s\" created.", value);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  free(oldfname);free(newfname); XtFree(value);
  XtDestroyWidget(widget);

}

void cpinputfileControl::cpinputfileCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpinputfileControl *obj = (cpinputfileControl *) client_data;
  obj->cpinputfile();
};

void cpinputfileControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpinputfileControl *obj = (cpinputfileControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void cpinputfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  cpinputfileControl *obj = (cpinputfileControl *) client_data;
  obj->help();
};

void cpinputfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "copy_inputfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on copying input files", vardesc);

}



//constructor

cpinputfileControl::cpinputfileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&cpinputfileControl::cpinputfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&cpinputfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// cpparamfileControl
//
//

void cpparamfileControl::cpparamfile()
{
  Widget dialog;
  XmString t, tempstr;
  FILE *fd;
  char systemcom[200];
  Widget errdialog;
  Arg args[5];
  int n=0;
  char *fnamefull, *fname;

  fnamefull=XmTextGetString(*text_filename);

  // get the filename without the directory name
  fname = getfname(fnamefull);

  // check the input file exists
  fd = fopen(fnamefull, "r");
  if(fd == NULL){
    snprintf(systemcom, 200, "file %s can't be opened.", fnamefull);
    errdialog = PostErrorDialog(_w, label, systemcom);
    free(fnamefull);free(fname);
    return;
  }
  fclose(fd);


  snprintf(systemcom, 200, "Copy file %s to:", fname);
  free(fnamefull);free(fname);
  t = XmStringCreateLocalized (systemcom);

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Copy an existing input file");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void cpparamfileControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[200];
  char *value;
  char vardesc[1000];
  int l;
  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  char *tmp;
  char *oldfname, *newfname, *dnamefull;
  char *dname;

  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "cpparamfile_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(widget, "Help on copying parameter files", vardesc);
      return;
    default:
      break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }
  if(!endswith(value, ".dat")){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "Parameter files must end in \".dat\"\nTry again.");
    XtFree(value);
    return;
  }

  // current directory

  tmp=XmTextGetString(*text_filename);
  oldfname=lrtrim(tmp);free(tmp);
  dnamefull=getdnamef(oldfname); // with terminal '/'

  // construct the new filename

  if(!startswith(value, dnamefull))
    newfname=enclose(value, dnamefull, "");
  free(dnamefull);

  dname=getdname(newfname); // not redundant!
  if(!fileexists(dname)){
    snprintf(systemcom, 200, "Directory %s does not exist\nTry again.", dname);
    dialog = PostLongDialog(_w, "Problem with what you've typed", systemcom);
    free(oldfname);free(dname);free(newfname); XtFree(value);
    return;
  }
  free(dname);

  // check if file exists
  if(fileexists(newfname)){
    snprintf(systemcom, 200, "File %s already exists", value);
    dialog = PostErrorDialog(_w, label, systemcom);
    free(oldfname);free(newfname); XtFree(value);
    return;
  }

  // Try to copy

  if(cpfile(newfname, oldfname)<0){
    dialog = PostErrorDialog(_w, label, "Error in copying - see terminal for details");
    free(oldfname);free(newfname);
    XtFree(value);
    return;
  }

  snprintf(systemcom, 200, "File \"%s\" created.", value);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  free(oldfname);free(newfname); XtFree(value);
  XtDestroyWidget(widget);

}

void cpparamfileControl::cpparamfileCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpparamfileControl *obj = (cpparamfileControl *) client_data;
  obj->cpparamfile();
};

void cpparamfileControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpparamfileControl *obj = (cpparamfileControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void cpparamfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  cpparamfileControl *obj = (cpparamfileControl *) client_data;
  obj->help();
};

void cpparamfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "copy_paramfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on copying parameter files", vardesc);

}



//constructor

cpparamfileControl::cpparamfileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&cpparamfileControl::cpparamfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&cpparamfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor









//
//
// editfilesimpControl
//
//

void editfilesimpControl::editfilesimp()
{
  char systemcom[400];
  char *inputfstr = XmTextGetString(*inputfile);
  char editorstr[50];
  Widget Errdialog;
  Widget confirmdialog;
  int numf;
  char *outfstr;
  char *fnm;
  int outfilechanged=0;
  XmString str, yes, titlestring;

  numf=getnumblocks(inputfstr, " :,");
  fnm=getnthblk(inputfstr, 1);
  outfstr=strdup(fnm);
  if(!fileexists(fnm) && !startswith(fnm, "outdat/")){
    free(fnm);
    fnm=enclose(outfstr, "outdat/", "");
    outfilechanged=1; // outdat added
  }
  if(numf>1){
    if(outfilechanged==0)
      outfilechanged=2; // only first file being taken
    else
      outfilechanged=3; // outdat added and only first file being taken
  }
  free(outfstr);
  XtFree(inputfstr);

  if(!fileexists(fnm)){
    Errdialog = PostErrorDialog(_w, label, "No file found.");
    free(fnm);
    return;
  }


  // warn the user that the output file name has changed

  ANS=0;
  if(outfilechanged!=0){
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);

    if(outfilechanged==1)
      snprintf(systemcom, 400, "The output file was assumed to be in the outdat directory, i.e.\nIt has been changed to \"%s\". To continue, click\non \"Continue\", otherwise cancel and choose another file name.", fnm);
    else if (outfilechanged==2)
      snprintf(systemcom, 400, "Multiple output files were displayed, but the\nfirst is being opened. If this is okay click\non \"Continue\", otherwise cancel.");
    else if (outfilechanged==3)
      snprintf(systemcom, 400, "Multiple output files were displayed, but only the first is being opened.\n\nThis file was assumed to be in the outdat directory, i.e.\nIt has been changed to \"%s\". To continue, click\non \"Continue\", otherwise cancel and choose another file name.", fnm);

    titlestring = XmStringCreateLocalized("Confirm output file name");

    str = XmStringCreateLtoR (systemcom, "MY_TAG");
    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedialog, (XtPointer) this);
    //  XtAddCallback(confirmdialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
    XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, (XtPointer) this);    // the widget must only be destroyed when answer is set to 1 - don't call DestWidget

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (ANS==0)
      XtAppProcessEvent (*app1, XtIMAll);

    //  XtPopdown(XtParent(confirmdialog));

    if(ANS==2){ // cancel button pressed
      free(fnm);
      return;
    }

  }


  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    free(fnm);
    return;
  }


  snprintf(systemcom, 400, "%s %s &", editorstr, fnm);
  telluser(label, systemcom);
  if(system(systemcom)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with opening file.");
    free(fnm);
    return;
  }

  free(fnm);
}

void editfilesimpControl::editfilesimpCallback(Widget, XtPointer client_data, XtPointer){
  editfilesimpControl *obj = (editfilesimpControl *) client_data;
  obj->editfilesimp();
};

void editfilesimpControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  editfilesimpControl *obj = (editfilesimpControl *) client_data;
  obj->help();
};

void editfilesimpControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "view_outputfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on viewing output files", vardesc);

}


//constructor

editfilesimpControl::editfilesimpControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&editfilesimpControl::editfilesimpCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&editfilesimpControl::helpCallback,
		(XtPointer) this);

} // end of constructor


//
//
// editheadfileControl
//
//

void editheadfileControl::editheadfile()
{
  char systemcom[200];
  char *inputfstr;
  char editorstr[50];
  Widget Errdialog;

  inputfstr=strdup("plotout/plotheader");
  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    free(inputfstr);
    return;
  }
  snprintf(systemcom, 200, "%s %s &", editorstr, inputfstr);
  telluser(label, systemcom);
  system(systemcom);
  free(inputfstr);

}

void editheadfileControl::editheadfileCallback(Widget, XtPointer client_data, XtPointer){
  editheadfileControl *obj = (editheadfileControl *) client_data;
  obj->editheadfile();
};

void editheadfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  editheadfileControl *obj = (editheadfileControl *) client_data;
  obj->help();
};

void editheadfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "edit_headfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on viewing output files", vardesc);

}


//constructor

editheadfileControl::editheadfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&editheadfileControl::editheadfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&editheadfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor



//
//
// viewpfileControl
//
//

void viewpfileControl::viewpfile()
{
  char systemcom[200];
  char *inputfstr = XmTextGetString(*inputfile);
  char editorstr[50];
  int l=0;
  Widget Errdialog;

  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  snprintf(systemcom, 200, "%s %s &", editorstr, inputfstr);
  telluser(label, systemcom);
  l=system(systemcom);
  if(l!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR in opening file: see terminal for details");
    return;
  }

  XtFree(inputfstr);

}

void viewpfileControl::viewpfileCallback(Widget, XtPointer client_data, XtPointer){
  viewpfileControl *obj = (viewpfileControl *) client_data;
  obj->viewpfile();
};

void viewpfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  viewpfileControl *obj = (viewpfileControl *) client_data;
  obj->help();
};

void viewpfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "view_pfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on viewing the parameter file", vardesc);

}


//constructor

viewpfileControl::viewpfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&viewpfileControl::viewpfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&viewpfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor



//
//
// searchvarControl
//
//


void searchvarControl::TimeoutCursors(Boolean on) // the simple version
{
  static int locked; // initialised to zero by default - internal static variables are private to a function, but are not reinitialised each time the function is called. 
  static Cursor cursor;
  XSetWindowAttributes attrs;
  Display *dpy = XtDisplay (*toplevel);
  XEvent event;

  if (on)
    locked++;
  else
    locked--;
  if(locked > 1 || locked ==1 && on == 0)
    return;

  //  fprintf(stderr, "locked = %d\n", locked);

  stopped=false;

  if(!cursor){
    cursor = XCreateFontCursor(dpy, XC_watch);
    //    fprintf(stderr, "cursor created\n");
  }

  attrs.cursor = on ? cursor : None;

  XChangeWindowAttributes (dpy, XtWindow (*toplevel), CWCursor, &attrs);

  XFlush (dpy);

  if (on){}
  else{
    while (XCheckMaskEvent (dpy,
                            ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
			    | PointerMotionMask | KeyPressMask, &event)){}
  }

}


void searchvarControl::searchvar()
{
  Widget dialog;
  XmString t;
  Arg args[5];
  int n=0;
  XmString titlestring;

  t = XmStringCreateLocalized ("Enter the variable/parameter/keyword to search for:");

  XtSetArg (args[n], XmNautoUnmanage, False); n++;
  XtSetArg (args[n], XmNselectionLabelString, t); n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  XmStringFree (t);

  titlestring = XmStringCreateLocalized("Select a possibility");
  XtVaSetValues (dialog,
		 XmNdialogTitle,     titlestring,
		 NULL);

  XmStringFree(titlestring);

  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);


  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);

}

void searchvarControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  int i, l, len, lentot=0;
  Widget dialog, Helpdialog, scrolledwindow;
  char **lines;
  int totlines=0;
  char othertype[50];
  char *value;
  char buf[10000];
  char systemcom[200];
  Arg args[10];
  int n=0;
  XmString m;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }
  

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "search_dialog_helpbutton", buf, 10000);
    if(l<0){
      Helpdialog = PostErrorDialog(_w, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(_w, "Help on searching for quantities", buf);
    return;
  default:
    break;
  }
  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(strcmp(value, "")==0){
    dialog = PostErrorDialog(_w, label, "Enter something to search for...");
    XtFree(value);
    return;
  }

  TOC (True, toplevel);
  TOC1 (True, &widget);

  snprintf(systemcom, 200, "%s sysfiles/alldescs.dat", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "description file could not be made - see terminal for details...");
    TOC1 (False, &widget);
    XtDestroyWidget (widget);
    TOC (False, toplevel);
    XtFree(value);
    return;
  }


  totlines = orderlines(value, "", "sysfiles/alldescs.dat", &lines, othertype, 50);

  lentot=0;strcpy(buf, ""); // safe
  if(totlines > 0){
    for(i=0;i<totlines;i++){
      len=strlen(lines[i]);
      lentot+=len+1;
      if(lentot<9999){
        strcat(buf, lines[i]); // safe
	strcat(buf, "\n");
      }
    }


    m= XmStringCreateLocalized ("Possible matches:");
    XtSetArg(args[n], XmNmessageString, m); n++;
    dialog = XmCreateMessageDialog (_w, "matches", args, n);
    XtSetSensitive (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);

    n=0;
    XtSetArg(args[n], XmNscrollVertical, True); n++;
    XtSetArg(args[n], XmNscrollHorizontal, False); n++;
    XtSetArg(args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg(args[n], XmNeditable, False); n++;
    XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
    XtSetArg(args[n], XmNwordWrap, True); n++;
    XtSetArg(args[n], XmNvalue, buf); n++;
    XtSetArg(args[n], XmNrows, 10); n++;
    XtSetArg(args[n], XmNcolumns, 100); n++;

    scrolledwindow=XmCreateScrolledText(dialog, "matchtext", args, n);

    XtManageChild(scrolledwindow);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
    TOC1 (False, &widget);
    TOC (False, toplevel);
    telluser(label, "Possible matches listed...");
    //    TimeoutCursors (False);
    XtFree(value);
    freearraydat(lines, totlines);
    return;
  }

  dialog = PostErrorDialog(_w, label, "No matches found...");
  TOC1 (False, &widget);
  TOC (False, toplevel);
  XtFree(value);
  return;

}



void searchvarControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  searchvarControl *obj = (searchvarControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void searchvarControl::searchvarCallback(Widget, XtPointer client_data, XtPointer){
  searchvarControl *obj = (searchvarControl *) client_data;
  obj->searchvar();
};

void searchvarControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  searchvarControl *obj = (searchvarControl *) client_data;
  obj->help();
};

void searchvarControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "search_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on searching for quantities", vardesc);

}

//constructor

searchvarControl::searchvarControl(Widget parent, Widget *toplevel1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  toplevel = toplevel1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&searchvarControl::searchvarCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&searchvarControl::helpCallback,
		(XtPointer) this);
} // end of constructor


//
//
// selectfileControl
//
//

void selectfileControl::selectfile()
{
  bool autoreg;
  char *dirmask;
  XmString  xmstr_input_dirmask;
  Widget text_w;
  char *inputfilereal;

  autoreg = XmToggleButtonGetState((*tog)->baseWidget());

  // get the current filename

  inputfstr = XmTextGetString(*inputfile);
  if(autoreg==false){
    if(!startswith(inputfstr, "models/inputs/"))
      inputfilereal = enclose(inputfstr, "models/inputs/", "");
    else
      inputfilereal = strdup(inputfstr);
  }
  else{
    if(!startswith(inputfstr, "models/autoreginputs/"))
      inputfilereal = enclose(inputfstr, "models/autoreginputs/", "");
    else
      inputfilereal = strdup(inputfstr);
  }

  dirmask=getdmask(inputfilereal, ".dat");
  xmstr_input_dirmask = XmStringCreateSimple (dirmask);
  free(dirmask);

  text_w = XmFileSelectionBoxGetChild(*fsdialog, XmDIALOG_TEXT);

  XmFileSelectionDoSearch(*fsdialog, xmstr_input_dirmask);
  XmTextSetString(text_w, inputfilereal);
  XmTextSetCursorPosition (text_w, strlen (inputfilereal));

  XtManageChild (*fsdialog);
  XmStringFree(xmstr_input_dirmask);XtFree(inputfstr);free(inputfilereal);
  return;

}

void selectfileControl::selectfileCallback(Widget, XtPointer client_data, XtPointer){
  selectfileControl *obj = (selectfileControl *) client_data;
  obj->selectfile();
};

void selectfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectfileControl *obj = (selectfileControl *) client_data;
  obj->help();
};

void selectfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "select_file_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on selecting input files", vardesc);

}


//constructor

selectfileControl::selectfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1, plotToggle **tog1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  tog = tog1;
  fsdialog=fsdlg;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectfileControl::selectfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor


//
//
// selectpfileControl
//
//

void selectpfileControl::selectpfile()
{
  bool autoreg;
  char *dirmask;
  XmString  xmstr_input_dirmask;
  Widget text_w;
  char *inputfilereal;

  autoreg = XmToggleButtonGetState((*tog)->baseWidget());

  // get the current filename

  inputfstr = XmTextGetString(*inputfile);
  if(!startswith(inputfstr, "models/pvals/"))
    inputfilereal = enclose(inputfstr, "models/pvals/", "");
  else
    inputfilereal = strdup(inputfstr);
  XtFree(inputfstr);

  // get dirmask
  dirmask=getdmask(inputfilereal, ".dat");
  xmstr_input_dirmask = XmStringCreateSimple (dirmask);
  free(dirmask);

  text_w = XmFileSelectionBoxGetChild(*fsdialog, XmDIALOG_TEXT);

  XmFileSelectionDoSearch(*fsdialog, xmstr_input_dirmask);
  XmTextSetString(text_w, inputfilereal);
  XmTextSetCursorPosition (text_w, strlen (inputfilereal));

  XtManageChild (*fsdialog);
  XmStringFree(xmstr_input_dirmask);free(inputfilereal);
  return;

}

void selectpfileControl::selectpfileCallback(Widget, XtPointer client_data, XtPointer){
  selectpfileControl *obj = (selectpfileControl *) client_data;
  obj->selectpfile();
};

void selectpfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectpfileControl *obj = (selectpfileControl *) client_data;
  obj->help();
};

void selectpfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "select_pfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on selecting parameter files", vardesc);

}


//constructor

selectpfileControl::selectpfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1, plotToggle **tog1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  tog = tog1;
  fsdialog=fsdlg;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectpfileControl::selectpfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectpfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor





//
//
// selectoutvarControl
//
//

void selectoutvarControl::selectoutvar()
{
  Widget dialog;
  XmString t, *str, tempstr;
  int i=0, len, l;
  FILE *fd;
  char oneline[200];
  int lim=200;
  int totlines;
  char **lines;
  char oneword[100];
  char systemcom[200];
  Widget errdialog;
  Arg args[5];
  int n=0;
  char makecom[50];
  char *fname;
  char *fnm;

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  snprintf(systemcom, 200, "%s src/lib/P_makeoutputinfo", makecom);
  ex_cmnd(systemcom, text_comout, toplevel);


  fnm=XmTextGetString(*text_outfile);
  fname=getnthblk(fnm,1);
  XtFree(fnm);

  if(!startswith(fname, "outdat/")){
    fnm=strdup(fname);
    free(fname);
    fname=enclose(fnm, "outdat/", "");
    free(fnm);
  }

  if(!fileexists(fname)){
    errdialog = PostErrorDialog(_w, label, "The output file does not exist.");
    free(fname);
    return;
  }

  snprintf(systemcom, 200, "./src/lib/P_makeoutputinfo %s", fname); // makes sysfiles/outputvars.dat

  l=system(systemcom);
  if(l!=0){
    errdialog = PostErrorDialog(_w, label, "ERROR with getting file info. Check the file.");
    free(fname);
    return;
  }

  totlines = getnoncomlines("sysfiles/outputvars.dat"); // made by makeoutputinfo.c
  lines = charar(0, totlines-1);

  fd = fopen("sysfiles/outputvars.dat", "r");
  if(fd == NULL){
    snprintf(systemcom, 200, "File \"sysfiles/outputvars.dat\" can't be opened.");
    errdialog = PostErrorDialog(_w, label, systemcom);
    free(fname);
    free_charar(lines, 0, totlines-1);
    return;
  }
  i=0;
  while((len = getline0(fd, oneline, lim)) > 0){
    if(!iscomsep(oneline)){
      lines[i++] = strdup(oneline);
      getnthword(oneline, oneword, 100, 1);
    }
  } 
  totlines=i;
  fclose(fd);



  str = (XmString *) XtMalloc (totlines*sizeof (XmString));
  t = XmStringCreateLocalized ("outputvars");
  for (i = 0; i < totlines; i++)
    str[i] = XmStringCreateLocalized (lines[i]);

  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  tempstr = XmStringCreateLocalized("Select the output variable");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   totlines,
		 XmNmustMatch,       True,
		 XmNdialogTitle, tempstr,
		 NULL);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtUnmanageChild(XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));

  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNnoMatchCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);


  XmStringFree (t);
  while (--i >= 0){
    XmStringFree (str[i]);
    free (lines[i]);
  }

  free_charar(lines, 0, totlines-1);

  //  XtFree (str); /* now free array pointer */
  XtManageChild (dialog);

  XtPopup (XtParent (dialog), XtGrabNone);
}

void selectoutvarControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char varname[100], varname1[100];
  char msg[256], prompt[100], *value;
  int dialog_type;
  Widget Helpdialog;
  char vardesc[1000];
  int l;
  Widget list_w;
  Widget text_w;
  char *text;
  char *enteredtext;
  XmString *strlist;
  int u_bound;
  int i=0;
  int flag=0;

  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  list_w = XmSelectionBoxGetChild (widget, XmDIALOG_LIST);
  text_w = XmSelectionBoxGetChild (widget, XmDIALOG_TEXT);


  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "selectoutvar_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(widget, "Help", vardesc);
      return;
    case XmCR_OK:
      strcpy(prompt, "Selection: "); // safe
      dialog_type = XmDIALOG_MESSAGE;
      break;
    case XmCR_NO_MATCH:
      fprintf(stderr, "no match.\n");
      strcpy(prompt, "Not a valid selection: "); // safe
      dialog_type = XmDIALOG_ERROR;
      break;
    default:
      strcpy(prompt, "Unknown selection: "); // safe
      dialog_type = XmDIALOG_ERROR;
      break;
  }


  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  snprintf (msg, 256, "%s%s", prompt, value);

  if (cbs->reason != XmCR_NO_MATCH) {
    getnthword(value, varname, 100, 1);
    XmTextSetString(*text_outvar, varname);
    XmTextSetCursorPosition (*text_outvar, strlen (varname));
    flag = 1;
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
  }
  else{ // in case there is no match, do a search...
    enteredtext = XmTextFieldGetString (text_w);
    getnthword(enteredtext, varname, 100, 1);
    if(!enteredtext || !*enteredtext){
      XtFree(enteredtext);
    }

    else{
      XtVaGetValues (list_w,
		     XmNitemCount, &u_bound,
		     XmNitems, &strlist,
		     NULL);
    
      while(i<u_bound){
	if(!XmStringGetLtoR (strlist[i], XmFONTLIST_DEFAULT_TAG, &text))
	  break;

	getnthword(text, varname1, 100, 1);

	if(strcmp(varname, varname1)==0){
	  flag=1;
	  break;
	}
	else{i++;}
	XtFree(text);

      }
      if((flag==0) && (strlen(enteredtext)>1)){ // variable not found
	i=0;
	while(i<u_bound){
        // does it occur as a substring?
	  if(!XmStringGetLtoR (strlist[i], XmFONTLIST_DEFAULT_TAG, &text))
	    break;
	  if(strstr(text, enteredtext)!=NULL){
	    flag=1;
	    break;
	  }
	  else{i++;}
	  XtFree(text);
	}
      }
    }

    if(flag==0){
      if (isonlyspace(varname))
	(void) PostErrorDialog(_w, label, "Please enter something");
      else
	(void) PostErrorDialog(_w, label, "Couldn't find any matching variable");
    }
    else{
      XmTextFieldSetString (text_w, text);
      XtFree(text);
    }

  }

  XtFree (value);
}


void selectoutvarControl::selectoutvarCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  selectoutvarControl *obj = (selectoutvarControl *) client_data;
  obj->selectoutvar();
};

void selectoutvarControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  selectoutvarControl *obj = (selectoutvarControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void selectoutvarControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectoutvarControl *obj = (selectoutvarControl *) client_data;
  obj->help();
};

void selectoutvarControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "selectoutvar_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on selecting an output variable", vardesc);

}


//constructor

selectoutvarControl::selectoutvarControl(Widget parent, Widget *toplevel1, Widget *text_outputfile, Widget *text_comout1, Widget *text_outvar1, char *name, char display[], char *filename1, messagelabel **label1, plotToggle **tog1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_outvar = text_outvar1;
  tog = tog1;
  filename = filename1;
  toplevel = toplevel1;
  text_comout = text_comout1;
  text_outfile = text_outputfile;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectoutvarControl::selectoutvarCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectoutvarControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// selectmodelControl
//
//

void selectmodelControl::selectmodel()
{
  Widget dialog;
  XmString t, *str, titlestring;
  int i=0;
  //    extern void dialog_callback();
  int totlines=0;
  char **lines;
  Arg args[5];
  int n=0;

  lines = genlinearraydat(filename, &totlines);

  str = (XmString *) XtMalloc (totlines*sizeof (XmString));
  t = XmStringCreateLocalized ("models available");
  for (i = 0; i < totlines; i++)
    str[i] = XmStringCreateLocalized (lines[i]);
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the model");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   totlines,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNnoMatchCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XmStringFree (t);
  freearraydat(lines, totlines);

  while (--i >= 0)
    XmStringFree (str[i]);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void selectmodelControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char modname[100];
  char vardesc[1000];
  char *tmp;
  char *defvar;
  char msg[256], prompt[100], *value;
  int dialog_type, l=0;
  Widget Helpdialog;
  Widget Errdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "selectmodel_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(_w, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(_w, "Help on model selection", vardesc);
      return;
    case XmCR_OK:
      strcpy(prompt, "Selection: "); //safe
      dialog_type = XmDIALOG_MESSAGE;
      break;
    case XmCR_NO_MATCH:
      //      fprintf(stderr, "hello\n");
      strcpy(prompt, "Not a valid selection: "); //safe
      dialog_type = XmDIALOG_ERROR;
      break;
    default:
      strcpy(prompt, "Unknown selection: "); //safe
      dialog_type = XmDIALOG_ERROR;
      break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  if(strcmp(value, "") == 0){
    Errdialog = PostErrorDialog(_w, label, "Nothing selected. The model is being left unchanged...");
    XtFree (value);
    XtDestroyWidget (widget);
    return;
  }

  snprintf (msg, 256, "%s%s", prompt, value);

  if(dialog_type != XmDIALOG_MESSAGE)
    (void) PostDialog (XtParent(XtParent (widget)), dialog_type, msg);

  if (cbs->reason != XmCR_NO_MATCH) {
    XtPopdown (XtParent (widget));
    getnthblock(value, modname, 100, 1);
    XmTextSetString(*text_outvar, modname);
    XmTextSetCursorPosition (*text_outvar, strlen (modname));

    tmp=getmoddef("userfiles/model_defaults", modname, "modname");
    if(strcmp(tmp, modname) != 0){
      Errdialog = PostErrorDialog(_w, label, "ERROR: Model not found.");
      XtFree (value);
      XtDestroyWidget (widget);
      free(tmp);
      return;
    }

	  
    // always set by default to dynamic simulations when you change the model
    // must do this first otherwise it interferes with the input file setting
    tmp=getmoddef("userfiles/model_defaults", modname, "auto");
    XmToggleButtonSetState((*tog)->baseWidget(), false, true);
    //  getnthblock(vardesc, modname, 100, 5);
    if(strcmp(tmp, "noauto") == 0)
      XtSetSensitive((*tog)->baseWidget(), FALSE);
    else
      XtSetSensitive((*tog)->baseWidget(), TRUE);
    free(tmp);

    tmp=getmoddef("userfiles/model_defaults", modname, "input");
    XmTextSetString(*text_infile, tmp);
    XmTextSetCursorPosition (*text_infile, strlen (tmp));
    free(tmp);

    defvar=getmoddef("userfiles/model_defaults", modname, "var");
    if(strcmp(defvar, "?")==0){
      free(defvar);defvar=strdup("");
    }
    XmTextSetString(*plot_text, defvar);

    tmp=getmoddef("userfiles/model_defaults", modname, "pvals");
    XmTextSetString(*text_pfile, tmp);
    XmTextSetCursorPosition (*text_pfile, strlen (tmp));
    free(tmp);

    tmp=getmoddef("userfiles/model_defaults", modname, "module");
    XmTextSetString(*text_mfile, tmp);
    XmTextSetCursorPosition (*text_mfile, strlen (tmp));
    free(tmp);

    tmp=getmoddef("userfiles/model_defaults", modname, "modeldesc");
    XmTextSetString(*text_datfile, tmp);
    XmTextSetCursorPosition (*text_datfile, strlen (tmp));
    free(tmp);


    snprintf (msg, 256, "%s %s", makecom, modname);
    l=system(msg);
    if(l!=0){
      Errdialog = PostErrorDialog(_w, label, "the model could not be made. See terminal for details...");
      XtFree (value);
      XtDestroyWidget (widget);
      return;
    }
    XtDestroyWidget (widget); 
  }
  XtFree (value);
  telluser(label, "model changed");
  (*pc)->clearall("");
  (*vc)->clearall(defvar);
  free(defvar);
}

void selectmodelControl::selectmodelCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  selectmodelControl *obj = (selectmodelControl *) client_data;
  obj->selectmodel();
};

void selectmodelControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  selectmodelControl *obj = (selectmodelControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void selectmodelControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectmodelControl *obj = (selectmodelControl *) client_data;
  obj->help();
};

void selectmodelControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "select_model_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on selecting a model", vardesc);

}


//constructor

selectmodelControl::selectmodelControl(Widget parent, Widget *toplevel1, Widget *text_comout1, Widget *text_outvar1, Widget *text_infile1, Widget *plot_text1, Widget *text_paramfile, Widget *text_modulefile, Widget *text_datafile, char *name, char display[], char *filename1, messagelabel **label1, plotToggle **tog1, paramControl **pc1, varControl **vc1): BasicComponent (name){
  char systemcom[200];
  char makecom[50];
  Widget Errdialog;
  char defmodel[100];

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if(getoption("userfiles/gui_options", "DEF_MODEL", defmodel, 100)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }


  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_outvar = text_outvar1;
  tog = tog1;
  plot_text=plot_text1;
  text_pfile = text_paramfile;
  text_mfile = text_modulefile;
  text_datfile = text_datafile;
  filename = filename1;
  toplevel = toplevel1;
  text_infile = text_infile1;
  text_comout = text_comout1;
  pc = pc1;
  vc = vc1;

  // ensure this default behaviour each time the interface starts

  snprintf(systemcom, 200, "%s %s", makecom, defmodel);
  system(systemcom);

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectmodelControl::selectmodelCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectmodelControl::helpCallback,
		(XtPointer) this);
} // end of constructor





//
//
// reducefileControl
//
//

void reducefileControl::reducefile()
{
  Widget dialog;
  XmString t, tempstr;
  Widget Errdialog;
  Widget confirmdialog;
  Arg args[5];
  int n=0;
  char *inputfstr;
  int numf;
  char *fnm;
  char *outfstr;
  char systemcom[200];
  int chngd=0;
  XmString str, yes, titlestring;

  // check the output file exists


  inputfstr=XmTextGetString(*text_filename);
  numf=getnumblocks(inputfstr, " :,");
  fnm=getnthblk(inputfstr, 1);
  XtFree(inputfstr);
  outfstr=strdup(fnm);
  if(!fileexists(fnm) && !startswith(fnm, "outdat/")){
    free(fnm);
    fnm=enclose(outfstr, "outdat/", "");
    chngd=1;
  }


  if(!fileexists(fnm)){
    if(chngd==0)
      snprintf(systemcom, 200, "No file %s was found", fnm);
    else
      snprintf(systemcom, 200, "No file %s or %s was found", outfstr, fnm);
    Errdialog = PostErrorDialog(_w, label, systemcom);
    free(outfstr);
    free(fnm);
    return;
  }

  if(numf>1 || chngd > 0){ // input file name changed
    ANS=0;
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
    snprintf(systemcom, 200, "The file to be reduced is being assumed to be %s.\nTo continue press \"Continue\".\n", fnm);
    titlestring = XmStringCreateLocalized("Filename");
    str = XmStringCreateLtoR (systemcom, "MY_TAG");

    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedialog, (XtPointer) this);
    XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, (XtPointer) this);

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (ANS == 0)
      XtAppProcessEvent (*app1, XtIMAll);

    if(ANS==2){ // cancel button pressed
      free(outfstr);
      free(fnm);
      return;
    }
  }
  free(outfstr);
  free(fnm);

  t = XmStringCreateLocalized ("Enter new file name followed by desired variables");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Create a reduced output file");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);

  XtPopup (XtParent (dialog), XtGrabNone);
}

void reducefileControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[200];
  char *value;
  char *fname;
  char vardesc[1000];
  int l;
  Widget dialog;
  Widget Helpdialog;
  char *inputfstr;
  char *outfstr;
  int numf;
  char *fnm;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];
  char *tmp;
  char *vars;
  Widget confirmdialog;
  XmString str, yes, titlestring;

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "reducefile_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(widget, "Help on reduced data files", vardesc);
      return;
    default:
      break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  fname = getnthblk(value, 1);
  vars = getrst(fname, value);
  XtFree(value);


  if(isonlyspace(vars)){
    snprintf(systemcom, 200, "You need to enter an output file\nand some variables.");
    dialog = PostErrorDialog(_w, label, systemcom);
    free(fname);free(vars);
    return;
  }

  if(!startswith(fname, "outdat/")){
    tmp=strdup(fname);free(fname);
    fname=enclose(tmp, "outdat/", "");

    ANS=0;
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
    snprintf(systemcom, 200, "The file name that you chose: %s has been\nchanged to %s. To continue press \"Continue\".\n", tmp, fname);
    free(tmp); 
    titlestring = XmStringCreateLocalized("Filename changed");
    str = XmStringCreateLtoR (systemcom, "MY_TAG");
    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedialog, (XtPointer) this);
    XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, (XtPointer) this);

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (ANS == 0)
      XtAppProcessEvent (*app1, XtIMAll);

    if(ANS==2){ // cancel button pressed
      free(fname);free(vars);
      return;
    }

  }

  if(fileexists(fname)){
    snprintf(systemcom, 200, "File %s already exists", fname);
    dialog = PostErrorDialog(_w, label, systemcom);
    free(fname);free(vars);
    return;
  }

  snprintf(systemcom, 200, "%s src/lib/P_reddat", makecom);
  
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"src/lib/P_reddat\" - see terminal for details.");
    free(fname);free(vars);
    XtDestroyWidget(widget);
    return;
  }

  inputfstr=XmTextGetString(*text_filename);
  numf=getnumblocks(inputfstr, " :,");
  fnm=getnthblk(inputfstr, 1);
  XtFree(inputfstr);
  outfstr=strdup(fnm);
  if(!fileexists(fnm) && !startswith(fnm, "outdat/")){
    free(fnm);
    fnm=enclose(outfstr, "outdat/", "");
  }

  free(outfstr);


  snprintf(systemcom, 200, "./src/lib/P_reddat %s %s %s", fnm, fname, vars);
  free(fnm);
  
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "reduced file creation failed - see terminal for details.");
    free(fname);free(vars);
    XtDestroyWidget(widget);
    return;
  }

  snprintf(systemcom, 200, "reduced file \"%s\" created.", fname);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  free(fname);free(vars);
  XtDestroyWidget(widget);

}

void reducefileControl::reducefileCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  reducefileControl *obj = (reducefileControl *) client_data;
  obj->reducefile();
};

void reducefileControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  reducefileControl *obj = (reducefileControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void reducefileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  reducefileControl *obj = (reducefileControl *) client_data;
  obj->help();
};

void reducefileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "reduce_outputfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on reducing output files", vardesc);

}



//constructor

reducefileControl::reducefileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&reducefileControl::reducefileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&reducefileControl::helpCallback,
		(XtPointer) this);
} // end of constructor





