/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

*/

//#include <sys/types.h>
//#include <unistd.h>
//#include <time.h> // for sleep
#include <stdio.h> // for printf
#include <stdlib.h> // for system
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/SelectioB.h>
#include <Xm/MessageB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>
#include <Xm/FileSB.h>
#include <Xm/Separator.h>
#include <sys/types.h>
#include <Xm/ToggleB.h>
#include <sys/stat.h>

#include "BasicComponent.h"
#include "messagelabel.h"


#include "../common/stringmanip.h"
#include "editsys.h"

void telluser(messagelabel **label, char mess[]); // defined in control.cc - just needs to be declared
void free_charar(char **m, long nl, long nh);
Widget PostErrorDialog(Widget parent, messagelabel **label, char *msg); // in controls.cc
Widget PostEDialog(Widget parent, char *msg);
Widget PostLongDialog(Widget parent, char *title, char *msg);
Widget PostDialog(Widget parent, int dialog_type, char *msg); // in controls.cc
Widget PostHelpDialog(Widget parent, char *title, char *msg); // in controls.cc
void DestWidget(Widget widget, XtPointer client_data, XtPointer call_data); // in controls.cc
void TOC(Boolean on, Widget *wid); // in controls.cc
void changedialog(Widget widget, XtPointer client_data, XtPointer call_data); // in controls.cc
extern int ANS;


//
//
// viewmfileControl
//
//

void viewmfileControl::viewmfile()
{
  char systemcom[200];
  char *inputfstr = XmTextGetString(*inputfile);
  char editorstr[100];
  Widget Errdialog;
  struct stat fst;
  XmString str, yes, titlestring;
  Widget confirmdialog;
  char *inputfilereal;

  if(!startswith(inputfstr, "models/modules/"))
    inputfilereal = enclose(inputfstr, "models/modules/", "");
  else
    inputfilereal = strdup(inputfstr);


  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 100)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    free(inputfilereal);
    XtFree(inputfstr);
    return;
  }

  ANS = 0;

  lstat(inputfilereal, &fst);

  //  fprintf(stderr, "number of links: %d\n", fst.st_nlink);
  if(((fst.st_mode & S_IFMT) == S_IFLNK)){
    //    fprintf(stderr, "it's a link.\n");
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
    snprintf(systemcom, 200, "Note that the file you want to\nopen is probably a shared file.\nBe aware of this if you edit it.");
    titlestring = XmStringCreateLocalized("WARNING: a shared file");
    str = XmStringCreateLtoR (systemcom, "MY_TAG");

    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedialog, XtPointer (this));
    XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, XtPointer (this));

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while (ANS==0)
      XtAppProcessEvent (*app1, XtIMAll);

    if(ANS==2){ // cancel button pressed
      free(inputfilereal);
      XtFree(inputfstr);
      return;
    }

  }

  snprintf(systemcom, 200, "%s %s &", editorstr, inputfstr);
  telluser(label, systemcom);
  system(systemcom);

  XtFree(inputfstr);
  free(inputfilereal);

}



void viewmfileControl::viewmfileCallback(Widget, XtPointer client_data, XtPointer){
  viewmfileControl *obj = (viewmfileControl *) client_data;
  obj->viewmfile();
};

void viewmfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  viewmfileControl *obj = (viewmfileControl *) client_data;
  obj->help();
};

void viewmfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "view_mfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on viewing the module file", vardesc);

}


//constructor

viewmfileControl::viewmfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&viewmfileControl::viewmfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&viewmfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor




//
//
// parsemfileControl
//
//

void parsemfileControl::parsemfile()
{
  char systemcom[200];
  char *inputfstr = XmTextGetString(*inputfile);
  XmString str, yes, titlestring;
  Widget confirmdialog;
  Widget dialog;
  int l;

  ANS = 0;

  confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
  snprintf(systemcom, 200, "Are you sure that you want to update\nvariables and parameters in this file?");
  titlestring = XmStringCreateLocalized("WARNING: this alters the file");
  str = XmStringCreateLtoR (systemcom, "MY_TAG");
  yes = XmStringCreateLocalized ("Continue");

  XtVaSetValues (confirmdialog,
		 XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		 XmNokLabelString, yes,
		 XmNmessageString, str,
		 XmNdialogTitle,     titlestring,
		 NULL);

  XmStringFree(yes); XmStringFree (str); XmStringFree(titlestring);

  XtAddCallback(confirmdialog, XmNokCallback, changedialog, XtPointer (this));
  XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, XtPointer (this));

  XtManageChild (confirmdialog);
  XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

  XtPopup (XtParent (confirmdialog), XtGrabNone);

  // don't return control until we have an answer. 

  while (ANS==0)
    XtAppProcessEvent (*app1, XtIMAll);

  if(ANS==2){ // cancel button pressed
    XtFree(inputfstr);
    return;
  }
  // This bit adds variables and parameters where necessary... Be careful with its inclusion

  l=quickparse(inputfstr, "userfiles/remsfile.dat");
  if(l==0){
    dialog = PostLongDialog(_w, "Nothing to do", "Nothing was changed - the file was up to date.");
    XtFree(inputfstr);
    return;
  }
  else if (l<0){
    dialog = PostErrorDialog(_w, label, "Parsing failed. See terminal for details.");
    XtFree(inputfstr);
    return;
  }
  else{
    if(l==1)
      snprintf(systemcom, 200, "Parsing successful. %d change was made to the file\n\n\t%s\n\nSee terminal for details.", l, inputfstr);
    else
      snprintf(systemcom, 200, "Parsing successful. %d changes were made to the file\n\n    %s\n\nSee terminal for details.", l, inputfstr);
    dialog = PostLongDialog(_w, "Parsing successful", systemcom);
    XtFree(inputfstr);
    return;
  }

  XtFree(inputfstr);

}


void parsemfileControl::parsemfileCallback(Widget, XtPointer client_data, XtPointer){
  parsemfileControl *obj = (parsemfileControl *) client_data;
  obj->parsemfile();
};

void parsemfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  parsemfileControl *obj = (parsemfileControl *) client_data;
  obj->help();
};

void parsemfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "parse_mfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on parsing the module file", vardesc);

}


//constructor

parsemfileControl::parsemfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&parsemfileControl::parsemfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&parsemfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor



//
//
// scanmfileControl
//
//

void scanmfileControl::scanmfile()
{
  char *inputfstr = XmTextGetString(*inputfile);
  Widget Helpdialog;
  char outstr[2000];
  int l;

  if((l=quickscan(inputfstr, "userfiles/remsfile.dat", outstr, 2000)) < 0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in scanning file: see terminal for details");
    return;
  }

  Helpdialog = PostHelpDialog(_w, "Scan results for file", outstr);

}

void scanmfileControl::scanmfileCallback(Widget, XtPointer client_data, XtPointer){
  scanmfileControl *obj = (scanmfileControl *) client_data;
  obj->scanmfile();
};

void scanmfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  scanmfileControl *obj = (scanmfileControl *) client_data;
  obj->help();
};

void scanmfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "scan_mfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on scanning the module file", vardesc);

}


//constructor

scanmfileControl::scanmfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&scanmfileControl::scanmfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&scanmfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor


//
//
// selectmfileControl
//
//

void selectmfileControl::selectmfile()
{
  char *dirmask;
  XmString  xmstr_input_dirmask;
  Widget text_w;
  char *inputfilereal;


  // get the current filename

  inputfstr = XmTextGetString(*inputfile);
  if(!startswith(inputfstr, "models/modules/"))
    inputfilereal = enclose(inputfstr, "models/modules/", "");
  else
    inputfilereal = strdup(inputfstr);
  XtFree(inputfstr);

  // get dirmask
  dirmask=getdmask(inputfilereal, ".dt");
  xmstr_input_dirmask = XmStringCreateSimple (dirmask);
  free(dirmask);

  text_w = XmFileSelectionBoxGetChild(*fsdialog, XmDIALOG_TEXT);

  XmFileSelectionDoSearch(*fsdialog, xmstr_input_dirmask);
  XmTextSetString(text_w, inputfilereal);
  XmTextSetCursorPosition (text_w, strlen (inputfilereal));

  XtManageChild (*fsdialog);
  XmStringFree(xmstr_input_dirmask);free(inputfilereal);
  return;

}

void selectmfileControl::selectmfileCallback(Widget, XtPointer client_data, XtPointer){
  selectmfileControl *obj = (selectmfileControl *) client_data;
  obj->selectmfile();
};

void selectmfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectmfileControl *obj = (selectmfileControl *) client_data;
  obj->help();
};

void selectmfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "select_mfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on selecting parameter files", vardesc);

}


//constructor

selectmfileControl::selectmfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  fsdialog=fsdlg;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectmfileControl::selectmfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectmfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor



//
//
// viewdatfileControl
//
//

void viewdatfileControl::viewdatfile()
{
  char systemcom[200];
  char *inputfstr = XmTextGetString(*inputfile);
  char editorstr[100];
  Widget Errdialog;
  char *inputfilereal;

  if(!startswith(inputfstr, "models/modeldescs/"))
    inputfilereal = enclose(inputfstr, "models/modeldescs/", "");
  else
    inputfilereal = strdup(inputfstr);


  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 100)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    XtFree(inputfstr);
    free(inputfilereal);
    return;
  }

  snprintf(systemcom, 200, "%s %s &", editorstr, inputfstr);
  telluser(label, systemcom);
  system(systemcom);
  XtFree(inputfstr);
  free(inputfilereal);
  return;
}

void viewdatfileControl::viewdatfileCallback(Widget, XtPointer client_data, XtPointer){
  viewdatfileControl *obj = (viewdatfileControl *) client_data;
  obj->viewdatfile();
};

void viewdatfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  viewdatfileControl *obj = (viewdatfileControl *) client_data;
  obj->help();
};

void viewdatfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "view_datfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Viewing model descriptor files", vardesc);

}


//constructor

viewdatfileControl::viewdatfileControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&viewdatfileControl::viewdatfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&viewdatfileControl::helpCallback,
		(XtPointer) this);

} // end of constructor


//
//
// selectdatfileControl
//
//

void selectdatfileControl::selectdatfile()
{
  char *dirmask;
  XmString  xmstr_input_dirmask;
  Widget text_w;
  char *inputfilereal;

  // get the current filename

  inputfstr = XmTextGetString(*inputfile);
  if(!startswith(inputfstr, "models/modeldescs/"))
    inputfilereal = enclose(inputfstr, "models/modeldescs/", "");
  else
    inputfilereal = strdup(inputfstr);
  XtFree(inputfstr);

  // get dirmask
  dirmask=getdmask(inputfilereal, ".dat");
  xmstr_input_dirmask = XmStringCreateSimple (dirmask);
  free(dirmask);

  text_w = XmFileSelectionBoxGetChild(*fsdialog, XmDIALOG_TEXT);

  XmFileSelectionDoSearch(*fsdialog, xmstr_input_dirmask);
  XmTextSetString(text_w, inputfilereal);
  XmTextSetCursorPosition (text_w, strlen (inputfilereal));

  XtManageChild (*fsdialog);
  XmStringFree(xmstr_input_dirmask);free(inputfilereal);
  return;
}

void selectdatfileControl::selectdatfileCallback(Widget, XtPointer client_data, XtPointer){
  selectdatfileControl *obj = (selectdatfileControl *) client_data;
  obj->selectdatfile();
};

void selectdatfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  selectdatfileControl *obj = (selectdatfileControl *) client_data;
  obj->help();
};

void selectdatfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "select_datfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Selecting a model descriptor file", vardesc);

}


//constructor

selectdatfileControl::selectdatfileControl(Widget parent, Widget *infile, Widget *fsdlg, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  fsdialog=fsdlg;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&selectdatfileControl::selectdatfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&selectdatfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor


//
//
// scanmodelControl
//
//

void scanmodelControl::scanmodel()
{
  //  char *inputfstr = XmTextGetString(*inputfile);
  Widget Helpdialog;
  char outstr[5000];
  int lentot=4999;
  int i, l;
  char makecom[50];
  char **vararray;
  char **paramarray;
  int numvars=0;
  int numparams=0;
  char tempstr1[300];
  char systemcom[100];
  char sep[4];

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    Helpdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }
  TOC(True, toplevel);
  snprintf(systemcom, 100, "%s sysfiles/varbles.dat", makecom);
  l=system(systemcom);
  if(l!=0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "Problem scanning model - see terminal for details.");
    TOC(False, toplevel);
    return;
  }

  snprintf(systemcom, 100, "%s sysfiles/params.dat", makecom);
  l=system(systemcom);
  if(l!=0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "Problem scanning model - see terminal for details.");
    TOC(False, toplevel);
    return;
  }

  TOC(False, toplevel);

  vararray = genarraydat("sysfiles/varbles.dat", &numvars);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  if(numvars+numparams>30)
    strcpy(sep, "  "); // safe
  else
    strcpy(sep, " \n"); // safe

  outstr[0]=0;
  snprintf(tempstr1, 300, "This model defines dynamics for the following %d variables: \n", numvars);
  strncat(outstr, tempstr1, lentot);

  for(i=0;i<numvars;i++){
    snprintf(tempstr1, 300, "%s ", vararray[i]);
    strncat(outstr, tempstr1, lentot-strlen(outstr));
  }

  snprintf(tempstr1, 300, "\n\nThis model uses the following %d parameters: \n", numparams);
  strncat(outstr, tempstr1, lentot-strlen(outstr));

  for(i=0;i<numparams;i++){
    snprintf(tempstr1, 300, "%s ", paramarray[i]);
    strncat(outstr, tempstr1, lentot-strlen(outstr));
  }

  if((int)strlen(outstr)==lentot){
    outstr[lentot-30]=0;
    strcat(outstr, " ... [no more space]\n"); // safe
  }

  wrap(outstr, 100);


  Helpdialog = PostHelpDialog(_w, "Scan results for model", outstr);
  //  free(modname); XtFree(inputfstr); free(deffile);
}


void scanmodelControl::scanmodelCallback(Widget, XtPointer client_data, XtPointer){
  scanmodelControl *obj = (scanmodelControl *) client_data;
  obj->scanmodel();
};

void scanmodelControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  scanmodelControl *obj = (scanmodelControl *) client_data;
  obj->help();
};

void scanmodelControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "scan_model_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on scanning the model", vardesc);
}


//constructor

scanmodelControl::scanmodelControl(Widget parent, Widget *toplevel1, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;
  toplevel=toplevel1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&scanmodelControl::scanmodelCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&scanmodelControl::helpCallback,
		(XtPointer) this);

} // end of constructor

//
//
// cpdescfileControl
//
//

void cpdescfileControl::cpdescfile()
{
  Widget dialog;
  XmString t, tempstr;
  FILE *fd;
  char systemcom[200];
  Widget errdialog;
  Arg args[5];
  int n=0;
  char *fnamefull, *fname;

  fnamefull=XmTextGetString(*text_filename);

  // get the filename without the directory name
  fname = getfname(fnamefull);

  // check the input file exists
  fd = fopen(fnamefull, "r");
  if(fd == NULL){
    snprintf(systemcom, 200, "file %s can't be opened.", fnamefull);
    errdialog = PostErrorDialog(_w, label, systemcom);
    free(fnamefull);free(fname);
    return;
  }
  fclose(fd);


  snprintf(systemcom, 200, "Copy file %s to:", fname);
  free(fnamefull);free(fname);
  t = XmStringCreateLocalized (systemcom);

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Copy an existing input file");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void cpdescfileControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[200];
  char *value;
  char vardesc[1000];
  int l;
  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
 
  char *tmp;
  char *oldfname, *newfname, *dnamefull;
  char *dname;

  switch (cbs->reason) {
    case XmCR_HELP:
      l=getdesc("helpfiles/GUI_WID_HELP", "cpdescfile_dialog_helpbutton", vardesc, 1000);
      if(l<0){
	Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
	return;
      }
      Helpdialog = PostHelpDialog(widget, "Help on copying input files", vardesc);
      return;
    default:
      break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }
  if(!endswith(value, ".dat")){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "Descriptor files must end in \".dat\"\nTry again.");
    XtFree(value);
    return;
  }

  // current directory

  tmp=XmTextGetString(*text_filename);
  oldfname=lrtrim(tmp);free(tmp);
  dnamefull=getdnamef(oldfname); // with terminal '/'

  // construct the new filename

  if(!startswith(value, dnamefull))
    newfname=enclose(value, dnamefull, "");
  free(dnamefull);

  dname=getdname(newfname); // not redundant!
  if(!fileexists(dname)){
    snprintf(systemcom, 200, "Directory %s does not exist\nTry again.", dname);
    dialog = PostLongDialog(_w, "Problem with what you've typed", systemcom);
    free(oldfname);free(dname);free(newfname); XtFree(value);
    return;
  }
  free(dname);

  // check if file exists
  if(fileexists(newfname)){
    snprintf(systemcom, 200, "File %s already exists", value);
    dialog = PostErrorDialog(_w, label, systemcom);
    free(oldfname);free(newfname); XtFree(value);
    return;
  }

  if(cpfile(newfname, oldfname)<0){
    dialog = PostErrorDialog(_w, label, "Error in copying - see terminal for details");
    free(oldfname);free(newfname);
    XtFree(value);
    return;
  }


  snprintf(systemcom, 200, "File \"%s\" created.", value);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  free(oldfname);free(newfname); XtFree(value);
  XtDestroyWidget(widget);

}

void cpdescfileControl::cpdescfileCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpdescfileControl *obj = (cpdescfileControl *) client_data;
  obj->cpdescfile();
};

void cpdescfileControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpdescfileControl *obj = (cpdescfileControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void cpdescfileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  cpdescfileControl *obj = (cpdescfileControl *) client_data;
  obj->help();
};

void cpdescfileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "copy_descfile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on copying input files", vardesc);

}



//constructor

cpdescfileControl::cpdescfileControl(Widget parent, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&cpdescfileControl::cpdescfileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&cpdescfileControl::helpCallback,
		(XtPointer) this);
} // end of constructor







//
//
// changevarControl
//
//

void changevarControl::changevar()
{
  char systemcom[500];
  char *inputfstr = XmTextGetString(*inputfile);
  XmString str, yes, titlestring;
  Widget confirmdialog;
  XmString t, tempstr;
  Widget dialog;
  Arg args[5];
  int n=0;

  ANS = 0;

  confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
  snprintf(systemcom, 500, "Are you sure you want to change the name\nof a variable or parameter in this model?\nShared module files (if any) will also be\nchanged. Only input files in a directory\nunique to the model will be changed.");
  titlestring = XmStringCreateLocalized("WARNING: this alters the file");
  str = XmStringCreateLtoR (systemcom, "MY_TAG");
  yes = XmStringCreateLocalized ("Continue");

  XtVaSetValues (confirmdialog,
		 XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		 XmNokLabelString, yes,
		 XmNmessageString, str,
		 XmNdialogTitle,     titlestring,
		 NULL);

  XmStringFree(yes); XmStringFree (str); XmStringFree(titlestring);

  XtAddCallback(confirmdialog, XmNokCallback, changedialog, XtPointer (this));
  XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, XtPointer (this));

  XtManageChild (confirmdialog);
  XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

  XtPopup (XtParent (confirmdialog), XtGrabNone);

  // don't return control until we have an answer. 

  while(ANS==0)
    XtAppProcessEvent (*app1, XtIMAll);

  if(ANS==2){ // cancel button pressed
    XtFree(inputfstr);
    return;
  }

  t = XmStringCreateLocalized ("Enter the current name followed by the new name");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Change the name of a quantity");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t); XmStringFree(tempstr);

  XtFree(inputfstr);

  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);

  XtPopup (XtParent (dialog), XtGrabNone);
}


void changevarControl::changevarCallback(Widget, XtPointer client_data, XtPointer){
  changevarControl *obj = (changevarControl *) client_data;
  obj->changevar();
};

void changevarControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  changevarControl *obj = (changevarControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void changevarControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[1000];
  char *value;
  char vardesc[1000];
  int l;
  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char *parname[2];
  char *newparname[2];
  char *modname = XmTextGetString(*inputfile);
  char *tvar;
  int comcount=0;
  int warn=0;

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "changevar_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      free(modname);
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on changing the name of a quantity", vardesc);
    free(modname);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  // change the variable name v and v_fcn (otherwise oneparamfunc will have a problem...)

  parname[0] = getnthwd(value, 1);
  newparname[0] = getnthwd(value, 2);
  tvar = getnthwd(value, 3);
  XtFree(value);

  parname[1] = (char*) malloc(sizeof(char) * (strlen(parname[0]) + 5));
  newparname[1] = (char*) malloc(sizeof(char) * (strlen(newparname[0]) + 5));
  sprintf(parname[1], "%s_fcn", parname[0]); // safe
  sprintf(newparname[1], "%s_fcn", newparname[0]); // safe

  if(isonlyspace(parname[0]) || isonlyspace(newparname[0]) || !isonlyspace(tvar)){
    dialog = PostErrorDialog(_w, label, "Please specify exactly two variables");
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(tvar);
    free(modname);
    return;
  }

  if(strcmp(parname[0], newparname[0])==0){
    dialog = PostErrorDialog(_w, label, "Variable names are the same. Nothing is being done.");
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(tvar);
    free(modname);
    return;
  }

  free(tvar);

  l=checkparname(modname, newparname, 2, &comcount);
  if(l>0){
    snprintf(systemcom, 1000, "A variable with the new name %s already exists. This could\nlead to serious problems. Nothing being done.", newparname[0]);
    dialog = PostEDialog(_w, systemcom);
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(modname);
    return;
  }

  if(comcount>0) // comment line with variable
    warn=1;


  l=changeparname(modname, parname, newparname, 2, "userfiles/remsfile.dat");

  if(l==-2){
    dialog = PostErrorDialog(_w, label, "Problem changing variable name\nYou cannot use a reserved word\nas a variable or parameter name.\nSee terminal for details.");
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(modname);
    return;
  }
  else if(l<0){
    dialog = PostErrorDialog(_w, label, "Problem changing variable name - see terminal for details.");
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(modname);
    return;
  }
  else if(l==0){
    snprintf(systemcom, 1000, "No occurences of this variable were found in the model.\nNothing was done.");
    Helpdialog = PostHelpDialog(_w, "operation successful", systemcom);
    free_datarray(parname, 2);
    free_datarray(newparname, 2);
    free(modname);
    XtDestroyWidget(widget);
    return;
  }

  if(warn==0)
    snprintf(systemcom, 1000, "Variable name successfully changed.\n%d lines containing this quantity were changed.\n", l);
  else
    snprintf(systemcom, 1000, "Variable name %s successfully changed to %s.\n%d lines containing this quantity were changed.\n\nWARNING: some comment lines already seemed to\ncontain references to %s\n\nSee the terminal for any other warnings.", parname[0], newparname[0], l, newparname[0]);

  system("touch sysfiles/sysstate/moddir");
  system("touch sysfiles/sysstate/which_desc_file");
  Helpdialog = PostHelpDialog(_w, "operation successful", systemcom);

  free_datarray(parname, 2);
  free_datarray(newparname, 2);
  free(modname);
  XtDestroyWidget(widget);
  return;

}


void changevarControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  changevarControl *obj = (changevarControl *) client_data;
  obj->help();
};

void changevarControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "changevar_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on parsing the module file", vardesc);

}


//constructor

changevarControl::changevarControl(Widget parent, Widget *infile, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  inputfile = infile;
  app1 = app;


  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&changevarControl::changevarCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&changevarControl::helpCallback,
		(XtPointer) this);

} // end of constructor


//
//
// editinfofileControl
//
//

void editinfofileControl::editinfofile()
{
  char systemcom[200];
  char *modname;
  char editorstr[100];
  Widget Errdialog;
  XmString str, yes, titlestring;
  Widget confirmdialog;
  char *inputfile;

  modname = XmTextGetString(*text_mod);

  if(getoption("userfiles/gen_options", "EDITOR", editorstr, 100)!=0){
    Errdialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    XtFree(modname);
    return;
  }

  inputfile = (char*) malloc(sizeof(char) * (strlen(modname) + 40));
  sprintf(inputfile, "models/modeldescs/%s/info", modname); // safe

  if(!fileexists(inputfile)){

    ANS=0;
    confirmdialog = XmCreateQuestionDialog (_w, "confirm", NULL, 0);
    snprintf(systemcom, 200, "An info file for this model does not seem to exist.\nWould you like to create one?");
    titlestring = XmStringCreateLocalized("Create info file?");
    str = XmStringCreateLtoR (systemcom, "MY_TAG");

    yes = XmStringCreateLocalized ("Continue");

    XtVaSetValues (confirmdialog,
		   XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL, 
		   XmNokLabelString, yes,
		   XmNmessageString, str,
		   XmNdialogTitle,     titlestring,
		   NULL);

    XmStringFree(yes);
    XmStringFree (str);
    XmStringFree(titlestring);

    XtAddCallback(confirmdialog, XmNokCallback, changedialog, XtPointer (this));
    XtAddCallback(confirmdialog, XmNcancelCallback, changedialog, XtPointer (this));

    XtManageChild (confirmdialog);
    XtUnmanageChild(XmMessageBoxGetChild (confirmdialog, XmDIALOG_HELP_BUTTON));

    XtPopup (XtParent (confirmdialog), XtGrabNone);

    // don't return control until we have an answer. 

    while(ANS==0)
      XtAppProcessEvent (*app1, XtIMAll);

    if(ANS==2){ // cancel button pressed
      free(inputfile);
      XtFree(modname);
      return;
    }

  }

  snprintf(systemcom, 200, "%s %s &", editorstr, inputfile);
  telluser(label, systemcom);
  system(systemcom);

  free(inputfile);
  XtFree(modname);

}


void editinfofileControl::editinfofileCallback(Widget, XtPointer client_data, XtPointer){
  editinfofileControl *obj = (editinfofileControl *) client_data;
  obj->editinfofile();
};

void editinfofileControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  editinfofileControl *obj = (editinfofileControl *) client_data;
  obj->help();
};

void editinfofileControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "edit_infofile_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on info files", vardesc);

}


//constructor

editinfofileControl::editinfofileControl(Widget parent, Widget *text_modname, char *name, char display[], messagelabel **label1, XtAppContext *app): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_mod=text_modname;
  app1 = app;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&editinfofileControl::editinfofileCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&editinfofileControl::helpCallback,
		(XtPointer) this);

} // end of constructor




//
//
// setmoddefControl
//
//

void setmoddefControl::setmoddef()
{
  Widget dialog;
  XmString t, *str, titlestring;
  int i=0;
  Arg args[5];
  int n=0;

  str = (XmString *) XtMalloc (7*sizeof (XmString));
  t = XmStringCreateLocalized ("defaults");
  str[i++] = XmStringCreateLocalized ("input file");
  str[i++] = XmStringCreateLocalized ("key variable");
  str[i++] = XmStringCreateLocalized ("parameter value file");
  str[i++] = XmStringCreateLocalized ("autoregulation?");
  str[i++] = XmStringCreateLocalized ("module file");
  str[i++] = XmStringCreateLocalized ("model descriptor file");
  str[i++] = XmStringCreateLocalized ("autoreg input file");
  XtSetArg(args[n], XmNautoUnmanage, False); n++;

  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the quantity to set");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   7,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring); XmStringFree (t);
  while (--i >= 0)
    XmStringFree (str[i]);

  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNnoMatchCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void setmoddefControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char vardesc[1000];
  char msg[256], prompt[100], *value;
  int dialog_type, l=0;
  XmString t, tempstr;
  char *mod;
  Widget dialog;
  Widget Helpdialog;
  Widget Errdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  Arg args[5];
  int n=0;
  char str1[200];
  char *moddefval;

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "setmoddef_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(_w, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(_w, "Help on model default setting", vardesc);
    return;
  case XmCR_OK:
    strcpy(prompt, "Selection: "); // safe
    dialog_type = XmDIALOG_MESSAGE;
    break;
  case XmCR_NO_MATCH:
    strcpy(prompt, "Not a valid selection: "); // safe
    dialog_type = XmDIALOG_ERROR;
    break;
  default:
    strcpy(prompt, "Unknown selection: "); // safe
    dialog_type = XmDIALOG_ERROR;
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  if(strcmp(value, "") == 0){
    Errdialog = PostErrorDialog(_w, label, "Nothing selected. No model defaults will be changed...");
    XtFree (value);
    XtDestroyWidget (widget);
    return;
  }


  if(strcmp(value, "input file")==0)
    snprintf(defobj, 20, "input");
  else if(strcmp(value, "autoreg input file")==0)
    snprintf(defobj, 20, "autoinput");
  else if(strcmp(value, "key variable")==0)
    snprintf(defobj, 20, "var");
  else if(strcmp(value, "parameter value file")==0)
    snprintf(defobj, 20, "pvals");
  else if(strcmp(value, "autoregulation?")==0)
    snprintf(defobj, 20, "auto");
  else if(strcmp(value, "module file")==0)
    snprintf(defobj, 20, "module");
  else if(strcmp(value, "model descriptor file")==0)
    snprintf(defobj, 20, "modeldesc");

  mod = XmTextGetString(*text_mod);
  snprintf (msg, 256, "%s%s", prompt, value);
  // neaten
  moddefval = getmoddef1("userfiles/model_defaults", mod, defobj);


  if(dialog_type != XmDIALOG_MESSAGE)
    (void) PostDialog (XtParent(XtParent (widget)), dialog_type, msg);

  if (cbs->reason != XmCR_NO_MATCH) {
    XtPopdown (XtParent (widget));
    snprintf(str1, 200, "Change the value of %s in model %s\nfrom current value: %s to:", value, mod, moddefval);
    t = XmStringCreateLocalized (str1);

    XtSetArg (args[n], XmNselectionLabelString, t);n++;
    XtSetArg(args[n], XmNautoUnmanage, False);n++;

    dialog = XmCreatePromptDialog (_w, "prompt", args, n);
    tempstr = XmStringCreateLocalized ("Change a model default");
    XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);


    XmStringFree(t);
    XmStringFree(tempstr);
    free(mod);
    free(moddefval);

    //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
    XtAddCallback(dialog, XmNokCallback, dialog1_callback, (XtPointer) this);
    XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
    XtAddCallback(dialog, XmNhelpCallback, dialog1_callback, (XtPointer) this);

    XtManageChild (dialog);
    XtPopup (XtParent (dialog), XtGrabNone);
  }

  XtFree(value);

}

void setmoddefControl::dlg1(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;
  char *mod;
  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "setmoddef_dialog1_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on reduced data files", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  mod = XmTextGetString(*text_mod);

  l=changemoddef("userfiles/model_defaults", mod, defobj, value);

  if(l!=0){
    if(l==-2){
      dialog = PostErrorDialog(_w, label, "You can't have an empty value.");
      XtFree(value);
      free(mod);
      return;
    }
    dialog = PostErrorDialog(_w, label, "Problem changing a default - see terminal for details.");
    XtFree(value);
    free(mod);
    return;
  }

  snprintf(systemcom, 100, "model default value changed.");
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  XtFree(value);
  free(mod);
  XtDestroyWidget(widget);
}


void setmoddefControl::setmoddefCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  setmoddefControl *obj = (setmoddefControl *) client_data;
  obj->setmoddef();
};



void setmoddefControl::dialog1_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  setmoddefControl *obj = (setmoddefControl *) client_data;
  obj->dlg1(widget, client_data, call_data);
};

void setmoddefControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  setmoddefControl *obj = (setmoddefControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void setmoddefControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  setmoddefControl *obj = (setmoddefControl *) client_data;
  obj->help();
};

void setmoddefControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "setmoddef_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(XtParent(_w), "Help on setting model defaults", vardesc);

}


//constructor

setmoddefControl::setmoddefControl(Widget parent, Widget *toplevel1, Widget *text_modname, char *name, char display[], messagelabel **label1): BasicComponent (name){

  defobj[0]=0;
  label = label1; // set pointer to the correct label
  text_mod = text_modname;
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  toplevel = toplevel1;


  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&setmoddefControl::setmoddefCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&setmoddefControl::helpCallback,
		(XtPointer) this);
} // end of constructor





//
//
// gennewsysControl
//
//

void gennewsysControl::gennewsys()
{
  Widget dialog;
  XmString t, tempstr;
  Arg args[5];
  int n=0;

  t = XmStringCreateLtoR ("Enter a name for your new model", "My_TAG");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Name for the new system");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}



void gennewsysControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;
  XmString t, tempstr, yes, no;
  Arg args[5];
  int n=0;

  Widget dialog;
  Widget Helpdialog;
  Widget Descdialog;
  char makecom[50];
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;



  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "gennewsys_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on generating a new model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);


  if(getnumtokens(value) > 1){
    dialog = PostLongDialog(_w, "Problem with model name", "ERROR: A model-name can only be one word long.\nPlease try again.");
    XtFree(value);
    return;
  }

  getnthblock(value, modname, 100, 1);
  XtFree(value);

  if(strcmp(modname, "") == 0){
    dialog = PostLongDialog(_w, "Problem with model name", "You must enter a model name.");
    return;
  }


  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if(modelexists(modname)){
    dialog = PostLongDialog(_w, "Problem with model", "There appears to be a model by this name.\nTry another name.");
    return;
  }

  snprintf(systemcom, 100, "%s src/lib/P_gennewsys", makecom);
  //ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"gennewsys\" - see terminal for details.");
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_gennewsys %s", modname);
  //  ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "New system creation failed - see terminal for details.");
    return;
  }

  // Are we adding the new model to the user interface?

  t = XmStringCreateLtoR ("The new model has been created. Please add a\nbrief description (max 40 characters) and\npress yes to add to the interface.\n\nBrief model description:", "My_TAG");

  yes = XmStringCreateLocalized ("Yes");
  no = XmStringCreateLocalized ("Cancel");
  n=0;
  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;
  XtSetArg(args[n], XmNokLabelString, yes);n++;
  XtSetArg(args[n], XmNcancelLabelString, no);n++;

  Descdialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Description of the new system");
  XtVaSetValues(Descdialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(yes);
  XmStringFree(no);
  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(Descdialog, XmNokCallback, descdialog_callback, (XtPointer) this);
  //  XtAddCallback(Descdialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(Descdialog, XmNcancelCallback, descdialog_callback, (XtPointer) this);
  XtAddCallback(Descdialog, XmNhelpCallback, descdialog_callback, (XtPointer) this);

  XtManageChild (Descdialog);
  XtPopup (XtParent (Descdialog), XtGrabNone);
  XtDestroyWidget(widget);

}

void gennewsysControl::descdlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char *value;
  char systemcom[100];
  int l;
  char vardesc[1000];
  FILE *fd;
  Widget Helpdialog;
  Widget dialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "gennewsys_desc_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "A description of your model", vardesc);
    return;
  case XmCR_CANCEL:
    Helpdialog = PostHelpDialog(XtParent(XtParent (widget)), "Model created", "The model has been created, but\nnot added to the interface.");
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    return;
  default:
    break;
  }

  strncpy(moddesc, value, 99);
  XtFree(value);

  fd = fopen("userfiles/model_types.dat", "a");
  if(fd==NULL){
    dialog = PostLongDialog(_w, "ERROR", "Couldn't open the file \"userfiles/model_types.dat\"\nThe new system has been created, but you will need to\nadd it to this file by hand if you want to access it\nvia the user interface.");
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget);
    return;
  }
  fprintf(fd, "%s (%s)\n", modname, moddesc);
  fclose(fd);

  snprintf(systemcom, 100, "New system %s created.", modname);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  XtPopdown (XtParent (widget));
  XtDestroyWidget(widget);
  return;
}


void gennewsysControl::gennewsysCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  gennewsysControl *obj = (gennewsysControl *) client_data;
  obj->gennewsys();
};

void gennewsysControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  gennewsysControl *obj = (gennewsysControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void gennewsysControl::descdialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  gennewsysControl *obj = (gennewsysControl *) client_data;
  obj->descdlg(widget, client_data, call_data);
};


void gennewsysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  gennewsysControl *obj = (gennewsysControl *) client_data;
  obj->help();
};

void gennewsysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "gennewsys_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on generating new models", vardesc);

}


//constructor

gennewsysControl::gennewsysControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&gennewsysControl::gennewsysCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&gennewsysControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// cpsysControl
//
//

void cpsysControl::cpsys()
{
  Widget dialog;
  XmString t, titlestring;
  XmString *str;
  Arg args[5];
  int n=0;
  int i;

  int totlines=0;
  char **lines;


  lines = genlinearraydat("userfiles/model_types.dat", &totlines);

  // selection box

  str = (XmString *) XtMalloc (totlines*sizeof (XmString));
  t = XmStringCreateLocalized ("models available");
  for (i = 0; i < totlines; i++)
    str[i] = XmStringCreateLocalized (lines[i]);
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the model to copy");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   totlines,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring);

  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  XtAddCallback(dialog, XmNokCallback, dlg0_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dlg0_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XmStringFree (t);
  freearraydat(lines, totlines);

  while (--i >= 0)
    XmStringFree (str[i]);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);

}

void cpsysControl::dlg0(Widget widget, XtPointer client_data, XtPointer call_data)
{
  Widget dialog, Helpdialog;
  char vardesc[1000];
  int l;
  char *value;
  XmString t, tempstr;
  Arg args[5];
  int n=0;
  char systemcom[200];
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "cpsys_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on copying a model", vardesc);
    return;
  default:
    break;
  }


  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with model name", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }
  getnthblock(value, oldmodname, 100, 1);
  XtFree(value);

  if(!modelexists(oldmodname)){
    dialog = PostLongDialog(_w, "Problem with model", "There does not appear to be a model by this name.\nTry again.");
    return;
  }



  snprintf(systemcom, 200, "You are copying the model %s.\nEnter a name for the copy you want\nto create:", oldmodname);
  t = XmStringCreateLtoR (systemcom, "My_TAG");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("New model name:");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);


  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
  XtDestroyWidget(widget);
}


void cpsysControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;
  XmString t, tempstr, yes, no;
  Arg args[5];
  int n=0;

  Widget dialog;
  Widget Helpdialog;
  Widget Descdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];



  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "cpsys_dialog1_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on copying a model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(getnumtokens(value) > 1){
    dialog = PostLongDialog(_w, "Problem with model name", "ERROR: The model name must consist of a single word.\nTry again.");
    XtFree(value);
    return;
  }
  
  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }

  getnthblock(value, modname, 100, 1);
  XtFree(value); // not needed again

  // model exists

  if(modelexists(modname)){
    dialog = PostLongDialog(_w, "ERROR", "A system by this name already exists. If you\nwant to create another one, either give it a new\nname or delete the old system first.");
    return;
  }

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    XtDestroyWidget(widget);
    return;
  }

  snprintf(systemcom, 100, "%s src/lib/P_cpsys", makecom);
  //ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "ERROR: Problem making \"cpsys\" - see terminal for details.");
    XtDestroyWidget(widget);
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_cpsys %s %s", modname, oldmodname);
  //  ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Copying failed - see terminal for details.");
    XtDestroyWidget(widget);
    return;
  }

  // Are we adding the new model to the user interface?

  t = XmStringCreateLtoR ("The new model has been created. Would you\nlike to add it to the user interface?\n\nIf so, then enter a brief description of the model\n(max 40 characters) and press yes.\n\nBrief model description:", "My_TAG");

  yes = XmStringCreateLocalized ("Yes");
  no = XmStringCreateLocalized ("No");
  n=0;
  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;
  XtSetArg(args[n], XmNokLabelString, yes);n++;
  XtSetArg(args[n], XmNcancelLabelString, no);n++;

  Descdialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("Description of the new system");
  XtVaSetValues(Descdialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(yes);
  XmStringFree(no);
  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(Descdialog, XmNokCallback, descdialog_callback, (XtPointer) this);
  XtAddCallback(Descdialog, XmNcancelCallback, descdialog_callback, (XtPointer) this);
  XtAddCallback(Descdialog, XmNhelpCallback, descdialog_callback, (XtPointer) this);

  XtManageChild (Descdialog);
  XtPopup (XtParent (Descdialog), XtGrabNone);
  XtDestroyWidget(widget);

}


void cpsysControl::descdlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char *value;
  char systemcom[100];
  int l;
  char vardesc[1000];
  FILE *fd;
  Widget Helpdialog;
  Widget dialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "cpsys_desc_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "A description of your model", vardesc);
    return;
  case XmCR_CANCEL:
    Helpdialog = PostHelpDialog(XtParent(XtParent (widget)), "Copy created", "The copy has been created, but\nnot added to the interface.");
    XtPopdown (XtParent (widget));
    XtDestroyWidget (widget); 
    return;
  default:
    break;
  }

  strncpy(moddesc, value, 99);
  XtFree(value);

  fd = fopen("userfiles/model_types.dat", "a");
  if(fd==NULL){
    dialog = PostLongDialog(_w, "ERROR", "Couldn't open the file \"userfiles/model_types.dat\"\nThe new system has been created, but you will need to\nadd it to this file by hand if you want to access it\nvia the user interface.");
    return;
  }
  fprintf(fd, "%s (%s)\n", modname, moddesc);
  fclose(fd);

  snprintf(systemcom, 100, "System %s created.", modname);
  dialog = PostHelpDialog(_w, "operation successful", systemcom);
  telluser(label, systemcom);

  XtDestroyWidget(widget);
  return;
}






void cpsysControl::cpsysCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpsysControl *obj = (cpsysControl *) client_data;
  obj->cpsys();
};

void cpsysControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpsysControl *obj = (cpsysControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void cpsysControl::descdialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpsysControl *obj = (cpsysControl *) client_data;
  obj->descdlg(widget, client_data, call_data);
};

void cpsysControl::dlg0_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  cpsysControl *obj = (cpsysControl *) client_data;
  obj->dlg0(widget, client_data, call_data);
};


void cpsysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  cpsysControl *obj = (cpsysControl *) client_data;
  obj->help();
};

void cpsysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "cpsys_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on copying models", vardesc);

}


//constructor

cpsysControl::cpsysControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&cpsysControl::cpsysCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&cpsysControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// rmsysControl
//
//

void rmsysControl::rmsys()
{
  Widget dialog;
  XmString t, tempstr;
  Arg args[5];
  int n=0;

  t = XmStringCreateLtoR ("Are you QUITE SURE that you want to\ndelete a model?\n\nHave you ARCHIVED the model in case\nyou want to use it later?\n\nIf you are sure, enter the name of the\nmodel you wish to delete:\n", "My_TAG");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("system to delete");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void rmsysControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;

  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];


  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "rmsys_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on deleting a model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(getnumtokens(value) > 1){
    dialog = PostLongDialog(_w, "Problem with model name", "ERROR: The model name must consist of a single word.\nTry again.");
    XtFree(value);
    return;
  }
  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }

  getnthblock(value, modname, 100, 1);
  XtFree(value);
  


  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }


  snprintf(systemcom, 100, "%s src/lib/P_rmsys", makecom);
  //ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"rmsys\" - see terminal for details.");
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_rmsys %s", modname);
  //  ex_cmnd(systemcom, text_comout, toplevel);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Model deletion failed - see terminal for details.");
    return;
  }

  snprintf(systemcom, 100, "Model %s deleted.", modname);
  dialog = PostHelpDialog(_w, "Operation successful", systemcom);
  telluser(label, systemcom);

  XtDestroyWidget(widget);

}

void rmsysControl::rmsysCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  rmsysControl *obj = (rmsysControl *) client_data;
  obj->rmsys();
};

void rmsysControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  rmsysControl *obj = (rmsysControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void rmsysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  rmsysControl *obj = (rmsysControl *) client_data;
  obj->help();
};

void rmsysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "rmsys_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on deleting models", vardesc);

}


//constructor

rmsysControl::rmsysControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&rmsysControl::rmsysCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&rmsysControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// subtomodelControl
//
//

void subtomodelControl::subtomodel()
{
  Widget dialog;
  XmString t, tempstr;
  Arg args[5];
  int n=0;

  t = XmStringCreateLtoR ("Converting a submodel to a model causes shared files\nto be converted to hard copies for use exclusively\nby the model. It means that the model can be updated\nindependently of other models (assuming that the\nsubmodel itself does not lend files to other models.)\n\nThe process can be reversed but needs to be reversed\nby hand. So if you are converting a submodel to a\nmodel, then it is a good idea to copy it first.\n\nEnter the name of the submodel you wish to convert:", "My_TAG");

  XtSetArg (args[n], XmNselectionLabelString, t);n++;
  XtSetArg(args[n], XmNautoUnmanage, False);n++;

  dialog = XmCreatePromptDialog (_w, "prompt", args, n);
  tempstr = XmStringCreateLocalized ("submodel to convert");
  XtVaSetValues(dialog, XmNdialogTitle, tempstr, NULL);

  XmStringFree(t);
  XmStringFree(tempstr);

  //  XtSetSensitive (XmSelectionBoxGetChild (dialog, XmDIALOG_HELP_BUTTON), False);
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);
}

void subtomodelControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[200];
  char *value;
  char vardesc[2000];
  int l;

  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];


  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "subtomodel_dialog_helpbutton", vardesc, 2000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on converting a submodel to a model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);

  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }

  getnthblock(value, modname, 100, 1);
  XtFree(value);

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if(!modelexists(modname)){
    dialog = PostErrorDialog(_w, label, "The model does not seem to exist.\nNothing is being done.");
    return;
  }

  snprintf(systemcom, 200, "%s src/lib/P_subtomodel", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"subtomodel\" - see terminal for details.");
    return;
  }
  snprintf(systemcom, 200, "src/lib/P_subtomodel %s", modname);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Conversion failed - see terminal for details.");
    return;
  }

  snprintf(systemcom, 200, "Submodel %s successfully converted to a model.", modname);
  dialog = PostHelpDialog(_w, "Operation successful", systemcom);
  telluser(label, systemcom);

  XtDestroyWidget(widget);

}

void subtomodelControl::subtomodelCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  subtomodelControl *obj = (subtomodelControl *) client_data;
  obj->subtomodel();
};

void subtomodelControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  subtomodelControl *obj = (subtomodelControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void subtomodelControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  subtomodelControl *obj = (subtomodelControl *) client_data;
  obj->help();
};

void subtomodelControl::help(){
  Widget Helpdialog;
  char vardesc[2000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "subtomodel_button", vardesc, 2000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on converting submodels to models", vardesc);

}


//constructor

subtomodelControl::subtomodelControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&subtomodelControl::subtomodelCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&subtomodelControl::helpCallback,
		(XtPointer) this);
} // end of constructor




//
//
// archivesysControl
//
//

void archivesysControl::archivesys()
{
  Widget dialog;
  //  XmString t, tempstr;
  XmString t, *str, titlestring;
  int i=0;
  Arg args[5];
  int n=0;
  int totlines=0;
  char **lines;

  lines = genlinearraydat("userfiles/model_types.dat", &totlines);

  str = (XmString *) XtMalloc (totlines*sizeof (XmString));
  t = XmStringCreateLocalized ("models available");
  for (i = 0; i < totlines; i++)
    str[i] = XmStringCreateLocalized (lines[i]);
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the model to archive");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   totlines,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring);

  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XmStringFree (t);
  freearraydat(lines, totlines);

  while (--i >= 0)
    XmStringFree (str[i]);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);

}

void archivesysControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;

  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];


  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "archivesys_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on archiving a model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  getnthblock(value, modname, 100, 1);
  XtFree(value);


  if(strcmp(modname, "") == 0){
    dialog = PostLongDialog(_w, "Problem with model name", "You must enter a model name.");
    return;
  }

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    return;
  }

  if(!modelexists(modname)){
    dialog = PostErrorDialog(_w, label, "The model does not seem to exist.\nNothing is being done. Try again.");
    return;
  }


  snprintf(systemcom, 100, "%s src/lib/P_archivemodel", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"archivesys\" - see terminal for details.");
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_archivemodel %s", modname);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Model archiving failed - see terminal for details.");
    return;
  }

  snprintf(systemcom, 100, "Model %s archived.", modname);
  dialog = PostHelpDialog(_w, "Operation successful", systemcom);
  telluser(label, systemcom);

  XtDestroyWidget(widget);

}

void archivesysControl::archivesysCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  archivesysControl *obj = (archivesysControl *) client_data;
  obj->archivesys();
};

void archivesysControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  archivesysControl *obj = (archivesysControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void archivesysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  archivesysControl *obj = (archivesysControl *) client_data;
  obj->help();
};

void archivesysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "archivesys_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on archiving models", vardesc);

}


//constructor

archivesysControl::archivesysControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&archivesysControl::archivesysCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&archivesysControl::helpCallback,
		(XtPointer) this);
} // end of constructor





//
//
// backupControl
//
//

void backupControl::backup()
{
  Widget dialog;
  //  XmString t, tempstr;
  XmString t, *str, titlestring;
  int i=0;
  Arg args[5];
  int n=0;
  int totlines=0;
  char **lines;

  lines = genlinearraydat("userfiles/model_types.dat", &totlines);

  str = (XmString *) XtMalloc (totlines*sizeof (XmString));
  t = XmStringCreateLocalized ("models available");
  for (i = 0; i < totlines; i++)
    str[i] = XmStringCreateLocalized (lines[i]);
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the model to back up");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   totlines,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring);

  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XmStringFree (t);
  freearraydat(lines, totlines);

  while (--i >= 0)
    XmStringFree (str[i]);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);

}

void backupControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  char *tmp;
  char *newmodname;
  int l;
  FILE *fd;

  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];


  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "backup_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on backing up a model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);
  getnthblock(value, modname, 100, 1);
  XtFree(value);

  if(strcmp(modname, "") == 0){
    dialog = PostLongDialog(_w, "Problem with model name", "You must enter a model name.\nTry again.");
    return;
  }

  // check if model exists
  if(!modelexists(modname)){
    dialog = PostErrorDialog(_w, label, "The model does not seem to exist.\nNothing is being done. Try again.");
    return;
  }


  tmp  = timestr();
  newmodname = (char*) malloc(sizeof(char) * (strlen(modname) + strlen(tmp) + 5));
  sprintf(newmodname, "%s%s", modname, tmp);
  free(tmp);


  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    free(newmodname);
    return;
  }


  // copy the model

  snprintf(systemcom, 100, "%s src/lib/P_cpsys", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem copying model - see terminal for details.");
    free(newmodname);
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_cpsys %s %s", newmodname, modname);
  // fprintf(stderr, "command = %s\n", systemcom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Copying failed - see terminal for details.");
    free(newmodname);
    return;
  }

  // add to interface

  fd = fopen("userfiles/model_types.dat", "a");
  if(!fd){
    fprintf(stderr, "WARNING: Couldn't open the file \"userfiles/model_types.dat\" The copy has been created but will not be visible on the interface.");
  }
  fprintf(fd, "%s (backup of %s)\n", newmodname, modname);
  fclose(fd);

  // convert to a submodel if necessary

  snprintf(systemcom, 200, "%s src/lib/P_subtomodel", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"subtomodel\" - see terminal for details.");
    free(newmodname);
    return;
  }
  snprintf(systemcom, 200, "src/lib/P_subtomodel %s", newmodname);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Conversion to submodel failed - see terminal for details.");
    free(newmodname);
    return;
  }

  // archives

  snprintf(systemcom, 100, "%s src/lib/P_archivemodel", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making backup - see terminal for details.");
    free(newmodname);
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_archivemodel %s", newmodname);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Model archiving failed - see terminal for details.");
    free(newmodname);
    return;
  }

  // deletes

  snprintf(systemcom, 100, "%s src/lib/P_rmsys", makecom);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"rmsys\" - see terminal for details.");
    free(newmodname);
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_rmsys %s", newmodname);
  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Model deletion failed - see terminal for details.");
    free(newmodname);
    return;
  }

  snprintf(systemcom, 200, "Backup of %s to %s was successful", modname, newmodname);
  fprintf(stderr, "Backup complete.\n\n");
  dialog = PostHelpDialog(_w, "Operation successful", systemcom);

  free(newmodname);
  XtDestroyWidget(widget);

}

void backupControl::backupCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  backupControl *obj = (backupControl *) client_data;
  obj->backup();
};

void backupControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  backupControl *obj = (backupControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void backupControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  backupControl *obj = (backupControl *) client_data;
  obj->help();
};

void backupControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "backup_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on archiving models", vardesc);

}


//constructor

backupControl::backupControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&backupControl::backupCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&backupControl::helpCallback,
		(XtPointer) this);
} // end of constructor



//
//
// restoresysControl
//
//

void restoresysControl::restoresys()
{
  Widget dialog;
  XmString t, titlestring;
  XmString *str;
  Arg args[5];
  int n=0;
  int i=0;
  char *archs[100];
  int nummodels=0;

  nummodels = getdc("models/archives", ".tar", archs, 100);

  str = (XmString *) XtMalloc (nummodels*sizeof (XmString));
  t = XmStringCreateLtoR("Archives available. Note that an archive cannot be\nrestored if a model of this name already exists. ", "My_TAG");
  for (i = 0; i < nummodels; i++)
    str[i] = XmStringCreateLocalized (archs[i]);
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateSelectionDialog (_w, "selection", args, n);
  titlestring = XmStringCreateLocalized("Select the archive");
  XtVaSetValues (dialog,
		 XmNlistLabelString, t,
		 XmNlistItems,       str,
		 XmNlistItemCount,   nummodels,
		 XmNdialogTitle,     titlestring,
		 XmNmustMatch,       False,
		 NULL);

  XmStringFree(titlestring);

  XtUnmanageChild (XmSelectionBoxGetChild (dialog, XmDIALOG_APPLY_BUTTON));
  XtAddCallback(dialog, XmNokCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNnoMatchCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNhelpCallback, dialog_callback, (XtPointer) this);
  XtAddCallback(dialog, XmNcancelCallback, DestWidget, (XtPointer) this);

  XmStringFree (t);
  free_datarray(archs, nummodels);

  while (--i >= 0)
    XmStringFree (str[i]);

  XtManageChild (dialog);
  XtPopup (XtParent (dialog), XtGrabNone);

}

void restoresysControl::dlg(Widget widget, XtPointer client_data, XtPointer call_data)
{
  char systemcom[100];
  char *value;
  char vardesc[1000];
  int l;
  char archname[100];
  char *mname;

  Widget dialog;
  Widget Helpdialog;
  XmSelectionBoxCallbackStruct *cbs = 
    (XmSelectionBoxCallbackStruct *) call_data;
  char makecom[50];


  switch (cbs->reason) {
  case XmCR_HELP:
    l=getdesc("helpfiles/GUI_WID_HELP", "restoresys_dialog_helpbutton", vardesc, 1000);
    if(l<0){
      Helpdialog = PostErrorDialog(widget, label, "ERROR in getting help: see terminal for details");
      return;
    }
    Helpdialog = PostHelpDialog(widget, "Help on restoring an archived model", vardesc);
    return;
  default:
    break;
  }

  XmStringGetLtoR (cbs->value, XmFONTLIST_DEFAULT_TAG, &value);


  if(isonlyspace(value)){
    dialog = PostLongDialog(_w, "Problem with what you've typed", "You must enter something.\nTry again.");
    XtFree(value);
    return;
  }

  getnthblock(value, archname, 100, 1);
  XtFree(value);



  if(endswith(archname, ".tar.gz"))
    mname = strchp(archname, "", ".tar.gz");
  else if(endswith(archname, ".tgz"))
    mname = strchp(archname, "", ".tgz");
  else if(endswith(archname, ".tar"))
    mname = strchp(archname, "", ".tar");
  else 
    mname = strdup(archname);

  if(!archiveexists(mname)){
    dialog = PostErrorDialog(_w, label, "The archive does not seem to exist.\nNothing is being done. Try again.");
    free(mname);
    return;
  }

  if(modelexists(mname)){
    dialog = PostErrorDialog(_w, label, "The model already seems to exist.\nIt cannot be restored.");
    free(mname);
    return;
  }

  if(getoption("userfiles/gen_options", "MAKE", makecom, 50)!=0){
    dialog = PostErrorDialog(_w, label, "ERROR with getting an option: see terminal for details");
    free(mname);
    return;
  }

  snprintf(systemcom, 100, "%s src/lib/P_restoremodel", makecom);

  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Problem making \"src/lib/P_restoremodel\" - see terminal for details.");
    free(mname);
    return;
  }
  snprintf(systemcom, 100, "src/lib/P_restoremodel %s", mname);

  l=system(systemcom);
  if(l!=0){
    dialog = PostErrorDialog(_w, label, "Model restoration failed - see terminal for details.");
    free(mname);
    return;
  }

  snprintf(systemcom, 100, "Model %s restored.", mname);
  dialog = PostHelpDialog(_w, "Operation successful", systemcom);
  telluser(label, systemcom);
  free(mname);
  XtDestroyWidget(widget);
}

void restoresysControl::restoresysCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  restoresysControl *obj = (restoresysControl *) client_data;
  obj->restoresys();
};

void restoresysControl::dialog_callback(Widget widget, XtPointer client_data, XtPointer call_data){
  restoresysControl *obj = (restoresysControl *) client_data;
  obj->dlg(widget, client_data, call_data);
};

void restoresysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  restoresysControl *obj = (restoresysControl *) client_data;
  obj->help();
};

void restoresysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "restoresys_button", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), label, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on restoring archived models", vardesc);

}


//constructor

restoresysControl::restoresysControl(Widget parent, Widget *toplevel1, Widget *text_filename1, char *name, char display[], messagelabel **label1): BasicComponent (name){
  label = label1; // set pointer to the correct label
  XmString xmstr;
  xmstr = XmStringCreateSimple(display);
  text_filename = text_filename1;
  toplevel = toplevel1;
  moddesc[0] = 0;
  modname[0] = 0;

  // Create the widget

  _w = XtVaCreateWidget
    (_name, 
     xmPushButtonWidgetClass, parent,
     XmNlabelString, xmstr,
     XmNmarginWidth, 10,
     XmNmarginHeight, 6,
     NULL);

  XmStringFree (xmstr); 

  XtAddCallback(_w, XmNactivateCallback,
		&restoresysControl::restoresysCallback,
		(XtPointer) this);

  XtAddCallback(_w, XmNhelpCallback,
		&restoresysControl::helpCallback,
		(XtPointer) this);
} // end of constructor



//
//
// constructor
//
//

editsysControl::editsysControl(Widget parent, Widget *toplevel1, Widget *text_modname, Widget *mfsdlg1, Widget *datfsdlg1, char *name, char display[], char *funcfile, char *descfile, int *paramschanged, XtAppContext *app): BasicComponent (name){

  XmString xmstr_fname;
  XmString xmstr_batch;
  char DEF_MODEL[100];
  char DEF_MODULE_FILE[100];
  char DEF_MODELDAT_FILE[100];
  char *tmp;

  toplevel=toplevel1;
  app1 = app;
  text_mod = text_modname;
  mfsdlg = mfsdlg1;
  datfsdlg = datfsdlg1;
  // load the parameters

  getoption("userfiles/gui_options", "DEF_MODEL", DEF_MODEL, 100);
  tmp = getmoddef("userfiles/model_defaults", DEF_MODEL, "module");
  strncpy(DEF_MODULE_FILE, tmp, 99);
  free(tmp);
  tmp = getmoddef("userfiles/model_defaults", DEF_MODEL, "modeldesc");
  strncpy(DEF_MODELDAT_FILE, tmp, 99);
  free(tmp);


  _w = XtVaCreateWidget (_name,
			 xmFormWidgetClass, parent,
			 NULL);

  //   XtVaSetValues (_w,
  // 		 XmNfractionBase, 100,
  // 		 NULL);

  separator1 = XtVaCreateManagedWidget 
    ("separator1",
     xmSeparatorWidgetClass, _w,
     XmNleftAttachment, XmATTACH_FORM,
     XmNrightAttachment, XmATTACH_FORM,
     XmNtopPosition, 0,
     XmNtopAttachment, XmATTACH_POSITION,
     //      XmNtopWidget, _w,
     //      XmNtopAttachment, XmATTACH_WIDGET,
     XmNbottomAttachment, XmATTACH_NONE,
     NULL);

  message_frame = XtVaCreateManagedWidget
    ("message_frame",
     xmFrameWidgetClass, _w,
     XmNleftAttachment, XmATTACH_FORM,
     XmNrightAttachment, XmATTACH_FORM,
     XmNtopPosition, 80,
     XmNtopAttachment, XmATTACH_POSITION,
     XmNbottomPosition, 95,
     XmNbottomAttachment, XmATTACH_POSITION,
     NULL);

  XtAddCallback (message_frame, XmNhelpCallback, genhelpCallback, message_frame);

  messages = new messagelabel(message_frame, "messages", "(messages)");
  messages->manage();


  rc1 = XtVaCreateManagedWidget 
    ("rc1",
     xmRowColumnWidgetClass, _w,
     XmNorientation, XmHORIZONTAL,
     XmNleftAttachment, XmATTACH_FORM,
     XmNrightPosition, 80,
     XmNrightAttachment, XmATTACH_POSITION,
     XmNtopPosition, 4,
     XmNtopAttachment, XmATTACH_POSITION,
     XmNbottomPosition, 31,
     XmNbottomAttachment, XmATTACH_POSITION,
     NULL);

  //
  // The module file area
  //

  frame_module = XtVaCreateManagedWidget 
    ("frame_module",
     xmFrameWidgetClass, rc1,
     XmNshadowType,  XmSHADOW_ETCHED_IN,
     NULL);

  rc2 = XtVaCreateManagedWidget 
    ("rc2",
     xmRowColumnWidgetClass, frame_module,
     XmNorientation,  XmVERTICAL,
     NULL);

  rc21 = XtVaCreateManagedWidget 
    ("rc21",
     xmRowColumnWidgetClass, rc2,
     XmNorientation,  XmHORIZONTAL,
     NULL);

  rc22 = XtVaCreateManagedWidget 
    ("rc22",
     xmRowColumnWidgetClass, rc2,
     XmNorientation,  XmHORIZONTAL,
     NULL);

  xmstr_fname = XmStringCreateSimple("Module file:");
  label_module = XtVaCreateManagedWidget
    ("label_module",
     xmLabelWidgetClass, rc21,
     XmNlabelString, xmstr_fname,
     NULL);

  XmStringFree (xmstr_fname);

  text_module = XtVaCreateManagedWidget
    ("text_module",
     xmTextWidgetClass, rc22,
     XmNeditable, False,
     XmNcolumns, 40,
     NULL);
  // 

  XmTextSetString(text_module, DEF_MODULE_FILE); // default
  XmTextSetCursorPosition (text_module, strlen (DEF_MODULE_FILE));

  button_selectmfile = new selectmfileControl(rc21, &text_module, mfsdlg, "button_selectmfile", "choose", &messages);
  button_selectmfile->manage();

  button_viewmfile = new viewmfileControl(rc21, &text_module, "button_viewmfile", "edit", &messages, app1);
  button_viewmfile->manage();


  button_parsemfile = new parsemfileControl(rc21, &text_module, "button_parsemfile", "parse", &messages, app1);
  button_parsemfile->manage();

  button_scanmfile = new scanmfileControl(rc21, &text_module, "button_scanmfile", "scan", &messages, app1);
  button_scanmfile->manage();


  //
  // The modeldescs area
  //


  frame_datfile = XtVaCreateManagedWidget 
    ("frame_datfile",
     xmFrameWidgetClass, rc1,
     XmNshadowType,  XmSHADOW_ETCHED_IN,
     NULL);

  rc3 = XtVaCreateManagedWidget 
    ("rc3",
     xmRowColumnWidgetClass, frame_datfile,
     XmNorientation,  XmVERTICAL,
     NULL);

  rc31 = XtVaCreateManagedWidget 
    ("rc31",
     xmRowColumnWidgetClass, rc3,
     XmNorientation,  XmHORIZONTAL,
     NULL);

  rc32 = XtVaCreateManagedWidget 
    ("rc32",
     xmRowColumnWidgetClass, rc3,
     XmNorientation,  XmHORIZONTAL,
     NULL);

  xmstr_fname = XmStringCreateSimple("Descriptor:");
  label_module = XtVaCreateManagedWidget
    ("label_module",
     xmLabelWidgetClass, rc31,
     XmNlabelString, xmstr_fname,
     NULL);

  XmStringFree (xmstr_fname);

  text_datfile = XtVaCreateManagedWidget
    ("text_datfile",
     xmTextWidgetClass, rc32,
     XmNeditable, False,
     XmNcolumns, 40,
     NULL);
  // 


  XmTextSetString(text_datfile, DEF_MODELDAT_FILE); // default
  XmTextSetCursorPosition (text_datfile, strlen (DEF_MODELDAT_FILE));

  button_selectdatfile = new selectdatfileControl(rc31, &text_datfile, datfsdlg, "button_selectmdatfile", "choose", &messages);
  button_selectdatfile->manage();

  button_viewdatfile = new viewdatfileControl(rc31, &text_datfile, "button_viewdatfile", "edit", &messages);
  button_viewdatfile->manage();

  button_scanmodel = new scanmodelControl(rc31, toplevel, &text_module, "button_scanmodel", "scan model", &messages, app1);
  button_scanmodel->manage();

  button_cpdescfile = new cpdescfileControl(rc31, &text_datfile, "button_cpdescfile", "copy", &messages);
  button_cpdescfile->manage();


  //
  // The batch editing area
  //

  rc1a = XtVaCreateManagedWidget 
    ("rc1a",
     xmRowColumnWidgetClass, _w,
     XmNorientation, XmHORIZONTAL,
     XmNleftAttachment, XmATTACH_FORM,
     XmNrightPosition, 80,
     XmNrightAttachment, XmATTACH_POSITION,
     XmNtopPosition, 33,
     XmNtopAttachment, XmATTACH_POSITION,
     XmNbottomPosition, 65,
     XmNbottomAttachment, XmATTACH_POSITION,
     NULL);

  frame_batch = XtVaCreateManagedWidget 
    ("frame_batch",
     xmFrameWidgetClass, rc1a,
     XmNshadowType,  XmSHADOW_ETCHED_IN,
     NULL);

  rc5 = XtVaCreateManagedWidget 
    ("rc5",
     xmRowColumnWidgetClass, frame_batch,
     XmNorientation,  XmVERTICAL,
     NULL);

  rc51 = XtVaCreateManagedWidget 
    ("rc51",
     xmRowColumnWidgetClass, rc5,
     XmNorientation,  XmHORIZONTAL,
     NULL);

  rc52 = XtVaCreateManagedWidget 
    ("rc52",
     xmRowColumnWidgetClass, rc5,
     XmNorientation,  XmHORIZONTAL,
     NULL);
  rc53 = XtVaCreateManagedWidget 
    ("rc53",
     xmRowColumnWidgetClass, rc5,
     XmNorientation,  XmHORIZONTAL,
     NULL);


  xmstr_batch = XmStringCreateSimple("Edit current model");
  label_batch = XtVaCreateManagedWidget
    ("label_batch",
     xmLabelWidgetClass, rc51,
     XmNlabelString, xmstr_batch,
     NULL);
  XmStringFree (xmstr_batch);

  button_changevar = new changevarControl(rc52, text_mod, "button_changevar", "change variable name", &messages, app1);
  button_changevar->manage();

  button_setmoddef = new setmoddefControl(rc52, toplevel1, text_mod, "button_setmoddef", "set model defaults", &messages);
  button_setmoddef->manage();

  button_editinfofile = new editinfofileControl(rc53, text_mod, "button_editinfofile", "edit info file", &messages, app1);
  button_editinfofile->manage();



  //
  // The full model editing areas
  //

  rc4 = XtVaCreateManagedWidget 
    ("rc4",
     xmRowColumnWidgetClass, _w,
     XmNorientation, XmVERTICAL,
     XmNleftPosition, 82,
     XmNleftAttachment, XmATTACH_POSITION,
     XmNrightAttachment, XmATTACH_FORM,
     XmNtopPosition, 4,
     XmNtopAttachment, XmATTACH_POSITION,
     XmNbottomPosition, 80,
     XmNbottomAttachment, XmATTACH_POSITION,
     NULL);

  button_gennewsys = new gennewsysControl(rc4, toplevel1, &text_datfile, "button_gennewsys", "create new model", &messages);
  button_gennewsys->manage();

  button_cpsys = new cpsysControl(rc4, toplevel1, &text_datfile, "button_cpsys", "copy a model", &messages);
  button_cpsys->manage();

  button_archivesys = new archivesysControl(rc4, toplevel1, &text_datfile, "button_archivesys", "archive a model", &messages);
  button_archivesys->manage();

  button_backup = new backupControl(rc4, toplevel1, &text_datfile, "button_backup", "back up a model", &messages);
  button_backup->manage();

  button_subtomodel = new subtomodelControl(rc4, toplevel1, &text_datfile, "button_subtomodel", "submodel to model", &messages);
  button_subtomodel->manage();


  button_rmsys = new rmsysControl(rc4, toplevel1, &text_datfile, "button_rmsys", "delete a model", &messages);
  button_rmsys->manage();


  button_restoresys = new restoresysControl(rc4, toplevel1, &text_datfile, "button_restoresys", "restore a model", &messages);
  button_restoresys->manage();



  XtAddCallback (frame_module, XmNhelpCallback, genhelpCallback, frame_module);
  XtAddCallback (frame_batch, XmNhelpCallback, genhelpCallback, frame_batch);
  XtAddCallback (frame_datfile, XmNhelpCallback, genhelpCallback, frame_datfile);
  //   XtAddCallback (text_module, XmNhelpCallback, genhelpCallback, text_module);



  XtAddCallback(_w, XmNhelpCallback,
		&editsysControl::helpCallback,
		(XtPointer) this);

}

void editsysControl::helpCallback(Widget, XtPointer client_data, XtPointer){
  editsysControl *obj = (editsysControl *) client_data;
  obj->help();
};

void editsysControl::help(){
  Widget Helpdialog;
  char vardesc[1000];
  int l;

  l=getdesc("helpfiles/GUI_WID_HELP", "editsys_area", vardesc, 1000);
  if(l<0){
    Helpdialog = PostErrorDialog(XtParent(_w), &messages, "ERROR in getting help: see terminal for details");
    return;
  }
  Helpdialog = PostHelpDialog(_w, "Help on the model editing interface", vardesc);

}


void editsysControl::genhelpCallback(Widget widget, XtPointer client_data, XtPointer call_data){
  editsysControl *obj = (editsysControl *) client_data;
  obj->genhelp(widget, client_data, call_data);
};


void editsysControl::genhelp(Widget widget, XtPointer client_data, XtPointer call_data){
  Widget Helpdialog;
  char vardesc[3000];
  char title[100];
  int l;

  Widget *tempw = (Widget *) client_data;

  if (!XtName(*tempw)){
    fprintf(stderr, "no widget name\n");
    return;
  }

  // what is help being requested for?
  if(strcmp(XtName(*tempw), "text_name") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "param_text_name", vardesc, 3000);
    strcpy(title, "Parameter name"); // safe
  }
  else if(strcmp(XtName(*tempw), "frame_module") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "module_area", vardesc, 3000);
    strcpy(title, "Viewing and editing module files"); // safe
  }
  else if(strcmp(XtName(*tempw), "frame_datfile") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "datfile_area", vardesc, 3000);
    strcpy(title, "Viewing and editing the model descriptor file"); // safe
  }
  else if(strcmp(XtName(*tempw), "message_frame") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "paramvar_label_area", vardesc, 3000);
    strcpy(title, "Brief messages"); // safe
  }
  else if(strcmp(XtName(*tempw), "message_frame") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "paramvar_label_area", vardesc, 3000);
    strcpy(title, "Brief messages"); // safe
  }
  else if(strcmp(XtName(*tempw), "frame_batch") == 0){
    l=getdesc("helpfiles/GUI_WID_HELP", "frame_batch", vardesc, 3000);
    strcpy(title, "Miscellaneous model editing stuff"); // safe
  }

  if(l<0){
    fprintf(stderr, "No help could be found for this area.\n");
    return;
  }

  Helpdialog = PostHelpDialog(widget, title, vardesc);

}



