/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"

int inputautoregdat(char *infile, int *numchangeparams, char *chosen_param[], int maxpars, double *time_step, double *step_vals, double *init_vals, double *min_val, double *max_val, int *hyst, int *nummorphs, char *morphs[], double morphvals[], int maxmorphs, char *paramfilename)
{
  int i, j, len;
  int numstepvals=0;
  int numinitvals=0;
  FILE *fd;
  int maxv = 20;
  char *vars[20];
  int lim_line = 1000;
  char oneline[1000];
  char *fieldval;
  char *mphs[30];
  int tempnummorphs;
  char *tmp;

  // memory allocated for chosen_params in this routine is freed in autoreg.cc


  fd = fopen(infile, "r");
  if (fd == NULL){
    fprintf(stderr, "ERROR in inputautoregdat in \"inputautoregdat.c\": File %s could not be opened for reading.\n", infile);
    return -1; //exit: datafile not found
  }
  // defaults

  *numchangeparams = 0;
  //  chosen_param[0] = strdup("P_a");
  *time_step = 10000.0;
  *step_vals = 0.0;
  *init_vals = 0.0; 
  *min_val = 0.0;
  *max_val = 0.0;
  *nummorphs = 0;
  *hyst = 0;


  if(fd == NULL){
    fprintf(stderr, "ERROR in routine \"inputautoregdat\". File: %s could not be opened...\n", infile);
    return -1;
  }
  while((len= getline(fd, oneline, lim_line)) > 0){
    if(!(iscomsep(oneline))){
      if(strstr(oneline, "chosen_param:")){
	fieldval = getrst("chosen_param:", oneline);
        if(isonlyspace(fieldval)){
	  *numchangeparams = 0;
        }
        else{
          *numchangeparams = chemgets(fieldval, vars, maxv);
	  if(*numchangeparams > maxpars){
	    fprintf(stderr, "WARNING in routine \"inputautoregdat\". Too many parameters to change. Only %d being changed\n", maxpars);
	    *numchangeparams = maxpars;
	  }
        }
	free(fieldval);
        for(i=0;i<*numchangeparams;i++){
          chosen_param[i] = strdup(vars[i]);
        }
      }
      else if(strstr(oneline, "time_step:")){
	fieldval = getrst("time_step:", oneline);
        if(isonlyspace(fieldval)){
	  fprintf(stderr, "WARNING in routine \"inputautoregdat\". Using default time step of 10000.\n");
	}
	else if(isnumber(fieldval)){
          *time_step = atof(fieldval);
	}
	else{
	  tmp = getcvalfromfile(paramfilename, fieldval);
	  if(isonlyspace(tmp)){ 
	    fprintf(stderr, "ERROR in \"inputautoregdat\": A value for parameter %s could not be found in file %s. Exiting...\n", fieldval, paramfilename); 
	    free(fieldval);free(tmp);
	    return -1; 
	  } 
	  else{
	    *time_step = atof(tmp);
	    free(tmp);
	  }
        }
	free(fieldval);
        if(fabs(*time_step) < 0.00001){
          fprintf(stderr, "ERROR in routine \"inputautoregdat\". Time step is too small.\n");
          return -1;
        }
      }
      //
      // parameter values to reset
      //
      else if(strstr(oneline, "reset:")){
	fieldval = getrst("reset:", oneline);

        if(isonlyspace(fieldval))
          tempnummorphs=0;
        else{
          tempnummorphs = chemgets(fieldval, mphs, 3*maxmorphs);
          if(tempnummorphs%3 != 0){
	    fprintf(stderr, "ERROR in routine \"inputautoregdat\". Morphs incorrect.\n");
	    return -1;
          }
          *nummorphs = tempnummorphs/3;
          j=0;
          for(i=0;i<tempnummorphs;i++){
	    morphs[j] = strdup(mphs[i]);i++; // memory freed in autoreg.cc
	    morphvals[2*j] = atof(mphs[i]);i++;
	    morphvals[2*j+1] = atof(mphs[i]);
	    j++;
          }
	}
	free(fieldval);
      }

      else if(strstr(oneline, "step_vals:")){
	fieldval = getrst("step_vals:", oneline);
        if(!isonlyspace(fieldval)){
          numstepvals = chemgets(fieldval, vars, maxv);
        }
	free(fieldval);
	if(numstepvals < maxpars){
          for(i=0;i<numstepvals;i++){
            *(step_vals+i) = atof(vars[i]);
          }
	}
      }
      else if(strstr(oneline, "init_vals:")){
	fieldval = getrst("init_vals:", oneline);
        if(!isonlyspace(fieldval)){
          numinitvals = chemgets(fieldval, vars, maxv);
        }
	free(fieldval);
	if(numinitvals < maxpars){
          for(i=0;i<numinitvals;i++){
            *(init_vals + i) = atof(vars[i]);
          }
	}
      }
      else if(strstr(oneline, "min_val:")){
	fieldval = getrst("min_val:", oneline);
        if(!isonlyspace(fieldval)){
          *min_val = atof(fieldval);
	}
	free(fieldval);
      }
      else if(strstr(oneline, "max_val:")){
	fieldval = getrst("max_val:", oneline);
        if(!isonlyspace(fieldval)){
          *max_val = atof(fieldval);
	}
	free(fieldval);
      }
      else if(strstr(oneline, "hyst:")){
	fieldval = getrst("hyst:", oneline);
	*hyst = atoi(fieldval);
	free(fieldval);
      }
    }
  }
  if(*numchangeparams!=0 && numstepvals != *numchangeparams){
    fprintf(stderr, "ERROR in routine \"inputautoregdat\". Must be the same number of step_vals as changeparams.\n");
    return -1;
  }
  if(*numchangeparams!=0 && numinitvals != *numchangeparams){
    fprintf(stderr, "ERROR in routine \"inputautoregdat\". Must be the same number of init_vals as changeparams.\n");
    return -1;
  }
  if(numstepvals == 0){
    fprintf(stderr, "ERROR in routine \"inputautoregdat\". \"step_vals:\" must not be empty.\n");
    return -1;
  }
  if(*numchangeparams==0 && fabs(*max_val) > 0.001){
    fprintf(stderr, "WARNING in \"inputautoregdat\": No parameter being changed. \"max_val:\" being ignored.\n");
  }
  if(*numchangeparams==0 && fabs(*min_val) > 0.001){
    fprintf(stderr, "WARNING in \"inputautoregdat\": No parameter being changed. \"min_val:\" being ignored.\n");
  }
  return 0;
}

