/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include "common/nrsimp.h"

int inputdat(int seg, double ***Pvals, char *infile, int *numchangeparams, char *chosen_param[], int maxpars, int *fixedstep, double *time_step, int *nummorphs, char *morphs[], double morphvals[], int maxmorphs, int *ktot, int *loopnum, int *wantsteady, int *perturb, int *numsegs)
{
  int i, k, len, fixstp=0, numv;
  double timstp;
  FILE *fd;
  int maxv = 20;
  char *pname;
  char *vars[20];
  char *mphs[3*maxmorphs];
  int tempnummorphs;
  int lim_line = 1000;
  char line[1000];
  char oneline[1000];
  char *rline;
  char block[10000];
  double tempval = 0.0;
  int block_pos;
  fd = fopen(infile, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in routine \"inputdat\". File: %s could not be opened...\n", infile);
    return -1;
  }

  *numsegs=0;
  // how many segments?
  while((len= getline(fd, oneline, lim_line)) > 0){
    if(!(iscom(oneline)) && strstr(oneline, "******") != NULL){
      (*numsegs)++;
    }
  }
  (*numsegs)++;
  (*numsegs)/=2;

  fclose(fd);
  fd = fopen(infile, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in routine \"inputdat\". File: %s could not be opened...\n", infile);
    return -1;
  }

  if(seg > *numsegs-1){
    fprintf(stderr, "ERROR in routine \"inputdat\". There is no segment number %d in file \"%s\". Now exiting.\n", seg+1, infile);
    fclose(fd);
    return -1;
  }

  // skip initial segments

  for(i=0;i<seg;i++){
    len = getblock(fd, block, "******", 10000);
    while((len= getline(fd, oneline, lim_line)) > 0 && strstr(oneline, "******") == NULL){}
  }

  len = getblock(fd, block, "******", 10000);
  block_pos=0;
  *numchangeparams = 0; // default
  while(block_pos<len){
    getlinefromblock(&block_pos, block, line, lim_line);
    if(iscomsep(line)){}
    else if(strstr(line, "chosen_param:")){
      rline = getrst("chosen_param:", line);
      if(strcmp(rline, "") == 0){
        fprintf(stderr, "WARNING in routine \"inputdat\". no parameters to change.\n");
	*numchangeparams = 0; // this is allowable - a steady simulation
      }
      else{
        *numchangeparams = chemgets(rline, vars, maxv);
        if(*numchangeparams > maxpars){
          fprintf(stderr, "ERROR in routine \"inputdat\". Too many parameters to change.\n");
	  free(rline);
	  return -1;
        }
      }
      free(rline);
      for(i=0;i<*numchangeparams;i++){
        chosen_param[i] = strdup(vars[i]); // freed in sys.cc
      }
    }
    else if(strstr(line, "time_step:")){
    //
    // time step
    //
      rline = getrst("time_step:", line);
      if(strcmp(rline, "") == 0){
        fixstp = 0;
        timstp = 1.0;
      }
      else if(strcmp(rline, "timestamp") == 0){
        fixstp = 2;
        timstp = 1.0;
      }
      else{
        fixstp = 1;
        timstp = atof(rline);
      }
      if(fabs(timstp) < 0.00001){
        fprintf(stderr, "ERROR in routine \"inputdat\". Time step is too small.\n");
	free(rline);
        return -1;
      }
      else{
        if(fixstp == 0 || fixstp == 2)
	  *fixedstep = 0;
        else
	  *fixedstep = 1;
        *time_step = timstp;
      }
      free(rline);
    }

    //
    // parameter values to reset
    //
    else if(strstr(line, "reset:") != NULL){
      rline = getrst("reset:", line);

      if(isonlyspace(rline))
        tempnummorphs=0;
      else{
        tempnummorphs = chemgets(rline, mphs, 3*maxmorphs);
        if(tempnummorphs%3 != 0){
	  fprintf(stderr, "ERROR in routine \"inputdat\". Morphs incorrect.\n");
	  return -1;
	}
        *nummorphs = tempnummorphs/3;
        k=0;
        for(i=0;i<tempnummorphs;i++){
	  morphs[k] = strdup(mphs[i]);i++; // freed in sys.cc
	  morphvals[2*k] = atof(mphs[i]);i++;
	  morphvals[2*k+1] = atof(mphs[i]);
	  k++;
	}

      }
      free(rline);
    }
    //
    // steady
    //
    else if(strstr(line, "steady:")){
      pname = getname("steady:", line);
      if(strcmp(pname, "yes")==0)
	*wantsteady=100000; // default
      else if(strcmp(pname, "none")==0 || strcmp(pname, "no")==0)
	*wantsteady=-1;
      else{
	// negative numbers mean you steady without first morphing.
	*wantsteady = atoi(pname);
      }
      free(pname);
    }
    //
    // random perturbations?
    //
    else if(strstr(line, "perturb:")){
      pname = getname("perturb:", line);
      if(strcmp(pname, "no")==0 || strcmp(pname, "0") == 0)
        *perturb=0; // default
      else if(strcmp(pname, "yes")==0 || strcmp(pname, "1") == 0)
	*perturb=1;
      else{
	fprintf(stderr, "WARNING in routine \"inputdat\". value of perturb = %s value not recognised. Should be \"yes\" or \"no\"\n", pname);
	*perturb=0;
      }
      free(pname);
    }

    //
    // do we want to loop the file?
    //
    else if(strstr(line, "loop:")){
      rline = getrst("loop:", line);

      if(strcmp(rline, "")==0)
        *loopnum=1;
      else{
	*loopnum = atoi(rline);
	if(*loopnum <= 0){
	  fprintf(stderr, "WARNING: you can't loop \"%s\" times, resetting to 1\n", oneline);
	  *loopnum=1;
	}
	if(*loopnum > 1000)
	  fprintf(stderr, "WARNING: looping %d times - this could take a while.\n", *loopnum);
      }
      free(rline);
    }

  }
  k=0;
  //
  // first parse
  //

  while((len=getline(fd, oneline, lim_line)) > 0 && strstr(oneline, "******") == NULL){
    if(!(iscomsep(oneline))){
      k++;
      numv = chemgets(oneline, vars, maxv);
      if(fixstp == 0 || fixstp == 2){
	if (numv != *numchangeparams + 1){
	  fprintf(stderr, "numv = %d\tnumchangeparams = %d\n", numv, *numchangeparams);
	  fprintf(stderr, "ERROR in routine \"inputdat\". Variable time step, but incorrect data given on line number %d\n", k);
	  return -1;
	}
      }
      else{
	if(numv != *numchangeparams){
	  fprintf(stderr, "ERROR in routine \"inputdat\". Fixed timestep, but incorrect data on line number %d\n", k);
	  return -1;
	}
      }
    }
  }
  *ktot = k;
  //  fprintf(stderr, "ktot = %d\n", k);
  *Pvals = dmatrix(0, k-1 , 0, numv-1);
  //
  // second parse
  //
  k=0;
  rewind(fd);
  getblock(fd, block, "******", 10000);
  while((len= getline(fd, oneline, lim_line)) > 0 && strstr(oneline, "******") == NULL){
    if(!(iscomsep(oneline))){
      numv = chemgets(oneline, vars, maxv);
      //      printf("k=%d\n", k);
      for(i=0;i<numv;i++){
	if(fixstp == 2 && i==0){
	  if(k==0){
	    tempval = atof(vars[i]);
	    (*Pvals)[k][i] = atof(vars[i]);
	    //	    fprintf(stderr, "k=%d, i=%d, pvals = %.4f\n", k, i, (*Pvals)[k][i]);
	  }
	  else{
	    (*Pvals)[k][i] = atof(vars[i]) - tempval;
	    tempval = atof(vars[i]);
	    //	    fprintf(stderr, "k=%d, i=%d, pvals = %.4f\n", k, i, (*Pvals)[k][i]);
	  }
	}
	else
	  (*Pvals)[k][i] = atof(vars[i]);
	//	printf("Pvals[%d][%d] = %.4f\n", k, i, Pvals[k][i]);
      }
      k++;
    }
  }
  return 0;
}
