/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include "common/nrsimp.h"

int ltxnamecmp(int ind1, int ind2, char *barray[]);

int getallinfo(FILE *ltxfile, char *descfile, char *datfile, char *valfile, char varname[], char vardesc[], int lim, char htmlfuncfile[], int evolswitch, char evolfile[]);
char *modify(char *t);
int modify1(char *s, char *t, int lim);
void swap(void *v[], int i, int j);
void qsortt(void *v[], int left, int right, int (*comp)(void *, void *)); // can't get the library qsort to work for some reason. 
void qsortass(void *v[], void *v1[], int left, int right, int (*comp)(void *, void *));

int main()
{
  int i;
  int lim = 1000;
  char vardesc[1000];
  char **varltxarray;
  char **paramltxarray;
  char paramdesc[1000];
  char *simpltxname=NULL;
  FILE *ltxfile, *ltxfile1;
  char *filename = "allvartexfile";
  char *filename1 = "allparamtexfile";
  char *filename2 = "sysfiles/params.dat";
  char *valfile = "sysfiles/allcurrent_pvals.dat";

  char **vararray;
  int numvars=0;
  char **allsysparamarray;
  int numsysparams=0;

  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  allsysparamarray = genarraydat("sysfiles/params.dat", &numsysparams);

  // create the two arrays of semi-latexed names

  varltxarray = charar(0, numvars-1);
  paramltxarray = charar(0, numsysparams-1);

  for(i=0;i<numvars;i++){
    simpltxname = getltxname("sysfiles/temp/vars.desc", vararray[i]);
    varltxarray[i] = strdup(simpltxname);
    free(simpltxname);
  }
  for(i=0;i<numvars;i++){
    printf("%s\t%s\n", vararray[i], varltxarray[i]);
  }

  for(i=0;i<numsysparams;i++){
    simpltxname = getltxname("sysfiles/temp/params.desc", allsysparamarray[i]);
    paramltxarray[i] = strdup(simpltxname);
    free(simpltxname);
  }

  qsortass((void **) vararray, (void **) varltxarray, 0, numvars-1, (int (*)(void*, void*)) strcmp);
  qsortass((void **) allsysparamarray, (void **) paramltxarray, 0, numsysparams-1, (int (*)(void*, void*)) strcmp);

  ltxfile = fopen(filename,"w");
  if (ltxfile == NULL){
    fprintf(stderr, "ERROR in \"makeallinfo.c\": File %s could not be opened for writing.\n", filename);
    return -1; 
  }
  ltxfile1 = fopen(filename1,"w");
  if (ltxfile1 == NULL){
    fprintf(stderr, "ERROR in \"makeallinfo.c\": File %s could not be opened for writing.\n", filename1);
    return -1; 
  }

  i=0;
  fprintf(ltxfile, "%%generated by makeallinfo.c\n");
  fprintf(ltxfile1, "%%generated by makeallinfo.c\n");
  fprintf(ltxfile, "\\begin{longtable}[p]{llp{9cm}lp{9cm}}\n");
  fprintf(ltxfile, "variable (click for evolution) & code version & brief description & default initial value & other comments\\\\\n");
  fprintf(ltxfile, "\\hline\n");
  fprintf(ltxfile, "\\endfirsthead\nvariable & code version & brief description & default initial value & other comments\\\\\n\\hline\n\\endhead\n");
  fprintf(ltxfile, "\\multicolumn{5}{r}{}\\\\\n\\hline\n\\multicolumn{5}{r}{ctd...}\\\\\n\\endfoot\n\\multicolumn{5}{r}{}\\\\\n\\hline\n\\multicolumn{5}{r}{variable table ends}\\\\\n\\endlastfoot\n");

  while(i<numvars){
    getallinfo(ltxfile, "sysfiles/temp/vars.desc", "sysfiles/varbles.dat", "sysfiles/allcurrentinits.dat", vararray[i++], vardesc, lim, "initfuncs.html", 1, "readable.html");
  }
  fprintf(ltxfile, "\\end{longtable}\n");
  
  i=0;
  fprintf(ltxfile1, "\\begin{longtable}[p]{llp{12cm}lp{12cm}}\n");
  fprintf(ltxfile1, "parameter & code version & brief description & default value & other comments\\\\\n");
  fprintf(ltxfile1, "\\hline\n");
  fprintf(ltxfile1, "\\endfirsthead\nparameter & code version & brief description & default value & other comments\\\\\n\\hline\n\\endhead\n");
  fprintf(ltxfile1, "\\multicolumn{5}{r}{}\\\\\n\\hline\n\\multicolumn{5}{r}{ctd...}\\\\\n\\endfoot\n\\multicolumn{5}{r}{}\\\\\n\\hline\n\\multicolumn{5}{r}{parameter table ends.}\\\\\n\\endlastfoot\n");
  while(i<numsysparams){
    fprintf(stderr, "allsysparamarray[%d] = %s\n", i, allsysparamarray[i]);
    getallinfo(ltxfile1, "sysfiles/temp/params.desc",  filename2, valfile, allsysparamarray[i++], paramdesc, lim, "o2funcs.html", 0, "readable.html");
  }
  fprintf(ltxfile1, "\\end{longtable}\n");
  

  printf("\n\n\n");
  for(i=0;i<numvars;i++)
    printf("%s ", vararray[i]);
  printf("\n");

  printf("\n\n\n");
  for(i=0;i<numsysparams;i++)
    printf("%s ", allsysparamarray[i]);
  printf("\n");

  fclose(ltxfile);
  fclose(ltxfile1);

  freearraydat(vararray, numvars);
  freearraydat(allsysparamarray, numsysparams);
  freearraydat(varltxarray, numvars);
  freearraydat(paramltxarray, numsysparams);

  return 0;
}

int getallinfo(FILE *ltxfile, char *descfile, char *datfile, char *valfile, char varname[], char vardesc[], int lim, char htmlfuncfile[], int evolswitch, char evolfile[])
{
  int flag, found=0, ended, len1; 
  char *varnamemod=NULL;
  char *varnameltx=NULL;
  char oneword[50];
  char isfcnword[50];
  char *valword;
  char *vardesctemp;
  int len;
  int lim_line = 1000;
  char oneline[1000];
  char twoline[1000];
  char *othercomsline=NULL;
  char *ocline=NULL;
  char *unitline=NULL;
  int totlen=0;
  FILE *fd, *fd1;
  fd = fopen(descfile, "r");
  if (fd == NULL){
    fprintf(stderr, "ERROR in routine getallinfo in \"makeallinfo.c\" - file %s could not be opened.\n", descfile);
    return -1;
  }
  vardesc[0] = '\0'; // initialise the description
  twoline[0] = '\0';

  while((found == 0) && (len = getline0(fd, oneline, lim_line)) > 0){
    ended = 0;
    if (strcmp(varname, oneline) == 0){
      found = 1;
      // get the description

      while((len = getline0(fd, oneline, lim_line)) > 0 && strstr(oneline, "latex:") == NULL){
	totlen+=len;
	if(totlen < lim)
	  strcat(vardesc, oneline); // safe
	else
	  fprintf(stderr, "ERROR in \"makeallinfo.c\": line too long - reset \"lim\" in routine getallinfo.\n");
      }

      // get the latex name

      if (strstr(oneline, "latex:") != NULL){
	varnameltx = getrst("latex:", oneline);
      }

      // get the code version of the variable name

      varnamemod = enclose(varname, "\\verb|", "|");

      // print the description

      vardesctemp = modify(vardesc);

      //
      // the values
      //

      fd1 = fopen(valfile, "r");
      if (fd1 == NULL){
	fprintf(stderr, "ERROR in \"makeallinfo.c\": File %s could not be opened for reading.\n", valfile);
	return -1;
      }
      flag = 0;
      while(flag == 0 && (len1 = getline(fd1, twoline, 1000)) > 0){
        if(!iscomsep(twoline)){
	  getnthword(twoline, oneword, 40, 1);
	  if(strcmp(oneword, varname) == 0){
	    flag = 1;
	    valword = getname(varname, twoline);
	  }
	}
      }
      fclose(fd1);


      //
      // Whether o2 or not
      //

      isfcnword[0] = '\0';
      fd1 = fopen(datfile, "r");
      if (fd1 == NULL){
	fprintf(stderr, "ERROR in routine getallinfo in \"makeallinfo.c\" - file %s could not be opened for reading.\n", datfile);
	return -1;
      }
      flag = 0;
      while(flag == 0 && (len1 = getline(fd1, twoline, 1000)) > 0){
	if(!(iscomsep(twoline))){
	  getnthword(twoline, oneword, 40, 1);
	  if(strcmp(oneword, varname) == 0){
	    flag = 1;
	    getnthword(twoline, isfcnword, 40, 2);
	  }
	}
      }
      fclose(fd1);

      // get the other comments if any

      twoline[0] = '\0';
      while((len = getline0(fd, oneline, lim_line)) > 0 && strstr(oneline, "******") == NULL){
	if(strstr(oneline, "othercoms:") != NULL){
	  ocline = getrst("othercoms:", oneline);
	  othercomsline = modify(twoline);
	  free(ocline);
	}
	else if(strstr(oneline, "units:") != NULL){
	  unitline = getrst("units:", oneline);
	}
      }

      // print the information
      if(varnameltx == NULL)
	varnameltx = strdup("");
      if(unitline == NULL)
	unitline = strdup("");

      if(strcmp(varnameltx, "")==0)
	fprintf(ltxfile, "$?$ ");
      else{
	if(evolswitch == 0){ // is it a variable?
	  fprintf(ltxfile, "$%s$", varnameltx);
	}
	else{
	  fprintf(ltxfile, " \\htmladdnormallink{$%s$}{%s#%s_evol} ", varnameltx, evolfile, varname);
	}
      }
      fprintf(ltxfile, " & %s & ", varnamemod);
      fprintf(ltxfile, "%s & ", vardesctemp);
      fprintf(ltxfile, "%s %s", valword, unitline);

      free(valword);
      free(varnameltx);
      free(vardesctemp);
      free(unitline);
      free(varnamemod);

      if(strstr(isfcnword, "_fcn") == NULL)
	fprintf(ltxfile, " & ");
      else
	fprintf(ltxfile, " \\htmladdnormallink{(fcn)}{%s#%s} & ", htmlfuncfile, isfcnword);
      if(othercomsline == NULL)
	othercomsline = strdup("");
      fprintf(ltxfile, "%s\\\\\n", othercomsline);

      free(othercomsline);

    }
  }

  if(found == 0){
    fprintf(stderr, "WARNING in \"makeallinfo.c\": variable \"%s\" not found by routine getallinfo.\n", varname);
    varnamemod = enclose(varname, "\\verb|", "|");
    fprintf(ltxfile, "(%s) & (%s) & &\\\\\n", varnamemod, varnamemod);
    free(varnamemod);
  }
  fclose(fd);
  return totlen;
}




char *modify(char *t)
{
  int i = 0, j = 0;
  char *s;
  // count certain characters
  while(t[i] != '\0'){
    if((t[i] == '>') || (t[i] == '<')){j++;}
    i++;
  }
  s = (char*) malloc(sizeof(char) * ((int)strlen(t) + 2*j + 1));
  i=0;j=0;
  while(t[i] != '\0'){
    if((t[i] == '>') || (t[i] == '<')){
       s[j++] = '$';
       s[j++] = t[i++];
       s[j++] = '$';
    }
    else{s[j++] = t[i++];}
  }
  s[j++] = '\0'; // close off s
  return s;
}

int modify1(char *s, char *t, int lim)
     /* replaces all _ in t with \_ in s */
{
  int i = 0, j = 0;
  while(t[i] != '\0'){
    if(j == lim-1){
      fprintf(stderr, "ERROR in routine modify in \"makeallinfo.c\": limit of variable size reached.\n");
      break;
    }
    if(t[i] != '_')
      s[j++] = t[i++];
    else{
      if(t[i-1] == '\\')
        s[j++] = t[i++];
      else{
        s[j++] = '\\';
        s[j++] = t[i++];
      }
    }
  }
  s[j++] = '\0'; // close off s
  return j;
}





void qsortt(void *v[], int left, int right, int (*comp)(void *, void *))
{// p120 Kernighan and ritchie
  int i, last;
  void swap(void *v[], int, int);

  if (left >= right)
    return;
  swap(v, left, (left + right)/2);
  last = left;
  for (i = left + 1; i <= right; i++)
    if ((*comp)(v[i], v[left]) < 0)
      swap(v, ++last, i);
  swap(v, left, last);
  qsortt(v, left, last-1, comp);
  qsortt(v, last+1, right, comp);
}

/* int associate(int ind, char b[], char *barray[]) */
/* { */
/*   strcpy(b, barray[ind]); */
/*   return ind; */
/* } */

/* int ltxnamecmp(int ind1, int ind2, char *barray[]) */
/* { */
/*   char a_ass[50]; */
/*   char b_ass[50]; */
/*   associate(ind1, a_ass, barray); */
/*   associate(ind2, b_ass, barray); */
/*   return strcmp(a_ass, b_ass); */
/* } */

void qsortass(void *v[], void *v1[], int left, int right, int (*comp)(void *, void *))
{ // the first array is sorted according to the second...
  int i, last;
  void swap(void *v[], int, int);

  if (left >= right)
    return;
  swap(v, left, (left + right)/2);
  swap(v1, left, (left + right)/2);
  last = left;
  for (i = left + 1; i <= right; i++){
    if ((*comp)(v1[i], v1[left]) < 0){
      swap(v, ++last, i);
      swap(v1, last, i);
    }
  }
  swap(v, left, last);
  swap(v1, left, last);
  qsortass(v, v1, left, last-1, comp);
  qsortass(v, v1, last+1, right, comp);
}

void swap(void *v[], int i, int j)
{
  void *temp;

  temp = v[i];
  v[i] = v[j];
  v[j] = temp;
}
