/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include "common/nrsimp.h"


int getallprocinfo(FILE *ltxfile, char *descfile, char *vartype, char varname[], char vardesc[], int lim);
char *modify(char *t);
void swap(void *v[], int i, int j);
void qsortt(void *v[], int left, int right, int (*comp)(void *, void *)); // can't get the library qsort to work for some reason. 
void qsortass(void *v[], void *v1[], int left, int right, int (*comp)(void *, void *));

int main()
{
  int i;
  char vardesc[1000];
  FILE *ltxfile;
  char *filename = "allproctexfile";
  int numreacs=0, numalgs=0;
  char **reacnames;
  char **algnames;
  char reacfile[100];
  char algrelfile[100];

  strcpy(reacfile, "sysfiles/sysstate/reacs.dat"); // safe
  strcpy(algrelfile, "sysfiles/sysstate/algrels.dat"); // safe


  reacnames = genarraydat(reacfile, &numreacs);
  algnames = genarraydat(algrelfile, &numalgs);

  ltxfile = fopen(filename,"w");
  if (ltxfile == NULL){
    fprintf(stderr, "ERROR in \"makeallprocinfo.c\" - file %s could not be opened for writing.\n", filename);
    return -1;
  }

  fprintf(ltxfile, "%%generated by makeallprocinfo.c\n");
  fprintf(ltxfile, "\\begin{longtable}[p]{llp{9cm}p{9cm}}\n");
  fprintf(ltxfile, "brief name (process type: site) & process name & description & other comments\\\\\n");
  fprintf(ltxfile, "\\hline\n");
  fprintf(ltxfile, "\\endfirsthead\nbrief name (process type: site) & process name & description & other comments\\\\\n\\hline\n\\endhead\n");
  fprintf(ltxfile, "\\multicolumn{4}{r}{}\\\\\n\\hline\n\\multicolumn{4}{r}{ctd...}\\\\\n\\endfoot\n\\multicolumn{4}{r}{}\\\\\n\\hline\n\\multicolumn{4}{r}{table of processes ends}\\\\\n\\endlastfoot\n");

  i=0;
  while(i<numreacs){
    getallprocinfo(ltxfile, "sysfiles/temp/allprocs.desc", "reaction", reacnames[i++], vardesc, 1000);
  }
  i=0;
  while(i<numalgs){
    getallprocinfo(ltxfile, "sysfiles/temp/allprocs.desc", "algebraic relation", algnames[i++], vardesc, 1000);
  }
  fprintf(ltxfile, "\\end{longtable}\n");
  
  fclose(ltxfile);
  return 0;
}

int getallprocinfo(FILE *ltxfile, char *descfile, char *vartype, char varname[], char vardesc[], int lim)
{
  int found=0;
  char *procname=NULL;
  char *vardesctemp=NULL;
  int len;
  int lim_line = 1000;
  char oneline[1000];
  char *twoline=NULL;
  char *othercomsline=NULL;
  char *siteline=NULL;
  int totlen=0;
  FILE *fd;
  fd = fopen(descfile, "r");
  if (fd == NULL){
    fprintf(stderr, "ERROR in routine getallprocinfo in \"makeallprocinfo.c\" - file %s could not be opened for reading.\n", descfile);
    return -1;
  }
  vardesc[0] = '\0'; // initialise the description

  while((found == 0) && (len = getline0(fd, oneline, lim_line)) > 0){
    if (strcmp(varname, oneline) == 0){
      found = 1;

      // get the description

      while((len = getline0(fd, oneline, lim_line)) > 0 && strstr(oneline, "name:") == NULL){
	totlen+=len;
	if(totlen < lim)
	  strcat(vardesc, oneline); // safe
	else
	  fprintf(stderr, "ERROR in \"makeallprocinfo.c\": line too long - reset \"lim\" in routine getallprocinfo.\n");
      }
      vardesctemp = modify(vardesc);

      // get the name

      if (strstr(oneline, "name:") != NULL)
	procname = getrst("name:", oneline);

      // get the site if any and other comments if any
      while((len = getline0(fd, oneline, lim_line)) > 0 && strstr(oneline, "******") == NULL){
	if(strstr(oneline, "site:") != NULL)
	  siteline = getrst("site:", oneline);
	else if(strstr(oneline, "othercoms:") != NULL){
	  twoline = getrst("othercoms:", oneline);
	  othercomsline = modify(twoline);
	  free(twoline);
	}
      }

      if(othercomsline==NULL)
	othercomsline=strdup("");
      if(procname==NULL)
	procname=strdup("");
      if(siteline==NULL)
	siteline=strdup("");

      // print the data

      fprintf(ltxfile, "\\verb|%s| (%s: %s) & ", varname, vartype, siteline);

      if(strcmp(procname, "") == 0){
	fprintf(ltxfile, " %s [no name given] &", varname);
      }
      else{
	if(strcmp(vartype, "reaction") == 0){
	    fprintf(ltxfile, "\\htmladdnormallink[%s]{%s}{#%sreac} & ", varname, procname, varname);
	}
	else{
	  fprintf(ltxfile, "\\HTMLcode[NAME = \"%s\"]{A}{%s} & ", varname, procname);
	}
      }

      fprintf(ltxfile, "%s & ", vardesctemp);
      fprintf(ltxfile, "%s\\\\\n", othercomsline);

      free(vardesctemp);
      free(othercomsline);
      free(procname);
      free(siteline);
 
    }

  }

  if(found == 0){
    fprintf(stderr, "WARNING in \"makeallprocinfo.c\": process \"%s\" not found by routine getallprocinfo.\n", varname);
    fprintf(ltxfile, "\\verb|%s| (%s) & & &\\\\\n", varname, vartype);
  }
  fclose(fd);

  return totlen;
}


char *modify(char *t)
{
  int i = 0, j = 0;
  char *s;
  // count certain characters
  while(t[i] != '\0'){
    if((t[i] == '>') || (t[i] == '<')){j++;}
    i++;
  }
  s = (char*) malloc(sizeof(char) * ((int)strlen(t) + 2*j + 1));
  i=0;j=0;
  while(t[i] != '\0'){
    if((t[i] == '>') || (t[i] == '<')){
       s[j++] = '$';
       s[j++] = t[i++];
       s[j++] = '$';
    }
    else{s[j++] = t[i++];}
  }
  s[j++] = '\0'; // close off s
  return s;
}

void qsortt(void *v[], int left, int right, int (*comp)(void *, void *))
{// p120 Kernighan and ritchie
  int i, last;
  void swap(void *v[], int, int);

  if (left >= right)
    return;
  swap(v, left, (left + right)/2);
  last = left;
  for (i = left + 1; i <= right; i++)
    if ((*comp)(v[i], v[left]) < 0)
      swap(v, ++last, i);
  swap(v, left, last);
  qsortt(v, left, last-1, comp);
  qsortt(v, last+1, right, comp);
}



void qsortass(void *v[], void *v1[], int left, int right, int (*comp)(void *, void *))
{ // the first array is sorted according to the second...
  int i, last;
  void swap(void *v[], int, int);

  if (left >= right)
    return;
  swap(v, left, (left + right)/2);
  swap(v1, left, (left + right)/2);
  last = left;
  for (i = left + 1; i <= right; i++){
    if ((*comp)(v1[i], v1[left]) < 0){
      swap(v, ++last, i);
      swap(v1, last, i);
    }
  }
  swap(v, left, last);
  swap(v1, left, last);
  qsortass(v, v1, left, last-1, comp);
  qsortass(v, v1, last+1, right, comp);
}

void swap(void *v[], int i, int j)
{
  void *temp;

  temp = v[i];
  v[i] = v[j];
  v[j] = temp;
}
