/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// Reads the datafile userfiles/fixedvars.dat
// produces the files sysfiles/fixedvars.dat

#include "common/stringmanip.h"

int main()
{
  int len, i, k, fixglob=0, nonfixglob=0, fixloc=0, nonfixloc=0, fixtot=0, nonfixtot=0;
  FILE *fd, *fd0, *fd2;
  int lim = 2000;
  char oneline[2000]; // for comments too
  char *varsline;
  int maxgets = 1000; // maximum number of variables transported
  char *fixedvars[1000];
  char *nonfixed[1000];
  char *fixedvars1[1000];
  char *nonfixed1[1000];
  char **fixedvars2;
  char **nonfixed2;
  char *filename = "userfiles/fixedvars.dat";
  char outputparamfile[100];

  int numallvars=0;
  char **allvararray;

  allvararray = genarraydat("sysfiles/allvarbles.dat", &numallvars);

  strcpy(outputparamfile, "sysfiles/sysstate/output_params.dat"); //safe


  // globally fixed variables
  fd = fopen(filename, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"makefixedvars.c\": File %s could not be opened for reading.\n", filename);
    return -1;
  }
  while((len = getline(fd, oneline, lim)) > 0){
    if(!(iscom(oneline))){
      if(strstr(oneline, "fixedvars:") != NULL){
      	varsline = getrst("fixedvars:", oneline);
        fixglob = chemgets(varsline, fixedvars, maxgets);
	free(varsline);
      }
      else if(strstr(oneline, "allfixedbut:") != NULL){
      	varsline = getrst("allfixedbut:", oneline);
        nonfixglob = chemgets(varsline, nonfixed, maxgets);
	free(varsline);
      }
    }
  }
  fclose(fd);

  // variables always fixed for a particular system
  fd0 = fopen(outputparamfile, "r");
  if(fd0 == NULL){
    fprintf(stderr, "ERROR in \"makefixedvars.c\": File %s could not be opened for reading.\n", outputparamfile);
    return -1;
  }
  while((len = getline(fd0, oneline, lim)) > 0){
    if(!(iscom(oneline))){
      if(strstr(oneline, "fixedvars:") != NULL){
      	varsline = getrst("fixedvars:", oneline);
        fixloc = chemgets(varsline, fixedvars1, maxgets);
	free(varsline);
      }
      else if(strstr(oneline, "allfixedbut:") != NULL){
	varsline = getrst("allfixedbut:", oneline);
        nonfixloc = chemgets(varsline, nonfixed1, maxgets);
	free(varsline);
      }
    }
  }
  fclose(fd0);

  fixtot = combarrays(&fixedvars2, fixedvars, fixglob, fixedvars1, fixloc);
  nonfixtot = combarrays(&nonfixed2, nonfixed, nonfixglob, nonfixed1, nonfixloc);

  if(fixtot > 0 || nonfixtot > 0){
    fprintf(stderr, "\n\nWARNING: Some variables are being held fixed:\n");
  }
  // some checking

  if((fixtot > 0) && (nonfixtot > 0)){
    fprintf(stderr, "WARNING in makefixedvars: you have both fixed variables and allfixedbut... allfixedbut being ignored.\n");
    nonfixtot = 0;
  }

  fd2 = fopen("sysfiles/fixedvars.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"makefixedvars.c\": File \"sysfiles/fixedvars.dat\" could not be opened for writing.\n");
    return -1;
  }

  fprintf(fd2, "// This file is generated by \"../makefixedvars.c\". It holds the fixed variables\n\n");


  // fixedvars



  if(nonfixtot == 0){
    for(i=0;i<fixtot-1;i++){
      if(isinarray(allvararray, numallvars, fixedvars2[i]) < 0)
        fprintf(stderr, "ERROR in makefixedvars.c: %s is not a variable.\n", fixedvars2[i]);
      fprintf(fd2, "%s\n", fixedvars2[i]);
      fprintf(stderr, "%s\n", fixedvars2[i]);
    }
    if(fixtot>0){
      fprintf(fd2, "%s\n", fixedvars2[i]);
      fprintf(stderr, "%s\n", fixedvars2[i]);
    }
  }
  // allfixedbut
  else{
    for(i=0;i<nonfixtot;i++){ // confirming
      if(isinarray(allvararray, numallvars, nonfixed2[i]) < 0)
        fprintf(stderr, "ERROR in makefixedvars.c: %s is not a variable.\n", nonfixed[i]);
    }

    k=0;
    for(i=0;i<numallvars;i++){
      if(isinarray(nonfixed2, nonfixtot, allvararray[i]) < 0){
        k++;
        if(k==numallvars-nonfixtot){
	  fprintf(fd2, "%s\n", allvararray[i]);
	  fprintf(stderr, "%s\n", allvararray[i]);
	}
        else{
	  fprintf(fd2, "%s\n", allvararray[i]);
	  fprintf(stderr, "%s\n", allvararray[i]);
	}
      }
    }
  }

  fclose(fd2);

  freearraydat(fixedvars2, fixtot);
  freearraydat(nonfixed2, nonfixtot);
  freearraydat(allvararray, numallvars);
  return 0;
}
