/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include "common/nrsimp.h"

int getdesc1(char *descfile, char varname[], char vardesc[], int lim);

int main(int argc, char *argv[])
{
  int i, suc=0;
  int lim = 5000;
  char vardesc[5000];
  FILE *fd;
  int numall=0;
  int numtemp=0;
  char **allarray;
  char **temparray;
  char *tfile;

  if(argc < 2){
    fprintf(stderr, "ERROR in makeoutputinfo: you must enter a data file name.\n");
    return -1;
  }
  else if(argc > 2){
    fprintf(stderr, "WARNING in makeoutputinfo: only the first argument is being read.\n");
  }
  if(!fileexists(argv[1])){
    fprintf(stderr, "ERROR in makeoutputinfo: Data file %s does not exist.\n", argv[1]);
    return -1;
  }

  fd = fopen("sysfiles/outputvars.dat", "w");
  if(!fd){
    fprintf(stderr, "ERROR in makeoutputinfo.c: \"sysfiles/outputvars.dat\" could not be opened...\n");
    return -1;
  }
  fprintf(fd, "// generated by makeoutputinfo.c - do not edit\n\n");

  tfile = (char*) malloc(sizeof(char) * (strlen(argv[1])+10));
  sprintf(tfile, "%s.tvars", argv[1]); // safe

  allarray = getdfilevars(argv[1], &numall);
  temparray = genarraydat(tfile, &numtemp);
  free(tfile);

  for(i=0;i<numall;i++){
    suc = getdesc1("sysfiles/temp/vars.desc", allarray[i], vardesc, lim);
    if(suc<0){
      suc = getdesc1("sysfiles/temp/params.desc", allarray[i], vardesc, lim);
    }
    if(!isonlyspace(vardesc))
      fprintf(fd, "%s (%s)\n", allarray[i], vardesc);
    else if(strcmp(allarray[i], "time")==0)
      fprintf(fd, "%s \n", allarray[i]);
    else if(startswith(allarray[i], "T_"))
      fprintf(fd, "%s (rate of %s)\n", allarray[i], allarray[i]+2);
    else
      fprintf(fd, "%s [no details]\n", allarray[i]);
  }

  for(i=0;i<numtemp;i++){
    suc = getdesc1("sysfiles/temp/vars.desc", temparray[i], vardesc, lim);
    if(suc<0){
      suc = getdesc1("sysfiles/temp/params.desc", temparray[i], vardesc, lim);
    }
    if(!isonlyspace(vardesc))
      fprintf(fd, "%s (%s)\n", temparray[i], vardesc);
    else if(startswith(temparray[i], "T_"))
      fprintf(fd, "%s (rate of %s)\n", temparray[i], temparray[i]+2);
    else if(startswith(temparray[i], "R_"))
      fprintf(fd, "%s (reduced rate of %s)\n", temparray[i], temparray[i]+2);
    else
      fprintf(fd, "%s [no details]\n", temparray[i]);
  }

  fclose(fd);

  freearraydat(allarray, numall);
  freearraydat(temparray, numtemp);
  return 0;
}

int getdesc1(char *descfile, char varname[], char vardesc[], int lim)
{
  int found=0;
  int len;
  int lim_line = 1000;
  char oneline[1000];
  int totlen=0;
  FILE *fd;
  char *vn;

  fd = fopen(descfile, "r");
  if (fd == NULL){
    fprintf(stderr, "ERROR in routine getallinfo in \"makeallinfo.c\" - file %s could not be opened.\n", descfile);
    return -1;
  }
  vardesc[0] = '\0'; // initialise the description

  while((found == 0) && (len = getline0(fd, oneline, lim_line)) > 0){
    if(!iscom(oneline)){
      vn = getnthwd(oneline, 1);
      if (strcmp(vn, varname) == 0){
	found = 1;
	// get the description

	while((len = getline0(fd, oneline, lim_line)) > 0 && strstr(oneline, "othercoms") == NULL && strstr(oneline, "units:") == NULL && strstr(oneline, "latex:") == NULL && strstr(oneline, "******") == NULL){
	  totlen+=len;
	  if(totlen < lim)
	    strcat(vardesc, oneline); // safe
	  else{
	    fprintf(stderr, "ERROR in \"makeallinfo.c\": line too long - reset \"lim\" in routine getdesc1.\n");
	    fclose(fd);
	    free(vn);
	    return -2;
	  }
	}

      }
      free(vn);
    }
  }

  fclose(fd);
  if(found == 0){
/*     fprintf(stderr, "WARNING in \"makeoutputinfo.c\": variable \"%s\" not found by routine getdesc1.\n", varname); */
    return -3;
  }
  return totlen;
}

