/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"

char *getvarname(char fname[], int varindex);
int writetwocommands(char *filename, char *datafilename, char *var1, char *ind1, char *var2, char *ind2, char *var3, char *ind3); // for two plot variables
int writecontcommands(char *filename, char *datafilename, char *var1, char *ind1, char *var2, char *ind2); // for continuous plotting
int writetwocontcommands(char *filename, char *datafilename, char *var1, char *ind1, char *var2, char *ind2, char *var3, char *ind3); // for continuous plotting with two variables

int plotall(char *xvar, char *plotstr, char *outputfstr, char *ops, int contplot, char *remsfile); // in outfuncs.c

int main(int argc, char *argv[])
{

  int k1, l;
  char systemcom[200];
  char plotcomsstr[100];
  char plotheaderstr[100];
  char gnuplotcom[100];
  int flag, suc;
  int i;
  char entered[100];
  char outfile[100];
  char *var1str;
  char *plotstr;
  FILE *fd;
  char line[100];
  char *outfilename;
  char remsfile[50];

  if(fileexists("userfiles/mathrems.dat"))
    strcpy(remsfile, "userfiles/mathrems.dat"); // safe
  else
    strcpy(remsfile, "userfiles/remsfile.dat"); // safe

  if((argc>1) && strlen(argv[1]) > 50){
    fprintf(stderr, "ERROR: output file name too long. Exiting...\n");
    return -1;
  }

  // get the gnuplot command
  if(getoption("userfiles/gen_options", "GNUPLOT", gnuplotcom, 100)!=0){
    fprintf(stderr, "ERROR in makeplot: couldn't get the option \"GNUPLOT\" from the file \"userfiles/gen_options\"\n");
    return -1;
  }

  // get the file which will store the plot commands
  if(getoption("userfiles/gen_options", "PLOTCOMS", plotcomsstr, 100)!=0){
    fprintf(stderr, "ERROR in makeplot: couldn't get the option \"PLOTCOMS\" from the file \"userfiles/gen_options\"\n");
    return -1;
  }
  // get the file which stores the plot header information
  if(getoption("userfiles/gen_options", "PLOTHEADER", plotheaderstr, 100)!=0){
    fprintf(stderr, "ERROR in makeplot: couldn't get the option \"PLOTHEADER\" from the file \"userfiles/gen_options\"\n");
    return -1;
  }

 // model


  fd = fopen("sysfiles/sysstate/which_output_file", "r");
  if (fd==NULL){
    fprintf(stderr, "Couldn't open the file: \"sysfiles/sysstate/which_output_file\". Exiting...\n");
    return -1;
  }
  getline(fd, line, 100);
  outfilename = getnthblk(line, 1);
  fclose(fd);

  if (argc <= 2){
    flag = 0;
    if(argc == 2){
      strcpy(outfile, argv[1]); // safe
      snprintf(systemcom, 200, "test -r %s", argv[1]);
      suc = system(systemcom);
      if(suc!=0){
        printf("File \"%s\" not found.\nTry again or type enter to use current file: ", entered);
      }
      else{
	flag = 1;
      }
    }
    if(argc == 1)
      printf("Enter the data file from which to plot.\n (Default is \"%s\"): ", outfilename);

    while(flag == 0){
      i=0;
      while(i<99 && (entered[i++] = getchar()) != '\n'){}
      i--;entered[i] = 0; // get rid of the terminal newline character

      if(strlen(entered) == 0){
	strncpy(outfile, outfilename, 99);
        printf("Using current file: \"%s\"\n\n", outfilename);
        snprintf(systemcom, 200, "test -r %s", outfilename);
        suc = system(systemcom);
        if(suc!=0){
          printf("No file \"%s\" found.\nPerhaps you need to run the simulation with \"./sim\" first. Exiting.\n", outfilename);
	  free(outfilename);
	  return -1;
        }
	else{
	  flag = 1;
	}
      }
      else{
        snprintf(systemcom, 200, "test -r %s", entered);
        suc = system(systemcom);
        if(suc!=0){
          printf("No file \"%s\" found.\nTry again or type enter to use current file: ", entered);
        }
        else{
	  strcpy(outfile, entered); // safe
	  flag = 1;
	}
      }
    }

    printf("Enter the x variable.\n (Default is the first variable in the file): ");

    flag = 0;
    var1str=strdup("");
    while(flag == 0){
      free(var1str);
      ent(entered, 100);

      if(isonlyspace(entered)){
	var1str = getvarname(outfile, 1);
        printf("Using first variable: %s\n\n", var1str);
        flag = 1;
      }
      else{
	k1 = getdatfileindex(outfile, entered);
	var1str = strdup(entered);
	if(k1 <= 0)
          printf("Variable %s not found.\nTry again or type enter to use default: ", entered);
	else
	  flag = 1;
      }
    }


    printf("\nEnter an expression, or expressions separated by the character \":\"\n (Default - second variable in the file): ");

    flag = 0;
    while(flag == 0){
      ent(entered, 100);

      if (strcmp(entered, "quit") == 0 || strcmp(entered, "exit") == 0){
	plotstr = strdup(""); flag=1;printf("\n");
      }
      else if(isonlyspace(entered))
	plotstr = getvarname(outfile, 2);
      else
	plotstr = strdup(entered);

      if(flag==0){
        l=plotall(var1str, plotstr, outfile, "", 0, remsfile);
        printf("\nEnter another expression, or several variables separated by colons \":\"\n (Default - second variable in the file, \"quit\" to quit): ");
      }

    }
    free(outfilename); free(plotstr); free(var1str);
    return 0;
  }
  else{
    fprintf(stderr, "ERROR: makeplot takes a maximum of one argument. Exiting...\n");
    free(outfilename);
    return -1;
  }

  free(outfilename);
  return 0;

}
