/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// generates from a data file a reduced data file only containing certain variables that we are interested in: can be important in dealing with very large data files...

#include "common/stringmanip.h"
#include "common/nrsimp.h"

int main(int argc, char *argv[])
{
  int i, j, k, numtokens;
  FILE *fd, *fd1;
  int len;
  int lim=40000;
  char oneline[40000];
  char *varnameline;
  int *d;
  int numvars;
  char **outarray;
  char *infilename;
  char *outfilename;
  char *tmpin, *tmpout;

  if(argc<4){
    fprintf(stderr, "ERROR in makereddatfile.c: You need an input file name, an output file name, and at least one output variable.\n");
    return -1;
  }

  numvars = argc-3;
  outarray = charar(0, numvars-1);
  infilename = strdup(argv[1]);
  outfilename = strdup(argv[2]);

  for(i=3;i<argc;i++)
    outarray[i-3] = strdup(argv[i]);

  if(strcmp(infilename, outfilename) == 0){
    fprintf(stderr, "ERROR in makereddatfile.c: you cannot have the same input and output file.\n");
    free(infilename); free(outfilename);
    freearraydat(outarray, argc-3);
    return -2;
  }

  fd = fopen(infilename, "r");

  if(fd == NULL){
    fprintf(stderr, "ERROR in makereddatfile: file %s could not be opened.\n", infilename);
    free(infilename); free(outfilename);
    freearraydat(outarray, argc-3);
    return -3;
  }

  fd1 = fopen(outfilename, "w");

  if(fd1 == NULL){
    fprintf(stderr, "ERROR in makereddatfile: file %s could not be opened.\n", outfilename);
    free(infilename); free(outfilename);
    freearraydat(outarray, argc-3);
    fclose(fd);
    return -4;
  }

  while((len = getline(fd, oneline, lim)) > 0){
    if(!(iscomsep(oneline)))
      break;
  }
  if(len==lim-1){
    fprintf(stderr, "ERROR in makereddatfile: the line length is too long in file %s.\n", infilename);
    free(infilename); free(outfilename);
    freearraydat(outarray, argc-3);
    fclose(fd);fclose(fd1);
    return -4;
  }

  numtokens = getnumtokens(oneline);

  d = ivector(0, numtokens-1);

  fprintf(fd1, "#");
  j=0;
  for(i=1;i<=numtokens;i++){
    varnameline = getnthwd(oneline, i);
    if((k=isinarray(outarray, numvars, varnameline)) >= 0){
      d[j++] = i;
      if(j==1)
	fprintf(fd1, "%s", varnameline);
      else
	fprintf(fd1, "\t%s", varnameline);
    }
    free(varnameline);
  }
  fprintf(fd1, "\n");
  numvars = j;


  while((len = getline(fd, oneline, lim)) > 0){
    if(len==lim-1){
      fprintf(stderr, "ERROR in makereddatfile: the line length is too long in file %s.\n", infilename);
      free(infilename); free(outfilename);
      freearraydat(outarray, argc-3);
      fclose(fd);fclose(fd1);
      return -4;
    }
    if(!(iscomsep(oneline))){
      for(i=0;i<numvars;i++){
	varnameline = getnthtkn(oneline, d[i]);
	if(i == 0)
	  fprintf(fd1, "%s", varnameline);
	else
	  fprintf(fd1, "\t%s", varnameline);
	free(varnameline);
      }
      fprintf(fd1, "\n");
    }
  }

  fclose(fd);
  fclose(fd1);

  // copy pvals and tvars files

  tmpin=enclose(infilename, "", ".pvals");tmpout=enclose(outfilename, "", ".pvals");
  cpfile(tmpout, tmpin);
  free(tmpin);free(tmpout);

  tmpin=enclose(infilename, "", ".tvars");tmpout=enclose(outfilename, "", ".tvars");
  cpfile(tmpout, tmpin);
  free(tmpin);free(tmpout);

  //memory freeing
  free(infilename);
  free(outfilename);
  freearraydat(outarray, argc-3);

  return 0;
}
