/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

/* this file generates the files sysfiles/mass.c and sysfiles/rhs.c. and also the more readable files, rhs1.c and readable. rpar[i] is the ith variable in paramarray, and v[i] is the ith variable in vararray The translation routines are in stringmanip, so this must be linked in */

#include "syssize.h"
#include "common/stringmanip.h"

int main()
{
  int i, j, k, k1, m, m1, ind_vec, len, flag;
  FILE *fd, *fd1, *fd2, *fd3, *fd4;
  char varholdername[4];
  char paramholdername[10];
  char derivsholdername[10];
  char numholdername[10];
  char *temp, *temp1, *temp2, *temp3, *temp4;
  char constr[100], constr1[100];
  int lim=500;
  char oneline[500];
  char varname[40];
  char **vararray;
  int numvars=0;
  char **paramarray;
  int numparams=0;
  char **chemnames;
  int numchems=0;
  char **tempvs;
  char **tempexprs;
  int numtempvs;
  int warnlevel=1;
  static char *globtrans1[] = {"xtm"};
  static char *globtrans2[] = {"(*_x)"};

  struct DE allDEs[MAXDE];
  int numDEs;
  struct algrel allalgrels[MAXALG];
  int numalgrels;

  chemnames = genarraydat("sysfiles/chemnames.dat", &numchems);
  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  // paramarray = genarraydat("sysfiles/paramarray.dat", &numparams);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  numtempvs = genarraysdat("sysfiles/tempvars2.dat", &tempvs, &tempexprs);

  // Read in the DEs and the algrels from the data files

  numDEs = readDEfile(allDEs, MAXDE, "sysfiles/allDEs1.dat", "sysfiles/vararray.dat", "sysfiles/allparams.dat", "userfiles/remsfile.dat");
  numalgrels = readalgfile(allalgrels, MAXALG, "sysfiles/allalgrels1.dat", "sysfiles/vararray.dat", "sysfiles/allparams.dat", "userfiles/remsfile.dat");

  strcpy(varholdername, "_v"); // safe
  strcpy(paramholdername, "rpar"); //safe
  strcpy(derivsholdername, "_fv"); // safe;
  strcpy(numholdername, "_n"); // safe;

  fd = fopen("sysfiles/mass.c", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/mass.c\" could not be opened for writing.\n");
    return -1;
  }
  fd1 = fopen("sysfiles/rhs1.c", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/rhs1.c\" could not be opened for writing.\n");
    return -1;
  }
  fd2 = fopen("sysfiles/rhs.c", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/rhs.c\" could not be opened for writing.\n");
    return -1;
  }
  fd3 = fopen("sysfiles/readable", "w");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/readable\" could not be opened for writing.\n");
    return -1;
  }
  fd4 = fopen("sysfiles/constraints.dat", "r");
  if(fd4 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/constraints.dat\" could not be opened for reading.\n");
    return -1;
  }

  /* the mass file preliminaries */

  fprintf(fd, "/* This file is generated by the program \"../makesys.c\". It contains the jacobian in transposed form */\n\n");

  fprintf(fd, "void mas(int *%s, double *_am, int *lmas, double *%s, int *ipar)\n", numholdername, paramholdername);
  fprintf(fd, "{\n\n  /* _am is a (lmas,%s) matrix in column major storage */\n  double *J[*%s];\n  int i, j;\n  /* set J to first elt. of each column */\n", numholdername, numholdername);
  fprintf(fd, "  for (i=0;i<*%s;++i)\n    J[i]=&_am[i* *lmas];\n\n", numholdername);
  fprintf(fd, "  /* initialise all to zero */\n  for(i=0;i<*%s;i++){\n    for(j=0;j<*%s;j++)\n      J[i][j] = 0.0;\n  }\n\n\n", numholdername, numholdername);

  /* the untranslated rhs file preliminaries */

  fprintf(fd1, "/* This file is generated by the program \"../makesys.c\". It contains the rhs in an untranslated form */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#include \"../src/sys.h\"\nconst double Pi;\n// = 3.14159265;\n\n");
  fprintf(fd1, "void rhs(int *%s, double *_x, double *%s, double *%s, double *%s, int *ipar)\n{\n", numholdername, varholdername, derivsholdername, paramholdername);

  /* the translated rhs file preliminaries */

  fprintf(fd2, "/* This file is generated by the program \"../makesys.c\". It contains the rhs in a translated form */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#include \"../src/sys.h\"\nconst double Pi;\n// = 3.14159265;\n\n");
  fprintf(fd2, "void rhs(int *%s, double *_x, double *%s, double *%s, double *%s, int *ipar)\n{\n", numholdername, varholdername, derivsholdername, paramholdername);

  // the readable file preliminaries
  fprintf(fd3, "// This file is generated by the program \"../makesys.c\". It contains the system in readable form \n\n");

  // The temporary variables

  for(i=0;i<numtempvs;i++){
    if(!(isonlyspace(tempexprs[i]))){
      temp = transalt1(tempexprs[i], varholdername, vararray, numvars);
      temp1 = transfunc2(temp, globtrans1, globtrans2, 1); // xtmp translated
      temp2 = transalt1(temp1, paramholdername, paramarray, numparams);
      fprintf(fd1, "  double %s = %s;\n", tempvs[i], tempexprs[i]);
      fprintf(fd2, "  double %s = %s;\n",  tempvs[i], temp2);
      free(temp);free(temp1);free(temp2);
    }
  }

      ///////////

  // dealing with the constraints - for large models you might want to sacrifice warning for speed by setting warnlevel to zero

  if (warnlevel==1){

    while((len = getline(fd4, oneline, lim)) > 0){
      j=0;
      if(!iscomsep(oneline)){
        k=0;
        while((k < 40) && oneline[j] && (oneline[j] != ':') && (oneline[j] != '\n') && (oneline[j] != 13))
	  varname[k++] = oneline[j++];
        varname[k] = '\0';
        flag = 0; 
        if(isinarray(vararray, numvars, varname) >=0){
	  k=0;
	  while((oneline[j] == ':') || (isspace((int) oneline[j])) || (oneline[j] == 13)){j++;}
	  while((k < 100) && (oneline[j] != ':') && (oneline[j] != '\n') && (oneline[j] != 13)){
	    constr[k++] = oneline[j++];
	  }
	  constr[k] = '\0';
	  if(strcmp(constr, "") == 0){
	    fprintf(stderr, "ERROR in \"makesys.c\": no constraint is provided for %s.\n", varname);
	    flag = 1;
	  }
	  k=0;
	  while((oneline[j] == ':') || (isspace((int) oneline[j])) || (oneline[j] == 13)){j++;}
	  while((k < 100) && (oneline[j] != ':') && (oneline[j] != '\n') && (oneline[j] != 13))
	    constr1[k++] = oneline[j++];
	  constr1[k] = '\0';
	  if(strcmp(constr1, "") == 0){
	    fprintf(stderr, "ERROR in \"makesys.c\": no alternative is provided when %s is violated.\n", constr);
	    flag = 1;
	  }
	  if(flag == 0){
            temp = transalt1(constr, varholdername, vararray, numvars);
            temp1 = transalt1(temp, paramholdername, paramarray, numparams);
            temp2 = transalt1(constr1, varholdername, vararray, numvars);
            temp3 = transalt1(temp2, paramholdername, paramarray, numparams);
            temp4 = transalt1(varname, varholdername, vararray, numvars);
	    fprintf(fd2, "  if (%s){\n    fprintf(stderr, \"WARNING in function rhs: %s=%%.4e out of range\\n\", \%s);\n    %s;\n  }\n", temp1, varname, temp4, temp3);
	    fprintf(fd1, "  if (%s){\n    fprintf(stderr, \"WARNING in function rhs: %s=%%.4e out of range\\n\", \%s);\n    %s;\n  }\n", constr, varname, temp4, constr1);
	    free(temp);free(temp1);free(temp2);free(temp3);free(temp4);
	  }
        }
      }
    }
  }


  //
  // DEs
  // 

  /* take a DE, get the index of its key variable, get the indices of all its left variables, make the entries
   */

  ind_vec=0;
  for(i=0;i<numDEs;i++){ 
    /* get the index of the key variable in question */
    m1 = isinarray(vararray, numvars, allDEs[i].keyvar);
    if (m1 < 0){
      fprintf(stderr, "ERROR in makesys.c: the key variable \"%s\" was not found in \"vararray\" stored in file \"sysfiles/vararray.dat\".\n\n", allDEs[i].keyvar);
      return -2;
    }
    else{
      fprintf(fd1, "  f%s = %s;\n", vararray[m1], allDEs[i].rterm);

      // the readable file
      fprintf(fd3, "%s\n", vararray[m1]);
      for(j=0;j<allDEs[i].numleft;j++){
	if(strcmp(allDEs[i].valsleft[j], "1.0") != 0)
	  fprintf(fd3, "%sD%s ", allDEs[i].valsleft[j], allDEs[i].varsleft[j]);
	else
	  fprintf(fd3, "D%s ", allDEs[i].varsleft[j]);
      }
      fprintf(fd3, "= %s\n******\n", allDEs[i].rterm);
      
      // the vecfcn file
      m = isinarray(chemnames, numchems, allDEs[i].keyvar);

      // the system file
      temp = transalt1(allDEs[i].rterm, varholdername, vararray, numvars);
      temp1 = transfunc2(temp, globtrans1, globtrans2, 1); // xtmp translated
      temp2 = transalt1(temp1, paramholdername, paramarray, numparams);
      fprintf(fd2, "  %s[%d] = %s;\n", derivsholdername, m1, temp2);
      free(temp);free(temp1);free(temp2);
    }

    /* get the indices of the left variables - for mass */

    for(j=0;j<allDEs[i].numleft;j++){
      if ((k1 = isinarray(vararray, numvars, allDEs[i].varsleft[j])) < 0) 
	fprintf(stderr, "ERROR in makesys.c: the variable \"%s\" was not found in \"vararray\" in file \"sysfiles/vararray.dat\".\n\n", allDEs[i].varsleft[j]);
      else
	fprintf(fd, "  J[%d][%d] = %s;\n", k1, m1, allDEs[i].valsleft[j]);
    }
  }

  //
  //
  // Algrels
  //
  //
  
  for(i=0;i<numalgrels;i++){ /* for each algrel (only for the RHS) */
    /* get the index of the key variable in question */
    if ((m1 = isinarray(vararray, numvars, allalgrels[i].keyvar)) < 0) 
      fprintf(stderr, "ERROR in makesys.c: the key variable \"%s\" was not found in \"vararray\" in file \"sysfiles/vararray.dat\".\n\n", allalgrels[i].keyvar);
    else{
      temp = transalt1(allalgrels[i].rterm, varholdername, vararray, numvars);
      temp1 = transfunc2(temp, globtrans1, globtrans2, 1); // xtmp translated
      temp2 = transalt1(temp1, paramholdername, paramarray, numparams);
      fprintf(fd1, "  f%s = %s;\n", vararray[m1], allalgrels[i].rterm);
      fprintf(fd3, "%s\n%s = 0\n******\n", vararray[m1], allalgrels[i].rterm);
      fprintf(fd2, "  %s[%d] = %s;\n", derivsholdername, m1, temp2);
      free(temp);free(temp1);free(temp2);
    }

  }


  fprintf(fd1, "}\n\n");
  fprintf(fd2, "}\n\n");
  fprintf(fd, "}\n\n");

  // temporary variables (only readable file...)

  for(i=0;i<numtempvs;i++){
    if(!(isonlyspace(tempexprs[i])))
      fprintf(fd3, "  %s\n%s-%s=0\n******\n", tempvs[i], tempexprs[i], tempvs[i]);
  }

  fclose(fd);
  fclose(fd1);
  fclose(fd2);
  fclose(fd3);
  fclose(fd4);

  freearraydat(tempvs, numtempvs);
  freearraydat(tempexprs, numtempvs);



  fd1 = fopen("sysfiles/tvfuncr.c", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/tvfuncr.c\" could not be opened for writing.\n");
    return -1;
  }
  fd2 = fopen("sysfiles/tvfunc.c", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"makesys.c\": File \"sysfiles/tvfunc.c\" could not be opened for writing.\n");
    return -1;
  }

  fprintf(fd1, "/* This file is generated by the program \"../makesys.c\". It contains the readable version of the function to calculate the temporary variables */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#include \"../src/sys.h\"\nconst double Pi;\n// = 3.14159265;\n\n");
  fprintf(fd1, "void tvfunc(double *_x, double *_tempvs, double *%s, double *%s)\n{\n", varholdername, paramholdername);

  /* the translated rhs file preliminaries */

  fprintf(fd2, "/* This file is generated by the program \"../makesys.c\". It contains the function to calculate the temporary variables */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#include \"../src/sys.h\"\nconst double Pi;\n// = 3.14159265;\n\n");
  fprintf(fd2, "void tvfunc(double *_x, double *_tempvs, double *%s, double *%s)\n{\n", varholdername, paramholdername);



  numtempvs = genarraysdat("sysfiles/alltempstrs2.dat", &tempvs, &tempexprs);
  for(i=0;i<numtempvs;i++){
    if(!(isonlyspace(tempexprs[i]))){
      temp = transalt1(tempexprs[i], varholdername, vararray, numvars);
      temp1 = transfunc2(temp, globtrans1, globtrans2, 1); // xtmp translated
      temp2 = transalt1(temp1, paramholdername, paramarray, numparams);free(temp);
      temp = transalt1(temp2, "_tempvs", tempvs, numtempvs);
      fprintf(fd1, "  %s = %s;\n", tempvs[i], tempexprs[i]);
      fprintf(fd2, "  _tempvs[%d] = %s;\n", i, temp);
      free(temp);free(temp1);free(temp2);
    }
  }
  fprintf(fd1, "}\n");  fprintf(fd2, "}\n");
  fclose(fd1);
  fclose(fd2);
  freearraydat(chemnames, numchems);
  freearraydat(vararray, numvars);

  freearraydat(tempvs, numtempvs);
  freearraydat(tempexprs, numtempvs);
  freearraydat(paramarray, numparams);
  free_algarray(allalgrels, numalgrels);
  free_DEarray(allDEs, numDEs);

  return 0;

}

