/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "radau5/radau/radau.h"
#include "common/stringmanip.h" 
#include "common/nrsimp.h" 
void jac_dummy(int *n, double *x, double *y, double *dfy,
	       int *ldfy, double *rpar, double *ipar);
void rhs(int *n, double *x, double *v, double *fv, double *rpar, int *ipar);
void setval1(char s[], double val, char *paramarray[], int numparams, double *rpar, char *paramfileout, int override, char *setarray[], int numset, double *invals);

//
//
// the morph1 routine
//
//

void morph1(int *n, 
	   void fcn(int*,double*,double*,double*,double*,int*),
	   double *y, double *h, 
	   double *rtol, double *atol, int *itol,
	   void jac(int*, double*, double*, double*, int*, double*, double*),
	   int *ijac, int *mljac, int *mujac,
	   void mas(int *n,double *am, int *lmas, double *rpar, int *ipar),
	   int *imas, int *mlmas, int *mumas,
	   void solout(int*,double*,double*,double*,double*,int*,int*,double*,int*,int*),
	   int *iout,
	   double *work, int *lwork, int *iwork, int *liwork,
	   double *rpar, int *ipar, int *idid, int numparams, char *paramarray[], char *varname, double varstart, double varend, int steps, double tot_time, char *paramfileout, double *invals, char *settable[], int numsettable){

  // paramfileout only used for output by setval
  
  int i=0, flag=0;
  double stepsize;
  double x, xend;

  // settable parameter

  if (isinarray(settable, numsettable, varname) >=0)
    flag=1;

  if(flag==0){
    fprintf(stderr, "ERROR in morph1.cc: variable %s not found, or it's not a settable parameter. Nothing being done.\n", varname);
    return;
  }


  stepsize = (varend-varstart)/steps;
  for (i=0;i<steps;i++){
    setval1(varname, varstart + i*stepsize, paramarray, numparams, rpar, paramfileout, 1, settable, numsettable, invals);
    x=0.0;
    xend = tot_time/steps;

    RADAU5(n,rhs,&x,y,&xend,h,
	   rtol,atol,itol,
	   jac_dummy,ijac,mljac,mujac,
	   mas,imas,mlmas,mumas,
	   solout,iout,
	   work, lwork,iwork,liwork,rpar,ipar,idid);
  }
  // the final step
  x=0.0;
  xend=tot_time/steps;
  setval1(varname, varend, paramarray, numparams, rpar, paramfileout, 1, settable, numsettable, invals);
  RADAU5(n,rhs,&x,y,&xend,h,
	 rtol,atol,itol,
	 jac_dummy,ijac,mljac,mujac,
	 mas,imas,mlmas,mumas,
	 solout,iout,
	 work,lwork,iwork,liwork,rpar,ipar,idid);
}


//
//
// the multimorph1 routine
//
//

void multimorph1(int *n, 
	   void fcn(int*,double*,double*,double*,double*,int*),
	   double *y, double *h, 
	   double *rtol, double *atol, int *itol,
	   void jac(int*, double*, double*, double*, int*, double*, double*),
	   int *ijac, int *mljac, int *mujac,
	   void mas(int *n,double *am, int *lmas, double *rpar, int *ipar),
	   int *imas, int *mlmas, int *mumas,
	   void solout(int*,double*,double*,double*,double*,int*,int*,double*,int*,int*),
	   int *iout,
	   double *work, int *lwork, int *iwork, int *liwork,
	   double *rpar, int *ipar, int *idid, int numparams, char *paramarray[], char *varname[], double morphvals[], int nummph, int steps, double tot_time, char *paramfileout, double *invals, char *settable[], int numsettable){

  // paramfileout only used for output by setval
  
  int i=0, j;
  double *stepsize;
  double x, xend;
  int *flag;

  flag = ivector(0, nummph-1);
  stepsize = dvector(0, nummph-1);

  for(j=0;j<nummph;j++){
    flag[j]=0;
    if (isinarray(settable, numsettable, varname[j]) >=0)
      flag[j]=1;
    if(flag[j]==0){
      fprintf(stderr, "WARNING in multimorph1.cc: %s unavailable or not settable. Nothing being done to it.\n", varname[j]);
    }
  }

  for(j=0;j<nummph;j++){
    if(flag[j] == 1)
      stepsize[j] = (morphvals[2*j+1]-morphvals[2*j])/steps;
  }
  for (i=0;i<steps;i++){
    for(j=0;j<nummph;j++){
      if(flag[j] == 1)
	setval1(varname[j], morphvals[2*j] + i*stepsize[j], paramarray, numparams, rpar, paramfileout, 1, settable, numsettable, invals);
    }
    x=0.0;
    xend = tot_time/steps;

    RADAU5(n,rhs,&x,y,&xend,h,
	   rtol,atol,itol,
	   jac_dummy,ijac,mljac,mujac,
	   mas,imas,mlmas,mumas,
	   solout,iout,
	   work, lwork,iwork,liwork,rpar,ipar,idid);
  }
  // the final step
  x=0.0;
  xend=tot_time/steps;
  for(j=0;j<nummph;j++){
    if(flag[j] == 1)
      setval1(varname[j], morphvals[2*j+1], paramarray, numparams, rpar, paramfileout, 1, settable, numsettable, invals);
  }
  RADAU5(n,rhs,&x,y,&xend,h,
	 rtol,atol,itol,
	 jac_dummy,ijac,mljac,mujac,
	 mas,imas,mlmas,mumas,
	 solout,iout,
	 work,lwork,iwork,liwork,rpar,ipar,idid);

  free_dvector(stepsize, 0, nummph-1);
  free_ivector(flag, 0, nummph-1);
}

void soldummy(int *nr,double *xold,double *x,
	     double *y,
	     double *cont,int *lrc,
	     int *n,
	     double *rpar,int *ipar,
	     int *irtrn)
{
  // the dummy solout function 
}

void jac_dummy(int *n, double *x, double *y, double *dfy,
		   int *ldfy, double *rpar, double *ipar)
{
	
}
