/* these ludecomposition and backsubstiution routines are taken from numerical recipes. But we assume everything is 0-offset We also make all floats doubles */

#include <math.h>
#include "../common/nrsimp.h"
#define TINY 1.0e-20

void ludcmp(double **a, int n, int *indx, double *d)

{

    int i, imax, j, k;
    double big, dum, sum, temp;
    double *vv;

    vv = dvector(0, n-1);
    *d=1.0;
    for(i=0;i<n;i++){
      big = 0.0;
      for(j=0;j<n;j++)
	if ((temp=fabs(a[i][j])) > big) big=temp;
      if (big == 0.0) nrerror("Singular matrix in routine ludcmp");
      vv[i]=1.0/big;
    }
    //   pp(1);
    for(j=0;j<n;j++){
      for (i=0;i<j;i++){
	sum=a[i][j];
	for(k=0;k<i;k++) sum -= a[i][k]*a[k][j];
	a[i][j]=sum;
      }
      big=0.0;
      for (i=j;i<n;i++){
	sum=a[i][j];
	for(k=0;k<j;k++)
	  sum -= a[i][k]*a[k][j];
	a[i][j]=sum;
	if ((dum=vv[i]*fabs(sum)) >= big) {
	  big=dum;
	  imax=i;
	}
      }
      if (j != imax) {
	for(k=0;k<n;k++) {
	  dum=a[imax][k];
	  a[imax][k]=a[j][k];
	  a[j][k]=dum;
	}
	*d = -(*d);
	vv[imax]=vv[j];
      }
      indx[j]=imax;
      if (a[j][j] == 0.0) a[j][j]=TINY;
      if (j!=n-1){
	dum=1.0/(a[j][j]);
	for (i=j+1;i<n;i++) a[i][j] *= dum;
      }
    }
    free_dvector(vv, 0, n-1);
}

void lubksb(double **a, int n, int *indx, double b[])
{
  int i, ii=0, ip, j;
  double sum;

  for(i=0;i<n;i++){
    ip=indx[i];
    sum=b[ip];
    b[ip]=b[i];
    if (ii)
      for (j=ii;j<=i-1;j++) sum -=a[i][j]*b[j];
    else if (sum) ii=i;
    b[i]=sum;
  }
  for (i=n-1;i>=0;i--){
    sum=b[i];
    for(j=i+1;j<n;j++) sum -= a[i][j]*b[j];
    b[i]=sum/a[i][i];
  }
}



