// Taken from Numerical Recipes, p382, but everything is zero offset and we make all floats doubles.

#include <stdio.h>
#include <math.h>
#include "../common/nrsimp.h"

#define FREERETURN {free_dmatrix(fjac, 0, n-1, 0, n-1);free_dvector(fvec, 0, n-1);free_dvector(p, 0, n-1);free_ivector(indx, 0, n-1);return;}
#define EPS 1.0e-4

void ludcmp(double **a, int n, int *indx, double *d);
void lubksb(double **a, int n, int *indx, double b[]);

void mnewt(int ntrial, double x[], int n, double tolx, double tolf, double *otherparams, void (*usrfcn)(double *, int, double *, double **, double *)){
  //Given an initial guess x[0..n-1] for a root in n dimensions take ntrial Newton Raphson steps to improve the root. Stop if the root converges in either summed absolute variable increments tolx or summed absolute function values tolf.

  int k, i, *indx;
  double errx, errf, d, *fvec, **fjac, *p;
  indx=ivector(0, n-1);
  p=dvector(0, n-1);
  fvec=dvector(0, n-1);
  fjac=dmatrix(0, n-1, 0, n-1);

  for(k=0;k<ntrial;k++){
    for(i=0;i<n;i++)
      fprintf(stderr, "x[%d] = %.4f\t", i, x[i]);
    if(n>0)
      fprintf(stderr, "\n");
    else
      fprintf(stderr, "*");
    usrfcn(x, n, fvec, fjac, otherparams);
    errf=0.0;
    for(i=0;i<n;i++) errf += fabs(fvec[i]);
    if(errf <= tolf) {FREERETURN}
    for(i=0;i<n;i++) p[i] = -fvec[i];

    ludcmp(fjac, n, indx, &d);
    lubksb(fjac, n, indx, p);

    errx = 0.0;
    for(i=0;i<n;i++){
      errx += fabs(p[i]);
      x[i] += p[i];
    }
    if (errx<=tolx) {
      FREERETURN
    }
  }
  FREERETURN
}

void fdjac(int n, double x[], double fvec[], double **df, double otherparams[], void (*vecfunc)(int, double [], double [], double []))
     // Computes forward difference approximation to the Jacobian. ON input x[0..n-1] is the point at which the Jacobian is to be evaluated, fvec [0..n-1] is the vector of function values at the point, and vecfunc(n, x, f) is a user supplied routine that returns the vector of functions at x. On output, df[0..n-1][0..n-1] is the Jacobian array
{
  int i, j;
  double h, temp, *f;
  f=dvector(0,n-1);
  for(j=0;j<n;j++){
    temp = x[j];
    h=EPS*fabs(temp);
    if (fabs(h)<1.0e-8) h=EPS; 
    x[j] = temp+h;
    h=x[j]-temp;
    (*vecfunc)(n, x, f, otherparams);
    x[j]=temp;
    for(i=0;i<n;i++) df[i][j]=(f[i]-fvec[i])/h; // forward difference formula
  }
  free_dvector(f, 0, n-1);
}
