#include "../common/stringmanip.h"
#include "../common/nrsimp.h"

const double Pi = 3.14159265; 
void pfunc(double *pvals, double *outvals);

double heavi(double s)
{
  return (s < 0.0) ? 0.0 : 1.0;
}


#define MAXPARS 800
#include "sys/vecfcnfile.c" // includes solvevars and the vecfcn

void fdjac(int n, double x[], double fvec[], double **df, double *otherparams, void (*vecfunc)(int, double [], double [], double []));

void usrfun(double *x, int n, double *fvec, double **fjac, double *otherparams)
{
  vecfcn(n, x, fvec, otherparams);
  fdjac(n, x, fvec, fjac, otherparams, vecfcn);
}

void mnewt(int ntrial, double x[], int n, double tolx, double tolf, double *otherparams, void (*usrfcn)(double *, int, double *, double **, double *)); // in the library

int main(){
  int i;
  double *x;
  double tolx = 0.000001;
  double tolf = 0.000001;

  FILE *fd;
  char *outputfile = "sys/solve_params.dat";
  double *p;
  char **paramarray;
  int numparams=0;
  double *rpar;
  char **settable;
  int numsettable = 0;
  double *setpvals;

  settable = genarraydat("../../sysfiles/settable.dat", &numsettable);
  paramarray = genarraydat("../../sysfiles/params.dat", &numparams);
  setpvals = dvector(0, numsettable-1);
  p = dvector(0, numparams-1);
  rpar = dvector(0, numparams-1);



  setparamsfromfile("../../userfiles/current_pvals.dat", settable, numsettable, setpvals, rpar, pfunc);

  for(i=0;i<numparams;i++)
    p[i] = rpar[i];

  fd = fopen(outputfile, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"newton/test.cc\": File %s could not be opened for writing.\n", outputfile);
    return -1;
  }

  x = dvector(0, numsolvevars-1);

#include "sys/initfcnfile.c"

  mnewt(10, x, numsolvevars, tolx, tolf, p, usrfun);

  for(i=0;i<numsolvevars;i++){
    fprintf(fd, "%s\t%.6e\n", solvevars[i], x[i]);
  }

  fclose(fd);

  freearraydat(paramarray, numparams);
  freearraydat(settable, numsettable);
  free_dvector(setpvals, 0, numsettable-1);
  free_dvector(p, 0, MAXPARS-1);
  free_dvector(x, 0, numsolvevars-1);
  free_dvector(rpar, 0, numparams-1);
  return 0;
}



