#include <stdio.h>
#include <math.h>
#include "../common/nrsimp.h"

#define TINY 1.0e-20
#define FREERETURN {free_dmatrix(fjac, 0, n-1, 0, n-1);free_dvector(fvec, 0, n-1);free_dvector(p, 0, n-1);free_ivector(indx, 0, n-1);return;}

void usrfun(double *x, int n, double *fvec, double **fjac);
void mnewt(int ntrial, double x[], int n, double tolx, double tolf, void (*usrfcn)(double *, int, double *, double **)); // in the library

void ludcmp(double **a, int n, int *indx, double *d);
void lubksb(double **a, int n, int *indx, double b[]);

int main(){
  int i;
  double *x;
  double tolx = 0.001;
  double tolf = 0.001;

  x = dvector(0, 1);

  x[0] = 2.0;
  x[1] = 1.0;

  printf("intial: x = %.4f\ty = %.4f\n", x[0], x[1]);
  mnewt(10, x, 2, tolx, tolf, usrfun);

  free_dvector(x, 0, 1);

  return 0;
}

void usrfun(double *x, int n, double *fvec, double **fjac)
{
  fvec[0] = x[0]*x[0] + x[1]*x[1] - 4.0;
  fvec[1] = x[0]*x[1] - 1.0;

  fjac[0][0] = 2.0*x[0];
  fjac[0][1] = 2.0*x[1];
  fjac[1][0] = x[1];
  fjac[1][1] = x[0];
}

void ludcmp(double **a, int n, int *indx, double *d)

{

    int i, imax, j, k;
    double big, dum, sum, temp;
    double *vv;

    vv = dvector(0, n-1);
    *d=1.0;
    for(i=0;i<n;i++){
      big = 0.0;
      for(j=0;j<n;j++)
	if ((temp=fabs(a[i][j])) > big) big=temp;
      if (big == 0.0) nrerror("Singular matrix in routine ludcmp");
      vv[i]=1.0/big;
    }
    for(j=0;j<n;j++){
      for (i=0;i<j;i++){
	sum=a[i][j];
	for(k=0;k<i;k++) sum -= a[i][k]*a[k][j];
	a[i][j]=sum;
      }
      big=0.0;
      for (i=j;i<n;i++){
	sum=a[i][j];
	for(k=0;k<j;k++)
	  sum -= a[i][k]*a[k][j];
	a[i][j]=sum;
	if ((dum=vv[i]*fabs(sum)) >= big) {
	  big=dum;
	  imax=i;
	}
      }
      if (j != imax) {
	for(k=0;k<n;k++) {
	  dum=a[imax][k];
	  a[imax][k]=a[j][k];
	  a[j][k]=dum;
	}
	*d = -(*d);
	vv[imax]=vv[j];
      }
      indx[j]=imax;
      if (a[j][j] == 0.0) a[j][j]=TINY;
      if (j!=n-1){
	dum=1.0/(a[j][j]);
	for (i=j+1;i<n;i++) a[i][j] *= dum;
      }
    }
    free_dvector(vv, 0, n-1);
}

void lubksb(double **a, int n, int *indx, double b[])
{
  int i, ii=0, ip, j;
  double sum;

  for(i=0;i<n;i++){
    ip=indx[i];
    sum=b[ip];
    b[ip]=b[i];
    if (ii)
      for (j=ii;j<=i-1;j++) sum -=a[i][j]*b[j];
    else if (sum) ii=i;
    b[i]=sum;
  }
  for (i=n-1;i>=0;i--){
    sum=b[i];
    for(j=i+1;j<n;j++) sum -= a[i][j]*b[j];
    b[i]=sum/a[i][i];
  }
}

void mnewt(int ntrial, double x[], int n, double tolx, double tolf, void (*usrfun)(double *, int, double *, double **)){
  //Given an initial guess x[0..n-1] for a root in n dimensions take ntrial Newton Raphson steps to improve the root. Stop if the root converges in either summed absolute variable increments tolx or summed absolute function values tolf.

  int k, i, *indx;
  double errx, errf, d, *fvec, **fjac, *p;
  indx=ivector(0, n-1);
  p=dvector(0, n-1);
  fvec=dvector(0, n-1);
  fjac=dmatrix(0, n-1, 0, n-1);

  for(k=0;k<ntrial;k++){
    printf("x = %.4f\ny=%.4f\n", x[0], x[1]);
    usrfun(x, n, fvec, fjac);
    errf=0.0;
    for(i=0;i<n;i++) errf += fabs(fvec[i]);
    if(errf <= tolf) {FREERETURN}
    for(i=0;i<n;i++) p[i] = -fvec[i];

    ludcmp(fjac, n, indx, &d);
    lubksb(fjac, n, indx, p);
    errx = 0.0;
    for(i=0;i<n;i++){
      errx += fabs(p[i]);
      x[i] += p[i];
    }
    if (errx<=tolx) {FREERETURN}
  }
  FREERETURN

}
