/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */


// Translates the various parameter functions into a single function

#include "common/stringmanip.h"

int main()
{
  FILE *fd, *fd1;
  int i, l, numtvs;
  char *tempv[2000];
  char block[2000];
  char block1[2000];
  char *block2;
  char *block3;
  char *block4;
  int numo2p=0;
  char **o2parray;
  char **o2deps;
  char **paramarray;
  int numneeded = 0;
  char **settable;
  int numsettable = 0;
  char **ratenamearray;
  char **ratearray;
  int numrates;
  char **vararray;
  int numvars=0;


  //
  //
  // First the parameters
  //
  //

  fd = fopen("sysfiles/paramfunc.c", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in oneparamfunc.c: file: \"sysfiles/paramfunc.c\" could not be opened for writing...\n");
    return -1;
  }

  fd1 = fopen("sysfiles/paramfuncr.c", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in oneparamfunc.c: file: \"sysfiles/paramfuncr.c\" could not be opened for writing...\n");
    return -1;
  }

  numrates = genarraysdat("sysfiles/reacratesn.dat", &ratenamearray, &ratearray);
  paramarray = genarraydat("sysfiles/params.dat", &numneeded);
  settable = genarraydat("sysfiles/settable.dat", &numsettable);
  vararray = genarraydat("sysfiles/vararray.dat", &numvars);

  numo2p = genarraysdat("sysfiles/ordparams.dat", &o2parray, &o2deps);
  numtvs = gettvs("sysfiles/o2funcsr.cc", tempv, 2000);

  fprintf(fd1, "/* This file is generated by the program \"../oneparamfunc.c\". It contains the parameter setting function in an untranslated form */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#define max(A, B) ((A) > (B) ? (A) : (B))\n#define min(A, B) ((A) < (B) ? (A) : (B))\n#define heavi(A) ((A) < (0.0) ? (0.0) : (1.0))\nconst double Pi;\n// = 3.14159265;\ndouble setkvals1(double K_eq, double halftime);\ndouble setkvals2(double K_eq, double halftime);\ndouble setkvals1acid(double K_eq, double halftime, double normdiss);\ndouble setkvals2acid(double K_eq, double halftime, double normdiss);\n\n");
  fprintf(fd1, "void pfunc(double *pvals, double *outvals)\n{\n");

  fprintf(fd, "/* This file is generated by the program \"../oneparamfunc.c\". It contains the parameter setting function */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#define max(A, B) ((A) > (B) ? (A) : (B))\n#define min(A, B) ((A) < (B) ? (A) : (B))\n#define heavi(A) ((A) < (0.0) ? (0.0) : (1.0))\nconst double Pi;\n// = 3.14159265;\ndouble setkvals1(double K_eq, double halftime);\ndouble setkvals2(double K_eq, double halftime);\ndouble setkvals1acid(double K_eq, double halftime, double normdiss);\ndouble setkvals2acid(double K_eq, double halftime, double normdiss);\n\n");
  fprintf(fd, "void pfunc(double *pvals, double *outvals)\n{\n");


  for(i=0;i<numtvs;i++)
    fprintf(fd, "  double %s;\n", tempv[i]);

  for(i=0;i<numo2p;i++){
    if(isinarray(paramarray, numneeded, o2parray[i])<0)
      fprintf(fd, "  double %s;\n", o2parray[i]);
  }

  for(i=0;i<numo2p;i++){

    if(findfunc1("sysfiles/o2funcsr.cc", o2parray[i], block, 2000, 3)==0){
      transfsubf(block, o2parray[i], block1, 2000);
      block2 = transfunc1(block1, "n", vararray, numvars);
    }
    else if((l = isinarray(ratenamearray, numrates, o2parray[i])) >= 0){
      block2 = (char*) malloc(sizeof(char) * (strlen(o2parray[i]) + strlen(ratearray[l]) + 10));
      sprintf(block2, "\n  %s = %s;\n", o2parray[i], ratearray[l]); // safe
    }
    else{
      fprintf(stderr, "WARNING in oneparamfunc.c: %s doesn't seem to have an associated function.\n", o2parray[i]);
      block2 = strdup("");
    }
    block3 = transalt1(block2, "pvals", settable, numsettable);
    block4 = transalt1(block3, "outvals", paramarray, numneeded);

    fprintf(fd, "%s\n", block4);
    fprintf(fd1, "%s\n", block2);
    free(block2);free(block3);free(block4);

  }

  for(i=0;i<numneeded;i++){
    if((l = isinarray(settable, numsettable, paramarray[i])) >= 0){ // settable
      fprintf(fd, "  outvals[%d] = pvals[%d];\n", i, l);
    }
    //    else{
    //      fprintf(fd, "  outvals[%d] = %s;\n", i, paramarray[i]);
    //    }
  }

  fprintf(fd1, "}\n\n");
  fprintf(fd, "}\n\n");

  fclose(fd);
  fclose(fd1);

  free_datarray(tempv, numtvs);
  freearraydat(o2parray, numo2p);
  freearraydat(o2deps, numo2p); 
  freearraydat(ratenamearray, numrates);
  freearraydat(ratearray, numrates);
  //
  //
  // Now variable initialisation
  //
  //

  fd = fopen("sysfiles/varfunc.c", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in oneparamfunc.c: file: \"sysfiles/paramfunc.c\" could not be opened for writing...\n");
    return -1;
  }

  fd1 = fopen("sysfiles/varfuncr.c", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in oneparamfunc.c: file: \"sysfiles/paramfuncr.c\" could not be opened for writing...\n");
    return -1;
  }

  numo2p = genarraysdat("sysfiles/ordvarbles.dat", &o2parray, &o2deps);
  numtvs = gettvs("sysfiles/initfuncsr.cc", tempv, 2000);

  numrates = genarraysdat("sysfiles/reacratesn1.dat", &ratenamearray, &ratearray);

  fprintf(fd1, "/* This file is generated by the program \"../oneparamfunc.c\". It contains the variable initialisation function in an untranslated form */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#define max(A, B) ((A) > (B) ? (A) : (B))\n#define min(A, B) ((A) < (B) ? (A) : (B))\n#define heavi(A) ((A) < (0.0) ? (0.0) : (1.0))\nconst double Pi;\n// = 3.14159265;\ndouble setkvals1(double K_eq, double halftime);\ndouble setkvals2(double K_eq, double halftime);\ndouble setkvals1acid(double K_eq, double halftime, double normdiss);\ndouble setkvals2acid(double K_eq, double halftime, double normdiss);\n\n");
  fprintf(fd1, "void vfunc(double *pvals, double *outvals)\n{\n");

  fprintf(fd, "/* This file is generated by the program \"../oneparamfunc.c\". It contains the variable initialisation function */\n\n#include <stdio.h>\n#include <stdlib.h>\n#include <math.h>\n#define max(A, B) ((A) > (B) ? (A) : (B))\n#define min(A, B) ((A) < (B) ? (A) : (B))\n#define heavi(A) ((A) < (0.0) ? (0.0) : (1.0))\nconst double Pi;\n// = 3.14159265;\ndouble setkvals1(double K_eq, double halftime);\ndouble setkvals2(double K_eq, double halftime);\ndouble setkvals1acid(double K_eq, double halftime, double normdiss);\ndouble setkvals2acid(double K_eq, double halftime, double normdiss);\n\n");
  fprintf(fd, "void vfunc(double *pvals, double *outvals)\n{\n");


  for(i=0;i<numtvs;i++)
    fprintf(fd, "  double %s;\n", tempv[i]);

  //  for(i=0;i<numo2p;i++)
  //    fprintf(fd, "  double %s;\n", o2parray[i]);

  for(i=0;i<numo2p;i++){
    if(findfunc1("sysfiles/initfuncsr.cc", o2parray[i], block, 2000, 3)==0){
      block3 = transfunc1(block, "n", vararray, numvars); // to be safe
      transfsubf(block3, o2parray[i], block1, 2000);
      free(block3);
    }
    else if((l = isinarray(ratenamearray, numrates, o2parray[i])) >= 0){
      snprintf(block1, 2000, "\n  %s = %s;\n", o2parray[i], ratearray[l]);
    }
    else{
      fprintf(stderr, "WARNING in oneparamfunc.c: %s doesn't seem to have an associated function.\n", o2parray[i]);
    }

    fprintf(fd1, "%s\n", block1);
    block2 = transalt1(block1, "pvals", paramarray, numneeded);
    block3 = transalt1(block2, "outvals", vararray, numvars);
    fprintf(fd, "%s\n", block3);
    //    fprintf(fd, "%s\n", block2);
    free(block2); free(block3);

  }

  //  for(i=0;i<numvars;i++){
  //    fprintf(fd, "  outvals[%d] = %s;\n", i, vararray[i]);
  //  }

  fprintf(fd1, "}\n\n");
  fprintf(fd, "}\n\n");

  fclose(fd);
  fclose(fd1);



  free_datarray(tempv, numtvs);
  freearraydat(o2parray, numo2p);
  freearraydat(o2deps, numo2p);  
  freearraydat(paramarray, numneeded);
  freearraydat(settable, numsettable);
  freearraydat(vararray, numvars);
  freearraydat(ratenamearray, numrates);
  freearraydat(ratearray, numrates);

  return 0;
}
