#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "syssize.h"

// like setindmulti4.c, except that we assume that there is a linear relationship between r_0i and r_mi. r_t1 and r_t2 not used for fitting. 

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


// the number of independent quantities being fit.
#define NQ 2
// the number of parameters whose values will actually be changed
#define NP 2
char *fitparam[2] = {"b", "c"};

int numpts=13;
double xvals[13] = {84, 86, 86, 83, 86, 87, 87, 86, 84, 77, 78, 87, 86};
double yvals[13] = {1002, 912, 932, 918, 943, 991, 1111, 991, 976, 865, 921, 1078, 1112};


// b is the steepness, c is the centre

double expfn(double x, double b, double c){
  return 1.0/(1.0+ exp(b*(x-c)));
}

double ptdist2(double x, double y, double b, double c, double (*expfn)(double x, double b, double c)){
  double k = y-expfn(x, b, c);
  return k*k;
}

double ptdistall(double x[], double y[], int numpts, double b, double c){
  double ret=0;
  int i;
  for(i=0;i<numpts;i++)
    ret+=ptdist2(x[i], y[i], b, c, expfn);
 
  return sqrt(ret);
}



double amotsa(double xv[], double yv[], int numpts, double vvals[], double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*ptdistall)(double x[], double y[], int numpts, double b, double c), int ihi, double *yhi, double fac);


double ran1(long *idum);

void amebsa(double xv[], double yv[], int numpts, double vinit[], double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*ptdistall)(double x[], double y[], int numpts, double b, double c), int *iter, double temptr);

#define GET_PSUM \
  for (n=1;n<=ndim;n++) {\
  for (sum=0.0,m=1;m<=mpts;m++) sum += p[m][n];\
  psum[n]=sum;}
long idum;
double tt;


int main(){

  int i, j;
  double vinit[NP];
  int iter;
  double temptr;

  double **pinit;
  double *finit;
  double *tempv;
  double *pb;
  double yb;
  double xvtot=0;
  double yvtot=0;

  // rescale the values

  for(i=0;i<numpts;i++){
    xvtot+=xvals[i];
    yvtot+=yvals[i];
  }

  fprintf(stderr, "xav = %.4e\tyav = %.4e\n", xvtot/numpts, yvtot/numpts);


  for(i=0;i<numpts;i++){
    xvals[i] = xvals[i]*numpts/xvtot;
    yvals[i] = yvals[i]*numpts/yvtot;
  }

// initialisation of b and c

  vinit[0]=-1.0;
  vinit[1]=-1.0;


  // specific to this particular fitting

  pinit=dmatrix(1, NQ+1, 1, NQ); 
  finit=dvector(1, NQ+1);
  tempv=dvector(1, NQ);
  pb=dvector(1, NQ);



  // p is the key variable which stores the corrections to the values of b and c

  for(i=1;i<=NQ+1;i++){
    for(j=1;j<=NQ;j++){
      if(i==1)
	pinit[i][j] = 1.0;
      else if(i==j+1) // scale 
	pinit[i][j]=2.0;
      else
	pinit[i][j]=1.0;
    }
  }

  //
  // initial values of fitting at corners of the simplex
  //

  for(i=1;i<=NQ+1;i++){
    for(j=1;j<=NQ;j++)
      tempv[j] = pinit[i][j];
    finit[i] = ptdistall(xvals, yvals, numpts, vinit[0]*tempv[1], vinit[1]*tempv[2]);
    fprintf(stderr, "finit[i] = %.4e\n", finit[i]);
  }

  iter = 1000;
  temptr = 2.0;
  //temptr = 2.0;
  yb=100; // so that the best value is stored. 
  for(i=0;i<5;i++){
    iter=20000; //=200 normally
    //    iter=12-2*i;
    temptr *=0.5;
    amebsa(xvals, yvals, numpts, vinit, pinit, finit, NQ, pb, &yb, 0.0001, ptdistall, &iter, temptr);
    fprintf(stderr, "*\n*\n*\niteration %d\tBest so far: ", i+1);
    fprintf(stderr, "b = %.4e\n", vinit[0]*pb[1]);
    fprintf(stderr, "c = %.4e\n", vinit[1]*pb[2]);
    fprintf(stderr, "fit = %.4e\n", yb);

  }

  free_dmatrix(pinit, 1, NQ+1, 1, NQ);
  free_dvector(finit, 1, NQ+1);
  free_dvector(tempv, 1, NQ);
  free_dvector(pb, 1, NQ);

  return 0;
}



// these routines are all adapted from numerical recipes in C

double ran1(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;

  if (*idum <= 0 || !iy) {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum < 0) *idum += IM;
      if (j < NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}

// This core routine performs one fitting step at a given temperature


void amebsa(double xv[], double yv[], int numpts, double vinit[], double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*ptdistall)(double x[], double y[], int numpts, double b, double c), int *iter, double temptr)
{
  int i,ihi,ilo,j,m,n,mpts=ndim+1;
  double rtol,sum,swap,yhi,ylo,ynhi,ysave,yt,ytry,*psum;

  psum=dvector(1,ndim);
  tt = -temptr;
  GET_PSUM
    for (;;) {
      ilo=1;
      ihi=2;
      ynhi=ylo=y[1]+tt*log(ran1(&idum));
      yhi=y[2]+tt*log(ran1(&idum));
      if (ylo > yhi) {
	ihi=1;
	ilo=2;
	ynhi=yhi;
	yhi=ylo;
	ylo=ynhi;
      }
      for (i=3;i<=mpts;i++) {
	yt=y[i]+tt*log(ran1(&idum));
	if (yt <= ylo) {
	  ilo=i;
	  ylo=yt;
	}
	if (yt > yhi) {
	  ynhi=yhi;
	  ihi=i;
	  yhi=yt;
	} else if (yt > ynhi) {
	  ynhi=yt;
	}
      }
      rtol=2.0*fabs(yhi-ylo)/(fabs(yhi)+fabs(ylo));
      if (rtol < ftol || *iter < 0) {
	swap=y[1];
	y[1]=y[ilo];
	y[ilo]=swap;
	for (n=1;n<=ndim;n++) {
	  swap=p[1][n];
	  p[1][n]=p[ilo][n];
	  p[ilo][n]=swap;
	}
	break;
      }
      *iter -= 2;
      ytry=amotsa(xv, yv, numpts, vinit, p,y,psum,ndim,pb,yb,ptdistall,ihi,&yhi,-1.0);
      if (ytry <= ylo) {
	ytry=amotsa(xv, yv, numpts, vinit, p,y,psum,ndim,pb,yb,ptdistall,ihi,&yhi,2.0);
      } else if (ytry >= ynhi) {
	ysave=yhi;
	ytry=amotsa(xv, yv, numpts, vinit, p,y,psum,ndim,pb,yb,ptdistall,ihi,&yhi,0.5);
	if (ytry >= ysave) {
	  for (i=1;i<=mpts;i++) {
	    if (i != ilo) {
	      for (j=1;j<=ndim;j++) {
		psum[j]=0.5*(p[i][j]+p[ilo][j]);
		p[i][j]=psum[j];
	      }
	      y[i]=(*ptdistall)(xv, yv, numpts, vinit[0]*psum[1], vinit[1]*psum[2]);
	    }
	  }
	  *iter -= ndim;
	  GET_PSUM
	    }
      } else ++(*iter);
    }
  free_dvector(psum,1,ndim);
}


double amotsa(double xv[], double yv[], int numpts, double vvals[], double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*ptdistall)(double x[], double y[], int numpts, double b, double c), int ihi, double *yhi, double fac)
{
  int j;
  double fac1,fac2,yflu,ytry,*ptry;

  ptry=dvector(1,ndim);
  fac1=(1.0-fac)/ndim;
  fac2=fac1-fac;
  for (j=1;j<=ndim;j++)
    ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
  ytry=(*ptdistall)(xv, yv, numpts, vvals[0]*ptry[1], vvals[1]*ptry[2]);
  if (ytry <= *yb) {
    for (j=1;j<=ndim;j++) pb[j]=ptry[j];
    *yb=ytry;
    fprintf(stderr, "yb = %.4e\n", *yb);
  }
  yflu=ytry-tt*log(ran1(&idum));
  if (yflu < *yhi) {
    y[ihi]=ytry;
    *yhi=yflu;
    for (j=1;j<=ndim;j++) {
      psum[j] += ptry[j]-p[ihi][j];
      p[ihi][j]=ptry[j];
    }
  }
  free_dvector(ptry,1,ndim);
  return yflu;
}

