#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "syssize.h"

// like setindmulti4.c, except that we assume that there is a linear relationship between r_0i and r_mi. r_t1 and r_t2 not used for fitting. 

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


// healthiest group
/* int numpts=13; */
/* double xvals[13] = {84, 86, 86, 83, 86, 87, 87, 86, 84, 77, 78, 87, 86}; */
/* double yvals[13] = {1002, 912, 932, 918, 943, 991, 1111, 991, 976, 865, 921, 1078, 1112}; */

// next group

/* int numpts=13; */
/* double xvals[13] = {73, 85, 87, 83, 84, 84, 78, 89, 70, 77, 77, 79, 77}; */
/* double yvals[13] = {787, 822, 811, 775, 812, 823, 779, 821, 776, 765, 812, 832, 798}; */

// next group

/* int numpts=12; */
/* double xvals[12] = {81, 74, 72, 72, 69, 77, 69, 74, 72, 73, 76, 71}; */
/* double yvals[12] = {613, 545, 571, 513, 567, 601, 531, 499, 543, 521, 600, 611}; */

// worst group

/* int numpts=10; */
/* double xvals[10] = {68, 60, 59, 64, 67, 60, 57, 66, 59, 75}; */
/* double yvals[10] = {321, 289, 291, 333, 301, 287, 199, 314, 231, 201}; */

// intervention group before intervention

/* int numpts = 8; */
/* double xvals[8] = {71, 75, 69, 74, 72, 71, 78, 72}; */
/* double yvals[8] = {566, 698, 529, 717, 569, 632, 660, 701}; */

// intervention group after intervention

int numpts = 8;
double xvals[8] = {73, 77, 71, 70, 69, 72, 77, 71};
double yvals[8] = {734, 779, 730, 711, 667, 729, 801, 689};

// Persson dogs
// a value of 8 mmHg has been subtracted from arterial pressure in each case to get RPP
/* int numpts=11; */
/* // double xvals[11] = {112, 102, 91, 85, 80, 70, 60, 50, 40, 31, 16}; */
/* double xvals[11] = {104, 94, 83, 77, 72, 62, 52, 42, 32, 23, 8}; */
/* double yvals[11] = {2.45, 2.4, 2.47, 2.5, 2.4, 2.4, 2.1, 1.65, 1.35, 0.7, 0.0}; */


// b is the steepness, c is the centre (point of inflection) (Turkstra functions)

/* double expfn(double x, double b, double c){ */
/*   return 1.0/(1.0+ exp(b*(x-c))); */
/* } */

// Michaelis menten type functions

double fn2(double x, double b, double K){
  return 0.0000809*x*pow((x*(1.0 - 0.95*pow((x+K), 2)/(pow(b, 2) + pow((x+K), 2)))), 4);
}

double expfn(double x, double b, double c){
  return b*x/(c + x);
}

double ptdist2(double x, double y, double b, double c, double (*expfn)(double x, double b, double c)){
  double k = y-expfn(x, b, c);
  return k*k;
}

double ptdistall(double x[], double y[], int numpts, double b, double c, double (*expfn)(double x, double b, double c)){
  double ret=0;
  int i;
  for(i=0;i<numpts;i++)
    ret+=ptdist2(x[i], y[i], b, c, expfn);
 
  return sqrt(ret);
}

// double gridb1=0.0;
// double gridb2=-5.0;
// double gridc1=0.01;
// double gridc2=5.0;
double gridb1=40.9;
double gridb2=40.9;
double gridc1=-1;
double gridc2=30;
int gridsizeb=1;
int gridsizec=1000;

int main(){

  int i, j;
  double xvtot=0;
  double yvtot=0;
  double xav, yav;
  double fit;
  double fitbest=10000;
  double should;
  double bval, cval;
  double binc, cinc;
  double bbest, cbest;

  binc = (gridb2 - gridb1)/gridsizeb;
  cinc = (gridc2 - gridc1)/gridsizec;

  for(i=0;i<numpts;i++){
    xvtot+=xvals[i];
    yvtot+=yvals[i];
  }
  xav = xvtot/numpts;
  yav=yvtot/numpts;

  fprintf(stderr, "xav = %.4e\tyav = %.4e\n", xav, yav);

/*   for(i=0;i<numpts;i++){ */
/*     xvals[i] = xvals[i]*numpts/xvtot; */
/*     yvals[i] = yvals[i]*numpts/yvtot; */
/*   } */

  for(i=0;i<gridsizeb;i++){
    bval= gridb1 + i*binc;
    for(j=0;j<gridsizec;j++){
      cval = gridc1 + j*cinc;
      fit = ptdistall(xvals, yvals, numpts, bval, cval, fn2);
      if(fit< fitbest){
/* 	should = xav*(log(2.0-sqrt(3))+bval*cval)/bval; */
/* 	fprintf(stderr, "b=%.4e\tc=%.4e\tfit=%.4e\tshoulder = %.4e\n", bval, cval, fit, should); */
	fprintf(stderr, "b=%.4e\tc=%.4e\tfit=%.4e\n", bval, cval, fit);
	bbest=bval;cbest=cval;
	fitbest=fit;
      }
    }

  }
  //  fprintf(stderr, "function = %.4e*x/(%.4e + x)\n", bbest*yav, cbest*xav);

  return 0;
}




