/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// This file uses the information from
// sysfiles/paramarray.dat and sysfiles/vararray.dat
// to generate the cleaned, reduced versions of 
// params.dat, o2funcsr.cc, initfuncs.cc

#include "common/stringmanip.h"
#include "syssize.h" // for MAXVARS etc.

// memory is allocated for vout in this routine
int getonesetdeps(int ind_vout, char *parm, char *vout[], int maxvout, char *paramfile, char *vararray[], int numvars, char *funcfile, char *reacratesnfile, char *rems[], int numrems)
{
  //
  // This is a recursive routine
  // It takes the string parm,
  // looks for it in the array vout and returns its index and exits if it's already there
  // looks for it in the parameter data file (the gross one with all the parameters)
  // - if it's a set parameter, it adds it to vout and exits
  // - if it's an o2 parameter, gets its dependencies and calls getonesetdeps for each of them. 
  // if not found in the parameter data file, it looks for it in the variable data file
  // - if it has an associated function, it gets its dependencies and calls getonesetdeps for each of them. 
  // It does not look at reaction rates.


  int j;
  int flag=0, suc;
  char block1[2000];
  int numdeps;
  char *vars[40];
  char *vl;
  char *tmp;

  // note: variable initialisation parameters have already been included in paramarray

  if(isinarray(vout, ind_vout, parm) >=0 || strcmp(parm, "__ZZ__")==0)
    return ind_vout;

  // is it a parameter?

  vl = getcvalfromfileq(paramfile, parm);
  if (strcmp(vl, "__ZZ__")==0){ // settable (see routine getvalfromfileq)
    
    ind_vout = addvar1(ind_vout, parm, vout, maxvout);
    free(vl);return ind_vout;
  }
  if(!isonlyspace(vl)){
    flag = 1;
    ind_vout = addvar1(ind_vout, parm, vout, maxvout);

    if(isnumber(vl)){free(vl);return ind_vout;}
    else if(strstr(vl, "_fcn") != NULL){
      suc = findfunc1(funcfile, parm, block1, 2000, 1);
      if(suc == -1){
	fprintf(stderr, "ERROR in getonesetdeps: function for %s couldn't be found in file %s.\n", parm, funcfile);
      }
      else{
	numdeps = getdepsfromfunc(block1, vars, 40, rems, numrems);
	for(j=0;j<numdeps;j++){
	  if(isinarray(vararray, numvars, vars[j]) >=0){
	    fprintf(stderr, "WARNING in getonesetdeps: parameter %s appears to depend on key variable %s. The variable is being replaced with its normal value %sn.\n", parm, vars[j], vars[j]);
	    tmp = strdup(vars[j]);free(vars[j]);
	    vars[j] = (char*) malloc(sizeof(char) * (strlen(tmp)+2));
	    sprintf(vars[j], "%sn", tmp); // safe
	    free(tmp);
	  }
	  ind_vout = getonesetdeps(ind_vout, vars[j], vout, maxvout, paramfile, vararray, numvars, funcfile, reacratesnfile, rems, numrems);
	}
	free_datarray(vars, numdeps);
      }
    }
    else{ // defined directly
      numdeps = getvars(vl, vars, 40, rems, numrems);
      for(j=0;j<numdeps;j++){
	if(isinarray(vararray, numvars, vars[j]) >=0){
	  fprintf(stderr, "WARNING in getonesetdeps: parameter %s appears to depend on key variable %s. The variable is being replaced with its normal value %sn.\n", parm, vars[j], vars[j]);
	  tmp = strdup(vars[j]);free(vars[j]);
	  vars[j] = (char*) malloc(sizeof(char) * (strlen(tmp)+2));
	  sprintf(vars[j], "%sn", tmp); // safe
	  free(tmp);
	}
	ind_vout = getonesetdeps(ind_vout, vars[j], vout, maxvout, paramfile, vararray, numvars, funcfile, reacratesnfile, rems, numrems);
      }
      free_datarray(vars, numdeps);
    }
  }
  free(vl);

  if(flag==0){
    vl = getcvalfromfileq(reacratesnfile, parm);
    if(!isonlyspace(vl)){
      flag = 1;
      numdeps = getvars(vl, vars, 40, rems, numrems);
      for(j=0;j<numdeps;j++){
	if(isinarray(vararray, numvars, vars[j]) >=0){
	  fprintf(stderr, "WARNING in getonesetdeps: parameter %s appears to depend on key variable %s. The variable is being replaced with its normal value %sn.\n", parm, vars[j], vars[j]);
	  tmp = strdup(vars[j]);free(vars[j]);
	  vars[j] = (char*) malloc(sizeof(char) * (strlen(tmp)+2));
	  sprintf(vars[j], "%sn", tmp); // safe
	  free(tmp);
	}
	ind_vout = getonesetdeps(ind_vout, vars[j], vout, maxvout, paramfile, vararray, numvars, funcfile, reacratesnfile, rems, numrems);
      }
      free_datarray(vars, numdeps);
    }
    free(vl);
  }
/*   fprintf(stderr, "got here\n"); */
/*   if(flag==0) // default: assume a number */
/*     ind_vout = addvar1(ind_vout, parm, vout, maxvout); */

  return ind_vout;
}

// memory is allocated for vout in this routine
int getmultideps(char *vin[], int numvin, char *vout[], int maxvout, char *paramfile, char *varfile, char *funcfile, char *reacratesnfile, char *rems[], int numrems)
{
  int i, z;
  int numvars=0;
  char **vararray;
  vararray = genarraydat(varfile, &numvars);
  z=0;
  for(i=0;i<numvin;i++){
    z = getonesetdeps(z, vin[i], vout, maxvout, paramfile, vararray, numvars, funcfile, reacratesnfile, rems, numrems);
  }
  freearraydat(vararray, numvars);
  return z;
}


int writedepsfile(char *o2pfile, char *funcfile, char *reacratefile, char *varfile, char *depsfile, char *rems[], int numrems)
{
  int i, j, k=0, l, suc, linelen, numdeps;
  FILE *fd, *fd2;
  char line[1000];
  char *block;
  char block1[2000];
  char *block2;
  char *oneword;
  char *vars[40];
  int numrates;
  char **ratenamearray;
  char **ratearray;
  int rind[1000];
  char **vararray;
  int numvars=0;

  fd = fopen(o2pfile, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"writedepsfile\" in \"parseparams.c\": File \"%s\" could not be opened for reading.\n", o2pfile);
    return -1;
  }


  fd2 = fopen(depsfile, "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"writedepsfile\" in \"parseparams.c\": File \"%s\" could not be opened for writing.\n", depsfile);
    return -1;
  }

  fprintf(fd2, "// generated by parseparams.c\n\n");

  numrates = genarraysdat(reacratefile, &ratenamearray, &ratearray);
  vararray = genarraydat(varfile, &numvars);

  while((linelen = getline(fd, line, 1000)) > 0){
    if(!iscomsep(line)){
      oneword = getnthwd(line, 1);
      block = getrst(oneword, line);
      if(isnumber(block))
	fprintf(fd2, "%s\n", oneword);
      else if(strstr(block, "_fcn") != NULL){
	suc = findfunc1(funcfile, oneword, block1, 2000, 1);
	if(suc == -1)
	  fprintf(stderr, "ERROR in writedepsfile: function for %s couldn't be found in file %s.\n", oneword, funcfile);
	else{
	  numdeps = getdepsfromfunc(block1, vars, 40, rems, numrems);
	  fprintf(fd2, "%s ", oneword);
	  for(j=0;j<numdeps;j++){
	    if ((l=isinarray(ratenamearray, numrates, vars[j])) >=0){
	      if(!isinlist(l, rind, k))
		rind[k++]=l;
	    }
	    fprintf(fd2, "%s ", vars[j]);
	  }
	  fprintf(fd2, "\n");
	  free_datarray(vars, numdeps);
	}
      }
      else{ // defined directly
	numdeps = getvars(block, vars, 40, rems, numrems);
	fprintf(fd2, "%s ", oneword);
	for(j=0;j<numdeps;j++){
	  if ((l=isinarray(ratenamearray, numrates, vars[j])) >=0){
	    if(!isinlist(l, rind, k))
	      rind[k++]=l;
	  }
	  fprintf(fd2, "%s ", vars[j]);
	}
	fprintf(fd2, "\n");
	free_datarray(vars, numdeps);
      }
      free(block);
      free(oneword);
    }
  }

  // the rate terms

  for(i=0;i<k;i++){
    block2 = transfunc1(ratearray[rind[i]], "n", vararray, numvars);
    numdeps = getvars(block2, vars, 40, rems, numrems);
    fprintf(fd2, "%s ", ratenamearray[rind[i]]);
    for(j=0;j<numdeps;j++)
      fprintf(fd2, "%s ", vars[j]);
    fprintf(fd2, "\n");
    free_datarray(vars, numdeps);
    free(block2);
  }

  freearraydat(ratenamearray, numrates);
  freearraydat(ratearray, numrates);
  freearraydat(vararray, numvars);

  fclose(fd);
  fclose(fd2);
  return 0;
}

int main()
{
  char *pout[MAXPARAMS];
  int i, k, tot, len, found;
  FILE *fd, *fd1, *fd2, *fd3, *fd4;
  char *oneword;
  char *twoword;
  char line[500];
  int len_func = 5000;
  char *block;
  char block1[5000];
  int numrems=0;
  char **rems;
  char **dname;
  char **darray;
  char **tempvs;
  char **exprs;
  int numtempvs;



  char **vararray;
  int numvars=0;
  char **paramarray;
  int numparams=0;
  int nump;
  int suc;  
  int ord[2*MAXVARS+MAXPARAMS];
  char *vl;
  int numold;
  char **chms=NULL;
  int numrightval;


  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  paramarray = genarraydat("sysfiles/paramarray.dat", &numparams);
  rems = genarraydat("userfiles/remsfile.dat", &numrems);

  tot = getmultideps(paramarray, numparams, pout, MAXPARAMS, "sysfiles/allparams.dat", "sysfiles/varbles.dat", "sysfiles/temp/o2funcsr.cc", "sysfiles/reacratesn.dat", rems, numrems);

  //
  // cleaned params.dat file and settable.dat file
  //

  fd1 = fopen("sysfiles/params.dat", "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/params.dat\" could not be opened for writing.\n");
    return -1;
  }
  fd2 = fopen("sysfiles/settable.dat", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/settable.dat\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd1, "// Generated by parseparams.c\n\n");
  fprintf(fd2, "// Generated by parseparams.c\n\n");

  for(i=0;i<tot;i++){
    vl = getcvalfromfileq("sysfiles/allparams.dat", pout[i]);
    if (strcmp(vl, "__ZZ__")==0){ // settable (see routine getvalfromfileq)
      fprintf(fd1, "%s\t0.0\n", pout[i]);
      fprintf(fd2, "%s\n", pout[i]);
    }
    else if(!isonlyspace(vl)){
      fprintf(fd1, "%s\t%s\n", pout[i], vl);
      if(isnumber(vl))
	fprintf(fd2, "%s\n", pout[i]);
    }
    else{
      fprintf(stderr, "ERROR in \"parseparams.c\": A value/rule for parameter %s could not be found. Exiting.\n", pout[i]);
      fclose(fd1); fclose(fd2); free(vl);
      free_datarray(pout, tot);
      return -1;
    }
    free(vl);
  }


  fclose(fd1);
  fclose(fd2);

  free_datarray(pout, tot);

  //
  // cleaned o2funcsr.cc file
  //

  fd = fopen("sysfiles/params.dat", "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/params.dat\" could not be opened for reading.\n");
    return -1;
  }
  fd1 = fopen("sysfiles/temp/o2funcsr.cc", "r");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/temp/o2funcsr.cc\" could not be opened for reading.\n");
    return -1;
  }
  fd2 = fopen("sysfiles/o2funcsr.cc", "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/o2funcsr.cc\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd2, "//Generated by parseparams.c\n\n");



  fd3 = fopen("sysfiles/o2params.dat", "w");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/o2params.dat\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd3, "//Needed o2 parameters. Generated by parseparams.c\n\n");

  fd4 = fopen("sysfiles/commonfuncs.c", "w");
  if(fd4 == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/commonfuncs.c\" could not be opened for writing.\n");
    return -1;
  }
  fprintf(fd4, "//Common parameter functions. Generated by parseparams.c\n\n");

  // preamble
  getblock(fd1, block1, "COMMON", len_func); // don't bother printing
  getblock(fd1, block1, "SPECIFIC", len_func);

  // to simplify things... needs tidying

  fprintf(fd2, "double setkvals1(double K_eq, double halftime);\ndouble setkvals2(double K_eq, double halftime);\ndouble setkvals1acid(double K_eq, double halftime, double normdiss);\ndouble setkvals2acid(double K_eq, double halftime, double normdiss);\n/***************************** SPECIFIC FUNCTIONS *******************/\n");

  fprintf(fd4, "%s", block1);
  fclose(fd1);
  fclose(fd4);
  // main part

  while((len = getline(fd, line, 500)) > 0){
    i=0;
    if(!iscomsep(line)){
      oneword = getnthwd(line, 1);
      block = getrst(oneword, line);
      twoword = getnthtkn(line, 2);
      if (strstr(twoword, "fcn") != NULL){
	fprintf(fd3, "%s", line);
	suc = findfunc1("sysfiles/temp/o2funcsr.cc", oneword, block1, len_func, 2);
	if(suc == -1){
	  fprintf(stderr, "ERROR in parseparams.c: function for %s couldn't be found in file \"sysfiles/temp/o2funcsr.cc\"\n", oneword);
	}
	else
	  fprintf(fd2, "%s", block1);
      }
      else if(!(isnumber(block))){
	fprintf(fd3, "%s", line);
	fprintf(fd2, "double %s_fcn()\n{\n  return %s;\n}\n//////\n", oneword, block);
      }
      free(block);
      free(twoword);
      free(oneword);
    }
  }

  fclose(fd);
  fclose(fd2);
  fclose(fd3);




  freearraydat(paramarray, numparams);

  //
  // parameter dependencies
  //


  writedepsfile("sysfiles/o2params.dat", "sysfiles/o2funcsr.cc", "sysfiles/reacrates.dat", "sysfiles/vararray.dat", "sysfiles/alldeps.dat", rems, numrems);
  numparams = genarraysdat("sysfiles/alldeps.dat", &dname, &darray);

  ordertree(dname, darray, numparams, ord, 2*MAXVARS+MAXPARAMS);

  fd=fopen("sysfiles/ordparams.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/ordparams.dat\" could not be opened for writing.\n");
    return -1;
  }
  for(i=0;i<numparams;i++){ // print out in order
    fprintf(fd, "%s\t%s\n", dname[ord[i]], darray[ord[i]]);
  }
  fclose(fd);

  freearraydat(dname, numparams);
  freearraydat(darray, numparams);

  // 
  // variable dependencies
  //

  writedepsfile("sysfiles/varbles.dat", "sysfiles/initfuncsr.cc", "sysfiles/reacrates.dat", "sysfiles/vararray.dat", "sysfiles/allvardeps.dat", rems, numrems);
  numparams = genarraysdat("sysfiles/allvardeps.dat", &dname, &darray);

  ordertree(dname, darray, numparams, ord, 2*MAXVARS+MAXPARAMS);

  fd=fopen("sysfiles/ordvarbles.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/ordvarbles.dat\" could not be opened for writing.\n");
    return -1;
  }
  for(i=0;i<numparams;i++){ // print out in order
    fprintf(fd, "%s\t%s\n", dname[ord[i]], darray[ord[i]]);
  }
  fclose(fd);


  // alltempstrs2.dat contains only those strings which can definitely be translated
  // do this until there is no change
  numold=0;
  paramarray = genarraydat("sysfiles/params.dat", &nump);
  cpfile("sysfiles/alltempstrs2.dat", "sysfiles/alltempstrs.dat");
  numtempvs=1;

  while(numtempvs!=numold){
    numtempvs = genarraysdat("sysfiles/alltempstrs2.dat", &tempvs, &exprs);
    numold=numtempvs;
    fd2 = fopen("sysfiles/alltempstrs2.dat", "w");
    if(fd2 == NULL){
      fprintf(stderr, "ERROR in parseparams.c: File \"sysfiles/alltempstrs2.dat\" could not be opened...\n");
      return -1;
    }
    for(i=0;i<numtempvs;i++){
      found=1;
      chms = getvs(exprs[i], &numrightval, rems, numrems);
      for(k=0;k<numrightval;k++){ // for each variable in the expression
	if(isinarray(vararray, numvars, chms[k]) >= 0 || cisinarray(paramarray, nump, chms[k]) >= 0 || cisinarray(tempvs, numtempvs, chms[k]) >= 0){}
	else{
	  found=0;
	  break;
	}
      }
      if(found==1){ 
	fprintf(fd2, "%s\t%s\n", tempvs[i], exprs[i]);
      }
      freearraydat(chms, numrightval);
    }

    fclose(fd2);
    freearraydat(tempvs, numtempvs);
    freearraydat(exprs, numtempvs);
  }

  numtempvs = genarraysdat("sysfiles/alltempstrs2.dat", &tempvs, &exprs);
  if(ordertree(tempvs, exprs, numtempvs, ord, 2*MAXVARS+MAXPARAMS)<0){
    fprintf(stderr, "ERROR in \"parseparams.c\": Failure in ordertree - you have defined some temporary variables recursively.\n");
    freearraydat(tempvs, numtempvs);
    freearraydat(exprs, numtempvs);
    return -1;
  }
  fd=fopen("sysfiles/alltempstrs2.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"parseparams.c\": File \"sysfiles/alltempstrs2.dat\" could not be opened for writing.\n");
    freearraydat(tempvs, numtempvs);
    freearraydat(exprs, numtempvs);
    return -1;
  }
  for(i=0;i<numtempvs;i++){ // print out in order
    fprintf(fd, "%s\t%s\n", tempvs[ord[i]], exprs[ord[i]]);
  }
  fclose(fd);
  freearraydat(tempvs, numtempvs);
  freearraydat(exprs, numtempvs);


  freearraydat(vararray, numvars);
  freearraydat(dname, numparams);
  freearraydat(darray, numparams);
  freearraydat(paramarray, numparams);

  freearraydat(rems, numrems);

  return 0;
}
