/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include <unistd.h> // for getcwd

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  char line[500];
  char block[1000];
  int l, len, flag;
  char oneword[50];
  char systemcom[200];


  if(argc < 2){
    fprintf(stderr, "ERROR in restoremodel.c: you must enter the name of the system you wish to restore.\n");
    return -1;
  }

  fprintf(stderr, "Checking if there are any existing references to the model.\n");

  // checking if various model files already exist

  if(modelexists(argv[1])){
    fprintf(stderr, "Some model files appear to exist already. You must delete the model before you can restore it from an archive. Now exiting\n");
    return -1;
  }

  // checking if userfiles/model_defaults already 
  // contains references to the model

  fd1 = fopen("userfiles/model_defaults", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_defaults\".\n");
    return -1;
  }
  flag = 0;
  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);
      if(strcmp(oneword, argv[1]) == 0){
	flag = 1;
	break;
      }
    }
  }
  fclose(fd1);

  if(flag == 1){
    fprintf(stderr, "A reference to the model %s was found in the file \"userfiles/model_defaults\". You must delete the model before you can restore it from an archive. Now exiting\n", argv[1]);
    return -1;
  }

  // checking if model_types_make already 
  // contains references to the model

  fd1 = fopen("userfiles/model_types_make", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types_make\".\n");
    return -1;
  }
  flag = 0;
  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);
      if(strcmp(oneword, argv[1]) == 0){
	flag = 1;
	break;
      }
    }
  }
  fclose(fd1);

  if(flag == 1){
    fprintf(stderr, "A reference to the model %s was found in the file \"userfiles/model_types_make\". You must delete the model before you can restore it from an archive. Now exiting\n", argv[1]);
    return -1;
  }

  // checking if userfiles/model_types.dat already 
  // contains references to the model

  fd1 = fopen("userfiles/model_types.dat", "r");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types.dat\".\n");
    return -1;
  }
  flag = 0;
  while((len = getline(fd1, line, 500))> 0){
    if(!(iscomsep(line))){
      getnthword(line, oneword, 50, 1);
      if(strcmp(oneword, argv[1]) == 0){
	flag = 1;
	break;
      }
    }
  }
  fclose(fd1);

  if(flag == 1){
    fprintf(stderr, "A reference to the model %s was found in the file \"userfiles/model_types.dat\". You must delete the model before you can restore it from an archive. Now exiting\n", argv[1]);
    return -1;
  }

  fprintf(stderr, "\tNo existing references found.\n");

  // copying the archive

  fprintf(stderr, "Copying the archive.\n");

  snprintf(systemcom, 200, "cp models/archives/%s.tar .", argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR with copying archive to current directory. Now exiting.\n");
    return -1;
  }

  // unpacking

  fprintf(stderr, "Unpacking the archive.\n");

  snprintf(systemcom, 200, "tar xvf %s.tar", argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  if((l= system(systemcom)) != 0){
    fprintf(stderr, "ERROR with unpacking the archive. Now exiting.\n");
    snprintf(systemcom, 200, "rm -fr %s.tar", argv[1]);
    system(systemcom);
    return -1;
  }
  snprintf(systemcom, 200, "rm -fr %s.tar", argv[1]);
  system(systemcom);


  // the harder parts
  
  fprintf(stderr, "Getting data from the file %s_bits which stores various bits of information about the model.\n", argv[1]);

  snprintf(systemcom, 200, "tempfiles/%s_bits", argv[1]);
  fd1 = fopen(systemcom, "r");
  if(fd1==NULL){
    // for compatibility with the old method
    snprintf(systemcom, 200, "models/archives/%s_bits", argv[1]);
    fd1 = fopen(systemcom, "r");
    if(fd1==NULL){
      fprintf(stderr, "Couldn't open the file \"%s\".\nThe model is restored but you need to update \"userfiles/model_defaults\", \"userfiles/model_types_make\" and \"userfiles/model_types.dat\" by hand.\n", systemcom);
      return -1;
    }
  }

  while((len = getline(fd1, line, 500))> 0){
    if(iscomsep(line)){}
    else if(strstr(line, "USERFILES/MODEL_DEFAULTS") != NULL){
      fprintf(stderr, "\tUpdating file \"userfiles/model_defaults\"\n");
      getblockmin(fd1, block, "******", "******", 1000);
      fd = fopen("userfiles/model_defaults", "a");
      if(fd==NULL){
	fprintf(stderr, "Couldn't open the file \"userfiles/model_defaults\".\nThe model is restored but you need to update \"userfiles/model_defaults\", \"userfiles/model_types_make\" and \"userfiles/model_types.dat\" by hand.\n");
	fclose(fd1);
	return -1;
      }
      fprintf(fd, "%s", block);
      fclose(fd);
    }
    else if(strstr(line, "MODEL_TYPES_MAKE") != NULL){
      fprintf(stderr, "\tUpdating file \"userfiles/model_types_make\"\n");
      getblockmin(fd1, block, "******", "******", 1000);
      fd = fopen("userfiles/model_types_make", "a");
      if(fd==NULL){
	fprintf(stderr, "Couldn't open the file \"userfiles/model_types_make\".\nThe model is restored but you need to update \"userfiles/model_types_make\" and \"userfiles/model_types.dat\" by hand.\n");
	fclose(fd1);
	return -1;
      }
      fprintf(fd, "%s: src/lib/P_makemodel userfiles/model_defaults\nifneq ($(MODEL), %s)\n\techo %s > sysfiles/sysstate/which_model_file\n\t./src/lib/P_makemodel\nendif\n", argv[1], argv[1], argv[1]);
  //      fprintf(fd, "%s", block);
      fclose(fd);
    }
    else if(strstr(line, "USERFILES/MODEL_TYPES.DAT") != NULL){
      getblockmin(fd1, block, "******", "******", 1000);
      fprintf(stderr, "\tUpdating file \"userfiles/model_types.dat\"\n");
      fd = fopen("userfiles/model_types.dat", "a");
      if(fd==NULL){
	fprintf(stderr, "Couldn't open the file \"userfiles/model_types.dat\".\nThe model is restored but you need to update \"userfiles/model_types.dat\" by hand.\n");
	fclose(fd1);
	return -1;
      }
      fprintf(fd, "%s", block);
      fclose(fd);
    }
  }

  fclose(fd1);

  fprintf(stderr, "Model \"%s\" successfully restored.\n", argv[1]);

  return 0;
}
