/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"
#include <sys/stat.h>

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  char line[500];
  int l, len, flag;
  char oneword[50];
  char rmcall[10] = "rm -fr ";

  char systemcom[200];


  if(argc < 2){
    fprintf(stderr, "ERROR in rmsys.c: you must enter the name of the system you wish to remove.\n");
    return -1;
  }
  else if(strlen(argv[1]) > 30){ // for safety
    fprintf(stderr, "ERROR in rmsys.c: system name too long.\n");
    return -1;
  }

  // the modeldescs directory

  sprintf(systemcom, "%smodels/modeldescs/%s/", rmcall, argv[1]); // safe
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);

  // the module directory
  sprintf(systemcom, "%smodels/modules/%s/", rmcall, argv[1]); // safe
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);

  // the default input file directory
  sprintf(systemcom, "%smodels/inputs/%s/", rmcall, argv[1]); // safe
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);

  // the default parameter file directory

  sprintf(systemcom, "%smodels/pvals/%s/", rmcall, argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);

  // the autoreg input file directory

  sprintf(systemcom, "%smodels/autoreginputs/%s/", rmcall, argv[1]);
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);
  fprintf(stderr, "%s\n", systemcom);
  system(systemcom);

  // back up and update the model_defaults file

  fprintf(stderr, "Updating file userfiles/model_defaults\n");

  l=system("cp userfiles/model_defaults tempfiles/model_defaults");
  if(l!=0){
    fprintf(stderr, "Couldn't copy the file \"userfiles/model_defaults\", hence couldn't update this file. You can do it manually.\n");
    return -1;
  }
  fd = fopen("tempfiles/model_defaults", "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"tempfiles/model_defaults\". Hence couldn't update the file \"userfiles/model_defaults\". You can do it manually.\n");
    return -1;
  }
  fd1 = fopen("userfiles/model_defaults", "w");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_defaults\". Hence couldn't update this file. You can do it manually.\n");
    fclose(fd);
    return -1;
  }

  while((len = getline(fd, line, 500))> 0){
    if(iscomsep(line))
      fprintf(fd1, "%s", line);
    else{
      getnthblock(line, oneword, 50, 1);
      if(strcmp(argv[1], oneword)!=0)
	fprintf(fd1, "%s", line);
    }
  }

  fclose(fd);
  fclose(fd1);



  // back up and update the model_types_make

  fprintf(stderr, "Updating file userfiles/model_types_make\n");

  l=system("cp userfiles/model_types_make tempfiles/model_types_make");
  if(l!=0){
    fprintf(stderr, "Couldn't copy the file \"userfiles/model_types_make\", hence couldn't update this file. You can do it manually.\n");
    return -1;
  }
  fd = fopen("tempfiles/model_types_make", "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"tempfiles/model_types_make\". Hence couldn't update the file \"userfiles/model_types_make\". You can do it manually.\n");
    return -1;
  }
  fd1 = fopen("userfiles/model_types_make", "w");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types_make\". Hence couldn't update this file. You can do it manually.\n");
    fclose(fd);
    return -1;
  }

  flag = 0;
  while((len = getline(fd, line, 500))> 0){
    if(iscomsep(line) && flag == 0)
      fprintf(fd1, "%s", line);
    else{
      getnthtoken(line, oneword, 50, 1);

      if(strcmp(argv[1], oneword)==0)
	flag = 1;

      if(flag == 0)
	fprintf(fd1, "%s", line);

      if(strcmp("endif", oneword)==0)
	flag = 0;

    }
  }

  fclose(fd);
  fclose(fd1);


  // back up and update userfiles/model_types.dat

  fprintf(stderr, "Updating file userfiles/model_types.dat\n");

  l=system("cp userfiles/model_types.dat tempfiles/model_types.dat");
  if(l!=0){
    fprintf(stderr, "Couldn't copy the file \"userfiles/model_types.dat\", hence couldn't update this file. You can do it manually.\n");
    return -1;
  }
  fd = fopen("tempfiles/model_types.dat", "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"tempfiles/model_types.dat\". Hence couldn't update the file \"userfiles/model_types.dat\". You can do it manually.\n");
    return -1;
  }
  fd1 = fopen("userfiles/model_types.dat", "w");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"userfiles/model_types.dat\". Hence couldn't update this file. You can do it manually.\n");
    fclose(fd);
    return -1;
  }

  while((len = getline(fd, line, 500))> 0){
    if(iscomsep(line))
      fprintf(fd1, "%s", line);
    else{
      getnthblock(line, oneword, 50, 1);
      if(strcmp(argv[1], oneword)!=0)
	fprintf(fd1, "%s", line);
    }
  }

  fclose(fd);
  fclose(fd1);

  fprintf(stderr, "Model \"%s\" successfully deleted.\n", argv[1]);

  return 0;
}
