#include "common/stringmanip.h"


int main(int argc, char *argv[]){
  double range = 10.0; // a percentage

  FILE *fd, *fd1;
  int i, j, l;
  char systemcom[200];
  double tempupval;
  double tempdownval;
  char modname[100];
  char inputfile[100]; 
  char paramfile[100];
  char *tempword=NULL;
  double val;
  double downval;
  double upval;

  int numparams=0;
  char **paramarray;

  int numvars=0;
  char **vararray;


  if(getoption("userfiles/sensitivity", "model", modname, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"model\". Exiting.\n");
    return -1;
  }
  if(getoption("userfiles/sensitivity", "input_file", inputfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"input_file\". Exiting.\n");
    return -1;
  }
  if(getoption("userfiles/sensitivity", "param_file", paramfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"param_file\". Exiting.\n");
    return -1;
  }


  snprintf(systemcom, 200, "make %s", modname);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"make %s\"\n", modname);
    return -1;
  }

  fd = fopen("sysfiles/sysstate/which_param_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", paramfile);
  fclose(fd);

  // copy the input file

  snprintf(systemcom, 200, "cat %s > models/inputs/tempinput.dat", inputfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"%s\"\n", systemcom);
    return -1;
  }

  // set the input file

  fd = fopen("sysfiles/sysstate/which_input_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't open the file \"sysfiles/sysstate/which_input_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", "models/inputs/tempinput.dat");
  fclose(fd);

  // make

  l=system("make");
  if(l!=0){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"make\"\n");
    return -1;
  }

  l=system("make sysfiles/allcurrent_pvals.dat");
  if(l!=0){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"make sysfiles/allcurrent_pvals.dat\"\n");
    return -1;
  }

/*   l=system("make sysfiles/settable.dat"); */
/*   if(l!=0){ */
/*     fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"make sysfiles/settable.dat\"\n"); */
/*     return -1; */
/*   } */

  l=system("make sysfiles/output.dat");
  if(l!=0){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"make sysfiles/output.dat\"\n");
    return -1;
  }

  // get the array of all settable parameters and of all variables

  paramarray = genarraydat("sysfiles/settable.dat", &numparams);
  vararray = genarraydat("sysfiles/output.dat", &numvars);

  fd1 = fopen("outdat/sensitivity_matrix.dat", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in sensitivity.c. Couldn't open the file \"outdat/sensitivity_matrix.dat\" for writing.\n");
    return -1;
  }

  fprintf(fd1, "#paramname\tindex");
  for(j=0;j<numvars;j++)
    fprintf(fd1, "\t%s", vararray[j]);
  fprintf(fd1, "\n");

  for(i=0;i<numparams;i++){
    tempword = getcvalfromfile("sysfiles/allcurrent_pvals.dat", paramarray[i]);
    val = atof(tempword);
    free(tempword);
    downval = (1 - range/200.0)*val;
    upval = (1 + range/200.0)*val;

    // run the simulation where the parameter value is lowered

    fd = fopen("models/inputs/tempinput.dat", "w");
    if(fd==NULL){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't open the file \"models/inputs/tempinput.dat\" for writing.\n");
      return -1;
    }
    fprintf(fd, "reset: %s, %.4e, %.4e\n", paramarray[i], val, downval);
    fclose(fd);

    snprintf(systemcom, 200, "cat %s >> models/inputs/tempinput.dat", inputfile);
    l=system(systemcom);
    if(l!=0){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"%s\"\n", systemcom);
      return -1;
    }

    snprintf(systemcom, 200, "src/lib/P_run models/inputs/tempinput.dat %s outdat/tempoutdown.dat > outdat/tempoutdown.dat", paramfile);
    l=system(systemcom);

    if(l!=0){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"%s\"\n", systemcom);
      return -1;
    }

    // run the simulation where the parameter value is raised

    fd = fopen("models/inputs/tempinput.dat", "w");
    if(fd==NULL){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't open the file \"models/inputs/tempinput.dat\" for writing.\n");
      return -1;
    }
    fprintf(fd, "reset: %s, %.4e, %.4e\n", paramarray[i], val, upval);
    fclose(fd);

    snprintf(systemcom, 200, "cat %s >> models/inputs/tempinput.dat", inputfile);
    l=system(systemcom);
    if(l!=0){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"%s\"\n", systemcom);
      return -1;
    }

    snprintf(systemcom, 200, "src/lib/P_run models/inputs/tempinput.dat %s outdat/tempoutdown.dat > outdat/tempoutup.dat", paramfile);
    l=system(systemcom);

    if(l!=0){
      fprintf(stderr, "ERROR in sensitivity.c. Couldn't run the command \"%s\"\n", systemcom);
      return -1;
    }

    fprintf(fd1, "%s\t%d", paramarray[i], i);
    for(j=0;j<numvars;j++){
      tempupval = timeaverage("outdat/tempoutup.dat", 0, 100, vararray[j]);
      tempdownval = timeaverage("outdat/tempoutdown.dat", 0, 100, vararray[j]);

      //      fprintf(stderr, "var = %s\ttempupval = %.4e\ttempdownval = %.4e\n", vararray[j], tempupval, tempdownval);
      fprintf(fd1, "\t%.4e", (tempupval-tempdownval)*200.0/range/(tempupval+tempdownval));
    }
    fprintf(fd1, "\n");

  }

  fclose(fd1);

  freearraydat(vararray, numvars);
  freearraydat(paramarray, numparams);

  return 0;
}
