#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "syssize.h"

// like setindmulti4.c, except that we assume that there is a linear relationship between r_0i and r_mi. r_t1 and r_t2 not used for fitting. 

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


// the number of independent quantities being fit.
#define NQ 4
// the number of parameters whose values will actually be changed
#define NP 6
char *fitparam[6] = {"r_m2", "r_02", "r_m1", "r_01", "CMRglucnw", "Vol_blood_frac"};

double multiTOIdist(char *inputfile, char *paramfile, double vinit[], double vvals[]);

double amotsa(char *inputfile, char *paramfile, double vinit[], double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*funk)(char *inputfile, char *paramfile, double vinit[], double vvals[]), int ihi, double *yhi, double fac);


double ran1(long *idum);

void amebsa(char *inputfile, char *paramfile, double vinit[], double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*funk)(char *inputfile, char *paramfile, double vinit[], double vvals[]), int *iter, double temptr);


#define GET_PSUM \
  for (n=1;n<=ndim;n++) {\
  for (sum=0.0,m=1;m<=mpts;m++) sum += p[m][n];\
  psum[n]=sum;}
long idum;
double tt;


int main(int argc, char *argv[]){

  FILE *fd;
  int i, j, l;
  char systemcom[200];
  char *vname = NULL;
  double vinit[NP];
  char modname[100];
  char inputfile[100]; 
  char paramfile[100];
  int iter;
  double temptr;

  double **pinit;
  double *finit;
  double *tempv;
  double *pb;
  double yb;
  char basdatfile[100];

  if (argc<2)
    strcpy(basdatfile, "ilias_art"); // safe, default
  else{
    if(strlen(argv[1]) > 50){
      fprintf(stderr, "ERROR: filename too long: %s\n", argv[1]);
      return -1;
    }
    strcpy(basdatfile, argv[1]); // safe
  }


  if(getoption(basdatfile, "model", modname, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"model\". Exiting.\n");
    return -1;
  }
  if(getoption(basdatfile, "input_file", inputfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"input_file\". Exiting.\n");
    return -1;
  }
  if(getoption(basdatfile, "param_file", paramfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"param_file\". Exiting.\n");
    return -1;
  }

  snprintf(systemcom, 200, "cat %s > tempfiles/tempinput.dat", inputfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"%s\"\n", systemcom);
    return -1;
  }

  snprintf(systemcom, 200, "make %s", modname);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"make full\"\n");
    return -1;
  }

  fd = fopen("sysfiles/sysstate/which_param_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", paramfile);
  fclose(fd);

  fd = fopen("sysfiles/sysstate/which_input_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't open the file \"sysfiles/sysstate/which_input_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", inputfile);
  fclose(fd);

  l=system("make");
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"make\"\n");
    return -1;
  }

  l=system("make sysfiles/allcurrent_pvals.dat");
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"make sysfiles/allcurrent_pvals.dat\"\n");
    return -1;
  }

  for(i=0;i<NP;i++){
    vname = getcvalfromfile("sysfiles/allcurrent_pvals.dat", fitparam[i]);
    vinit[i] = atof(vname);
    free(vname);
  }

  // specific to this particular fitting

  pinit=dmatrix(1, NQ+1, 1, NQ);
  finit=dvector(1, NQ+1);
  tempv=dvector(1, NQ);
  pb=dvector(1, NQ);




  for(i=1;i<=NQ+1;i++){
    for(j=1;j<=NQ;j++){
      if(i==1)
	pinit[i][j] = 1.0;
      else if(i==j+1) // scale 
	pinit[i][j]=1.05;
      else
	pinit[i][j]=1.0;
    }
  }


  for(i=1;i<=NQ+1;i++){
    for(j=1;j<=NQ;j++)
      tempv[j] = pinit[i][j];
    finit[i] = multiTOIdist(inputfile, paramfile, vinit, tempv);
  }

  iter = 1000;
  //temptr = 0.0;
  temptr = 2.0;
  yb=100; // so that the best value is stored. 
  for(i=0;i<5;i++){
    iter=200; //=200 normally
    //    iter=12-2*i;
    temptr *=0.5;
    amebsa(inputfile, paramfile, vinit, pinit, finit, NQ, pb, &yb, 0.005, multiTOIdist, &iter, temptr);
    fprintf(stderr, "*\n*\n*\niteration %d\tBest so far: ", i+1);


    fprintf(stderr, "r_m2 = %.4e, ", pb[1]*vinit[0]);
    fprintf(stderr, "r_02 = %.4e, ", pb[1]*vinit[1]);
    fprintf(stderr, "r_m1 = %.4e, ", pb[2]*vinit[2]);
    fprintf(stderr, "r_01 = %.4e, ", pb[2]*vinit[3]);
    fprintf(stderr, "CMRglucnw = %.4e, ", pb[3]*vinit[4]);
    fprintf(stderr, "Vol_blood_frac = %.4e\n", pb[4]*vinit[5]);
  }

  free_dmatrix(pinit, 1, NQ+1, 1, NQ);
  free_dvector(finit, 1, NQ+1);
  free_dvector(tempv, 1, NQ);
  free_dvector(pb, 1, NQ);

  return 0;
}


double multiTOIdist(char *inputfile, char *paramfile, double vinit[], double vvals[])
{
  FILE *fd;
  int i, l;
  double diff1;
  char systemcom[200];
  double pen;

  fd = fopen(inputfile, "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't open the file \"%s\" for writing.\n", inputfile);
    return -1;
  }
  fprintf(fd, "reset: ");
  fprintf(fd, "r_m2, %.4e, %.4e, ", vinit[0], vvals[1]*vinit[0]);
  fprintf(fd, "r_02, %.4e, %.4e, ", vinit[1], vvals[1]*vinit[1]);
  fprintf(fd, "r_m1, %.4e, %.4e, ", vinit[2], vvals[2]*vinit[2]);
  fprintf(fd, "r_01, %.4e, %.4e, ", vinit[3], vvals[2]*vinit[3]);
  fprintf(fd, "CMRglucnw, %.4e, %.4e, ", vinit[4], vvals[3]*vinit[4]);
  fprintf(fd, "Vol_blood_frac, %.4e, %.4e\n", vinit[5], vvals[4]*vinit[5]);

  fprintf(stderr, "r_m2 = %.4e, ", vvals[1]*vinit[0]);
  fprintf(stderr, "r_02 = %.4e, ", vvals[1]*vinit[1]);
  fprintf(stderr, "r_m1 = %.4e, ", vvals[2]*vinit[2]);
  fprintf(stderr, "r_01 = %.4e, ", vvals[2]*vinit[3]);
  fprintf(stderr, "CMRglucnw = %.4e, ", vvals[3]*vinit[4]);
  fprintf(stderr, "Vol_blood_frac = %.4e\n", vvals[4]*vinit[5]);

  fclose(fd);
  fprintf(stderr, "\n");

  snprintf(systemcom, 200, "cat tempfiles/tempinput.dat >> %s", inputfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"%s\"\n", systemcom);
    return -1;
  }
  snprintf(systemcom, 200, "src/lib/P_run %s %s outdat/tempout.dat > outdat/tempout.dat", inputfile, paramfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindmulti2.c. Couldn't run the command \"%s\"\n", inputfile);
    return -1;
  }
  //when changing the fitting time remember to change the value of the second time
  //  diff1 = distaverage("outdat/tempout.dat", 0, 2000, "TOI", 0.0, 1.0, "TOI1sup", 0.0, 1.0, 0, 0);
  diff1 = distaverage("outdat/tempout.dat", 0, 2000, "TOI", 0.0, 1.0, "TOIsup", 0.0, 1.0, 0, 0);
  // + distaverage("outdat/tempout.dat", 0, 2000, "V_mca", 0.0, 1.0, "Vmca_sup", 0.0, 1.0, 0, 1);

  // If we want to add a penalty for straying too far from normal values, then we 
  // can add a term which is a function of vvals
  pen=0.0;
  for(i=0;i<4;i++)
    pen+=(vvals[i+1]-1.0)*(vvals[i+1]-1.0);
  pen=10.0*sqrt(pen);

  fprintf(stderr, "diff1 + pen = %.4e + %.4e\n", diff1, pen);

  //  fprintf(stderr, "diff1= %.4e\n", diff1);
  //  return diff1;
  return diff1+pen;

}

// these routines are all adapted from numerical recipes in C

double ran1(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;

  if (*idum <= 0 || !iy) {
    if (-(*idum) < 1) *idum=1;
    else *idum = -(*idum);
    for (j=NTAB+7;j>=0;j--) {
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum < 0) *idum += IM;
      if (j < NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}

// This core routine performs one fitting step at a given temperature


void amebsa(char *inputfile, char *paramfile, double vinit[], double **p, double y[], int ndim, double pb[], double *yb, double ftol, double (*funk)(char *inputfile, char *paramfile, double vinit[], double vvals[]), int *iter, double temptr)
{
  int i,ihi,ilo,j,m,n,mpts=ndim+1;
  double rtol,sum,swap,yhi,ylo,ynhi,ysave,yt,ytry,*psum;

  psum=dvector(1,ndim);
  tt = -temptr;
  GET_PSUM
    for (;;) {
      ilo=1;
      ihi=2;
      ynhi=ylo=y[1]+tt*log(ran1(&idum));
      yhi=y[2]+tt*log(ran1(&idum));
      if (ylo > yhi) {
	ihi=1;
	ilo=2;
	ynhi=yhi;
	yhi=ylo;
	ylo=ynhi;
      }
      for (i=3;i<=mpts;i++) {
	yt=y[i]+tt*log(ran1(&idum));
	if (yt <= ylo) {
	  ilo=i;
	  ylo=yt;
	}
	if (yt > yhi) {
	  ynhi=yhi;
	  ihi=i;
	  yhi=yt;
	} else if (yt > ynhi) {
	  ynhi=yt;
	}
      }
      rtol=2.0*fabs(yhi-ylo)/(fabs(yhi)+fabs(ylo));
      if (rtol < ftol || *iter < 0) {
	swap=y[1];
	y[1]=y[ilo];
	y[ilo]=swap;
	for (n=1;n<=ndim;n++) {
	  swap=p[1][n];
	  p[1][n]=p[ilo][n];
	  p[ilo][n]=swap;
	}
	break;
      }
      *iter -= 2;
      ytry=amotsa(inputfile, paramfile, vinit, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,-1.0);
      if (ytry <= ylo) {
	ytry=amotsa(inputfile, paramfile, vinit, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,2.0);
      } else if (ytry >= ynhi) {
	ysave=yhi;
	ytry=amotsa(inputfile, paramfile, vinit, p,y,psum,ndim,pb,yb,funk,ihi,&yhi,0.5);
	if (ytry >= ysave) {
	  for (i=1;i<=mpts;i++) {
	    if (i != ilo) {
	      for (j=1;j<=ndim;j++) {
		psum[j]=0.5*(p[i][j]+p[ilo][j]);
		p[i][j]=psum[j];
	      }
	      y[i]=(*funk)(inputfile, paramfile, vinit, psum);
	    }
	  }
	  *iter -= ndim;
	  GET_PSUM
	    }
      } else ++(*iter);
    }
  free_dvector(psum,1,ndim);
}


double amotsa(char *inputfile, char *paramfile, double vinit[], double **p, double y[], double psum[], int ndim, double pb[], double *yb, double (*funk)(char *inputfile, char *paramfile, double vinit[], double vvals[]), int ihi, double *yhi, double fac)
{
  int j;
  double fac1,fac2,yflu,ytry,*ptry;

  ptry=dvector(1,ndim);
  fac1=(1.0-fac)/ndim;
  fac2=fac1-fac;
  for (j=1;j<=ndim;j++)
    ptry[j]=psum[j]*fac1-p[ihi][j]*fac2;
  ytry=(*funk)(inputfile, paramfile, vinit, ptry);
  if (ytry <= *yb) {
    for (j=1;j<=ndim;j++) pb[j]=ptry[j];
    *yb=ytry;
  }
  yflu=ytry-tt*log(ran1(&idum));
  if (yflu < *yhi) {
    y[ihi]=ytry;
    *yhi=yflu;
    for (j=1;j<=ndim;j++) {
      psum[j] += ptry[j]-p[ihi][j];
      p[ihi][j]=ptry[j];
    }
  }
  free_dvector(ptry,1,ndim);
  return yflu;
}

