#include "common/stringmanip.h"
#include "syssize.h"

#define GOLD 1.618034
#define GLIMIT 100.0
#define TINY 1.0e-20
#define ITMAX 100
#define ITMAX1 200
#define CGOLD 0.3819660
#define ZEPS 1.0e-10
#define TOL 2.0e-4

static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

#define min(A, B) ((A) < (B) ? (A) : (B))
#define SHFT2(a, b, c) (a)=(b);(b)=(c);
#define SHFT3(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
#define SIGN(a,b) ((b) > 0.0 ? fabs(a) : -fabs(a))
static double maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ? (maxarg1) : (maxarg2))
#define RR 0.61803399
#define CC (1.0-RR)

double *dvector(long nl, long nh);
void free_dvector(double *v, long nl, long nh);

void mnbrak(char *inputfile, char *paramfile, char *fitparam, double vinit, double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval));

double golden(char *inputfile, char *paramfile, char *fitparam, double vinit, double ax, double bx, double cx, double (*f)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval), double tol, double *xmin);

double brent(char *inputfile, char *paramfile, char *fitparam, double vinit, double ax, double bx, double cx, double (*f)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval), double tol, double *xmin);
void powell(char *inputfile, char *paramfile, char *fitparam, double vinit, double p[], double **xi, int n, double ftol, int *iter, double *fret, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vvals[]));

double TOIdist(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval);
void linmin(char *inputfile, char *paramfile, char *fitparam, double vinit, double p[], double xi[], int n, double *fret, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vvals[]));

double f1dim(char *inputfile, char *paramfile, char *fitparam, double vinit, double x);

// some external variables

int ncom;
double *pcom,*xicom;
double (*nrfunc)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vvals[]);

int main(int argc, char *argv[]){
  double inc_size = 2.0; // a percentage

  FILE *fd;
  int l;
  char *fitparam = "r_m2";
  char systemcom[200];
  int sign=1;
  double change;
  char *vname = NULL;
  double vinit;
  double vval;
  char modname[100];
  char inputfile[100]; 
  char paramfile[100];
  double diff;
  double xmin;

  double ax, bx, cx, fa, fb, fc;

  if(getoption("ilias_art", "model", modname, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"model\". Exiting.\n");
    return -1;
  }
  if(getoption("ilias_art", "input_file", inputfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"input_file\". Exiting.\n");
    return -1;
  }
  if(getoption("ilias_art", "param_file", paramfile, 100)!=0){
    fprintf(stderr, "ERROR with getting the option \"param_file\". Exiting.\n");
    return -1;
  }

  snprintf(systemcom, 200, "cat %s > tempfiles/tempinput.dat", inputfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"%s\"\n", systemcom);
    return -1;
  }

  snprintf(systemcom, 200, "make %s", modname);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"make full\"\n");
    return -1;
  }

  fd = fopen("sysfiles/sysstate/which_param_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't open the file \"sysfiles/sysstate/which_param_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", paramfile);
  fclose(fd);

  fd = fopen("sysfiles/sysstate/which_input_file", "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't open the file \"sysfiles/sysstate/which_input_file\" for writing.\n");
    return -1;
  }
  fprintf(fd, "%s", inputfile);
  fclose(fd);

  l=system("make");
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"make\"\n");
    return -1;
  }

  l=system("make sysfiles/allcurrent_pvals.dat");
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"make sysfiles/allcurrent_pvals.dat\"\n");
    return -1;
  }

  vname = getcvalfromfile("sysfiles/allcurrent_pvals.dat", fitparam);
  vinit = atof(vname);
  free(vname);
  vval = vinit;

  ///
  ///
  ///
  ///
  ///
  ///


  change = (double)sign*inc_size/100.0*vinit;

  ax = vinit;
  bx = vinit+change;

  mnbrak(inputfile, paramfile, fitparam, vinit, &ax, &bx, &cx, &fa, &fb, &fc, TOIdist);

  // alternative strategies...

  //  diff = golden(inputfile, paramfile, fitparam, vinit, ax, bx, cx, TOIdist, 0.005, &xmin);

  diff = brent(inputfile, paramfile, fitparam, vinit, ax, bx, cx, TOIdist, 0.005, &xmin);

  fprintf(stderr, "Final values: %s = %.4e: diff = %.4e\n\n", fitparam, xmin, diff);

  return 0;
}

double TOIdist(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval)
{
  FILE *fd;
  int l;
  double diff1;
  char systemcom[200];

  fd = fopen(inputfile, "w");
  if(fd==NULL){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't open the file \"%s\" for writing.\n", inputfile);
    return -1;
  }
  fprintf(fd, "reset: %s, %.4e, %.4e\n", fitparam, vinit, vval);
  fclose(fd);

  fprintf(stderr, "%s = %.4e\n", fitparam, vval);

  snprintf(systemcom, 200, "cat tempfiles/tempinput.dat >> %s", inputfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"%s\"\n", systemcom);
    return -1;
  }
  snprintf(systemcom, 200, "src/lib/P_run %s %s outdat/tempout.dat > outdat/tempout.dat", inputfile, paramfile);
  l=system(systemcom);
  if(l!=0){
    fprintf(stderr, "ERROR in setindparams.c. Couldn't run the command \"%s\"\n", inputfile);
    return -1;
  }

  diff1 = distaverage("outdat/tempout.dat", 0, 60, "TOI", 0.0, 1.0, "TOIsup", 0.0, 1.0, 0, 0);
  fprintf(stderr, "diff1 = %.4e\n", diff1);
  return diff1;

}

void mnbrak(char *inputfile, char *paramfile, char *fitparam, double vinit, double *ax, double *bx, double *cx, double *fa, double *fb, double *fc, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval))
{

  double ulim,u,r,q,fu,dum;

  *fa = (*func)(inputfile, paramfile, fitparam, vinit, *ax);
  *fb = (*func)(inputfile, paramfile, fitparam, vinit, *bx);
  if(*fb > *fa) {
    SHFT3(dum, *ax, *bx, dum)
    SHFT3(dum, *fb, *fa, dum)
  }
  *cx = (*bx)+GOLD*(*bx-*ax);
  *fc=(*func)(inputfile, paramfile, fitparam, vinit, *cx);
  while (*fb > *fc){
    r=(*bx-*ax)*(*fb-*fc);
    q=(*bx-*cx)*(*fb-*fa);
    u=(*bx)-((*bx-*cx)*q-(*bx-*ax)*r)/(2.0*SIGN(FMAX(fabs(q-r),TINY),q-r));
    ulim=(*bx)+GLIMIT*(*cx-*bx);

    if((*bx-u)*(u-*cx) > 0.0){
      fu = (*func)(inputfile, paramfile, fitparam, vinit, u);
      if(fu < *fc){
	*ax=(*bx);
	*bx=u;
	*fa=(*fb);
	*fb=fu;
	return;
      }
      else if(fu > *fb){
	*cx=u;
	*fc=fu;
	return;
      }
      u=(*cx)+GOLD*(*cx-*bx);
      fu=(*func)(inputfile, paramfile, fitparam, vinit, u);
    }
    else if((*cx-u)*(u-ulim) > 0.0){
      fu=(*func)(inputfile, paramfile, fitparam, vinit, u);
      if(fu< *fc){
	SHFT3(*bx, *cx, u, *cx+GOLD*(*cx-*bx))
	SHFT3(*fb,*fc,fu,(*func)(inputfile, paramfile, fitparam, vinit, u))
      }
    }
    else if ((u-ulim)*(ulim-*cx) >= 0.0){
      u=ulim;
      fu = (*func)(inputfile, paramfile, fitparam, vinit, u);
    }
    else{
      u=(*cx)+GOLD*(*cx-*bx);
      fu=(*func)(inputfile, paramfile, fitparam, vinit, u);
    }
    SHFT3(*ax,*bx,*cx,u);
    SHFT3(*fa,*fb,*fc,fu);
  }
}


double golden(char *inputfile, char *paramfile, char *fitparam, double vinit, double ax, double bx, double cx, double (*f)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval), double tol, double *xmin)
{

  double f1,f2,x0,x1,x2,x3;
  x0=ax;
  x3=cx;
  if(fabs(cx-bx) > fabs(bx-ax)){
    x1=bx;
    x2=bx+CC*(cx-bx);
  }
  else{
    x2=bx;
    x1=bx-CC*(bx-ax);
  }
  f1=(*f)(inputfile, paramfile, fitparam, vinit, x1);
  f2=(*f)(inputfile, paramfile, fitparam, vinit, x2);

  while(fabs(x3-x0) > tol*(fabs(x1)+fabs(x2))){
    if(f2<f1){
      SHFT3(x0,x1,x2,RR*x1+CC*x3);
      SHFT2(f1,f2,(*f)(inputfile, paramfile, fitparam, vinit, x2));
    }
    else{
      SHFT3(x3,x2,x1,RR*x2+CC*x0);
      SHFT2(f2,f1,(*f)(inputfile, paramfile, fitparam, vinit, x1));
    }
  }
  if(f1<f2){
    *xmin=x1;
    return f1;
  }
  else{
    *xmin=x2;
    return f2;
  }
}



double brent(char *inputfile, char *paramfile, char *fitparam, double vinit, double ax, double bx, double cx, double (*f)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vval), double tol, double *xmin)
{
  int iter;
  double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
  double e=0.0;

  a=(ax < cx ? ax : cx);
  b=(ax > cx ? ax : cx);
  x=w=v=bx;
  fw=fv=fx=(*f)(inputfile, paramfile, fitparam, vinit, x);
  for (iter=1;iter<=ITMAX;iter++) {
    xm=0.5*(a+b);
    tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
    if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
      *xmin=x;
      return fx;
    }
    if (fabs(e) > tol1) {
      r=(x-w)*(fx-fv);
      q=(x-v)*(fx-fw);
      p=(x-v)*q-(x-w)*r;
      q=2.0*(q-r);
      if (q > 0.0) p = -p;
      q=fabs(q);
      etemp=e;
      e=d;
      if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      else {
	d=p/q;
	u=x+d;
	if (u-a < tol2 || b-u < tol2)
	  d=SIGN(tol1,xm-x);
      }
    } 
    else {
      d=CGOLD*(e=(x >= xm ? a-x : b-x));
    }
    u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
    fu=(*f)(inputfile, paramfile, fitparam, vinit, u);
    if (fu <= fx) {
      if (u >= x) a=x; else b=x;
      SHFT3(v,w,x,u);
      SHFT3(fv,fw,fx,fu);
    } 
    else {
      if (u < x) a=u; else b=u;
      if (fu <= fw || w == x) {
	v=w;
	w=u;
	fv=fw;
	fw=fu;
      } 
      else if (fu <= fv || v == x || v == w) {
	v=u;
	fv=fu;
      }
    }
  }
  fprintf(stderr, "Too many iterations in brent\n");
  *xmin=x;
  return fx;
}


void powell(char *inputfile, char *paramfile, char *fitparam, double vinit, double p[], double **xi, int n, double ftol, int *iter, double *fret, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vvals[]))
{
  int i,ibig,j;
  double del,fp,fptt,t,*pt,*ptt,*xit;

  pt=dvector(1,n);
  ptt=dvector(1,n);
  xit=dvector(1,n);
  *fret=(*func)(inputfile, paramfile, fitparam, vinit, p);
  for (j=1;j<=n;j++) pt[j]=p[j];
  for (*iter=1;;++(*iter)) {
    fp=(*fret);
    ibig=0;
    del=0.0;
    for (i=1;i<=n;i++) {
      for (j=1;j<=n;j++) xit[j]=xi[j][i];
      fptt=(*fret);
      linmin(inputfile, paramfile, fitparam, vinit, p,xit,n,fret,func);
      if (fptt-(*fret) > del) {
	del=fptt-(*fret);
	ibig=i;
      }
    }
    if (2.0*(fp-(*fret)) <= ftol*(fabs(fp)+fabs(*fret))+TINY) {
      free_dvector(xit,1,n);
      free_dvector(ptt,1,n);
      free_dvector(pt,1,n);
      return;
    }
    if (*iter == ITMAX1){
      fprintf(stderr, "powell exceeding maximum iterations.");
      exit(1);
    }
    for (j=1;j<=n;j++) {
      ptt[j]=2.0*p[j]-pt[j];
      xit[j]=p[j]-pt[j];
      pt[j]=p[j];
    }
    fptt=(*func)(inputfile, paramfile, fitparam, vinit, ptt);
    if (fptt < fp) {
      t=2.0*(fp-2.0*(*fret)+fptt)*SQR(fp-(*fret)-del)-del*SQR(fp-fptt);
      if (t < 0.0) {
	linmin(inputfile, paramfile, fitparam, vinit, p,xit,n,fret,func);
	for (j=1;j<=n;j++) {
	  xi[j][ibig]=xi[j][n];
	  xi[j][n]=xit[j];
	}
      }
    }
  }
}


void linmin(char *inputfile, char *paramfile, char *fitparam, double vinit, double p[], double xi[], int n, double *fret, double (*func)(char *inputfile, char *paramfile, char *fitparam, double vinit, double vvals[]))
{
  int j;
  double xx,xmin,fx,fb,fa,bx,ax;

  ncom=n;
  pcom=dvector(1,n);
  xicom=dvector(1,n);
  nrfunc=func;
  for (j=1;j<=n;j++) {
    pcom[j]=p[j];
    xicom[j]=xi[j];
  }
  ax=0.0;
  xx=1.0;
  mnbrak(inputfile, paramfile, fitparam, vinit, &ax,&xx,&bx,&fa,&fx,&fb,f1dim);
  *fret=brent(inputfile, paramfile, fitparam, vinit, ax,xx,bx,f1dim,TOL,&xmin);
  for (j=1;j<=n;j++) {
    xi[j] *= xmin;
    p[j] += xi[j];
  }
  free_dvector(xicom,1,n);
  free_dvector(pcom,1,n);
}

double f1dim(char *inputfile, char *paramfile, char *fitparam, double vinit, double x)
{
  int j;
  double f,*xt;

  xt=dvector(1,ncom);
  for (j=1;j<=ncom;j++) xt[j]=pcom[j]+x*xicom[j];
  f=(*nrfunc)(inputfile, paramfile, fitparam, vinit, xt);
  free_dvector(xt,1,ncom);
  return f;
}
