/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// Splits the description of a model into its component files

#include "common/stringmanip.h"

int main(int argc, char *argv[])
{
  FILE *fd, *fd1, *fd2, *fd3, *fd4, *fd5, *fd6, *fd7, *fd8, *fd9;
  int len;
  int lim_line = 500;
  char line[500];
  char line1[500];
  char oneword[50];
  int lim_block = 10000;
  char block[10000];
  int block_pos;
  int errcode;
  char filename[80];
  char transfile[80];
  char algfile[80];
  char reacfile[80];
  char outputfile[80];
  char DEfile[80];
  char DEtermfile[80];
  char modfile[80];
  char outvarfile[80];
  char autoregoutvarfile[80];
  char *allprocs[2000]; // names of all processes
  int i, k=0;
  int outvflag=0;
  int autoflag=0;
  char *varname;


  if(argc < 2){
    fprintf(stderr, "ERROR in splitdescfiles.c: you need to provide the name of the directory.\n");
    return -1;
  }

  if(strlen(argv[1]) > 60){
    fprintf(stderr, "ERROR in splitdescfiles.c: directory name %s too long.\n", argv[1]);
    return -1;
  }

  fd = fopen("sysfiles/sysstate/which_desc_file", "r");
  if(fd!=NULL){
    getline(fd, line, 500);
    varname = getnthblk(line, 1);
    lrtrim(varname);
    if(strstr(varname, argv[1])) // right directory
      snprintf(filename, 80, "%s", varname); 
    else
      sprintf(filename, "%salldata.dat", argv[1]); //safe
    fclose(fd);
  }
  else{
    sprintf(filename, "%salldata.dat", argv[1]); //safe
  }
  sprintf(transfile, "sysfiles/sysstate/transvars.dat"); //safe
  sprintf(algfile, "sysfiles/sysstate/algrels.dat"); //safe
  sprintf(reacfile, "sysfiles/sysstate/reacs.dat"); //safe
  sprintf(outputfile, "sysfiles/sysstate/output_params.dat"); //safe
  sprintf(DEfile, "sysfiles/sysstate/DEs.dat"); //safe
  sprintf(DEtermfile, "sysfiles/sysstate/DEterms.dat"); //safe
  sprintf(modfile, "sysfiles/sysstate/modfiles.dat"); //safe
  sprintf(outvarfile, "sysfiles/outvars.dat"); //safe
  sprintf(autoregoutvarfile, "sysfiles/autoregoutvars.dat"); //safe

  fd = fopen(filename, "r");
  if(fd == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", filename);
    return -1;
  }

  fd1 = fopen(transfile, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", transfile);
    return -1;
  }
  fd2 = fopen(algfile, "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", algfile);
    return -1;
  }
  fd3 = fopen(reacfile, "w");
  if(fd3 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", reacfile);
    return -1;
  }
  fd4 = fopen(outputfile, "w");
  if(fd4 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", outputfile);
    return -1;
  }
  fd5 = fopen(DEfile, "w");
  if(fd5 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", DEfile);
    return -1;
  }
  fd6 = fopen(DEtermfile, "w");
  if(fd6 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", DEtermfile);
    return -1;
  }
  fd7 = fopen(modfile, "w");
  if(fd7 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", modfile);
    return -1;
  }
  fd8 = fopen(outvarfile, "w");
  if(fd8 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", outvarfile);
    return -1;
  }
  fd9 = fopen(autoregoutvarfile, "w");
  if(fd9 == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"%s\" could not be opened...\n", autoregoutvarfile);
    return -1;
  }

  while((len = getline(fd, line, lim_line)) > 0){
    if(strstr(line, "TRANSVARS") != NULL){
      errcode = getblockmin(fd, block, "endTRANSVARS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", filename);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd1, "%s", block);
    }
    else if(strstr(line, "ALGRELS") != NULL){
      errcode = getblockmin(fd, block, "endALGRELS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", algfile);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd2, "%s", block);

      block_pos=0;
      while(block_pos<errcode){
        getlinefromblock(&block_pos, block, line1, lim_line);
	if(!(iscomsep(line1))){
	  getnthword(line1, oneword, 50, 1);
	  allprocs[k++] = strdup(oneword);
	  if(k>1999){fprintf(stderr, "ERROR in splitdescfiles: too many processes in model.\n");  free_datarray(allprocs, k); return -1;}
	}
      }

    }
    else if(strstr(line, "REACS") != NULL){
      errcode = getblockmin(fd, block, "endREACS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", reacfile);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd3, "%s", block);

      block_pos=0;
      while(block_pos<errcode){
        getlinefromblock(&block_pos, block, line1, lim_line);
	if(!(iscomsep(line1))){
	  getnthword(line1, oneword, 50, 1);
	  allprocs[k++] = strdup(oneword);
	  if(k>1999){fprintf(stderr, "ERROR in splitdescfiles: too many processes in model.\n");  free_datarray(allprocs, k); return -1;}
	}
      }

    }
    else if(strstr(line, "OUTPUT_PARAMS") != NULL){
      errcode = getblockmin(fd, block, "endOUTPUT_PARAMS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", outputfile);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd4, "%s", block);
    }
    else if(strstr(line, "DEQS") != NULL){
      errcode = getblockmin(fd, block, "endDEQS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", DEfile);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd5, "%s", block);

      block_pos=0;
      while(block_pos<errcode){
        getlinefromblock(&block_pos, block, line1, lim_line);
	if(!(iscomsep(line1))){
	  getnthword(line1, oneword, 50, 1);
	  allprocs[k++] = strdup(oneword);
	  if(k>1999){fprintf(stderr, "ERROR in splitdescfiles: too many processes in model.\n");  free_datarray(allprocs, k); return -1;}
	}
      }

    }
    else if(strstr(line, "DETERMS") != NULL){
      errcode = getblockmin(fd, block, "endDETERMS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", DEtermfile);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd6, "%s", block);

      block_pos=0;
      while(block_pos<errcode){
        getlinefromblock(&block_pos, block, line1, lim_line);
	if(!(iscomsep(line1))){
	  getnthword(line1, oneword, 50, 1);
	  allprocs[k++] = strdup(oneword);
	  if(k>1999){fprintf(stderr, "ERROR in splitdescfiles: too many processes in model.\n");  free_datarray(allprocs, k); return -1;}
	}
      }


    }
    else if(strstr(line, "MODFILES") != NULL){
      errcode = getblockmin(fd, block, "endMODFILES", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", filename);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd7, "%s", block);
    }
    else if(strstr(line, "OUTVARS") != NULL){
      outvflag=1;
      errcode = getblockmin(fd, block, "endOUTVARS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", filename);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd8, "%s", block);
    }
    else if(strstr(line, "AUTOREGOUTVS") != NULL){
      autoflag=1;
      errcode = getblockmin(fd, block, "endAUTOREGOUTVS", "endALLDATFILE", lim_block);
      if(errcode == -1){
	fprintf(stderr, "ERROR in splitdescfiles.c: there is an error in \"%s\".\n", filename);
	free_datarray(allprocs, k);
	return -1;
      }
      else
	fprintf(fd9, "%s", block);
    }

  }

  // for compatibility - where this section didn't exist

  if(outvflag==0){
    fprintf(fd8, "// enter variables you want the system to output when you run simulations.\n// Default is all\n\nALL\n");
  }

  if(autoflag==0){
    fprintf(fd9, "// enter variables you want the system to output when you run static simulation.\n// Default is all\n\nALL\n");
  }


  fclose(fd);
  fclose(fd1);
  fclose(fd2);
  fclose(fd3);
  fclose(fd4);
  fclose(fd5);
  fclose(fd6);
  fclose(fd7);
  fclose(fd8);
  fclose(fd9);


  fd = fopen("sysfiles/allprocs.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in splitdescfiles.c: \"sysfiles/allprocs.dat\" could not be opened...\n");
    free_datarray(allprocs, k);
    return -1;
  }

  fprintf(fd, "//Generated by splitdescfiles.c\n\n");
  for(i=0;i<k;i++)
    fprintf(fd, "%s\n", allprocs[i]);

  fclose(fd);

  free_datarray(allprocs, k);

  return 0;
}
