/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

#include "common/stringmanip.h"

int main(int argc, char *argv[])
{
  FILE *fd, *fd1;
  char line[500];
  int i, l, len, flag;
  char *oneword;
  char tempname[100];
  char moddir1[100];
  char moddir2[100];
  char moddir3[100];
  char moddir4[100];
  int issubmodel;
  struct stat fst;
  char *defpfile;

  char systemcom[200];

  if(argc < 2){
    fprintf(stderr, "ERROR in editsys.c: you must enter the name of the submodel you wish to convert to a model.\n");
    return -1;
  }

  fprintf(stderr, "Checking if the model exists.\n");

  snprintf(moddir1, 100, "models/modules/%s", argv[1]);
  snprintf(moddir2, 100, "models/modeldescs/%s", argv[1]);
  snprintf(moddir3, 100, "models/inputs/%s", argv[1]);
  snprintf(moddir4, 100, "models/pvals/%s", argv[1]);

  // checking if various model files already exist

  if((stat(moddir1, &fst) != -1) && (stat(moddir2, &fst) != -1) && (stat(moddir3, &fst) != -1) && (stat(moddir4, &fst) != -1)){} // model files seem to exist
  else{
    fprintf(stderr, "The model does not appear to be a well-formed model suitable for conversion to a submodel. The following are missing:\n");
    if (stat(moddir1, &fst) != -1)
      fprintf(stderr, "\tThe directory models/modules/%s\n", argv[1]);
    if (stat(moddir2, &fst) != -1)
      fprintf(stderr, "\tThe directory models/modeldescs/%s", argv[1]);
    if (stat(moddir3, &fst) != -1)
      fprintf(stderr, "\tThe directory models/inputs/%s", argv[1]);
    if (stat(moddir4, &fst) != -1)
      fprintf(stderr, "\tThe directory models/pvals/%s", argv[1]);
    fprintf(stderr, "The model does not appear to be well formed. Try copying it before converting to a model. Now exiting\n");
    return -1;
  }
  fprintf(stderr, "\t...model exists and is well formed.\n");

  // checking if the model really is a submodel:
  fprintf(stderr, "Checking if the model really is a submodel.\n");

  snprintf(tempname, 100, "models/modeldescs/%s/alldata.dat", argv[1]);
  fd = fopen(tempname, "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"%s\". Exiting without making any changes.\n", tempname);
    return -1;
  }

  issubmodel=0;
  flag = 0;
  while((len = getline(fd, line, 500))> 0){
    if (strstr(line, "endMODFILES") != NULL)
      flag = 0;

    i=0;
    if(flag != 0 && !(iscomsep(line))){
      issubmodel = 1;
      break;
    }

    if(strstr(line, "MODFILES") != NULL && strstr(line, "endMODFILES") == NULL)
      flag = 1;
  }

  fclose(fd);

  if(issubmodel == 0){
    fprintf(stderr, "%s appears to be a model already and not a submodel. Exiting without making any changes.\n", argv[1]);
    return 0;
  }
  fprintf(stderr, "\t...the model is a submodel.\n");

  fprintf(stderr, "Modifying the alldata.dat file and converting symbolic links to files.\n");

  // copying the alldata file:

  snprintf(systemcom, 200, "cp -f models/modeldescs/%s/alldata.dat tempfiles/tempalldata.dat", argv[1]);
  if((l = system(systemcom)) != 0){
    fprintf(stderr, "Problem with copying the datafile. Exiting without making any changes.\n");
    return -1;
  }
  fd = fopen("tempfiles/tempalldata.dat", "r");
  if(fd==NULL){
    fprintf(stderr, "Couldn't open the file \"tempfiles/tempalldata.dat\". Exiting without making any changes.\n");
    return -1;
  }

  snprintf(tempname, 100, "models/modeldescs/%s/alldata.dat", argv[1]);
  fd1 = fopen(tempname, "w");
  if(fd1==NULL){
    fprintf(stderr, "Couldn't open the file \"%s\" for writing. Exiting without making any changes.\n", tempname);
    fclose(fd);
    return -1;
  }

  flag = 0;
  while((len = getline(fd, line, 500))> 0){
    if (strstr(line, "endMODFILES") != NULL)
      flag = 0;

    i=0;
    if(flag == 0)
      fprintf(fd1, "%s", line);
    else{
      if(iscomsep(line))
        fprintf(fd1, "%s", line);
      else{ // lines with a module file name
        oneword = getnthblk(line, 1);
	snprintf(systemcom, 200, "cp --remove-destination models/modules/%s models/modules/%s", oneword, argv[1]);
	if((l = system(systemcom)) != 0)
	  fprintf(stderr, "WARNING: problem with copying the module file %s.\n", oneword);
        fprintf(fd1, "//// %s", line); // comment out line
	free(oneword);
      }
    }

    if(strstr(line, "MODFILES") != NULL && strstr(line, "endMODFILES") == NULL)
      flag = 1;
  }

  fclose(fd);
  fclose(fd1);

  //
  // Default input file doesn't reside in model directory
  //

  defpfile = getmoddef("userfiles/model_defaults", argv[1], "input");
  snprintf(systemcom, 200, "/%s/", argv[1]);
  fprintf(stderr, "%s: %s\n", defpfile, systemcom);
  if(!strstr(defpfile, systemcom)){
    snprintf(systemcom, 200, "cp -f %s models/inputs/%s/_%s", defpfile, argv[1], defpfile);
    if((l = system(systemcom)) != 0){
      fprintf(stderr, "Problem with copying the default input file. Everything else is done and the submodel is now an independent model.\n");
    }
    fprintf(stderr, "Updating file userfiles/model_defaults\n");
    snprintf(systemcom, 200, "models/inputs/%s/_%s", argv[1], defpfile);
    changemoddef("userfiles/model_defaults", argv[1], "input", systemcom);
  }
  free(defpfile);

  //
  // Default parameter file doesn't reside in model directory
  //

  defpfile = getmoddef("userfiles/model_defaults", argv[1], "pvals");
  snprintf(systemcom, 200, "/%s/", argv[1]);
  fprintf(stderr, "%s: %s\n", defpfile, systemcom);
  if(!strstr(defpfile, systemcom)){
    snprintf(systemcom, 200, "cp -f %s models/pvals/%s/_%s_pvals.dat", defpfile, argv[1], argv[1]);
    if((l = system(systemcom)) != 0){
      fprintf(stderr, "Problem with copying the parameter value file. Everything else is done and the submodel is now an independent model.\n");
    }
    fprintf(stderr, "Updating file userfiles/model_defaults\n");
    snprintf(systemcom, 200, "models/pvals/%s/_%s_pvals.dat", argv[1], argv[1]);
    changemoddef("userfiles/model_defaults", argv[1], "pvals", systemcom);
  }
  free(defpfile);


  defpfile = getmoddef("userfiles/model_defaults", argv[1], "input");
  fprintf(stderr, "deffile: %s\n", defpfile);
  free(defpfile);

  fprintf(stderr, "\nSubmodel \"%s\" successfully converted to a model.\n", argv[1]);

  return 0;
}
