/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// Using radau/radau5 with C. This file contains the code which actually 
// integrates the system

#include "syssize.h"
#include "sys.h"
#include "common/stringmanip.h"
#include "common/nrsimp.h"
#include "radau5/radau/radau.h"
#include <unistd.h>
#include <time.h> // for random seeding in case we ever need it 

const double Pi = 3.14159265;

int numvars;
char **vararray;
int numparams;
char **paramarray;
double *rpar;
int nummatvars;
char **matvars;
int *ord; // allocated by solmat0, read by solmat
FILE *brf=NULL;
char **qvars;
int numqvars=0;


// the right-hand side function

void rhs(int *n, double *x, double *v, double *fv, double *rpar, int *ipar);

// the mass matrix
void mas(int *n, double *am, int *lmas, double *rpar, int *ipar);

// the parameter setting function

void pfunc(double *pvals, double *outvals); // in ../sysfiles/paramfunc.c


// Variables and function needed for patient data input

int inputdat(int seg, double ***Pvals, char *infile, int *numchangeparams, char *chosen_param[], int maxpars, int *fixedstep, double *time_step, int *nummorphs, char *morphs[], double morphvals[], int maxmorphs, int *ktot, int *loopnum, int *wantsteady, int *perturb, int *numsegs);

int numchangeparams;
char *chosen_param[10];
int fixedstep; 
double time_step;
int ktot;
double **Pvals;
int nummorphs; 
char *morphs[10]; 
double morphvals[20]; 
int maxmorphs;
int loopnum=1; // how many times to loop
int wantsteady=100000; // run for a while to deal with transients?
int perturb=0;
int numsegs=0;

// to let the output function know that this isn't an autoregulation curve plotter.

int autoreg = 0;
////////////

// This needs to be external because the solmat routine needs to know the iteration number

int kps; // iteration number

////////////

//
// files are declared externally so they 
// can be accessed by the solout routines
//

FILE *fd, *fd1, *fd2;
char *file, *file1, *file2, *file3;



void solmat0();
void solmat(int *nr, double *xold, double *x, double *y,
	    double *cont,int *lrc, int *n, double *rpar,int *ipar,
	    int *irtrn);

/////////////////


void solfinal1(FILE *fd, double *x, double *y, int numvars, char *vararray[], char *paramarray[], int numparams, double *rpar)
{
  int j, l;
  char *temp;
  double tempval;
  for(j=0;j<numvars;j++){
    temp = (char*) malloc(sizeof(char) * (strlen(vararray[j]) + 2));
    strcpy(temp, vararray[j]); // safe
    strcat(temp, "n"); // safe
    if((l=isinarray(paramarray, numparams, temp))>=0)
      tempval = rpar[l];
    else tempval = 1.0;
    fprintf(fd, "(%.4e): %s %.4e\n", *x, vararray[j], y[j]/tempval);
    free(temp);
  }
}



void runvals(FILE *fd, double *x, double *y, int numvars, char *vararray[], double *rpar)
  // my routine - to output the last-run values
{
  int j;
  fprintf(fd, "/* This file is generated by \"sys.cc\"  */\n/* It contains the last-run values of the variables */\n\n");
  fprintf(fd, "#define RUNFILE\n\n");
  fprintf(fd, "int numrunsetparams = %d;\n\n", numvars);
  fprintf(fd, "struct paramstr runparamset[%d] = {\n", numvars);
  for(j=0;j<numvars-1;j++){
    fprintf(fd, "{\"%s_run\", %.4e},\n", vararray[j], y[j]);
  }
  fprintf(fd, "{\"%s_run\", %.4e}};\n\n", vararray[j], y[j]);
}

//                                                                              
// routine to randomly perturb all entries in array s. ind                      
// controls the size of the perturbation                                        
//                                                                              

void randperturb(double *s, int N, float ind)
{
  int i;
  float p;
  p = pow(10,-ind);
  for(i=0;i<N; i++){
    if(fabs(s[i])>1e-10)
      s[i]=s[i]*(1.0 + (p*rand())/(double)RAND_MAX - p/2);
    else{
      s[i]=s[i] + 0.001*((p*rand())/(double)RAND_MAX - p/2);
    }
  }
}


/////////////////////////////////////////
//                                     //
//                                     //
//                                     //
//       NOW ENERING MAIN              //
//                                     //
//                                     //
//                                     //
/////////////////////////////////////////

int main(int argc, char *argv[])
{

  // for the definition of the (static) workspace
  int ND;
  int NS = 20;
  int LWORK;
  int LIWORK;
  double *yvq;
  double *work;
  int *iwork;
  double xtm, xend; // xtm is the time variable
  int i, j, idid, l;
  int input_suc;
  double *fy; // for checking purposes)
  double *fz;
  char *paramfilename; // name of the file holding current values of the parameters	

  char **settable;
  int numsettable = 0;
  double *setpvals;

  // dimension of the system
  int nmv;
  // do not compute the jacobian analytically
  int ijac=0;
  // structure of the jacobian
  int mljac;
  int mujac=0;
  // differential equation is not in explicit form
  int imas=1;
  int mlmas; // these seem to be mainly about storage
  int mumas=0;

  int lmas;

  int ipar=0; // no integer parameters
  int iout=1; // output routine used during integration
	
  // required tolerance
  int itol=1;
  double *rtol;
  double *atol;

  // initial step size
  double h=1.0e-6;
	
  int lwork;
  int liwork;
  int numDEs=0;
  char *nDEs=NULL;

  FILE *ftmp;
  char tempparamfilename[100];
  char tempvfilename[100];

  // random seeding
  int timeint;
  time_t timepoint;
  timeint = time(&timepoint); /* set and convert time to an integer */

  srand(timeint); /*use this integer as a random seed (if needed)*/

  //  FILE *stderr;
  //  stderr = fopen("errorfile", "w");

  /////////////////////////////////
  //                             //
  //      DECLARATIONS OVER      //
  //                             //
  /////////////////////////////////

  if(argc<4){
    fprintf(stderr, "ERROR in src/sys.cc. You need to supply three arguments.\n");
    return -1;
  }

  snprintf(tempparamfilename, 99, "%s.pvals", argv[3]);
  snprintf(tempvfilename, 99, "%s.tvars", argv[3]);

  if(argc>=5){
    qvars = genarraydat(argv[4], &numqvars);
    brf=fopen("outdat/__tmp_contplot.dat", "w");
    if(!brf){fprintf(stderr, "WARNING: brief output file \"__tmp_contplot.dat\" could not be opened.\n");}
  }

  nDEs = getcvalfromfile("sysfiles/othersysparams.dat", "numDEs");
  numDEs = atoi(nDEs);free(nDEs);

  vararray = genarraydat("sysfiles/vararray.dat", &numvars);
  paramarray = genarraydat("sysfiles/params.dat", &numparams);
  // paramarray = genarraydat("sysfiles/paramarray.dat", &numparams);
  settable = genarraydat("sysfiles/settable.dat", &numsettable);
  matvars = genarraydat("sysfiles/output.dat", &nummatvars);
  ord = ivector(0, nummatvars-1);

  // it seems to be safer on some systems to allocate memory in this way rather than directly defining the vectors...
  ND = numvars;
  LWORK = (int) ((NS+1)*ND*ND+(3*NS+3)*ND+20);
  LIWORK = (int) ((2+(NS-1)/2)*ND+20);
  nmv = numvars;
  mljac = numvars;
  mlmas = numvars;
  lmas = numvars;
  lwork = LWORK;
  liwork = LIWORK;
  work = dvector(0, LWORK-1);
  iwork = ivector(0, LIWORK-1);
  yvq = dvector(0, ND);
  rtol = dvector(0, numvars-1);
  atol = dvector(0, numvars-1);
  fz = dvector(0, numvars*numvars-1);
  rpar = dvector(0, numparams-1);
  setpvals = dvector(0, numsettable-1);

  //
  // files
  //



  file = "outdat/output";
  file1 = "outdat/endvals"; // currently unused
  file2 = "sysfiles/runparamvals.c";
  file3 = "sysfiles/tempcurrent_pvals.dat";


  fd = fopen(file, "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"sys.cc\": file %s could not be opened for writing.\n", file);
    return -1;
  }
  fd1 = fopen(file1, "w");
  if(fd1 == NULL){
    fprintf(stderr, "ERROR in \"sys.cc\": file %s could not be opened for writing.\n", file1);
    return -1;
  }
  fd2 = fopen(file2, "w");
  if(fd2 == NULL){
    fprintf(stderr, "ERROR in \"sys.cc\": file %s could not be opened for writing.\n", file2);
    return -1;
  }

  if(numvars == 0){
    fprintf(stderr, "An empty system. Exiting.\n");
    ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
    if(ftmp == NULL){
      fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
    }
    else{
      fprintf(ftmp, "END\n");
      fclose(ftmp);
    }
    return 0;
  }

  // let the system know that the program is running

  ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
  if(ftmp == NULL){
    fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
  }
  else{
    fprintf(ftmp, "%d\n", (int) getpid());
    fclose(ftmp);
  }

  //

  fprintf(fd, "\n***Parameter values***\n\n");

  paramfilename = strdup(argv[2]);

  if(setpvalsfromfile(paramfilename, settable, setpvals, numsettable)!=0){
    free(paramfilename);
    fprintf(stderr, "EXITING because of parameter setting problem.\n");
    // ensuring that the system knows that the simulation has finished
    ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
    if(ftmp == NULL){
      fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
    }
    else{
      fprintf(ftmp, "END PROBLEM\n");
      fclose(ftmp);
    }
    return -1;
  }
  pfunc(setpvals, rpar);

  free(paramfilename);


  // initialise the tolerances

  for (i=0;i<numvars;i++){
    rtol[i] = 2.0e-3;
    atol[i] = 1.0e-2*rtol[i];
  }
  //  fprintf(stderr, "numDEs = %d\n", numDEs);
  for (i=0;i<numDEs;i++){ // the differential variables
    rtol[i] = 1.0e-6;
    atol[i] = 1.0e-4*rtol[i];
  }


  // initialise the parameters (rpar) and print them out
  // problem: morphed parameters not changed yet...


  setval1("", 2.0, paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);

  for (i=0;i<numparams;i++){
    fprintf(fd, "%s =  %.4g\n", paramarray[i], rpar[i]);
  }

  fprintf(fd, "\n\n***initial conditions***\n\n");
	
  // set initial values and print them out
  xtm=0.0;

  setinits1(yvq, rpar);

  for (i=0;i<numvars;i++){
    fprintf(fd, "%s =  %.4g\n", vararray[i], yvq[i]);
  }

  // rhs check

  fprintf(fd, "\n\n**rhs check - if we are near an equilibrium, these should all be small**\n\n");



  fy = dvector(0, numvars);
  rhs(&numvars, &xtm, yvq, fy, rpar, &ipar);
  fprintf(fd, "\n\n");
  for(i=0;i<numvars;i++)
    fprintf(fd, "D%s = %.4e\n", vararray[i], fy[i]);
  fprintf(fd, "\n\n");
	

  // mass check
  fprintf(fd, "**mass check**\n");

  mas(&numvars, fz, &lmas, rpar, &ipar);
  fprintf(fd, "\n\n");
  for(i=0;i<numvars;i++)
    for(j=0;j<numvars;j++)		
      if(fz[numvars*i+j] != 0.0)
	fprintf(fd, "fz[%d][%d] = %.4e\n", i, j, fz[numvars*i+j]);
	

  // endpoint of integration
  //xend=60.0;

  // set default values
  for(i=0; i<20; i++){
    iwork[i]=0;
    work[i] = 0.0;	
  }

  // check parameters being passed

  fprintf(fd, "n = %d\n", nmv);
  fprintf(fd, "rtol = %.4e\n", *rtol);
  fprintf(fd, "atol = %.4e\n", *atol);
  fprintf(fd, "ijac = %d\n", ijac);
  fprintf(fd, "mljac = %d\n", mljac);
  fprintf(fd, "mujac = %d\n", mujac);
  fprintf(fd, "imas = %d\n", imas);
  fprintf(fd, "mlmas = %d\n", mlmas);
  fprintf(fd, "mumas = %d\n", mumas);
  fprintf(fd, "iout = %d\n", iout);
  fprintf(fd, "ipar = %d\n", ipar);


  // initial values - are we using the last run set of values or not?
#ifndef CONTIN // we set this
  setinits1(yvq, rpar);
#elif ! defined RUNFILE // this is defined if runparamvals.c exists
  setinits1(yvq, rpar);
#else
  for(i=0;i<numvars;i++)
    yvq[i] = runparamset[i].pvalue;
#endif
   
  h=1.0e-6;

  solmat0(); // print the heading for the output file

  xtm=0.0; // time

  //////////////////////////////////////
  //                                  //
  //                                  //                
  //                                  // 
  //                                  //   
  //       Entering the main loop     //
  //                                  //
  //                                  //
  //                                  // 
  //                                  //
  //////////////////////////////////////

  for(i=0; i<20; i++){
    iwork[i]=0;
    work[i]=0.0;
  }

  // remember here that work[0] here means work[1] in the fortran literature, etc.
  iwork[1] = 100000;
  iwork[3] = 2;
  iwork[4] = numDEs; // number of index 1 variables
  iwork[5] = numvars-numDEs; // number of index 2 variables
  work[0] = 1.0e-15; // rounding unit
  work[6] = 100.0; // maximum step size

  input_suc = inputdat(0, &Pvals, argv[1], &numchangeparams, chosen_param, 10, &fixedstep, &time_step, &nummorphs, morphs, morphvals, 10, &ktot, &loopnum, &wantsteady, &perturb, &numsegs);

  // fprintf(stderr, "%d, %.4f, %.4f\n", numchangeparams, Pvals[0][0], Pvals[1][1]);

  if(input_suc != 0){
    fprintf(stderr, "ERROR in sys.cc - data input unsuccessful. Exit status = %d. Now exiting.\n", input_suc);
    // ensuring that the system knows that the simulation has finished
    ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
    if(ftmp == NULL){
      fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
    }
    else{
      fprintf(ftmp, "END PROBLEM\n");
      fclose(ftmp);
    }
    //
    return -1;
  }

  fprintf(stderr, "nummorphs = %d\n", nummorphs);



  //
  // parameters to be reset
  //

  for(i=0;i<nummorphs;i++){
    fprintf(stderr, "Morphing %s:  %.4e  --> %.4e\n", morphs[i], morphvals[2*i], morphvals[2*i+1]);
  }
  if(nummorphs>0){
    multimorph1(&nmv,rhs,yvq,&h,
	       rtol,atol,&itol,
 	       jac_dummy,&ijac,&mljac,&mujac,
 	       mas,&imas,&mlmas,&mumas,
 	       soldummy,&iout,
 	       work,&lwork,iwork,&liwork,rpar,&ipar,&idid,
	       numparams, paramarray, morphs, morphvals, nummorphs,
	       10, 100000, "tempfiles/temp_pvals.dat", setpvals, settable, numsettable);
  }

  // so we get the morphed values

  cpfile(tempparamfilename, "tempfiles/temp_pvals.dat");
  cpfile(tempvfilename, "sysfiles/alltempstrs2.dat");
  cpfile("outdat/__tmp_contplot.dat.tvars", "sysfiles/alltempstrs2.dat");
  cpfile("outdat/__tmp_contplot.dat.pvals", "tempfiles/temp_pvals.dat");

  //
  //
  // Do we want initial steadying
  //
  //


  if (wantsteady == -1){ // no steadying at all

    if (fixedstep==1){
      for(i=0;i<numchangeparams;i++){
        setval1(chosen_param[i], Pvals[0][i], paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);
      }
    }
    else{

      for(i=0;i<numchangeparams;i++){
        setval1(chosen_param[i], Pvals[0][i+1], paramarray, numparams, rpar, "tempfiles/temp_pvals1.dat", 0, settable, numsettable, setpvals);
      }
    }

  }
  else if(wantsteady>0){ // stabilise at first parameter values

    for(i=0;i<numchangeparams;i++){

      if (fixedstep==1){
	l=isinarray(paramarray, numparams, chosen_param[i]);
	if(fabs((rpar[l] - Pvals[0][i])/rpar[l]) > 0.01){ // 1 percent difference or more
	  fprintf(stderr, "\t%s  %.4f  %.4f\n", chosen_param[i], rpar[l], Pvals[0][i]);

          morph1(&nmv,rhs,yvq,&h,
		 rtol,atol,&itol,
		 jac_dummy,&ijac,&mljac,&mujac,
		 mas,&imas,&mlmas,&mumas,soldummy,&iout,
		 work,&lwork,iwork,&liwork,rpar,&ipar,&idid,
		 numparams, paramarray, chosen_param[i], rpar[l], 
		 Pvals[0][i], 10, (double)wantsteady, "tempfiles/temp_pvals.dat", 
		 setpvals, settable, numsettable);
	}
      }
      else{
	l=isinarray(paramarray, numparams, chosen_param[i]);
	if(fabs((rpar[l] - Pvals[0][i+1])/rpar[l]) > 0.01){ // 1 percent difference or more
	  fprintf(stderr, "\t%s  %.4f  %.4f\n", chosen_param[i], rpar[l], Pvals[0][i+1]);
          morph1(&nmv,rhs,yvq,&h,
		 rtol,atol,&itol,
		 jac_dummy,&ijac,&mljac,&mujac,
		 mas,&imas,&mlmas,&mumas,
		 soldummy,&iout,
		 work,&lwork,iwork,&liwork,rpar,&ipar,&idid,
		 numparams, paramarray, chosen_param[i], rpar[l], 
		 Pvals[0][i+1], 10, (double)wantsteady, "tempfiles/temp_pvals.dat", 
		 setpvals, settable, numsettable);
	}
      }
    }


    // and now run for a bit

    xtm=0.0;
    xend = (double) wantsteady; 
    RADAU5(&nmv,rhs,&xtm,yvq,&xend,&h,
	   rtol,atol,&itol,
	   jac_dummy,&ijac,&mljac,&mujac,
	   mas,&imas,&mlmas,&mumas,
	   soldummy,&iout,
	   work,&lwork,iwork,&liwork,rpar,&ipar,&idid);

  }
  else if(wantsteady < -1){ // use first parameter values without morphing

    if (fixedstep==1){
      for(i=0;i<numchangeparams;i++){
        setval1(chosen_param[i], Pvals[0][i], paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);
      }
    }
    else{
      for(i=0;i<numchangeparams;i++)
        setval1(chosen_param[i], Pvals[0][i+1], paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);
    }


    //
    // to get rid of transients, we run for a bit..
    //
    
    xtm=0.0;
    xend = (double) (-wantsteady); 
    RADAU5(&nmv,rhs,&xtm,yvq,&xend,&h,
	   rtol,atol,&itol,
	   jac_dummy,&ijac,&mljac,&mujac,
	   mas,&imas,&mlmas,&mumas,
	   soldummy,&iout,
	   work,&lwork,iwork,&liwork,rpar,&ipar,&idid);

  }



  xtm = 0.0;
  xend = 0.0;

  //
  //
  //
  // Entering the main integration where output will be written
  //
  //
  //

  // To loop n times put n*ktot

  for(kps=0;kps<loopnum*ktot;kps++){ // ktot is stored in the input file
    // endpoint of integration

    if (fixedstep==1)
      xend=(kps+1)*time_step; // time_step is stored in the input file
    else
      xend+=Pvals[kps%ktot][0];

    // set default values 
    for(i=0; i<20; i++){
      iwork[i]=0;
      work[i]=0.0;
    }
    //    work[2] = -0.1;
    //work[3] = 0.001;
    work[0] = 1.0e-15; // rounding unit
    work[6] = 100.0; // maximum step size 
    iwork[3] = 2; // if this is equal to zero , then the extrapolated collocation solution is taken as the starting value for newton's method. If it is non-zero then zero starting values are used. Non-zero seems to be safer.
    iwork[4] = numDEs; // number of index 1 variables
    iwork[5] = numvars-numDEs; // number of index 2 variables
    fprintf(fd, "\n**********\n* radau5 *\n**********\n\n");

    if(fixedstep==1){
      for(i=0;i<numchangeparams;i++){
	setval1(chosen_param[i], Pvals[kps%ktot][i], paramarray, numparams, rpar, "tempfiles/temp_pvals.dat", 0, settable, numsettable, setpvals);
      }
    }
    else{
      for(i=0;i<numchangeparams;i++){
        setval1(chosen_param[i], Pvals[kps%ktot][i+1], paramarray, numparams, rpar, "tempfiles/temp_pvals1.dat", 0, settable, numsettable, setpvals);

      }
    }


    if(kps<0){

      RADAU5(&nmv,rhs,&xtm,yvq,&xend,&h,
	     rtol,atol,&itol,
	     jac_dummy,&ijac,&mljac,&mujac,
	     mas,&imas,&mlmas,&mumas,
	     soldummy,&iout,
	     work,&lwork,iwork,&liwork,rpar,&ipar,&idid);

    }
    else{

      if(perturb>0)
	randperturb(yvq, numvars, 2);

      RADAU5(&nmv,rhs,&xtm,yvq,&xend,&h,
	     rtol,atol,&itol,
	     jac_dummy,&ijac,&mljac,&mujac,
	     mas,&imas,&mlmas,&mumas,
	     solmat,&iout,
	     work,&lwork,iwork,&liwork,rpar,&ipar,&idid);
    }

    // print final solution 

    solfinal1(fd, &xtm, yvq, numvars, vararray, paramarray, numparams, rpar);
    
    // print statistics 
    fprintf(fd, "rtol = %1.2e\n", *rtol);
    fprintf(fd, "fcn= %i jac= %i step= %i accpt= %i rejct= %i dec= %i sol= %i\n", iwork[13],iwork[14],iwork[15],iwork[16],iwork[17],iwork[18],iwork[19]);
    fprintf(stderr, "*");
  }
  runvals(fd2, &xtm, yvq, numvars, vararray, rpar);

  fclose(fd);
  fclose(fd1);
  fclose(fd2);
  if(brf){fclose(brf);}
  fprintf(stderr, "\n");

  // Let the system know that the program has finished

  ftmp = fopen("sysfiles/sysstate/temp_pid_file", "w");
  if(ftmp == NULL){
    fprintf(stderr, "WARNING in \"sys.cc\": file \"sysfiles/sysstate/temp_pid_file\" could not be opened for writing.\n");
  }
  else{
    fprintf(ftmp, "END\n");
    fclose(ftmp);
  }

  while (--numchangeparams >= 0)
    free (chosen_param[numchangeparams]); /* free elements of array */

  while (--nummorphs >= 0)
    free (morphs[nummorphs]); /* free elements of array */

  free_dmatrix(Pvals, 0, ktot, 0, numchangeparams);
  free_dvector(work, 0, LWORK-1);
  free_ivector(iwork, 0, LIWORK-1);
  free_dvector(yvq, 0, ND);
  free_dvector(rtol, 0, numvars-1);
  free_dvector(atol, 0, numvars-1);
  free_dvector(fz, 0, numvars*numvars-1);
  free_dvector(rpar, 0, numparams-1);
  free_dvector(setpvals, 0, numsettable-1);
  freearraydat(vararray, numvars);
  freearraydat(paramarray, numparams);
  freearraydat(settable, numsettable);
  freearraydat(matvars, nummatvars);
  free_ivector(ord, 0, nummatvars-1);

  //compdatfiles("outdat/dat", "../sysarchiveold/braincirc2.0/outdat/dat");
  
  return 0;
}
