/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 

 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

 */

// translates temporary variables into the full forms in two files - sysfiles/allDEs.dat and sysfiles/allalgrels.dat. These become sysfiles/allDEs1.dat and sysfiles/allDEs2.dat

#include "common/stringmanip.h"
#include "syssize.h"

int main(){
  FILE *fd, *fd1;
  int len, i, j;
  char line[5000];
  char *varname;
  char *expr1;
  char *expr2;
  char **vararray;
  char **exprarray;
  int numlines=0;
  char filename1[50];
  char filename2[50];
  int ord[MAXVARS + MAXREAC];
  char *deps[MAXVARS];
  int numdeps;
  char *tmp;
  int flag=0;


  // 0 means variable creation, others mean string substitution
  int whichmethod=0;

  // concatenate the files "sysfiles/reacrates.dat" and "sysfiles/tempvars.dat"
  // into "sysfiles/alltempstrs.dat". "sysfiles/tempvars.dat" is the raw file 
  // containing *all* temporary variables.



  cpfile("sysfiles/alltempstrs.dat", "sysfiles/reacrates.dat");
  numlines = genbracarraysfromdat("sysfiles/tempvars.dat", &vararray, &exprarray);
  fd = fopen("sysfiles/alltempstrs.dat", "a");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"transDEalg.c\": File \"sysfiles/alltempstrs.dat\" could not be opened for writing.\n");
    return -1;
  }
  for(i=0;i<numlines;i++){
    if(checkbracs(exprarray[i])!=0){flag=1;}
    tmp=transsumit(exprarray[i]);
    fprintf(fd, "%s\t%s\n", vararray[i], tmp);
    free(tmp);
  }

  fclose(fd);

  if(flag==1){
    fprintf(stderr, "Exiting due to previous problem with brackets.\n");
    return -1;
  }

  //  catfile("sysfiles/alltempstrs.dat", "sysfiles/tempvars.dat");

  numlines = getnoncomlines("sysfiles/alltempstrs.dat");

  if(numlines == 0){ // no temporary strings
    cpfile("sysfiles/allDEs1.dat", "sysfiles/allDEs.dat");
    cpfile("sysfiles/allalgrels1.dat", "sysfiles/allalgrels.dat");
  }

  numlines = genbracarraysfromdat("sysfiles/alltempstrs.dat", &vararray, &exprarray);

  // Now translate/copy the DE file
  if(whichmethod==0)
    cpfile("sysfiles/allDEs1.dat", "sysfiles/allDEs.dat");
  else{
    strcpy(filename1, "sysfiles/allDEs.dat"); // safe
    strcpy(filename2, "sysfiles/allDEs1.dat"); // safe

    fd = fopen(filename1, "r");
    if(fd == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for reading.\n", filename1);
      return -1;
    }

    fd1 = fopen(filename2, "w");
    if(fd1 == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for writing.\n", filename2);
      fclose(fd);
      return -1;
    }

    while((len=getline(fd, line, 5000)) > 0){
      if(iscomsep(line) || strstr(line, "rterm:") == NULL){
	fprintf(fd1, "%s", line);
      } // comment/empty lines or lines which don't contain an rterm
      else{
	expr2 = transfunc2it(line, vararray, exprarray, numlines);
	fprintf(fd1, "%s", expr2);
	free(expr2);
      }
    }

    fclose(fd);
    fclose(fd1);

  }

  // now translate/copy the algrel files

  if(whichmethod==0)
    cpfile("sysfiles/allalgrels1.dat", "sysfiles/allalgrels.dat");
  else{
    strcpy(filename1, "sysfiles/allalgrels.dat"); // safe
    strcpy(filename2, "sysfiles/allalgrels1.dat"); // safe
    fd = fopen(filename1, "r");
    if(fd == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for reading.\n", filename1);
      return -1;
    }

    fd1 = fopen(filename2, "w");
    if(fd1 == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for writing.\n", filename2);
      fclose(fd);
      return -1;
    }

    while((len=getline(fd, line, 5000)) > 0){
      if(iscomsep(line) || strstr(line, "eqn:") == NULL)
	fprintf(fd1, "%s", line);
      else{
	expr2 = transfunc2it(line, vararray, exprarray, numlines);
	fprintf(fd1, "%s", expr2);
	free(expr2);
      }
    }

    fclose(fd);
    fclose(fd1);

  }
  // create tempvars2.dat. This file contains the temporary variables which have been left in the DEs and algrels, in the right order.

  numdeps=0;
  // recursively get all temporary variables needed by the DE and algrel files 
  if(getallrecvars("sysfiles/allDEs1.dat", "sysfiles/allalgrels1.dat", vararray, exprarray, numlines, deps, &numdeps, MAXVARS)<0){
    fprintf(stderr, "ERROR in \"transDEalg.c\": Too many dependencies. Exiting...\n");
    free_datarray(deps, numdeps);
    freearraydat(vararray, numlines);
    freearraydat(exprarray, numlines);
    return -1;
  }



  // order them by dependency put the ordering in array ord
  if(ordertree(vararray, exprarray, numlines, ord, MAXVARS+MAXREAC)<0){
    fprintf(stderr, "ERROR in \"transDEalg.c\": failure in ordertree. \nYou have defined some variables recursively. Exiting...\n");
    free_datarray(deps, numdeps);
    freearraydat(vararray, numlines);
    freearraydat(exprarray, numlines);
    return -1;
  }
  // create sysfiles/tempvars2.dat to store only those 
  // temporary variables needed in the DEs and algrels
  fd=fopen("sysfiles/tempvars2.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"transDEalg.c\": File \"sysfiles/tempvars2.dat\" could not be opened for writing.\n");
    return -1;
  }


  for(j=0;j<numlines;j++){ // print out in order
    if(isinarray(deps, numdeps, vararray[ord[j]])>=0) //is needed
      fprintf(fd, "%s\t%s\n", vararray[ord[j]], exprarray[ord[j]]);
  }
  fclose(fd);

  // create sysfiles/ordalltempvars.dat to store the ordered list of  
  // all temporary variables
  fd=fopen("sysfiles/ordalltempvars.dat", "w");
  if(fd == NULL){
    fprintf(stderr, "ERROR in \"transDEalg.c\": File \"sysfiles/ordalltempvars.dat\" could not be opened for writing.\n");
  }
  for(j=0;j<numlines;j++)
    fprintf(fd, "%s\t%s\n", vararray[ord[j]], exprarray[ord[j]]);
  fclose(fd);


  // free memory allocated in getallrecvars

  free_datarray(deps, numdeps);

  if(whichmethod==1){
    // create tempvars1.dat: this contains the genuine temporary variables translated recursively

    strcpy(filename1, "sysfiles/tempvars2.dat"); // safe
    strcpy(filename2, "sysfiles/tempvars1.dat"); // safe


    fd = fopen(filename1, "r");
    if(fd == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for reading.\n", filename1);
      freearraydat(vararray, numlines);
      freearraydat(exprarray, numlines);
      return -1;
    }

    fd1 = fopen(filename2, "w");
    if(fd1 == NULL){
      fprintf(stderr, "ERROR in \"transDEalg.c\": File \"%s\" could not be opened for writing.\n", filename2);
      fclose(fd);
      freearraydat(vararray, numlines);
      freearraydat(exprarray, numlines);
      return -1;
    }

    while((len=getline(fd, line, 5000)) > 0){
      if(iscomsep(line))
        fprintf(fd1, "%s", line);
      else{
        varname = getnthblk(line, 1);
        fprintf(fd1, "%s\t", varname);
        expr1 = getrst(varname, line);
        expr2 = transfunc2it(expr1, vararray, exprarray, numlines);
        fprintf(fd1, "%s\n", expr2);
        free(expr1); free(expr2); free(varname);
      }
    }

    fclose(fd);
    fclose(fd1);
  }
  // free allocated memory

  freearraydat(vararray, numlines);
  freearraydat(exprarray, numlines);

  return 0;

}
