/* Copyright (C) 2004-2007, Murad Banaji
 *
 * This file is part of BRAINCIRC
 *
 * BRAINCIRC is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2, 
 * or (at your option) any later version.
 *
 * BRAINCIRC is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with BRAINCIRC: see the file COPYING.  If not, write to 
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA. 
 * For comments, queries, or bug-reports about BRAINCIRC, please 
 * email braincirc@medphys.ucl.ac.uk

*/

#include "stringmanip.h"
#include "nrsimp.h"

#define min(A, B) ((A) < (B) ? (A) : (B))
#define max(A, B) ((A) > (B) ? (A) : (B))

//
// Get rid of trailing space
// adapted from kernighan and ritchie
// 


int trim(char s[])
{
  int n;

  for(n=strlen(s)-1;n>=0;n--)
    if(!(isspace(s[n])))
      break;
  s[n+1] = '\0';
  return n;
}

void ent(char entered[], int max){
  int i=0;
  fflush(stdin);
  while(i<max-1 && (entered[i++] = getchar()) != '\n'){}
  i--;entered[i] = 0; // get rid of the terminal newline character
}

char *timestr()
{
  char tmp[20];
  char *out;
  time_t timepoint;
  struct tm *ltime;
  time(&timepoint); /* get time */
  ltime = localtime(&timepoint);
  snprintf(tmp, 50, "%d_%d_%d_%d_%d", 1900+ltime->tm_year, ltime->tm_mon + 1, ltime->tm_mday, ltime->tm_hour, ltime->tm_min);
  out=strdup(tmp);
  return out;
}

//
//
//

void wrap(char *s, int max){
  int i=0, n=0;
  char *tmp;

  tmp = (char*) malloc(sizeof(char) * (strlen(s) + 1));

  while(s[i]){
    if(n>max && isspace(s[i])){tmp[i]='\n';n=0;}
    else{tmp[i]=s[i];}
    if (tmp[i]=='\n'){n=0;}
    i++;n++;
  }
  tmp[i]=0;

  strcpy(s, tmp); // safe
  free(tmp);
  return;

}

//
// replace newlines with spaces
//

void unwrap(char *s){
  int i=0;

  while(s[i]){
    if(s[i]=='\n')
      s[i] = ' ';
    i++;
  }
}

//
// replace single newlines with spaces
//

void unwrap1(char *s){
  int i=0;

  while(s[i]){
    if(s[i]=='\n'){
      if(s[i+1]!='\n'){
	s[i] = ' ';
	i++;
      }
      else
	while(s[i]=='\n'){i++;}

    }
    else
      i++;
  }
}


//
// left and right trim and return in a new string
//

char *lrtrim(char s[]){
  char *p;
  while((*s) && isspace(*s)){s++;}// left trim
  trim(s);
  p=strdup(s);
  return p;
}

//
// Get the prefix where a string ends in a number
//

char *getprefix(char s[]){
  char *p;
  int i=0;
  while(s[i] && !isdigit(s[i])){i++;}
  p = strchop2(s, 0, i);
  return p;
}

//
// clever version of isinarray
//

int cisinarray(char *v[], int numv, char *s){
  // is the string s a member of the array v? If so return its index, if not return -1
  int i;
  char *gn;
  for(i=0;i<numv;i++)
    if(strcmp(s, v[i]) == 0)
      return i;
  gn = getgenname(s);
  if(isonlyspace(gn)){
    free(gn);
    return -1;
  }
  else{
    for(i=0;i<numv;i++){
      if(strcmp(gn, v[i]) == 0){
        free(gn);
        return i;
      }
    }
  }
  return -1;
}


//
// get the numeric part
//

int getvindex(char s[]){
  int i=0, j;
  char *tmp;
  while(s[i] && !isdigit(s[i])){i++;}
  j=0;
  while(s[i] && isdigit(s[i])){i++;j++;}
  tmp=strchop2(s, i-j, j);
  return atoi(tmp);
}

//
// get what's after the numeric part
//

char *getpostindex(char s[]){
  int i=0;
  char *tmp;
  while(s[i] && !isdigit(s[i])){i++;}
  while(s[i] && isdigit(s[i])){i++;}
  tmp=strchop2(s, i, strlen(s)-i);
  return tmp;
}

//
// construct the generic name
//

char *getgenname(char s[]){
  char *p, *p1;
  char *out;
  p = getprefix(s);
  p1 = getpostindex(s);
  if(strcmp(s, p)==0){free(p);free(p1);out=strdup("");return out;}
  else{
    out = (char*) malloc(sizeof(char) * (strlen(p)+strlen(p1) + 2));
    sprintf(out, "%si%s", p, p1); // safe
    free(p);free(p1);
    return out;
  }
}


//
// get the number of non-comment lines and 
// maximum linelength in a file
//

int numflines(char *fname, int *maxline){
  FILE *fd;
  int i=0, c=0;
  int numl=0, len=0;
  char *line;
  (*maxline)=0;

  // first parse - find maximum line length

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "WARNING in numflines: \"%s\" could not be opened for reading.\n", fname);
    return -1;
  }

  while((c=getc(fd)) != EOF){
    i++;
    if (c=='\n'){
      numl++;
      (*maxline)=max(i, (*maxline));
      i=0;
    }
  }
  if(i!=0){ // final line does not end in newline
    numl++;
    (*maxline)=max(i, (*maxline));
  }

  (*maxline)+=2; // to hold the terminal character plus possible EOF

  // second parse - get number of non-comment lines

  rewind(fd);
  line = (char*) malloc(sizeof(char) * (*maxline));
  numl=0;
  while((len=getline(fd, line, *maxline))>0){if(!iscomsep(line)){numl++;}}
  free(line);
  fclose(fd);
  return numl;
}

//
// get the number of non-comment lines and 
// maximum linelength in a file
//

int numflinesdat(char *fname, int *maxline){
  FILE *fd;
  int i=0, c=0;
  int numl=0, len=0;
  char *line;
  (*maxline)=0;

  // first parse - find maximum line length

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "WARNING in numflinesdat: \"%s\" could not be opened for reading.\n", fname);
    return -1;
  }

  while((c=getc(fd)) != EOF){
    i++;
    if (c=='\n'){
      numl++;
      (*maxline)=max(i, (*maxline));
      i=0;
    }
  }
  if(i!=0){ // final line does not end in newline
    numl++;
    (*maxline)=max(i, (*maxline));
  }

  (*maxline)+=2; // to hold the terminal character

  // second parse - get number of non-comment lines

  rewind(fd);
  line = (char*) malloc(sizeof(char) * (*maxline));
  numl=0;
  while((len=getline(fd, line, *maxline))>0){if(!iscomsepdat(line)){numl++;}}
  free(line);
  fclose(fd);
  return numl;
}

char *delunder(char *in){ // deletes all underscores from a string
  int i=0, j=0;
  char *out;
  out = (char*) malloc(sizeof(char) * (strlen(in) + 1));
  while (in[i] != '\0'){
    if (in[i] == '_')
      i++;
    else
      out[j++] = in[i++];
  }
  out[j] = '\0';
  return out;
}

char *delspc(char *in){ // deletes all space from a string (allocates memory)
  int i=0, j=0;
  char *out;
  out = (char*) malloc(sizeof(char) * (strlen(in) + 1));
  while (in[i] != '\0'){
    if (isspace(in[i]))
      i++;
    else
      out[j++] = in[i++];
  }
  out[j] = '\0';
  return out;
}

char *wordtolower(char *in){ // converts a string to lower case (allocates memory)
  int i=0, j=0;
  char *out;
  out = (char*) malloc(sizeof(char) * (strlen(in) + 1));
  while (in[i] != '\0'){
    out[j++] = tolower(in[i++]);
  }
  out[j] = '\0';
  return out;
}

//
// translate pow(x, y) to ((x)**(y))
//

char *transpow(char *s){
  int i, j=0, k;
  char *p;
  int numpows=0;
  char *outstr;
  int bcount=0;
  int i1, i2, i3, i4;
  char *tmp0, *tmp1, *tmp2, *tmp3, *tmp4;

  while((p = strstr(s+j, "pow"))){
    j=strlen(s)-strlen(p)+3;
    numpows++;
  }
  if(numpows>0){
    outstr = (char*) malloc(sizeof(char) * (strlen(s) + numpows*2 + 1));
    outstr[0]=0;
    j=0; // counter in s
    while((p = strstr(s+j, "pow"))){
      i=0; // counter in p
      k=strlen(s+j)-strlen(p);
      strncat(outstr, s+j, k);
      j+=k; // current position in s is start of p
      j+=3; i+=3;// go to the end of pow
      while(p[i] && isspace(p[i])){i++;j++;} // skip space
      if(p[i]!='('){
	strncat(outstr, p, i);
	continue;
      }
      bcount=1;
      i+=1;j+=1; // skip '('
      i1=i;
      while(p[i] && (p[i]!=',' || bcount > 1)){
	if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
	i++;j++;
      }
      i2=i;
      if(bcount !=1 || p[i]!=','){
	strncat(outstr, p, i);
	continue;
      }
      i+=1;j+=1; // skip ','
      i3=i;
      while(p[i] && (p[i]!=')' || bcount > 1)){
	if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
	i++;j++;
      }
      i4=i;
      if(p[i]!=')'){
	strncat(outstr, p, i);
	continue;
      }

      i+=1;j+=1; // skip ')'
      tmp0=strdup12(p, i1, i2);
      tmp2=lrtrim(tmp0);
      tmp1=strdup12(p, i3, i4);
      tmp3=lrtrim(tmp1);
      tmp4 = (char*) malloc(sizeof(char) * (strlen(tmp2) + strlen(tmp3) + 9));
      if(getnumallvars(tmp2)>1 && getnumallvars(tmp3)>1)
        sprintf(tmp4, "((%s)**(%s))", tmp2, tmp3); // safe
      else if(getnumallvars(tmp2)>1)
        sprintf(tmp4, "((%s)**%s)", tmp2, tmp3); // safe
      else if(getnumallvars(tmp3)>1)
        sprintf(tmp4, "(%s**(%s))", tmp2, tmp3); // safe
      else
        sprintf(tmp4, "(%s**%s)", tmp2, tmp3); // safe

      strcat(outstr, tmp4); // safe
      free(tmp0); free(tmp1); free(tmp2); free(tmp3); free(tmp4);
    }
    strcat(outstr, s+j);
    return outstr;
  }
  else{
    outstr = strdup(s);
    return outstr;
  }

}

//
// iterative version of transpow
// also translates fabs to abs

char *transpowit(char s[])
{
  int i;
  char *temp, *temp1;
  char *arr1[] = {"fabs"};
  char *arr2[] = {"abs"};
  temp = transfunc2(s, arr1, arr2, 1);
  //  temp = strdup(s);

  for(i=0;i<1000;i++){
    temp1 = transpow(temp);
    if(strcmp(temp1, temp)==0){
      free(temp1);
      return temp;
    }
    free(temp); temp = strdup(temp1); free(temp1);
  }

  fprintf(stderr, "ERROR in transpowit - still translating after 1000 iterations. Exiting.\n");
  return temp;
}

//
// translate sum_i(xi, a, b) to (xa+...+xb)
//

char *transsum(char *s){
  int i, j=0, k, diff, lenint=0, imin, imax;
  char *p;
  int numpows=0;
  char *outstr=NULL;
  int bcount=0;
  int i1, i2, i3, i31, i32, i4, m;
  char *tmp0, *tmp1, *tmp2, *tmp3, *tmp31, *tmp32, *tmp4, *tmp5;


  while((p = strstr(s+j, "sum_i")) && (strcmp(p, s)==0 || isgood1(p-1, 0))){
    j=strlen(s)-strlen(p)+5;
    numpows++;
  }

  if(numpows==0){outstr=strdup(s);return outstr;}

  j=0; // counter in s
  while((p = strstr(s+j, "sum_i")) && (strcmp(p, s)==0 || isgood1(p-1, 0)) && j==0){ // only do first one
    i=0; // counter in p
    k=strlen(s+j)-strlen(p);
    j+=k; // current position in s is start of p
    j+=5; i+=5;// go to the end of sum_i
    while(p[i] && isspace(p[i])){i++;j++;} // skip space
    if(p[i]!='('){
      fprintf(stderr, "ERROR in transsum: malformed statement: %s. Expecting '('\n", s);
      outstr=strdup(s);return outstr;
    }
    bcount=1;
    i+=1;j+=1; // skip '('
    i1=i;
    while(p[i] && (p[i]!=',' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i2=i;
    if(bcount !=1 || p[i]!=','){
      fprintf(stderr, "ERROR in transsum: malformed statement: %s. Expecting ','\n", s);
      outstr=strdup(s);return outstr;
    }
    i+=1;j+=1; // skip ','
    i3=i;
    while(p[i] && (p[i]!=',' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i31=i;
    if(bcount !=1 || p[i]!=','){
      fprintf(stderr, "ERROR in transsum: malformed statement: %s. Expecting ','\n", s);
      outstr=strdup(s);return outstr;
    }
    i+=1;j+=1; // skip ','
    i32=i;
    while(p[i] && (p[i]!=')' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i4=i;
    if(p[i]!=')'){
      fprintf(stderr, "ERROR in transsum: malformed statement: %s. Expecting ')'\n", s);
      outstr=strdup(s);return outstr;
    }

    i+=1;j+=1; // skip ')'
    tmp0=strdup12(p, i1, i2);
    tmp2=lrtrim(tmp0); // base
    free(tmp0);
    if(strlen(tmp2) == 0){fprintf(stderr, "ERROR in transsum: malformed statement: %s.\n", s); free(tmp2);outstr=strdup(s);return outstr;}
    if(tmp2[strlen(tmp2)-1]!='i'){fprintf(stderr, "ERROR in transsum: malformed statement: %s. Variable name %s must end with an i.\n", s, tmp2); free(tmp2);outstr=strdup(s);return outstr;}
    tmp2[strlen(tmp2)-1]=0; // remove terminal i, safe
    tmp1=strdup12(p, i3, i31);
    tmp3=lrtrim(tmp1); // first index
    free(tmp1);
    if(!ispureint(tmp3)){fprintf(stderr, "ERROR in transsum: malformed statement: %s. %s is not an integer\n", s, tmp3); free(tmp2);free(tmp3);outstr=strdup(s);return outstr;}
    tmp31=strdup12(p, i32, i4);
    tmp32=lrtrim(tmp31); // second index
    free(tmp31);
    if(!ispureint(tmp32)){fprintf(stderr, "ERROR in transsum: malformed statement: %s. %s is not an integer\n", s, tmp32); free(tmp2);free(tmp3);free(tmp32);outstr=strdup(s);return outstr;}
    diff=atoi(tmp32)-atoi(tmp3);
    diff=abs(diff)+1;
    if(diff>0){imax=atoi(tmp32);lenint=strlen(tmp32);imin=atoi(tmp3);}else if(diff<0){imax=atoi(tmp3);lenint=strlen(tmp3);imin=atoi(tmp32);}else{imax=atoi(tmp32);lenint=strlen(tmp3);imin=atoi(tmp3);}
    outstr = (char*) malloc(sizeof(char) * (strlen(s) + diff*(strlen(tmp2) + lenint + 1) + 5));
    free(tmp2); free(tmp3); free(tmp32);
  }


  //
  // second parse
  //

  outstr[0]=0;
  j=0; // counter in s
  while((p = strstr(s+j, "sum_i")) && (strcmp(p, s)==0 || isgood1(p-1,0)) && j==0){
    i=0; // counter in p
    k=strlen(s+j)-strlen(p);
    strncat(outstr, s+j, k);
    j+=k; // current position in s is start of p
    j+=5; i+=5;// go to the end of sum_i
    while(p[i] && isspace(p[i])){i++;j++;} // skip space
    if(p[i]!='('){
      strncat(outstr, p, i);
      continue;
    }
    bcount=1;
    i+=1;j+=1; // skip '('
    i1=i;
    while(p[i] && (p[i]!=',' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i2=i;
    if(bcount !=1 || p[i]!=','){
      strncat(outstr, p, i);
      continue;
    }
    i+=1;j+=1; // skip ','
    i3=i;
    while(p[i] && (p[i]!=',' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i31=i;
    if(bcount !=1 || p[i]!=','){
      strncat(outstr, p, i);
      continue;
    }
    i+=1;j+=1; // skip ','
    i32=i;
    while(p[i] && (p[i]!=')' || bcount > 1)){
      if(p[i]=='('){bcount++;}else if (p[i]==')'){bcount--;}
      i++;j++;
    }
    i4=i;
    if(p[i]!=')'){
      strncat(outstr, p, i);
      continue;
    }

    i+=1;j+=1; // skip ')'
    tmp0=strdup12(p, i1, i2);
    tmp2=lrtrim(tmp0); // base
    free(tmp0);
    tmp2[strlen(tmp2)-1]=0; // remove terminal i, safe (would have failed earlier)
    tmp1=strdup12(p, i3, i31);
    tmp3=lrtrim(tmp1); // first index
    free(tmp1);
    tmp31=strdup12(p, i32, i4);
    tmp32=lrtrim(tmp31); // second index
    free(tmp31);
    diff=atoi(tmp32)-atoi(tmp3);
    if(diff>0){imax=atoi(tmp32);lenint=strlen(tmp32);imin=atoi(tmp3);}else if(diff<0){imax=atoi(tmp3);lenint=strlen(tmp3);imin=atoi(tmp32);}else{imax=atoi(tmp32);lenint=strlen(tmp3);imin=atoi(tmp3);}
    diff=abs(diff)+1;
    tmp4 = (char*) malloc(sizeof(char) * (diff*(strlen(tmp2) + lenint + 1) + 5));
    tmp5 = (char*) malloc(sizeof(char) * (strlen(tmp2) + lenint+2));
    strcpy(tmp4, "(");
    for(m=imin;m<imax;m++){
      sprintf(tmp5, "%s%d+",tmp2,m);
      strcat(tmp4,tmp5);
    }
    sprintf(tmp5, "%s%d)",tmp2,m);
    strcat(tmp4,tmp5);
    free(tmp5);
    strcat(outstr, tmp4); // safe
    free(tmp2); free(tmp3); free(tmp32); free(tmp4);
  }
  strcat(outstr, s+j);

  return outstr;

}

//
// iterative version of transsum
//

char *transsumit(char *s)
{
  int i;
  char *temp, *temp1;
  temp = strdup(s);

  for(i=0;i<1000;i++){
    temp1 = transsum(temp);
    if(strcmp(temp1, temp)==0){
      free(temp1);
      return temp;
    }
    free(temp); temp = strdup(temp1); free(temp1);
  }

  fprintf(stderr, "ERROR in transsumit - still translating after 1000 iterations. Exiting.\n");
  return temp;
}


//
// Like strdup but duplicates from (and including) position i1 
// upto (not including) position i2
//

char *strdup12(char *str, int i1, int i2) {
  char *ret;
  int c;
  if((i2-i1)>0){
    ret = (char*) malloc(sizeof(char) * (i2 - i1 + 1));
    for(c=i1; c<i2;c++)
      ret[c-i1] = str[c];
    ret[c-i1] = '\0';
    return ret;
  }
  else
    return NULL;
}

int iscom(char s[]){ // a comment line or an empty line
  int i=0;
  while((isspace((int) s[i]))){i++;}
  if ((s[i] == '/') || (s[i] == '\0')){
    return 1;
  }
  return 0;
}

// character terminates line or string

int isend(char c){
  if((c=='\n') || (c==13) || (c=='\0'))
    return 1;
  return 0;
}

int iscomblock(char s[]){ // a comment block
  char oneline[1000];
  int len;
  int pos=0;

  while((len=getlinefromblock(&pos, s, oneline, 1000))>0){
    if(!iscomsep(oneline))
      return 0;
  }

  return 1;
}

int iscomsep(char s[]){ // a comment line or separator line or empty
  int i=0;
  while((isspace((int) s[i]))){i++;}
  if (!s[i] || ((s[i] == '/') && (s[i+1] == '/')) || ((s[i] == '/') && (s[i+1] == '*')) || (s[i] == '*')){
    return 1;
  }
  return 0;
}

int iscomsepdat(char s[]){ // a comment line or separator line or data line in a datafile
  int i=0;
  while((isspace((int) s[i]))){i++;}
  if (!s[i] || ((s[i] == '/') && (s[i+1] == '/')) || ((s[i] == '/') && (s[i+1] == '*')) ||(s[i] == '*') || (s[i] == '#')){
    return 1;
  }
  return 0;
}

int isonlyspace(char *s){
  int c, k=0;
  while((c=s[k++]) != '\0'){
    if(!(isspace(c)))
      return 0;
  }
  return 1;
}

int fileexists(char *fname){
  struct stat fst;
  if (stat(fname, &fst)!=0)
    return 0;
  return 1;
}

// get the directory name. With the terminal character removed unless it is the only character. 

char *getdname(char *fname){
  int len;
  char *dname=strdup(fname);
  len=strlen(dname);
  while(len>=0 && (dname[len] != '/'))
    len--;
  dname[len+1] = 0;
  if(!isonlyspace(dname) && dname[strlen(dname)-1]=='/' && strcmp(dname, "/") !=0) // safe
    dname[strlen(dname)-1]=0; // remove terminal '/' if not only character
  return dname;
}

// get a dirmask

char *getdmask(char *fname, char *rstr){
  int len;
  char *dname=strdup(fname);
  char *dmask;
  len=strlen(dname);
  while(len>=0 && (dname[len] != '/'))
    len--;
  dname[len+1] = '\0';
  dmask = (char*) malloc(sizeof(char) * (strlen(dname) + strlen(rstr)+2));
  sprintf(dmask, "%s*%s", dname, rstr);
  free(dname);
  return dmask;
}

// like getdname, but never shaves off terminal '/'

char *getdnamef(char *fname){
  int len;
  char *dname=strdup(fname);
  len=strlen(dname);
  while(len>=0 && (dname[len] != '/'))
    len--;
  dname[len+1] = '\0';
  return dname;
}

// get the filename minus any directory names

char *getfname(char *fname){
  int i, len;
  char *fnametrue;
  len=strlen(fname);
  i=len;
  while(i>=0 && (fname[i] != '/'))
    i--;
  i++;
  fnametrue = strchop2(fname, i, len-i);
  return fnametrue;
}

int modelexists(char *modname){
  char *dirname;
  dirname = (char*) malloc(sizeof(char) * (strlen(modname) + 30));
  sprintf(dirname, "models/modeldescs/%s", modname); // safe
  if (fileexists(dirname)){
    free(dirname);
    return 1;
  }
  sprintf(dirname, "models/modules/%s", modname); // safe
  if (fileexists(dirname)){
    free(dirname);
    return 1;
  }
  sprintf(dirname, "models/inputs/%s", modname); // safe
  if (fileexists(dirname)){
    free(dirname);
    return 1;
  }
  sprintf(dirname, "models/pvals/%s", modname); // safe
  if (fileexists(dirname)){
    free(dirname);
    return 1;
  }
  free(dirname);
  return 0;
}

int archiveexists(char *modname){
  char *archname;
  archname = (char*) malloc(sizeof(char) * (strlen(modname) + 30));
  sprintf(archname, "models/archives/%s.tar", modname);
  if (fileexists(archname)){
    free(archname);
    return 1;
  }
  sprintf(archname, "models/archives/%s.tar.gz", modname);
  if (fileexists(archname)){
    free(archname);
    return 1;
  }
  sprintf(archname, "models/archives/%s.tgz", modname);
  if (fileexists(archname)){
    free(archname);
    return 1;
  }
  free(archname);
  return 0;
}


void strchop(char *s, char *t, int n, int n1)
     /* cuts off the first n characters and last n1 characters in string t and returns the result in string s. Poor error checking */
{
  if((int)(strlen(t)) < n+n1){
    fprintf(stderr, "ERROR in strchop in file stringmanip.c: string not long enough for operation\n");
    *s = '\0';
    return;
  }
  while ((*s = *(t+n)) != '\0'){
    s++;
    t++;
  }
  s = s-n1;
  *s = '\0';

}

void strchop1(char *s, int lim, char *t, int n, int n1)
     /* cuts off the first n characters and takes the next n1 characters in string t and returns the new string s. */
{
  int i=0;
  while (((*s = *(t+n)) != '\0') && (i < lim-1) && (i< n1)){
    s++;t++;i++;
  }
  *s = '\0';
  if ((i != n1) && (i==lim-1))
    fprintf(stderr, "ERROR in routine strchop1: the string was not long enough.\n");
  return;
}

char *strchop2(char *t, int n, int n1){
  // memory allocating version of strchop1
  char *s;
  int i=0;
  if((n<= (int)strlen(t)) && n1>0){
    s = (char*) malloc(sizeof(char) * (n1+1));
    while (t[n+i] && (i< n1)){s[i] = t[n+i];i++;}
    s[i] = '\0';
  }
  else
    s = strdup("");
  return s;
}

char *strchp(char *t, char *lstr, char *rstr){
  // removes initial string *lstr and final string rstr from t
  // otherwise just trims on both sides
  char *s, *tmp;
  int ll, rl;
  ll = strlen(lstr);rl=strlen(rstr);
  tmp = lrtrim(t);
  if((strncmp(tmp, lstr, ll)==0) && (strcmp(tmp+strlen(tmp)-rl, rstr)==0))
    s = strchop2(tmp, ll, strlen(tmp)-ll-rl);
  else if(strncmp(tmp, lstr, ll)==0)
    s = strchop2(tmp, ll, strlen(tmp)-ll);
  else if(strcmp(tmp+strlen(tmp)-rl, rstr)==0)
    s = strchop2(tmp, 0, strlen(tmp)-rl);
  else
    s = strdup(tmp);
  free(tmp);
  return s;
}

// does s start with lstr?

int startswith(char *s, char *lstr){
  char *p;
  char *tmp;
  tmp = lrtrim(s);
  p=strstr(tmp, lstr);
  if(!p || strcmp(p, tmp)!=0){
    free(tmp);
    return 0;
  }
  free(tmp);
  return 1;
}

// does s end with rstr?

int endswith(char *s, char *rstr){
  char *p;
  char *tmp;
  tmp = lrtrim(s);
  p=strstr(tmp, rstr);
  if(!p || strcmp(p, rstr)!=0){
    free(tmp);
    return 0;
  }
  free(tmp);
  return 1;
}

// trim and add initial and final strings if necessary

char *addifnecc(char *s, char *lstr, char *rstr){
  char *p, *p1;
  p1=lrtrim(s);
  p=strchp(s, lstr, rstr); 
  if(strcmp(p, p1)==0){ // neither is there
    free(p);
    p=(char*) malloc(sizeof(char) * (strlen(p1)+strlen(lstr) +strlen(rstr)+ 1));
    sprintf(p, "%s%s%s", lstr,p1,rstr); // safe
    free(p1);
    return p;
  }
  free(p);
  p=strchp(s, lstr, "");
  if(strcmp(p, p1)==0){ // left string not there
    free(p);
    p=(char*) malloc(sizeof(char) * (strlen(p1)+strlen(lstr) + 1));
    sprintf(p, "%s%s", lstr,p1); // safe
    free(p1);
    return p;
  }
  free(p);
  p=strchp(s, "", rstr);
  if(strcmp(p, p1)==0){ // right string not there
    free(p);
    p=(char*) malloc(sizeof(char) * (strlen(p1)+strlen(rstr) + 1));
    sprintf(p, "%s%s", p1,rstr); // safe
    free(p1);
    return p;
  }
  // both strings there
  free(p);
  return p1;

}

//
// must set all the parameters
//

int setpvalsfromfile(char *readfile, char *paramarray[], double *pvals, int numparams){
  int i;
  char *varval;

  for(i=0;i<numparams;i++){ 
    varval = getcvalfromfile(readfile, paramarray[i]); 
    if(isonlyspace(varval)){ 
      fprintf(stderr, "ERROR in setpvalsfromfile: A value for parameter %s could not be found in file %s. Exiting...\n", paramarray[i], readfile); 
      free(varval);
      return -1; 
    } 
    pvals[i]=atof(varval);
    free(varval);
  } 
  return 0;
}

int filehasstr(char *fname, char *str){
  FILE *fd;
  int len;
  char line[5000];
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in filehasstr - File: %s could not be opened for reading...\n", fname);
    return 0;
  }
  while((len=getline(fd, line, 5000)) > 0){
    if(strstr(line, str)){
      fclose(fd);
      return 1;
    }
  }
  fclose(fd);
  return 0;
}

int catarrays(char *out[], int max, char *in1[], int num1, char *in2[], int num2){
  // concatenate two character arrays. Includes allocating memory for each entry
  int i;
  if(num1 + num2 > max){
    fprintf(stderr, "ERROR in catarrays: not enough space in the output array.\n");
    return -1;
  }
  i=0;
  while(i<num1){
    out[i] = strdup(in1[i]);
    i++;
  }
  while(i < num1 + num2){
    out[i] = strdup(in2[i-num1]);
    i++;
  }
  return i;
}

int augmentarray(int curnum, char *out[], int max1, char *in1[], int num1){
  // augment entries in out with those in array in1
  int i, j;
  if(num1 + curnum > max1){
    fprintf(stderr, "ERROR in augmentarray: not enough space in the output array.\n");
    return -1;
  }
  i=curnum;
  j=0;
  while(j<num1){
    if(isinarray(out, i, in1[j])<0){
      out[i] = strdup(in1[j]);
      i++;
    }
    j++;
  }
  return i;
}


int combarrays(char ***out, char *in1[], int num1, char *in2[], int num2){
  // combine two arrays, checking that there isn't duplication. 
  // Allocates memory using charar and strdup

  int i, j=0;
  int d[num1+num2];
  for(i=0;i<num1+num2;i++){d[i]=0;}

  for(i=0;i<num1;i++)
    if(isinarray(in1, i, in1[i])<0){j++;d[i]=1;}
  for(i=0;i<num2;i++)
    if(isinarray(in1, num1, in2[i])<0 && isinarray(in2, i, in2[i])<0){j++;d[i+num1]=1;}

  (*out) = charar(0, j-1);
  j=0;
  for(i=0;i<num1;i++)
    if(d[i]==1){(*out)[j] = strdup(in1[i]);j++;}
  for(i=0;i<num2;i++)
    if(d[i+num1]==1){(*out)[j] = strdup(in2[i]);j++;}

  return j;
}

// is character array in1 a subarray of character array in2?

int issubarray(char *in1[], int num1, char *in2[], int num2){
  int i;
  for(i=0;i<num1;i++){
    if(isinarray(in2, num2, in1[i])<0)
      return 0;
  }
  return 1;
}

// given character array in1 which is a subarray of in2
// and given a set of values corresponding to the strings in in2
// find the corresponding values for in1

// in1 should be vararray, in2 should be got from a datafile

int gentrans(char *in1[], int num1, double d1[], char *in2[], int num2, double d2[]){
  int i;
  if(!issubarray(in1, num1, in2, num2))
    return -1;
  for(i=0;i<num1;i++)
    d1[i] = d2[isinarray(in2, num2, in1[i])];
  return 0;
}

int intersectarrays(char ***out, char *in1[], int num1, char *in2[], int num2){
  // intersect two arrays
  // Allocates memory using charar and strdup

  int i, j=0;
  int d[num1];
  for(i=0;i<num1;i++){d[i]=0;}

  for(i=0;i<num1;i++)
    if(isinarray(in1, i, in1[i])<0 && isinarray(in2, num2, in1[i])>=0){j++;d[i]=1;}

  (*out) = charar(0, j-1);
  j=0;
  for(i=0;i<num1;i++)
    if(d[i]==1){(*out)[j] = strdup(in1[i]);j++;}

  return j;
}

int samestr(char *s, char *t){
  // are two strings the same, up to white space?
  int i=0, j=0;

  while(isspace((int) s[i])) // skip space
    i++;
  while(isspace((int) t[j]))
    j++;
 
  while((s[i] != '\0') && (t[j] != '\0')){
    if(s[i] != t[j])
      return 0;
    i++;j++;
    while(isspace((int) s[i])) // skip space
      i++;
    while(isspace((int) t[j]))
      j++;
 
  }

  if((s[i] == '\0') && (t[j] == '\0'))
    return 1;
  else
    return 0;

}

//
// Take a set of variables vars[] and order them by
// dependency. Output the ordering in ord[]
// 


int ordertree(char *vars[], char *exprs[], int tot, int ord[], int max){

  int i, j, k, jlast, nd;
  int templist[100];
  j=0;

  for(;;){
    jlast = j;
    for(i=0;i<tot;i++){
      if(!(isinlist(i, ord, j))){ // i not in list
        nd = parse(exprs[i], vars, tot, templist, 100); // get the dependencies
        if(sublist(templist, nd, ord, j) && (j < max)){ // dependencies already listed
	  ord[j++] = i; // add i to list
        }
	if(j==max){
	  fprintf(stderr, "ERROR in ordertree: maximum number reached.\n");
	  return -1;
	}
      }
    }
    if(j==tot) // normal return
      return 0;
    if(j==jlast){
      fprintf(stderr, "ERROR in ordertree: there is recursion.\n");
      for(k=0;k<tot;k++)
	if(!isinlist(k, ord, j))
	  fprintf(stderr, "  %s:  %s\n", vars[k], exprs[k]);
      return -1;
    }
    if(j>tot){
      fprintf(stderr, "ERROR in ordertree\n");
      return -1;
    }
  }
}

//
// is the integer "i" in the list lst?
//

int isinlist(int i, int lst[], int tot){
  int j;
  for(j=0;j<tot;j++){
    if(i==lst[j])
      return 1;
  }
  return 0;
}

//
// are all members of list slst[] in list lst[]?
//

int sublist(int slst[], int totslst, int lst[], int tot){
  int j;
  for(j=0;j<totslst;j++){
    if(!(isinlist(slst[j], lst, tot)))
      return 0;
  }
  return 1;
}

//
// take a string expr. Get all tokens in the string.
// If they are variables, put them in order
// The output array "indcs" contains the indices of the variables.
//

int parse(char *expr, char *vars[], int totv, int indcs[], int max){
  int tempnum;
  int i, j=0;
  int k;
  char *chems[100];
  tempnum = getallvars(expr, chems, 100);
  for(i=0;i<tempnum;i++){
    if((k=isinarray(vars, totv, chems[i]))>=0 && (j < max))
      indcs[j++] = k;
    if(j==max){
      fprintf(stderr, "ERROR in parse in stringmanip.c. Maximum number of indices reached. Exiting...\n");
      free_datarray(chems, tempnum);
      return j;
    }
  }
  // free memory allocated using getallvars
  free_datarray(chems, tempnum);

  return j;
}

//
// allocates memory for entries in deps
//


int getallrecvars(char *DEfile, char *algfile, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps){
  FILE *fd;
  int len;
  char line[5000];
  // for each DE
  fd = fopen(DEfile, "r");
  while((len=getline(fd, line, 5000)) > 0){
    if(!(iscom(line)) && strstr(line, "rterm:")){
      getrecvars(line, vars, exprs, totv, deps, numdeps, maxdeps);
      if((*numdeps) == maxdeps){
        fprintf(stderr, "ERROR in getallrecvars: too many dependencies.\n");
	fclose(fd);
	return -1;
      }
    }
  }
  fclose(fd);

  // for each algrel
  fd = fopen(algfile, "r");
  while((len=getline(fd, line, 5000)) > 0){
    if(!(iscom(line)) && strstr(line, "eqn:"))
      getrecvars(line, vars, exprs, totv, deps, numdeps, maxdeps);
      if((*numdeps) == maxdeps){
        fprintf(stderr, "ERROR in getallrecvars: too many dependencies.\n");
	fclose(fd);
	return -1;
      }
  }
  fclose(fd);
  return 0;
}

//
// Recursive algorithm. Allocates memory for entries in deps
// Takes a string str, gets the variables on which it depends
// For each variable, if it's not in the array vars[], then do nothing
// Otherwise:
//     If it's in the array deps[] do nothing
//     Else add to the array vars[] and run getrecvars on it
//

int getrecvars(char *str, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps){
  int tempnum;
  int i;
  int k;
  char *chems[100];

  if((*numdeps) == maxdeps){
    fprintf(stderr, "ERROR in getrecvars: too many dependencies.\n");
    return (*numdeps);
  }
  // find tempvars in str
  tempnum = getallvars(str, chems, 100);
  for(i=0;i<tempnum;i++){
    if((k=isinarray(vars, totv, chems[i]))>=0){ // a tempvar
      if(isinarray(deps, *numdeps, chems[i])<0){ // not already there
        deps[(*numdeps)++] = strdup(chems[i]);
	getrecvars(exprs[k], vars, exprs, totv, deps, numdeps, maxdeps);
      }
    }
  }
  // free memory allocated using getallvars
  free_datarray(chems, tempnum);

  return *numdeps;
}

//
// recursive routine to get all the parameters which depend on a particular parameter.
//

int getdepson(char *pname, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps){
  int i;
  char *chems[100];
  int tempnum;
  for(i=0;i<totv;i++){
    tempnum=getallvars(exprs[i], chems, 100);
    if(isinarray(chems, tempnum, pname) >=0){
      if((*numdeps) == maxdeps-1){
	fprintf(stderr, "ERROR in getdepson: too many dependencies.\n");
	free_datarray(chems, tempnum);
	return (*numdeps);
      }
      if(isinarray(deps, *numdeps, vars[i]) < 0)
	deps[(*numdeps)++] = strdup(vars[i]);
      getdepson(vars[i], vars, exprs, totv, deps, numdeps, maxdeps);
    }
    free_datarray(chems, tempnum);
  }
  return (*numdeps);
}


//
// routine to get all the parameters which depend on pname
// uses the recursive routine getdepson
//

int getalldepson(char *pname, char *paramarray[], int numparams, char *depsfile, char *depsarray[], int max){
  int nump;
  char **parray = NULL;
  char **deps = NULL;
  int numdeps=0;

  numdeps = addvar1(numdeps, pname, depsarray, max);
  nump = genarraysdat(depsfile, &parray, &deps);
  getdepson(pname, parray, deps, nump, depsarray, &numdeps, max);

  freearraydat(parray, nump);
  freearraydat(deps, nump);

  return numdeps;

}


int getrecvars1(char *str, char *vars[], char *exprs[], int totv, char *deps[], int *numdeps, int maxdeps, char *paramarray[], int numparams){
  int tempnum;
  int i;
  int k;
  char *chems[100];
  tempnum = getallvars(str, chems, 100);
  for(i=0;i<tempnum;i++){
    if((k=isinarray(vars, totv, chems[i]))>=0){ // an o2 parameter
      if(isinarray(deps, *numdeps, chems[i])<0){ // not already there
	if((*numdeps) == maxdeps-1){
	  fprintf(stderr, "ERROR in getrecvars1: too many dependencies.\n");
	  free_datarray(chems, tempnum);
	  return (*numdeps);
	}
        deps[(*numdeps)++] = strdup(chems[i]);
	getrecvars1(exprs[k], vars, exprs, totv, deps, numdeps, maxdeps, paramarray, numparams);
      }
    }
    else if((k=isinarray(paramarray, numparams, chems[i])) >= 0){ // a settable parameter
      if(isinarray(deps, *numdeps, chems[i])<0){ // not already there
	if((*numdeps) == maxdeps-1){
	  fprintf(stderr, "ERROR in getrecvars1: too many dependencies.\n");
	  free_datarray(chems, tempnum);
	  return (*numdeps);
	}
        deps[(*numdeps)++] = strdup(chems[i]);
      }
    }

  }
  // free memory allocated using getallvars
  free_datarray(chems, tempnum);

  return *numdeps;
}


//
// routine to get all the dependencies of a parameter
// uses the recursive routine getrecvars1
// 

int getalldeps(char *pname, char *paramarray[], int numparams, char *depsfile, char *depsarray[], int max){
  int l, nump=0;
  char **parray;
  char **deps;
  int numdeps=0;

  numdeps = addvar1(numdeps, pname, depsarray, max);
  nump = genarraysdat(depsfile, &parray, &deps);
  if((l = isinarray(parray, nump, pname)) >= 0)
    getrecvars1(deps[l], parray, deps, nump, depsarray, &numdeps, max, paramarray, numparams);

  freearraydat(parray, nump);
  freearraydat(deps, nump);
  return numdeps;

}


int findfunc1(char *readfile, char *s, char *t, int max, int clean)
{
  // readfile is the file containing the functions
  // s is the name of a variable whose function is to be found
  // t is the string in which the function will be returned
  // max is the size of t (should be at least 1000)
  // the last character is a switch to decide whether to make the output more readable or in the original form. If a '1' is used at the end that means that it should be made readable. We would want to use this option primarily for the user interface...

  int linelen, found=0;
  char temp[100]; // to hold function name
  char line[1000];
  char block[2000];
  FILE *fd;
  char *tmp;
  if(max<10){
    fprintf(stderr, "ERROR in routine findfunc1 in stringmanip.c: you need a longer string.\n");
    return -1;
  }
  t[0] = '\0'; // initialise t
  fd = fopen(readfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine findfunc1 in stringmanip.c: file: %s could not be opened...\n", readfile);
    return -1;
  }
  // construct the function name

  snprintf(temp, 100, "double %s_fcn()", s);
  while((found == 0) && ((linelen = getline(fd, line, 1000)) > 0)){
    if((strstr(line, temp)) && !(iscom(line))){
      found = 1;
      if(clean == 1){
	getcleanblock(fd, block, "/////", 2000);
	if((int)(strlen(block)) + 5 >= max){
	  fprintf(stderr, "ERROR in findfunc1 in stringmanip.c: your block cannot hold the function %s.\n", block);
	  fclose(fd);
	  return -1;
	}
	else{
	  if(strstr(line, "{"))
	    strcpy(t, "{\n"); // safe
	  strcat(t, block); // safe
	  fclose(fd);
	  return 0;
	}
      }
      else if(clean == 0){
	getblock(fd, block, "/////", 2000);
	if((int)(strlen(block)) + 5 >= max){
	  fprintf(stderr, "ERROR in findfunc1 in stringmanip.c: your block cannot hold the function %s.\n", block);
	  fclose(fd);
	  return -1;
	}
	else{
	  if(strstr(line, "{"))
	    strcpy(t, "{\n"); // safe
	  strcat(t, block); // safe
	  fclose(fd);
	  return 0;
	}
      }
      else if(clean == 2){ // include the function line
	getblock(fd, block, "/////", 2000);
	if((int)(strlen(line)) + (int)(strlen(block)) +1 >= max){
	  fprintf(stderr, "ERROR in findfunc1 in stringmanip.c: your block cannot hold the function %s%s.\n", line, block);
	  fclose(fd);
	  return -1;
	}
	else{
	  strcpy(t, line); // safe
	  strcat(t, block); // safe
	  fclose(fd);
	  return 0;
	}
      }
      else if(clean == 3){
	getcleanblock(fd, block, "/////", 2000);
	if((int)(strlen(block)) + 5 >= max){
	  fprintf(stderr, "ERROR in findfunc1 in stringmanip.c: your block cannot hold the function %s.\n", block);
	  fclose(fd);
	  return -1;
	}
	else{
	  if(strstr(line, "{"))
	    strcpy(t, "{\n"); // safe
	  strcat(t, block); // safe
	  fclose(fd);
	  // get rid of initial and final brackets
	  tmp = strchp(t, "{", "}");
	  strcpy(t, tmp); // safe
	  free(tmp);
	  tmp = lrtrim(t);
	  strcpy(t, tmp); // safe
	  free(tmp);
	  return 0;
	}
      }
      else{
	fprintf(stderr, "ERROR in findfunc1 in stringmanip.c: the final argument must be 0, 1, 2 or 3.\n");
	fclose(fd);
	return -1;
      }
    }
  } 
  // not found
  fclose(fd);
  return -1;
}

int findfuncsimp(char *readfile, char *s, char *t, int max)
{
  // readfile is the file containing the functions in string form
  // s is the name of a variable whose function is to be found
  // t is the string in which the function will be returned
  // max is the size of t (should be at least 1000)
  // the last character is a switch to decide whether to make the output more readable or in the original form. If an 'r' is used at the end that means that it should be made readable. We would want to use this option primarily for the user interface...

  int linelen, found=0;
  char temp1[100]; 
  char line[1000];
  char *block;
  char *twoword;
  FILE *fd;
  fd = fopen(readfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine findfuncsimp in stringmanip.c: file: %s could not be opened...\n", readfile);
    return -1;
  }
  // construct the function name
  t[0] = '\0'; // initialise t
  while((found == 0) && ((linelen = getline(fd, line, 1000)) > 0)){
    if((strcmp(temp1, s) == 0) && !(iscom(line))){
      twoword = getnthtkn(line, 2);
      if(!endswith(twoword, "_fcn")){
	found = 1;
	block = getrst(s, line);
	if((int)(strlen(line)) >= max)
	  fprintf(stderr, "ERROR in findfuncsimp in stringmanip.c: your block cannot hold the line %s.\n", line);
	else
	  snprintf(t, max, "double %s_fcn(){\n  return %s;\n}\n//////\n", s, block);
	free(block);
      }
      free(twoword);
    }
  }
  fclose(fd);
  if (found == 1) return 0;
  else return -1;
}



// get names of temporary variables for all 
// functions in a file into an array
// allocates memory

int gettvs(char *fname, char *tempv[], int max){

  int i, flag;
  FILE *fd;
  char line[1000];
  char *name;
  char *tv;
  int len;
  int numtv=0;

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in gettvs - File: %s could not be opened for reading...\n", fname);
    return -1;
  }
  // initial block

  while((len=getline(fd, line, 1000)) > 0){
    if(strstr(line, "SPECIFIC"))
      break;
  }


  while((len=getline(fd, line, 1000)) > 0){
    if (strstr(line, "double")&& !strstr(line, "_fcn()")){ // temporary variable
      tv = getrst("double", line);
      flag=0;i=0;
      while(flag==0){
	i++;
	name = getnthwd(tv, i);
	if(isonlyspace(name)){flag=1;}
	else{
	  if(isinarray(tempv, numtv, name)<0){
	    if(numtv < max)
	      tempv[numtv++] = strdup(name);
	    else
	      fprintf(stderr, "ERROR in gettvs: array size exceeded.\n");
	  }
	}
	free(name);
      }
      free(tv);
    }
  }
  fclose(fd);
  return numtv;
}

// Take a C function and get the parameters on which it depends
// like getvars, but a bit cleverer

int getdepsfromfunc(char *fn, char *deps[], int maxdeps, char *rems[], int numrems){
  int i=0, j=0, len;
  char line[1000];
  char line1[1000];
  char *tempv[100];
  int numtv=0;
  char *tv;
  int numdeps=0;
  char varstr[2000];
  char tstr[100];
  char tstr1[100];
  char **vars;
  int ind=0;
  char name[50];
  int flag;

  varstr[0] = 0; // initialise
  // get all the temporary variables
  i=0;
  while((len=getlinefromblock(&i, fn, line, 1000)) > 0){
    if (!iscomsep(line) && strstr(line, "double")&& !strstr(line, "_fcn()")){ // temporary variables
      tv = getrst("double", line);
      flag=0;j=0;
      while(flag==0){
	j++;
	getnthword(tv, name, 50, j);
	//	fprintf(stderr, "%s\t", name);
	if(isonlyspace(name)){flag=1;}
	else{
	  if(isinarray(tempv, numtv, name) < 0){
	    if(numtv < 99)
	      tempv[numtv++] = strdup(name);
	    else
	      fprintf(stderr, "ERROR in getdepsfromfunc: array size exceeded.\n");
	  }
	}
      }
      //      fprintf(stderr, "%d\n", i);
      free(tv);
    }
  }

  //
  // get the strings describing temporary variables
  //

  i=0;
  while((len=getlinefromblock(&i, fn, line, 1000)) > 0){
    if(!iscomsep(line)){
      for(j=0;j<numtv;j++){
	snprintf(tstr, 100, "%s=", tempv[j]); 
	snprintf(tstr1, 100, "%s =", tempv[j]); 
	if(strstr(line, tstr)){
	  getrest(tstr, line, line1, 1000);
	  if((strlen(varstr) + strlen(line1) + 2) < 2000){
	    strcat(varstr, line1); // safe
	    strcat(varstr, " "); // safe
	  }
	  else{fprintf(stderr, "ERROR in getdepsfromfunc. String too long.\n");free_datarray(tempv, numtv);return -1;} 
	}
	else if(strstr(line, tstr1)){
	  getrest(tstr1, line, line1, 1000);
	  if((strlen(varstr) + strlen(line1) + 2) < 2000){
	    strcat(varstr, line1); // safe
	    strcat(varstr, " "); // safe
	  }
	  else{fprintf(stderr, "ERROR in getdepsfromfunc. String too long.\n");free_datarray(tempv, numtv);return -1;} 
	}
      }
    }
  }

  //
  // get any strings in "if" statements
  //
  i=0;
  while((len=getlinefromblock(&i, fn, line, 1000)) > 0){
    if(!iscomsep(line)){
      strcpy(tstr, "if("); // safe
      strcpy(tstr1, "if ("); // safe
      if(strstr(line, tstr)){
	getrest(tstr, line, line1, 1000);
	if((strlen(varstr) + strlen(line1) + 2) < 2000){
	  strcat(varstr, line1); // safe
	  strcat(varstr, " "); // safe
	}
	else{fprintf(stderr, "ERROR in getdepsfromfunc. String too long.\n");free_datarray(tempv, numtv);return -1;}
      }
      else if(strstr(line, tstr1)){
	getrest(tstr1, line, line1, 1000);
	if((strlen(varstr) + strlen(line1) + 2) < 2000){
	  strcat(varstr, line1); // safe
	  strcat(varstr, " "); // safe
	}
	else{fprintf(stderr, "ERROR in getdepsfromfunc. String too long.\n");free_datarray(tempv, numtv);return -1;}
      }
    }
  }


  //
  // get the strings from return
  //
  i=0;
  while((len=getlinefromblock(&i, fn, line, 1000)) > 0){
    if(!iscomsep(line) && strstr(line, "return ")){
      getrest("return ", line, line1, 1000);
      if((strlen(varstr) + strlen(line1) + 2) < 2000){
	strcat(varstr, line1); // safe
	strcat(varstr, " "); // safe
      }
      else{fprintf(stderr, "ERROR in getdepsfromfunc. String too long.\n");free_datarray(tempv, numtv);return -1;}
    }
  }

  // remove temporary variables

  vars = getvs(varstr, &numdeps, tempv, numtv); 
  varstr[0]=0; // reinitialise
  for(i=0;i<numdeps;i++){
    strcat(varstr, vars[i]); // safe
    strcat(varstr, " "); // safe
  }
  freearraydat(vars, numdeps);

  // remove rems

  vars = getvs(varstr, &numdeps, rems, numrems); 
  ind=0;
  for(i=0;i<numdeps;i++)
    ind = addvar1(ind, vars[i], deps, maxdeps);

  freearraydat(vars, numdeps);

  free_datarray(tempv, numtv);
  return ind;

}

// empty lines or ones with only a left bracket

int islbracline(char *s){
  int i=0;

  while(isspace(s[i]))
    i++;
  if (s[i] == '{')
    i++;
  while(isspace(s[i]))
    i++;
  if (s[i]==0)
    return 1;
  return 0;
}

// empty lines or ones with only a right bracket

int isrbracline(char *s){
  int i=0;
  while(isspace(s[i]))
    i++;
  if (s[i] == '}')
    i++;
  while(isspace(s[i]))
    i++;
  if (s[i]==0)
    return 1;
  return 0;
}


int transfsubf(char *fn, char *vname, char *outstr, int max){
  // uses cleaned input from findfunc1
  char line[1000];
  char *tline;
  int i, len;
  int outcount=0;
  char *tvars[20];
  int numtv=0;
  i=0;
  outstr[0] = 0; // initialise

  while((len=getlinefromblock(&i, fn, line, 1000)) > 0){
    if (strstr(line, "double") || isonlyspace(line) || iscomsep(line)){
      if(strstr(line, "double") && strstr(line, "{")){
	outcount+=4;
	if(outcount<max-1){strcat(outstr, "  {\n");} // safe
      else{fprintf(stderr, "ERROR in transfsubf in stringmanip.c. Function \n%s\n too long to be translated.\n", fn);}
      }
    } // lines to ignore
    else if (strstr(line, "return")){
      tline = getrst("return", line);
      outcount += strlen(tline) + strlen(vname) + 6;
      if(outcount<max-1){
	strcat(outstr, "  "); strcat(outstr, vname); strcat(outstr, " = "); // safe
	strcat(outstr, tline); strcat(outstr, "\n"); // safe
      }
      else{fprintf(stderr, "ERROR in transfsubf in stringmanip.c. Function \n%s\n too long to be translated.\n", fn);}
      free(tline);
    }
    else{
      outcount+=strlen(line)+2;
      if(outcount<max-1){
	strcat(outstr, "  "); // safe
	strcat(outstr, line); // safe
      }
      else{fprintf(stderr, "ERROR in transfsubf in stringmanip.c. Function \n%s\n too long to be translated.\n", fn);}
    }
  }

  free_datarray(tvars, numtv);
  return 0;
}


//
// delete multiple identical lines from file infname and write to file outfname
// 

int simplinefile(char *infname, char *outfname){

  FILE *fd, *fd1;
  int linelen;
  char line[2000];
  char *alllines[1000]; // maximum of 1000 lines
  int i, j, k;
  int flag=0;
  int retstat=0;

  fd = fopen(infname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in simplinefile - File: %s could not be opened for reading...\n", infname);
    return -1;
  }

  fd1 = fopen(outfname, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in simplinefile - File: %s could not be opened for writing...\n", outfname);
    return -1;
  }

  j=0; // line counter
  while((linelen = getline(fd, line, 2000)) > 0){
    i=0;
    flag=0;
    if(iscomsep(line))
      fprintf(fd1, "%s", line);
    else{
      alllines[j] = strdup(line);
      j++;
      for(k=0;k<j-1;k++){
	if(samestr(line, alllines[k])==1){ // already there - do nothing
	  flag=1;
	  retstat++;
	  // fprintf(stderr, "\n\n\nhello\n\n\n");
	  // some kind of warning here?
	  break;
	}
      }
      if(flag==0) // print
	fprintf(fd1, "%s", line);
    }
  }
 

  while(j-- > 0)
    free(alllines[j]);


  fclose(fd);
  fclose(fd1);

  return retstat;

}


//
// delete multiple identical blocks from file infname and write to file outfname
// 

int simpblockfile(char *infname, char *outfname, char end[]){

  FILE *fd, *fd1;
  int linelen;
  char line[5000];
  char *alllines[1000]; // maximum of 1000 blocks
  int j, k;
  int flag=0;
  int retstat=0;

  fd = fopen(infname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in simpblockfile - File: %s could not be opened for reading...\n", infname);
    return -1;
  }

  fd1 = fopen(outfname, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in simpblockfile - File: %s could not be opened for writing...\n", outfname);
    fclose(fd);
    return -1;
  }

  j=0; // line counter
  while((linelen =  getblock(fd, line, end, 5000)) > 0){
    flag=0;
    if(iscomblock(line)){
      fprintf(fd1, "%s", line);
    }
    else{
      alllines[j] = strdup(line);
      j++;
      for(k=0;k<j-1;k++){
	if(samestr(line, alllines[k])==1){ // already there - do nothing
	  flag=1;
	  retstat++;
	  // fprintf(stderr, "\n\n\nhello\n\n\n");
	  // some kind of warning here?
	  break;
	}
      }
      if(flag==0) // print
	fprintf(fd1, "%s", line);
    }
  }

 

  while(j-- > 0)
    free(alllines[j]);


  fclose(fd);
  fclose(fd1);

  return retstat;

}


//
// check for mulitple processes with the same name in file infname
// 

int checkprocsfile(char *infname, char end[]){

  FILE *fd;
  int linelen;
  char line[5000];
  char *pname;
  char *alllines[1000]; // maximum of 1000 blocks
  int j, k;
  int flag=0;
  int retstat=0;

  fd = fopen(infname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in checkprocsfile - File: %s could not be opened for reading...\n", infname);
    return -1;
  }

  j=0; // line counter
  while((linelen =  getblock(fd, line, end, 5000)) > 0){
    flag=0;
    if(iscomblock(line)){} // comment block - ignore
    else{
      pname=getname("name:", line);
      alllines[j] = strdup(pname);
      j++;
      for(k=0;k<j-1;k++){
	if(strcmp(pname, alllines[k])==0){ // already there - problem
	  flag=1;
	  retstat++;
	  // some kind of warning here?
	  break;
	}
      }
      free(pname);
    }
  }

 

  while(j-- > 0)
    free(alllines[j]);


  fclose(fd);

  return retstat;

}



//
// check for mulitple functions with the same name in file infname
// 

int checkfuncsfile(char *infname, char end[]){

  FILE *fd;
  int linelen, len;
  char line[5000];
  char pline[200];
  char *alllines[1000]; // maximum of 1000 blocks
  int i, j, k, coms;
  int flag=0;
  int retstat=0;

  fd = fopen(infname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in checkfuncsfile - File: %s could not be opened for reading...\n", infname);
    return -1;
  }

  j=0; // line counter
  while((linelen =  getblock(fd, line, end, 5000)) > 0){
    i=0;
    flag=0;
    if(iscomblock(line)){} // comment block - ignore
    else{
      coms=0;len=1;
      while(coms==0 && len!=0){
	len=getlinefromblock(&i, line, pline, 200);
	if(!(iscomsep(pline)))
	  coms=1;
      }
      alllines[j] = strdup(pline);
      j++;
      for(k=0;k<j-1;k++){
	if(samestr(pline, alllines[k])==1){ // already there - problem
	  flag=1;
	  retstat++;
	  // some kind of warning here?
	  break;
	}
      }
    }
  }

 

  while(j-- > 0)
    free(alllines[j]);


  fclose(fd);

  return retstat;

}



int isinarray(char *v[], int numv, char *s){
  // is the string s a member of the array v? If so return its index, if not return -1
  int i;
  for(i=0;i<numv;i++)
    if(strcmp(s, v[i]) == 0)
      return i;
  return -1;
}

int isnumstart(char *s, int j)
{
  // allowed first character of a number
  int c; /* to store EOF */
  c = s[j];
  if(isdigit(c) || (c == '.'))
    return 1;
  else
    return 0;
}

int isinnum(char *s, int j)
{
  // allowed characters inside a number
  int c; /* to store EOF */
  c = s[j];
  if(isdigit(c) || (c == '.'))
    return 1;
  if(c=='e'){
    if(j>0 && isdigit(s[j-1]))
      return 1;
  }
  if(c=='+' || c=='-'){
    if(j>1 && s[j-1]=='e' && isdigit(s[j-2]))
      return 1;
  }
  return 0;
}


int isvarstart(char *s, int j)
{
  // allowed first character of variable name
  int c; /* to store EOF */
  c = s[j];
  if(isalpha(c) || (c == '_'))
    return 1;
  else
    return 0;
}

int isinvar(char *s, int j)
{
  // allowed characters inside a variable name (includes numbers)
  int c; /* to store EOF */
  c = s[j];
  if(isalnum(c) || (c == '_')) 
    return 1;
  else
    return 0;
}

// check whether a string could be a variable name

int isvar(char *s, char *rem[], int numrems){
  int k=0;
  if(isvarstart(s, 0) == 0) // starting wrong
    return 0;
  if(isinarray(rem, numrems, s) >= 0) // is in rems
    return 0;
  while(s[k] != '\0'){
    if(isinvar(s, k) == 0)
      return 0;
    k++;
  }
  return 1;

}

int isnumber(char *s){
  // decide if string s is a number
  int k=0, pc=0;
  if(!s[k]) // empty
    return 0;
  while(isspace(s[k])) // skip space
    k++;
  if(!s[k]) // empty (apart from spaces)
    return 0;
  else if((s[k] == '-') || (s[k] == '+')){
    k++;
    while(isspace(s[k]))
      k++;
  }
  if(!s[k]) // empty (apart from spaces and + or - sign)
    return 0;

  if(!isnumstart(s, k))
    return 0;
  if(s[k]=='.'){pc++;}
  while(s[k] && !isspace(s[k])){
    if(!isinnum(s, k))
      return 0;
    if(s[k]=='.'){
      pc++;
      if(pc > 1)
	return 0;
    }
    k++;
  }
  while(isspace(s[k]))
    k++;
  if(s[k])
    return 0;

  return 1;
}

int ispureint(char *s){
  // decide if string s is a pure integer (no sign allowed)
  int k=0;
  while(s[k] !=0 && isspace(s[k])) //skip space
    k++;
  if(!isdigit(s[k])) // empty (apart from spaces)
    return 0;

  while(s[k]!=0 && isdigit(s[k]))
    k++;
  if(!(isspace(s[k])) && (s[k] != '\0'))
    return 0;
  while(s[k] !=0 && isspace(s[k])) //skip space
    k++;
  if(s[k]!=0) // empty (apart from spaces)
    return 0;

  return 1;
}


int getvars(char *s, char *v[], int max, char *rem[], int numrems){
  char *tempword;
  int i=0, j=0, k=0; // k is the counter in s, i is the counter of variables found

  while((i < max) && s[k]){
    j=0;
    while((isvarstart(s,k) == 0) && s[k]){k++;}
    while(isinvar(s,k)){j++;k++;}
    tempword = strchop2(s, k-j, j);
    if(strlen(tempword) > 0){
      if(isinarray(rem, numrems, tempword) < 0)
	i = addvar1(i, tempword, v, max);
    }
    free(tempword);
  }
  if(i==max){
    fprintf(stderr, "ERROR in routine getvars in file \"stringmanip.c\": maximum number of terms got.\n");
    return -1;
  }
  return i;
}

//
// memory allocating version of getvars
// free with freearraydat
//

char **getvs(char *s, int *num, char *rem[], int numrems){
  char *tempword;
  char **v=NULL, **v1=NULL;
  int tot=0;
  int i=0, j=0, k=0; // k is the counter in s, i is the counter of variables found

  tot = getnumvars(s, rem, numrems);
  v=charar(0, tot-1);


  while((i < tot) && s[k]){
    j=0;
    while((isvarstart(s,k) == 0) && s[k]){k++;}
    while(isinvar(s,k)){j++;k++;}
    tempword = strchop2(s, k-j, j);
    if(strlen(tempword) > 0){
      if(isinarray(rem, numrems, tempword) < 0)
	i = addvar1(i, tempword, v, tot);
    }
    free(tempword);
  }
  (*num)=i;

  if(i>0){
    v1=charar(0, i-1);
    for(k=0;k<i;k++)
      v1[k]=strdup(v[k]);
  }

  if(v){
    while(i-- > 0)
      free(v[i]);
    free_charar(v, 0, tot-1);
  }
  return v1; // null if nothing found
}

// If not at the start of a variable skip to the start of the next variable
// If at the start of a variable, skip to the start of the next variable
// returns the number of characters stepped

int stepvars(char *s){
  int j=0, k=0;
  while(isinvar(s,k)){j++;k++;}
  while(!isvarstart(s,k) && s[k]){k++;}
  return k;
}

int startsvar(char *s, char *vararray[], int numvars){
  int i, len;
  for(i=0;i<numvars;i++){
    len=strlen(vararray[i]);
    if((strncmp(s, vararray[i], len)==0) && isgood(s, len)){
      return i;
    }
  }
  return -1;
}

// returns the total number of variable names in a string minus rems. It returns a *different* value from getvars because getvars counts each variable name only once which getnumvars counts it as many times as necessary.

int getnumvars(char *s, char *rem[], int numrems){
  int i, j, k, l;
  i=1; j=0; k=0;
  while(!isvarstart(s,j) && s[j]){j++;}
  while(i!=0){
    if((l=startsvar(s+j, rem, numrems)) < 0){k++;} // var found
    i=stepvars(s+j);
    j+=i;
  }
  return k-1; // counts one at the very end...
}

int varnum(char *s, char *vararray[], int numvars){
  int i, j, k, l;
  i=1; j=0; k=0;
  while(!isvarstart(s,j) && s[j]){j++;}
  while(i!=0){
    if((l=startsvar(s+j, vararray, numvars)) >= 0){k++;} // var found
    i=stepvars(s+j);
    j+=i;
  }
  return k;
}

int maxvlen(char *vararray[], int numvars){
  int i, maxlen=0;
  for(i=0;i<numvars;i++)
    maxlen = max(maxlen, (int)(strlen(vararray[i])));
  return maxlen;
}

int maxldiff(char *array1[], char *array2[], int numvars){
  int i, d, maxlen=0;
  for(i=0;i<numvars;i++){
    if((d= (int) strlen(array2[i]) - (int) strlen(array1[i])) > 0)
      maxlen = max(maxlen, d);
  }
  return maxlen;
}
 
 

// exactly like getvars, but doesn't check for rems...
// memory is allocated for entries in v[]
// and can be freed with free_datarray

int getallvars(char *s, char *v[], int max){
  char *tempword;
  int i, j, k;
  i=0, k=0;
  while(s[k]){
    j=0;
    while(s[k] && !isvarstart(s,k)){k++;}
    while(isinvar(s,k)){j++;k++;} // get the word
    tempword = strchop2(s, k-j, j);
    if(!isonlyspace(tempword) && i<max){
      i = addvar1(i, tempword, v, max);
    }
    else if(!isonlyspace(tempword) && i==max){ // failed
      fprintf(stderr, "ERROR in routine getallvars in file \"stringmanip.c\": maximum number of terms got.\n");
      free(tempword);
      return i;
    }
    free(tempword);
  }

  return i;
}

//gets the total number of variables in an expression

int getnumallvars(char *s){
  int i, k;
  i=0, k=0;
  while(s[k]){
    while(s[k] && !isvarstart(s,k)){k++;}
    if(isvarstart(s,k)){
      i++;
      while((isinvar(s,k) != 0)){k++;}
    }
  }
  return i;
}


int getnthword(char *s, char *v, int len, int n){
  // get the nth valid word from a string s and put it in v. Returns the next position in  s
  int i, j, k;
  i=0, k=0;
  v[0]=0;
  if(len < 2){
    fprintf(stderr, "ERROR in getnthword in stringmanip.c: third argument must be at least 2.\n");
    return 0;
  }
  while(s[k]){
    j=0;
    while((isvarstart(s,k) == 0) && (s[k] != '\0')){k++;} // skip nonwords
    for(i=0;i<n-1;i++){
      while((isinvar(s,k) != 0) && (s[k] != '\0')){k++;} // skip word
      while((isvarstart(s,k) == 0) && (s[k] != '\0')){k++;} //skip nonwords
    }
    while((j<len-1) && (isinvar(s,k) != 0)){ // get the word
      v[j++] = s[k++];
    }
    v[j++] = '\0';
    if(j==len){
      fprintf(stderr, "WARNING: in routine getnthword in file stringmanip.c: word is longer than maximum length.\n");
    }
    return k;
  }
  return 0;
}


char *getnthwd(char *s, int n){
  // the memory allocating version of getnthword
  int i, j, k;
  i=0, k=0;
  char *v=NULL;
  while(s[k] != '\0'){
    while(s[k] && !isvarstart(s,k)){k++;} // skip nonwords
    for(i=0;i<n-1;i++){
      while(s[k] && isinvar(s,k)){k++;} // skip word
      while(s[k] && !isvarstart(s,k)){k++;} // skip nonwords
    }
    j=0;
    while(isinvar(s,k)){j++;k++;} // get the word
    v = strchop2(s, k-j, j);
    return v;
  }
  if(v==NULL)
    v=strdup("");
  return v;
}

char *getvalfromfile(char *fname, char *vname){
  int len;
  FILE *fd;
  char oneline[3000];
  char *varname;
  int lim=3000;
  char *tmp;
  char *t;
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getvalfromfile in \"stringmanip.c\": file %s could not be opened for reading.\n", fname);
    tmp = strdup("");
    return tmp;
  }

  while((len=getline(fd, oneline, lim))>0){
    if(!(iscomsep(oneline))){
      varname = getnthwd(oneline, 1);
      if(strcmp(varname, vname)==0){ // found
	t=getrst(varname, oneline);tmp=lrtrim(t);free(t);
	if(!isonlyspace(tmp)){
	  free(varname);
	  fclose(fd);
	  return tmp;
	}
	else{
	  fprintf(stderr, "ERROR in getvalfromfile in stringmanip.c. %s has no value in %s. Exiting.\n", vname, fname);
	  free(varname);
	  fclose(fd);
	  return tmp;
	}
      }
      free(varname);
    }
  }
  fclose(fd);
  //  fprintf(stderr, "ERROR in getvalfromfile in stringmanip.c: %s not found in %s.\n", vname, fname);
  tmp = strdup("");
  return tmp;

}

// 
// quiet version of getvalfromfile -- allows empty string
//

char *getvalfromfileq(char *fname, char *vname){
  int len;
  FILE *fd;
  char oneline[3000];
  char *varname;
  int lim=3000;
  char *tmp;
  char *t;
  fd = fopen(fname, "r");
  if(!fd){
    tmp = strdup("");
    return tmp;
  }

  while((len=getline(fd, oneline, lim))>0){
    if(!(iscomsep(oneline))){
      varname = getnthwd(oneline, 1);
      if(strcmp(varname, vname)==0){
	t=getrst(varname, oneline);tmp=lrtrim(t);free(t);
	if(!isonlyspace(tmp)){
	  free(varname); fclose(fd);
	  return tmp;
	}
	else{
	  free(varname); fclose(fd);
	  free(tmp);tmp = strdup("__ZZ__"); // empty value, but exists
	  return tmp;
	}
      }
      free(varname);
    }
  }
  fclose(fd);
  tmp = strdup("");
  return tmp;
}

//
// clever version of getvalfromfile - it recognises generic rules
// fails silently returning an empty string.
//

char *getcvalfromfile(char *fname, char *vname){
  char *tmp;
  char *pf;
  char *gv;
  int vind;
  char *p;

  tmp=getvalfromfile(fname, vname);
  if(!isonlyspace(tmp)){return tmp;} // found
  else{  // failed, now look for a generic rule
    free(tmp);
    pf=getgenname(vname);
    if(isonlyspace(pf)){
      //      fprintf(stderr, "ERROR in getcvalfromfile in stringmanip.c: %s not found in %s and is not a generic variable.\n", vname, fname);
      free(pf);tmp = strdup("");
      return tmp;
    }
    vind = getvindex(vname);
    if(vind> 1e10){
      fprintf(stderr, "ERROR in getcvalfromfile in stringmanip.c: integer %d is too long in name %s.\n", vind, vname);
      tmp = strdup(""); free(pf);
      return tmp;
    }
    gv = getvalfromfile(fname, pf);
    if(isonlyspace(gv)){
      //      fprintf(stderr, "ERROR in getcvalfromfile in stringmanip.c: %s not found in %s and no generic rule found either.\n", vname, fname);
      free(gv); tmp = strdup(""); free(pf);
      return tmp;
    }
    else if(isnumber(gv)){
      tmp=strdup(gv); free (gv); free(pf);
      return tmp;
    }
    else if((p = strstr(gv, pf)) && (strlen(p)==strlen(gv)) && (strlen(gv)!=strlen(pf))){
      tmp = (char*) malloc(sizeof(char) * (strlen(gv) + 15)); // should be safe
      pf[strlen(pf)-1] =0; // get rid of terminal "i". Safe as pf not only space
      sprintf(tmp, "%s%d%s", pf, vind, gv+strlen(pf)+1); // safe
      free(gv); free(pf);
      return tmp;
    }
    else{
      //      fprintf(stderr, "ERROR in getcvalfromfile in stringmanip.c: %s not found in %s and a well formed generic rule not found either.\n", vname, fname);
      free(gv); tmp = strdup(""); free(pf);
      return tmp;
    }
  }
}

//
// Version of getcvalfromfileq which uses getvalfromfileq
//

char *getcvalfromfileq(char *fname, char *vname){
  char *tmp;
  char *pf;
  char *gv;
  int vind;
  char *p;

  tmp=getvalfromfileq(fname, vname);
  if(!isonlyspace(tmp)){return tmp;} // found
  else{  // failed, now look for a generic rule
    free(tmp);
    pf=getgenname(vname);
    if(isonlyspace(pf)){
      free(pf);tmp = strdup("");
      return tmp;
    }
    vind = getvindex(vname);
    if(vind> 1e10){
      fprintf(stderr, "ERROR in getcvalfromfile in stringmanip.c: integer %d is too long in name %s.\n", vind, vname);
      tmp = strdup(""); free(pf);
      return tmp;
    }
    gv = getvalfromfileq(fname, pf);
    if(isonlyspace(gv)){ 
      free(gv); tmp = strdup(""); free(pf);
      return tmp;
    }
    else if(isnumber(gv) || strcmp(gv, "__ZZ__")==0){
      tmp=strdup(gv); free (gv); free(pf);
      return tmp;
    }
    else if((p = strstr(gv, pf)) && (strlen(p)==strlen(gv)) && (strlen(gv)!=strlen(pf))){
      tmp = (char*) malloc(sizeof(char) * (strlen(gv) + 15)); // should be safe
      pf[strlen(pf)-1] =0; // get rid of terminal "i". Safe because pf not only space
      sprintf(tmp, "%s%d%s", pf, vind, gv+strlen(pf)+1); // safe
      free(gv); free(pf);
      return tmp;
    }
    else{
      free(gv); free(pf); tmp = strdup(""); 
      return tmp;
    }
  }
}



int getnthblock(char *s, char *v, int len, int n){
  // get the nth valid block (only spaces count as separators) from a string s and put it in v. Returns the next position in  s. 
  int i, j, k;
  i=0, k=0;
  if(len < 2){
    fprintf(stderr, "ERROR in getnthblock in stringmanip.c: third argument must be at least 2.\n");
    return 0;
  }
  v[0] = '\0'; // in case we have an empty string, return an empty word.
  while(s[k] != '\0'){
    j=0;
    while(isspace((int) s[k])) // skip space
      k++;
    for(i=0;i<n-1;i++){
      while(!(isspace((int) s[k]))) // skip first word 
        k++;
      while(isspace((int) s[k])) //skip space
        k++;
    }
    while((j<len-1) && !(isspace((int) s[k])) && (s[k] != '\0')){ // get the word
      v[j++] = s[k++];
    }
    v[j++] = '\0';
    if(j==len){
      fprintf(stderr, "WARNING: in routine getnthblock in file stringmanip.c: word %s... is longer than maximum length %d.\n", v, len);
    }
    return k;
  }
  return 0;
}


char *getnthblk(char *s, int n){
  // memory allocating version of getnthblock
  int i=0, j, k=0;
  char *v=NULL;

  while(s[k] != '\0'){
    while(isspace((int) s[k])){k++;} // skip space
    for(i=0;i<n-1;i++){
      while(!(isspace((int) s[k])) && s[k]){k++;} // skip word 
      while(isspace((int) s[k])){k++;} //skip space
    }
    j=0;
    while(!(isspace((int) s[k])) && s[k]){j++;k++;} // get the word
    v=strchop2(s, k-j, j);
    return v;
  }    
  if(v==NULL)
    v=strdup("");
  return v;
}


int getoption(char *fname, char *optname, char v[], int max){
  FILE *fd;
  int len;
  char oneline[200];
  char *modname;
  char *modname1;
  int lim=200;

  if(max < 2){
    fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": you need a longer string to store the option.\n");
    return -1;
  }
  v[0] = 0; // return an empty string on any failure
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": file \"%s\" could not be opened for reading.\n", fname);
    return -1;
  }
  while((len = getline(fd, oneline, lim)) > 0){
    if(!(iscomsep(oneline))){
      modname = getnthblk(oneline, 1);
      if(strcmp(modname, optname) == 0){
        modname1 = getnthblk(oneline, 2);
        if((int)(strlen(modname1)) < max-1){
	  strcpy(v, modname1); // safe
	  if(isonlyspace(modname1)){
	    if(strcmp(optname, "EDITOR")==0){
	      if(fileexists("/usr/bin/gedit"))
		strncpy(v, "gedit", max-1);
	      else
	        fprintf(stderr, "WARNING in routine getoption in \"stringmanip.c\": Option \"%s\" in file \"%s\" is empty.\n", optname, fname);
	    }
	    else{
	      fprintf(stderr, "WARNING in routine getoption in \"stringmanip.c\": Option \"%s\" in file \"%s\" is empty.\n", optname, fname);
	    }
	  }
	  fclose(fd);free(modname);free(modname1);return 0;
	}
        else{
	  fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": Option \"%s\" in file \"%s\" has value \"%s\" which is too long.\n", optname, fname, modname1);
	  v[0] = 0;fclose(fd);free(modname);free(modname1);return -1;
        }
      }
      free(modname);
    }
  }
  fclose(fd);
  if(strcmp(optname, "EDITOR")==0){
    if(fileexists("/usr/bin/gedit")){
      strncpy(v, "gedit", max-1);
      return 0;
    }
    else
      fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": Option \"%s\" could not be found in file \"%s\".\n", optname, fname);
  }
  else{
    fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": Option \"%s\" could not be found in file \"%s\".\n", optname, fname);
    v[0] = 0;
  }
  return -2;
}

// quiet simple version of getoption

int getoptq(char *fname, char *optname, char v[], int max){
  FILE *fd;
  int len;
  char oneline[2000];
  char *modname;
  char *modname1;
  int lim=2000;

  if(max < 2){
    fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": you need a longer string to store the option.\n");
    return -1;
  }
  v[0] = 0; // return an empty string on any failure
  fd = fopen(fname, "r");
  if(!fd){
    return -1;
  }
  while((len = getline(fd, oneline, lim)) > 0){
    if(!(iscomsep(oneline))){
      modname = getnthblk(oneline, 1);
      if(strcmp(modname, optname) == 0){
        modname1 = getnthblk(oneline, 2);
        if((int)(strlen(modname1)) < max-1){
	  strcpy(v, modname1); // safe
	  fclose(fd);free(modname);free(modname1);return 0;
	}
        else{
	  fprintf(stderr, "ERROR in routine getoption in \"stringmanip.c\": Option \"%s\" in file \"%s\" has value \"%s\" which is too long.\n", optname, fname, modname1);
	  v[0] = 0;fclose(fd);free(modname);free(modname1);return -1;
        }
      }
      free(modname);
    }
  }
  fclose(fd);
  v[0]=0;
  return -2;
}

int getfileblk(char *fname, char v[], int max){
  // a logical function so returns 0 on any failure
  int c=0, i=0;
  FILE *fd;
  if(max>1)
    v[0] = 0; 
  else{
    fprintf(stderr, "ERROR in getfileblk in \"stringmanip.c\": block needs to be longer.\n");
    return 0;
  }

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getfileblk in \"stringmanip.c\": file \"%s\" could not be opened for reading.\n", fname);
    return 0;
  }
  while((c=getc(fd))!=EOF && isspace(c)){}
  if(c!=EOF){
    v[i++]=c;
    while((c=getc(fd))!=EOF && !isspace(c) && i<max-1){v[i++]=c;}
    v[i++]=0;
  }

  fclose(fd);

  if(strlen(v)==0)
    return 0;

  return i;
}

//
// get the value of some default for model modname from file fname
//

char *getmoddef(char *fname, char *modname, char *obj){
  FILE *fd;
  char *v;
  int len;
  int i=0;
  char vardesc[1000];
  char *varname;
  char *tmp;
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getmoddef: File \"%s\" could not be opened...\n", fname);
    v=strdup("");
    return v;
  }
  if (strcmp(obj, "modname")==0) // exists
    i=1;
  else if (strcmp(obj, "input")==0)
    i=2;
  else if (strcmp(obj, "var")==0)
    i=3;
  else if (strcmp(obj, "pvals")==0)
    i=4;
  else if (strcmp(obj, "auto")==0)
    i=5;
  else if (strcmp(obj, "module")==0)
    i=6;
  else if (strcmp(obj, "modeldesc")==0)
    i=7;
  else if (strcmp(obj, "autoinput")==0)
    i=8;
  else{
    fprintf(stderr, "ERROR in getmoddef: third argument must be one of \"modname\", \"input\", \"var\", \"pvals\", \"auto\", \"module\", \"modeldesc\" or \"autoinput\"\n");
    v=strdup("");
    fclose(fd);
    return v;
  }

  while((len = getline(fd, vardesc, 1000) > 0)){
    if(!iscomsep(vardesc)){
      varname = getnthblk(vardesc, 1);
      if(strcmp(modname, varname) == 0){
	v=getnthblk(vardesc, i);
        if(i==2 && !startswith(v, "models/inputs/")){
	  tmp= enclose(v, "models/inputs/", "");
	}
	else if(i==4 && !startswith(v, "models/pvals/")){
	  tmp= enclose(v, "models/pvals/", "");
	}
	else if(i==6 && !startswith(v, "models/modules/")){
	  tmp= enclose(v, "models/modules/", "");
	}
	else if(i==7 && !startswith(v, "models/modeldescs/")){
	  tmp= enclose(v, "models/modeldescs/", "");
	}
	else if(i==8 && !startswith(v, "models/autoreginputs/")){
	  tmp= enclose(v, "models/autoreginputs/", "");
	}
	else{
	  tmp=strdup(v);
	}
	fclose(fd);free(v);free(varname);
	return tmp;
      }
    }
  }
  fprintf(stderr, "WARNING in getmoddef: the model %s could not be found in file %s\n", modname, fname);
  fclose(fd);
  tmp=strdup("");
  return tmp;

}

//
// The version which removes unnecessary initial strings
//

char *getmoddef1(char *fname, char *modname, char *obj)
{
  char *tmp;
  char *moddefval;
  tmp = getmoddef(fname, modname, obj);
  if(strcmp(obj, "input")==0)
    moddefval=strchp(tmp, "models/inputs/", "");
  else if(strcmp(obj, "pvals")==0)
    moddefval=strchp(tmp, "models/pvals/", "");
  else if(strcmp(obj, "module")==0)
    moddefval=strchp(tmp, "models/modules/", "");
  else if(strcmp(obj, "modeldesc")==0)
    moddefval=strchp(tmp, "models/modeldescs/", "");
  else if(strcmp(obj, "autoinput")==0)
    moddefval=strchp(tmp, "models/autoreginputs/", "");
  else
    moddefval=strdup(tmp);
  free(tmp);
  return moddefval;
}


//
// change the value of some default for model modname from file fname
//

int changemoddef(char *fname, char *modname, char *obj, char *newval){
  FILE *fd, *fd1;
  int len;
  char vardesc[1000];
  char *oneword;
  char *input;
  char *var;
  char *pvals;
  char *auto1;
  char *module;
  char *modeldesc;
  char *autoinput;

  if(isonlyspace(newval)){
    fprintf(stderr, "ERROR in changemoddef: the new value cannot be empty. Nothing being done.\n");
    
    return -2;
  }
  if(cpfile("tempfiles/__temp_file.tmp", fname)<0){
    fprintf(stderr, "ERROR in changemoddef: Couldn't copy the file %s to \"tempfiles/__temp_file.tmp\". Nothing being done.\n", fname);
    
    return -1;
  }

  input = getmoddef1(fname, modname, "input");
  var = getmoddef1(fname, modname, "var");
  pvals = getmoddef1(fname, modname, "pvals");
  auto1 = getmoddef(fname, modname, "auto");
  module = getmoddef1(fname, modname, "module");
  modeldesc = getmoddef1(fname, modname, "modeldesc");
  autoinput = getmoddef1(fname, modname, "autoinput");

  fd = fopen("tempfiles/__temp_file.tmp", "r");
  if(!fd){
    fprintf(stderr, "ERROR in changemoddef: Couldn't open the file \"tempfiles/__temp_file.tmp\". Nothing being done.\n");
    free(input);free(var);free(pvals);free(auto1);free(module);free(modeldesc);
    return -1;
  }

  if (strcmp(obj, "input")==0){
    free(input);
    input = strchp(newval, "models/inputs/", "");
  }
  else if (strcmp(obj, "var")==0){
    free(var);
    var=strdup(newval);
  }
  else if (strcmp(obj, "pvals")==0){
    free(pvals);
    pvals = strchp(newval, "models/pvals/", "");
  }
  else if (strcmp(obj, "auto")==0){
    free(auto1);
    auto1=strdup(newval);
  }
  else if (strcmp(obj, "module")==0){
    free(module);
    module = strchp(newval, "models/modules/", "");
  }
  else if (strcmp(obj, "modeldesc")==0){
    free(modeldesc);
    modeldesc=strchp(newval, "models/modeldescs/", "");
  }
  else if (strcmp(obj, "autoinput")==0){
    free(autoinput);
    autoinput=strdup(newval);
  }
  else{
    fprintf(stderr, "ERROR in changemoddef: third argument must be one of \"input\", \"var\", \"pvals\", \"auto\", \"module\" or \"modeldesc\"\n");
    free(input);free(var);free(pvals);free(auto1);free(module);free(modeldesc);
    fclose(fd);
    return -1;
  }

  fd1 = fopen(fname, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in changemoddef: Couldn't open the file \"%s\". Nothing being done.\n", fname);
    fclose(fd);
    free(input);free(var);free(pvals);free(auto1);free(module);free(modeldesc);
    return -1;
  }
  while((len = getline(fd, vardesc, 1000))> 0){
    if(iscomsep(vardesc))
      fprintf(fd1, "%s", vardesc);
    else{
      oneword = getnthblk(vardesc, 1);
      if(strcmp(modname, oneword)!=0)
	fprintf(fd1, "%s", vardesc);
      else{
	fprintf(fd1, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n", modname, input, var, pvals, auto1, module, modeldesc, autoinput);
      }
      free(oneword);
    }
  }

  fclose(fd);
  fclose(fd1);
  free(input);free(var);free(pvals);free(auto1);free(module);free(modeldesc);free(autoinput);
  return 0;

}



int getnthtoken(char *s, char *v, int len, int n){
  // get the nth valid token from a string s and put it in v. Returns the next position in  s. Designed for numbers or words, but not necessarily very good... Try to think it through more carefully
  int i, j, k;
  i=0, k=0;
  int varstart;
  if(len < 2){
    fprintf(stderr, "ERROR in getnthtoken in stringmanip.c: third argument must be at least 2.\n");
    return 0;
  }
  v[0] = '\0'; // in case we have an empty string, return an empty word.
  while(s[k] != '\0'){
    j=0;
    while(!(isalnum(s[k])) && (s[k] != '-') && (s[k] != '_') && (s[k] != '.') && (s[k] != '\0')) // skip nonwords
      k++;
    for(i=0;i<n-1;i++){
      varstart=0;
      while((isalnum(s[k])) || (s[k] == '.') || ((s[k] == '+') && (k==0 || (s[k-1]=='e'))) || ((s[k] == '-')&& (k==0 || (s[k-1]=='e') || varstart==0))  || (s[k] == '_')){ // skip word including a number
        k++;
	varstart++;
      }
      while(!(isalnum(s[k])) && (s[k] != '-') && (s[k] != '_') && (s[k] != '.') && (s[k] != '\0')) //skip nonwords
        k++;
    }

    varstart=0;
    while((j<len-1) && ((isalnum(s[k])) || ((s[k] == '.') && isdigit(s[k-1])) || ((s[k] == '+') && (k==0 || (s[k-1]=='e'))) || ((s[k] == '-')&& (k==0 || (s[k-1]=='e') || varstart==0))  || (s[k] == '_'))){
      v[j++] = s[k++];
      varstart++;
    }
    v[j++] = '\0';
    if(j==len){
      fprintf(stderr, "WARNING: in routine getnthtoken in file stringmanip.c: word %s... is longer than maximum length.\n", v);
    }
    return j;
  }
  return 0;
}


char *getnthtkn(char *s, int n){
  //memory allocating version of getnthtoken
  char *v=NULL;
  int i, j, k;
  int varstart;

  i=0, k=0;
  if(n<1){
    fprintf(stderr, "ERROR in getnthtkn in stringmanip.c: second argument must be positive.\n");
    v=strdup("");
    return v;
  }
  while(s[k] != '\0'){
    j=0;
    while(!(isalnum(s[k])) && (s[k] != '-') && (s[k] != '_') && (s[k] != '.') && (s[k] != '\0')){k++;} // skip nonwords
    for(i=0;i<n-1;i++){
      varstart=0;
      while((isalnum(s[k])) || (s[k] == '.') || ((s[k] == '+') && (k==0 || (s[k-1]=='e'))) || ((s[k] == '-')&& (k==0 || (s[k-1]=='e') || varstart==0))  || (s[k] == '_')){ // skip word including a number
        k++;
	varstart++;
      }
      while(!(isalnum(s[k])) && (s[k] != '-') && (s[k] != '_') && (s[k] != '.') && (s[k] != '\0')){k++;} //skip nonwords
    }

    varstart=0;
    while(((isalnum(s[k])) || ((s[k] == '.') && isdigit(s[k-1])) || ((s[k] == '+') && (k==0 || (s[k-1]=='e'))) || ((s[k] == '-')&& (k==0 || (s[k-1]=='e') || varstart==0))  || (s[k] == '_'))){
      k++;j++;varstart++;
    }
    v = strchop2(s, k-j, j);
    return v;
  }
  if(v==NULL)
    v=strdup("");
  return v;
}

int getnumtokens(char *s){
  // returns the number of tokens in string s
  int j=0, k=0;
  while(s[k] && !isvarstart(s, k) && !isnumstart(s, k)) // skip nonwords
    k++;
  while(s[k]){
    if(isvarstart(s, k)){
      k++;
      while(isinvar(s, k)){k++;}
    }
    if(isnumstart(s, k)){
      k++;
      while(isinnum(s, k)){k++;}
    }
    while(s[k] && !isvarstart(s, k) && !isnumstart(s, k)){k++;}
    j++;
  }
  return j;

}



int getfield(char *descfile, char varname[], char *field, char vardesc[], int lim)
{
  // Get the field for varname into vardesc as follows:
  // 1) Open the file
  // 2) Search for a line with first or second word as the name of the variable
  // 3) Get the field prefaced by "field" into vardesc until either we hit the limit, or a newline character. 

  int i, found=0, fieldfound=0, k=0; 
  char *varnameline;
  char fieldname[32];
  int len;
  int lim_line = 1000;
  char oneline[1000];
  FILE *fd;
  fd = fopen(descfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getfield in \"stringmanip.c\": file %s could not be opened for reading.\n", descfile);
    return -1;
  }
  vardesc[0] = '\0'; // initialise the description

  while((found == 0) && (len = getline(fd, oneline, lim_line)) > 0){
    varnameline = getnthwd(oneline, 1); // 1st word
    if(strcmp(varnameline, varname) == 0){ // found block
      found = 1;
      fieldfound = 0;
      while((len = getline(fd, oneline, lim_line)) > 0 && strstr(oneline, "****") == NULL){
	i = getnthword(oneline, fieldname, 31, 1);
	if(strcmp(fieldname, field) == 0){
	  fieldfound = 1;
	  k=0;
	  while (!isend(oneline[i]) && k < lim-1){vardesc[k++] = oneline[i++];}    
	  vardesc[k++] = '\0';
	  if(k==lim)
	    fprintf(stderr, "WARNING in routine getfield: the maximum length of a field - %d - has been reached.\nReset lim.\n", lim); 
	  fclose(fd);
	  free(varnameline);
	  return k;
	}
      }
    }
    free(varnameline);
  }
  if(found == 0)
    fprintf(stderr, "variable \"%s\" not found by routine getfield.\n", varname);
  if(fieldfound == 0)
    fprintf(stderr, "variable \"%s\" found but field \"%s\" not found by routine getfield.\n", varname, field);
  fclose(fd);
  return k;


}

int getfieldsimp(char *descfile, char varname[], char vardesc[], int lim)
{
  // like getfield, but just searches for the field and gets the rest of the line

  int i=0, found=0, k=0; 
  char varnameline[100];
  int len;
  int lim_line = 1000;
  char oneline[1000];
  FILE *fd;
  fd = fopen(descfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getfieldsimp in \"stringmanip.c\": file %s could not be opened for reading.\n", descfile);
    return -1;
  }
  vardesc[0] = '\0'; // initialise the description

  while((found == 0) && (len = getline(fd, oneline, lim_line)) > 0){
    i = getnthword(oneline, varnameline, 100, 1); // 1st word
    //    printf("%s\n", varnameline);
    if(strcmp(varnameline, varname) == 0){ // found block
      found = 1;
      k=0;
      while (isspace(oneline[i]) || (oneline[i] == ':') || (oneline[i] == ';') || (oneline[i] == ',')){i++;} // skip space, etc
      while (!isend(oneline[i]) && k < lim-1){
	vardesc[k++] = oneline[i++];
      }
      vardesc[k++] = '\0';
      if(k==lim){
	fprintf(stderr, "WARNING in routine getfieldsimp: the maximum length of a field - %d - has been reached.\nReset lim.\n", lim);
      }
    }
  }
  if(found == 0)
    fprintf(stderr, "variable \"%s\" not found by routine getfieldsimp.\n", varname);
  fclose(fd);
  return k;
}

// like chemgets2, but for getting data with several arbitrary separation characters

int chemgets3(char *s, char *v[], int max, char *seps){
  // sep is the separator
  int i, j, k;
  i=0, k=0;
  while((i < max) && (s[k] != '\0') ){
    j=0;
    while(strchr(seps, s[k]) || isspace((int) s[k])){k++;} // skip white space
    while(!strchr(seps, s[k]) && !isend(s[k])){j++;k++;}
    if(j>0)
      v[i++] = strchop2(s, k-j, j);
  }
  if(i==max){
    fprintf(stderr, "ERROR in routine chemgets3 in file stringmanip.c: maximum number of terms got.\n");
  }
  return i;
}

// like chemgets, but for getting data with an arbitrary separation character

int chemgets2(char *s, char *v[], int max, char sep){
  // sep is the separator
  int i, j, k;
  i=0, k=0;
  while((i < max) && (s[k] != '\0') ){
    j=0;
    while((s[k] == sep) || isspace((int) s[k])){k++;} // skip white space
    while((s[k] != sep) && !isend(s[k])){j++;k++;}
    if(j>0)
      v[i++] = strchop2(s, k-j, j);
  }
  if(i==max){
    fprintf(stderr, "ERROR in routine chemgets2 in file stringmanip.c: maximum number of terms got.\n");
  }
  return i;
}

// follows format of chemgets3

int getnumblocks(char *s, char *seps){
  // sep is the separator
  int i, j, k;
  i=0, k=0;
  while(s[k] != 0){
    j=0;
    while(strchr(seps, s[k]) || isspace((int) s[k])){k++;} // skip white space
    while(!strchr(seps, s[k]) && !isend(s[k])){j++;k++;}
    if(j>0)
      i++;
  }
  return i;
}



// the following routines can be used for getting arrays of numeric data too

int chemgets(char *s, char *v[], int max){
  // get a comma/space separated field from string s into array v. Includes allocating memory for each entry in v
  int i, j, k;
  i=0, k=0;
  while((i < max) && (s[k] != '\0') ){
    j=0;
    while((s[k] == ',') || isspace(s[k])){k++;} // skip commas and white space
    while((s[k] != ',') && !isspace(s[k]) && !isend(s[k])){j++;k++;}
    if(j>0)    
      v[i++] = strchop2(s, k-j, j);
  }
  if(i==max){
    fprintf(stderr, "ERROR in routine chemgets in file stringmanip.c: maximum number of terms got.\n");
  }
  return i;
}

int chemgets1(char *s, char *v1[], char *v2[], int max){
  // get a comma/space separated field into two arrays v1 and v2 alternately. Includes allocating memory for each entry in v1 and v2
  int whicharray = 0;
  int i, j, k;
  i=0, k=0;
  while((i < max) && (s[k] != '\0') ){
    j=0;
    while((s[k] == ',') || isspace(s[k])){k++;} // skip commas and white space
    while((s[k] != ',') && !isend(s[k]) && !isspace(s[k])){j++;k++;}
    if(j>0){
      v1[i] = strchop2(s, k-j, j);
      whicharray = 1;
    }
    j=0; // back to the start of tempword
    while((s[k] == ',') || isspace(s[k])){k++;} // skip commas and white space
    while((j < 99) && (s[k] != ',') && !isend(s[k]) && !isspace(s[k])){j++;k++;}
    if(j > 0){    
      v2[i++] = strchop2(s, k-j, j);
      whicharray = 0;
    }
  }
  if(whicharray == 1){
    fprintf(stderr, "ERROR in routine chemgets1 in file stringmanip.c: not an even number of items to get:\n%s\n", s);
  }
  if(i==max){
    fprintf(stderr, "ERROR in routine chemgets1 in file stringmanip.c: maximum number of terms got.\n");
  }

  return i;
}


char *getname(char s[], char t[])
     /* searches for the string s in the string t. When it is found, returns the next word */
     /* allocates memory for the word which can be freed with free() */

{
  int i=0, i1, i2;
  char *nm;
  char *p;
  if(!(p=strstr(t, s))){
    nm=strdup("");
    return nm;
  }
  else if ((strlen(p) == strlen(t)) || (isgood1(p-1, 0) == 1)){ // not part of another word
    p+=strlen(s);
    i=0;
    while(isspace(p[i]) && (p[i] != '\0')){i++;}// skip space
    i1=i;
    while(!(isspace(p[i])) && (p[i] != '\0')){i++;}
    i2=i;
  }
  else{
    nm=strdup("");
    return nm;
  }
  if(i2==i1){
    nm=strdup("");
    return nm;
  }
  nm=strdup12(p, i1, i2);
  return nm;
}

// returns the position in string t after getting the word s and the next word

int getstartind(char s[], char t[]){
  int i=0;
  char *p;
  if(!(p=strstr(t, s))){
    return strlen(t);
  }
  else{
    i=strlen(t) - strlen(p); 
    i+=strlen(s);
    while(isspace(t[i]) && (t[i] != '\0')){i++;}// skip space
    while(!(isspace(t[i])) && (t[i] != '\0')){i++;} // skip word
  }
  return i;
}

int getname1(char s[], char t[], char v[], int vsize)
     /* searches for the string s in the string t. When it is found, takes the next word and puts it in v.  Like getname, but it only gets the next valid word */

{
  int len, len1, i=0, k, ret;
  char *tbit;

  ret = 0;
  len = strlen(s);
  len1 = strlen(t);
  // if not found, return an empty string:
  v[0] = '\0';
  while ((len < len1-i) && (ret == 0)){
    if ((t[i] == s[0]) && (isgood1(t, i-1) == 1)){ // allowable character before this one
      tbit = strchop2(t, i, len);
      if (strcmp(tbit, s) == 0){ /* have we found our string? */
        i+=len;
	/* get the next word */
	while(isspace(t[i])){i++;} // skip white space
	k=0;
	while ((isinvar(t, i) != 0) && (k < vsize-1)){
	  v[k] = t[i];
          k++;i++;
	}        
	ret = 1;
	v[k] = '\0';
	if(k==vsize-1){
	  fprintf(stderr, "ERROR in routine getname1 in \"stringmanip.c\" - ran out of memory. Incomplete variable = %s\n", v);
	  free(tbit);
	  return -1;
	}
      } // no full match
      else
	i++;
      free(tbit);
    } // no first letter match
    else
      i++;
  }
  return ret;
}


int getrest(char s[], char t[], char v[], int vsize)
     /* searches for the string s in the string t. When it is found, takes the rest of the line (i.e. until we encounter a newline character or the end of the string) and puts it in v. Useful for databases*/

{
  int c, i=0;
  char *p;
  v[0] = '\0'; // initialise v
  if(!(p=strstr(t, s))){
    return 0;
  }
  else if ((strlen(p) == strlen(t)) || (isgood1(p-1, 0) == 1)){
    p+=strlen(s);
    while(isspace(*p) && (*p != '\0') && (*p != '\n')){p++;}// skip space
    for(i=0; i<vsize-1 && (c = *p++)!='\0' && !isend(c);++i){v[i] = c;}
    v[i] = '\0';
    if(i>=vsize-1){
      fprintf(stderr, "ERROR in routine getrest in \"stringmanip.c\" - ran out of memory. Incomplete variable %s = %s.\n", s, v);
      return -1;
    }
    return 1;
  }
  return 0;
}

int getrestfromfile(char *fname, char t[], char v[], int vsize)
     // get the rest of the line beginning with "t" in file fname and put in "v"
{
  FILE *fd;
  int len;
  char vardesc[1000];
  char *varname=NULL;

  v[0]=0;

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getrestfromfile - file: \"%s\" could not be opened for reading...\n", fname);
    return -1;
  }

  while((len = getline(fd, vardesc, 1000) > 0)){
    if(!iscomsep(vardesc)){
      varname = getnthblk(vardesc, 1);
      if(strcmp(t, varname) == 0){
	getrest(t, vardesc, v, vsize);
	fclose(fd);free(varname);
	return 0;
      }
      free(varname);
    }
  }

  fclose(fd);
  return 0;
}

//
// A recursive function to find all the dependencies of a set of variables in parray
// given a starting variable. The recursion is not dangerous because the function exits when 
// it hits a something which has already been set
//


void recfill(char *varname, int *v1, int tot, char **parray, char **deps, char *rem[], int numrems){
  char **v = NULL;
  int i, ind;
  int numv=0;
  if ((ind = isinarray(parray, tot, varname)) >= 0 && v1[ind]==0){
    v1[ind]=1;
    v=getvs(deps[ind], &numv, rem, numrems);
    for(i=0;i<numv;i++)
      recfill(v[i], v1, tot, parray, deps, rem, numrems);
    freearraydat(v, numv);
  }
  return;
}

//
// Given a single tempvar-type variable, get a list of all tempvars on which it depends directly or indirectly
// Calls the recursive function recfill


char **getvarlistfromfile(char *fname, char *varname, int *num, char *rem[], int numrems){
  char **parray = NULL;
  char **deps = NULL;
  int nump=0;
  char **v = NULL;
  int i,j;
  int *v1=NULL;
  nump = genarraysdat(fname, &parray, &deps);
  v1 = ivector(0, nump-1);
  for(i=0;i<nump;i++) // initialise
    v1[i]=0;

  recfill(varname, v1, nump, parray, deps, rem, numrems);

  j=0;
  for(i=0;i<nump;i++){
    if(v1[i]==1){j++;}
  }

  v=charar(0, j-1);
  *num=j;
  j=0;
  for(i=0;i<nump;i++){
    if(v1[i]==1){v[j]=strdup(parray[i]);j++;}
  }

  free_ivector(v1, 0, nump-1);
  freearraydat(parray, nump);
  freearraydat(deps, nump);
  return v;
}

//
// get dependencies of a single variable
//

char **getvardepsfromfile(char *fname, char *varname, int *num, char *rem[], int numrems){
  char **parray = NULL;
  char **deps = NULL;
  int nump=0;
  int ind;
  char **v = NULL;
  int numv=0;
  nump = genarraysdat(fname, &parray, &deps);

  if ((ind = isinarray(parray, nump, varname)) >= 0)
    v=getvs(deps[ind], &numv, rem, numrems);

  freearraydat(parray, nump);
  freearraydat(deps, nump);
  *num=numv;
  return v;
}

// memory allocating version of getrest

char *getrst(char s[], char t[])
{
  int c, i=0;
  char *p;
  char *v;
  int len = strlen(s);

  if(!(p=strstr(t, s)))
    v = strdup("");
  else if ((strlen(p) == strlen(t)) || (isgood1(p-1, 0) == 1)){
    p+=len;
    while(isspace(*p) && (*p != '\0') && (*p != '\n')){p++;}// skip space
    for(i=0; (c = p[i])!='\0' && !isend(c);++i){}
    v = strchop2(p, 0, i);
  }
  else
    v = strdup("");

  return v;
}

char *getrstfromfile(char *fname, char t[])
     // get the rest of the line beginning with "t" in file fname
{
  FILE *fd;
  int len;
  char line[2000];
  char *vardesc=NULL;
  char *varname=NULL;


  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getrstfromfile - file: \"%s\" could not be opened for reading...\n", fname);
    vardesc=strdup("");
    return vardesc;
  }

  while((len = getline(fd, line, 2000) > 0)){
    if(!iscomsep(line)){
      varname = getnthblk(line, 1);
      if(strcmp(t, varname) == 0){
	vardesc = getrst(t, line);
	fclose(fd);free(varname);
	return vardesc;
      }
      free(varname);
    }
  }
  vardesc=strdup("");
  fclose(fd);
  return vardesc;
}


int cpfile(char *outfile, char *infile)
{
  FILE *fd, *fd1;
  int c=0, i;
  fd = fopen(infile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in cpfile - file: \"%s\" could not be opened for reading...\n", infile);
    return -1;
  }
  fd1 = fopen(outfile, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in cpfile - file: \"%s\" could not be opened for writing...\n", outfile);
    fclose(fd);
    return -1;
  }

  for(i=0; (c=getc(fd))!=EOF;++i)
    fputc(c, fd1);

  fclose(fd);
  fclose(fd1);
  return i;
}

// append infile to the end of outfile

int catfile(char *outfile, char *infile)
{
  FILE *fd, *fd1;
  int c=0, i;
  fd = fopen(infile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in catfile - file: \"%s\" could not be opened for reading...\n", infile);
    return -1;
  }
  fd1 = fopen(outfile, "a");
  if(!fd1){
    fprintf(stderr, "ERROR in catfile - file: \"%s\" could not be opened for appending...\n", outfile);
    fclose(fd);return -1;
  }
  for(i=0; (c=getc(fd))!=EOF;++i)
    fputc(c, fd1);

  fclose(fd);
  fclose(fd1);
  return i;
}

int getline(FILE *fp, char s[], int lim)
{
  /* store a line as a string, including the terminal newline character */
  int c=0, i;

  for(i=0; i<lim-1 && (c=getc(fp))!=EOF && c!='\n';++i){s[i] = c;}
  if (c == '\n') {s[i++] = c;}
  s[i] = '\0';
  if(i==lim-1 && c!='\n' && c!=0)
    fprintf(stderr, "WARNING in getline: line length exceeded. *%s*\n", s);
  return i;
}


int getlinefromblock(int *pos, char *block, char s[], int lim)
{
  int c=0, i=0;
  if(*pos > (int)strlen(block))
    return 0;
  for(i=0; i<lim-2 && (c=block[(*pos)++])!='\0' && c!='\n';++i)
    s[i] = c;
  if (c == '\n')
    s[i++] = c;
  s[i] = '\0';
  if(i==lim-1 && c!='\n' && c!=0)
    fprintf(stderr, "WARNING in getlinefromblock: line length exceeded.\n");
  return i;
}

int getline0(FILE *fp, char s[], int lim)
{
  /* same as getline except count the terminal n without including it */
  int c=0, i;

  for(i=0; i<lim-1 && (c=getc(fp))!=EOF && c!=13 && c!='\n';++i)
    s[i] = c;
  s[i] = '\0';
  if(i==lim-1 && c!='\n' && c!=EOF)
    fprintf(stderr, "WARNING in getline0: line length exceeded. %s\n", s);
  if (c == '\n' || c == 13) { // don't do anything
    return i+1;
  }
  else
    return i;
  // we return the length it would have if it had a newline character at the end
}

int getblock(FILE *fp, char s[], char end[], int lim)
{
  /* get a block of characters into s line by line. Stop when you encounter a line with the string "end" in it, or an empty line i.e. the end of the file */

  int found=0, j=0, len1; // j = global counter
  int lim_line = 1000;
  char oneline[1000];
  if(lim>1)
    s[0] = '\0'; // initialise s
  else
    return -1;

  while((found == 0) && ((len1 = getline(fp, oneline, lim_line)) > 0)){
    j+=len1; // global counter
    if(j >= lim-1){
      fprintf(stderr, "ERROR in routine \"getblock\" in file \"stringmanip.c\" - no more space in block.\n");
      fprintf(stderr, "\t block so far = %s\n", s);
      return j-len1;
    }
    if(strstr(oneline, end))
      found = 1;
    strcat(s, oneline); // safe
  }
  return j;
}

int getblockmin(FILE *fp, char s[], char end[], char errorvar[], int lim)
{
  /* get a block of characters into s line by line. Stop when you encounter a line with the string "end" in it, or an empty line. Do not include the last line */

  int found=0, j=0, len1, linenum=0; // j = global counter
  int lim_line = 1000;
  char oneline[1000];
  if(lim>1)
    s[0] = '\0'; // initialise s
  else
    return -1;

  while((found == 0) && ((len1 = getline(fp, oneline, lim_line)) > 0)){
    linenum++;
    j+=len1; // global counter
    if(j >= lim-1){
      fprintf(stderr, "ERROR in routine \"getblockmin\" in file \"stringmanip.c\" - no more space in block.\n");
      fprintf(stderr, "\t block so far = %s\n", s);
      return j-len1;
    }
    if(strstr(oneline, end))
      found = 1;
    else if (strstr(oneline, errorvar)){
      fprintf(stderr, "ERROR in routine \"getblockmin\" in file \"stringmanip.c\" - couldn't find %s. Line number %d.\n", end, linenum);
      return -1;
    }
    else
      strcat(s, oneline); // safe
  }
  return (int) strlen(s); // block length
}

int getcleanblock(FILE *fp, char s[], char end[], int lim)
{
  /* get a block of characters into s line by line ignoring comment lines. Stop when you encounter a line with the string "end" in it */

  int found=0, j=0, len1; // j = global counter
  int lim_line = 1000;
  char oneline[1000];
  if(lim>1)
    s[0] = '\0'; // initialise s
  else
    return -1;

  while((found == 0) && ((len1 = getline(fp, oneline, lim_line)) > 0)){
    j+=len1; // global counter
    if(j >= lim-1){
      fprintf(stderr, "ERROR in routine \"getcleanblock\" in file \"stringmanip.c\" - no more space in block.\n");
      fprintf(stderr, "\t block so far = %s\n", s);
      return j-len1;
    }
  
    if(strstr(oneline, end))
      found = 1;

    if(!iscom(oneline))
      strcat(s, oneline); // safe
  }
  return j;
}


int ins(int k, char t[], char s[], int lim)
     /* insert string t at position k into s - without
	the terminal character. Return the pointer to the 
	next free position. Use with care, because s is not terminated*/
{
  int i, len;
  i=0;
  len=strlen(t);
  if((k+len) >= lim-1){
    fprintf(stderr, "ERROR in routine \"ins\" in file \"stringmanip.c\": limit exceeded.\n");
  }  
  else{
    while(i<len){s[k++] = t[i++];}
  }
  return k;
}

int inspow(int k, char t[], char t1[], char s[], int lim)
     /* construct the string "pow(t, t1)" */
{
  int z;
  z = ins(k, "pow(", s, lim);
  z = ins(z, t, s, lim);
  z = ins(z, ", ", s, lim);
  z = ins(z, t1, s, lim);
  z = ins(z, ")", s, lim);
  return z;
}

void reverse(char s[])
{
  /* from the c book - reverse a string */
  int c, i, j;

  for (i=0, j=strlen(s)-1;i<j;i++, j--){
    c=s[i];
    s[i] = s[j];
    s[j] = c;
  }
}



void itoa(char s[], int n)
{
  /* from the c book - get a string from an integer */
  int i, sign;

  if ((sign = n) < 0) /* record sign */
    n=-n; /* make n positive */
  i=0;
  do { /* generate digits in reverse order */
    s[i++] = n % 10 + '0'; /* get next digit */
  } while ((n /=10) > 0); /* delete it */
  if (sign < 0)
    s[i++] = '-';
  s[i] = '\0';
  reverse(s);
}

// strchop2 is a better version of this

int getstring(char *s, char *t, int len)
     /* gets a string of length len or less if the 
	string being read terminates */
{
  int i=0;
  while(((s[i] = t[i]) != '\0') && (i<len))
    i++;
  if(s[i-1] == '\0') /* string ended */
    i--;
  else 
    s[i] = '\0'; /* add terminal character */
  return i;
}

int cleanblock(char *s, char *t, int len)
{
  // clean string t and put it in s (get rid of comment lines)
  int i=0, m=0, linelen;
  int maxlen=500;
  char temp[500];
  if(len>0)
    s[0]=0;
  else{
    fprintf(stderr, "ERROR in cleanblock in stringmanip.c: block length not long enough.\n");
    return -1;
  }
  temp[0]=0;
  while((linelen = getlinefromblock(&i, t, temp, maxlen)) > 0){
    if(!(iscom(temp))){
      m+=linelen;
      if(m < len-1)
	strcat(s, temp); // safe
      else
	fprintf(stderr, "ERROR in cleanblock in stringmanip.c: block length not long enough.\n");
    }
  }
  return m;

}

int isgood(char *s, int j)
     /* characters which can come after a variable name */
{
  int c; /* to store EOF */
  c = s[j];
  if (isspace(c) || (c == '+') || (c == '-') || (c == '*') || (c == '/') || (c == '=') || (c == '%') || (c == '!') || (c == '~') || (c == '^') || (c == '\0'))
    return 1;
  else if ((c == '(') || (c == ')') || (c == '[') || (c == ']') || (c == '<') || (c == '>') || (c == '&'))
    return 1;
  else if ((c == '|') || (c == '?') || (c == '.') || (c == ',') || (c == ';') || (c == ':') || (c == EOF))
    return 1;
  else
    return 0;
}



int isgood1(char *s, int j)
     /* characters which can come before a variable name. */
{
  int c;
  if (j>=0) 
    c=s[j]; 
  else 
    c = ' ';
  if (isspace(c) || (c == '+') || (c == '-') || (c == '*') || (c == '/') || (c == '=') || (c == '%') || (c == '!') || (c == '~') || (c == '^'))
    return 1;
  else if ((c == '(') || (c == ')') || (c == '[') || (c == ']') || (c == '<') || (c == '>') || (c == '&'))
    return 1;
  else if ((c == '|') || (c == '?') || (c == '.') || (c == ',') || (c == ';') || (c == ':'))
    return 1;
  else
    return 0;
}


int addifpos(char *tempoutst, int j, int maxlen, char *tempvar)
{
  // add string tempvar to string tempoutst of maximum length j, at position j if possible (do not close string tempoutst!)
  int m;
  if(j + (int)(strlen(tempvar)) < maxlen-1){
    for(m=0;m<(int)(strlen(tempvar));m++)
      tempoutst[j++] = tempvar[m];
    return j;
  }
  else 
    return -1;
}


char **transarray(char *s[], int max, char addon[]){
  // add on the string addon to each member of s and output in t (which must be the same size as s) Memory for each string is allocated in the routine. 
  int i;
  char **t;
  t=charar(0, max-1);
  for(i=0;i<max;i++){
    t[i] = (char*) malloc(sizeof(char) * ((int)strlen(s[i])+(int)strlen(addon) + 1));
    sprintf(t[i], "%s%s", s[i], addon); // safe
  }
  return t;
}    



int addvar1(int k, char *s, char *t[], int tmax)
     /* routine to check if a string s already belongs to a list of strings, and if not to add it to the end. Returns new free position in the list. tmax is the size of t. Memory for the entry in t is allocated using strdup if necessary */
{
  int i=0;

  if(k == tmax){
    fprintf(stderr, "k = %d, tmax = %d\n", k, tmax);
    fprintf(stderr, "ERROR in routine addvar1 in \"common/stringmanip.c\": maximum length of array reached. Nothing more can be added.\n");
    fprintf(stderr, "\t\tstring = %s\n", s);
    return tmax;
  }
  if(k > tmax){
    fprintf(stderr, "ERROR in routine addvar1 in \"common/stringmanip.c\": maximum length of array exceeded. Signals a previous error\n");
    return tmax;
  }
  while(i<k)
    if (strcmp(s, t[i++]) == 0){return k;}

  t[k] = strdup(s);
  return k+1;
}

//
// set the parameter values from a file in the standard format. 
//

int setparamsfromfile(char *readfile, char *settable[], int numsettable, double *setpvals, double *rpar, void pfunc(double*, double *)){
  int i;
  char *varval;

  for(i=0;i<numsettable;i++){
    varval = getcvalfromfile(readfile, settable[i]);
    if(!isonlyspace(varval))
      setpvals[i] = atof(varval);
    else{
      fprintf(stderr, "ERROR in setparamsfromfile: set parameter %s not given a value in \"%s\"- aborting the parameter setting.\n", settable[i], readfile);
      free(varval);
      return -1;
    }
    free(varval);
  }

  pfunc(setpvals, rpar);

  return 0;
}

//
// calculate temporary variable values from a file in the standard format. 
// setvals must already have been set
//

int settvvals(char *darray[], double *dvals, int numdfilevars, char *vararray[], int numvars, double *setpvals, double *rpar, void pfunc(double*, double *), double *tvars, void tvfunc(double *, double *, double *, double *)){
  double vvals[numvars];
  double timeval;
  int l;

  if(!issubarray(vararray, numvars, darray, numdfilevars)){
    fprintf(stderr, "ERROR in settvals: The data-file must contain values of all the variables.\n");
    return -1;
  }
  if((l=isinarray(darray, numdfilevars, "time"))<0)
    timeval=0.0;
  else
    timeval=dvals[l];

  // set rpar
  pfunc(setpvals, rpar);
  // set vvals
  gentrans(vararray, numvars, vvals, darray, numdfilevars, dvals);
  // calculate tvars
  tvfunc(&timeval, tvars, vvals, rpar);
  return 0;
}

// augment output datafile with the value of the temporary variables

int augmentdatfile(char *datfile, char *newdatfile, char *setfile, char *varfile, char *paramfile, void pfunc(double*, double *), void tvfunc(double *, double *, double *, double *)){
  char *pfile;
  char *tvfile;
  int i, l;
  char *varval;
  double *datfromline;
  char **settable;
  int numsettable=0;
  int *changeset=NULL;
  char **vararray=NULL;
  char **paramarray=NULL;
  int numvars=0;
  int numparams=0;
  char **darray;
  int numdfilevars=0;
  double *setpvals;
  FILE *fd, *fd1;
  char *line;
  int maxl;
  int numdatfromline;
  double *rpar;
  double *tvvals;
  char **tvnames=NULL;
  int numtvnames=0;
  int linelen;

  if(!fileexists(datfile)){
    fprintf(stderr, "No file such as %s. Exiting.\n", datfile);
    return -1;
  }


  // get the tempvar file and array of temporary variables associated with the simulation
  tvfile = (char*) malloc(sizeof(char) * (strlen(datfile) + 7));
  sprintf(tvfile, "%s.tvars", datfile); //safe
  tvnames = genarraydat(tvfile, &numtvnames);
  free(tvfile);
  if(numtvnames<=0){ // do nothing
    freearraydat(tvnames, numtvnames);
    return 0;
  }
  tvvals = dvector(0, numtvnames-1);

  // get the parameter value file and parameter array associated with the simulation
  pfile = (char*) malloc(sizeof(char) * (strlen(datfile) + 7));
  sprintf(pfile, "%s.pvals", datfile); //safe

  paramarray = genarraydat(paramfile, &numparams);
  rpar = dvector(0, numparams-1);

  settable = genarraydat(setfile, &numsettable);
  vararray = genarraydat(varfile, &numvars);
  setpvals = dvector(0, numsettable-1);

  // the vectors which are reset in during the simulation
  changeset = ivector(0, numsettable-1);
  for(i=0;i<numsettable;i++)
    changeset[i]=-1;

  for(i=0;i<numsettable;i++){
    varval = getcvalfromfile(pfile, settable[i]);
    if(!isonlyspace(varval))
      setpvals[i] = atof(varval);
    else{
      fprintf(stderr, "ERROR in augmentdatfile: set parameter %s not given a value in \"%s\"- aborting the parameter setting.\n", settable[i], pfile);
      free(varval);free(pfile);
      free_dvector(setpvals, 0, numsettable-1);
      free_ivector(changeset, 0, numsettable-1);
      free_dvector(rpar, 0, numparams-1);
      free_dvector(tvvals, 0, numtvnames-1);
      freearraydat(settable, numsettable);
      freearraydat(vararray, numvars);
      freearraydat(paramarray, numparams);
      freearraydat(tvnames, numtvnames);
      return -1;
    }
    free(varval);
  }
  darray = getdfilevars(datfile, &numdfilevars);

  for(i=0;i<numsettable;i++){
    if((l=isinarray(darray, numdfilevars, settable[i]))>=0)
      changeset[i]=l;
  }

  numflines(datfile, &maxl); // array size and maximum line length
  line = (char*) malloc(sizeof(char) * (maxl));


  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "Error in augmentdatfile: file %s couldn't be opened\n", datfile);
    free(pfile);
    free_dvector(setpvals, 0, numsettable-1);
    free_ivector(changeset, 0, numsettable-1);
    free_dvector(rpar, 0, numparams-1);
    free_dvector(tvvals, 0, numtvnames-1);
    freearraydat(settable, numsettable);
    freearraydat(vararray, numvars);
    freearraydat(paramarray, numparams);
    freearraydat(tvnames, numtvnames);
    freearraydat(darray, numdfilevars);
    return -1;
  }
  fd1 = fopen(newdatfile, "w");
  if(!fd1){
    fprintf(stderr, "Error in augmentdatfile: file %s couldn't be opened for writing.\n", newdatfile);
    free(pfile);
    free_dvector(setpvals, 0, numsettable-1);
    free_ivector(changeset, 0, numsettable-1);
    free_dvector(rpar, 0, numparams-1);
    free_dvector(tvvals, 0, numtvnames-1);
    freearraydat(settable, numsettable);
    freearraydat(vararray, numvars);
    freearraydat(paramarray, numparams);
    freearraydat(tvnames, numtvnames);
    freearraydat(darray, numdfilevars);
    return -1;
  }
  fprintf(fd1, "#");
  for(i=0;i<numdfilevars;i++)
    fprintf(fd1, "%s\t", darray[i]);
  for(i=0;i<numtvnames-1;i++)
    fprintf(fd1, "%s\t", tvnames[i]);
  fprintf(fd1, "%s\n", tvnames[i]);

  while((linelen = getline(fd, line, maxl)) > 0){
    if(!iscomsepdat(line)){
      datfromline = genfloatvec(line, &numdatfromline);
      if(numdatfromline!=numdfilevars){
	fprintf(stderr, "Error in augmentdatfile: there is a bad line in file %s.\n", datfile);
	fclose(fd);fclose(fd1);
	free_dvector(datfromline, 0, numdatfromline-1);
	free(pfile);
	free_dvector(setpvals, 0, numsettable-1);
	free_ivector(changeset, 0, numsettable-1);
	free_dvector(rpar, 0, numparams-1);
	free_dvector(tvvals, 0, numtvnames-1);
	freearraydat(settable, numsettable);
	freearraydat(vararray, numvars);
	freearraydat(paramarray, numparams);
	freearraydat(tvnames, numtvnames);
	freearraydat(darray, numdfilevars);
	return -1;
      }
      // reset parameters from data file if necessary
      for(i=0;i<numsettable;i++){
	if(changeset[i]>=0)
	  setpvals[i]=datfromline[changeset[i]];
      }
      //      pfunc(setpvals, rpar);

      settvvals(darray, datfromline, numdfilevars, vararray, numvars, setpvals, rpar, pfunc, tvvals, tvfunc);
      for(i=0;i<numdatfromline;i++)
	fprintf(fd1, "%.4e\t", datfromline[i]);
      for(i=0;i<numtvnames-1;i++)
	fprintf(fd1, "%.4e\t", tvvals[i]);
      fprintf(fd1, "%.4e\n", tvvals[i]);

      free_dvector(datfromline, 0, numdatfromline-1);
    }

  }


  fclose(fd);fclose(fd1);

  free(pfile);
  freearraydat(darray, numdfilevars);
  free_dvector(setpvals, 0, numsettable-1);
  free_ivector(changeset, 0, numsettable-1);
  free_dvector(rpar, 0, numparams-1);
  free_dvector(tvvals, 0, numtvnames-1);
  freearraydat(settable, numsettable);
  freearraydat(vararray, numvars);
  freearraydat(paramarray, numparams);
  freearraydat(tvnames, numtvnames);
  return 0;
}


//
// This routine takes a parameter datafile and a paramlist 
// and overrides all the values in the paramlist with the values 
// from the file. Do *not* subsequently use update
//


int overrideparamsfromfile(char *readfile, char *paramarray[], int numparams, double *rpar){ // use with great care
  int linelen, linenum = 0, ind, i, j, k;
  char *varname; // to hold variable name
  char *varval;
  char line[2000];
  int valsset[numparams]; // the list of indices to be updated
  double vals[numparams];
  FILE *fd;

  // initialise

  for(i=0;i<numparams;i++)
    valsset[i] = 0;

  fd = fopen(readfile, "r");
  if(!fd){
    fprintf(stderr, "Error in overrideparamsfromfile in \"src/params/funcs.cc\": file %s couldn't be opened - perhaps you are in the wrong directory?\n", readfile);
    return -1;
  }

  while((linelen = getline(fd, line, 2000)) > 0){
    linenum++;
    i=j=k=0;
    if(!iscomsep(line)){
      varname = getnthwd(line, 1);
      varval = getnthtkn(line, 2);
      if((ind = isinarray(paramarray, numparams, varname)) >= 0){ // parameter exists
	vals[ind] = atof(varval);
	valsset[ind] = 1;
      }
      else
	fprintf(stderr, "Error in overrideparamsfromfile in funcs.cc: line %d: %s not a valid parameter\n", linenum, varname);
      free(varname);
      free(varval);
    }
  }

  fclose(fd);

  for(i=0;i<numparams;i++){
    if(valsset[i] == 1){ // parameter to be overridden
      if(fabs(vals[i] - rpar[i]) > 0.0000001)
	fprintf(stderr, "value of %s being changed from %.6e to %.6e\n", paramarray[i], rpar[i], vals[i]);
      rpar[i]=vals[i];
    }
  }
  return 0;
}


double getparamvalue(char *readfile, char *vname)
{// the idea of this routine is to read a normal space separated file containing parameter values, and return the value of a particular parameter as a double. 
  int MAXLINELEN = 1500;
  int found = 0;
  int linelen, i, k;
  char varname[40]; // to hold variable name
  char varval[40]; // to hold variable value
  double vval = -999.999; // the actual value
  char line[1500];
  FILE *fd;

  fd = fopen(readfile, "r");
  if(!fd){
    fprintf(stderr, "Error in getparamvalue: file %s couldn't be opened - perhaps you are in the wrong directory?\n", readfile);
    return vval;
  }

  while((linelen = getline(fd, line, MAXLINELEN)) > 0){
    i=k=0;
    if(!(iscomsep(line))){
      i = getnthword(line, varname, 40, 1);
      if(strcmp(varname, vname)==0){
	found = 1;
        while(isspace((int) line[i])){i++;} // skip space
        // get next word
        while (k < 39 && !(isspace((int) line[i]))){
	  varval[k++] = line[i++];
        }
        varval[k] = '\0';
	break;    
      }
    }
  }
  fclose(fd);

  if(found == 0){
    fprintf(stderr, "Error in getparamvalue: %s can't be found in file %s... returning a value of -999.999\n", vname, readfile);
    return vval;
  }
  else{
    vval = atof(varval);
  }
  fprintf(stderr, "found vval = %.4f\n", vval);
  return vval;
}




int getparamvaluestr(char *readfile, char *vname, char *vval, int lim)
{// the idea of this routine is to read a normal space separated file containing parameter values, and return the value of a particular parameter as a string. 
  // note that this function returns 0 on FAILURE
  int MAXLINELEN = 1500;
  int found = 0;
  int linelen, i, k;
  char *varname; // to hold variable name
  char *varval=NULL;
  char line[1500];
  FILE *fd;

  if(lim>0)
    vval[0]=0;
  if(lim<10){
    fprintf(stderr, "Error in getparamvaluestr: the string to hold the parameter value must be at least 10 characters long.\n");
    return -1;
  }
  fd = fopen(readfile, "r");
  if(!fd){
    fprintf(stderr, "Error in getparamvaluestr: file %s couldn't be opened - perhaps you are in the wrong directory?\n", readfile);
    strcpy(vval, "-999.999"); // safe
    return -1;
  }

  while((linelen = getline(fd, line, MAXLINELEN)) > 0 && (found == 0)){
    i=k=0;
    if(!(iscomsep(line))){
      varname = getnthwd(line, 1);
      if(strcmp(varname, vname)==0){
	varval = getnthtkn(line, 2);
	found = 1;
      }
      free(varname);
    }
  }
  fclose(fd);

  if(varval==NULL)
    varval = strdup("");

  if(found == 0)
    strcpy(vval, "-999.999"); // safe
  else if((int)strlen(varval) > lim-1){
    fprintf(stderr, "Error in getparamvaluestr: the parameter value %s in file %s is too long.\n", varval, readfile);
    strcpy(vval, "-999.999"); // safe
  }
  else
    strcpy(vval, varval); // safe

  free(varval);
  return found;
}

void transalt2(char t[], int maxlen, char s[], char *readfile)
     // here the idea is that you get tokens one at a time, check if they are given a value in a particular datafile, and if so translate. 
{
  int i, j, k, varfound;
  char *tempvar;
  char varval[40];

  if((int)(strlen(s)) > maxlen){
    fprintf(stderr, "ERROR in routine transalt2 in file stringmanip.c: string longer than maxlen.\n");
    return;
  }

  i=0; // counter in s
  j=0; //counter in t
  k=0; // counter in tempvar
  while ((s[i] != '\0') && (j < maxlen)){
    while ((isvarstart(s, i) == 0) && (s[i] != '\0') && (j<maxlen)){t[j++]=s[i++];} // not start of variable
    k=0;
    while ((isinvar(s, i) == 1) && (s[i] != '\0')){k++;i++;} // is a variable
    tempvar = strchop2(s, i-k, k);

    if((varfound = getparamvaluestr(readfile, tempvar, varval, 40)) == 1) // found in file
      j = addifpos(t, j, maxlen, varval);
    else
      j = addifpos(t, j, maxlen, tempvar);

    if(j < 0){
      fprintf(stderr, "ERROR in routine transalt2 - no more space in string - now exiting.\n");
      t[0] = '\0'; // empty string
      free(tempvar);
      return;
    }
    free(tempvar);
  }
  t[j++] = '\0';
  return; 
}

//
//
// strchop2 is better
//

int getstrg(char *s, char *t, int len)
     /* gets a string of length len or less if the 
	string being read terminates */
{
  int i;
  i=0;
  while(((s[i] = t[i]) != '\0') && (i<len))
    i++;
  if(s[i-1] == '\0') /* string ended */
    i--;
  else 
    s[i] = '\0'; /* add terminal character */
  return i;
}


//
// The fully dynamic routine to generate an array from a list in a data file
//

char **genarraydat(char *datfile, int *num){
  // generate an array from a data file. Memory for each entry in the datafile is allocated. 
  int linelen, i=0;
  char *line;
  char **vararray;
  int numl=0;
  int maxl=0;
  FILE *fd;

  *num=0; // initialise
  numl = numflines(datfile, &maxl); // array size and maximum line length
  if(numl <=0){return NULL;}

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genarraydat - File: %s could not be opened...\n", datfile);
    return NULL;
  }
  vararray=charar(0, numl-1);
  line = (char*) malloc(sizeof(char) * (maxl));

  while((linelen = getline(fd, line, maxl)) > 0){
    if(!(iscomsep(line)))
      vararray[i++] = getnthtkn(line, 1);
  }
  fclose(fd);
  (*num)=i;
  free(line);
  return vararray;
}


char **genarrayndat(char *datfile, int *num, int n){
  // generate an array from a data file. Memory for each entry in the datafile is allocated. 
  int linelen, i=0;
  char *line;
  char **vararray;
  int numl=0;
  int maxl=0;
  FILE *fd;

  *num=0; // initialise
  numl = numflines(datfile, &maxl); // array size and maximum line length
  if(numl <=0){return NULL;}

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genarraydat - File: %s could not be opened...\n", datfile);
    return NULL;
  }
  vararray=charar(0, numl-1);
  line = (char*) malloc(sizeof(char) * (maxl));

  while((linelen = getline(fd, line, maxl)) > 0){
    if(!(iscomsep(line)))
      vararray[i++] = getnthtkn(line, n);
  }
  fclose(fd);
  (*num)=i;
  free(line);
  return vararray;
}

char **genlinearraydat(char *datfile, int *num){
  // generate an array from a data file. Each entry is a line. Memory is allocated and can be freed with freearraydat
  int linelen, i=0;
  char *line;
  char **vararray;
  int numl=0;
  int maxl=0;
  FILE *fd;

  *num=0; // initialise
  numl = numflines(datfile, &maxl); // array size and maximum line length
  if(numl<=0){
    return NULL;
  }

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genlinearraydat - File: %s could not be opened...\n", datfile);
    return NULL;
  }
  vararray=charar(0, numl-1);
  line = (char*) malloc(sizeof(char) * (maxl));

  while((linelen = getline(fd, line, maxl)) > 0){
    if(!(iscomsep(line)))
      vararray[i++] = lrtrim(line);
  }
  fclose(fd);
  (*num)=i;
  free(line);
  return vararray;
}


int freearraydat(char **array, int lim){
  int i=lim;
  if(!array){return 0;}
  while(i-- > 0)
    free(array[i]);
  free_charar(array, 0, lim-1);
  return 0;
}

//
// Get a cleaned process into a string. Allocates memory
// Fails silently. Returns an empty string on failure.
// Memory should be freed with free
//

char *getproc(char *fname, char *procname){
  char *procstr;
  int len;
  FILE *fd;
  int flag;
  char *oneline;
  char block[10000];

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getproc - File: %s could not be opened...\n", fname);
    exit(1);
  }
  flag = 0;
  while((len = getcleanblock(fd, block, "******", 10000)) > 0){
    if(iscomblock(block)){} // comment block - ignore
    else{
      oneline = getname("name:", block);
      if(strcmp(oneline, procname) == 0){
	procstr=strdup(block);
	fclose(fd);
	free(oneline);
	return procstr; 
      }
      free(oneline);
    }
  }
  fclose(fd);
  procstr=strdup("");
  return procstr;

}

// delete variable var from an array. It is assumed that
// ordering isn't important, so the last element of the array
// is put into the freed up space. Failure is quiet if var isn't found.

int delvar(char *array[], int lim, char *var){
  int i;
  for(i=0;i<lim;i++){
    if(strcmp(array[i], var) == 0){
      free(array[i]);
      array[i] = strdup(array[lim-1]);
      free(array[lim-1]);
      return lim-1;
    }
  }
  // not found
  return lim;
}

// delete all members of the array "remarray" from the array "array"

int delarray(char *array[], int lim, char *remarray[], int lim1){
  int i;
  for(i=0;i<lim1;i++)
    lim=delvar(array, lim, remarray[i]);
  return lim;
}

int free_datarray(char *array[], int lim){
  int i=lim;
  if(!array){return 0;}
  while(i-- > 0)
    free(array[i]);
  return 0;
}

// generate a vector of floats from a line

double *genfloatvec(char *line, int *num){
  int j;
  char *wd;
  double *dvec=NULL;
  int numt=0;
  numt=getnumtokens(line);
  dvec = dvector(0, numt-1);
  for(j=1;j<numt+1;j++){
    wd = getnthtkn(line, j);
    if(!isnumber(wd))
      dvec[j-1]=0.0;
    else
      dvec[j-1]=atof(wd);
    free(wd);
  }
  *num=numt;
  return dvec;
}

// fills a 2D array with values

int genfloatmfromdat(char *datfile, int maxl, double **array, int lim, int lim2){
  // generate two arrays from a data file
  // memory for the entries in the arrays is allocated
  int linelen, i, j, k=0;
  char line[maxl];
  char *wd;
  FILE *fd;

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genfloatmfromdat in file \"stringmanip.c\" - File: %s could not be opened...\n", datfile);
    return -1;
  }
  i=0;

  while((linelen = getline(fd, line, maxl)) > 0){
    if(!iscomsepdat(line)){
      i++;
      if(i> lim+1)
	fprintf(stderr, "ERROR in genfloatmfromdat in file \"stringmanip.c\" - array is full...\n");
      else{
	for(j=1;j<lim2+1;j++){
	  wd = getnthtkn(line, j);
	  if(!isnumber(wd))
	    array[i-1][j-1]=0.0;
	  else
	    array[i-1][j-1]=atof(wd);
	  free(wd);
	}
      }
    }
  }
  fclose(fd);
  return k;
}

//
// creates a 2D array from doubles from a datafile. memory is allocated
// and should be freed with if(darray){free_dmatrix(darray, 0, s1-1, 0, s2-1);}
// free(datnames);
//

char *genfloatmdat(char *datfile, double ***darray, int *s1, int *s2){
  int i=0, nl=0, linelen;
  int maxl=0;
  int flag=0;
  char *varn=NULL;
  FILE *fd;
  char *line;
  char *datnames=NULL;
  char *tmp;
  nl = numflinesdat(datfile, &maxl);
  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in  genfloatmdat in file \"stringmanip.c\" - File: %s could not be opened...\n", datfile);
    datnames=strdup("");
    return datnames;
  }
  line = (char*) malloc(sizeof(char) * (maxl+1));

  while((linelen = getline(fd, line, maxl)) > 0 && flag==0){
    tmp=lrtrim(line);
    if(tmp[0]=='#')
      datnames=strdup(tmp+1);
    free(tmp);

    if(!iscomsepdat(line)){
      flag=1;
      varn=strdup("__");i=1;
      while(!isonlyspace(varn)){
	free(varn);
	varn=getnthtkn(line, i);
	if(!isonlyspace(varn)){i++;}
      }
    }
  }
  i--;
  free(line);
  fclose(fd);
  (*s1)=nl;(*s2)=i;

  if(nl==0 || i==0)
    (*darray) = NULL;
  else{
    (*darray) = dmatrix(0, nl-1, 0, i-1);
    genfloatmfromdat(datfile, maxl, *darray, nl, i);
  }
  if(!datnames){datnames=strdup("");}
  return datnames;
}




//
// creates two arrays from a datafile. memory is allocated
//

int genarraysdat(char *datfile, char ***array1, char ***array2){
  int nl=0;
  int maxl=0;
  nl = numflines(datfile, &maxl);
  if(nl==0){(*array1)=NULL;(*array2)=NULL;return 0;}
  (*array1) = charar(0, nl-1);
  (*array2) = charar(0, nl-1);
  genarraysfromdat(datfile, maxl, *array1, *array2, nl);
  return nl;
}


int genarraysfromdat(char *datfile, int maxl, char *array1[], char *array2[], int lim){
  // generate two arrays from a data file
  // memory for the entries in the arrays is allocated
  int linelen, i, j, k=0;
  char line[maxl];
  char *varname;
  char *expression;
  char *wd;
  FILE *fd;

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genarraysfromdat in file \"stringmanip.c\" - File: %s could not be opened...\n", datfile);
    return -1;
  }

  while((linelen = getline(fd, line, maxl)) > 0){
    i=0;j=0;
    if(!iscomsep(line)){
      varname = getnthblk(line, 1);
      expression = getrst(varname, line);
      wd = getnthwd(line, 1);
      if(k < lim){
	array1[k] = strdup(wd);
	array2[k] = strdup(expression);
	k++;
      }
      else{
	fprintf(stderr, "ERROR in genarraysfromdat in \"stringmanip.c\": arrays are full:\n\tdatfile = %s\tarray size = %d\n", datfile, lim);
      }
      free(wd);
      free(expression);
      free(varname);
    }
  }
  fclose(fd);
  return k;
}

// enclose string s

char *enclose(char *s, char *lstr, char *rstr)
     /* put an encloser around a string */
{
  char *t;
  t = (char*) malloc(sizeof(char) * ((int)strlen(s) + (int)strlen(lstr) + (int)strlen(rstr) + 1));
  sprintf(t, "%s%s%s", lstr, s, rstr); // safe
  return t;
}

int bcheck(char *t)
{
  // checks if s needs to be bracketed - 0 if it needs to be bracketed
  int i=0;
  char c;
  int bcount=0;
  char *s;
  s=lrtrim(t);
  if(s[0] != '('){ // no initial bracket
    free(s);
    return 0;
  }
  while((c = s[i++]) != '\0'){
    if (c == '(')
      bcount++;
    else if (c == ')'){
      bcount--;
      if(bcount==0 && i!=(int)(strlen(s))){ // hits zero before end of string
	free(s);
	return 0;
      }
    }
  }
  free(s);
  return 1; // doesn't need to be bracketed
}

// bracket an expression if necessary

char *bracifnecc(char *s){
  char *tmp;
  if(getnumtokens(s)<=1)
    tmp = strdup(s);
  else{
    if(bcheck(s)!=0) // no need to bracket
      tmp = strdup(s);
    else
      tmp = enclose(s, "(", ")");
  }
  return tmp;
}

int genbracarraysfromdat(char *datfile, char ***array1, char ***array2){
  // generate two arrays from a data file
  // memory for the entries in the arrays is allocated
  // the second array contains bracketed versions of the second entry in the datafile
  int linelen, i, j, k=0;
  char *line;
  char *varname;
  char *varname1;
  char *expression;
  int nl=0;
  int maxl=0;
  FILE *fd;

  nl=numflines(datfile, &maxl);
  if(nl <=0){(*array1)=NULL;(*array2)=NULL;return 0;}

  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in genbracarraysfromdat in file \"stringmanip.c\" - File: %s could not be opened...\n", datfile);
    (*array1)=NULL;(*array2)=NULL;return -1;
  }

  (*array1) = charar(0, nl-1);
  (*array2) = charar(0, nl-1);
  line = (char*) malloc(sizeof(char) * (maxl));

  while((linelen = getline(fd, line, maxl+1)) > 0){
    i=0;
    j=0;
    if(!iscomsep(line)){
      varname = getnthblk(line, 1);
      expression = getrst(varname, line);
      varname1 = getnthwd(line, 1);
      if(k < nl){
	(*array1)[k] = strdup(varname1);
	(*array2)[k] = bracifnecc(expression);
	k++;
      }
      else
	fprintf(stderr, "ERROR in genbracarraysfromdat in \"stringmanip.c\": arrays are full.\n"); // should never execute
      free(varname1); free(expression); free(varname);
    }
  }
  fclose(fd);
  free(line);
  return k;
}


int getnoncomlines(char *datfile){
  FILE *fd;
  int len, numlines=0;
  char line[5000];
  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getnoncomlines: File \"%s\" could not be opened for reading.\n", datfile);
    return -1;
  }

  while((len=getline(fd, line, 5000)) > 0){
    if(!(iscomsep(line)))
      numlines++;
  }
  fclose(fd);
  return numlines;
}

char *getfirstnoncomline(char *datfile){
  FILE *fd;
  int len;
  char line[5000];
  char *ln;
  fd = fopen(datfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getfirstnoncomline: File \"%s\" could not be opened for reading.\n", datfile);
    ln=strdup(""); return ln;
  }
  while((len=getline(fd, line, 5000)) > 0){
    if(!(iscomsep(line))){
      ln=strdup(line);
      fclose(fd);
      return ln;
    }
  }
  fclose(fd);
  ln=strdup("");
  return ln;
}




char *transfunc1(char s[], char ins_string[], char *vararray[], int numvars)
{
  // take s, translate it, and place in t. 
  // translation is based on whether a string lives in
  // vararray or not. All we do is add the string ins_string 
  // to all variables from the array

  int i, j, l, len;
  int maxlen;
  char *tempoutst;
  int vn;
  char *tmp;

  vn = varnum(s, vararray, numvars);
  maxlen = strlen(s) + vn*strlen(ins_string) + 1;
  tempoutst = (char*) malloc(sizeof(char) * (maxlen));

  i=1;j=0;
  tempoutst[0]=0;
  while(!isvarstart(s,j) && s[j]){j++;}
  strncat(tempoutst, s, j);
  while(i!=0){
    if((l=startsvar(s+j, vararray, numvars)) >=0){
      len = strlen(vararray[l]);
      j+=len;
      tmp=enclose(vararray[l], "", ins_string);
      strcat(tempoutst, tmp); // safe
      free(tmp);
    }
    i=stepvars(s+j);
    strncat(tempoutst, s+j, i);
    j+=i;
  }
  return tempoutst;
}


char *transfunc1a(char s[], char ins_string1[], char ins_string2[], char *vararray[], int numvars)
{
  // take s, translate it, and place in t. 
  // translation is based on whether a string lives in
  // vararray or not. All we do is add the string ins_string1 
  // before each variable in the array and the string 
  // ins_string2 after each variable in the array

  int i, j, l, len;
  int maxlen;
  char *tempoutst;
  int vn;
  char *tmp;

  vn = varnum(s, vararray, numvars);
  maxlen = strlen(s) + vn*(strlen(ins_string1) +strlen(ins_string2)) + 1;
  tempoutst = (char*) malloc(sizeof(char) * (maxlen));

  i=1;j=0;
  tempoutst[0]=0;
  while(!isvarstart(s,j) && s[j]){j++;}
  strncat(tempoutst, s, j);
  while(i!=0){
    if((l=startsvar(s+j, vararray, numvars)) >=0){
      len = strlen(vararray[l]);
      j+=len;
      tmp=enclose(vararray[l], ins_string1, ins_string2);
      strcat(tempoutst, tmp); // safe
      free(tmp);
    }
    i=stepvars(s+j);
    strncat(tempoutst, s+j, i);
    j+=i;
  }
  return tempoutst;
}


char *transfunc2(char s[], char *array1[], char *array2[], int numvars)
{
  // take s, translate it, and place in t. 
  // translation is based on whether a string lives
  // array1 or not. All we do is replace with the appropriate
  // string in array2. A kind of find and replace function

  int i, j, l;
  int len, maxlen, maxdiff;
  char *tempoutst;
  int vn;
  char *tmp;

  if(numvars==0){
    tempoutst=strdup(s);
    return tempoutst;
  }

  vn = varnum(s, array1, numvars); // number of occurrences
  maxdiff =  maxldiff(array1, array2, numvars);
  maxlen = strlen(s) + vn*(maxdiff+2) + 2;
  tempoutst = (char*) malloc(sizeof(char) * (maxlen));

  i=1;j=0;
  tempoutst[0]=0;
  while(!isvarstart(s,j) && s[j]){j++;}// go to start of first variable
  strncat(tempoutst, s, j);

  while(i!=0){
    if((l=startsvar(s+j, array1, numvars)) >=0){
      len = strlen(array1[l]);
      j+=len;
      tmp=bracifnecc(array2[l]);
      strcat(tempoutst, tmp);
      free(tmp);
    }
    i=stepvars(s+j);
    strncat(tempoutst, s+j, i);
    j+=i;
  }
  return tempoutst;
}

// an iterative version of transfunc2 to avoid having to call it again and again

char *transfunc2it(char s[], char *array1[], char *array2[], int numvars)
{
  int i;
  char *temp;
  char *temp1;
  temp = strdup(s);

  for(i=0;i<1000;i++){
    temp1 = transfunc2(temp, array1, array2, numvars);
    if(strcmp(temp1, temp)==0){
      free(temp1);
      return temp;
    }
    free(temp);
    temp = strdup(temp1);
    free(temp1);
  }

  fprintf(stderr, "ERROR in transfunc2it - still translating after 1000 iterations. Exiting.\n");
  return temp;
}

char *transalt1(char s[], char *varname, char *vararray[], int numvars){
  int i;
  char *temp;
  char *tstr;
  char *array2[numvars];
  if(numvars>1e+10){
    fprintf(stderr, "ERROR in transalt1: too many variables.\n");
    temp = strdup("");
    return temp;
  }
  tstr = (char*) malloc(sizeof(char) * (strlen(varname) + 20));
  for(i=0;i<numvars;i++){
    sprintf(tstr, "%s[%d]", varname, i); // safe
    array2[i] = strdup(tstr);
  }
  temp = transfunc2it(s, vararray, array2, numvars);

  free(tstr);
  free_datarray(array2, numvars);
  return temp;
}


int checkbracs(char *s)
{
  // checks if s has an equal number of open and close brackets
  int i=0;
  char c;
  int lcount=0;
  int rcount=0;
  while((c = s[i++]) != '\0'){
    //    fprintf(stderr, "i=%d\n", i);
    if (c == '(')
      lcount++;
    else if (c == ')')
      rcount++;
  }
  if(lcount != rcount)
    fprintf(stderr, "\nERROR: The string %s has %d open-brackets and %d close-brackets.\n\n", s, lcount, rcount);

  return lcount - rcount;

}

// a recursive function

int isinstring(char *a, char *b)
{
  // is b (the variable) in a (the string)? Both must be well formed.
  // Uses strstr, but adds on the checking for whether the next character is actually part of the variable or not. 
  int i, j, len;
  i=1;j=0;
  len=strlen(b);
  if(!strstr(a, b))
    return 0;
  while(!isvarstart(a,j)){j++;}
  while(i!=0){
    if((strncmp(a+j, b, len)==0) && isgood(a+j, len)){return 1;}
    i=stepvars(a+j);
    j+=i;
  }
  return 0;
}

//
//
//
// Functions connected with output files. Not all are
// carefully checked
//
//
//
//


double timeaverage(char *datfile, double time1, double time2, char *varname)
{
  // a program to compute the average value of a variable over a time interval

  FILE *fd, *fd1;
  char oneline[5000];
  int i, len, numtokens, flag = 0;
  char varn[100];
  char varn1[100];
  double tval0, tval=0, tval1, val, val1;
  double areatot=0;
  int ival;
  int numlines=0;

  fd = fopen(datfile, "r");

  if(!fd){
    fprintf(stderr, "ERROR in timeaverage in stringmanip.c: file %s could not be opened.\n", datfile);
    return -1.0;
  }

  fd1 = fopen("tempfiles/tempcomp.dat", "w"); 
  if(!fd1){
    fprintf(stderr, "ERROR in timeaverage in stringmanip.c: file \"tempfiles/tempcomp.dat\" could not be opened for writing.\n");
    fclose(fd);
    return -1.0;
  }
  fprintf(fd1, "#time %s\n", varname);

  while((len = getline(fd, oneline, 5000)) > 0){
    if(!(iscomsep(oneline)))
      break;
  }

  numtokens = getnumtokens(oneline);

  for(i=1;i<=numtokens;i++){
    getnthword(oneline, varn, 100, i);
    if(strcmp(varname, varn) == 0){
      flag = 1;
      break;
    }
  }
  ival=i;


  if(flag == 0){
    fprintf(stderr, "ERROR in timeaverage in stringmanip.c: variable %s could not be found.\n", varname);
    fclose(fd);
    fclose(fd1);
    return -1.0;
  }

  // create the simplified file
  numlines=0;
  while((len = getline(fd, oneline, 5000)) > 0){
    if(!(iscomsep(oneline))){
      getnthtoken(oneline, varn, 100, 1);
      tval = atof(varn);
      if(((tval > time1) || fabs(tval-time1)<1.0e-6) && ((tval < time2) || fabs(tval-time1)<1.0e-6)){
	getnthtoken(oneline, varn, 100, ival);
	if(numlines==0 && (tval - time1) > 0.01){ // add an initial line...
	  fprintf(fd1, "%.4e\t%s\n", time1, varn);
	  numlines++;
	}
	fprintf(fd1, "%.4e\t%s\n", tval, varn);
	strcpy(varn1, varn); // safe
	numlines++;
      }
    }
  }
  // add a final line of data
  if(numlines>0 && fabs(time2-tval) > 0.01){
    fprintf(fd1, "%.4e\t%s\n", time2, varn1);
    numlines++;
  }

  fclose(fd);
  fclose(fd1);

  if(numlines==0){
    fprintf(stderr, "ERROR in timeaverage: No data in the given range. Exiting.\n");
    return -1.0;
  }

  // we never need the original file again, we can just use the simplified file

  fd1 = fopen("tempfiles/tempcomp.dat", "r");

  if(!fd1){
    fprintf(stderr, "ERROR in timeaverage in stringmanip.c: file \"tempfiles/tempcomp.dat\" could not be opened for reading.\n");
    return -1.0;
  }

  // first line holds the names
  len = getline(fd1, oneline, 5000);

  // second line - initial values
  len = getline(fd1, oneline, 5000);
  getnthtoken(oneline, varn, 100, 1);
  tval = atof(varn);
  tval0 = tval;
  getnthtoken(oneline, varn, 100, 2);
  val = atof(varn);

  if(numlines<3){
    //    fprintf(stderr, "WARNING: only one data point. This is being taken as the average.\n");
    fclose(fd1);
    return val;

  }

  while((len = getline(fd1, oneline, 5000)) > 0){
    if(!(iscom(oneline))){
      getnthtoken(oneline, varn, 100, 1);
      tval1 = atof(varn);
      if((tval1 - tval) < 0){
	fprintf(stderr, "ERROR in timeaverage in stringmanip.c: time must only increase, never decrease.\n\ttval1 = %.4e\ttval = %.4e", tval1, tval);
	fclose(fd1);
	return -1.0;
      }
      getnthtoken(oneline, varn, 100, 2);
      val1 = atof(varn);
      areatot = areatot + (tval1 - tval)*(val1 + val)/2.0;
      val = val1;
      tval = tval1;
    }
  }

  fclose(fd1);
  // the average value

  if(fabs(tval-tval0)>0)
    return areatot/(tval - tval0);
  else
    return 0.0;

}

// percentage change in a variable value between some baseline period, and some other period

double timediff(char *datfile, double time11, double time12, double time21, double time22, char *varname){
  double temp1, temp2;
  temp1 = timeaverage(datfile, time11, time12, varname);
  temp2 = timeaverage(datfile, time21, time22, varname);

  return (temp2-temp1)/temp1;
}

double distaverage(char *datfile, double time1, double time2, char *varname1, double off1, double scale1, char *varname2, double off2, double scale2, int type, int mom)
{
  // a program to compute the average distance between two variables over a given time interval

  FILE *fd, *fd1;
  char oneline[5000];
  int i, len, numtokens, flag = 0, flag1=0;
  char *varn;
  char *varn1;
  double varnval, varn1val;
  double tval0, tval, tval1;
  double xval, xval1, yval, yval1;
  double av1=0, av2=0;
  double areatot=0;
  int ival1=0, ival2=0;
  int numlines=0;
  double cpnt;
  double cpntval;
  int hascross;

  fd = fopen(datfile, "r");

  if(!fd){
    fprintf(stderr, "ERROR in distaverage in stringmanip.c: file %s could not be opened.\n", datfile);
    return -1.0;
  }

  fd1 = fopen("tempfiles/tempcomp.dat", "w"); 
  if(!fd1){
    fprintf(stderr, "ERROR in distaverage in stringmanip.c: file \"tempfiles/tempcomp.dat\" could not be opened for writing.\n");
    fclose(fd);
    return -1.0;
  }
  fprintf(fd1, "#time\t%s\t%s\n", varname1, varname2);

  while((len = getline(fd, oneline, 5000)) > 0){
    if(!(iscomsep(oneline)))
      break;
  }

  numtokens = getnumtokens(oneline);

  for(i=1;i<=numtokens;i++){
    varn = getnthwd(oneline, i);
    if(strcmp(varname1, varn) == 0){
      flag = 1;
      ival1=i;
    }
    else if(strcmp(varname2, varn) == 0){
      flag1 = 1;
      ival2=i;
    }
    free(varn);
    if(flag == 1 && flag1 == 1)
      break;
  }


  if(flag == 0){
    fprintf(stderr, "ERROR in distaverage in stringmanip.c: variable %s could not be found.\n", varname1);
    fclose(fd);
    fclose(fd1);
    return -1.0;
  }
  else if(flag1 == 0){
    fprintf(stderr, "ERROR in distaverage in stringmanip.c: variable %s could not be found.\n", varname2);
    fclose(fd);
    fclose(fd1);
    return -1.0;
  }

  // create the simplified file

  numlines=0;
  while((len = getline(fd, oneline, 5000)) > 0){
    if(!(iscom(oneline))){
      varn = getnthtkn(oneline, 1);tval = atof(varn);free(varn);
      if((tval >= time1) && (tval <=time2)){ // in the required interval?
	varn = getnthtkn(oneline, ival1);varnval = (atof(varn) - off1)/scale1;free(varn);
	av1+=varnval; // the average value of the first variable
	varn1 = getnthtkn(oneline, ival2);varn1val = (atof(varn1) - off2)/scale2;free(varn1);
	av2+=varn1val; // the average value of the second variable
	if(numlines==0 && (tval - time1) > 0.01){ // add an initial line...
	  fprintf(fd1, "%.4e\t%.4e\t%.4e\n", time1, varnval, varn1val);
	  numlines++;
	}
	fprintf(fd1, "%.4e\t%.4e\t%.4e\n", tval, varnval, varn1val);
	numlines++;
      }
    }
  }

  fclose(fd);
  fclose(fd1);

  if(numlines==0){
    fprintf(stderr, "No data in the given range. Exiting.\n");
    return -1.0;
  }

  av1/=numlines;
  av2/=numlines;

  // we never need the original file again, we can just use the simplified file

  fd1 = fopen("tempfiles/tempcomp.dat", "r");

  if(!fd1){
    fprintf(stderr, "ERROR in distaverage in stringmanip.c: file \"tempfiles/tempcomp.dat\" could not be opened for reading.\n");
    return -1.0;
  }

  // first line holds the names
  len = getline(fd1, oneline, 5000);

  // second line - initial values
  len = getline(fd1, oneline, 5000);
  varn = getnthtkn(oneline, 1);tval = atof(varn);free(varn);
  tval0 = tval;
  varn = getnthtkn(oneline, 2);xval = atof(varn);free(varn);
  varn = getnthtkn(oneline, 3);yval = atof(varn);free(varn);

  if(numlines<3){
    //    fprintf(stderr, "WARNING: only one data point. This is being taken as the average.\n");
    fclose(fd1);
    return fabs(xval-yval);

  }

  while((len = getline(fd1, oneline, 5000)) > 0){
    if(!(iscom(oneline))){
      varn = getnthtkn(oneline, 1);tval1 = atof(varn);free(varn);
      if((tval1 - tval) < 0){
	fprintf(stderr, "ERROR in distaverage in stringmanip.c: time must only increase, never decrease.\n\ttval1 = %.4f\ttval = %.4f", tval1, tval);
	fclose(fd1);
	return -1.0;
      }

      varn = getnthtkn(oneline, 2);
      if(mom==1) // remove the average value of the signal
	xval1 = atof(varn)-av1;
      else
	xval1 = atof(varn);
      free(varn);
      varn = getnthtkn(oneline, 3);
      if(mom==1) // remove the average value of the signal
	yval1 = atof(varn)-av2;
      else
	yval1 = atof(varn);
      free(varn);

      if(type == 0) // no check for crossing
	areatot = areatot + (tval1 - tval)*fabs((xval1 + xval) - (yval1 + yval))/2.0;
      else if(type == 1){ // check for crossing, etc.
        hascross=0; // do the linear interpolations cross?
        if(fabs(yval1 - yval - (xval1-xval)) > 0.001){
	  cpnt = (tval1*(xval1 + yval1) - tval*(xval + yval))/(yval1 - yval - (xval1-xval));
	  if(cpnt < tval1 && cpnt > tval){
	    hascross = 1;
	    cpntval = (cpnt*(xval1-xval) + (tval1*xval1 - tval*xval))/(tval1-tval);
	  }
        }
        if(hascross == 0){
	  areatot = areatot + (tval1 - tval)*fabs((xval1 + xval) - (yval1 + yval))/2.0;
        }
        else{ // there is a crossing
	  areatot = areatot + (tval1 - cpnt)*fabs((xval1 + cpntval)/2.0 - (yval1 + cpntval)/2.0) + (cpnt - tval)*fabs((cpntval + xval)/2.0 - (cpntval + yval)/2.0);
        }
      }
      xval = xval1;
      yval = yval1;
      tval = tval1;
    }
  }

  fclose(fd1);
  // the average value

  if(fabs(tval-tval0)>0)
    return areatot/(tval - tval0);
  else
    return 0.0;

}

//
// For two data files which have the same total time interval, but
// possibly different samplings. Generate two arrays, an integer
// array v1 and an array of double v2. v1[i] holds the line in file
// 2 with greatest time less than or equal to the time on line i in 
// file 1. v2[i] holds the weighting for (linear) interpolation to the 
// next time. Returns the number of lines
//


int resmple(char *fname1, char *fname2, int **v1, double **v2){
  int i, j, k, m;
  char **t1, **t2;
  int num1=0, num2=0;

  t1 = genarraydat(fname1, &num1);
  t2 = genarraydat(fname2, &num2);
  (*v1) = ivector(0, num1-1);
  (*v2) = dvector(0, num1-1);
  j=0;i=0;
  while(i < num1 && j < num2){
    if(strcmp(t1[i], t2[j])==0){ // both equal
      (*v1)[i]=j; (*v2)[i]=0.0;
      i++;j++; 
    }
    else if (atof(t1[i]) < atof(t2[j])){ // first less than second
       k=0;
      while(i < num1 && atof(t1[i]) < atof(t2[j])){i++; k++;}
      i-=k;
      for(m=0;m<k;m++){
	if(j>0){
	  (*v1)[i]=j-1;
	  (*v2)[i] = (atof(t1[i])-atof(t2[j-1]))/(atof(t2[j]) - atof(t2[j-1]));
	}
	else{(*v1)[i]=j; (*v2)[i]=0.0;}
	i++;
      }
    }
    else if (atof(t1[i]) > atof(t2[j])){ // first greater than second
       while(j < num2 && atof(t1[i]) > atof(t2[j])){j++;}
    }
    else{
      (*v1)[i]=j; (*v2)[i]=0.0;
      i++;j++;
    }
  }

  freearraydat(t1, num1);
  freearraydat(t2, num2);
  return num1;
}

//
// Assuming we have two arrays v1 and v2 generated by resmple
// calculate an array of differences in the values of variable vname
// scaled by the average value of vname
// 

double getdiff1(char *fname1, char *fname2, char *vname, int *v1, double *v2, int num1, double **diffs){
  int i;
  char **vals1, **vals2;
  int numv1, numv2;
  int ind1, ind2;
  double tmp1;
  double vav, diffav;

  ind1 = getdatfileindex(fname1, vname);
  ind2 = getdatfileindex(fname2, vname);
  if(ind1 < 1 || ind2 < 1){
    fprintf(stderr, "ERROR in getdiff1: variable %s could not be found.\n", vname);
    return -1;
  }

  (*diffs) = (double *) malloc(sizeof(double) * num1);

  vals1 = genarrayndat(fname1, &numv1, ind1);
  vals2 = genarrayndat(fname2, &numv2, ind2);

  vav=0;
  for(i=0;i<num1;i++){vav +=fabs(atof(vals1[i]));}
  vav/=num1;

  for(i=0;i<num1;i++){
    if(fabs(v2[i])<1e-8){
      (*diffs)[i]= fabs(atof(vals2[v1[i]])-atof(vals1[i]))/vav;
    }
    else{
      tmp1 = (1.0-v2[i])*atof(vals2[v1[i]]) + v2[i]*atof(vals2[v1[i]+1]);
      (*diffs)[i] = fabs(tmp1 - atof(vals1[i]))/vav;
    }
  }

  diffav=0;
  for(i=0;i<num1;i++){diffav +=fabs((*diffs)[i]);}
  diffav/=num1;

  freearraydat(vals1, numv1);
  freearraydat(vals2, numv2);
  return diffav;
}

//
// get all the variables stored in a data file
// into an array
//


char **getdfilevars(char *fname, int *num){
  FILE *fd;
  char **vararray;
  int i, k, c=0;
  char *line;
  int numvars;
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getdfilevars: file %s could not be opened.\n", fname);
    return NULL;
  }
  i=0;
  while((c=getc(fd)) != EOF && c != '\n'){i++;}
  line = (char*) malloc(sizeof(char) * (i+2));
  rewind(fd);
  getline(fd, line, i+2);
  fclose(fd);
  i=0;
  while(isspace(line[i])){i++;}
  if(line[i]!='#' && line[i]!='%' && line[i]!='/' && line[i]!='*'){
    fprintf(stderr, "ERROR in getdfilevars: first line of %s suggests it is not a valid datafile.\n", fname);
    free(line);
    return NULL;
  }
  k=getnumallvars(line);
  vararray = charar(0, k-1);
  numvars = getallvars(line, vararray, k);
  free(line);
  (*num)=numvars;
  return vararray;
}


//
// get all the variables stored in a data file
// into an array
//


int isgooddatfile(char *fname){
  FILE *fd;
  int c=0;
  fd = fopen(fname, "r");
  if(!fd)
    return 0;

  while((c=getc(fd)) != EOF && isspace(c)){}
  if(c!='#'){fclose(fd);return 0;} // no data line

  while((c=getc(fd)) != EOF && c!='\n'){} // skip data line
  while((c=getc(fd)) != EOF && isspace(c)){}
  if(c== EOF){fclose(fd);return 0;} // empty apart from data line
  while((c=getc(fd)) != EOF){
    if(c=='E' && (c=getc(fd))=='X' && (c=getc(fd))=='I' && (c=getc(fd))=='T')
      {fclose(fd);return 0;}
    if(c==EOF){fclose(fd);return 1;}
    if(c=='n' && (c=getc(fd))=='a' && (c=getc(fd))=='n')
      {fclose(fd);return 0;}
    if(c==EOF){fclose(fd);return 1;}
  }
  fclose(fd);
  return 1;
}

//
// Take two data files. Find the variables they share in common
// Calculate the average difference between the values in each
// file over a particular range. Output to file fout. It is like
// an intelligent "diff" routine for data files
//


int getalldiffs(char *fname1, char *fname2, char *fout){
  int i, j;
  int num1=0;
  int *v1=NULL;
  double *v2=NULL;
  char **vararray;
  char **vararray1;
  char **vararray2;
  int numvars=0, numvars1=0, numvars2;
  double **diffs;
  double tmp;
  FILE *fd;
  char **t1;

  // Get the variables in each file and find the intersection

  vararray=getdfilevars(fname1, &numvars);
  vararray1=getdfilevars(fname2, &numvars1);
  if(!vararray || !vararray1){
    fprintf(stderr, "ERROR in getalldiffs: files couldn't be opened.\n");
    return -1;
  }
  numvars2 = intersectarrays(&vararray2, vararray, numvars, vararray1, numvars1);
  freearraydat(vararray, numvars); freearraydat(vararray1, numvars1);

  fd = fopen(fout, "w");
  if(!fd){
    fprintf(stderr, "ERROR in getdfilevars: file %s could not be opened.\n", fout);
    freearraydat(vararray2, numvars2);
    return -1;
  }

  //
  // Output the first line
  //

  fprintf(fd, "#");
  for(i=0;i<numvars2-1;i++)
    fprintf(fd, "%s\t", vararray2[i]);
  fprintf(fd, "%s\n", vararray2[i]);

  diffs=(double **) malloc(sizeof(double*) * (numvars2)); // one pointer per variable
  for(j=0;j<num1;j++)
    diffs[i]=NULL;

  t1 = genarraydat(fname1, &num1);
  diffs[0]=(double *) malloc(sizeof(double) * num1);
  for(j=0;j<num1;j++)
    diffs[0][j]=atof(t1[j]);

  num1 = resmple(fname1, fname2, &v1, &v2); // allocates memory for v1 and v2

  for(i=1;i<numvars2;i++){
  fprintf(stderr, ".");
    tmp =  getdiff1(fname1, fname2, vararray2[i], v1, v2, num1, &diffs[i]);
    if(&diffs[i]==NULL){ // failure
      diffs[i]=(double *) malloc(sizeof(double) * num1);
      for(j=0;j<num1;j++)
	diffs[i][j]=0.0;
    }
  }

  for(j=1;j<num1;j++){
    for(i=0;i<numvars2-1;i++)
      fprintf(fd, "%.4e\t", diffs[i][j]);
    fprintf(fd, "%.4e\n", diffs[i][j]);
  }

  fclose(fd);

  i=numvars2;
  while(i-- > 0){
    if(diffs[i])
      free((char *) (diffs[i]));
  }
  free((char *) (diffs));

  freearraydat(vararray2, numvars2);
  free_ivector(v1, 0, num1-1);
  free_dvector(v2, 0, num1-1);
  return 0;
}

// a routine to compute the distance between all variables in two simulations. It is assumed that an identical set of variables is output

double alldist(char *datfile, char *datfile1, double time1, double time2)
{
  // first resample so that they are plotted at the same points

  FILE *fd, *fd0, *fd1;
  char oneline[10000];
  char twoline[10000];
  char threeline[10000];
  char nextline1[10000];
  char templine[10000];
  char templine1[10000];
  char lineout[10000];
  int i, len, numtokens, flag = 0;
  char varn1[80];
  double tval1, tval2, tval3, tvaltmp;
  char tempvar[100];
  int type;
  char **varnames;

  fd = fopen(datfile, "r");

  if(!fd){
    fprintf(stderr, "ERROR in alldist in stringmanip.c: file %s could not be opened.\n", datfile);
    return -1.0;
  }

  fd0 = fopen(datfile1, "r");

  if(!fd0){
    fprintf(stderr, "ERROR in alldist in stringmanip.c: file %s could not be opened.\n", datfile1);
    fclose(fd);
    return -1.0;
  }


  while((len = getline0(fd, oneline, 10000)) > 0){
    if(!(iscomsep(oneline)))
      break;
  }

  numtokens = getnumtokens(oneline);

  strcpy(templine, oneline); // safe

  while((len = getline0(fd0, oneline, 10000)) > 0){
    if(!(iscomsep(oneline)))
      break;
  }
  if(strcmp(oneline, templine) != 0){
    fprintf(stderr, "ERROR in alldist: the files %s and %s do not appear to have the same data. Exiting...\n", datfile, datfile1);
    fclose(fd);
    fclose(fd0);
    return -1.0;
  }


  // create one fully sampled file

  fd1 = fopen("tempfiles/tempcomp1.dat", "w"); 
  if(!fd1){
    fprintf(stderr, "ERROR in alldist in stringmanip.c: file \"tempfiles/tempcomp1.dat\" could not be opened for writing.\n");
    fclose(fd);
    fclose(fd0);
    return -1.0;
  }

  varnames = charar(0, numtokens-1);
  for(i=0;i<numtokens;i++)
    varnames[i] = getnthwd(oneline, i+1);

  fprintf(fd1, "%s\t", oneline);
  for(i=1;i<numtokens;i++){
    if(i<numtokens-1)
      fprintf(fd1, "_temp_%s\t", varnames[i]);
    else
      fprintf(fd1, "_temp_%s\n", varnames[i]);
  }

  freearraydat(varnames, numtokens);


  // get first lines from each file

  getline0(fd, templine, 10000);
  tval1 = gettval(templine);

  getline0(fd0, templine1, 10000);
  tval2 = gettval(templine1);



  if(tval1 > tval2){
    type=1; // midline from second file
    tvaltmp=tval1;
    tval1=tval2;
    tval3=tval2;
    tval2=tvaltmp;

    // create a special line
    snprintf(oneline, 10000, "%.4e\t", tval1);
    for(i=1;i<numtokens;i++){
      getnthtoken(templine, varn1, 80, i);
      if(i<numtokens-1)
	snprintf(tempvar, 100, "%s\t", varn1);
      else
	snprintf(tempvar, 100, "%s", varn1);
      if(strlen(oneline) + strlen(tempvar) + 1 < 10000)
	strcat(oneline, tempvar); // safe
      else{fprintf(stderr, "The line got too long. Exiting.\n");fclose(fd1);return -1.0;}
    }
    strcpy(twoline, templine); // safe
    strcpy(threeline, templine1); // safe
  }
  else{
    tval3=tval1;
    type=0; // midline from first file
    // create a special line
    snprintf(oneline, 10000, "%.4e\t", tval2);
    for(i=1;i<numtokens;i++){
      getnthtoken(templine1, varn1, 80, i);
      if(i<numtokens-1)
	snprintf(tempvar, 100, "%s\t", varn1);
      else
	snprintf(tempvar, 100, "%s", varn1);
      if(strlen(oneline) + strlen(tempvar) + 1 < 10000)
	strcat(oneline, tempvar); // safe
      else{fprintf(stderr, "The line got too long. Exiting.\n");fclose(fd1);return -1.0;}
    }
    strcpy(twoline, templine1); // safe
    strcpy(threeline, templine); // safe
  }

  // initial creation of three lines over, now loop
  //  fprintf(stderr, "got here.\n");
  flag=0;
  i=0;
  while(flag==0){
    midline (oneline, twoline, threeline, lineout, 10000, type);
    fprintf(fd1, "%s\n", lineout);
    if(type==0){
      if(getline0(fd, nextline1, 10000)<=0)
	break;
    }
    else{
      if(getline0(fd0, nextline1, 10000)<=0)
	break;
    }
    tvaltmp = gettval(nextline1);

    if(tvaltmp> tval3){ // change type
      if(type==0){type=1;}else{type=0;}
      tval1=tval3;
      tval3=tval2;
      tval2=tvaltmp;
      strcpy(oneline, threeline); // safe
      strcpy(threeline, twoline); // safe
      strcpy(twoline, nextline1); // safe
    }
    else{ // leave type as it is
      tval3=tvaltmp;
      strcpy(threeline, nextline1); // safe
    }


  }

  fclose(fd);
  fclose(fd0);
  fclose(fd1);

  return 0.0;

}

// no error checking

double gettval(char *s){
  char *varn;
  double tmp=0.0;
  varn = getnthtkn(s, 1);
  tmp = atof(varn);
  free(varn);
  return tmp;
}

int midline (char *line1, char *line2, char *line3, char lineout[], int max, int type){
  int i;
  char var1[100];
  double val1;
  int pos1;
  char var2[100];
  double val2;
  int pos2;
  char var3[100];
  int pos3;
  char tempvar[10];
  int numtokens;
  double dist;

  if(max < 50){
    fprintf(stderr, "ERROR in midline: maximum line length too short. Exiting.\n");
    return -1;
  }

  lineout[0] = 0;

  numtokens = getnumtokens(line1);
  if(numtokens != getnumtokens(line2)){
    fprintf(stderr, "ERROR in midline: different number of tokens on the two lines\n%s\n\nand\n\n%s\n\n", line1, line2);
    return -1;
  }

  pos1 = getnthtoken(line1, var1, 100, 1);
  pos2 = getnthtoken(line2, var2, 100, 1);
  pos3 = getnthtoken(line3, var3, 100, 1);

  dist=(atof(var3) - atof(var1))/(atof(var2) - atof(var1));

  snprintf(lineout, max, "%.4e\t", atof(var3));

  if(type==0){
    // midline from first file
    snprintf(lineout, max, "%s\t", line3);
    for(i=1;i<numtokens;i++){
      if((int)(strlen(lineout)) >= max-10){
	fprintf(stderr, "ERROR in midline: maximum line-length exceeded. Exiting.\n");
	return -1;
      }
      getnthtoken(line1, var1, 100, i+1);
      val1 = atof(var1);
      getnthtoken(line2, var2, 100, i+1);
      val2 = atof(var2);
      if(i<numtokens-1)
	snprintf(tempvar, 10, "%.4e\t", val1*dist + val2*(1-dist));
      else
	snprintf(tempvar, 10, "%.4e", val1*dist + val2*(1-dist));
      strncat(lineout, tempvar, max-strlen(lineout));
    }
  }
  else{
    snprintf(lineout, max, "%s\t", var3);
    // midline from second file
    for(i=1;i<numtokens;i++){
      if((int)(strlen(lineout)) >= max-10){
	fprintf(stderr, "ERROR in midline: maximum line-length exceeded. Exiting.\n");
	return -1;
      }

      getnthtoken(line1, var1, 100, i+1);
      val1 = atof(var1);
      getnthtoken(line2, var2, 100, i+1);
      val2 = atof(var2);
      if(i<numtokens-1)
	snprintf(tempvar, 10, "%.4e\t", val1*dist + val2*(1-dist));
      else
	snprintf(tempvar, 10, "%.4e", val1*dist + val2*(1-dist));
      strncat(lineout, tempvar, max-strlen(lineout));
    }
    strncat(lineout, line3+pos3, max-strlen(lineout));
  }

  return 0;
}


// convert a file with timestamp data to a normal file
int infileconvert(char *filein, char *fileout){
  FILE *fd, *fd1;
  int j, len, linelen, linenum;
  char block[1000];
  char line[200];
  char *word;
  int block_pos;
  char *tstep;
  int needtoconvert=0;
  double val, vallast=0;
  double tstepval=0;
  fd = fopen(filein, "r");
  if(!fd){
    fprintf(stderr, "ERROR in infileconvert in stringmanip.c: %s could not be opened for reading.\n", filein);
    return -1;
  }

  fd1 = fopen(fileout, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in infileconvert in stringmanip.c: %s could not be opened for writing.\n", fileout);
    fclose(fd);
    return -1;
  }

  // check if we need to convert the file...

  getblock(fd, block, "****", 1000);
  block_pos = 0;
  while ((linelen = getlinefromblock(&block_pos, block, line, 200)) > 0){
    if(iscomsep(line)){
      fprintf(fd1, "%s", line);
    } // comment/empty lines
    else if(strstr(line, "time_step:")){
      tstep = getname("time_step:", line);
      if(strcmp(tstep, "timestamp")!=0){// nothing to be done
	needtoconvert = 1;
	fprintf(fd1, "// converted from time-stepped file.\n");
	tstepval = atof(tstep);
      }
      else{
	needtoconvert = 2;
	fprintf(fd1, "// converted from time-stamped file.\n");
      }
      free(tstep);
    }
    else{
      fprintf(fd1, "%s", line);
    }
  }


  if(needtoconvert == 0){ // just copying the file
    while((len = getline(fd, line, 200)) > 0){
      fprintf(fd1, "%s", line);
    }
  }
  else if(needtoconvert == 1){ // time-stepped file
    while((len = getline(fd, line, 200)) > 0 && !(isonlyspace(line)))
      fprintf(fd1, "%.4e, %s", tstepval, line);
  }
  else{ // time stamped file
    linenum=0;
    while((len = getline(fd, line, 200)) > 0){
      j=0;
      if(iscomsep(line))
	fprintf(fd1, "%s", line);
      else{
        word = getnthtkn(line, 1);
        if(linenum==0){
	  val=atof(word);
	  vallast = val;
	  fprintf(fd1, "%s", line);
        }
        else{
	  val = atof(word)-vallast;
	  vallast = atof(word);
	  fprintf(fd1, "%.4e%s", val, line+strlen(word));
        }
	free(word);
        linenum++;
      }
    }
  }
  fclose(fd);
  fclose(fd1);
  return 0;
}

void cpfile1(char *f1, char *f2){
  FILE *fd, *fd1;
  int len;
  char line[5000];
  fd = fopen(f2, "r");
  fd1 = fopen(f1, "w");
  while((len=getline(fd, line, 5000)) > 0){
    fprintf(fd1, "%s", line);
  }
  fclose(fd);
  fclose(fd1);

}

int getdesc(char *descfile, char varname[], char vardesc[], int lim)
{
  // Get the comments about varname into vardesc as follows:
  // 1) Open the file
  // 2) Search for a line consisting solely of the name of the variable
  // 3) Get the next lines into vardesc until either we hit the limit, or we encounter a line with four ****s in it.

  int found=0, ended; // j = global counter
  char *varnameline;
  int len;
  int lim_line = 1000;
  char oneline[1000];
  int totlen=0;
  FILE *fd;
  fd = fopen(descfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getdesc - File: %s could not be opened...\n", descfile);
    vardesc[0] = '\0';
    return -1;
  }

  vardesc[0] = '\0'; // initialise the description

  while((found == 0) && (len = getline(fd, oneline, lim_line)) > 0){
    ended = 0;
    varnameline = getnthwd(oneline, 1);
    if (strcmp(varnameline, varname) == 0 && getnumtokens(oneline) == 1){
      found = 1;
      while((len = getline(fd, oneline, lim_line)) > 0 && !strstr(oneline, "****")){
	totlen+=len;
	if(totlen < lim-1){
	  strcat(vardesc, oneline); // safe
	}
	else{
	  fprintf(stderr, "ERROR in routine getdesc - line too long. Allow more space.\n");
	  free(varnameline);
	  return totlen-len;
	}
      }
    }
    free(varnameline);
  }

  fclose(fd);

  if(found == 0){
    fprintf(stderr, "WARNING in routine getdesc: variable \"%s\" not found in file \"%s\"\n", varname, descfile);
    return -2;
  }

  return totlen;

}

// insert string str into file fname before line number linenum. Note that the string str does not have to be terminated by a newline, though in most cases this would be desirable. The file is first copied to a temporary file, then opened as writeable, and then line-by-line the old lines are copied until the new string needs to be inserted. 

int insintofile(char *fname, int linenum, char *str){
  FILE *fd, *fd1;
  int i, len;
  char line[5000];
  cpfile("tempfiles/__temp_file.dt", fname);
  fd = fopen("tempfiles/__temp_file.dt", "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine insintofile - file: \"tempfiles/__temp_file.dt\" could not be opened...\n");
    return -1;
  }
  fd1 = fopen(fname, "w");
  if(!fd){
    fprintf(stderr, "ERROR in routine insintofile - file: \"%s\" could not be opened for writing...\n", fname);
    fclose(fd);
    return -1;
  }

  i=1;
  while ((i< linenum) && ((len = getline(fd, line, 5000)) > 0)){
    fprintf(fd1, "%s", line);
    i++;
  }
  fprintf(fd1, "%s", str);
  while ((len = getline(fd, line, 5000)) > 0)
    fprintf(fd1, "%s", line);
  fclose(fd);
  fclose(fd1);
  return i; // line number where insertion started

}

// add objects of type "type" (such as variables, parameters, etc.) into module files. 

int addnewobj(char *fname, char *type, char *str){
  FILE *fd;
  int len;
  char line[5000];
  char *word;
  char objtype[100];
  int hasobj;
  int linenum;
  int linetemp;
  int flag;
  char *tempstr;

  snprintf(objtype, 100, "end%s", type);

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine addnewobj - file: \"%s\" could not be opened...\n", fname);
    return -1;
  }

  // check if the file contains objects of this type

  hasobj=0;
  linenum=0;
  linetemp=0;
  flag =0;
  while (((len = getline(fd, line, 5000)) > 0) && (flag==0)){
    linenum++;
    if(!(iscomsep(line))){
      word = getnthwd(line, 1);
      if(strcmp(word, objtype)==0) // has this type
	flag=1;
      else if(strcmp(word, "endMODULE")==0) // insert at end
	flag=2;
      free(word);
    }
  }
  fclose(fd);

  if(flag==0){
    fprintf(stderr, "ERROR in addnewobj. The file %s does not appear to be a module file at all.\n", fname);
    return -1;
  }

  tempstr = (char*) malloc(sizeof(char) * (strlen(str) + strlen(type) + strlen(objtype)+50));

  if(flag==2) // no objects of this kind defined so far in module file
    sprintf(tempstr, "%s\n%s%s\n******\n", type, str, objtype); // safe
  else if(flag==1) // objects of this kind already defined in module file
    sprintf(tempstr, "%s", str); // safe

  len=insintofile(fname, linenum, tempstr);
  free(tempstr);
  return len; // return status of insintofile (positive if successful)
}

int addreactofile(char *fname){
  char *tempstr = "name: \ntype: \ncomps: \nstoics: \nleft: \ncompsleft: \nstoicsleft: \nright: \ncompsright: \nstoicsright: \nrates: \nrateterm: \n******\n";
  return addnewobj(fname, "REACTIONS", tempstr);
}

int addalgtofile(char *fname){
  char *tempstr = "name: \nkeyvar: \neqn: \n******\n";
  return addnewobj(fname, "ALGRELS", tempstr);
}

int addDEtofile(char *fname){
  char *tempstr = "name: \nlterm: \nrterm: \n******\n";
  return addnewobj(fname, "DIFFEQS", tempstr);
}

int addDEtermtofile(char *fname){
  char *tempstr = "name: \nkeyvar: \nrterm: \n******\n";
  return addnewobj(fname, "DETERMS", tempstr);
}

// check if a file has the variable/parameter, etc var of type type

int hasvar(char *fname, char *type, char *var){
  int len;
  char line[5000];
  FILE *fd;
  char *endtype, *word, *word1;

  endtype = (char*) malloc(sizeof(char) * (strlen(type) + 4));
  sprintf(endtype, "end%s", type); // safe

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine hasvar - file: \"%s\" could not be opened...\n", fname);
    return -1;
  }

  while (((len = getline(fd, line, 5000)) > 0)){
    if(!(iscomsep(line))){
      word = getnthwd(line, 1);
      if(strcmp(word, type)==0){ // has this type
	while (((len = getline(fd, line, 5000)) > 0)){
	  if(!(iscomsep(line))){
	    word1 = getnthwd(line, 1);
	    if(strcmp(word1, endtype)==0){
	      fclose(fd);free(word1);free(word);free(endtype);
	      return 0;
	    }
	    else if(strcmp(word1, var)==0){
	      fclose(fd);free(word1);free(word);free(endtype);
	      return 1;
	    }
	    free(word1);
	  }
	}
      }
      free(word);
    }
  }
  fclose(fd);
  free(endtype);
  return 0;
}


char *getvar(char *fname, char *type, char *var){
  int len;
  char line[5000];
  char *line1, *line2;
  FILE *fd;
  char *endtype, *word, *word1;
  char *emptywd = "";

  endtype = (char*) malloc(sizeof(char) * (strlen(type) + 4));
  sprintf(endtype, "end%s", type); // safe

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine getvar - file: \"%s\" could not be opened...\n", fname);
    return emptywd;
  }

  while (((len = getline(fd, line, 5000)) > 0)){
    if(!(iscomsep(line))){
      word = getnthwd(line, 1);
      if(strcmp(word, type)==0){ // has this type
	while (((len = getline(fd, line, 5000)) > 0)){
	  if(!(iscomsep(line))){
	    word1 = getnthwd(line, 1);
	    if(strcmp(word1, endtype)==0){
	      fclose(fd);free(word1);free(word);free(endtype);
	      return emptywd;
	    }
	    else if(strcmp(word1, var)==0){
	      line1 = getrst(word1, line);
	      line2 = lrtrim(line1);
	      fclose(fd);free(line1);free(word1);free(word);free(endtype);
	      return line2;
	    }
	    free(word1);
	  }
	}
      }
      free(word);
    }
  }
  fclose(fd);
  free(endtype);
  return emptywd;
}

// Extract an object of type "type" from a module file and return it in string "str"
// The object may be a multi-line object like REACTIONS, ALGRELS, DIFFEQS, DETERMS
// or a single line object (e.g. VARS or PARAMS)
// 


int getobj(FILE *fd, int *parsenum, char *type, char str[], int maxlen){
  int len;
  char line[5000];
  char *word;
  char *endtype;
  int blocktype=0;

  endtype = (char*) malloc(sizeof(char) * (strlen(type) + 4));
  sprintf(endtype, "end%s", type); // safe

  if((strcmp(type, "REACTIONS")==0) || (strcmp(type, "ALGRELS")==0) || (strcmp(type, "DIFFEQS")==0) || (strcmp(type, "TEMPLATES")==0) || (strcmp(type, "DETERMS")==0))
    blocktype=1;

  // parsenum = 0 means check if the file contains objects of this type

  str[0] = '\0';
  if(*parsenum==0){
    (*parsenum)++;
    while (((len = getline(fd, line, 5000)) > 0)){
      if(!(iscomsep(line))){
	word = getnthwd(line, 1);
	if(strcmp(word, type)==0){ // has this type
	  free(word);free(endtype);return 0;
	}
	free(word);
      }
    }
  }
  else{
    (*parsenum)++;
    while (((len = getline(fd, line, 5000)) > 0)){
      if(!(iscomsep(line))){
	word = getnthwd(line, 1);
	if(strcmp(word, endtype)==0){ // last block got
	  free(word);free(endtype);return 1;
	}
	else if(blocktype==0){ // a one-line object
	  strncpy(str, line, maxlen-1);
	  free(word);free(endtype);return 0;
	}
	else if(blocktype==1){ // a multi-line object
	  strncpy(str, line, maxlen-1);
	  while (((len = getline(fd, line, 5000)) > 0) && !strstr(line, "****"))
	    strncat(str, line, maxlen-strlen(str)-1);
	  free(word);free(endtype);return 0;
	}
	free(word);
      }
    }
  }
  free(endtype);
  return 1;
}

//
// Get an object of type "type" with name "name" from file "fname".
// Allocates memory and returns an empty string on failure. 
//

char *getnamedobj(char *fname, char *name, char *type){

  FILE *fd;
  int retstat=0;
  int parsenum=0;
  char tempstr[2000];
  char *nm;
  char *obj;

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getnamedobj: \"%s\" could not be opened for reading.\n", fname);
    obj=strdup("");
    return obj;
  }

  while(retstat==0){
    retstat=getobj(fd, &parsenum, type, tempstr, 2000);
    nm = getname("name:", tempstr);
    if(strcmp(nm, name) == 0){
      free(nm);fclose(fd);
      obj=strdup(tempstr);
      return obj;
    }
    free(nm);
  }
  fclose(fd);
  obj=strdup("");
  return obj;
}

//
// The simpler version of getnamedobj for files only containing one type of process
// The process is assumed to be of block type
//

char *gnameobj(char *fname, char *name){
  FILE *fd;
  char block[10000];
  int len;
  char *nm;
  char *obj;

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getnamedobj: \"%s\" could not be opened for reading.\n", fname);
    obj=strdup("");
    return obj;
  }

  while((len = getblock(fd, block, "******", 10000)) > 0){
    nm = getname("name:", block);
    if(strcmp(nm, name) == 0){
      free(nm);fclose(fd);
      obj=strdup(block);
      return obj;
    }
    free(nm);
  }
  fclose(fd);
  obj=strdup("");
  return obj;
}


// check a module file for variables/parameters

int scanmodfile(char *fname, char *remsfile, char *procdir, char *keyvars[], int maxkey, char *nonkeyvars[], int maxnonkey, int *nkey, int *nnonkey){

  FILE *fd;
  int parsenum=0;
  char tempstr[2000];
  char *oneline;
  char *line;
  int retstat;
  char **chms=NULL;
  char *chems[100];
  char *chems2[100];
  int i, j;
  int numkey=0;
  int numnonkey=0;
  int numvals=0;
  char tempvar[100];
  int numrems=0;
  char **rems;
  int numprocs=0;
  char **procnames=NULL;
  char *reacfname, *DEfname, *DEtermfname, *algrelfname;
  char *pname;
  int sing=0;
  char *wd;

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine scanmodfile - file: \"%s\" could not be opened...\n", fname);
    return -1;
  }

  rems = genarraydat(remsfile, &numrems);

  if(isonlyspace(procdir)){sing=1;}

  reacfname = (char*) malloc(sizeof(char) * (strlen(procdir) + 20));
  sprintf(reacfname, "%s/reacs.dat", procdir); // safe
  DEfname = (char*) malloc(sizeof(char) * (strlen(procdir) + 20));
  sprintf(DEfname, "%s/DEs.dat", procdir); // safe
  DEtermfname = (char*) malloc(sizeof(char) * (strlen(procdir) + 20));
  sprintf(DEtermfname, "%s/DEterms.dat", procdir); // safe
  algrelfname = (char*) malloc(sizeof(char) * (strlen(procdir) + 20));
  sprintf(algrelfname, "%s/algrels.dat", procdir); // safe

  //
  // parse the reactions
  //

  retstat=0;
  parsenum=0;
  numprocs=0;
  if(sing==0)
    procnames = genarraydat(reacfname, &numprocs);

  while(retstat==0){


    retstat=getobj(fd, &parsenum, "REACTIONS", tempstr, 2000);
    pname = getname("name:", tempstr);
    if((sing==1) || isinarray(procnames, numprocs, pname) >= 0){  
      if(parsenum>1 && retstat==0){
	oneline = getrst("left:", tempstr);
	numvals = chemgets1(oneline, chems, chems2, 100);
	free(oneline);	  
	for(i=0;i<numvals;i++)
	  numkey = addvar1(numkey, chems2[i], keyvars, maxkey);

	for(i=0;i<numvals;i++){
	  if(isvar(chems[i], rems, numrems) == 1) // are the stoichiometries strings?  
	    numnonkey = addvar1(numnonkey, chems[i], nonkeyvars, maxnonkey);
	}

	free_datarray(chems, numvals);
	free_datarray(chems2, numvals);
    
	oneline = getrst("right:", tempstr);
	numvals = chemgets1(oneline, chems, chems2, 100);
	free(oneline);
	for(i=0;i<numvals;i++)
	  numkey = addvar1(numkey, chems2[i], keyvars, maxkey);
	for(i=0;i<numvals;i++){
	  if(isvar(chems[i], rems, numrems) == 1) // are the stoichiometries strings?
	    numnonkey = addvar1(numnonkey, chems[i], nonkeyvars, maxnonkey);
	}
	free_datarray(chems, numvals);
	free_datarray(chems2, numvals);
	line = getname("comps:", tempstr);
	if(strcmp(line, "1") != 0 && !isonlyspace(line)){
	  oneline = getrst("compsleft:", tempstr);
	  numvals = chemgets(oneline, chems, 100);
	  free(oneline);
	  for(i=0;i<numvals;i++){
	    if(!isnumber(chems[i])){
	      snprintf(tempvar, 100, "%s", chems[i]);
	      numnonkey = addvar1(numnonkey, tempvar, nonkeyvars, maxnonkey);
	    }
	  }
	  free_datarray(chems, numvals);
      
	  oneline = getrst("compsright:", tempstr);
	  numvals = chemgets(oneline, chems, 100);
	  free(oneline);
	  for(i=0;i<numvals;i++){
	    if(!isnumber(chems[i])){
	      snprintf(tempvar, 100, "%s", chems[i]);
	      numnonkey = addvar1(numnonkey, tempvar, nonkeyvars, maxnonkey);
	    }
	  }
	  free_datarray(chems, numvals);
	}
	free(line);

	oneline = getrst("rates:", tempstr);
	numvals = chemgets(oneline, chems, 100);
	free(oneline);
	for(i=0;i<numvals;i++){
	  if(!isnumber(chems[i])){
	    numnonkey = addvar1(numnonkey, chems[i], nonkeyvars, maxnonkey);
	  }
	}
	free_datarray(chems, numvals);

	oneline = getrst("rateterm:", tempstr);
	chms=getvs(oneline, &numvals, rems, numrems);
	free(oneline);
	for(i=0;i<numvals;i++)
	  numnonkey = addvar1(numnonkey, chms[i], nonkeyvars, maxnonkey);
	freearraydat(chms, numvals);
      }
    }
    free(pname);
  }
  if(sing==0)
    freearraydat(procnames, numprocs);
  fclose(fd);

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine scanmodfile - file: \"%s\" could not be opened...\n", fname);
    free(reacfname);free(DEfname);free(DEtermfname);free(algrelfname);
    return -1;
  }
  //   fprintf(stderr, "\n\nkey before algs = %d\n\n", numkey);
  //
  // parse the algrels
  //

  retstat=0;
  parsenum=0;
  numprocs=0;
  if(sing==0)
    procnames = genarraydat(algrelfname, &numprocs);
  while(retstat==0){
    retstat=getobj(fd, &parsenum, "ALGRELS", tempstr, 2000);
    pname = getname("name:", tempstr);
    if(sing==1 || isinarray(procnames, numprocs, pname) >= 0){
      if(parsenum>1 && retstat==0){
	//      fprintf(stderr, "\n\nalg found.\n\n");
	line = getname("keyvar:", tempstr);
	numkey = addvar1(numkey, line, keyvars, maxnonkey);
	free(line);
	line = getrst("eqn:", tempstr);
	chms=getvs(line, &numvals, rems, numrems);
	free(line);
	for(i=0;i<numvals;i++)
	  numnonkey = addvar1(numnonkey, chms[i], nonkeyvars, maxnonkey);
	freearraydat(chms, numvals);
      }
    }
    free(pname);
  }
  if(sing==0)
    freearraydat(procnames, numprocs);
  fclose(fd);
  //  fprintf(stderr, "\n\nkey before des = %d\n\n", numkey);
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine scanmodfile - file: \"%s\" could not be opened...\n", fname);
    free(reacfname);free(DEfname);free(DEtermfname);free(algrelfname);
    return -1;
  }
  //
  // parse the DEs
  //

  retstat=0;
  parsenum=0;
  numprocs=0;
  if(sing==0)
    procnames = genarraydat(DEfname, &numprocs);
  while(retstat==0){
    retstat=getobj(fd, &parsenum, "DIFFEQS", tempstr, 2000);
    pname = getname("name:", tempstr);
    if(sing==1 || isinarray(procnames, numprocs, pname) >= 0){
      //    fprintf(stderr, "tempstr=%s\n", tempstr);
      if(parsenum>1 && retstat==0){
	oneline = getrst("lterm:", tempstr);
	numvals = chemgets1(oneline, chems, chems2, 100);
	free(oneline);
	for(i=0;i<numvals;i++)
	  numkey = addvar1(numkey, chems2[i], keyvars, maxkey);
	free_datarray(chems, numvals);
	free_datarray(chems2, numvals);
	oneline = getrst("rterm:", tempstr);
	chms = getvs(oneline, &numvals, rems, numrems);
	free(oneline);
	for(i=0;i<numvals;i++)
	  numnonkey = addvar1(numnonkey, chms[i], nonkeyvars, maxnonkey);
	freearraydat(chms, numvals);
      }
    }
    free(pname);
  }
  if(sing==0)
    freearraydat(procnames, numprocs);
  fclose(fd);
  //    fprintf(stderr, "out of DEs.\n");
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine scanmodfile - file: \"%s\" could not be opened...\n", fname);
    free(reacfname);free(DEfname);free(DEtermfname);free(algrelfname);
    return -1;
  }

  //
  // parse the DEterms
  //

  retstat=0;
  parsenum=0;
  numprocs=0;

  if(sing==0)
    procnames = genarraydat(DEtermfname, &numprocs);
  while(retstat==0){
    retstat=getobj(fd, &parsenum, "DETERMS", tempstr, 2000);
    pname = getname("name:", tempstr);
    if(sing==1 || isinarray(procnames, numprocs, pname) >= 0){
      if(parsenum>1 && retstat==0){
	line = getname("keyvar:", tempstr);
	numkey = addvar1(numkey, line, keyvars, maxkey);
	free(line);
	oneline = getrst("term:", tempstr);
	chms = getvs(oneline, &numvals, rems, numrems);
	free(oneline);
	for(i=0;i<numvals;i++)
	  numnonkey = addvar1(numnonkey, chms[i], nonkeyvars, maxnonkey);
	freearraydat(chms, numvals);
      }
    }
    free(pname);
  }
  if(sing==0)
    freearraydat(procnames, numprocs);
  fclose(fd);

  //
  // temporary variables
  //

  for(i=0;i<numnonkey;i++){
    if(hasvar(fname, "TEMPVS", nonkeyvars[i])){ // a temporary variable
      // add the other quantities that it depends on
      oneline = getvar(fname, "TEMPVS", nonkeyvars[i]);
      chms = getvs(oneline, &numvals, rems, numrems);
      free(oneline);
      for(j=0;j<numvals;j++)
	numnonkey = addvar1(numnonkey, chms[j], nonkeyvars, maxnonkey);
      freearraydat(chms, numvals);
    }
  }

  //
  // parameters already defined
  //

  for(i=0;i<numnonkey;i++){
    if(hasvar(fname, "PARAMS", nonkeyvars[i])){ // a temporary variable
      // add the other quantities that it depends on
      oneline = getvar(fname, "PARAMS", nonkeyvars[i]);
      wd=getnthwd(oneline, 2);
      if(!strstr(wd, "_fcn")){
        chms = getvs(oneline, &numvals, rems, numrems);
        for(j=0;j<numvals;j++)
	  numnonkey = addvar1(numnonkey, chms[j], nonkeyvars, maxnonkey);
        freearraydat(chms, numvals);
      }
      free(wd);
      free(oneline);
    }
  }

  // delete the keyvars and the rems from the array nonkeyvars

  numnonkey=delarray(nonkeyvars, numnonkey, keyvars, numkey);
  numnonkey=delarray(nonkeyvars, numnonkey, rems, numrems);
  /*   fprintf(stderr, "numkey = %d\n", numkey); */
  /*   fprintf(stderr, "numnonkey = %d\n", numnonkey); */

  /*   for(i=0;i<numkey;i++){ */
  /*     fprintf(stderr, "key[%d] = %s\n", i, keyvars[i]); */
  /*   } */

  *nkey=numkey;
  *nnonkey=numnonkey;
  freearraydat(rems, numrems);
  free(reacfname);free(DEfname);free(DEtermfname);free(algrelfname);

  return 0;


}

int quickparse(char *fname, char *remsfile){

  char tempstr[2000];
  char tempstr1[2000];
  char *keyvars[1000]; // to store true variables
  char *nonkeyvars[1000]; // to store nonkey variables (parameters)
  int i;
  int numkey=0;
  int numnonkey=0;
  int tot=0;

  if(scanmodfile(fname, remsfile, "", keyvars, 1000, nonkeyvars, 1000, &numkey, &numnonkey)!=0){
    fprintf(stderr, "ERROR in routine quickparse: routine scanmodfile failed...\n");
    return -1;

  }


  // add vars if necessary
  for(i=0;i<numkey;i++){
    //    fprintf(stderr, "i=%d\n", i);
    if (!(hasvar(fname, "VARS", keyvars[i]))){
      snprintf(tempstr, 2000, "%s\t%s_init\n", keyvars[i], keyvars[i]);
      addnewobj(fname, "VARS", tempstr);
      snprintf(tempstr, 2000, "%s_init", keyvars[i]);
      tot++;
      if(!(hasvar(fname, "PARAMS", tempstr))){
	snprintf(tempstr1, 2000, "%sn\n", keyvars[i]);
	if(!(hasvar(fname, "PARAMS", tempstr1))){
	  snprintf(tempstr, 2000, "%s_init\t0.0\n", keyvars[i]);
	  addnewobj(fname, "PARAMS", tempstr);
	  fprintf(stderr, "adding variable %s and parameter %s_init. You MUST initialise %s_init.\n", keyvars[i], keyvars[i], keyvars[i]);
	  tot++;
	}
	else{
	  snprintf(tempstr, 2000, "%s_init\t%sn\n", keyvars[i], keyvars[i]);
	  addnewobj(fname, "PARAMS", tempstr);
	  fprintf(stderr, "adding variable %s and parameter %s_init. %s_init is being initialised to %sn.\n", keyvars[i], keyvars[i], keyvars[i], keyvars[i]);
	  tot++;
	}
      }
      else{
	fprintf(stderr, "adding variable %s and initialising to parameter %s_init.\n", keyvars[i], keyvars[i]);
      }

    }
  }

  // parameters
  for(i=0;i<numnonkey;i++){
    if (!(hasvar(fname, "PARAMS", nonkeyvars[i]))){
      if(hasvar(fname, "TEMPVS", nonkeyvars[i])){ // a temporary variable
	// do nothing
      }
      else{ // not a temporary variable
	snprintf(tempstr, 2000, "%s\t0.0\n", nonkeyvars[i]);
	fprintf(stderr, "adding parameter %s. You MUST initialise it.\n", nonkeyvars[i]);
	addnewobj(fname, "PARAMS", tempstr);
	tot++;
      }
    }
  }

  fprintf(stderr, "numnonkey = %d\n", numnonkey);
  /*    for(i=0;i<numnonkey;i++){ */
  /*       fprintf(stderr, "nonkey[%d] = %s\n", i, nonkeyvars[i]); */
  /*     } */


  free_datarray(nonkeyvars, numnonkey);
  free_datarray(keyvars, numkey);
  return tot;
}

//
// This routine is like quickparse, but it doesn't actually update the file
// Output is returned in a string
//


int quickscan(char *fname, char *remsfile, char outstr[], int lim){

  char tempstr[2000];
  char tempstr1[200];
  int tot=1999;
  char *keyvars[1000]; // to store true variables
  char *nonkeyvars[1000]; // to store nonkey variables (parameters)
  int i;
  int numkey=0;
  int numnonkey=0;

  if(lim < 1){
    fprintf(stderr, "ERROR in routine quickscan: Your output string is too short. Exiting.\n");
    return -1;
  }
  strcpy(outstr, ""); // safe
  strcpy(tempstr, ""); // safe

  if(scanmodfile(fname, remsfile, "", keyvars, 1000, nonkeyvars, 1000, &numkey, &numnonkey)!=0){
    fprintf(stderr, "ERROR in routine quickscan: routine scanmodfile failed...\n");
    return -1;

  }

  fprintf(stderr, "This module defines dynamics for the following %d variables: \n", numkey);
  snprintf(tempstr1, 200, "This module defines dynamics for the following %d variables: \n", numkey);
  strncat(tempstr, tempstr1, tot);


  for(i=0;i<numkey;i++){
    if (!(hasvar(fname, "VARS", keyvars[i]))){
      fprintf(stderr, "\t%s\t(unlisted)\n", keyvars[i]);
      snprintf(tempstr1, 200, "  %s  (unlisted)\n", keyvars[i]);
      strncat(tempstr, tempstr1, tot-strlen(tempstr));
    }
    else{
      fprintf(stderr, "\t%s\t(listed)\n", keyvars[i]);
      snprintf(tempstr1, 200, "  %s  (listed)\n", keyvars[i]);
      strncat(tempstr, tempstr1, tot-strlen(tempstr));
    }
  }

  fprintf(stderr, "\nThis module uses the following %d parameters: \n", numnonkey);
  snprintf(tempstr1, 200, "\nThis module uses the following %d parameters: \n", numnonkey);
  strncat(tempstr, tempstr1, tot-strlen(tempstr));

  for(i=0;i<numnonkey;i++){
    if (!(hasvar(fname, "PARAMS", nonkeyvars[i]))){
      if(hasvar(fname, "TEMPVS", nonkeyvars[i])){
	fprintf(stderr, "\t%s\t(listed as temp variable)\n", nonkeyvars[i]);
	snprintf(tempstr1, 200, "  %s  (listed as temp variable)\n", nonkeyvars[i]);
	strncat(tempstr, tempstr1, tot-strlen(tempstr));
      }
      else{
	fprintf(stderr, "\t%s\t(unlisted)\n", nonkeyvars[i]);
	snprintf(tempstr1, 200, "  %s  (unlisted)\n", nonkeyvars[i]);
	strncat(tempstr, tempstr1, tot-strlen(tempstr));
      }
    }
    else{
      fprintf(stderr, "\t%s\t(listed)\n", nonkeyvars[i]);
      snprintf(tempstr1, 200, "  %s  (listed)\n", nonkeyvars[i]);
      strncat(tempstr, tempstr1, tot-strlen(tempstr));
    }
  }

  free_datarray(nonkeyvars, numnonkey);
  free_datarray(keyvars, numkey);

  strncpy(outstr, tempstr, lim-1);
  return 0;
}

//
// dirwalk for scandir
//


int dwlk2(char *dname, int (*fcn)(char *, char *, char *, char **, int *, char **, int *, int), char *descdname, char *remsfile, char *totkeyvars[], int *totkey, char *totnonkeyvars[], int *totnonkey, int max)
{
  // adapted from Kernighan and Ritchie
  char name[1024];
  struct dirent *dp;
  DIR *dfd;
  int tmp;
  int retstat=0;

  if ((dfd = opendir(dname)) == NULL){
    fprintf(stderr, "ERROR in dwlk2: can't open %s.\n", dname);
    return -1;
  }

  while ((dp = readdir(dfd)) != NULL){
    if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0 || endswith(dp->d_name, "~"))
      continue;
    if(strlen(dname)+strlen(dp->d_name)+2 > sizeof(name)){
      fprintf(stderr, "ERROR in dwlk2: name %s/%s too long.\n", dname, dp->d_name);
      retstat=-1;
    }
    else{
      snprintf(name, 1024, "%s/%s", dname, dp->d_name);
      tmp = (*fcn)(name, descdname, remsfile, totkeyvars, totkey, totnonkeyvars, totnonkey, max);
      if(tmp != 0)
	retstat=-1;
    }

  }
  closedir(dfd);
  return retstat;
}

int qsd(char *fname, char *descdname, char *remsfile, char *totkeyvars[], int *totkey, char *totnonkeyvars[], int *totnonkey, int max){
  int i;
  struct stat stbuf;
  int retstat=0;
  int isdir=0;
  char *keyvars[1000]; // to store true variables
  char *nonkeyvars[1000]; // to store nonkey variables (parameters)
  int numkey=0;
  int numnonkey=0;

  if(stat(fname, &stbuf) == -1) {
    fprintf(stderr, "ERROR in qsd: can't access %s\n", fname);
    return -1;
  }
  if((stbuf.st_mode & S_IFMT) == S_IFDIR){
    isdir=1;
    retstat = dwlk2(fname, qsd, descdname, remsfile, totkeyvars, totkey, totnonkeyvars, totnonkey, max);
  }
  else{
    if((retstat=scanmodfile(fname, remsfile, descdname, keyvars, 1000, nonkeyvars, 1000, &numkey, &numnonkey))!=0){
      fprintf(stderr, "ERROR in routine qsd: routine scanmodfile failed on file %s...\n", fname);
    }

    for(i=0;i<numkey;i++)
      (*totkey) = addvar1((*totkey), keyvars[i], totkeyvars, max);
    for(i=0;i<numnonkey;i++)
      (*totnonkey) = addvar1((*totnonkey), nonkeyvars[i], totnonkeyvars, max);
    free_datarray(nonkeyvars, numnonkey);
    free_datarray(keyvars, numkey);
  }

  return retstat;
}


// like quickscan, but scans the whole directory. If descdname is 
// an empty string, then it scans all processes it finds. Otherwise 
// it only scans those processes which are listed in the model descriptor
// file. 
//


int quickscandir(char *dname, char *descdname, char *remsfile, char outstr[], int lim){

  char *tempstr;
  char tempstr1[300];
  char *totkeyvars[2000];
  char *totnonkeyvars[2000];
  int totkey=0;
  int totnonkey=0;
  int i;
  int lentot=0;
  char sep[4];

  dwlk2(dname, qsd, descdname, remsfile, totkeyvars, &totkey, totnonkeyvars, &totnonkey, 2000);

  lentot=0;
  for(i=0;i<totkey;i++)
    lentot+=10+strlen(totkeyvars[i]);
  for(i=0;i<totnonkey;i++)
    lentot+=10+strlen(totnonkeyvars[i]);
  lentot+=1000;

  tempstr = (char*) malloc(sizeof(char) * (lentot+1));
  tempstr[0] = 0;
  if(totkey+totnonkey>30)
     strcpy(sep, "  "); // safe
  else
  strcpy(sep, " \n"); // safe
  snprintf(tempstr1, 300, "This model defines dynamics for the following %d variables: \n", totkey);
  strncat(tempstr, tempstr1, lentot);

  for(i=0;i<totkey;i++){
    if(((i+1)%10)==0)
      snprintf(tempstr1, 300, "%s\n", totkeyvars[i]);
    else
      snprintf(tempstr1, 300, "%s%s", totkeyvars[i], sep);
    strncat(tempstr, tempstr1, lentot-strlen(tempstr));
  }

  snprintf(tempstr1, 300, "\n\nThis model uses the following %d parameters: \n", totnonkey);
  strncat(tempstr, tempstr1, lentot-strlen(tempstr));

  for(i=0;i<totnonkey;i++){
    if(((i+1)%10)==0)
      snprintf(tempstr1, 300, "%s\n", totnonkeyvars[i]);
    else
      snprintf(tempstr1, 300, "%s%s", totnonkeyvars[i], sep);
    strncat(tempstr, tempstr1, lentot-strlen(tempstr));
  }

  free_datarray(totnonkeyvars, totnonkey);
  free_datarray(totkeyvars, totkey);

  if(lim < (int)(strlen(tempstr)))
    fprintf(stderr, "WARNING in routine quickscandir: Not enough memory allocated to list everything.\n");
  strncpy(outstr, tempstr, lim);
  free(tempstr);
  return strlen(outstr);
}


// get dir contents

int getdc(char *dname, char *filter, char *out[], int max){

  int i=0;
  struct dirent *dp;
  DIR *dfd;

  if ((dfd = opendir(dname)) == NULL){
    fprintf(stderr, "ERROR in getdc: can't open %s\n", dname);
    return -1;
  }

  while ((dp = readdir(dfd)) != NULL){
    if (strstr(dp->d_name, filter)){
      if(i<max-1)
	out[i++] = strdup(dp->d_name);
      else{
	fprintf(stderr, "WARNING in getdc: too many files to list in %s\n", dname);
	break;
      }
    }
  }
  return i;
}

int dwlk(char *dname, int (*fcn)(char *, char **, char **, int, char *, int *), char *parname[], char *newparname[], int numpars, char *remsfile, int *totcount)
{
  // adapted from Kernighan and Ritchie
  char name[1024];
  struct dirent *dp;
  DIR *dfd;
  int tmp;
  int retstat=0;

  if ((dfd = opendir(dname)) == NULL){
    fprintf(stderr, "ERROR in dwlk: can't open %s. This directory will be left unchanged.\n", dname);
    return -1;
  }

  while ((dp = readdir(dfd)) != NULL){
    if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0 || endswith(dp->d_name, "~"))
      continue;
    if(strlen(dname)+strlen(dp->d_name)+2 > sizeof(name)){
      fprintf(stderr, "ERROR in dwlk: name %s/%s too long. This file will be left unchanged.\n", dname, dp->d_name);
      retstat=-1;
    }
    else{
      snprintf(name, 1024, "%s/%s", dname, dp->d_name);
      tmp = (*fcn)(name, parname, newparname, numpars, remsfile, totcount);
      if(tmp != 0)
	retstat=-1;
    }

  }
  closedir(dfd);
  return retstat;
}

//
// change parameter name in file fname. If fname is a directory, 
// call dwlk
//


int chparname(char *fname, char *parname[], char *newparname[], int numpars, char *remsfile, int *totcount){
  FILE *fd;
  FILE *fd1;
  struct stat stbuf;
  char line[5000];
  char *line1;
  int len;
  int retstat=0;
  int startcount=(*totcount);
  int isdir=0;
  if(stat(fname, &stbuf) == -1) {
    fprintf(stderr, "ERROR in chparname: can't access %s\n", fname);
    return -1;
  }
  if((stbuf.st_mode & S_IFMT) == S_IFDIR){
    isdir=1;
    retstat = dwlk(fname, chparname, parname, newparname, numpars, remsfile, totcount);
  }
  else{
    if(cpfile("tempfiles/__temp_file.tmp", fname)<0){
      fprintf(stderr, "ERROR in chparname - couldn't copy file %s to \"tempfiles/__temp_file.tmp\". %s is being left unchanged\n", fname, fname);
      return -1;
    }
    fd = fopen("tempfiles/__temp_file.tmp", "r");
    if(!fd){
      fprintf(stderr, "ERROR in chparname - file: \"tempfiles/__temp_file.tmp\" could not be opened... %s is being left unchanged\n", fname);
      return -1;
    }
    fd1 = fopen(fname, "w");
    if(!fd1){
      fprintf(stderr, "ERROR in chparname - file: \"%s\" could not be opened for writing...\n%s is being left unchanged\n", fname, fname);
      fclose(fd);
      return -1;
    }

    while((len=getline(fd, line, 5000)) > 0){
      line1 = transfunc2(line, parname, newparname, numpars);
      if(strcmp(line1, line) !=0){(*totcount)++;}
      fprintf(fd1, "%s", line1);
      free(line1);
    }
    fclose(fd);
    fclose(fd1);
  }
  if(isdir==0 && (*totcount)-startcount > 0)
    fprintf(stderr, "%d lines of FILE %s changed.\n", (*totcount)-startcount, fname);
  else if(isdir==1 && (*totcount)-startcount > 0)
    fprintf(stderr, "  (%d lines of directory %s changed.)\n", (*totcount)-startcount, fname);

  return retstat;
}


// 
// change the name of a token throughout a model
//


int changeparname(char *modname, char *parname[], char *newparname[], int numpars, char *remsfile){

  int i, retstat=0;
  char *dname;
  int totcount=0, tmpcount;
  int numrems=0;
  char **rems;
  int flag=0;

  rems = genarraydat(remsfile, &numrems);
  for(i=0;i<numpars;i++){
    if(isinarray(rems, numrems, newparname[i])>=0){
      fprintf(stderr, "ERROR in changeparname - the word \"%s\" is a reserved word and cannot be used as a variable or parameter name...\n", newparname[i]);
      freearraydat(rems, numrems);
      return -2;
    }
  }

  fprintf(stderr, "WARNING: The name *will* be changed in any shared module files.\n");
  fprintf(stderr, "WARNING: The name *will not* be changed in any shared input files.\n");

  // the modules files

  tmpcount=totcount;

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/modules/%s", modname); // safe
  retstat = dwlk(dname, chparname, parname, newparname, numpars, remsfile, &totcount);
  if(retstat!=0)
    fprintf(stderr, "\tWARNING: success was not complete.\n");
  free(dname);

  if(totcount <= tmpcount)
    fprintf(stderr, "No module files were changed.\n");

  tmpcount=totcount;
  // the files in the modeldescs directory

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/modeldescs/%s", modname); // safe
  retstat = dwlk(dname, chparname, parname, newparname, numpars, remsfile, &totcount);
  if(retstat!=0){flag=1;}
  free(dname);

  if(totcount <= tmpcount)
    fprintf(stderr, "No model descriptor files were changed.\n");

  tmpcount=totcount;

  // the files in the inputs directory (if there is one)

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/inputs/%s", modname); // safe
  retstat = dwlk(dname, chparname, parname, newparname, numpars, remsfile, &totcount);
  if(retstat!=0){flag=1;}
  free(dname);

  if(totcount <= tmpcount)
    fprintf(stderr, "No input files were changed.\n");

  tmpcount=totcount;
  // the parameter value file

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/pvals/%s", modname); // safe
  retstat = dwlk(dname, chparname, parname, newparname, numpars, remsfile, &totcount);
  if(retstat!=0){flag=1;}
  free(dname);


  if(totcount <= tmpcount)
    fprintf(stderr, "No parameter value files were changed.\n");

  if(totcount>0 && flag==1){
    fprintf(stderr, "WARNING: some files were changed, but success was not complete.\n");
  }

  if(totcount==0)
    fprintf(stderr, "The quantity was not found. Nothing was changed.\n");
  return totcount;
}

//
// dirwalk for checkparname
//

int dwlk1(char *dname, int (*fcn)(char *, char **, char **, int, int *, int *), char *parname[], char *newparname[], int numpars, int *totcount, int *comcount)
{
  // adapted from Kernighan and Ritchie
  char name[1024];
  struct dirent *dp;
  DIR *dfd;
  int tmp;
  int retstat=0;

  if ((dfd = opendir(dname)) == NULL){
    fprintf(stderr, "ERROR in dwlk1: can't open %s\n", dname);
    return -1;
  }

  while ((dp = readdir(dfd)) != NULL){
    if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0 || endswith(dp->d_name, "~"))
      continue;
    if(strlen(dname)+strlen(dp->d_name)+2 > sizeof(name)){
      fprintf(stderr, "ERROR in dwlk1: name %s/%s too long.\n", dname, dp->d_name);
      retstat=-1;
    }
    else{
      snprintf(name, 1024, "%s/%s", dname, dp->d_name);
      tmp = (*fcn)(name, parname, newparname, numpars, totcount, comcount);
      if(tmp != 0)
	retstat=-1;
    }

  }
  closedir(dfd);
  return retstat;
}

//
// check if a parameter name exists in a directory
//

int chkparname(char *fname, char *parname[], char *newparname[], int numpars, int *totcount, int *comcount){
  FILE *fd;
  struct stat stbuf;
  char line[5000];
  char *line1;
  int len;
  int retstat=0;
  if(stat(fname, &stbuf) == -1) {
    fprintf(stderr, "ERROR in chkparname: can't access %s\n", fname);
    return -1;
  }
  if((stbuf.st_mode & S_IFMT) == S_IFDIR)
    retstat = dwlk1(fname, chkparname, parname, newparname, numpars, totcount, comcount);
  else{
     fd = fopen(fname, "r");
      if(!fd){
	fprintf(stderr, "ERROR in chkparname - file: \"%s\" could not be opened...\n", fname);
	return -1;
      }

      while((len=getline(fd, line, 5000)) > 0){
	line1 = transfunc2(line, parname, newparname, numpars);
	if(strcmp(line1, line) !=0){
	  if(iscomsep(line)){(*comcount)++;} // occurs in comment lines
	  else{(*totcount)++;}
	}
	free(line1);
      }
      fclose(fd);
  }
  return retstat;
}

int checkparname(char *modname, char *parname[], int numpars, int *comcount){

  int i;
  char *dname;
  int totcount=0;
  char *newparname[numpars];
  int retstat=0;

  (*comcount)=0;
  for(i=0;i<numpars;i++){
    newparname[i] = (char*) malloc(sizeof(char) * (strlen(parname[i]) + 3));
    sprintf(newparname[i], "__%s", parname[i]); // safe
  }

  // the modules files

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/modules/%s", modname); // safe
  retstat = dwlk1(dname, chkparname, parname, newparname, numpars, &totcount, comcount);
  if(retstat!=0)
    fprintf(stderr, "\tWARNING in checkparname: success was not complete.\n");
  free(dname);

  // descriptor files

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/modeldescs/%s", modname); // safe
  retstat = dwlk1(dname, chkparname, parname, newparname, numpars, &totcount, comcount);
  if(retstat!=0)
    fprintf(stderr, "\tWARNING in checkparname: success was not complete.\n");
  free(dname);

  // the files in the inputs directory (if there is one)

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/inputs/%s", modname); // safe
  retstat = dwlk1(dname, chkparname, parname, newparname, numpars, &totcount, comcount);
  if(retstat!=0)
    fprintf(stderr, "\tWARNING in checkparname: success was not complete.\n");
  free(dname);

  // the parameter value file

  dname = (char*) malloc(sizeof(char) * (strlen(modname) + 50));
  sprintf(dname, "models/pvals/%s", modname); // safe
  retstat = dwlk1(dname, chkparname, parname, newparname, numpars, &totcount, comcount);
  if(retstat!=0)
    fprintf(stderr, "\tWARNING in checkparname: success was not complete.\n");
  free(dname);

  free_datarray(newparname, numpars);
  return totcount;
}


// change parameter pname from value valold to value valnew in file pfile

int pchange(char *pfile, char *pname, char *valold, char *valnew){
  FILE *fd, *fd1;
  char line[1000];
  char *word;
  char *word1;
  int found=-1;
  int flag=0;
  int len;


  fd = fopen(pfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine pchange: file \"%s\" could not be opened for reading.\n", pfile);
    return -1;
  }
  fclose(fd);

  cpfile("tempfiles/temppfile", pfile);
  fd = fopen("tempfiles/temppfile", "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine pchange: file \"tempfiles/temppfile\" could not be opened for reading.\n");
    return -1;
  }
  fd1 = fopen(pfile, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in routine pchange: file \"%s\" could not be opened for reading.\n", pfile);
    fclose(fd);
    return -1;
  }

  while((len=getline(fd, line, 1000)) > 0){
    if(!(iscomsep(line))){
      word = getnthwd(line, 1);
      if(strcmp(word, pname) == 0){ // parameter found
	found=0;
	word1 = getnthtkn(line, 2);
	if(strcmp(word1, valold) == 0)
	  fprintf(fd1, "%s\t%s\n", word, valnew);
	else{
	  fprintf(stderr, "ERROR in pchange. Value must have been changed by hand.\n");
	  flag=-1;
	  fprintf(fd1, "%s", line);
	}
	free(word1);
      }
      else // parameter not found
	fprintf(fd1, "%s", line);
      free(word);
    }
    else // comment or separator
      fprintf(fd1, "%s", line);
  }

  fclose(fd);
  fclose(fd1);
  return min(found, flag);

}



int undoparamsession(char *pchangefile){

  FILE *fd, *fd1;
  char block[5000];
  char block1[5000];
  int len, linelen;
  char line[1000];
  char *pfile;
  int numsessions=0;
  int i, numchanges, l;
  char *pname, *pvalold, *pvalnew;
  int tempnum;
  int flag=0;

  block1[0] = '\0';

  cpfile("tempfiles/temppch.log", pchangefile);
  fd = fopen("tempfiles/temppch.log", "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine undoparamsession: file \"tempfiles/temppch.log\" could not be opened for reading.\n");
    return -1;
  }
  fd1 = fopen(pchangefile, "w");
  if(!fd1){
    fprintf(stderr, "ERROR in routine undoparamsession: file \"%s\" could not be opened for reading.\n", pchangefile);
    fclose(fd);
    return -1;
  }

  // get the last session

  while((len = getblock(fd, block, "ENDSESSION", 5000)) > 0 && strstr(block, "ENDSESSION")){
    if(numsessions>0)
      fprintf(fd1, "%s", block1); 
      strcpy(block1, block); // safe
    numsessions++;
  }
  fclose(fd);
  fclose(fd1);

  // get the number of changes in the session

  i=0;
  numchanges=0;
  while((linelen = getlinefromblock(&i, block1, line, 1000)) > 0){
    if(!(iscom(line))){
      if(strstr(line, "File:")){
	numchanges++;
      }
    }
  }
  // undo the changes in reverse order

  while(numchanges>0){
    i=0;
    tempnum=0;
    while((linelen = getlinefromblock(&i, block1, line, 1000)) > 0){
      if(!(iscom(line))){
	if(strstr(line, "File:")){
	  tempnum++;
	  if(tempnum==numchanges){
	    pfile = getrst("File:", line);
	    getlinefromblock(&i, block1, line, 1000); // next line
	    pname = getnthwd(line, 1);
	    pvalold = getnthtkn(line, 2);
	    pvalnew = getnthtkn(line, 4);
	    l=pchange(pfile, pname, pvalnew, pvalold);
	    free(pname);free(pvalold);free(pvalnew);free(pfile);
	    if(l!=0)
	      flag=1;
	    numchanges--;
	    continue;
	  }

	}

      }
    }
  }
  return flag;
}



char *cleansq(char *t)
{
  // get rid of square brackets and $ signs from string t and put in string s
  int i=0, j=0;
  char *s;
  s = strdup(t);
  while(t[i] != '\0'){
    if(t[i] == '[' || t[i] == ']' || t[i] == '$'){i++;}
    else{s[j++] = t[i++];}
  }
  s[j++] = '\0';
  return s;
}

int tidy(char *s, char *t)
{
  // when there is an underscore, deal with it
  int i=0, j=0;
  int hasund=0;
  while(t[i] != '\0'){
    if(hasund==0 && t[i] == '_'){
      s[j++] = t[i++];
      s[j++] = '{';
      hasund++;
    }
    else if(hasund>0 && t[i] == '_'){
      s[j++] = t[i++];
      s[j++] = ',';
      hasund++;
    }
    else
      s[j++] = t[i++];
  }
  if(hasund > 0)
    s[j++] = '}';
  s[j++] = '\0';
  return j;
}
char *getltxname(char *descfile, char varname[])
{
  int found=0, ended; 
  int len;
  int lim_line = 1000;
  char oneline[1000];
  FILE *fd;
  char *ltxname=NULL;
  char *simpltxname=NULL;
  fd = fopen(descfile, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getltxname: File %s could not be opened for reading.\n", descfile);
    simpltxname = strdup(varname);
    return simpltxname;
  }

  while((found == 0) && (len = getline0(fd, oneline, lim_line)) > 0){
    ended = 0;
    if (strcmp(varname, oneline) == 0){
      found = 1;
      while((len = getline0(fd, oneline, lim_line)) > 0 && !strstr(oneline, "latex:")){}
      if (strstr(oneline, "latex:")){
	ltxname = getrst("latex:", oneline);
	simpltxname = strchp(ltxname, "\\chem{", "}");
      }
    }
  }
  if(simpltxname==NULL){
    ltxname = strdup(varname); simpltxname = strdup("");
  }

  fclose(fd);
  free(ltxname);
  return simpltxname;

}


char *getltx(char **namearray, char **ltarray, int numlt, char *varname)
{
  int i;
  char *ltxname=NULL;
  char *simpltxname=NULL;
  for(i=0;i<numlt;i++){
    if(strcmp(namearray[i], varname)==0){
      ltxname=strdup(ltarray[i]);
      simpltxname = strchp(ltxname, "\\chem{", "}");
      free(ltxname);
      return simpltxname;
    }
  }
  simpltxname = strdup(varname);
  fprintf(stderr, "WARNING in getltx: a latex version of variable \"%s\" was not found.\n", varname);
  return simpltxname;
}

char *getltxname1(char *descfile, char *descfile1, char varname[]){
  char *tmp, *tmp1;
  tmp = getltxname(descfile, varname);
  if(isonlyspace(tmp)){
    free(tmp);
    tmp = getltxname(descfile1, varname);
  }
  tmp1 = cleansq(tmp);
  free(tmp);
  if(isonlyspace(tmp1)){
    fprintf(stderr, "WARNING in getltxname1: variable \"%s\" not found in file \"%s\" or file \"%s\".\n", varname, descfile, descfile1);
    free(tmp1);
    tmp1=strdup(varname);
  }
  return tmp1;
}


void latexview(struct react myreact)
     /* produces latex output which needs the macro twoway and all the other macros */
{
  int i;
  char *temp=NULL, *temp1=NULL, *temp2=NULL, *temp3=NULL, *temp4=NULL, *temp5=NULL, *temp6=NULL, *temp7=NULL;

  printf("\\begin{equation}\n");
  printf("(%s) \\quad", myreact.reacttype);
  if (myreact.comps > 1)
    printf(" (%dC) \\quad ", myreact.comps);
  if (myreact.stoics > 0)
    printf(" (S) \\quad ");

  if(myreact.numleft > 0){
    for(i=0;i<myreact.numleft -1;i++){
      temp = ltx(myreact.varsleft[i]);
      temp1 = ltx(myreact.valsleft[i]);
      if (atoi(myreact.valsleft[i]) != 1)
	printf("\\chem{%s %s} + ", temp1, temp);
      else
	printf("\\chem{%s} + ", temp);
      free(temp);
      free(temp1);
    }

    temp = ltx(myreact.varsleft[i]);
    if (atoi(myreact.valsleft[i]) != 1)
      printf("\\chem{%s %s} ", myreact.valsleft[myreact.numleft-1], temp);
    else
      printf("\\chem{%s} ", temp);
    free(temp);
  }

  /* the bit in the middle */

  if((strcmp(myreact.reacttype, "MA1") == 0)  ||  (strcmp(myreact.reacttype, "elec") == 0) || (strcmp(myreact.reacttype, "use") == 0) ){
    temp = ltx(myreact.rates[0]);
    printf(" \\stackrel\{\\chem{%s}}{\\longrightarrow}  ", temp);
    free(temp);
  }
  else if((strcmp(myreact.reacttype, "MA2") == 0) || (strcmp(myreact.reacttype, "BEMA2") == 0) || (strcmp(myreact.reacttype, "ECMA2") == 0) || (strcmp(myreact.reacttype, "CMMA2") == 0)){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    printf(" \\twoway{%s}{%s}  ", temp, temp1);
    free(temp); free(temp1);
  }
  else if((strcmp(myreact.reacttype, "rateterm") == 0)||(strcmp(myreact.reacttype, "ratetermsimp") == 0)||(strcmp(myreact.reacttype, "simp") == 0)){
    printf(" \\stackrel\{\\chem{}}{\\longrightarrow}  ");

  }
  else if(strcmp(myreact.reacttype, "MM1") == 0 || strcmp(myreact.reacttype, "CEMM1") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    printf(" \\stackrel\{\\chem{%s, \\,\\, %s}}{\\longrightarrow}  ", temp, temp1);
    free(temp); free(temp1);
  }
  else if(strcmp(myreact.reacttype, "MM1sub2") == 0 || strcmp(myreact.reacttype, "ATPshut") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    temp2 = ltx(myreact.rates[2]);
    printf(" \\mmsubtwo{%s}{%s}{%s}  ", temp, temp1, temp2);
    free(temp); free(temp1);free(temp2);
  }
  else if(strcmp(myreact.reacttype, "MM1sub3") == 0 || strcmp(myreact.reacttype, "MAshut") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    temp2 = ltx(myreact.rates[2]);
    temp3 = ltx(myreact.rates[3]);
    printf(" \\mmsubthree{%s}{%s}{%s}{%s}  ", temp, temp1, temp2, temp3);
    free(temp); free(temp1);free(temp2);free(temp3);
  }
  else if(strcmp(myreact.reacttype, "MM1sub4") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    temp2 = ltx(myreact.rates[2]);
    temp3 = ltx(myreact.rates[3]);
    temp4 = ltx(myreact.rates[4]);
    printf(" \\mmsubfour{%s}{%s}{%s}{%s}{%s}  ", temp, temp1, temp2, temp3, temp4);
    free(temp); free(temp1);free(temp2);free(temp3);free(temp4);
  }
  else if(strcmp(myreact.reacttype, "FD") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    temp2 = ltx(myreact.rates[2]);
    printf(" \\twowayMM{%s}{%s, \\,\\,%s}  ", temp, temp1, temp2);
    free(temp); free(temp1);free(temp2);
  }
  else if((strcmp(myreact.reacttype, "MM2") == 0) || (strcmp(myreact.reacttype, "BEMM2") == 0)){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    printf(" \\twowayMM{%s}{%s}  ", temp, temp1);
    free(temp); free(temp1);
  }
  else if(strcmp(myreact.reacttype, "prod") == 0){
    temp = ltx(myreact.rates[0]);
    printf(" \\stackrel\{\\chem{%s}}{\\longrightarrow}  ", temp);
    free(temp);
  }
  else if(strcmp(myreact.reacttype, "coop4") == 0){
    temp = ltx(myreact.rates[0]);
    temp1 = ltx(myreact.rates[1]);
    temp2 = ltx(myreact.rates[2]);
    temp3 = ltx(myreact.rates[3]);
    temp4 = ltx(myreact.rates[4]);
    temp5 = ltx(myreact.rates[5]);
    temp6 = ltx(myreact.rates[6]);
    temp7 = ltx(myreact.rates[7]);
    printf(" \\twowayMM{%s}{%s} \\twowayMM{%s}{%s} \\twowayMM{%s}{%s} \\twowayMM{%s}{%s}  ", temp, temp1, temp2, temp3, temp4, temp5, temp6, temp7);
    free(temp); free(temp1);free(temp2);free(temp3);free(temp4);free(temp5);free(temp6);free(temp7);
  }
  else
    fprintf(stderr, "%s is not a type of reaction.\n", myreact.reacttype);


  if(myreact.numright > 0) {
    temp = ltx(myreact.varsright[0]);
    if (atoi(myreact.valsright[0]) != 1)
      printf("\\chem{%s %s}", myreact.valsright[0], temp);
    else
      printf("\\chem{%s}", temp);
    free(temp);

    for(i=1;i<myreact.numright;i++){
      temp = ltx(myreact.varsright[i]);
      if (atoi(myreact.valsright[i]) != 1)
	printf(" + \\chem{%s %s}", myreact.valsright[i], temp);
      else
	printf(" + \\chem{%s}", temp);
      free(temp);
    }
  }
  printf("\n\\end{equation}\n\n");
}

void latexview1(struct react myreact, char *ltxfile, int withhtml)
{
  // produces latex output which needs the macro twoway and all the other macros. Differs from latexview only in that simpltxname is used to get rid of square brackets. withhtml =0 for no html.sty
  int i;
  char *temp1=NULL, *temp2=NULL, *temp3=NULL, *temp4=NULL, *temp5=NULL, *temp6=NULL, *temp7=NULL, *temp8=NULL;
  char **namearray = NULL;
  char **ltarray = NULL;
  int numnames;
  if(withhtml==0)
    printf("[%s]  ", myreact.reactname);
  else
    printf("\\htmladdnormallink[%sreac]{%s}{#%s} \\begin{rawhtml}&nbsp;&nbsp;&nbsp;&nbsp;\\end{rawhtml}\\htmladdnormallink{(rate)}{reacrates.html#%s_rate}  ", myreact.reactname, myreact.reactname, myreact.reactname, myreact.reactname);

  printf("(%s) ", myreact.reacttype);
  if (myreact.comps > 1)
    printf("(%dC) ", myreact.comps);
  if (myreact.stoics > 0)
    printf("(S)");
  printf("\n");
  printf("\\begin{equation}\n");

  numnames = genarraysdat(ltxfile, &namearray, &ltarray);

  if(myreact.numleft > 0){
    for(i=0;i<myreact.numleft -1;i++){
      temp1 = getltx(namearray, ltarray, numnames, myreact.varsleft[i]);
      if(!(isnumber(myreact.valsleft[i]))){
	temp2=getltx(namearray, ltarray, numnames, myreact.valsleft[i]);
	printf("\\chem{%s %s} + ", temp2, temp1);
	free(temp2);
      }
      else{
	if (atoi(myreact.valsleft[i]) != 1)
	  printf("\\chem{%s %s} + ", myreact.valsleft[i], temp1);
	else
	  printf("\\chem{%s} + ", temp1);
      }
      free(temp1);
    }

    temp1 = getltx(namearray, ltarray, numnames, myreact.varsleft[i]);
    if(!(isnumber(myreact.valsleft[i]))){
      temp2=getltx(namearray, ltarray, numnames, myreact.valsleft[i]);
      printf("\\chem{%s %s} ", temp2, temp1);
      free(temp2);
    }
    else{
      if (atoi(myreact.valsleft[i]) != 1)
	printf("\\chem{%s %s} ", myreact.valsleft[myreact.numleft-1], temp1);
      else
	printf("\\chem{%s} ", temp1);
      free(temp1);
    }
  }

  /* the bit in the middle */

  if((strcmp(myreact.reacttype, "MA1") == 0)  ||  (strcmp(myreact.reacttype, "elec") == 0) ||  (strcmp(myreact.reacttype, "use") == 0) ){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    printf(" \\stackrel\{\\chem{%s}}{\\longrightarrow}  ", temp1);
    free(temp1);

  }
  else if((strcmp(myreact.reacttype, "MA2") == 0) || (strcmp(myreact.reacttype, "BEMA2") == 0) || (strcmp(myreact.reacttype, "ECMA2") == 0) || (strcmp(myreact.reacttype, "CMMA2") == 0)){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    printf(" \\twoway{%s}{%s}  ", temp1, temp2);
    free(temp1);free(temp2);

  }
  else if((strcmp(myreact.reacttype, "rateterm") == 0)||(strcmp(myreact.reacttype, "ratetermsimp") == 0)||(strcmp(myreact.reacttype, "simp") == 0)){
    if((int)(strlen(myreact.rateterm)) < 10){
      temp1 = getltx(namearray, ltarray, numnames, myreact.rateterm);
      printf(" \\stackrel\{\\chem{%s}}{\\longrightarrow}  ", temp1);
      free(temp1);
    }
    else
      printf(" \\stackrel\{\\chem{}}{\\longrightarrow}  ");
  }
  else if(strcmp(myreact.reacttype, "MM1") == 0 || strcmp(myreact.reacttype, "CEMM1") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    printf(" \\stackrel\{\\chem{%s, \\,\\, %s}}{\\longrightarrow}  ", temp1, temp2);
    free(temp1);free(temp2);

  }
  else if(strcmp(myreact.reacttype, "MM1sub2") == 0 || strcmp(myreact.reacttype, "ATPshut") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    temp3 = getltx(namearray, ltarray, numnames, myreact.rates[2]);
    printf(" \\mmsubtwo{%s}{%s}{%s}  ", temp1, temp2, temp3);
    free(temp1);free(temp2);free(temp3);

  }
  else if(strcmp(myreact.reacttype, "MM1sub3") == 0 || strcmp(myreact.reacttype, "MAshut") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    temp3 = getltx(namearray, ltarray, numnames, myreact.rates[2]);
    temp4 = getltx(namearray, ltarray, numnames, myreact.rates[3]);
    printf(" \\mmsubthree{%s}{%s}{%s}{%s}  ", temp1, temp2, temp3, temp4);
    free(temp1);free(temp2);free(temp3);free(temp4);
  }
  else if(strcmp(myreact.reacttype, "MM1sub4") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    temp3 = getltx(namearray, ltarray, numnames, myreact.rates[2]);
    temp4 = getltx(namearray, ltarray, numnames, myreact.rates[3]);
    temp5 = getltx(namearray, ltarray, numnames, myreact.rates[4]);
    printf(" \\mmsubfour{%s}{%s}{%s}{%s}{%s}  ", temp1, temp2, temp3, temp4, temp5);
    free(temp1);free(temp2);free(temp3);free(temp4);free(temp5);

  }
  else if(strcmp(myreact.reacttype, "FD") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    temp3 = getltx(namearray, ltarray, numnames, myreact.rates[2]);
    printf(" \\twowayMM{%s}{%s, \\,\\,%s}  ", temp1, temp2, temp3);
    free(temp1);free(temp2);free(temp3);

  }
  else if((strcmp(myreact.reacttype, "MM2") == 0) || (strcmp(myreact.reacttype, "BEMM2") == 0)){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    printf(" \\twowayMM{%s}{%s}  ", temp1, temp2);
    free(temp1);free(temp2);

  }
  else if(strcmp(myreact.reacttype, "prod") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    printf(" \\stackrel\{\\chem{%s}}{\\longrightarrow}  ", temp1);
    free(temp1);
  }
  else if(strcmp(myreact.reacttype, "coop4") == 0){
    temp1 = getltx(namearray, ltarray, numnames, myreact.rates[0]);
    temp2 = getltx(namearray, ltarray, numnames, myreact.rates[1]);
    temp3 = getltx(namearray, ltarray, numnames, myreact.rates[2]);
    temp4 = getltx(namearray, ltarray, numnames, myreact.rates[3]);
    temp5 = getltx(namearray, ltarray, numnames, myreact.rates[4]);
    temp6 = getltx(namearray, ltarray, numnames, myreact.rates[5]);
    temp7 = getltx(namearray, ltarray, numnames, myreact.rates[6]);
    temp8 = getltx(namearray, ltarray, numnames, myreact.rates[7]);
    printf(" \\twowayMM{%s}{%s} \\twowayMM{%s}{%s} \\twowayMM{%s}{%s} \\twowayMM{%s}{%s}  ", temp1, temp2, temp3, temp4, temp5, temp6, temp7, temp8);
    free(temp1);free(temp2);free(temp3);free(temp4);free(temp5);free(temp6);free(temp7);free(temp8);
  }
  else
    fprintf(stderr, "%s is not a type of reaction.\n", myreact.reacttype);

  if(myreact.numright > 0) {
    temp1 = getltx(namearray, ltarray, numnames, myreact.varsright[0]);
    if(!(isnumber(myreact.valsright[0]))){
      temp2=getltx(namearray, ltarray, numnames, myreact.valsright[0]);
      printf("\\chem{%s %s}", temp2, temp1);
      free(temp2);
    }
    else{
      if (atoi(myreact.valsright[0]) != 1)
	printf("\\chem{%s %s}", myreact.valsright[0], temp1);
      else
	printf("\\chem{%s}", temp1);
    }
    free(temp1);


    for(i=1;i<myreact.numright;i++){
      temp1 = getltx(namearray, ltarray, numnames, myreact.varsright[i]);
      if(!(isnumber(myreact.valsright[i]))){
	temp2=getltx(namearray, ltarray, numnames, myreact.valsright[i]);
	printf("+ \\chem{%s %s}", temp2, temp1);
	free(temp2);
      }
      else{
	if (atoi(myreact.valsright[i]) != 1)
	  printf(" + \\chem{%s %s}", myreact.valsright[i], temp1);
	else
	  printf(" + \\chem{%s}", temp1);
      }
      free(temp1);
    }
  }


  printf("\n\\end{equation}\n");
  freearraydat(namearray, numnames);
  freearraydat(ltarray, numnames);
  return;
}


void quickview(struct react myreact)
     /* produces text output for quick viewing */
{
  int i;
  printf("\n");
  if(myreact.numleft > 0){
    for(i=0;i<myreact.numleft -1;i++){
      if (atoi(myreact.valsleft[i]) != 1)
	printf("%s%s + ", myreact.valsleft[i], myreact.varsleft[i]);
      else
	printf("%s + ", myreact.varsleft[i]);
    }
    if (atoi(myreact.valsleft[i]) != 1)
      printf("%s%s \t", myreact.valsleft[i], myreact.varsleft[i]);
    else
      printf("%s \t", myreact.varsleft[i]);
  }

  if (myreact.twoway == 2)
    printf("-->(%s)<--(%s) \t", myreact.rates[0], myreact.rates[1]);
  else
    printf("-->(%s) \t", myreact.rates[0]);

  if(myreact.numright > 0){
    if (atoi(myreact.valsright[0]) != 1)
      printf("%s%s", myreact.valsright[0], myreact.varsright[0]);
    else
      printf("%s", myreact.varsright[0]);


    for(i=1;i<myreact.numright;i++){
      if (atoi(myreact.valsright[i]) != 1)
	printf(" + %s%s", myreact.valsright[i], myreact.varsright[i]);
      else
	printf(" + %s", myreact.varsright[i]);
    }
  }
  printf("\n\n");
}

char *ltx1(char *t)
     /* replaces an initial "_" in string t with "\\overline " */
{
  char *tmp, *tmp1, *s;
  tmp = lrtrim(t);
  if(tmp[0] == '_'){
    tmp1=strcut(tmp, 1);
    s = (char*) malloc(sizeof(char) * (strlen(tmp1) + 12));
    sprintf(s, "\\overline{%s}", tmp1); // safe
    free(tmp1);
  }
  else 
    s = strdup(tmp);
  free(tmp);
  return s;
}

char *transltx(char *varname){
  char *tmp, *tmp1;
  int i=0, j=0, c;
  int lcount=0;
  int inmid=0;
  tmp = ltx1(varname);
  while((c = tmp[i++]) != 0){
    if (c == '_')
      lcount++;
  }
  tmp1 = (char*) malloc(sizeof(char) * (strlen(tmp) + 2*lcount + 1));
  i=0;
  while((c = tmp[i++]) != 0){
    if (c == '_'){
      inmid++;
      tmp1[j++]=c;
      tmp1[j++]='{';
    }
    else if(inmid>0 && !isalnum(c)){
      inmid--;
      tmp1[j++]='}';
      tmp1[j++]=c;
    }
    else{
      tmp1[j++]=c;
    }
  }
  while(inmid>0){
    tmp1[j++]='}';
    inmid--;
  }
  tmp1[j++]=0;
  return tmp1;
}



char *bracket(char t[])
     /* searches string t for "_". When it is found, changes to "_{", closes all brackets, and puts the result in string s */

{
  int i=0, j=0, k=0;
  char *s;
  while(t[i] != '\0'){
    if ((t[i] == '_') && (t[i+1] != '{'))
      k++;i++;
  }
  s = (char*) malloc(sizeof(char) * (strlen(t) + 2*k + 1));
  i=0;j=0;

  while (t[i] != '\0'){
    if ((t[i] == '_') && (t[i+1] != '{')){
      s[j++] = t[i++];
      s[j++] = '{';
    }
    else{s[j++] = t[i++];}
  }
  i=0;
  while(i < k){s[j++] = '}';i++;}
  s[j] = '\0';
  return s;

}

char *ltx(char *t)
{
  char *tmp, *s;
  tmp = ltx1(t);
  s = bracket(tmp);
  free(tmp);
  return s;
}


char *strcut(char *t, int n)
     /* cuts off the first n characters in string t and returns the result in string s */
{
  char *tmp;
  tmp = strdup(t+n);
  return tmp;
}

//
//
// reaction handling
//
//

struct react reactcopy(struct react react1)
     /* copies a structure inlcuding assigning memory - watch out for this! */
{
  int i;
  struct react temp;
  temp.reactname = strdup(react1.reactname);
  temp.reacttype = strdup(react1.reacttype);
  temp.comps = react1.comps;
  temp.stoics = react1.stoics;
  temp.numleft = react1.numleft;
  temp.numright = react1.numright;
  temp.twoway = react1.twoway;
  temp.varsleft = charar(0, react1.numleft-1);
  temp.valsleft = charar(0, react1.numleft-1); 
  temp.varsright = charar(0, react1.numright-1);
  temp.valsright = charar(0, react1.numright-1);
  temp.rates = charar(0, react1.twoway-1);
  temp.compsleft = charar(0, react1.numleft-1);
  temp.compsright = charar(0, react1.numright-1);
  temp.stoicsleft = charar(0, react1.numleft-1);
  temp.stoicsright = charar(0, react1.numright-1);

  for (i=0;i<react1.numleft;i++){
    temp.varsleft[i] = strdup(react1.varsleft[i]);
    temp.valsleft[i] = strdup(react1.valsleft[i]);
    if(react1.comps > 1){
      temp.compsleft[i] = strdup(react1.compsleft[i]);
    }
    if(react1.stoics > 0){
      temp.stoicsleft[i] = strdup(react1.stoicsleft[i]);
    }
  }

  for (i=0;i<react1.numright;i++){
    temp.varsright[i] = strdup(react1.varsright[i]);
    temp.valsright[i] = strdup(react1.valsright[i]);
    if(react1.comps > 1){
      temp.compsright[i] = strdup(react1.compsright[i]);
    }
    if(react1.stoics > 0){
      temp.stoicsright[i] = strdup(react1.stoicsright[i]);
    }
  }

  for (i=0;i<react1.twoway;i++)
    temp.rates[i] = strdup(react1.rates[i]);

  temp.rateterm = strdup(react1.rateterm);
  return temp;
}

void free_reac(struct react react1){
  int i;
  free(react1.reactname);
  free(react1.reacttype);
  free(react1.rateterm);

  for(i=0;i<react1.numleft;i++){
    free(react1.varsleft[i]);
    free(react1.valsleft[i]);
    if(react1.comps > 1){free(react1.compsleft[i]);}
    if(react1.stoics > 0){free(react1.stoicsleft[i]);}
  }

  for(i=0;i<react1.numright;i++){
    free(react1.varsright[i]);
    free(react1.valsright[i]);
    if(react1.comps > 1){free(react1.compsright[i]);}
    if(react1.stoics > 0){free(react1.stoicsright[i]);}
  }
  for (i=0;i<react1.twoway;i++)
    free(react1.rates[i]);

  free_charar(react1.varsleft, 0, react1.numleft-1);
  free_charar(react1.valsleft, 0, react1.numleft-1); 
  free_charar(react1.varsright, 0, react1.numright-1);
  free_charar(react1.valsright, 0, react1.numright-1);
  free_charar(react1.rates, 0, react1.twoway-1);

  if(react1.stoics != 0){
    free_charar(react1.stoicsleft, 0, react1.numleft-1);
    free_charar(react1.stoicsright, 0, react1.numright-1);
  }
  if(react1.comps > 1){
    free_charar(react1.compsleft, 0, react1.numleft-1);
    free_charar(react1.compsright, 0, react1.numright-1);
  }

}

void free_reacarray(struct react reactarray[], int numreacs){
  while(--numreacs >= 0)
    free_reac(reactarray[numreacs]);
}

// read in an algebraic relation in the form of a string, parse it, and create an algrel structure. Includes allocating memory. 

int readreac(struct react *tempreac, char *block)
{
  int j, nrexpt=0;
  char *line;
  char *oneline;
  int maxgets = 100;
  char *chems1[100];
  char *chems2[100];
  int numleftval, numrightval, numstoicsval, numratesval, numcompsval;
  int retval=0;
  int compsval;
  int stoicsval;
 

  //
  // name
  //
  line = getname("name:", block);
  if(strcmp(line, "") == 0){ // not a reaction
    free(line);
    return -1;
  }
  tempreac->reactname = strdup(line);
  free(line);
  //
  // type
  //
  line = getname("type:", block);
  if(strcmp(line, "") == 0){ // default
    free(line);
    line = strdup("rateterm");
  }
  tempreac->reacttype = strdup(line);
  free(line);
  //
  // comps
  //

  line = getname("comps:", block);
  if(strcmp(line, "") == 0){ // defaults to one
    free(line);
    line = strdup("1");
  }
  compsval = atoi(line);
  free(line);
  tempreac->comps = compsval;


  //
  // stoics
  //

  line = getname("stoics:", block);
  if(strcmp(line, "") == 0){ // default to 0
    free(line);
    line = strdup("0");
  }
  stoicsval = atoi(line);
  free(line);
  tempreac->stoics = stoicsval;

  //
  // left
  //
      
  oneline = getrst("left:", block);
  numleftval = chemgets1(oneline, chems1, chems2, maxgets);
  free(oneline);
  tempreac->numleft = numleftval;
  tempreac->varsleft = charar(0, numleftval-1);
  tempreac->valsleft = charar(0, numleftval-1);
  if(compsval > 1)
    tempreac->compsleft = charar(0, numleftval-1);
  if(stoicsval != 0)
    tempreac->stoicsleft = charar(0, numleftval-1);
  for (j=0;j<numleftval;j++){
    tempreac->valsleft[j] = strdup(chems1[j]);
    tempreac->varsleft[j] = strdup(chems2[j]);
  }
  free_datarray(chems1, numleftval);
  free_datarray(chems2, numleftval);

  //
  // right
  //

  oneline = getrst("right:", block);
  numrightval = chemgets1(oneline, chems1, chems2, maxgets);
  free(oneline);
  tempreac->numright = numrightval;
  tempreac->varsright = charar(0, numrightval-1);
  tempreac->valsright = charar(0, numrightval-1);
  if(compsval > 1){
    tempreac->compsright = charar(0, numrightval-1);
  }
  if(stoicsval != 0){
    tempreac->stoicsright = charar(0, numrightval-1);
  }
  for (j=0;j<numrightval;j++){
    tempreac->valsright[j] = strdup(chems1[j]);
    tempreac->varsright[j] = strdup(chems2[j]);
  }
  free_datarray(chems1, numrightval);
  free_datarray(chems2, numrightval);

  //
  // stoics
  //

  if(stoicsval != 0){
    oneline = getrst("stoicsleft:", block);
    numstoicsval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    if (numstoicsval != numleftval){
      fprintf(stderr, "ERROR in readreac: number of stoics does not match number of leftchems for %s.\n", tempreac->reactname);
      retval=-2;
      for (j=0;j<numleftval;j++)
	tempreac->stoicsleft[j] = strdup("");
    }
    else{
      for (j=0;j<numleftval;j++)
	tempreac->stoicsleft[j] = strdup(chems1[j]);
    }
    free_datarray(chems1, numstoicsval);

    oneline = getrst("stoicsright:", block);
    numstoicsval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    if (numstoicsval != numrightval){
      fprintf(stderr, "ERROR in readreac: number of stoics does not match number of rightchems for %s.\n", tempreac->reactname);
	retval=-2;
      for (j=0;j<numrightval;j++)
	tempreac->stoicsright[j] = strdup("");
    }
    else{
      for (j=0;j<numrightval;j++)
	tempreac->stoicsright[j] = strdup(chems1[j]);
    }
    free_datarray(chems1, numstoicsval);

  }

  //
  // comps
  //

  if(compsval > 1){
    oneline = getrst("compsleft:", block);
    numcompsval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    if (numcompsval != numleftval){
      fprintf(stderr, "ERROR in readreac: number of comps does not match number of leftchems for %s.\n", tempreac->reactname);
      retval=-2;
      for (j=0;j<numleftval;j++)
	tempreac->compsleft[j] = strdup("");
    }
    else{
      for (j=0;j<numleftval;j++)
	tempreac->compsleft[j] = strdup(chems1[j]);
    }
    free_datarray(chems1, numcompsval);

    oneline = getrst("compsright:", block);
    numcompsval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    if (numcompsval != numrightval){
      fprintf(stderr, "ERROR in readreac: number of comps does not match number of rightchems for %s.\n", tempreac->reactname);
      retval=-2;
    }
    else{
      for (j=0;j<numrightval;j++){
	tempreac->compsright[j] = strdup(chems1[j]);
      }
    }
    free_datarray(chems1, numcompsval);

  }

  //
  // rates
  //

  oneline = getrst("rates:", block);
  numratesval = chemgets(oneline, chems1, maxgets);
  free(oneline);
  tempreac->twoway = numratesval;
  tempreac->rates = charar(0, numratesval-1);

  if(strcmp(tempreac->reacttype, "MA2") == 0){nrexpt = 2;}
  else if(strcmp(tempreac->reacttype, "MA1") == 0){nrexpt = 1;}
  else if(strcmp(tempreac->reacttype, "MM1") == 0){nrexpt = 2;}
  else if(strcmp(tempreac->reacttype, "MM1sub2") == 0){nrexpt = 3;}
  else if(strcmp(tempreac->reacttype, "MM1sub3") == 0){nrexpt = 4;}
  else if(strcmp(tempreac->reacttype, "MM1sub4") == 0){nrexpt = 5;}
  else if(strcmp(tempreac->reacttype, "MM1sub5") == 0){nrexpt = 6;}
  else if(strcmp(tempreac->reacttype, "rateterm") == 0){nrexpt = 1;}
  else if(strcmp(tempreac->reacttype, "prod") == 0){nrexpt = 1;}
  else if(strcmp(tempreac->reacttype, "FD") == 0){nrexpt = 3;}
  else if(strcmp(tempreac->reacttype, "ratetermsimp") == 0|| strcmp(tempreac->reacttype, "simp") == 0){nrexpt = 0;}

  if(tempreac->twoway<nrexpt){
    fprintf(stderr, "ERROR in readreac: reaction \"%s\": A reaction of type %s must have %d rate(s) in the \"rates:\" field.\n", tempreac->reactname, tempreac->reacttype, nrexpt);
    retval=-2;
  }
  else if(tempreac->twoway>nrexpt){
    fprintf(stderr, "WARNING in readreac: reaction \"%s\": A reaction of type %s must only have %d rate(s) in the \"rates:\" field.\n", tempreac->reactname, tempreac->reacttype, nrexpt);
  }

  for (j=0;j<numratesval;j++){
    tempreac->rates[j] = strdup(chems1[j]);
  }
  free_datarray(chems1, numratesval);

  //
  // rateterm
  //

  oneline = getrst("rateterm:", block);
  if(strcmp(oneline, "") != 0){
    checkbracs(oneline);
    tempreac->rateterm = strdup(oneline);
  }
  else
    tempreac->rateterm = strdup("");
  free(oneline);

  if((strcmp(tempreac->reacttype, "rateterm")==0 || strcmp(tempreac->reacttype, "ratetermsimp")==0) && isonlyspace(tempreac->rateterm)){
    fprintf(stderr, "ERROR in readreac: reaction \"%s\": A reaction of type %s must have a rate term.\n", tempreac->reactname, tempreac->reacttype);
    retval=-2;
  }

  if(retval!=0){ // failure: free memory
    free(tempreac->reactname);
    free(tempreac->reacttype);
    free(tempreac->rateterm);
    for (j=0;j<numleftval;j++){
      free(tempreac->valsleft[j]);
      free(tempreac->varsleft[j]);
    }
    free_charar(tempreac->varsleft, 0, numleftval-1);
    free_charar(tempreac->valsleft, 0, numleftval-1);

    for (j=0;j<numrightval;j++){
      free(tempreac->valsright[j]);
      free(tempreac->varsright[j]);
    }
    free_charar(tempreac->varsright, 0, numrightval-1);
    free_charar(tempreac->valsright, 0, numrightval-1);

    for (j=0;j<numratesval;j++)
      free(tempreac->rates[j]);
    free_charar(tempreac->rates, 0, numratesval-1);
    if(stoicsval!=0){
      for (j=0;j<numleftval;j++)
	free(tempreac->stoicsleft[j]);
      free_charar(tempreac->stoicsleft, 0, numleftval-1);
      for (j=0;j<numrightval;j++)
	free(tempreac->stoicsright[j]);
      free_charar(tempreac->stoicsright, 0, numrightval-1);
    }
    if(compsval>1){
      for (j=0;j<numleftval;j++)
	free(tempreac->compsleft[j]);
      free_charar(tempreac->compsleft, 0, numleftval-1);
      for (j=0;j<numrightval;j++)
	free(tempreac->compsright[j]);
      free_charar(tempreac->compsright, 0, numrightval-1);
    }
  }
  return retval;
}

// read a file of reactions (none commented out) into an array of reactions

int readreacfile(struct react reacarray[], int max, char *filename)
{
  int len;
  int i=0;
  char block[10000];
  FILE *fd;
  int retstat;

  fd = fopen(filename, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine readreacfile: \"%s\" could not be opened...\n", filename);
    return -1;
  }

  while((i< max) && (len = getblock(fd, block, "******", 10000)) > 0){
    if((retstat=readreac(&(reacarray[i]), block)) == 0){ // successful read
      i++;
    }
    if(retstat==-2){
      fprintf(stderr, "ERROR in readreacfile - a reaction could not be read. Now exiting.\n");
      fclose(fd);
      free_reacarray(reacarray, i);
      return -2;
    }
    if(i==max){
      fprintf(stderr, "ERROR in readreacfile - maximum number of reactions - %d - reached. Now exiting.\n", i);
      fclose(fd);
      free_reacarray(reacarray, i);
      return -2;
    }

  }

  fclose(fd);
  return i;

}



struct DEterm DEtermcopy(struct DEterm DEterm1)
     /* copies a structure inlcuding assigning memory - watch out for this! */
{
  struct DEterm temp;
  temp.name = strdup(DEterm1.name);
  temp.keyvar = strdup(DEterm1.keyvar);
  temp.rterm = strdup(DEterm1.rterm);
  return temp;
}

void free_DEterm(struct DEterm DEterm1){
  free(DEterm1.name);
  free(DEterm1.keyvar);
  free(DEterm1.rterm);
}

void free_DEtermarray(struct DEterm DEtermarray[], int numDEterms){
  while(--numDEterms >= 0)
    free_DEterm(DEtermarray[numDEterms]);
}



// read in a DE term in the form of a string, parse it, and create a DEterm structure. Includes allocating memory. 

int readDEterm(struct DEterm *tempDEterm, char *block)
{
  char *line;


  // name
  line = getname("name:", block);
  if(strcmp(line, "") != 0){
    tempDEterm->name = strdup(line);
    free(line);
    // keyvar
    line = getname("keyvar:", block);
    tempDEterm->keyvar = strdup(line);
    free(line);

    // the term...
    line = getrst("term:", block);
    checkbracs(line);
    tempDEterm->rterm = strdup(line);
    free(line);
  } 
  else // not a DEterm...
    return -1;

  return 0;
}

// read a file of DEterms (none commented out) into an array of DEterms.

int readDEtermfile(struct DEterm DEtermarray[], int max, char *filename)
{
  int len;
  int i=0;
  char block[10000];
  FILE *fd;

  fd = fopen(filename, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine readDEtermfile: \"%s\" could not be opened...\n", filename);
    return -1;
  }

  while((len = getblock(fd, block, "******", 10000)) > 0){
    if(readDEterm(&(DEtermarray[i]), block) == 0){ // successful read
      i++;
    }
    if(i==max){
      fprintf(stderr, "ERROR in readDEtermfile - maximum number of DEterms - %d - reached. Now exiting.\n", i);
      fclose(fd);
      return -2;
    }

  }

  fclose(fd);
  return i;

}


//
//
// algrel handling
//
//

struct algrel algrelcopy(struct algrel algrel1)
     /* copies a structure inlcuding assigning memory - watch out for this! */
{
  int i;
  struct algrel temp;
  temp.name = strdup(algrel1.name);
  temp.keyvar = strdup(algrel1.keyvar);
  temp.numvars = algrel1.numvars;
  temp.numparams = algrel1.numparams;

  temp.vars = charar(0, algrel1.numvars-1);
  temp.params = charar(0, algrel1.numparams-1);

  for (i=0;i<algrel1.numvars;i++)
    temp.vars[i] = strdup(algrel1.vars[i]);


  for (i=0;i<algrel1.numparams;i++)
    temp.params[i] = strdup(algrel1.params[i]);

  temp.rterm = strdup(algrel1.rterm);

  return temp;
}

void free_algrel(struct algrel algrel1){
  int i;
  free(algrel1.name);
  free(algrel1.keyvar);
  free(algrel1.rterm);

  for(i=0;i<algrel1.numvars;i++){
    free(algrel1.vars[i]);
  }
  free_charar(algrel1.vars, 0, algrel1.numvars);

  for(i=0;i<algrel1.numparams;i++){
    free(algrel1.params[i]);
  }
  free_charar(algrel1.params, 0, algrel1.numparams);

}


void free_algarray(struct algrel algarray[], int numalgs){
  while(--numalgs >= 0)
    free_algrel(algarray[numalgs]);
}



// read in an algebraic relation in the form of a string, parse it, and create an algrel structure. Includes allocating memory. 

int readalgrel(struct algrel *tempalg, char *block, char *vararray[], int numvars, char *allparamarray[], int numallparams, char *rems[], int numrems)
{
  int k;
  char *oneline;
  int maxgets = 100;
  char **chms=NULL;
  char *chems1[100];
  char *chems2[100];
  char *chems3[100];
  int numparamsval, numvarsval, numpval, numallval;
  char *line;
 
  //
  // name
  //
  line = getname("name:", block);
  if(strcmp(line, "") != 0){
    tempalg->name = strdup(line);
    free(line);

    //
    // keyvar
    //

    line = getname("keyvar:", block);
    tempalg->keyvar = strdup(line);
    free(line);

    //
    // params and othervars
    //

    numparamsval = 0;
    numvarsval = 0;

    // first any extra parameters

    oneline = getrst("params:", block);
    numpval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    for(k=0;k<numpval;k++){
      if(isinarray(allparamarray, numallparams, chems1[k]) >= 0){
	numparamsval = addvar1(numparamsval, chems1[k], chems3, maxgets);
      }
    }
    free_datarray(chems1, numpval);

    // the eqn...
    oneline = getrst("eqn:", block);
    checkbracs(oneline);
    tempalg->rterm = strdup(oneline);
    chms = getvs(oneline, &numallval, rems, numrems);
    free(oneline);
    for(k=0;k<numallval;k++){
      if(isinarray(vararray, numvars, chms[k]) >= 0){
	numvarsval = addvar1(numvarsval, chms[k], chems2, maxgets);
      }
      else if(isinarray(allparamarray, numallparams, chms[k]) >= 0){
	numparamsval = addvar1(numparamsval, chms[k], chems3, maxgets);
      }
      else { // could be a temporary variable
	//	fprintf(stderr, "WARNING in routine readalgrel, %s is neither a parameter nor a variable.\n", chms[k]);
      }
    }
    freearraydat(chms, numallval);

    tempalg->numparams = numparamsval;
    tempalg->params = charar(0, numparamsval-1);
    for (k=0;k<numparamsval;k++)
      tempalg->params[k] = strdup(chems3[k]);

    tempalg->vars = charar(0, numvarsval-1);

    for (k=0;k<numvarsval;k++)
      tempalg->vars[k] = strdup(chems2[k]);

  } 
  else // not an algrel...
    return -1;

  free_datarray(chems2, numvarsval);
  free_datarray(chems3, numparamsval);

  return 0;
}

// read a file of algebraic relations (none commented out) into an array of algrels.

int readalgfile(struct algrel algarray[], int max, char *filename, char *varfile, char *allparamfile, char *remsfile)
{
  int len;
  int i=0;
  char block[10000];
  FILE *fd;

  int numrems=0;
  char **rems;

  char **vararray;
  int numvars=0;
  char **allparamarray;
  int numallparams=0;

  vararray = genarraydat(varfile, &numvars);
  allparamarray = genarraydat(allparamfile, &numallparams);
  rems = genarraydat(remsfile, &numrems);

  fd = fopen(filename, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine readalgfile: \"%s\" could not be opened...\n", filename);
    return -1;
  }

  while((len = getblock(fd, block, "******", 10000)) > 0){
    if(readalgrel(&(algarray[i]), block, vararray, numvars, allparamarray, numallparams, rems, numrems) == 0){ // successful read
      i++;
    }
    if(i==max){
      fprintf(stderr, "ERROR in readalgfile - maximum number of algrelss - %d - reached. Now exiting.\n", i);
      fclose(fd);
      return -2;
    }

  }

  freearraydat(vararray, numvars);
  freearraydat(allparamarray, numallparams);
  freearraydat(rems, numrems);

  fclose(fd);
  return i;

}




//
//
// DE handling
//
//

struct DE DEcopy(struct DE DE1)
     /* copies a structure inlcuding assigning memory - watch out for this! */
{
  int i;
  struct DE temp;
  temp.name = strdup(DE1.name);
  temp.numleft = DE1.numleft;
  temp.numright = DE1.numright;
  temp.keyvar = strdup(DE1.keyvar);
  temp.numparams = DE1.numparams;

  temp.varsleft = charar(0, DE1.numleft-1);
  temp.valsleft = charar(0, DE1.numleft-1);
  temp.varsright = charar(0, DE1.numright-1);
  temp.params = charar(0, DE1.numparams-1);

  temp.rterm = strdup(DE1.rterm);

  for(i=0; i<DE1.numleft; i++){
    temp.varsleft[i] = strdup(DE1.varsleft[i]);
    temp.valsleft[i] = strdup(DE1.valsleft[i]);
  }

  for(i=0; i<DE1.numright; i++)
    temp.varsright[i] = strdup(DE1.varsright[i]);


  for(i=0; i<DE1.numparams; i++)
    temp.params[i] = strdup(DE1.params[i]);

  return temp;
}

// free a DE which has had memory allocated.

void free_DE(struct DE ODE){
  int i;
  free(ODE.name);
  free(ODE.keyvar);
  free(ODE.rterm);

  for(i=0;i<ODE.numleft;i++){
    free(ODE.varsleft[i]);
    free(ODE.valsleft[i]);
  }
  free_charar(ODE.varsleft, 0, ODE.numleft);
  free_charar(ODE.valsleft, 0, ODE.numleft);

  for(i=0;i<ODE.numright;i++){
    free(ODE.varsright[i]);
  }
  free_charar(ODE.varsright, 0, ODE.numright);

  for(i=0;i<ODE.numparams;i++){
    free(ODE.params[i]);
  }
  free_charar(ODE.params, 0, ODE.numparams);


}

void free_DEarray(struct DE DEarray[], int numDEs){
  while(--numDEs >= 0)
    free_DE(DEarray[numDEs]);
}


// take a DE in the form of a string, parse it, and create a DE structure. Includes allocating memory. 

int readDE(struct DE *tempDE, char *block, char *vararray[], int numvars, char *allparamarray[], int numallparams, char *rems[], int numrems)
{
  int k;
  char *oneline;
  int defvar;
  int maxgets = 100;
  char **chms;
  char *chems1[100];
  char *chems2[100];
  char *chems3[100];
  int numleftval, numparamsval, numvarsval, numpval, numallval;
  char *line;

  defvar = 0;
  //
  // name
  //
  line = getname("name:", block);
  if(strcmp(line, "") != 0){
    tempDE->name = strdup(line);
    free(line);

    //
    // keyvar
    //

    line = getname("keyvar:", block);
    if(strcmp(line, "") == 0){
      defvar=1;
    }
    else{
      tempDE->keyvar = strdup(line);
    }
    free(line);

    //
    // lterm and rterm
    //

    oneline = getrst("lterm:", block);
    numleftval = chemgets1(oneline, chems1, chems2, maxgets);
    free(oneline);
    if(defvar == 1){
      tempDE->keyvar = strdup(chems2[0]);
    }
    tempDE->numleft = numleftval;

    tempDE->varsleft = charar(0, numleftval-1);
    tempDE->valsleft = charar(0, numleftval-1);

    for (k=0;k<numleftval;k++){
      tempDE->valsleft[k] = strdup(chems1[k]);
      tempDE->varsleft[k] = strdup(chems2[k]);
    }
    free_datarray(chems1, numleftval);
    free_datarray(chems2, numleftval);

    //
    // params and othervars
    //

    numparamsval = 0;
    numvarsval = 0;

    // first any extra ones

    oneline = getrst("params:", block);
    numpval = chemgets(oneline, chems1, maxgets);
    free(oneline);
    for(k=0;k<numpval;k++){
      if(isinarray(allparamarray, numallparams, chems1[k]) >= 0){
	numparamsval = addvar1(numparamsval, chems1[k], chems3, maxgets);
      }
    }
    free_datarray(chems1, numpval);

    // now the others...

    oneline = getrst("rterm:", block);
    checkbracs(oneline);
    tempDE->rterm = strdup(oneline);
    chms = getvs(oneline, &numallval, rems, numrems);
    free(oneline);
    for(k=0;k<numallval;k++){
      if(isinarray(vararray, numvars, chms[k]) >= 0){
	numvarsval = addvar1(numvarsval, chms[k], chems2, maxgets);
      }
      else if(isinarray(allparamarray, numallparams, chms[k]) >= 0){
	numparamsval = addvar1(numparamsval, chms[k], chems3, maxgets);
      }
      else {// could be a temporary variable
	//	fprintf(stderr, "ERROR in routine readDE, %s is neither a parameter nor a variable.\n", chms[k]);
      }
    }
    freearraydat(chms, numallval);

    tempDE->numparams = numparamsval;
    tempDE->params = charar(0, numparamsval-1);
    for (k=0;k<numparamsval;k++)
      tempDE->params[k] = strdup(chems3[k]);

    tempDE->numright = numvarsval;
    tempDE->varsright = charar(0, numvarsval-1);

    for (k=0;k<numvarsval;k++)
      tempDE->varsright[k] = strdup(chems2[k]);
  } 
  else // not a valid DE
    return -1;

  free_datarray(chems2, numvarsval);
  free_datarray(chems3, numparamsval);

  return 0;
}

// read a file of differential equations (none commented out) into an array of DEs.

int readDEfile(struct DE DEarray[], int max, char *filename, char *varfile, char *allparamfile, char *remsfile)
{
  int len;
  int i=0;
  char block[10000];
  FILE *fd;

  int numrems=0;
  char **rems;

  char **vararray;
  int numvars=0;
  char **allparamarray;
  int numallparams=0;

  vararray = genarraydat(varfile, &numvars);
  allparamarray = genarraydat(allparamfile, &numallparams);
  rems = genarraydat(remsfile, &numrems);
 

  fd = fopen(filename, "r");
  if(!fd){
    fprintf(stderr, "ERROR in routine readDEfile: \"%s\" could not be opened...\n", filename);
    return -1;
  }

  while((len = getblock(fd, block, "******", 10000)) > 0){
    if(readDE(&(DEarray[i]), block, vararray, numvars, allparamarray, numallparams, rems, numrems) == 0)
      i++;
    if(i==max){
      fprintf(stderr, "ERROR in readDEfile - maximum number of DEs - %d - reached. Now exiting.\n", i);
      fclose(fd);
      return -2;
    }

  }

  fclose(fd);

  freearraydat(vararray, numvars);
  freearraydat(allparamarray, numallparams);
  freearraydat(rems, numrems);

  return i;

}


void printchems(int numreacs, struct react allreacs[])
     /* Read a reaction array and print out a list of all the chemicals involved in it */
{
  char *chems[2000];
  int i, j;
  int numchems=0;

  for(i=0;i<numreacs;i++){
    for(j=0;j<allreacs[i].numleft;j++){
      if(isinarray(chems, numchems, allreacs[i].varsleft[j])<=0){
	if(numchems==2000){
	  fprintf(stderr, "ERROR in printchems: More than 2000 chemicals. Exiting.\n");
	  free_datarray(chems, numchems);return;
	}
	chems[numchems++]=strdup(allreacs[i].varsleft[j]);
      }
    }
    for(j=0;j<allreacs[i].numright;j++){
      if(isinarray(chems, numchems, allreacs[i].varsright[j])<=0){
	if(numchems==2000){
	  fprintf(stderr, "ERROR in printchems: More than 2000 chemicals. Exiting.\n");
	  free_datarray(chems, numchems);return;
	}
	chems[numchems++]=strdup(allreacs[i].varsright[j]);
      }
    }
  }
  printf("\nThere are %d chemicals. These are:\n", numchems);
  for(i=0;i<numchems;i++)
    printf("%s\n", chems[i]);
  printf("\n");
  free_datarray(chems, numchems);
  return;

}

//
// routines for output datafiles
//


//
// get all the variables stored in a data file
//


int getdatfileindex(char *fname, char *vname){
  FILE *fd;
  char **vararray;
  int i, k, c=0, l;
  char *line;
  int numvars;
  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in getdatfileindex: file %s could not be opened.\n", fname);
    return -3;
  }
  i=0;
  while((c=getc(fd)) != EOF && c != '\n'){i++;}
  line = (char*) malloc(sizeof(char) * (i+2));
  rewind(fd);
  getline(fd, line, i+2);
  fclose(fd);
  i=0;
  while(isspace(line[i])){i++;}
  if(line[i]!='#' && line[i]!='%' && line[i]!='/' && line[i]!='*'){
    fprintf(stderr, "ERROR in getdatfileindex: first line of %s suggests it is not a valid datafile.\n", fname);
    free(line);
    return -1;
  }

  k=getnumallvars(line+i+1);
  vararray = charar(0, k-1);
  numvars = getallvars(line, vararray, k);
  free(line);
  l = isinarray(vararray, numvars, vname);
  freearraydat(vararray, numvars);
  if(l>=0)
    return l+1;
  else
    return -1;

}



int getdatafilevals(char fname[], int varindex, double varval, double max_error, int varindex1[], int numvars, double varvals[])
{
  // takes output file fname, searches its varindexth column entries for the first one that is close to varval with a difference less than max_error and returns the value of the variable with index varindex1
  FILE *fd;
  int i=0;
  char temp[2000];
  char tempword[100];
  char tempword1[100];
  int len;

  fd = fopen(fname, "r");
  if (!fd){
    fprintf(stderr, "ERROR in getdatafilevals: File %s can't be opened\n", fname);
    return 0; //exit: datafile not found
  }
  while((len = getline(fd, temp, 2000)) > 0){ // max length 2000
    i=0;
    while(isspace(temp[i++])){} // ignore white space
    if((temp[i] == '#') || (temp[i] == '%') || (temp[i] == '/') || (temp[i] == '*')){} // ignore comment lines
    else{
      getnthtoken(temp, tempword, 100, varindex);
      if(fabs(atof(tempword) - varval) < max_error){
	for(i=0;i<numvars;i++){
	  getnthtoken(temp, tempword1, 100, varindex1[i]);
	  varvals[i] = atof(tempword1);
	}
	return 0;
      }
    }
  }
  fprintf(stderr, "ERROR in getdatafilevals: no matching value found.\n");
  for(i=0;i<numvars;i++){
    varvals[i] = -999.999;
  }
  return -1;

}


int getdatfiledata(char fname[], char varname[], double varval, char *outvarname[], int numoutvars, double outvarvals[], double max_error)
{
  int i;
  int tempindex;
  int outvarindices[numoutvars];
  tempindex = getdatfileindex(fname, varname);
  for(i=0;i<numoutvars;i++){
    outvarindices[i] = getdatfileindex(fname, outvarname[i]);
  }
  getdatafilevals(fname, tempindex, varval, max_error, outvarindices, numoutvars, outvarvals);
  return 0;
}


int compdatfiles(char fname[], char fname1[])
{

  int len, i=0, j=0;
  FILE *fd, *fd1;
  char temp[10000], temp1[10000], var[40], var1[40]; //length of line then variable
  char **vararray=NULL;
  char **vararray1=NULL;
  int numvars=0, numvars1=0;
  char *rems[] = {};
  int numrems = 0;
  double val, val1;

  fprintf(stderr, "Comparing data files.\n");

  fd = fopen(fname, "r");
  if (!fd){
    fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: File %s can't be opened\n", fname);
    return -1; //exit: datafile not found
  }
  fd1 = fopen(fname1, "r");
  if (!fd1){
    fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: File %s can't be opened\n", fname1);
    return -1; //exit: datafile not found
  }

  if((len = getline(fd, temp, 10000)) > 0){ // max length 10000
    while((temp[i] == '#') || (temp[i] == '%') || (temp[i] == '/') || (temp[i] == '*')){ /* ignore initial characters */
      i++;
    }
    if(i == 0){ /* no marker character */
      fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: First line must start with \"#\" or \"%%\" or \"/\" or \"*\"...\n");
      return -1;
    }
    else
      vararray = getvs(temp, &numvars, rems, numrems);
  }
  if((len = getline(fd1, temp, 10000)) > 0){ // max length 10000
    while((temp[i] == '#') || (temp[i] == '%') || (temp[i] == '/') || (temp[i] == '*')){ /* ignore initial characters */
      i++;
    }
    if(i == 0){ /* no marker character */
      fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: First line must start with \"#\" or \"%%\" or \"/\" or \"*\"...\n");
      freearraydat(vararray, numvars);
      return -1;
    }
    else
      vararray1 = getvs(temp, &numvars1, rems, numrems);
  }
  fprintf(stderr, "numvars = %d\n", numvars);

  if(numvars != numvars1){
    fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: Number of variables in the two files differs.\n");
    freearraydat(vararray, numvars);
    freearraydat(vararray1, numvars1);
    return -1;
  }

  for(i=0;i<numvars;i++){
    if(strcmp(vararray[i], vararray1[i])!=0){
      fprintf(stderr, "ERROR in compdatfiles in stringmanip.c: %dth variables differ: %s\t%s\n", i, vararray[i], vararray1[i]);
      freearraydat(vararray, numvars);
      freearraydat(vararray1, numvars1);
      return -1;
    }
  }

  while(len>0){
    j++;
    len=getline(fd, temp, 10000);
    len=getline(fd1, temp1, 10000);
    if(strcmp(temp, temp1) == 0){
      fprintf(stderr, "lines %d identical\n", j);
    }
    else{
      for(i=1;i<numvars+1;i++){
	getnthtoken(temp, var, 40, i);
	getnthtoken(temp1, var1, 40, i);
	if(isonlyspace(var) || isonlyspace(var1)){
	  fprintf(stderr, "var %d, line %d empty\n", i, j);
	}
	else{
	  val = atof(var);
	  val1 = atof(var1);
	  if(fabs(val-val1) > 1e-4){
	    fprintf(stderr, "line %d, %s: %s\t%s\n", j, vararray[i-1], var, var1);
	  }
	}
	/*       if(strcmp(var, var1) !=0){ */
	/* 	fprintf(stderr, "line %d, %s: %s\t%s\n", j, vararray[i-1], var, var1); */
	/*       } */
      }
    }
  }

  freearraydat(vararray, numvars);
  freearraydat(vararray1, numvars1);

  fclose(fd);
  fclose(fd1);
  return 0;
}


int orderlines(char *pname, char *type, char *fname, char ***lines, char othertype[], int lim){
  int i, j=0, len, found;
  FILE *fd;
  char line[2000];
  char *typewd;
  char *pnamewd;
  char *templine, *lineform;
  char *tempstr, *pnameform, *tempstr2, *pnamewdform;
  char *p;
  int j1=0, j2=0, j3=0, j4=0, j5=0;
  int jtot;
  int anytype=0;
  char *typearray[10];
  int numtypes=0;

  if(lim<10){
    fprintf(stderr, "ERROR in orderlines: lim must be at least 10. Exiting.\n");
    return -1;
  }

  if(isonlyspace(type)){anytype=1;}


  (*lines)=NULL;
  strcpy(othertype, "");

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in orderlines: File %s could not be opened...", fname);
    return -1;
  }

  numtypes = getallvars(type, typearray, 10);
  tempstr = delunder(pname); pnameform = wordtolower(tempstr); free(tempstr);

  while((len = getline0(fd, line, 2000)) > 0){
    if(!iscomsep(line)){
      typewd = getnthwd(line, 1);
      i=0;while(line[i] && line[i] != ':'){i++;}// get past the type 
      pnamewd = getnthwd(line + i, 1);
      if(anytype==1 || isinarray(typearray, numtypes, typewd)>=0){
	templine = delunder(line); lineform = wordtolower(templine); free(templine);
	tempstr2 = delunder(pnamewd); pnamewdform = wordtolower(tempstr2); free(tempstr2);
	if((p=strstr(pnamewd, pname)) && (strcmp(p, pnamewd)==0)){found=1;j++;j1++;}
	else if((strstr(pnamewd, pname))){found=1;j++;j2++;}
	else if(strcmp(pnameform, pnamewdform)==0){found = 1;j++;j3++;}
	else if(strstr(pnamewdform, pnameform)){found = 1;j++;j4++;}
	else if(strstr(lineform, pnameform)){found = 1;j++;j5++;}
	free(pnamewdform); free(lineform);
      }
      else{
	if(strcmp(pnamewd, pname)==0){ // parameter of some other type found
	  strncpy(othertype, typewd, lim-1);
	}
      }
      free(pnamewd); free(typewd);
    }
  }
  fclose(fd);

  jtot=j;
  if(j==0){
    free(pnameform); free_datarray(typearray, numtypes);
    return 0;
  }

  (*lines)=charar(0, j-1);

  fd = fopen(fname, "r");
  if(!fd){
    fprintf(stderr, "ERROR in orderlines: File %s could not be opened...", fname);
    free(pnameform); free_datarray(typearray, numtypes);
    return -1;
  }
  j=0;j2+=j1;j3+=j2;j4+=j3;
  while((len = getline0(fd, line, 500)) > 0){
    if(!iscomsep(line)){
      typewd = getnthwd(line, 1);
      i=0;while(line[i] && line[i] != ':'){i++;}// get past the type 
      pnamewd = getnthwd(line + i, 1);
      if(anytype==1 || isinarray(typearray, numtypes, typewd)>=0){
	templine = delunder(line); lineform = wordtolower(templine); free(templine);
        tempstr2 = delunder(pnamewd); pnamewdform = wordtolower(tempstr2); free(tempstr2);

        if((p=strstr(pnamewd, pname)) && (strcmp(p, pnamewd)==0)){(*lines)[j++] = strdup(line);}
        else if((strstr(pnamewd, pname))){(*lines)[j1++] = strdup(line);}
        else if(strcmp(pnameform, pnamewdform)==0){(*lines)[j2++] = strdup(line);}
        else if(strstr(pnamewdform, pnameform)){(*lines)[j3++] = strdup(line);}
        else if(strstr(lineform, pnameform)){(*lines)[j4++] = strdup(line);}
	free(pnamewdform); free(lineform);
      }
      free(pnamewd);free(typewd);
    }
  }
  fclose(fd);

  free(pnameform); free_datarray(typearray, numtypes);
  return jtot;
}

//
// Translate from a template into a process
//

char *transtemp(char *s, char *vars[], int tot){
  char *kv, *kv1;
  char *vline;
  char *tm, *tm1;
  int numvals;
  char *chems[100];
  char *outstr;
  char *nm, *ct, *tmp, *cmps, *tway, *lft, *clft, *rgt, *crgt, *rts, *rtrm;
  int sz;

  if(tot<1){
    fprintf(stderr, "ERROR in transtemp: a template takes at least one variable. Exiting\n");
    outstr = strdup("");
    return outstr;
  }
  vline = getrst("vars:", s);
  numvals = getallvars(vline, chems, 100);
  free(vline);
  if(numvals != tot){
    fprintf(stderr, "ERROR in transtemp: the number of variables needed does not match the template. Exiting\n");
    free_datarray(chems, numvals);
    outstr = strdup("");
    return outstr;
  }
  ct = getname("category:", s);
  if(strcmp(ct, "DEterm")==0){
    tm = getrst("term:", s);
    tm1 = transfunc2(tm, chems, vars, numvals);
    free(tm);
    kv = getname("keyvar:", s);
    kv1 = transfunc2(kv, chems, vars, numvals);
    free(kv);
    nm = getname("addstr:", s);

    outstr = (char*) malloc(sizeof(char) * (2*strlen(kv1) + strlen(tm1) + strlen(nm) + 50));
    sprintf(outstr, "******\nname: %s%s\nkeyvar: %s\nterm: %s\n", kv1, nm, kv1, tm1); // safe
    free(tm1); free(kv1);free(nm);
  }
  else if(strcmp(ct, "reaction")==0 || strcmp(ct, "reac")==0){
    nm = getname("addstr:", s);
    sz=0;
    tmp = getrst("type:", s); tm1 = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(tm1)+7; 
    tmp = getrst("comps:", s); cmps = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(cmps)+8;
    tmp = getrst("twoway:", s); tway = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(tway)+9;
    tmp = getrst("left:", s); lft = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(lft)+7;
    tmp = getrst("compsleft:", s); clft = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(clft)+12;
    tmp = getrst("right:", s); rgt = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(rgt)+8;
    tmp = getrst("compsright:", s); crgt = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(crgt)+12;
    tmp = getrst("rates:", s); rts = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(rts)+8;
    tmp = getrst("rateterm:", s); rtrm = transfunc2(tmp, chems, vars, numvals); free(tmp);
    sz+=strlen(rtrm)+8;
    outstr = (char*) malloc(sizeof(char) * (sz + 100));
    sprintf(outstr, "******\nname: %s%s\n", vars[0], nm);
    strcat(outstr, "type: ");strcat(outstr, tm1);strcat(outstr, "\n");
    if(!isonlyspace(cmps)){strcat(outstr, "comps: ");strcat(outstr, cmps);strcat(outstr, "\n");}
    if(!isonlyspace(tway)){strcat(outstr, "twoway: ");strcat(outstr, tway);strcat(outstr, "\n");}
    strcat(outstr, "left: ");strcat(outstr, lft);strcat(outstr, "\n");
    if(!isonlyspace(clft)){strcat(outstr, "compsleft: ");strcat(outstr, clft);strcat(outstr, "\n");}
    strcat(outstr, "right: ");strcat(outstr, rgt);strcat(outstr, "\n");
    if(!isonlyspace(crgt)){strcat(outstr, "compsright: ");strcat(outstr, crgt);strcat(outstr, "\n");}
    if(!isonlyspace(rts)){strcat(outstr, "rates: ");strcat(outstr, rts);strcat(outstr, "\n");}
    if(!isonlyspace(rtrm)){strcat(outstr, "rateterm: ");strcat(outstr, rtrm);strcat(outstr, "\n");}
    free(nm);free(tm1);free(cmps);free(tway);free(lft);free(clft);free(rgt);free(crgt);free(rts);free(rtrm);
  }
  else if(isonlyspace(ct)){
    fprintf(stderr, "ERROR in transtemp: a template must be of a particular kind. The template \n%s\n seems to have no \"category:\" field. Exiting...\n", s);
    outstr = strdup("");
  }
  else{
    fprintf(stderr, "ERROR in transtemp: a template of kind %s is not recognised. Exiting...\n", ct);
    outstr = strdup("");
  }
  free(ct);
  free_datarray(chems, numvals);
  return outstr;
}

